/****************************************************/
/*													*/
/*	Code on ADSP-21365 for "PDAP-BF Interface		*/
/*													*/
/*													*/
/*		- Initial Version							*/
/*				7th August, 2004					*/
/*													*/
/*						kunal singh		 			*/
/*						DSP Apps Engineer			*/
/*						Analog Devices				*/
/*													*/
/****************************************************/		
#include <Cdef21365.h>
#include <def21365.h>
#include <stdio.h>
#include <signal.h>
#include <sru2136x_new.h>




/****************************************************************/
/*																*/
/*																*/
/*	This Example code has been developed 						*/
/*	for interfacing the PDAP port of the ADSP-21365				*/
/*	with BF-532 External Port									*/
/*																*/
/*	The PDAP would operate in MODE00 (2x16bit)					*/
/*																*/
/*																*/
/*	The External Connections required between the 				*/
/*	ADSP-21365 and BF-532 are									*/
/*																*/
/*	ADSP-21365			 				BF-532					*/
/*																*/
/*	DAI_PIN05..20 (PDAP_DAT)	<--		D0..D15					*/
/*	DAI_PIN01	  (PDAP_CLK)	<--		SCLK0					*/
/*	DAI_PIN03 	 (PDAP_HOLD)	<--		AMS0					*/
/*	IRQ0						<--		PF4						*/
/*	FLAG0						-->		PF6						*/
/*																*/
/*	 															*/
/*	PDAP would be configured to latch the 						*/
/*	data at the negative edge of the PDAP_CLK					*/
/*	This is in accordance with the data/clk relationship		*/
/*	available on BF-532											*/
/*																*/
/*																*/
/*	 															*/
/*	The data transfers from PDAP FIFO to the internal 			*/
/*	memory would be done with DMAs								*/
/*	Simple (No Ping Pong) DMAs would be used					*/
/*	    														*/
/*																*/
/*																*/
/*																*/
/****************************************************************/





void dai_low_isr(int i);
void irq0_low_isr(int i);

/****************************************************/
/*													*/
/* dai_low_isr(): The PDAP DMA over interrupt		*/ 
/*													*/
/* irq0_low_isr(): ISR for the IRQ0					*/
/*													*/			
/****************************************************/


void Enable_PDAP(void);
/****************************************/
/*										*/
/*  Enable the PDAP	to receive data at	*/
/*	Memory index <- dma_index			*/
/*	Count		 <- dma_count			*/
/*										*/
/*										*/
/****************************************/



volatile unsigned int  pdap_command[2];
/********************************************/
/* 											*/
/*	Control data would be stored in the 	*/
/*	above memory buffer						*/
/*	The first element corresonds to the 	*/
/*	memory address for the received data	*/
/*	The second element corresponds to the 	*/
/*	count of DMA transfers					*/
/*											*/	
/* 											*/
/********************************************/
	

volatile unsigned int command_flag;
/****************************************************/
/*													*/
/*	BF-532 would interrupt (IRQ0) the ADSP-21365	*/
/*													*/	
/*	If command_flag == 0 =>							*/
/*		a command word has to be received			*/
/*													*/	
/*	Else 											*/
/*		The Interrupt would be ingnored				*/
/*													*/	
/****************************************************/



volatile unsigned int dma_index;
volatile unsigned int dma_count;
/****************************************/
/*										*/
/* 	DMA parameters for the PDAP channel	*/
/*										*/
/*	dma_index 	: IDP_DMA_I0			*/
/*	dma_count	: IDP_DMA_C0Count		*/
/*										*/
/****************************************/

unsigned int temp;	
/************************************/
/*									*/
/*	Variable for Temporary Storage	*/
/*									*/
/************************************/



/********************************/
/*								*/
/*		Main Function 			*/
/*								*/
/********************************/

void main()
{

	command_flag == 0;
	//	Initially command_flag == 0;

	interrupt(SIG_P12,dai_low_isr);
	// Interrupt handler for PDAP DMA over interrupt	

	asm("#include <def21365.h>");
	asm("bit set mode2 IRQ0E;");
	// Define IRQ0 as edge sensitive

	interrupt(SIG_IRQ0,irq0_low_isr);
	// interrupt handler for IRQ0

	asm("bit set FLAGS 0x2;");
	// configure the FLAG0 as o/p
	
	asm("bit clr FLAGS 0x1;");
	// clear the FLAG0

	
	
	
	// Disable the o/p drivers on the DAI pins
	// To ensure that these are not driven by the DSP
	SRU (LOW, PBEN01_I); 
	SRU (LOW, PBEN02_I); 
	SRU (LOW, PBEN03_I); 
	SRU (LOW, PBEN04_I); 
	SRU (LOW, PBEN05_I); 
	SRU (LOW, PBEN06_I); 
	SRU (LOW, PBEN07_I); 
	SRU (LOW, PBEN08_I); 
	SRU (LOW, PBEN09_I); 
	SRU (LOW, PBEN10_I); 
	SRU (LOW, PBEN11_I); 
	SRU (LOW, PBEN12_I); 
	SRU (LOW, PBEN13_I); 
	SRU (LOW, PBEN14_I); 
	SRU (LOW, PBEN15_I); 
	SRU (LOW, PBEN16_I); 
	SRU (LOW, PBEN17_I); 
	SRU (LOW, PBEN18_I); 
	SRU (LOW, PBEN19_I); 
	SRU (LOW, PBEN20_I); 


	SRU (DAI_PB01_O, IDP0_CLK_I); 
	SRU (DAI_PB03_O, IDP0_FS_I); 
	// 	Configure the SRU for PDAP_CLK and PDAP_HOLD
	
	
   while(1) 
   {
   }
   
	
}






void irq0_low_isr(int i)
{
	if(command_flag == 0)
	// A valid Data Request (no previous transfers pending)
	{
		command_flag =1;
		
		dma_index = (unsigned int) pdap_command;
		dma_count = 0x2;
		
		Enable_PDAP();
		
	
		asm("bit clr FLAGS 0x1;");
		asm("bit set FLAGS 0x1;");
		asm("bit clr FLAGS 0x1;");
		// Toggle the FLAG0 from low to high
		// This would interrupt the BF-532
			
	}
	else
	// Previous transfers are pending
	{
		
		printf("\n an error has occured\n");
		// corrective actions can be taken
	
	}
}



void dai_low_isr(int i)
{	
	
	int j;
	
	temp = *pDAI_IRPTL_L;
	// Read the DAI interrupt 	
	
	if ( temp == IDP_DMA0_INT)
	// 	confirm the source of the interrupt
	//	before processing the ISR
	{	
		
		if(command_flag == 1)
		// A Command word has been received
		{
			
			command_flag = 0;
			
			dma_index = pdap_command[0];
			dma_count = pdap_command[1];
			
			Enable_PDAP();
			
			asm("bit clr FLAGS 0x1;");
			asm("bit set FLAGS 0x1;");
			asm("bit clr FLAGS 0x1;");
			// Toggle the FLAG0 from low to high
			// This would interrupt the BF-532
			// Toggle the flag1 from low to high
			// This would interrupt the BF-532
		
		}
		else
		{
			//	Data has been received 
			//	Any action can be taken here
			//	For example you can process this data
			//	However this example does not 
			//	include any data processing
			
		}
	}
	
	
}



void Enable_PDAP(void)
{

	*pIDP_CTL0 = 0;
	*pIDP_CTL1 = 0;
//	disable the CIDP globally 

	*pIDP_PP_CTL = 0;
//	clear the PDAP control register	

	*pIDP_PP_CTL |= IDP_PDAP_RESET;
//	reset the PDAP state machine	

	*pIDP_PP_CTL |= IDP_PDAP_PACKING0;
//	select the mode00



//	Enable the PDAP Mask bits for DAI_PIN05..20	
	*pIDP_PP_CTL |= IDP_P20_PDAPMASK|
					IDP_P19_PDAPMASK| 
					IDP_P18_PDAPMASK|
					IDP_P17_PDAPMASK|
					IDP_P16_PDAPMASK|
					IDP_P15_PDAPMASK|
					IDP_P14_PDAPMASK|
					IDP_P13_PDAPMASK|
					IDP_P12_PDAPMASK|
					IDP_P11_PDAPMASK|
					IDP_P10_PDAPMASK|
					IDP_P09_PDAPMASK|
					IDP_P08_PDAPMASK|
					IDP_P07_PDAPMASK|
					IDP_P06_PDAPMASK|
					IDP_P05_PDAPMASK;
	   

	*pIDP_PP_CTL |= IDP_PDAP_CLKEDGE;
// define the active clock edge	for the PDAP

	*pIDP_PP_CTL |= IDP_PDAP_EN	;
//	enable the PDAP		

	*pIDP_DMA_M0  = 0x1;
	*pIDP_DMA_C0 =  dma_count;
	*pIDP_DMA_I0 =  dma_index;
//	configure PDAP DMA parameter registers		

	*pIDP_CTL1 |= IDP_EN0|IDP_DMA_EN0;
	*pIDP_CTL0 |= IDP_DMA_EN|IDP_EN;
//	Enable the PDAP with DMAs

}
