//==============================================================================
//
//  SRU.h (c) Copyright 2004 Analog Devices, Inc.  All rights reserved.
//
//  This include file contains a list of macro "defines" to enable the
//  programmer to use symbolic names for ADSP-21364/21365 SRU
//  This is intended to simplify patching in the Signal Routing Unit (SRU)
//  The SRU macro is as follows: SRU(OUT,IN)
//
//  03.FEB.2004 - GL Initial SRU including all ADSP-2136x defines
//			based on the 15.DEC.2003 version for the ADSP-2126x.  
//  ============================================================================

//==============================================================================
//
//      The macro
//
//==============================================================================
#ifndef __SRU_H___

#ifdef _LANGUAGE_C
#define SRU(OUT,IN)                      \
  * (volatile int *) IN##_##OUT##_REG &= IN##_MASK; \
  * (volatile int *) IN##_##OUT##_REG |= (IN##_##OUT##_ENCODING << IN##_SHIFT);     \

#else
#define SRU(OUT,IN)                      \
  r0 = dm(IN##_##OUT##_REG);          \
  r1 = IN##_MASK;                        \
  r0 = r0 and r1;                        \
  r1 = IN##_##OUT##_ENCODING;            \
  r0 = r0 OR LSHIFT r1 BY IN##_SHIFT;    \
  dm(IN##_##OUT##_REG) = r0;

#endif

/*
#ifdef _LANGUAGE_C
#define SRU(OUT,IN)                      \
  * (volatile int *) IN##_##OUT##_REG &= IN##_MASK; \
  * (volatile int *) IN##_##OUT##_REG |= (IN##_##OUT##_ENCODING<<IN##_SHIFT);     \

#else
#define SRU(OUT,IN)                      \
  r0 = dm(IN##_##OUT##_REG);          \
  r1 = IN##_MASK;                        \
  r0 = r0 and r1;                        \
  r1 = IN##_##OUT##_ENCODING;            \
  r0 = r0 OR LSHIFT r1 BY IN##_SHIFT;    \
  dm(IN##_##OUT##_REG) = r0;

#endif

*/


/*============================================================================*
 *                                                                            *
 *  GROUP A                                                                   *
 *                                                                            *
 *============================================================================*/

//------------------------------------------------------------------------------
//  Group A: Input bitfield masks & shifts
//------------------------------------------------------------------------------
//SRU_CLK0
#define SPORT0_CLK_I_MASK                           0xFFFFFFE0
#define SPORT0_CLK_I_SHIFT                          0
#define SPORT1_CLK_I_MASK                           0xFFFFFC1F
#define SPORT1_CLK_I_SHIFT                          5
#define SPORT2_CLK_I_MASK                           0xFFFF83FF
#define SPORT2_CLK_I_SHIFT                          10
#define SPORT3_CLK_I_MASK                           0xFFF07FFF
#define SPORT3_CLK_I_SHIFT                          15
#define SPORT4_CLK_I_MASK                           0xFE0FFFFF
#define SPORT4_CLK_I_SHIFT                          20
#define SPORT5_CLK_I_MASK                           0xC1FFFFFF
#define SPORT5_CLK_I_SHIFT                          25
//SRU_CLK1
#define SRC0_CLK_IP_I_MASK                           0xFFFFFFE0
#define SRC0_CLK_IP_I_SHIFT                          0
#define SRC0_CLK_OP_I_MASK                           0xFFFFFC1F
#define SRC0_CLK_OP_I_SHIFT                          5
#define SRC1_CLK_IP_I_MASK                           0xFFFF83FF
#define SRC1_CLK_IP_I_SHIFT                          10
#define SRC1_CLK_OP_I_MASK                           0xFFF07FFF
#define SRC1_CLK_OP_I_SHIFT                          15
#define SRC2_CLK_IP_I_MASK                           0xFE0FFFFF
#define SRC2_CLK_IP_I_SHIFT                          20
#define SRC2_CLK_OP_I_MASK                           0xC1FFFFFF
#define SRC2_CLK_OP_I_SHIFT                          25
//SRU_CLK2
#define SRC3_CLK_IP_I_MASK                           0xFFFFFFE0
#define SRC3_CLK_IP_I_SHIFT                          0
#define SRC3_CLK_OP_I_MASK                           0xFFFFFC1F
#define SRC3_CLK_OP_I_SHIFT                          5
#define DIT_CLK_I_MASK                               0xFFFF83FF
#define DIT_CLK_I_SHIFT                              10
#define IDP0_CLK_I_MASK                             0xFFF07FFF
#define IDP0_CLK_I_SHIFT                            15
#define PDAP_CLK_I_MASK                             0xFFF07FFF
#define PDAP_CLK_I_SHIFT                            15
#define IDP1_CLK_I_MASK                             0xFE0FFFFF
#define IDP1_CLK_I_SHIFT                            20
#define IDP2_CLK_I_MASK                             0xC1FFFFFF
#define IDP2_CLK_I_SHIFT                            25
//SRU_CLK3
#define IDP3_CLK_I_MASK                             0xFFFFFFE0
#define IDP3_CLK_I_SHIFT                            0
#define IDP4_CLK_I_MASK                             0xFFFFFC1F
#define IDP4_CLK_I_SHIFT                            5
#define IDP5_CLK_I_MASK                             0xFFFF83FF
#define IDP5_CLK_I_SHIFT                            10
#define IDP6_CLK_I_MASK                             0xFFF07FFF
#define IDP6_CLK_I_SHIFT                            15
#define IDP7_CLK_I_MASK                             0xFE0FFFFF
#define IDP7_CLK_I_SHIFT                            20
#define DIT_HFCLK_I_MASK                             0xC1FFFFFF
#define DIT_HFCLK_I_SHIFT                            25
//SRU_CLK4
#define PCG_EXTA_I_MASK                             0xFFFFFFE0
#define PCG_EXTA_SHIFT                              0
#define PCG_EXTB_I_MASK                             0xFFFFFC1F
#define PCG_EXTB_I_SHIFT                            5
#define SPDIF_PLLCLK_I_MASK                         0xFFFF83FF
#define SPDIF_PLLCLK_I_SHIFT                        10
#define SPIB_CLK_I_MASK                             0xFFF07FFF
#define SPIB_CLK_I_SHIFT                            15
#define PCG_SYNC_CLKA_I_MASK                        0xFE0FFFFF
#define PCG_SYNC_CLKA_I_SHIFT                       20
#define PCG_SYNC_CLKB_I_MASK                        0xC1FFFFFF
#define PCG_SYNC_CLKB_I_SHIFT                       25

//------------------------------------------------------------------------------
//  Group A: Bitfield encodings
//------------------------------------------------------------------------------

#define SPORT0_CLK_I_DAI_PB01_O_ENCODING            0x00
#define SPORT1_CLK_I_DAI_PB01_O_ENCODING            0x00
#define SPORT2_CLK_I_DAI_PB01_O_ENCODING            0x00
#define SPORT3_CLK_I_DAI_PB01_O_ENCODING            0x00
#define SPORT4_CLK_I_DAI_PB01_O_ENCODING            0x00
#define SPORT5_CLK_I_DAI_PB01_O_ENCODING            0x00
#define SRC0_CLK_IP_I_DAI_PB01_O_ENCODING           0x00
#define SRC0_CLK_OP_I_DAI_PB01_O_ENCODING           0x00
#define SRC1_CLK_IP_I_DAI_PB01_O_ENCODING           0x00
#define SRC1_CLK_OP_I_DAI_PB01_O_ENCODING           0x00
#define SRC2_CLK_IP_I_DAI_PB01_O_ENCODING           0x00
#define SRC2_CLK_OP_I_DAI_PB01_O_ENCODING           0x00
#define SRC3_CLK_IP_I_DAI_PB01_O_ENCODING           0x00
#define SRC3_CLK_OP_I_DAI_PB01_O_ENCODING           0x00
#define DIT_CLK_I_DAI_PB01_O_ENCODING               0x00
#define IDP0_CLK_I_DAI_PB01_O_ENCODING              0x00
#define PDAP_CLK_I_DAI_PB01_O_ENCODING              0x00
#define IDP1_CLK_I_DAI_PB01_O_ENCODING              0x00
#define IDP2_CLK_I_DAI_PB01_O_ENCODING              0x00
#define IDP3_CLK_I_DAI_PB01_O_ENCODING              0x00
#define IDP4_CLK_I_DAI_PB01_O_ENCODING              0x00
#define IDP5_CLK_I_DAI_PB01_O_ENCODING              0x00
#define IDP6_CLK_I_DAI_PB01_O_ENCODING              0x00
#define IDP7_CLK_I_DAI_PB01_O_ENCODING              0x00
#define DIT_HFCLK_I_DAI_PB01_O_ENCODING             0x00
#define PCG_EXTA_I_DAI_PB01_O_ENCODING              0x00
#define PCG_EXTB_I_DAI_PB01_O_ENCODING              0x00
#define SPDIF_PLLCLK_I_DAI_PB01_O_ENCODING          0x00
#define SPIB_CLK_I_DAI_PB01_O_ENCODING              0x00
#define PCG_SYNC_CLKA_I_DAI_PB01_O_ENCODING         0x00
#define PCG_SYNC_CLKB_I_DAI_PB01_O_ENCODING         0x00

#define SPORT0_CLK_I_DAI_PB02_O_ENCODING            0x01
#define SPORT1_CLK_I_DAI_PB02_O_ENCODING            0x01
#define SPORT2_CLK_I_DAI_PB02_O_ENCODING            0x01
#define SPORT3_CLK_I_DAI_PB02_O_ENCODING            0x01
#define SPORT4_CLK_I_DAI_PB02_O_ENCODING            0x01
#define SPORT5_CLK_I_DAI_PB02_O_ENCODING            0x01
#define SRC0_CLK_IP_I_DAI_PB02_O_ENCODING           0x01
#define SRC0_CLK_OP_I_DAI_PB02_O_ENCODING           0x01
#define SRC1_CLK_IP_I_DAI_PB02_O_ENCODING           0x01
#define SRC1_CLK_OP_I_DAI_PB02_O_ENCODING           0x01
#define SRC2_CLK_IP_I_DAI_PB02_O_ENCODING           0x01
#define SRC2_CLK_OP_I_DAI_PB02_O_ENCODING           0x01
#define SRC3_CLK_IP_I_DAI_PB02_O_ENCODING           0x01
#define SRC3_CLK_OP_I_DAI_PB02_O_ENCODING           0x01
#define DIT_CLK_I_DAI_PB02_O_ENCODING               0x01
#define IDP0_CLK_I_DAI_PB02_O_ENCODING              0x01
#define PDAP_CLK_I_DAI_PB02_O_ENCODING              0x01
#define IDP1_CLK_I_DAI_PB02_O_ENCODING              0x01
#define IDP2_CLK_I_DAI_PB02_O_ENCODING              0x01
#define IDP3_CLK_I_DAI_PB02_O_ENCODING              0x01
#define IDP4_CLK_I_DAI_PB02_O_ENCODING              0x01
#define IDP5_CLK_I_DAI_PB02_O_ENCODING              0x01
#define IDP6_CLK_I_DAI_PB02_O_ENCODING              0x01
#define IDP7_CLK_I_DAI_PB02_O_ENCODING              0x01
#define DIT_HFCLK_I_DAI_PB02_O_ENCODING             0x01
#define PCG_EXTA_I_DAI_PB02_O_ENCODING              0x01
#define PCG_EXTB_I_DAI_PB02_O_ENCODING              0x01
#define SPDIF_PLLCLK_I_DAI_PB02_O_ENCODING          0x01
#define SPIB_CLK_I_DAI_PB02_O_ENCODING              0x01
#define PCG_SYNC_CLKA_I_DAI_PB02_O_ENCODING         0x01
#define PCG_SYNC_CLKB_I_DAI_PB02_O_ENCODING         0x01


#define SPORT0_CLK_I_DAI_PB03_O_ENCODING            0x02
#define SPORT1_CLK_I_DAI_PB03_O_ENCODING            0x02
#define SPORT2_CLK_I_DAI_PB03_O_ENCODING            0x02
#define SPORT3_CLK_I_DAI_PB03_O_ENCODING            0x02
#define SPORT4_CLK_I_DAI_PB03_O_ENCODING            0x02
#define SPORT5_CLK_I_DAI_PB03_O_ENCODING            0x02
#define SRC0_CLK_IP_I_DAI_PB03_O_ENCODING           0x02
#define SRC0_CLK_OP_I_DAI_PB03_O_ENCODING           0x02
#define SRC1_CLK_IP_I_DAI_PB03_O_ENCODING           0x02
#define SRC1_CLK_OP_I_DAI_PB03_O_ENCODING           0x02
#define SRC2_CLK_IP_I_DAI_PB03_O_ENCODING           0x02
#define SRC2_CLK_OP_I_DAI_PB03_O_ENCODING           0x02
#define SRC3_CLK_IP_I_DAI_PB03_O_ENCODING           0x02
#define SRC3_CLK_OP_I_DAI_PB03_O_ENCODING           0x02
#define DIT_CLK_I_DAI_PB03_O_ENCODING               0x02
#define IDP0_CLK_I_DAI_PB03_O_ENCODING              0x02
#define PDAP_CLK_I_DAI_PB03_O_ENCODING              0x02
#define IDP1_CLK_I_DAI_PB03_O_ENCODING              0x02
#define IDP2_CLK_I_DAI_PB03_O_ENCODING              0x02
#define IDP3_CLK_I_DAI_PB03_O_ENCODING              0x02
#define IDP4_CLK_I_DAI_PB03_O_ENCODING              0x02
#define IDP5_CLK_I_DAI_PB03_O_ENCODING              0x02
#define IDP6_CLK_I_DAI_PB03_O_ENCODING              0x02
#define IDP7_CLK_I_DAI_PB03_O_ENCODING              0x02
#define PCG_EXTA_I_DAI_PB03_O_ENCODING              0x02
#define DIT_HFCLK_I_DAI_PB03_O_ENCODING             0x02
#define PCG_EXTB_I_DAI_PB03_O_ENCODING              0x02
#define SPDIF_PLLCLK_I_DAI_PB03_O_ENCODING          0x02
#define SPIB_CLK_I_DAI_PB03_O_ENCODING              0x02
#define PCG_SYNC_CLKA_I_DAI_PB03_O_ENCODING         0x02
#define PCG_SYNC_CLKB_I_DAI_PB03_O_ENCODING         0x02

#define SPORT0_CLK_I_DAI_PB04_O_ENCODING            0x03
#define SPORT1_CLK_I_DAI_PB04_O_ENCODING            0x03
#define SPORT2_CLK_I_DAI_PB04_O_ENCODING            0x03
#define SPORT3_CLK_I_DAI_PB04_O_ENCODING            0x03
#define SPORT4_CLK_I_DAI_PB04_O_ENCODING            0x03
#define SPORT5_CLK_I_DAI_PB04_O_ENCODING            0x03
#define SRC0_CLK_IP_I_DAI_PB04_O_ENCODING           0x03
#define SRC0_CLK_OP_I_DAI_PB04_O_ENCODING           0x03
#define SRC1_CLK_IP_I_DAI_PB04_O_ENCODING           0x03
#define SRC1_CLK_OP_I_DAI_PB04_O_ENCODING           0x03
#define SRC2_CLK_IP_I_DAI_PB04_O_ENCODING           0x03
#define SRC2_CLK_OP_I_DAI_PB04_O_ENCODING           0x03
#define SRC3_CLK_IP_I_DAI_PB04_O_ENCODING           0x03
#define SRC3_CLK_OP_I_DAI_PB04_O_ENCODING           0x03
#define DIT_CLK_I_DAI_PB04_O_ENCODING               0x03
#define IDP0_CLK_I_DAI_PB04_O_ENCODING              0x03
#define PDAP_CLK_I_DAI_PB04_O_ENCODING              0x03
#define IDP1_CLK_I_DAI_PB04_O_ENCODING              0x03
#define IDP2_CLK_I_DAI_PB04_O_ENCODING              0x03
#define IDP3_CLK_I_DAI_PB04_O_ENCODING              0x03
#define IDP4_CLK_I_DAI_PB04_O_ENCODING              0x03
#define IDP5_CLK_I_DAI_PB04_O_ENCODING              0x03
#define IDP6_CLK_I_DAI_PB04_O_ENCODING              0x03
#define IDP7_CLK_I_DAI_PB04_O_ENCODING              0x03
#define DIT_HFCLK_I_DAI_PB04_O_ENCODING             0x03
#define PCG_EXTA_I_DAI_PB04_O_ENCODING              0x03
#define PCG_EXTB_I_DAI_PB04_O_ENCODING              0x03
#define SPDIF_PLLCLK_I_DAI_PB04_O_ENCODING          0x03
#define SPIB_CLK_I_DAI_PB04_O_ENCODING              0x03
#define PCG_SYNC_CLKA_I_DAI_PB04_O_ENCODING         0x03
#define PCG_SYNC_CLKB_I_DAI_PB04_O_ENCODING         0x03

#define SPORT0_CLK_I_DAI_PB05_O_ENCODING            0x04
#define SPORT1_CLK_I_DAI_PB05_O_ENCODING            0x04
#define SPORT2_CLK_I_DAI_PB05_O_ENCODING            0x04
#define SPORT3_CLK_I_DAI_PB05_O_ENCODING            0x04
#define SPORT4_CLK_I_DAI_PB05_O_ENCODING            0x04
#define SPORT5_CLK_I_DAI_PB05_O_ENCODING            0x04
#define SRC0_CLK_IP_I_DAI_PB05_O_ENCODING           0x04
#define SRC0_CLK_OP_I_DAI_PB05_O_ENCODING           0x04
#define SRC1_CLK_IP_I_DAI_PB05_O_ENCODING           0x04
#define SRC1_CLK_OP_I_DAI_PB05_O_ENCODING           0x04
#define SRC2_CLK_IP_I_DAI_PB05_O_ENCODING           0x04
#define SRC2_CLK_OP_I_DAI_PB05_O_ENCODING           0x04
#define SRC3_CLK_IP_I_DAI_PB05_O_ENCODING           0x04
#define SRC3_CLK_OP_I_DAI_PB05_O_ENCODING           0x04
#define DIT_CLK_I_DAI_PB05_O_ENCODING               0x04
#define IDP0_CLK_I_DAI_PB05_O_ENCODING              0x04
#define PDAP_CLK_I_DAI_PB05_O_ENCODING              0x04
#define IDP1_CLK_I_DAI_PB05_O_ENCODING              0x04
#define IDP2_CLK_I_DAI_PB05_O_ENCODING              0x04
#define IDP3_CLK_I_DAI_PB05_O_ENCODING              0x04
#define IDP4_CLK_I_DAI_PB05_O_ENCODING              0x04
#define IDP5_CLK_I_DAI_PB05_O_ENCODING              0x04
#define IDP6_CLK_I_DAI_PB05_O_ENCODING              0x04
#define IDP7_CLK_I_DAI_PB05_O_ENCODING              0x04
#define DIT_HFCLK_I_DAI_PB05_O_ENCODING             0x04
#define PCG_EXTA_I_DAI_PB05_O_ENCODING              0x04
#define PCG_EXTB_I_DAI_PB05_O_ENCODING              0x04
#define SPDIF_PLLCLK_I_DAI_PB05_O_ENCODING          0x04
#define SPIB_CLK_I_DAI_PB05_O_ENCODING              0x04
#define PCG_SYNC_CLKA_I_DAI_PB05_O_ENCODING         0x04
#define PCG_SYNC_CLKB_I_DAI_PB05_O_ENCODING         0x04

#define SPORT0_CLK_I_DAI_PB06_O_ENCODING            0x05
#define SPORT1_CLK_I_DAI_PB06_O_ENCODING            0x05
#define SPORT2_CLK_I_DAI_PB06_O_ENCODING            0x05
#define SPORT3_CLK_I_DAI_PB06_O_ENCODING            0x05
#define SPORT4_CLK_I_DAI_PB06_O_ENCODING            0x05
#define SPORT5_CLK_I_DAI_PB06_O_ENCODING            0x05
#define SRC0_CLK_IP_I_DAI_PB06_O_ENCODING           0x05
#define SRC0_CLK_OP_I_DAI_PB06_O_ENCODING           0x05
#define SRC1_CLK_IP_I_DAI_PB06_O_ENCODING           0x05
#define SRC1_CLK_OP_I_DAI_PB06_O_ENCODING           0x05
#define SRC2_CLK_IP_I_DAI_PB06_O_ENCODING           0x05
#define SRC2_CLK_OP_I_DAI_PB06_O_ENCODING           0x05
#define SRC3_CLK_IP_I_DAI_PB06_O_ENCODING           0x05
#define SRC3_CLK_OP_I_DAI_PB06_O_ENCODING           0x05
#define DIT_CLK_I_DAI_PB06_O_ENCODING               0x05
#define IDP0_CLK_I_DAI_PB06_O_ENCODING              0x05
#define PDAP_CLK_I_DAI_PB06_O_ENCODING              0x05
#define IDP1_CLK_I_DAI_PB06_O_ENCODING              0x05
#define IDP2_CLK_I_DAI_PB06_O_ENCODING              0x05
#define IDP3_CLK_I_DAI_PB06_O_ENCODING              0x05
#define IDP4_CLK_I_DAI_PB06_O_ENCODING              0x05
#define IDP5_CLK_I_DAI_PB06_O_ENCODING              0x05
#define IDP6_CLK_I_DAI_PB06_O_ENCODING              0x05
#define IDP7_CLK_I_DAI_PB06_O_ENCODING              0x05
#define DIT_HFCLK_I_DAI_PB06_O_ENCODING             0x05
#define PCG_EXTA_I_DAI_PB06_O_ENCODING              0x05
#define PCG_EXTB_I_DAI_PB06_O_ENCODING              0x05
#define SPDIF_PLLCLK_I_DAI_PB06_O_ENCODING          0x05
#define SPIB_CLK_I_DAI_PB06_O_ENCODING              0x05
#define PCG_SYNC_CLKA_I_DAI_PB06_O_ENCODING         0x05
#define PCG_SYNC_CLKB_I_DAI_PB06_O_ENCODING         0x05

#define SPORT0_CLK_I_DAI_PB07_O_ENCODING            0x06
#define SPORT1_CLK_I_DAI_PB07_O_ENCODING            0x06
#define SPORT2_CLK_I_DAI_PB07_O_ENCODING            0x06
#define SPORT3_CLK_I_DAI_PB07_O_ENCODING            0x06
#define SPORT4_CLK_I_DAI_PB07_O_ENCODING            0x06
#define SPORT5_CLK_I_DAI_PB07_O_ENCODING            0x06
#define SRC0_CLK_IP_I_DAI_PB07_O_ENCODING           0x06
#define SRC0_CLK_OP_I_DAI_PB07_O_ENCODING           0x06
#define SRC1_CLK_IP_I_DAI_PB07_O_ENCODING           0x06
#define SRC1_CLK_OP_I_DAI_PB07_O_ENCODING           0x06
#define SRC2_CLK_IP_I_DAI_PB07_O_ENCODING           0x06
#define SRC2_CLK_OP_I_DAI_PB07_O_ENCODING           0x06
#define SRC3_CLK_IP_I_DAI_PB07_O_ENCODING           0x06
#define SRC3_CLK_OP_I_DAI_PB07_O_ENCODING           0x06
#define DIT_CLK_I_DAI_PB07_O_ENCODING               0x06
#define IDP0_CLK_I_DAI_PB07_O_ENCODING              0x06
#define PDAP_CLK_I_DAI_PB07_O_ENCODING              0x06
#define IDP1_CLK_I_DAI_PB07_O_ENCODING              0x06
#define IDP2_CLK_I_DAI_PB07_O_ENCODING              0x06
#define IDP3_CLK_I_DAI_PB07_O_ENCODING              0x06
#define IDP4_CLK_I_DAI_PB07_O_ENCODING              0x06
#define IDP5_CLK_I_DAI_PB07_O_ENCODING              0x06
#define IDP6_CLK_I_DAI_PB07_O_ENCODING              0x06
#define IDP7_CLK_I_DAI_PB07_O_ENCODING              0x06
#define DIT_HFCLK_I_DAI_PB07_O_ENCODING             0x06
#define PCG_EXTA_I_DAI_PB07_O_ENCODING              0x06
#define PCG_EXTB_I_DAI_PB07_O_ENCODING              0x06
#define SPDIF_PLLCLK_I_DAI_PB07_O_ENCODING          0x06
#define SPIB_CLK_I_DAI_PB07_O_ENCODING              0x06
#define PCG_SYNC_CLKA_I_DAI_PB07_O_ENCODING         0x06
#define PCG_SYNC_CLKB_I_DAI_PB07_O_ENCODING         0x06

#define SPORT0_CLK_I_DAI_PB08_O_ENCODING            0x07
#define SPORT1_CLK_I_DAI_PB08_O_ENCODING            0x07
#define SPORT2_CLK_I_DAI_PB08_O_ENCODING            0x07
#define SPORT3_CLK_I_DAI_PB08_O_ENCODING            0x07
#define SPORT4_CLK_I_DAI_PB08_O_ENCODING            0x07
#define SPORT5_CLK_I_DAI_PB08_O_ENCODING            0x07
#define SRC0_CLK_IP_I_DAI_PB08_O_ENCODING           0x07
#define SRC0_CLK_OP_I_DAI_PB08_O_ENCODING           0x07
#define SRC1_CLK_IP_I_DAI_PB08_O_ENCODING           0x07
#define SRC1_CLK_OP_I_DAI_PB08_O_ENCODING           0x07
#define SRC2_CLK_IP_I_DAI_PB08_O_ENCODING           0x07
#define SRC2_CLK_OP_I_DAI_PB08_O_ENCODING           0x07
#define SRC3_CLK_IP_I_DAI_PB08_O_ENCODING           0x07
#define SRC3_CLK_OP_I_DAI_PB08_O_ENCODING           0x07
#define DIT_CLK_I_DAI_PB08_O_ENCODING               0x07
#define IDP0_CLK_I_DAI_PB08_O_ENCODING              0x07
#define PDAP_CLK_I_DAI_PB08_O_ENCODING              0x07
#define IDP1_CLK_I_DAI_PB08_O_ENCODING              0x07
#define IDP2_CLK_I_DAI_PB08_O_ENCODING              0x07
#define IDP3_CLK_I_DAI_PB08_O_ENCODING              0x07
#define IDP4_CLK_I_DAI_PB08_O_ENCODING              0x07
#define IDP5_CLK_I_DAI_PB08_O_ENCODING              0x07
#define IDP6_CLK_I_DAI_PB08_O_ENCODING              0x07
#define IDP7_CLK_I_DAI_PB08_O_ENCODING              0x07
#define DIT_HFCLK_I_DAI_PB08_O_ENCODING             0x07
#define PCG_EXTA_I_DAI_PB08_O_ENCODING              0x07
#define PCG_EXTB_I_DAI_PB08_O_ENCODING              0x07
#define SPDIF_PLLCLK_I_DAI_PB08_O_ENCODING          0x07
#define SPIB_CLK_I_DAI_PB08_O_ENCODING              0x07
#define PCG_SYNC_CLKA_I_DAI_PB08_O_ENCODING         0x07
#define PCG_SYNC_CLKB_I_DAI_PB08_O_ENCODING         0x07

#define SPORT0_CLK_I_DAI_PB09_O_ENCODING            0x08
#define SPORT1_CLK_I_DAI_PB09_O_ENCODING            0x08
#define SPORT2_CLK_I_DAI_PB09_O_ENCODING            0x08
#define SPORT3_CLK_I_DAI_PB09_O_ENCODING            0x08
#define SPORT4_CLK_I_DAI_PB09_O_ENCODING            0x08
#define SPORT5_CLK_I_DAI_PB09_O_ENCODING            0x08
#define SRC0_CLK_IP_I_DAI_PB09_O_ENCODING           0x08
#define SRC0_CLK_OP_I_DAI_PB09_O_ENCODING           0x08
#define SRC1_CLK_IP_I_DAI_PB09_O_ENCODING           0x08
#define SRC1_CLK_OP_I_DAI_PB09_O_ENCODING           0x08
#define SRC2_CLK_IP_I_DAI_PB09_O_ENCODING           0x08
#define SRC2_CLK_OP_I_DAI_PB09_O_ENCODING           0x08
#define SRC3_CLK_IP_I_DAI_PB09_O_ENCODING           0x08
#define SRC3_CLK_OP_I_DAI_PB09_O_ENCODING           0x08
#define DIT_CLK_I_DAI_PB09_O_ENCODING               0x08
#define IDP0_CLK_I_DAI_PB09_O_ENCODING              0x08
#define PDAP_CLK_I_DAI_PB09_O_ENCODING              0x08
#define IDP1_CLK_I_DAI_PB09_O_ENCODING              0x08
#define IDP2_CLK_I_DAI_PB09_O_ENCODING              0x08
#define IDP3_CLK_I_DAI_PB09_O_ENCODING              0x08
#define IDP4_CLK_I_DAI_PB09_O_ENCODING              0x08
#define IDP5_CLK_I_DAI_PB09_O_ENCODING              0x08
#define IDP6_CLK_I_DAI_PB09_O_ENCODING              0x08
#define IDP7_CLK_I_DAI_PB09_O_ENCODING              0x08
#define DIT_HFCLK_I_DAI_PB09_O_ENCODING             0x08
#define PCG_EXTA_I_DAI_PB09_O_ENCODING              0x08
#define PCG_EXTB_I_DAI_PB09_O_ENCODING              0x08
#define SPDIF_PLLCLK_I_DAI_PB09_O_ENCODING          0x08
#define SPIB_CLK_I_DAI_PB09_O_ENCODING              0x08
#define PCG_SYNC_CLKA_I_DAI_PB09_O_ENCODING         0x08
#define PCG_SYNC_CLKB_I_DAI_PB09_O_ENCODING         0x08

#define SPORT0_CLK_I_DAI_PB10_O_ENCODING            0x09
#define SPORT1_CLK_I_DAI_PB10_O_ENCODING            0x09
#define SPORT2_CLK_I_DAI_PB10_O_ENCODING            0x09
#define SPORT3_CLK_I_DAI_PB10_O_ENCODING            0x09
#define SPORT4_CLK_I_DAI_PB10_O_ENCODING            0x09
#define SPORT5_CLK_I_DAI_PB10_O_ENCODING            0x09
#define SRC0_CLK_IP_I_DAI_PB10_O_ENCODING           0x09
#define SRC0_CLK_OP_I_DAI_PB10_O_ENCODING           0x09
#define SRC1_CLK_IP_I_DAI_PB10_O_ENCODING           0x09
#define SRC1_CLK_OP_I_DAI_PB10_O_ENCODING           0x09
#define SRC2_CLK_IP_I_DAI_PB10_O_ENCODING           0x09
#define SRC2_CLK_OP_I_DAI_PB10_O_ENCODING           0x09
#define SRC3_CLK_IP_I_DAI_PB10_O_ENCODING           0x09
#define SRC3_CLK_OP_I_DAI_PB10_O_ENCODING           0x09
#define DIT_CLK_I_DAI_PB10_O_ENCODING               0x09
#define IDP0_CLK_I_DAI_PB10_O_ENCODING              0x09
#define PDAP_CLK_I_DAI_PB10_O_ENCODING              0x09
#define IDP1_CLK_I_DAI_PB10_O_ENCODING              0x09
#define IDP2_CLK_I_DAI_PB10_O_ENCODING              0x09
#define IDP3_CLK_I_DAI_PB10_O_ENCODING              0x09
#define IDP4_CLK_I_DAI_PB10_O_ENCODING              0x09
#define IDP5_CLK_I_DAI_PB10_O_ENCODING              0x09
#define IDP6_CLK_I_DAI_PB10_O_ENCODING              0x09
#define IDP7_CLK_I_DAI_PB10_O_ENCODING              0x09
#define DIT_HFCLK_I_DAI_PB10_O_ENCODING             0x09
#define PCG_EXTA_I_DAI_PB10_O_ENCODING              0x09
#define PCG_EXTB_I_DAI_PB10_O_ENCODING              0x09
#define SPDIF_PLLCLK_I_DAI_PB10_O_ENCODING          0x09
#define SPIB_CLK_I_DAI_PB10_O_ENCODING              0x09
#define PCG_SYNC_CLKA_I_DAI_PB10_O_ENCODING         0x09
#define PCG_SYNC_CLKB_I_DAI_PB10_O_ENCODING         0x09

#define SPORT0_CLK_I_DAI_PB11_O_ENCODING            0x0A
#define SPORT1_CLK_I_DAI_PB11_O_ENCODING            0x0A
#define SPORT2_CLK_I_DAI_PB11_O_ENCODING            0x0A
#define SPORT3_CLK_I_DAI_PB11_O_ENCODING            0x0A
#define SPORT4_CLK_I_DAI_PB11_O_ENCODING            0x0A
#define SPORT5_CLK_I_DAI_PB11_O_ENCODING            0x0A
#define SRC0_CLK_IP_I_DAI_PB11_O_ENCODING           0x0A
#define SRC0_CLK_OP_I_DAI_PB11_O_ENCODING           0x0A
#define SRC1_CLK_IP_I_DAI_PB11_O_ENCODING           0x0A
#define SRC1_CLK_OP_I_DAI_PB11_O_ENCODING           0x0A
#define SRC2_CLK_IP_I_DAI_PB11_O_ENCODING           0x0A
#define SRC2_CLK_OP_I_DAI_PB11_O_ENCODING           0x0A
#define SRC3_CLK_IP_I_DAI_PB11_O_ENCODING           0x0A
#define SRC3_CLK_OP_I_DAI_PB11_O_ENCODING           0x0A
#define DIT_CLK_I_DAI_PB11_O_ENCODING               0x0A
#define IDP0_CLK_I_DAI_PB11_O_ENCODING              0x0A
#define PDAP_CLK_I_DAI_PB11_O_ENCODING              0x0A
#define IDP1_CLK_I_DAI_PB11_O_ENCODING              0x0A
#define IDP2_CLK_I_DAI_PB11_O_ENCODING              0x0A
#define IDP3_CLK_I_DAI_PB11_O_ENCODING              0x0A
#define IDP4_CLK_I_DAI_PB11_O_ENCODING              0x0A
#define IDP5_CLK_I_DAI_PB11_O_ENCODING              0x0A
#define IDP6_CLK_I_DAI_PB11_O_ENCODING              0x0A
#define IDP7_CLK_I_DAI_PB11_O_ENCODING              0x0A
#define DIT_HFCLK_I_DAI_PB11_O_ENCODING             0x0A
#define PCG_EXTA_I_DAI_PB11_O_ENCODING              0x0A
#define PCG_EXTB_I_DAI_PB11_O_ENCODING              0x0A
#define SPDIF_PLLCLK_I_DAI_PB11_O_ENCODING          0x0A
#define SPIB_CLK_I_DAI_PB11_O_ENCODING              0x0A
#define PCG_SYNC_CLKA_I_DAI_PB11_O_ENCODING         0x0A
#define PCG_SYNC_CLKB_I_DAI_PB11_O_ENCODING         0x0A

#define SPORT0_CLK_I_DAI_PB12_O_ENCODING            0x0B
#define SPORT1_CLK_I_DAI_PB12_O_ENCODING            0x0B
#define SPORT2_CLK_I_DAI_PB12_O_ENCODING            0x0B
#define SPORT3_CLK_I_DAI_PB12_O_ENCODING            0x0B
#define SPORT4_CLK_I_DAI_PB12_O_ENCODING            0x0B
#define SPORT5_CLK_I_DAI_PB12_O_ENCODING            0x0B
#define SRC0_CLK_IP_I_DAI_PB12_O_ENCODING           0x0B
#define SRC0_CLK_OP_I_DAI_PB12_O_ENCODING           0x0B
#define SRC1_CLK_IP_I_DAI_PB12_O_ENCODING           0x0B
#define SRC1_CLK_OP_I_DAI_PB12_O_ENCODING           0x0B
#define SRC2_CLK_IP_I_DAI_PB12_O_ENCODING           0x0B
#define SRC2_CLK_OP_I_DAI_PB12_O_ENCODING           0x0B
#define SRC3_CLK_IP_I_DAI_PB12_O_ENCODING           0x0B
#define SRC3_CLK_OP_I_DAI_PB12_O_ENCODING           0x0B
#define DIT_CLK_I_DAI_PB12_O_ENCODING               0x0B
#define IDP0_CLK_I_DAI_PB12_O_ENCODING              0x0B
#define PDAP_CLK_I_DAI_PB12_O_ENCODING              0x0B
#define IDP1_CLK_I_DAI_PB12_O_ENCODING              0x0B
#define IDP2_CLK_I_DAI_PB12_O_ENCODING              0x0B
#define IDP3_CLK_I_DAI_PB12_O_ENCODING              0x0B
#define IDP4_CLK_I_DAI_PB12_O_ENCODING              0x0B
#define IDP5_CLK_I_DAI_PB12_O_ENCODING              0x0B
#define IDP6_CLK_I_DAI_PB12_O_ENCODING              0x0B
#define IDP7_CLK_I_DAI_PB12_O_ENCODING              0x0B
#define DIT_HFCLK_I_DAI_PB12_O_ENCODING             0x0B
#define PCG_EXTA_I_DAI_PB12_O_ENCODING              0x0B
#define PCG_EXTB_I_DAI_PB12_O_ENCODING              0x0B
#define SPDIF_PLLCLK_I_DAI_PB12_O_ENCODING          0x0B
#define SPIB_CLK_I_DAI_PB12_O_ENCODING              0x0B
#define PCG_SYNC_CLKA_I_DAI_PB12_O_ENCODING         0x0B
#define PCG_SYNC_CLKB_I_DAI_PB12_O_ENCODING         0x0B

#define SPORT0_CLK_I_DAI_PB13_O_ENCODING            0x0C
#define SPORT1_CLK_I_DAI_PB13_O_ENCODING            0x0C
#define SPORT2_CLK_I_DAI_PB13_O_ENCODING            0x0C
#define SPORT3_CLK_I_DAI_PB13_O_ENCODING            0x0C
#define SPORT4_CLK_I_DAI_PB13_O_ENCODING            0x0C
#define SPORT5_CLK_I_DAI_PB13_O_ENCODING            0x0C
#define SRC0_CLK_IP_I_DAI_PB13_O_ENCODING           0x0C
#define SRC0_CLK_OP_I_DAI_PB13_O_ENCODING           0x0C
#define SRC1_CLK_IP_I_DAI_PB13_O_ENCODING           0x0C
#define SRC1_CLK_OP_I_DAI_PB13_O_ENCODING           0x0C
#define SRC2_CLK_IP_I_DAI_PB13_O_ENCODING           0x0C
#define SRC2_CLK_OP_I_DAI_PB13_O_ENCODING           0x0C
#define SRC3_CLK_IP_I_DAI_PB13_O_ENCODING           0x0C
#define SRC3_CLK_OP_I_DAI_PB13_O_ENCODING           0x0C
#define DIT_CLK_I_DAI_PB13_O_ENCODING               0x0C
#define IDP0_CLK_I_DAI_PB13_O_ENCODING              0x0C
#define PDAP_CLK_I_DAI_PB13_O_ENCODING              0x0C
#define IDP1_CLK_I_DAI_PB13_O_ENCODING              0x0C
#define IDP2_CLK_I_DAI_PB13_O_ENCODING              0x0C
#define IDP3_CLK_I_DAI_PB13_O_ENCODING              0x0C
#define IDP4_CLK_I_DAI_PB13_O_ENCODING              0x0C
#define IDP5_CLK_I_DAI_PB13_O_ENCODING              0x0C
#define IDP6_CLK_I_DAI_PB13_O_ENCODING              0x0C
#define IDP7_CLK_I_DAI_PB13_O_ENCODING              0x0C
#define DIT_HFCLK_I_DAI_PB13_O_ENCODING             0x0C
#define PCG_EXTA_I_DAI_PB13_O_ENCODING              0x0C
#define PCG_EXTB_I_DAI_PB13_O_ENCODING              0x0C
#define SPDIF_PLLCLK_I_DAI_PB13_O_ENCODING          0x0C
#define SPIB_CLK_I_DAI_PB13_O_ENCODING              0x0C
#define PCG_SYNC_CLKA_I_DAI_PB13_O_ENCODING         0x0C
#define PCG_SYNC_CLKB_I_DAI_PB13_O_ENCODING         0x0C

#define SPORT0_CLK_I_DAI_PB14_O_ENCODING            0x0D
#define SPORT1_CLK_I_DAI_PB14_O_ENCODING            0x0D
#define SPORT2_CLK_I_DAI_PB14_O_ENCODING            0x0D
#define SPORT3_CLK_I_DAI_PB14_O_ENCODING            0x0D
#define SPORT4_CLK_I_DAI_PB14_O_ENCODING            0x0D
#define SPORT5_CLK_I_DAI_PB14_O_ENCODING            0x0D
#define SRC0_CLK_IP_I_DAI_PB14_O_ENCODING           0x0D
#define SRC0_CLK_OP_I_DAI_PB14_O_ENCODING           0x0D
#define SRC1_CLK_IP_I_DAI_PB14_O_ENCODING           0x0D
#define SRC1_CLK_OP_I_DAI_PB14_O_ENCODING           0x0D
#define SRC2_CLK_IP_I_DAI_PB14_O_ENCODING           0x0D
#define SRC2_CLK_OP_I_DAI_PB14_O_ENCODING           0x0D
#define SRC3_CLK_IP_I_DAI_PB14_O_ENCODING           0x0D
#define SRC3_CLK_OP_I_DAI_PB14_O_ENCODING           0x0D
#define DIT_CLK_I_DAI_PB14_O_ENCODING               0x0D
#define IDP0_CLK_I_DAI_PB14_O_ENCODING              0x0D
#define PDAP_CLK_I_DAI_PB14_O_ENCODING              0x0D
#define IDP1_CLK_I_DAI_PB14_O_ENCODING              0x0D
#define IDP2_CLK_I_DAI_PB14_O_ENCODING              0x0D
#define IDP3_CLK_I_DAI_PB14_O_ENCODING              0x0D
#define IDP4_CLK_I_DAI_PB14_O_ENCODING              0x0D
#define IDP5_CLK_I_DAI_PB14_O_ENCODING              0x0D
#define IDP6_CLK_I_DAI_PB14_O_ENCODING              0x0D
#define IDP7_CLK_I_DAI_PB14_O_ENCODING              0x0D
#define DIT_HFCLK_I_DAI_PB14_O_ENCODING             0x0D
#define PCG_EXTA_I_DAI_PB14_O_ENCODING              0x0D
#define PCG_EXTB_I_DAI_PB14_O_ENCODING              0x0D
#define SPDIF_PLLCLK_I_DAI_PB14_O_ENCODING          0x0D
#define SPIB_CLK_I_DAI_PB14_O_ENCODING              0x0D
#define PCG_SYNC_CLKA_I_DAI_PB14_O_ENCODING         0x0D
#define PCG_SYNC_CLKB_I_DAI_PB14_O_ENCODING         0x0D

#define SPORT0_CLK_I_DAI_PB15_O_ENCODING            0x0E
#define SPORT1_CLK_I_DAI_PB15_O_ENCODING            0x0E
#define SPORT2_CLK_I_DAI_PB15_O_ENCODING            0x0E
#define SPORT3_CLK_I_DAI_PB15_O_ENCODING            0x0E
#define SPORT4_CLK_I_DAI_PB15_O_ENCODING            0x0E
#define SPORT5_CLK_I_DAI_PB15_O_ENCODING            0x0E
#define SRC0_CLK_IP_I_DAI_PB15_O_ENCODING           0x0E
#define SRC0_CLK_OP_I_DAI_PB15_O_ENCODING           0x0E
#define SRC1_CLK_IP_I_DAI_PB15_O_ENCODING           0x0E
#define SRC1_CLK_OP_I_DAI_PB15_O_ENCODING           0x0E
#define SRC2_CLK_IP_I_DAI_PB15_O_ENCODING           0x0E
#define SRC2_CLK_OP_I_DAI_PB15_O_ENCODING           0x0E
#define SRC3_CLK_IP_I_DAI_PB15_O_ENCODING           0x0E
#define SRC3_CLK_OP_I_DAI_PB15_O_ENCODING           0x0E
#define IDP0_CLK_I_DAI_PB15_O_ENCODING              0x0E
#define PDAP_CLK_I_DAI_PB15_O_ENCODING              0x0E
#define IDP1_CLK_I_DAI_PB15_O_ENCODING              0x0E
#define IDP2_CLK_I_DAI_PB15_O_ENCODING              0x0E
#define IDP3_CLK_I_DAI_PB15_O_ENCODING              0x0E
#define IDP4_CLK_I_DAI_PB15_O_ENCODING              0x0E
#define IDP5_CLK_I_DAI_PB15_O_ENCODING              0x0E
#define IDP6_CLK_I_DAI_PB15_O_ENCODING              0x0E
#define IDP7_CLK_I_DAI_PB15_O_ENCODING              0x0E
#define DIT_HFCLK_I_DAI_PB15_O_ENCODING             0x0E
#define PCG_EXTA_I_DAI_PB15_O_ENCODING              0x0E
#define PCG_EXTB_I_DAI_PB15_O_ENCODING              0x0E
#define SPDIF_PLLCLK_I_DAI_PB15_O_ENCODING          0x0E
#define SPIB_CLK_I_DAI_PB15_O_ENCODING              0x0E
#define PCG_SYNC_CLKA_I_DAI_PB15_O_ENCODING         0x0E
#define PCG_SYNC_CLKB_I_DAI_PB15_O_ENCODING         0x0E

#define SPORT0_CLK_I_DAI_PB16_O_ENCODING            0x0F
#define SPORT1_CLK_I_DAI_PB16_O_ENCODING            0x0F
#define SPORT2_CLK_I_DAI_PB16_O_ENCODING            0x0F
#define SPORT3_CLK_I_DAI_PB16_O_ENCODING            0x0F
#define SPORT4_CLK_I_DAI_PB16_O_ENCODING            0x0F
#define SPORT5_CLK_I_DAI_PB16_O_ENCODING            0x0F
#define SRC0_CLK_IP_I_DAI_PB16_O_ENCODING           0x0F
#define SRC0_CLK_OP_I_DAI_PB16_O_ENCODING           0x0F
#define SRC1_CLK_IP_I_DAI_PB16_O_ENCODING           0x0F
#define SRC1_CLK_OP_I_DAI_PB16_O_ENCODING           0x0F
#define SRC2_CLK_IP_I_DAI_PB16_O_ENCODING           0x0F
#define SRC2_CLK_OP_I_DAI_PB16_O_ENCODING           0x0F
#define SRC3_CLK_IP_I_DAI_PB16_O_ENCODING           0x0F
#define SRC3_CLK_OP_I_DAI_PB16_O_ENCODING           0x0F
#define IDP0_CLK_I_DAI_PB16_O_ENCODING              0x0F
#define PDAP_CLK_I_DAI_PB16_O_ENCODING              0x0F
#define IDP1_CLK_I_DAI_PB16_O_ENCODING              0x0F
#define IDP2_CLK_I_DAI_PB16_O_ENCODING              0x0F
#define IDP3_CLK_I_DAI_PB16_O_ENCODING              0x0F
#define IDP4_CLK_I_DAI_PB16_O_ENCODING              0x0F
#define IDP5_CLK_I_DAI_PB16_O_ENCODING              0x0F
#define IDP6_CLK_I_DAI_PB16_O_ENCODING              0x0F
#define IDP7_CLK_I_DAI_PB16_O_ENCODING              0x0F
#define DIT_HFCLK_I_DAI_PB16_O_ENCODING             0x0F
#define PCG_EXTA_I_DAI_PB16_O_ENCODING              0x0F
#define PCG_EXTB_I_DAI_PB16_O_ENCODING              0x0F
#define SPDIF_PLLCLK_I_DAI_PB16_O_ENCODING          0x0F
#define SPIB_CLK_I_DAI_PB16_O_ENCODING              0x0F
#define PCG_SYNC_CLKA_I_DAI_PB16_O_ENCODING         0x0F
#define PCG_SYNC_CLKB_I_DAI_PB16_O_ENCODING         0x0F

#define SPORT0_CLK_I_DAI_PB17_O_ENCODING            0x10
#define SPORT1_CLK_I_DAI_PB17_O_ENCODING            0x10
#define SPORT2_CLK_I_DAI_PB17_O_ENCODING            0x10
#define SPORT3_CLK_I_DAI_PB17_O_ENCODING            0x10
#define SPORT4_CLK_I_DAI_PB17_O_ENCODING            0x10
#define SPORT5_CLK_I_DAI_PB17_O_ENCODING            0x10
#define SRC0_CLK_IP_I_DAI_PB17_O_ENCODING           0x10
#define SRC0_CLK_OP_I_DAI_PB17_O_ENCODING           0x10
#define SRC1_CLK_IP_I_DAI_PB17_O_ENCODING           0x10
#define SRC1_CLK_OP_I_DAI_PB17_O_ENCODING           0x10
#define SRC2_CLK_IP_I_DAI_PB17_O_ENCODING           0x10
#define SRC2_CLK_OP_I_DAI_PB17_O_ENCODING           0x10
#define SRC3_CLK_IP_I_DAI_PB17_O_ENCODING           0x10
#define SRC3_CLK_OP_I_DAI_PB17_O_ENCODING           0x10
#define IDP0_CLK_I_DAI_PB17_O_ENCODING              0x10
#define PDAP_CLK_I_DAI_PB17_O_ENCODING              0x10
#define IDP1_CLK_I_DAI_PB17_O_ENCODING              0x10
#define IDP2_CLK_I_DAI_PB17_O_ENCODING              0x10
#define IDP3_CLK_I_DAI_PB17_O_ENCODING              0x10
#define IDP4_CLK_I_DAI_PB17_O_ENCODING              0x10
#define IDP5_CLK_I_DAI_PB17_O_ENCODING              0x10
#define IDP6_CLK_I_DAI_PB17_O_ENCODING              0x10
#define IDP7_CLK_I_DAI_PB17_O_ENCODING              0x10
#define DIT_HFCLK_I_DAI_PB17_O_ENCODING             0x10
#define PCG_EXTA_I_DAI_PB17_O_ENCODING              0x10
#define PCG_EXTB_I_DAI_PB17_O_ENCODING              0x10
#define SPDIF_PLLCLK_I_DAI_PB17_O_ENCODING          0x10
#define SPIB_CLK_I_DAI_PB17_O_ENCODING              0x10
#define PCG_SYNC_CLKA_I_DAI_PB17_O_ENCODING         0x10
#define PCG_SYNC_CLKB_I_DAI_PB17_O_ENCODING         0x10

#define SPORT0_CLK_I_DAI_PB18_O_ENCODING            0x11
#define SPORT1_CLK_I_DAI_PB18_O_ENCODING            0x11
#define SPORT2_CLK_I_DAI_PB18_O_ENCODING            0x11
#define SPORT3_CLK_I_DAI_PB18_O_ENCODING            0x11
#define SPORT4_CLK_I_DAI_PB18_O_ENCODING            0x11
#define SPORT5_CLK_I_DAI_PB18_O_ENCODING            0x11
#define SRC0_CLK_IP_I_DAI_PB18_O_ENCODING           0x11
#define SRC0_CLK_OP_I_DAI_PB18_O_ENCODING           0x11
#define SRC1_CLK_IP_I_DAI_PB18_O_ENCODING           0x11
#define SRC1_CLK_OP_I_DAI_PB18_O_ENCODING           0x11
#define SRC2_CLK_IP_I_DAI_PB18_O_ENCODING           0x11
#define SRC2_CLK_OP_I_DAI_PB18_O_ENCODING           0x11
#define SRC3_CLK_IP_I_DAI_PB18_O_ENCODING           0x11
#define SRC3_CLK_OP_I_DAI_PB18_O_ENCODING           0x11
#define IDP0_CLK_I_DAI_PB18_O_ENCODING              0x11
#define PDAP_CLK_I_DAI_PB18_O_ENCODING              0x11
#define IDP1_CLK_I_DAI_PB18_O_ENCODING              0x11
#define IDP2_CLK_I_DAI_PB18_O_ENCODING              0x11
#define IDP3_CLK_I_DAI_PB18_O_ENCODING              0x11
#define IDP4_CLK_I_DAI_PB18_O_ENCODING              0x11
#define IDP5_CLK_I_DAI_PB18_O_ENCODING              0x11
#define IDP6_CLK_I_DAI_PB18_O_ENCODING              0x11
#define IDP7_CLK_I_DAI_PB18_O_ENCODING              0x11
#define DIT_HFCLK_I_DAI_PB18_O_ENCODING             0x11
#define PCG_EXTA_I_DAI_PB18_O_ENCODING              0x11
#define PCG_EXTB_I_DAI_PB18_O_ENCODING              0x11
#define SPDIF_PLLCLK_I_DAI_PB18_O_ENCODING          0x11
#define SPIB_CLK_I_DAI_PB18_O_ENCODING              0x11
#define PCG_SYNC_CLKA_I_DAI_PB18_O_ENCODING         0x11
#define PCG_SYNC_CLKB_I_DAI_PB18_O_ENCODING         0x11

#define SPORT0_CLK_I_DAI_PB19_O_ENCODING            0x12
#define SPORT1_CLK_I_DAI_PB19_O_ENCODING            0x12
#define SPORT2_CLK_I_DAI_PB19_O_ENCODING            0x12
#define SPORT3_CLK_I_DAI_PB19_O_ENCODING            0x12
#define SPORT4_CLK_I_DAI_PB19_O_ENCODING            0x12
#define SPORT5_CLK_I_DAI_PB19_O_ENCODING            0x12
#define SRC0_CLK_IP_I_DAI_PB19_O_ENCODING           0x12
#define SRC0_CLK_OP_I_DAI_PB19_O_ENCODING           0x12
#define SRC1_CLK_IP_I_DAI_PB19_O_ENCODING           0x12
#define SRC1_CLK_OP_I_DAI_PB19_O_ENCODING           0x12
#define SRC2_CLK_IP_I_DAI_PB19_O_ENCODING           0x12
#define SRC2_CLK_OP_I_DAI_PB19_O_ENCODING           0x12
#define SRC3_CLK_IP_I_DAI_PB19_O_ENCODING           0x12
#define SRC3_CLK_OP_I_DAI_PB19_O_ENCODING           0x12
#define IDP0_CLK_I_DAI_PB19_O_ENCODING              0x12
#define PDAP_CLK_I_DAI_PB19_O_ENCODING              0x12
#define IDP1_CLK_I_DAI_PB19_O_ENCODING              0x12
#define IDP2_CLK_I_DAI_PB19_O_ENCODING              0x12
#define IDP3_CLK_I_DAI_PB19_O_ENCODING              0x12
#define IDP4_CLK_I_DAI_PB19_O_ENCODING              0x12
#define IDP5_CLK_I_DAI_PB19_O_ENCODING              0x12
#define IDP6_CLK_I_DAI_PB19_O_ENCODING              0x12
#define IDP7_CLK_I_DAI_PB19_O_ENCODING              0x12
#define DIT_HFCLK_I_DAI_PB19_O_ENCODING             0x12
#define PCG_EXTA_I_DAI_PB19_O_ENCODING              0x12
#define PCG_EXTB_I_DAI_PB19_O_ENCODING              0x12
#define SPDIF_PLLCLK_I_DAI_PB19_O_ENCODING          0x12
#define SPIB_CLK_I_DAI_PB19_O_ENCODING              0x12
#define PCG_SYNC_CLKA_I_DAI_PB19_O_ENCODING         0x12
#define PCG_SYNC_CLKB_I_DAI_PB19_O_ENCODING         0x12

#define SPORT0_CLK_I_DAI_PB20_O_ENCODING            0x13
#define SPORT1_CLK_I_DAI_PB20_O_ENCODING            0x13
#define SPORT2_CLK_I_DAI_PB20_O_ENCODING            0x13
#define SPORT3_CLK_I_DAI_PB20_O_ENCODING            0x13
#define SPORT4_CLK_I_DAI_PB20_O_ENCODING            0x13
#define SPORT5_CLK_I_DAI_PB20_O_ENCODING            0x13
#define SRC0_CLK_IP_I_DAI_PB20_O_ENCODING           0x13
#define SRC0_CLK_OP_I_DAI_PB20_O_ENCODING           0x13
#define SRC1_CLK_IP_I_DAI_PB20_O_ENCODING           0x13
#define SRC1_CLK_OP_I_DAI_PB20_O_ENCODING           0x13
#define SRC2_CLK_IP_I_DAI_PB20_O_ENCODING           0x13
#define SRC2_CLK_OP_I_DAI_PB20_O_ENCODING           0x13
#define SRC3_CLK_IP_I_DAI_PB20_O_ENCODING           0x13
#define SRC3_CLK_OP_I_DAI_PB20_O_ENCODING           0x13
#define IDP0_CLK_I_DAI_PB20_O_ENCODING              0x13
#define PDAP_CLK_I_DAI_PB20_O_ENCODING              0x13
#define IDP1_CLK_I_DAI_PB20_O_ENCODING              0x13
#define IDP2_CLK_I_DAI_PB20_O_ENCODING              0x13
#define IDP3_CLK_I_DAI_PB20_O_ENCODING              0x13
#define IDP4_CLK_I_DAI_PB20_O_ENCODING              0x13
#define IDP5_CLK_I_DAI_PB20_O_ENCODING              0x13
#define IDP6_CLK_I_DAI_PB20_O_ENCODING              0x13
#define IDP7_CLK_I_DAI_PB20_O_ENCODING              0x13
#define DIT_HFCLK_I_DAI_PB20_O_ENCODING             0x13
#define PCG_EXTA_I_DAI_PB20_O_ENCODING              0x13
#define PCG_EXTB_I_DAI_PB20_O_ENCODING              0x13
#define SPDIF_PLLCLK_I_DAI_PB20_O_ENCODING          0x13
#define SPIB_CLK_I_DAI_PB20_O_ENCODING              0x13
#define PCG_SYNC_CLKA_I_DAI_PB20_O_ENCODING         0x13
#define PCG_SYNC_CLKB_I_DAI_PB20_O_ENCODING         0x13

#define SPORT0_CLK_I_SPORT0_CLK_O_ENCODING            0x14
#define SPORT1_CLK_I_SPORT0_CLK_O_ENCODING            0x14
#define SPORT2_CLK_I_SPORT0_CLK_O_ENCODING            0x14
#define SPORT3_CLK_I_SPORT0_CLK_O_ENCODING            0x14
#define SPORT4_CLK_I_SPORT0_CLK_O_ENCODING            0x14
#define SPORT5_CLK_I_SPORT0_CLK_O_ENCODING            0x14
#define SRC0_CLK_IP_I_SPORT0_CLK_O_ENCODING           0x14
#define SRC0_CLK_OP_I_SPORT0_CLK_O_ENCODING           0x14
#define SRC1_CLK_IP_I_SPORT0_CLK_O_ENCODING           0x14
#define SRC1_CLK_OP_I_SPORT0_CLK_O_ENCODING           0x14
#define SRC2_CLK_IP_I_SPORT0_CLK_O_ENCODING           0x14
#define SRC2_CLK_OP_I_SPORT0_CLK_O_ENCODING           0x14
#define SRC3_CLK_IP_I_SPORT0_CLK_O_ENCODING           0x14
#define SRC3_CLK_OP_I_SPORT0_CLK_O_ENCODING           0x14
#define IDP0_CLK_I_SPORT0_CLK_O_ENCODING              0x14
#define PDAP_CLK_I_SPORT0_CLK_O_ENCODING              0x14
#define IDP1_CLK_I_SPORT0_CLK_O_ENCODING              0x14
#define IDP2_CLK_I_SPORT0_CLK_O_ENCODING              0x14
#define IDP3_CLK_I_SPORT0_CLK_O_ENCODING              0x14
#define IDP4_CLK_I_SPORT0_CLK_O_ENCODING              0x14
#define IDP5_CLK_I_SPORT0_CLK_O_ENCODING              0x14
#define IDP6_CLK_I_SPORT0_CLK_O_ENCODING              0x14
#define IDP7_CLK_I_SPORT0_CLK_O_ENCODING              0x14
#define DIT_HFCLK_I_SPORT0_CLK_O_ENCODING             0x14
#define PCG_EXTA_I_SPORT0_CLK_O_ENCODING              0x14
#define PCG_EXTB_I_SPORT0_CLK_O_ENCODING              0x14
#define SPDIF_PLLCLK_I_SPORT0_CLK_O_ENCODING          0x14
#define SPIB_CLK_I_SPORT0_CLK_O_ENCODING              0x14
#define PCG_SYNC_CLKA_I_SPORT0_CLK_O_ENCODING         0x14
#define PCG_SYNC_CLKB_I_SPORT0_CLK_O_ENCODING         0x14
 
#define SPORT0_CLK_I_SPORT1_CLK_O_ENCODING            0x15
#define SPORT1_CLK_I_SPORT1_CLK_O_ENCODING            0x15
#define SPORT2_CLK_I_SPORT1_CLK_O_ENCODING            0x15
#define SPORT3_CLK_I_SPORT1_CLK_O_ENCODING            0x15
#define SPORT4_CLK_I_SPORT1_CLK_O_ENCODING            0x15
#define SPORT5_CLK_I_SPORT1_CLK_O_ENCODING            0x15
#define SRC0_CLK_IP_I_SPORT1_CLK_O_ENCODING           0x15
#define SRC0_CLK_OP_I_SPORT1_CLK_O_ENCODING           0x15
#define SRC1_CLK_IP_I_SPORT1_CLK_O_ENCODING           0x15
#define SRC1_CLK_OP_I_SPORT1_CLK_O_ENCODING           0x15
#define SRC2_CLK_IP_I_SPORT1_CLK_O_ENCODING           0x15
#define SRC2_CLK_OP_I_SPORT1_CLK_O_ENCODING           0x15
#define SRC3_CLK_IP_I_SPORT1_CLK_O_ENCODING           0x15
#define SRC3_CLK_OP_I_SPORT1_CLK_O_ENCODING           0x15
#define IDP0_CLK_I_SPORT1_CLK_O_ENCODING              0x15
#define PDAP_CLK_I_SPORT1_CLK_O_ENCODING              0x15
#define IDP1_CLK_I_SPORT1_CLK_O_ENCODING              0x15
#define IDP2_CLK_I_SPORT1_CLK_O_ENCODING              0x15
#define IDP3_CLK_I_SPORT1_CLK_O_ENCODING              0x15
#define IDP4_CLK_I_SPORT1_CLK_O_ENCODING              0x15
#define IDP5_CLK_I_SPORT1_CLK_O_ENCODING              0x15
#define IDP6_CLK_I_SPORT1_CLK_O_ENCODING              0x15
#define IDP7_CLK_I_SPORT1_CLK_O_ENCODING              0x15
#define DIT_HFCLK_I_SPORT1_CLK_O_ENCODING             0x15
#define PCG_EXTA_I_SPORT1_CLK_O_ENCODING              0x15
#define PCG_EXTB_I_SPORT1_CLK_O_ENCODING              0x15
#define SPDIF_PLLCLK_I_SPORT1_CLK_O_ENCODING          0x15
#define SPIB_CLK_I_SPORT1_CLK_O_ENCODING              0x15
#define PCG_SYNC_CLKA_I_SPORT1_CLK_O_ENCODING         0x15
#define PCG_SYNC_CLKB_I_SPORT1_CLK_O_ENCODING         0x15
 
#define SPORT0_CLK_I_SPORT2_CLK_O_ENCODING            0x16
#define SPORT1_CLK_I_SPORT2_CLK_O_ENCODING            0x16
#define SPORT2_CLK_I_SPORT2_CLK_O_ENCODING            0x16
#define SPORT3_CLK_I_SPORT2_CLK_O_ENCODING            0x16
#define SPORT4_CLK_I_SPORT2_CLK_O_ENCODING            0x16
#define SPORT5_CLK_I_SPORT2_CLK_O_ENCODING            0x16
#define SRC0_CLK_IP_I_SPORT2_CLK_O_ENCODING           0x16
#define SRC0_CLK_OP_I_SPORT2_CLK_O_ENCODING           0x16
#define SRC1_CLK_IP_I_SPORT2_CLK_O_ENCODING           0x16
#define SRC1_CLK_OP_I_SPORT2_CLK_O_ENCODING           0x16
#define SRC2_CLK_IP_I_SPORT2_CLK_O_ENCODING           0x16
#define SRC2_CLK_OP_I_SPORT2_CLK_O_ENCODING           0x16
#define SRC3_CLK_IP_I_SPORT2_CLK_O_ENCODING           0x16
#define SRC3_CLK_OP_I_SPORT2_CLK_O_ENCODING           0x16
#define IDP0_CLK_I_SPORT2_CLK_O_ENCODING              0x16
#define PDAP_CLK_I_SPORT2_CLK_O_ENCODING              0x16
#define IDP1_CLK_I_SPORT2_CLK_O_ENCODING              0x16
#define IDP2_CLK_I_SPORT2_CLK_O_ENCODING              0x16
#define IDP3_CLK_I_SPORT2_CLK_O_ENCODING              0x16
#define IDP4_CLK_I_SPORT2_CLK_O_ENCODING              0x16
#define IDP5_CLK_I_SPORT2_CLK_O_ENCODING              0x16
#define IDP6_CLK_I_SPORT2_CLK_O_ENCODING              0x16
#define IDP7_CLK_I_SPORT2_CLK_O_ENCODING              0x16
#define DIT_HFCLK_I_SPORT2_CLK_O_ENCODING             0x16
#define PCG_EXTA_I_SPORT2_CLK_O_ENCODING              0x16
#define PCG_EXTB_I_SPORT2_CLK_O_ENCODING              0x16
#define SPDIF_PLLCLK_I_SPORT2_CLK_O_ENCODING          0x16
#define SPIB_CLK_I_SPORT2_CLK_O_ENCODING              0x16
#define PCG_SYNC_CLKA_I_SPORT2_CLK_O_ENCODING         0x16
#define PCG_SYNC_CLKB_I_SPORT2_CLK_O_ENCODING         0x16

#define SPORT0_CLK_I_SPORT3_CLK_O_ENCODING            0x17
#define SPORT1_CLK_I_SPORT3_CLK_O_ENCODING            0x17
#define SPORT2_CLK_I_SPORT3_CLK_O_ENCODING            0x17
#define SPORT3_CLK_I_SPORT3_CLK_O_ENCODING            0x17
#define SPORT4_CLK_I_SPORT3_CLK_O_ENCODING            0x17
#define SPORT5_CLK_I_SPORT3_CLK_O_ENCODING            0x17
#define SRC0_CLK_IP_I_SPORT3_CLK_O_ENCODING           0x17
#define SRC0_CLK_OP_I_SPORT3_CLK_O_ENCODING           0x17
#define SRC1_CLK_IP_I_SPORT3_CLK_O_ENCODING           0x17
#define SRC1_CLK_OP_I_SPORT3_CLK_O_ENCODING           0x17
#define SRC2_CLK_IP_I_SPORT3_CLK_O_ENCODING           0x17
#define SRC2_CLK_OP_I_SPORT3_CLK_O_ENCODING           0x17
#define SRC3_CLK_IP_I_SPORT3_CLK_O_ENCODING           0x17
#define SRC3_CLK_OP_I_SPORT3_CLK_O_ENCODING           0x17
#define IDP0_CLK_I_SPORT3_CLK_O_ENCODING              0x17
#define PDAP_CLK_I_SPORT3_CLK_O_ENCODING              0x17
#define IDP1_CLK_I_SPORT3_CLK_O_ENCODING              0x17
#define IDP2_CLK_I_SPORT3_CLK_O_ENCODING              0x17
#define IDP3_CLK_I_SPORT3_CLK_O_ENCODING              0x17
#define IDP4_CLK_I_SPORT3_CLK_O_ENCODING              0x17
#define IDP5_CLK_I_SPORT3_CLK_O_ENCODING              0x17
#define IDP6_CLK_I_SPORT3_CLK_O_ENCODING              0x17
#define IDP7_CLK_I_SPORT3_CLK_O_ENCODING              0x17
#define DIT_HFCLK_I_SPORT3_CLK_O_ENCODING             0x17
#define PCG_EXTA_I_SPORT3_CLK_O_ENCODING              0x17
#define PCG_EXTB_I_SPORT3_CLK_O_ENCODING              0x17
#define SPDIF_PLLCLK_I_SPORT3_CLK_O_ENCODING          0x17
#define SPIB_CLK_I_SPORT3_CLK_O_ENCODING              0x17
#define PCG_SYNC_CLKA_I_SPORT3_CLK_O_ENCODING         0x17
#define PCG_SYNC_CLKB_I_SPORT3_CLK_O_ENCODING         0x17

#define SPORT0_CLK_I_SPORT4_CLK_O_ENCODING            0x18
#define SPORT1_CLK_I_SPORT4_CLK_O_ENCODING            0x18
#define SPORT2_CLK_I_SPORT4_CLK_O_ENCODING            0x18
#define SPORT3_CLK_I_SPORT4_CLK_O_ENCODING            0x18
#define SPORT4_CLK_I_SPORT4_CLK_O_ENCODING            0x18
#define SPORT5_CLK_I_SPORT4_CLK_O_ENCODING            0x18
#define SRC0_CLK_IP_I_SPORT4_CLK_O_ENCODING           0x18
#define SRC0_CLK_OP_I_SPORT4_CLK_O_ENCODING           0x18
#define SRC1_CLK_IP_I_SPORT4_CLK_O_ENCODING           0x18
#define SRC1_CLK_OP_I_SPORT4_CLK_O_ENCODING           0x18
#define SRC2_CLK_IP_I_SPORT4_CLK_O_ENCODING           0x18
#define SRC2_CLK_OP_I_SPORT4_CLK_O_ENCODING           0x18
#define SRC3_CLK_IP_I_SPORT4_CLK_O_ENCODING           0x18
#define SRC3_CLK_OP_I_SPORT4_CLK_O_ENCODING           0x18
#define IDP0_CLK_I_SPORT4_CLK_O_ENCODING              0x18
#define PDAP_CLK_I_SPORT4_CLK_O_ENCODING              0x18
#define IDP1_CLK_I_SPORT4_CLK_O_ENCODING              0x18
#define IDP2_CLK_I_SPORT4_CLK_O_ENCODING              0x18
#define IDP3_CLK_I_SPORT4_CLK_O_ENCODING              0x18
#define IDP4_CLK_I_SPORT4_CLK_O_ENCODING              0x18
#define IDP5_CLK_I_SPORT4_CLK_O_ENCODING              0x18
#define IDP6_CLK_I_SPORT4_CLK_O_ENCODING              0x18
#define IDP7_CLK_I_SPORT4_CLK_O_ENCODING              0x18
#define DIT_HFCLK_I_SPORT4_CLK_O_ENCODING             0x18
#define PCG_EXTA_I_SPORT4_CLK_O_ENCODING              0x18
#define PCG_EXTB_I_SPORT4_CLK_O_ENCODING              0x18
#define SPDIF_PLLCLK_I_SPORT4_CLK_O_ENCODING          0x18
#define SPIB_CLK_I_SPORT4_CLK_O_ENCODING              0x18
#define PCG_SYNC_CLKA_I_SPORT4_CLK_O_ENCODING         0x18
#define PCG_SYNC_CLKB_I_SPORT4_CLK_O_ENCODING         0x18

#define SPORT0_CLK_I_SPORT5_CLK_O_ENCODING            0x19
#define SPORT1_CLK_I_SPORT5_CLK_O_ENCODING            0x19
#define SPORT2_CLK_I_SPORT5_CLK_O_ENCODING            0x19
#define SPORT3_CLK_I_SPORT5_CLK_O_ENCODING            0x19
#define SPORT4_CLK_I_SPORT5_CLK_O_ENCODING            0x19
#define SPORT5_CLK_I_SPORT5_CLK_O_ENCODING            0x19
#define SRC0_CLK_IP_I_SPORT5_CLK_O_ENCODING           0x19
#define SRC0_CLK_OP_I_SPORT5_CLK_O_ENCODING           0x19
#define SRC1_CLK_IP_I_SPORT5_CLK_O_ENCODING           0x19
#define SRC1_CLK_OP_I_SPORT5_CLK_O_ENCODING           0x19
#define SRC2_CLK_IP_I_SPORT5_CLK_O_ENCODING           0x19
#define SRC2_CLK_OP_I_SPORT5_CLK_O_ENCODING           0x19
#define SRC3_CLK_IP_I_SPORT5_CLK_O_ENCODING           0x19
#define SRC3_CLK_OP_I_SPORT5_CLK_O_ENCODING           0x19
#define IDP0_CLK_I_SPORT5_CLK_O_ENCODING              0x19
#define PDAP_CLK_I_SPORT5_CLK_O_ENCODING              0x19
#define IDP1_CLK_I_SPORT5_CLK_O_ENCODING              0x19
#define IDP2_CLK_I_SPORT5_CLK_O_ENCODING              0x19
#define IDP3_CLK_I_SPORT5_CLK_O_ENCODING              0x19
#define IDP4_CLK_I_SPORT5_CLK_O_ENCODING              0x19
#define IDP5_CLK_I_SPORT5_CLK_O_ENCODING              0x19
#define IDP6_CLK_I_SPORT5_CLK_O_ENCODING              0x19
#define IDP7_CLK_I_SPORT5_CLK_O_ENCODING              0x19
#define DIT_HFCLK_I_SPORT5_CLK_O_ENCODING             0x19
#define PCG_EXTA_I_SPORT5_CLK_O_ENCODING              0x19
#define PCG_EXTB_I_SPORT5_CLK_O_ENCODING              0x19
#define SPDIF_PLLCLK_I_SPORT5_CLK_O_ENCODING          0x19
#define SPIB_CLK_I_SPORT5_CLK_O_ENCODING              0x19
#define PCG_SYNC_CLKA_I_SPORT5_CLK_O_ENCODING         0x19
#define PCG_SYNC_CLKB_I_SPORT5_CLK_O_ENCODING         0x19

#define SPORT0_CLK_I_DIR_CLK_O_ENCODING            0x1A
#define SPORT1_CLK_I_DIR_CLK_O_ENCODING            0x1A
#define SPORT2_CLK_I_DIR_CLK_O_ENCODING            0x1A
#define SPORT3_CLK_I_DIR_CLK_O_ENCODING            0x1A
#define SPORT4_CLK_I_DIR_CLK_O_ENCODING            0x1A
#define SPORT5_CLK_I_DIR_CLK_O_ENCODING            0x1A
#define SRC0_CLK_IP_I_DIR_CLK_O_ENCODING           0x1A
#define SRC0_CLK_OP_I_DIR_CLK_O_ENCODING           0x1A
#define SRC1_CLK_IP_I_DIR_CLK_O_ENCODING           0x1A
#define SRC1_CLK_OP_I_DIR_CLK_O_ENCODING           0x1A
#define SRC2_CLK_IP_I_DIR_CLK_O_ENCODING           0x1A
#define SRC2_CLK_OP_I_DIR_CLK_O_ENCODING           0x1A
#define SRC3_CLK_IP_I_DIR_CLK_O_ENCODING           0x1A
#define SRC3_CLK_OP_I_DIR_CLK_O_ENCODING           0x1A
#define IDP0_CLK_I_DIR_CLK_O_ENCODING              0x1A
#define PDAP_CLK_I_DIR_CLK_O_ENCODING              0x1A
#define IDP1_CLK_I_DIR_CLK_O_ENCODING              0x1A
#define IDP2_CLK_I_DIR_CLK_O_ENCODING              0x1A
#define IDP3_CLK_I_DIR_CLK_O_ENCODING              0x1A
#define IDP4_CLK_I_DIR_CLK_O_ENCODING              0x1A
#define IDP5_CLK_I_DIR_CLK_O_ENCODING              0x1A
#define IDP6_CLK_I_DIR_CLK_O_ENCODING              0x1A
#define IDP7_CLK_I_DIR_CLK_O_ENCODING              0x1A
#define DIT_HFCLK_I_DIR_CLK_O_ENCODING             0x1A
#define PCG_EXTA_I_DIR_CLK_O_ENCODING              0x1A
#define PCG_EXTB_I_DIR_CLK_O_ENCODING              0x1A
#define SPDIF_PLLCLK_I_DIR_CLK_O_ENCODING          0x1A
#define SPIB_CLK_I_DIR_CLK_O_ENCODING              0x1A
#define PCG_SYNC_CLKA_I_DIR_CLK_O_ENCODING         0x1A
#define PCG_SYNC_CLKB_I_DIR_CLK_O_ENCODING         0x1A

#define SPORT0_CLK_I_DIR_TDMCLK_O_ENCODING            0x1B
#define SPORT1_CLK_I_DIR_TDMCLK_O_ENCODING            0x1B
#define SPORT2_CLK_I_DIR_TDMCLK_O_ENCODING            0x1B
#define SPORT3_CLK_I_DIR_TDMCLK_O_ENCODING            0x1B
#define SPORT4_CLK_I_DIR_TDMCLK_O_ENCODING            0x1B
#define SPORT5_CLK_I_DIR_TDMCLK_O_ENCODING            0x1B
#define SRC0_CLK_IP_I_DIR_TDMCLK_O_ENCODING           0x1B
#define SRC0_CLK_OP_I_DIR_TDMCLK_O_ENCODING           0x1B
#define SRC1_CLK_IP_I_DIR_TDMCLK_O_ENCODING           0x1B
#define SRC1_CLK_OP_I_DIR_TDMCLK_O_ENCODING           0x1B
#define SRC2_CLK_IP_I_DIR_TDMCLK_O_ENCODING           0x1B
#define SRC2_CLK_OP_I_DIR_TDMCLK_O_ENCODING           0x1B
#define SRC3_CLK_IP_I_DIR_TDMCLK_O_ENCODING           0x1B
#define SRC3_CLK_OP_I_DIR_TDMCLK_O_ENCODING           0x1B
#define IDP0_CLK_I_DIR_TDMCLK_O_ENCODING              0x1B
#define PDAP_CLK_I_DIR_TDMCLK_O_ENCODING              0x1B
#define IDP1_CLK_I_DIR_TDMCLK_O_ENCODING              0x1B
#define IDP2_CLK_I_DIR_TDMCLK_O_ENCODING              0x1B
#define IDP3_CLK_I_DIR_TDMCLK_O_ENCODING              0x1B
#define IDP4_CLK_I_DIR_TDMCLK_O_ENCODING              0x1B
#define IDP5_CLK_I_DIR_TDMCLK_O_ENCODING              0x1B
#define IDP6_CLK_I_DIR_TDMCLK_O_ENCODING              0x1B
#define IDP7_CLK_I_DIR_TDMCLK_O_ENCODING              0x1B
#define DIT_HFCLK_I_DIR_TDMCLK_O_ENCODING             0x1B
#define PCG_EXTA_I_DIR_TDMCLK_O_ENCODING              0x1B
#define PCG_EXTB_I_DIR_TDMCLK_O_ENCODING              0x1B
#define SPDIF_PLLCLK_I_DIR_TDMCLK_O_ENCODING          0x1B
#define SPIB_CLK_I_DIR_TDMCLK_O_ENCODING              0x1B
#define PCG_SYNC_CLKA_I_DIR_TDMCLK_O_ENCODING         0x1B
#define PCG_SYNC_CLKB_I_DIR_TDMCLK_O_ENCODING         0x1B

#define SPORT0_CLK_I_PCG_CLKA_O_ENCODING            0x1C
#define SPORT1_CLK_I_PCG_CLKA_O_ENCODING            0x1C
#define SPORT2_CLK_I_PCG_CLKA_O_ENCODING            0x1C
#define SPORT3_CLK_I_PCG_CLKA_O_ENCODING            0x1C
#define SPORT4_CLK_I_PCG_CLKA_O_ENCODING            0x1C
#define SPORT5_CLK_I_PCG_CLKA_O_ENCODING            0x1C
#define SRC0_CLK_IP_I_PCG_CLKA_O_ENCODING           0x1C
#define SRC0_CLK_OP_I_PCG_CLKA_O_ENCODING           0x1C
#define SRC1_CLK_IP_I_PCG_CLKA_O_ENCODING           0x1C
#define SRC1_CLK_OP_I_PCG_CLKA_O_ENCODING           0x1C
#define SRC2_CLK_IP_I_PCG_CLKA_O_ENCODING           0x1C
#define SRC2_CLK_OP_I_PCG_CLKA_O_ENCODING           0x1C
#define SRC3_CLK_IP_I_PCG_CLKA_O_ENCODING           0x1C
#define SRC3_CLK_OP_I_PCG_CLKA_O_ENCODING           0x1C
#define IDP0_CLK_I_PCG_CLKA_O_ENCODING              0x1C
#define PDAP_CLK_I_PCG_CLKA_O_ENCODING              0x1C
#define IDP1_CLK_I_PCG_CLKA_O_ENCODING              0x1C
#define IDP2_CLK_I_PCG_CLKA_O_ENCODING              0x1C
#define IDP3_CLK_I_PCG_CLKA_O_ENCODING              0x1C
#define IDP4_CLK_I_PCG_CLKA_O_ENCODING              0x1C
#define IDP5_CLK_I_PCG_CLKA_O_ENCODING              0x1C
#define IDP6_CLK_I_PCG_CLKA_O_ENCODING              0x1C
#define IDP7_CLK_I_PCG_CLKA_O_ENCODING              0x1C
#define DIT_HFCLK_I_PCG_CLKA_O_ENCODING             0x1C
#define PCG_EXTA_I_PCG_CLKA_O_ENCODING              0x1C
#define PCG_EXTB_I_PCG_CLKA_O_ENCODING              0x1C
#define SPDIF_PLLCLK_I_PCG_CLKA_O_ENCODING          0x1C
#define SPIB_CLK_I_PCG_CLKA_O_ENCODING              0x1C
#define PCG_SYNC_CLKA_I_PCG_CLKA_O_ENCODING         0x1C
#define PCG_SYNC_CLKB_I_PCG_CLKA_O_ENCODING         0x1C

#define SPORT0_CLK_I_PCG_CLKB_O_ENCODING            0x1D
#define SPORT1_CLK_I_PCG_CLKB_O_ENCODING            0x1D
#define SPORT2_CLK_I_PCG_CLKB_O_ENCODING            0x1D
#define SPORT3_CLK_I_PCG_CLKB_O_ENCODING            0x1D
#define SPORT4_CLK_I_PCG_CLKB_O_ENCODING            0x1D
#define SPORT5_CLK_I_PCG_CLKB_O_ENCODING            0x1D
#define SRC0_CLK_IP_I_PCG_CLKB_O_ENCODING           0x1D
#define SRC0_CLK_OP_I_PCG_CLKB_O_ENCODING           0x1D
#define SRC1_CLK_IP_I_PCG_CLKB_O_ENCODING           0x1D
#define SRC1_CLK_OP_I_PCG_CLKB_O_ENCODING           0x1D
#define SRC2_CLK_IP_I_PCG_CLKB_O_ENCODING           0x1D
#define SRC2_CLK_OP_I_PCG_CLKB_O_ENCODING           0x1D
#define SRC3_CLK_IP_I_PCG_CLKB_O_ENCODING           0x1D
#define SRC3_CLK_OP_I_PCG_CLKB_O_ENCODING           0x1D
#define IDP0_CLK_I_PCG_CLKB_O_ENCODING              0x1D
#define PDAP_CLK_I_PCG_CLKB_O_ENCODING              0x1D
#define IDP1_CLK_I_PCG_CLKB_O_ENCODING              0x1D
#define IDP2_CLK_I_PCG_CLKB_O_ENCODING              0x1D
#define IDP3_CLK_I_PCG_CLKB_O_ENCODING              0x1D
#define IDP4_CLK_I_PCG_CLKB_O_ENCODING              0x1D
#define IDP5_CLK_I_PCG_CLKB_O_ENCODING              0x1D
#define IDP6_CLK_I_PCG_CLKB_O_ENCODING              0x1D
#define IDP7_CLK_I_PCG_CLKB_O_ENCODING              0x1D
#define DIT_HFCLK_I_PCG_CLKB_O_ENCODING             0x1D
#define PCG_EXTA_I_PCG_CLKB_O_ENCODING              0x1D
#define PCG_EXTB_I_PCG_CLKB_O_ENCODING              0x1D
#define SPDIF_PLLCLK_I_PCG_CLKB_O_ENCODING          0x1D
#define SPIB_CLK_I_PCG_CLKB_O_ENCODING              0x1D
#define PCG_SYNC_CLKA_I_PCG_CLKB_O_ENCODING         0x1D
#define PCG_SYNC_CLKB_I_PCG_CLKB_O_ENCODING         0x1D

#define SPORT0_CLK_I_LOW_ENCODING            0x1E
#define SPORT1_CLK_I_LOW_ENCODING            0x1E
#define SPORT2_CLK_I_LOW_ENCODING            0x1E
#define SPORT3_CLK_I_LOW_ENCODING            0x1E
#define SPORT4_CLK_I_LOW_ENCODING            0x1E
#define SPORT5_CLK_I_LOW_ENCODING            0x1E
#define SRC0_CLK_IP_I_LOW_ENCODING           0x1E
#define SRC0_CLK_OP_I_LOW_ENCODING           0x1E
#define SRC1_CLK_IP_I_LOW_ENCODING           0x1E
#define SRC1_CLK_OP_I_LOW_ENCODING           0x1E
#define SRC2_CLK_IP_I_LOW_ENCODING           0x1E
#define SRC2_CLK_OP_I_LOW_ENCODING           0x1E
#define SRC3_CLK_IP_I_LOW_ENCODING           0x1E
#define SRC3_CLK_OP_I_LOW_ENCODING           0x1E
#define IDP0_CLK_I_LOW_ENCODING              0x1E
#define PDAP_CLK_I_LOW_ENCODING              0x1E
#define IDP1_CLK_I_LOW_ENCODING              0x1E
#define IDP2_CLK_I_LOW_ENCODING              0x1E
#define IDP3_CLK_I_LOW_ENCODING              0x1E
#define IDP4_CLK_I_LOW_ENCODING              0x1E
#define IDP5_CLK_I_LOW_ENCODING              0x1E
#define IDP6_CLK_I_LOW_ENCODING              0x1E
#define IDP7_CLK_I_LOW_ENCODING              0x1E
#define DIT_HFCLK_I_LOW_ENCODING             0x1E
#define PCG_EXTA_I_LOW_ENCODING              0x1E
#define PCG_EXTB_I_LOW_ENCODING              0x1E
#define SPDIF_PLLCLK_I_LOW_ENCODING          0x1E
#define SPIB_CLK_I_LOW_ENCODING              0x1E
#define PCG_SYNC_CLKA_I_LOW_ENCODING         0x1E
#define PCG_SYNC_CLKB_I_LOW_ENCODING         0x1E

#define SPORT0_CLK_I_HIGH_ENCODING            0x1F
#define SPORT1_CLK_I_HIGH_ENCODING            0x1F
#define SPORT2_CLK_I_HIGH_ENCODING            0x1F
#define SPORT3_CLK_I_HIGH_ENCODING            0x1F
#define SPORT4_CLK_I_HIGH_ENCODING            0x1F
#define SPORT5_CLK_I_HIGH_ENCODING            0x1F
#define SRC0_CLK_IP_I_HIGH_ENCODING           0x1F
#define SRC0_CLK_OP_I_HIGH_ENCODING           0x1F
#define SRC1_CLK_IP_I_HIGH_ENCODING           0x1F
#define SRC1_CLK_OP_I_HIGH_ENCODING           0x1F
#define SRC2_CLK_IP_I_HIGH_ENCODING           0x1F
#define SRC2_CLK_OP_I_HIGH_ENCODING           0x1F
#define SRC3_CLK_IP_I_HIGH_ENCODING           0x1F
#define SRC3_CLK_OP_I_HIGH_ENCODING           0x1F
#define IDP0_CLK_I_HIGH_ENCODING              0x1F
#define PDAP_CLK_I_HIGH_ENCODING              0x1F
#define IDP1_CLK_I_HIGH_ENCODING              0x1F
#define IDP2_CLK_I_HIGH_ENCODING              0x1F
#define IDP3_CLK_I_HIGH_ENCODING              0x1F
#define IDP4_CLK_I_HIGH_ENCODING              0x1F
#define IDP5_CLK_I_HIGH_ENCODING              0x1F
#define IDP6_CLK_I_HIGH_ENCODING              0x1F
#define IDP7_CLK_I_HIGH_ENCODING              0x1F
#define DIT_HFCLK_I_HIGH_ENCODING             0x1F
#define PCG_EXTA_I_HIGH_ENCODING              0x1F
#define PCG_EXTB_I_HIGH_ENCODING              0x1F
#define SPDIF_PLLCLK_I_HIGH_ENCODING          0x1F
#define SPIB_CLK_I_HIGH_ENCODING              0x1F
#define PCG_SYNC_CLKA_I_HIGH_ENCODING         0x1F
#define PCG_SYNC_CLKB_I_HIGH_ENCODING         0x1F


//------------------------------------------------------------------------------
//  Group A: Register associations
//------------------------------------------------------------------------------
//SRU_CLK0
#define SPORT0_CLK_I_DAI_PB01_O_REG                 0x2430
#define SPORT1_CLK_I_DAI_PB01_O_REG                 0x2430
#define SPORT2_CLK_I_DAI_PB01_O_REG                 0x2430
#define SPORT3_CLK_I_DAI_PB01_O_REG                 0x2430
#define SPORT4_CLK_I_DAI_PB01_O_REG                 0x2430
#define SPORT5_CLK_I_DAI_PB01_O_REG                 0x2430
#define SPORT0_CLK_I_DAI_PB02_O_REG                 0x2430
#define SPORT1_CLK_I_DAI_PB02_O_REG                 0x2430
#define SPORT2_CLK_I_DAI_PB02_O_REG                 0x2430
#define SPORT3_CLK_I_DAI_PB02_O_REG                 0x2430
#define SPORT4_CLK_I_DAI_PB02_O_REG                 0x2430
#define SPORT5_CLK_I_DAI_PB02_O_REG                 0x2430
#define SPORT0_CLK_I_DAI_PB03_O_REG                 0x2430
#define SPORT1_CLK_I_DAI_PB03_O_REG                 0x2430
#define SPORT2_CLK_I_DAI_PB03_O_REG                 0x2430
#define SPORT3_CLK_I_DAI_PB03_O_REG                 0x2430
#define SPORT4_CLK_I_DAI_PB03_O_REG                 0x2430
#define SPORT5_CLK_I_DAI_PB03_O_REG                 0x2430
#define SPORT0_CLK_I_DAI_PB04_O_REG                 0x2430
#define SPORT1_CLK_I_DAI_PB04_O_REG                 0x2430
#define SPORT2_CLK_I_DAI_PB04_O_REG                 0x2430
#define SPORT3_CLK_I_DAI_PB04_O_REG                 0x2430
#define SPORT4_CLK_I_DAI_PB04_O_REG                 0x2430
#define SPORT5_CLK_I_DAI_PB04_O_REG                 0x2430
#define SPORT0_CLK_I_DAI_PB05_O_REG                 0x2430
#define SPORT1_CLK_I_DAI_PB05_O_REG                 0x2430
#define SPORT2_CLK_I_DAI_PB05_O_REG                 0x2430
#define SPORT3_CLK_I_DAI_PB05_O_REG                 0x2430
#define SPORT4_CLK_I_DAI_PB05_O_REG                 0x2430
#define SPORT5_CLK_I_DAI_PB05_O_REG                 0x2430
#define SPORT0_CLK_I_DAI_PB06_O_REG                 0x2430
#define SPORT1_CLK_I_DAI_PB06_O_REG                 0x2430
#define SPORT2_CLK_I_DAI_PB06_O_REG                 0x2430
#define SPORT3_CLK_I_DAI_PB06_O_REG                 0x2430
#define SPORT4_CLK_I_DAI_PB06_O_REG                 0x2430
#define SPORT5_CLK_I_DAI_PB06_O_REG                 0x2430
#define SPORT0_CLK_I_DAI_PB07_O_REG                 0x2430
#define SPORT1_CLK_I_DAI_PB07_O_REG                 0x2430
#define SPORT2_CLK_I_DAI_PB07_O_REG                 0x2430
#define SPORT3_CLK_I_DAI_PB07_O_REG                 0x2430
#define SPORT4_CLK_I_DAI_PB07_O_REG                 0x2430
#define SPORT5_CLK_I_DAI_PB07_O_REG                 0x2430
#define SPORT0_CLK_I_DAI_PB08_O_REG                 0x2430
#define SPORT1_CLK_I_DAI_PB08_O_REG                 0x2430
#define SPORT2_CLK_I_DAI_PB08_O_REG                 0x2430
#define SPORT3_CLK_I_DAI_PB08_O_REG                 0x2430
#define SPORT4_CLK_I_DAI_PB08_O_REG                 0x2430
#define SPORT5_CLK_I_DAI_PB08_O_REG                 0x2430
#define SPORT0_CLK_I_DAI_PB09_O_REG                 0x2430
#define SPORT1_CLK_I_DAI_PB09_O_REG                 0x2430
#define SPORT2_CLK_I_DAI_PB09_O_REG                 0x2430
#define SPORT3_CLK_I_DAI_PB09_O_REG                 0x2430
#define SPORT4_CLK_I_DAI_PB09_O_REG                 0x2430
#define SPORT5_CLK_I_DAI_PB09_O_REG                 0x2430
#define SPORT0_CLK_I_DAI_PB10_O_REG                 0x2430
#define SPORT1_CLK_I_DAI_PB10_O_REG                 0x2430
#define SPORT2_CLK_I_DAI_PB10_O_REG                 0x2430
#define SPORT3_CLK_I_DAI_PB10_O_REG                 0x2430
#define SPORT4_CLK_I_DAI_PB10_O_REG                 0x2430
#define SPORT5_CLK_I_DAI_PB10_O_REG                 0x2430
#define SPORT0_CLK_I_DAI_PB11_O_REG                 0x2430
#define SPORT1_CLK_I_DAI_PB11_O_REG                 0x2430
#define SPORT2_CLK_I_DAI_PB11_O_REG                 0x2430
#define SPORT3_CLK_I_DAI_PB11_O_REG                 0x2430
#define SPORT4_CLK_I_DAI_PB11_O_REG                 0x2430
#define SPORT5_CLK_I_DAI_PB11_O_REG                 0x2430
#define SPORT0_CLK_I_DAI_PB12_O_REG                 0x2430
#define SPORT1_CLK_I_DAI_PB12_O_REG                 0x2430
#define SPORT2_CLK_I_DAI_PB12_O_REG                 0x2430
#define SPORT3_CLK_I_DAI_PB12_O_REG                 0x2430
#define SPORT4_CLK_I_DAI_PB12_O_REG                 0x2430
#define SPORT5_CLK_I_DAI_PB12_O_REG                 0x2430
#define SPORT0_CLK_I_DAI_PB13_O_REG                 0x2430
#define SPORT1_CLK_I_DAI_PB13_O_REG                 0x2430
#define SPORT2_CLK_I_DAI_PB13_O_REG                 0x2430
#define SPORT3_CLK_I_DAI_PB13_O_REG                 0x2430
#define SPORT4_CLK_I_DAI_PB13_O_REG                 0x2430
#define SPORT5_CLK_I_DAI_PB13_O_REG                 0x2430
#define SPORT0_CLK_I_DAI_PB14_O_REG                 0x2430
#define SPORT1_CLK_I_DAI_PB14_O_REG                 0x2430
#define SPORT2_CLK_I_DAI_PB14_O_REG                 0x2430
#define SPORT3_CLK_I_DAI_PB14_O_REG                 0x2430
#define SPORT4_CLK_I_DAI_PB14_O_REG                 0x2430
#define SPORT5_CLK_I_DAI_PB14_O_REG                 0x2430
#define SPORT0_CLK_I_DAI_PB15_O_REG                 0x2430
#define SPORT1_CLK_I_DAI_PB15_O_REG                 0x2430
#define SPORT2_CLK_I_DAI_PB15_O_REG                 0x2430
#define SPORT3_CLK_I_DAI_PB15_O_REG                 0x2430
#define SPORT4_CLK_I_DAI_PB15_O_REG                 0x2430
#define SPORT5_CLK_I_DAI_PB15_O_REG                 0x2430
#define SPORT0_CLK_I_DAI_PB16_O_REG                 0x2430
#define SPORT1_CLK_I_DAI_PB16_O_REG                 0x2430
#define SPORT2_CLK_I_DAI_PB16_O_REG                 0x2430
#define SPORT3_CLK_I_DAI_PB16_O_REG                 0x2430
#define SPORT4_CLK_I_DAI_PB16_O_REG                 0x2430
#define SPORT5_CLK_I_DAI_PB16_O_REG                 0x2430
#define SPORT0_CLK_I_DAI_PB17_O_REG                 0x2430
#define SPORT1_CLK_I_DAI_PB17_O_REG                 0x2430
#define SPORT2_CLK_I_DAI_PB17_O_REG                 0x2430
#define SPORT3_CLK_I_DAI_PB17_O_REG                 0x2430
#define SPORT4_CLK_I_DAI_PB17_O_REG                 0x2430
#define SPORT5_CLK_I_DAI_PB17_O_REG                 0x2430
#define SPORT0_CLK_I_DAI_PB18_O_REG                 0x2430
#define SPORT1_CLK_I_DAI_PB18_O_REG                 0x2430
#define SPORT2_CLK_I_DAI_PB18_O_REG                 0x2430
#define SPORT3_CLK_I_DAI_PB18_O_REG                 0x2430
#define SPORT4_CLK_I_DAI_PB18_O_REG                 0x2430
#define SPORT5_CLK_I_DAI_PB18_O_REG                 0x2430
#define SPORT0_CLK_I_DAI_PB19_O_REG                 0x2430
#define SPORT1_CLK_I_DAI_PB19_O_REG                 0x2430
#define SPORT2_CLK_I_DAI_PB19_O_REG                 0x2430
#define SPORT3_CLK_I_DAI_PB19_O_REG                 0x2430
#define SPORT4_CLK_I_DAI_PB19_O_REG                 0x2430
#define SPORT5_CLK_I_DAI_PB19_O_REG                 0x2430
#define SPORT0_CLK_I_DAI_PB20_O_REG                 0x2430
#define SPORT1_CLK_I_DAI_PB20_O_REG                 0x2430
#define SPORT2_CLK_I_DAI_PB20_O_REG                 0x2430
#define SPORT3_CLK_I_DAI_PB20_O_REG                 0x2430
#define SPORT4_CLK_I_DAI_PB20_O_REG                 0x2430
#define SPORT5_CLK_I_DAI_PB20_O_REG                 0x2430
#define SPORT0_CLK_I_SPORT0_CLK_O_REG               0x2430
#define SPORT1_CLK_I_SPORT0_CLK_O_REG               0x2430
#define SPORT2_CLK_I_SPORT0_CLK_O_REG               0x2430
#define SPORT3_CLK_I_SPORT0_CLK_O_REG               0x2430
#define SPORT4_CLK_I_SPORT0_CLK_O_REG               0x2430
#define SPORT5_CLK_I_SPORT0_CLK_O_REG               0x2430
#define SPORT0_CLK_I_SPORT0_CLK_O_REG               0x2430
#define SPORT1_CLK_I_SPORT1_CLK_O_REG               0x2430
#define SPORT2_CLK_I_SPORT1_CLK_O_REG               0x2430
#define SPORT3_CLK_I_SPORT1_CLK_O_REG               0x2430
#define SPORT4_CLK_I_SPORT1_CLK_O_REG               0x2430
#define SPORT5_CLK_I_SPORT1_CLK_O_REG               0x2430
#define SPORT0_CLK_I_SPORT2_CLK_O_REG               0x2430
#define SPORT1_CLK_I_SPORT2_CLK_O_REG               0x2430
#define SPORT2_CLK_I_SPORT2_CLK_O_REG               0x2430
#define SPORT3_CLK_I_SPORT2_CLK_O_REG               0x2430
#define SPORT4_CLK_I_SPORT2_CLK_O_REG               0x2430
#define SPORT5_CLK_I_SPORT2_CLK_O_REG               0x2430
#define SPORT0_CLK_I_SPORT3_CLK_O_REG               0x2430
#define SPORT1_CLK_I_SPORT3_CLK_O_REG               0x2430
#define SPORT2_CLK_I_SPORT3_CLK_O_REG               0x2430
#define SPORT3_CLK_I_SPORT3_CLK_O_REG               0x2430
#define SPORT4_CLK_I_SPORT3_CLK_O_REG               0x2430
#define SPORT5_CLK_I_SPORT3_CLK_O_REG               0x2430
#define SPORT0_CLK_I_SPORT4_CLK_O_REG               0x2430
#define SPORT1_CLK_I_SPORT4_CLK_O_REG               0x2430
#define SPORT2_CLK_I_SPORT4_CLK_O_REG               0x2430
#define SPORT3_CLK_I_SPORT4_CLK_O_REG               0x2430
#define SPORT4_CLK_I_SPORT4_CLK_O_REG               0x2430
#define SPORT5_CLK_I_SPORT4_CLK_O_REG               0x2430
#define SPORT0_CLK_I_SPORT5_CLK_O_REG               0x2430
#define SPORT1_CLK_I_SPORT5_CLK_O_REG               0x2430
#define SPORT2_CLK_I_SPORT5_CLK_O_REG               0x2430
#define SPORT3_CLK_I_SPORT5_CLK_O_REG               0x2430
#define SPORT4_CLK_I_SPORT5_CLK_O_REG               0x2430
#define SPORT5_CLK_I_SPORT5_CLK_O_REG               0x2430
#define SPORT0_CLK_I_DIR_CLK_O_REG                  0x2430
#define SPORT1_CLK_I_DIR_CLK_O_REG                  0x2430
#define SPORT2_CLK_I_DIR_CLK_O_REG                  0x2430
#define SPORT3_CLK_I_DIR_CLK_O_REG                  0x2430
#define SPORT4_CLK_I_DIR_CLK_O_REG                  0x2430
#define SPORT5_CLK_I_DIR_TDMCLK_O_REG               0x2430
#define SPORT0_CLK_I_DIR_TDMCLK_O_REG               0x2430
#define SPORT1_CLK_I_DIR_TDMCLK_O_REG               0x2430
#define SPORT2_CLK_I_DIR_TDMCLK_O_REG               0x2430
#define SPORT3_CLK_I_DIR_TDMCLK_O_REG               0x2430
#define SPORT4_CLK_I_DIR_TDMCLK_O_REG               0x2430
#define SPORT5_CLK_I_DIR_TDMCLK_O_REG               0x2430
#define SPORT0_CLK_I_PCG_CLKA_O_REG                 0x2430
#define SPORT1_CLK_I_PCG_CLKA_O_REG                 0x2430
#define SPORT2_CLK_I_PCG_CLKA_O_REG                 0x2430
#define SPORT3_CLK_I_PCG_CLKA_O_REG                 0x2430
#define SPORT4_CLK_I_PCG_CLKA_O_REG                 0x2430
#define SPORT5_CLK_I_PCG_CLKA_O_REG                 0x2430
#define SPORT0_CLK_I_PCG_CLKB_O_REG                 0x2430
#define SPORT1_CLK_I_PCG_CLKB_O_REG                 0x2430
#define SPORT2_CLK_I_PCG_CLKB_O_REG                 0x2430
#define SPORT3_CLK_I_PCG_CLKB_O_REG                 0x2430
#define SPORT4_CLK_I_PCG_CLKB_O_REG                 0x2430
#define SPORT5_CLK_I_PCG_CLKB_O_REG                 0x2430
#define SPORT0_CLK_I_LOW_REG                        0x2430
#define SPORT1_CLK_I_LOW_REG                        0x2430
#define SPORT2_CLK_I_LOW_REG                        0x2430
#define SPORT3_CLK_I_LOW_REG                        0x2430
#define SPORT4_CLK_I_LOW_REG                        0x2430
#define SPORT5_CLK_I_LOW_REG                        0x2430
#define SPORT0_CLK_I_HIGH_REG                       0x2430
#define SPORT1_CLK_I_HIGH_REG                       0x2430
#define SPORT2_CLK_I_HIGH_REG                       0x2430
#define SPORT3_CLK_I_HIGH_REG                       0x2430
#define SPORT4_CLK_I_HIGH_REG                       0x2430
#define SPORT5_CLK_I_HIGH_REG                       0x2430

//SRU_CLK1
#define SRC0_CLK_IP_I_DAI_PB01_O_REG                 0x2431
#define SRC0_CLK_OP_I_DAI_PB01_O_REG                 0x2431
#define SRC1_CLK_IP_I_DAI_PB01_O_REG                 0x2431
#define SRC1_CLK_OP_I_DAI_PB01_O_REG                 0x2431
#define SRC2_CLK_IP_I_DAI_PB01_O_REG                 0x2431
#define SRC2_CLK_OP_I_DAI_PB01_O_REG                 0x2431
#define SRC0_CLK_IP_I_DAI_PB02_O_REG                 0x2431
#define SRC0_CLK_OP_I_DAI_PB02_O_REG                 0x2431
#define SRC1_CLK_IP_I_DAI_PB02_O_REG                 0x2431
#define SRC1_CLK_OP_I_DAI_PB02_O_REG                 0x2431
#define SRC2_CLK_IP_I_DAI_PB02_O_REG                 0x2431
#define SRC2_CLK_OP_I_DAI_PB02_O_REG                 0x2431
#define SRC0_CLK_IP_I_DAI_PB03_O_REG                 0x2431
#define SRC0_CLK_OP_I_DAI_PB03_O_REG                 0x2431
#define SRC1_CLK_IP_I_DAI_PB03_O_REG                 0x2431
#define SRC1_CLK_OP_I_DAI_PB03_O_REG                 0x2431
#define SRC2_CLK_IP_I_DAI_PB03_O_REG                 0x2431
#define SRC2_CLK_OP_I_DAI_PB03_O_REG                 0x2431
#define SRC0_CLK_IP_I_DAI_PB04_O_REG                 0x2431
#define SRC0_CLK_OP_I_DAI_PB04_O_REG                 0x2431
#define SRC1_CLK_IP_I_DAI_PB04_O_REG                 0x2431
#define SRC1_CLK_OP_I_DAI_PB04_O_REG                 0x2431
#define SRC2_CLK_IP_I_DAI_PB04_O_REG                 0x2431
#define SRC2_CLK_OP_I_DAI_PB04_O_REG                 0x2431
#define SRC0_CLK_IP_I_DAI_PB05_O_REG                 0x2431
#define SRC0_CLK_OP_I_DAI_PB05_O_REG                 0x2431
#define SRC1_CLK_IP_I_DAI_PB05_O_REG                 0x2431
#define SRC1_CLK_OP_I_DAI_PB05_O_REG                 0x2431
#define SRC2_CLK_IP_I_DAI_PB05_O_REG                 0x2431
#define SRC2_CLK_OP_I_DAI_PB05_O_REG                 0x2431
#define SRC0_CLK_IP_I_DAI_PB06_O_REG                 0x2431
#define SRC0_CLK_OP_I_DAI_PB06_O_REG                 0x2431
#define SRC1_CLK_IP_I_DAI_PB06_O_REG                 0x2431
#define SRC1_CLK_OP_I_DAI_PB06_O_REG                 0x2431
#define SRC2_CLK_IP_I_DAI_PB06_O_REG                 0x2431
#define SRC2_CLK_OP_I_DAI_PB06_O_REG                 0x2431
#define SRC0_CLK_IP_I_DAI_PB07_O_REG                 0x2431
#define SRC0_CLK_OP_I_DAI_PB07_O_REG                 0x2431
#define SRC1_CLK_IP_I_DAI_PB07_O_REG                 0x2431
#define SRC1_CLK_OP_I_DAI_PB07_O_REG                 0x2431
#define SRC2_CLK_IP_I_DAI_PB07_O_REG                 0x2431
#define SRC2_CLK_OP_I_DAI_PB07_O_REG                 0x2431
#define SRC0_CLK_IP_I_DAI_PB08_O_REG                 0x2431
#define SRC0_CLK_OP_I_DAI_PB08_O_REG                 0x2431
#define SRC1_CLK_IP_I_DAI_PB08_O_REG                 0x2431
#define SRC1_CLK_OP_I_DAI_PB08_O_REG                 0x2431
#define SRC2_CLK_IP_I_DAI_PB08_O_REG                 0x2431
#define SRC2_CLK_OP_I_DAI_PB08_O_REG                 0x2431
#define SRC0_CLK_IP_I_DAI_PB09_O_REG                 0x2431
#define SRC0_CLK_OP_I_DAI_PB09_O_REG                 0x2431
#define SRC1_CLK_IP_I_DAI_PB09_O_REG                 0x2431
#define SRC1_CLK_OP_I_DAI_PB09_O_REG                 0x2431
#define SRC2_CLK_IP_I_DAI_PB09_O_REG                 0x2431
#define SRC2_CLK_OP_I_DAI_PB09_O_REG                 0x2431
#define SRC0_CLK_IP_I_DAI_PB10_O_REG                 0x2431
#define SRC0_CLK_OP_I_DAI_PB10_O_REG                 0x2431
#define SRC1_CLK_IP_I_DAI_PB10_O_REG                 0x2431
#define SRC1_CLK_OP_I_DAI_PB10_O_REG                 0x2431
#define SRC2_CLK_IP_I_DAI_PB10_O_REG                 0x2431
#define SRC2_CLK_OP_I_DAI_PB10_O_REG                 0x2431
#define SRC0_CLK_IP_I_DAI_PB11_O_REG                 0x2431
#define SRC0_CLK_OP_I_DAI_PB11_O_REG                 0x2431
#define SRC1_CLK_IP_I_DAI_PB11_O_REG                 0x2431
#define SRC1_CLK_OP_I_DAI_PB11_O_REG                 0x2431
#define SRC2_CLK_IP_I_DAI_PB11_O_REG                 0x2431
#define SRC2_CLK_OP_I_DAI_PB11_O_REG                 0x2431
#define SRC0_CLK_IP_I_DAI_PB12_O_REG                 0x2431
#define SRC0_CLK_OP_I_DAI_PB12_O_REG                 0x2431
#define SRC1_CLK_IP_I_DAI_PB12_O_REG                 0x2431
#define SRC1_CLK_OP_I_DAI_PB12_O_REG                 0x2431
#define SRC2_CLK_IP_I_DAI_PB12_O_REG                 0x2431
#define SRC2_CLK_OP_I_DAI_PB12_O_REG                 0x2431
#define SRC0_CLK_IP_I_DAI_PB13_O_REG                 0x2431
#define SRC0_CLK_OP_I_DAI_PB13_O_REG                 0x2431
#define SRC1_CLK_IP_I_DAI_PB13_O_REG                 0x2431
#define SRC1_CLK_OP_I_DAI_PB13_O_REG                 0x2431
#define SRC2_CLK_IP_I_DAI_PB13_O_REG                 0x2431
#define SRC2_CLK_OP_I_DAI_PB13_O_REG                 0x2431
#define SRC0_CLK_IP_I_DAI_PB14_O_REG                 0x2431
#define SRC0_CLK_OP_I_DAI_PB14_O_REG                 0x2431
#define SRC1_CLK_IP_I_DAI_PB14_O_REG                 0x2431
#define SRC1_CLK_OP_I_DAI_PB14_O_REG                 0x2431
#define SRC2_CLK_IP_I_DAI_PB14_O_REG                 0x2431
#define SRC2_CLK_OP_I_DAI_PB14_O_REG                 0x2431
#define SRC0_CLK_IP_I_DAI_PB15_O_REG                 0x2431
#define SRC0_CLK_OP_I_DAI_PB15_O_REG                 0x2431
#define SRC1_CLK_IP_I_DAI_PB15_O_REG                 0x2431
#define SRC1_CLK_OP_I_DAI_PB15_O_REG                 0x2431
#define SRC2_CLK_IP_I_DAI_PB15_O_REG                 0x2431
#define SRC2_CLK_OP_I_DAI_PB15_O_REG                 0x2431
#define SRC0_CLK_IP_I_DAI_PB16_O_REG                 0x2431
#define SRC0_CLK_OP_I_DAI_PB16_O_REG                 0x2431
#define SRC1_CLK_IP_I_DAI_PB16_O_REG                 0x2431
#define SRC1_CLK_OP_I_DAI_PB16_O_REG                 0x2431
#define SRC2_CLK_IP_I_DAI_PB16_O_REG                 0x2431
#define SRC2_CLK_OP_I_DAI_PB16_O_REG                 0x2431
#define SRC0_CLK_IP_I_DAI_PB17_O_REG                 0x2431
#define SRC0_CLK_OP_I_DAI_PB17_O_REG                 0x2431
#define SRC1_CLK_IP_I_DAI_PB17_O_REG                 0x2431
#define SRC1_CLK_OP_I_DAI_PB17_O_REG                 0x2431
#define SRC2_CLK_IP_I_DAI_PB17_O_REG                 0x2431
#define SRC2_CLK_OP_I_DAI_PB17_O_REG                 0x2431
#define SRC0_CLK_IP_I_DAI_PB18_O_REG                 0x2431
#define SRC0_CLK_OP_I_DAI_PB18_O_REG                 0x2431
#define SRC1_CLK_IP_I_DAI_PB18_O_REG                 0x2431
#define SRC1_CLK_OP_I_DAI_PB18_O_REG                 0x2431
#define SRC2_CLK_IP_I_DAI_PB18_O_REG                 0x2431
#define SRC2_CLK_OP_I_DAI_PB18_O_REG                 0x2431
#define SRC0_CLK_IP_I_DAI_PB19_O_REG                 0x2431
#define SRC0_CLK_OP_I_DAI_PB19_O_REG                 0x2431
#define SRC1_CLK_IP_I_DAI_PB19_O_REG                 0x2431
#define SRC1_CLK_OP_I_DAI_PB19_O_REG                 0x2431
#define SRC2_CLK_IP_I_DAI_PB19_O_REG                 0x2431
#define SRC2_CLK_OP_I_DAI_PB19_O_REG                 0x2431
#define SRC0_CLK_IP_I_DAI_PB20_O_REG                 0x2431
#define SRC0_CLK_OP_I_DAI_PB20_O_REG                 0x2431
#define SRC1_CLK_IP_I_DAI_PB20_O_REG                 0x2431
#define SRC1_CLK_OP_I_DAI_PB20_O_REG                 0x2431
#define SRC2_CLK_IP_I_DAI_PB20_O_REG                 0x2431
#define SRC2_CLK_OP_I_DAI_PB20_O_REG                 0x2431
#define SRC0_CLK_IP_I_SPORT0_CLK_O_REG               0x2431
#define SRC0_CLK_OP_I_SPORT0_CLK_O_REG               0x2431
#define SRC1_CLK_IP_I_SPORT0_CLK_O_REG               0x2431
#define SRC1_CLK_OP_I_SPORT0_CLK_O_REG               0x2431
#define SRC2_CLK_IP_I_SPORT0_CLK_O_REG               0x2431
#define SRC2_CLK_OP_I_SPORT0_CLK_O_REG               0x2431
#define SRC0_CLK_IP_I_SPORT0_CLK_O_REG               0x2431
#define SRC0_CLK_OP_I_SPORT1_CLK_O_REG               0x2431
#define SRC1_CLK_IP_I_SPORT1_CLK_O_REG               0x2431
#define SRC1_CLK_OP_I_SPORT1_CLK_O_REG               0x2431
#define SRC2_CLK_IP_I_SPORT1_CLK_O_REG               0x2431
#define SRC2_CLK_OP_I_SPORT1_CLK_O_REG               0x2431
#define SRC0_CLK_IP_I_SPORT2_CLK_O_REG               0x2431
#define SRC0_CLK_OP_I_SPORT2_CLK_O_REG               0x2431
#define SRC1_CLK_IP_I_SPORT2_CLK_O_REG               0x2431
#define SRC1_CLK_OP_I_SPORT2_CLK_O_REG               0x2431
#define SRC2_CLK_IP_I_SPORT2_CLK_O_REG               0x2431
#define SRC2_CLK_OP_I_SPORT2_CLK_O_REG               0x2431
#define SRC0_CLK_IP_I_SPORT3_CLK_O_REG               0x2431
#define SRC0_CLK_OP_I_SPORT3_CLK_O_REG               0x2431
#define SRC1_CLK_IP_I_SPORT3_CLK_O_REG               0x2431
#define SRC1_CLK_OP_I_SPORT3_CLK_O_REG               0x2431
#define SRC2_CLK_IP_I_SPORT3_CLK_O_REG               0x2431
#define SRC2_CLK_OP_I_SPORT3_CLK_O_REG               0x2431
#define SRC0_CLK_IP_I_SPORT4_CLK_O_REG               0x2431
#define SRC0_CLK_OP_I_SPORT4_CLK_O_REG               0x2431
#define SRC1_CLK_IP_I_SPORT4_CLK_O_REG               0x2431
#define SRC1_CLK_OP_I_SPORT4_CLK_O_REG               0x2431
#define SRC2_CLK_IP_I_SPORT4_CLK_O_REG               0x2431
#define SRC2_CLK_OP_I_SPORT4_CLK_O_REG               0x2431
#define SRC0_CLK_IP_I_SPORT5_CLK_O_REG               0x2431
#define SRC0_CLK_OP_I_SPORT5_CLK_O_REG               0x2431
#define SRC1_CLK_IP_I_SPORT5_CLK_O_REG               0x2431
#define SRC1_CLK_OP_I_SPORT5_CLK_O_REG               0x2431
#define SRC2_CLK_IP_I_SPORT5_CLK_O_REG               0x2431
#define SRC2_CLK_OP_I_SPORT5_CLK_O_REG               0x2431
#define SRC0_CLK_IP_I_DIR_CLK_O_REG                  0x2431
#define SRC0_CLK_OP_I_DIR_CLK_O_REG                  0x2431
#define SRC1_CLK_IP_I_DIR_CLK_O_REG                  0x2431
#define SRC1_CLK_OP_I_DIR_CLK_O_REG                  0x2431
#define SRC2_CLK_IP_I_DIR_CLK_O_REG                  0x2431
#define SRC2_CLK_OP_I_DIR_TDMCLK_O_REG               0x2431
#define SRC0_CLK_IP_I_DIR_TDMCLK_O_REG               0x2431
#define SRC0_CLK_OP_I_DIR_TDMCLK_O_REG               0x2431
#define SRC1_CLK_IP_I_DIR_TDMCLK_O_REG               0x2431
#define SRC1_CLK_OP_I_DIR_TDMCLK_O_REG               0x2431
#define SRC2_CLK_IP_I_DIR_TDMCLK_O_REG               0x2431
#define SRC2_CLK_OP_I_DIR_TDMCLK_O_REG               0x2431
#define SRC0_CLK_IP_I_PCG_CLKA_O_REG                 0x2431
#define SRC0_CLK_OP_I_PCG_CLKA_O_REG                 0x2431
#define SRC1_CLK_IP_I_PCG_CLKA_O_REG                 0x2431
#define SRC1_CLK_OP_I_PCG_CLKA_O_REG                 0x2431
#define SRC2_CLK_IP_I_PCG_CLKA_O_REG                 0x2431
#define SRC2_CLK_OP_I_PCG_CLKA_O_REG                 0x2431
#define SRC0_CLK_IP_I_PCG_CLKB_O_REG                 0x2431
#define SRC0_CLK_OP_I_PCG_CLKB_O_REG                 0x2431
#define SRC1_CLK_IP_I_PCG_CLKB_O_REG                 0x2431
#define SRC1_CLK_OP_I_PCG_CLKB_O_REG                 0x2431
#define SRC2_CLK_IP_I_PCG_CLKB_O_REG                 0x2431
#define SRC2_CLK_OP_I_PCG_CLKB_O_REG                 0x2431
#define SRC0_CLK_IP_I_LOW_REG                        0x2431
#define SRC0_CLK_OP_I_LOW_REG                        0x2431
#define SRC1_CLK_IP_I_LOW_REG                        0x2431
#define SRC1_CLK_OP_I_LOW_REG                        0x2431
#define SRC2_CLK_IP_I_LOW_REG                        0x2431
#define SRC2_CLK_OP_I_LOW_REG                        0x2431
#define SRC0_CLK_IP_I_HIGH_REG                       0x2431
#define SRC0_CLK_OP_I_HIGH_REG                       0x2431
#define SRC1_CLK_IP_I_HIGH_REG                       0x2431
#define SRC1_CLK_OP_I_HIGH_REG                       0x2431
#define SRC2_CLK_IP_I_HIGH_REG                       0x2431
#define SRC2_CLK_OP_I_HIGH_REG                       0x2431

//SRU_CLK2
#define SRC3_CLK_IP_I_DAI_PB01_O_REG              0x2432
#define SRC3_CLK_OP_I_DAI_PB01_O_REG              0x2432
#define IDP0_CLK_I_DAI_PB01_O_REG                 0x2432
#define PDAP_CLK_I_DAI_PB01_O_REG                 0x2432
#define IDP1_CLK_I_DAI_PB01_O_REG                 0x2432
#define IDP2_CLK_I_DAI_PB01_O_REG                 0x2432
#define SRC3_CLK_IP_I_DAI_PB02_O_REG              0x2432
#define SRC3_CLK_OP_I_DAI_PB02_O_REG              0x2432
#define IDP0_CLK_I_DAI_PB02_O_REG                 0x2432
#define PDAP_CLK_I_DAI_PB02_O_REG                 0x2432
#define IDP1_CLK_I_DAI_PB02_O_REG                 0x2432
#define IDP2_CLK_I_DAI_PB02_O_REG                 0x2432
#define SRC3_CLK_IP_I_DAI_PB03_O_REG              0x2432
#define SRC3_CLK_OP_I_DAI_PB03_O_REG              0x2432
#define IDP0_CLK_I_DAI_PB03_O_REG                 0x2432
#define PDAP_CLK_I_DAI_PB03_O_REG                 0x2432
#define IDP1_CLK_I_DAI_PB03_O_REG                 0x2432
#define IDP2_CLK_I_DAI_PB03_O_REG                 0x2432
#define SRC3_CLK_IP_I_DAI_PB04_O_REG              0x2432
#define SRC3_CLK_OP_I_DAI_PB04_O_REG              0x2432
#define IDP0_CLK_I_DAI_PB04_O_REG                 0x2432
#define PDAP_CLK_I_DAI_PB04_O_REG                 0x2432
#define IDP1_CLK_I_DAI_PB04_O_REG                 0x2432
#define IDP2_CLK_I_DAI_PB04_O_REG                 0x2432
#define SRC3_CLK_IP_I_DAI_PB05_O_REG              0x2432
#define SRC3_CLK_OP_I_DAI_PB05_O_REG              0x2432
#define IDP0_CLK_I_DAI_PB05_O_REG                 0x2432
#define PDAP_CLK_I_DAI_PB05_O_REG                 0x2432
#define IDP1_CLK_I_DAI_PB05_O_REG                 0x2432
#define IDP2_CLK_I_DAI_PB05_O_REG                 0x2432
#define SRC3_CLK_IP_I_DAI_PB06_O_REG              0x2432
#define SRC3_CLK_OP_I_DAI_PB06_O_REG              0x2432
#define IDP0_CLK_I_DAI_PB06_O_REG                 0x2432
#define PDAP_CLK_I_DAI_PB06_O_REG                 0x2432
#define IDP1_CLK_I_DAI_PB06_O_REG                 0x2432
#define IDP2_CLK_I_DAI_PB06_O_REG                 0x2432
#define SRC3_CLK_IP_I_DAI_PB07_O_REG              0x2432
#define SRC3_CLK_OP_I_DAI_PB07_O_REG              0x2432
#define IDP0_CLK_I_DAI_PB07_O_REG                 0x2432
#define PDAP_CLK_I_DAI_PB07_O_REG                 0x2432
#define IDP1_CLK_I_DAI_PB07_O_REG                 0x2432
#define IDP2_CLK_I_DAI_PB07_O_REG                 0x2432
#define SRC3_CLK_IP_I_DAI_PB08_O_REG              0x2432
#define SRC3_CLK_OP_I_DAI_PB08_O_REG              0x2432
#define IDP0_CLK_I_DAI_PB08_O_REG                 0x2432
#define PDAP_CLK_I_DAI_PB08_O_REG                 0x2432
#define IDP1_CLK_I_DAI_PB08_O_REG                 0x2432
#define IDP2_CLK_I_DAI_PB08_O_REG                 0x2432
#define SRC3_CLK_IP_I_DAI_PB09_O_REG              0x2432
#define SRC3_CLK_OP_I_DAI_PB09_O_REG              0x2432
#define IDP0_CLK_I_DAI_PB09_O_REG                 0x2432
#define PDAP_CLK_I_DAI_PB09_O_REG                 0x2432
#define IDP1_CLK_I_DAI_PB09_O_REG                 0x2432
#define IDP2_CLK_I_DAI_PB09_O_REG                 0x2432
#define SRC3_CLK_IP_I_DAI_PB10_O_REG              0x2432
#define SRC3_CLK_OP_I_DAI_PB10_O_REG              0x2432
#define IDP0_CLK_I_DAI_PB10_O_REG                 0x2432
#define PDAP_CLK_I_DAI_PB10_O_REG                 0x2432
#define IDP1_CLK_I_DAI_PB10_O_REG                 0x2432
#define IDP2_CLK_I_DAI_PB10_O_REG                 0x2432
#define SRC3_CLK_IP_I_DAI_PB11_O_REG              0x2432
#define SRC3_CLK_OP_I_DAI_PB11_O_REG              0x2432
#define IDP0_CLK_I_DAI_PB11_O_REG                 0x2432
#define PDAP_CLK_I_DAI_PB11_O_REG                 0x2432
#define IDP1_CLK_I_DAI_PB11_O_REG                 0x2432
#define IDP2_CLK_I_DAI_PB11_O_REG                 0x2432
#define SRC3_CLK_IP_I_DAI_PB12_O_REG              0x2432
#define SRC3_CLK_OP_I_DAI_PB12_O_REG              0x2432
#define IDP0_CLK_I_DAI_PB12_O_REG                 0x2432
#define PDAP_CLK_I_DAI_PB12_O_REG                 0x2432
#define IDP1_CLK_I_DAI_PB12_O_REG                 0x2432
#define IDP2_CLK_I_DAI_PB12_O_REG                 0x2432
#define SRC3_CLK_IP_I_DAI_PB13_O_REG              0x2432
#define SRC3_CLK_OP_I_DAI_PB13_O_REG              0x2432
#define IDP0_CLK_I_DAI_PB13_O_REG                 0x2432
#define PDAP_CLK_I_DAI_PB13_O_REG                 0x2432
#define IDP1_CLK_I_DAI_PB13_O_REG                 0x2432
#define IDP2_CLK_I_DAI_PB13_O_REG                 0x2432
#define SRC3_CLK_IP_I_DAI_PB14_O_REG              0x2432
#define SRC3_CLK_OP_I_DAI_PB14_O_REG              0x2432
#define IDP0_CLK_I_DAI_PB14_O_REG                 0x2432
#define PDAP_CLK_I_DAI_PB14_O_REG                 0x2432
#define IDP1_CLK_I_DAI_PB14_O_REG                 0x2432
#define IDP2_CLK_I_DAI_PB14_O_REG                 0x2432
#define SRC3_CLK_IP_I_DAI_PB15_O_REG              0x2432
#define SRC3_CLK_OP_I_DAI_PB15_O_REG              0x2432
#define IDP0_CLK_I_DAI_PB15_O_REG                 0x2432
#define PDAP_CLK_I_DAI_PB15_O_REG                 0x2432
#define IDP1_CLK_I_DAI_PB15_O_REG                 0x2432
#define IDP2_CLK_I_DAI_PB15_O_REG                 0x2432
#define SRC3_CLK_IP_I_DAI_PB16_O_REG              0x2432
#define SRC3_CLK_OP_I_DAI_PB16_O_REG              0x2432
#define IDP0_CLK_I_DAI_PB16_O_REG                 0x2432
#define PDAP_CLK_I_DAI_PB16_O_REG                 0x2432
#define IDP1_CLK_I_DAI_PB16_O_REG                 0x2432
#define IDP2_CLK_I_DAI_PB16_O_REG                 0x2432
#define SRC3_CLK_IP_I_DAI_PB17_O_REG              0x2432
#define SRC3_CLK_OP_I_DAI_PB17_O_REG              0x2432
#define IDP0_CLK_I_DAI_PB17_O_REG                 0x2432
#define PDAP_CLK_I_DAI_PB17_O_REG                 0x2432
#define IDP1_CLK_I_DAI_PB17_O_REG                 0x2432
#define IDP2_CLK_I_DAI_PB17_O_REG                 0x2432
#define SRC3_CLK_IP_I_DAI_PB18_O_REG              0x2432
#define SRC3_CLK_OP_I_DAI_PB18_O_REG              0x2432
#define IDP0_CLK_I_DAI_PB18_O_REG                 0x2432
#define PDAP_CLK_I_DAI_PB18_O_REG                 0x2432
#define IDP1_CLK_I_DAI_PB18_O_REG                 0x2432
#define IDP2_CLK_I_DAI_PB18_O_REG                 0x2432
#define SRC3_CLK_IP_I_DAI_PB19_O_REG              0x2432
#define SRC3_CLK_OP_I_DAI_PB19_O_REG              0x2432
#define IDP0_CLK_I_DAI_PB19_O_REG                 0x2432
#define PDAP_CLK_I_DAI_PB19_O_REG                 0x2432
#define IDP1_CLK_I_DAI_PB19_O_REG                 0x2432
#define IDP2_CLK_I_DAI_PB19_O_REG                 0x2432
#define SRC3_CLK_IP_I_DAI_PB20_O_REG              0x2432
#define SRC3_CLK_OP_I_DAI_PB20_O_REG              0x2432
#define IDP0_CLK_I_DAI_PB20_O_REG                 0x2432
#define PDAP_CLK_I_DAI_PB20_O_REG                 0x2432
#define IDP1_CLK_I_DAI_PB20_O_REG                 0x2432
#define IDP2_CLK_I_DAI_PB20_O_REG                 0x2432
#define SRC3_CLK_IP_I_SPORT0_CLK_O_REG            0x2432
#define SRC3_CLK_OP_I_SPORT0_CLK_O_REG            0x2432
#define IDP0_CLK_I_SPORT0_CLK_O_REG               0x2432
#define PDAP_CLK_I_SPORT0_CLK_O_REG               0x2432
#define IDP1_CLK_I_SPORT0_CLK_O_REG               0x2432
#define IDP2_CLK_I_SPORT0_CLK_O_REG               0x2432
#define SRC3_CLK_IP_I_SPORT0_CLK_O_REG            0x2432
#define SRC3_CLK_OP_I_SPORT1_CLK_O_REG            0x2432
#define IDP0_CLK_I_SPORT1_CLK_O_REG               0x2432
#define PDAP_CLK_I_SPORT1_CLK_O_REG               0x2432
#define IDP1_CLK_I_SPORT1_CLK_O_REG               0x2432
#define IDP2_CLK_I_SPORT1_CLK_O_REG               0x2432
#define SRC3_CLK_IP_I_SPORT2_CLK_O_REG            0x2432
#define SRC3_CLK_OP_I_SPORT2_CLK_O_REG            0x2432
#define IDP0_CLK_I_SPORT2_CLK_O_REG               0x2432
#define PDAP_CLK_I_SPORT2_CLK_O_REG               0x2432
#define IDP1_CLK_I_SPORT2_CLK_O_REG               0x2432
#define IDP2_CLK_I_SPORT2_CLK_O_REG               0x2432
#define SRC3_CLK_IP_I_SPORT3_CLK_O_REG            0x2432
#define SRC3_CLK_OP_I_SPORT3_CLK_O_REG            0x2432
#define IDP0_CLK_I_SPORT3_CLK_O_REG               0x2432
#define PDAP_CLK_I_SPORT3_CLK_O_REG               0x2432
#define IDP1_CLK_I_SPORT3_CLK_O_REG               0x2432
#define IDP2_CLK_I_SPORT3_CLK_O_REG               0x2432
#define SRC3_CLK_IP_I_SPORT4_CLK_O_REG            0x2432
#define SRC3_CLK_OP_I_SPORT4_CLK_O_REG            0x2432
#define IDP0_CLK_I_SPORT4_CLK_O_REG               0x2432
#define PDAP_CLK_I_SPORT4_CLK_O_REG               0x2432
#define IDP1_CLK_I_SPORT4_CLK_O_REG               0x2432
#define IDP2_CLK_I_SPORT4_CLK_O_REG               0x2432
#define SRC3_CLK_IP_I_SPORT5_CLK_O_REG            0x2432
#define SRC3_CLK_OP_I_SPORT5_CLK_O_REG            0x2432
#define IDP0_CLK_I_SPORT5_CLK_O_REG               0x2432
#define PDAP_CLK_I_SPORT5_CLK_O_REG               0x2432
#define IDP1_CLK_I_SPORT5_CLK_O_REG               0x2432
#define IDP2_CLK_I_SPORT5_CLK_O_REG               0x2432
#define SRC3_CLK_IP_I_DIR_CLK_O_REG               0x2432
#define SRC3_CLK_OP_I_DIR_CLK_O_REG               0x2432
#define IDP0_CLK_I_DIR_CLK_O_REG                  0x2432
#define PDAP_CLK_I_DIR_CLK_O_REG                  0x2432
#define IDP1_CLK_I_DIR_CLK_O_REG                  0x2432
#define IDP2_CLK_I_DIR_TDMCLK_O_REG               0x2432
#define SRC3_CLK_IP_I_DIR_TDMCLK_O_REG            0x2432
#define SRC3_CLK_OP_I_DIR_TDMCLK_O_REG            0x2432
#define IDP0_CLK_I_DIR_TDMCLK_O_REG               0x2432
#define PDAP_CLK_I_DIR_TDMCLK_O_REG               0x2432
#define IDP1_CLK_I_DIR_TDMCLK_O_REG               0x2432
#define IDP2_CLK_I_DIR_TDMCLK_O_REG               0x2432
#define SRC3_CLK_IP_I_PCG_CLKA_O_REG              0x2432
#define SRC3_CLK_OP_I_PCG_CLKA_O_REG              0x2432
#define IDP0_CLK_I_PCG_CLKA_O_REG                 0x2432
#define PDAP_CLK_I_PCG_CLKA_O_REG                 0x2432
#define IDP1_CLK_I_PCG_CLKA_O_REG                 0x2432
#define IDP2_CLK_I_PCG_CLKA_O_REG                 0x2432
#define SRC3_CLK_IP_I_PCG_CLKB_O_REG              0x2432
#define SRC3_CLK_OP_I_PCG_CLKB_O_REG              0x2432
#define IDP0_CLK_I_PCG_CLKB_O_REG                 0x2432
#define PDAP_CLK_I_PCG_CLKB_O_REG                 0x2432
#define IDP1_CLK_I_PCG_CLKB_O_REG                 0x2432
#define IDP2_CLK_I_PCG_CLKB_O_REG                 0x2432
#define SRC3_CLK_IP_I_LOW_REG                     0x2432
#define SRC3_CLK_OP_I_LOW_REG                     0x2432
#define IDP0_CLK_I_LOW_REG                        0x2432
#define PDAP_CLK_I_LOW_REG                        0x2432
#define IDP1_CLK_I_LOW_REG                        0x2432
#define IDP2_CLK_I_LOW_REG                        0x2432
#define SRC3_CLK_IP_I_HIGH_REG                    0x2432
#define SRC3_CLK_OP_I_HIGH_REG                    0x2432
#define IDP0_CLK_I_HIGH_REG                       0x2432
#define PDAP_CLK_I_HIGH_REG                       0x2432
#define IDP1_CLK_I_HIGH_REG                       0x2432
#define IDP2_CLK_I_HIGH_REG                       0x2432

//SRU_CLK3
#define IDP3_CLK_I_DAI_PB01_O_REG                 0x2433
#define IDP4_CLK_I_DAI_PB01_O_REG                 0x2433
#define IDP5_CLK_I_DAI_PB01_O_REG                 0x2433
#define IDP6_CLK_I_DAI_PB01_O_REG                 0x2433
#define IDP7_CLK_I_DAI_PB01_O_REG                 0x2433
#define DIT_HFCLK_I_DAI_PB01_O_REG                0x2433
#define IDP3_CLK_I_DAI_PB02_O_REG                 0x2433
#define IDP4_CLK_I_DAI_PB02_O_REG                 0x2433
#define IDP5_CLK_I_DAI_PB02_O_REG                 0x2433
#define IDP6_CLK_I_DAI_PB02_O_REG                 0x2433
#define IDP7_CLK_I_DAI_PB02_O_REG                 0x2433
#define DIT_HFCLK_I_DAI_PB02_O_REG                0x2433
#define IDP3_CLK_I_DAI_PB03_O_REG                 0x2433
#define IDP4_CLK_I_DAI_PB03_O_REG                 0x2433
#define IDP5_CLK_I_DAI_PB03_O_REG                 0x2433
#define IDP6_CLK_I_DAI_PB03_O_REG                 0x2433
#define IDP7_CLK_I_DAI_PB03_O_REG                 0x2433
#define DIT_HFCLK_I_DAI_PB03_O_REG                0x2433
#define IDP3_CLK_I_DAI_PB04_O_REG                 0x2433
#define IDP4_CLK_I_DAI_PB04_O_REG                 0x2433
#define IDP5_CLK_I_DAI_PB04_O_REG                 0x2433
#define IDP6_CLK_I_DAI_PB04_O_REG                 0x2433
#define IDP7_CLK_I_DAI_PB04_O_REG                 0x2433
#define DIT_HFCLK_I_DAI_PB04_O_REG                0x2433
#define IDP3_CLK_I_DAI_PB05_O_REG                 0x2433
#define IDP4_CLK_I_DAI_PB05_O_REG                 0x2433
#define IDP5_CLK_I_DAI_PB05_O_REG                 0x2433
#define IDP6_CLK_I_DAI_PB05_O_REG                 0x2433
#define IDP7_CLK_I_DAI_PB05_O_REG                 0x2433
#define DIT_HFCLK_I_DAI_PB05_O_REG                0x2433
#define IDP3_CLK_I_DAI_PB06_O_REG                 0x2433
#define IDP4_CLK_I_DAI_PB06_O_REG                 0x2433
#define IDP5_CLK_I_DAI_PB06_O_REG                 0x2433
#define IDP6_CLK_I_DAI_PB06_O_REG                 0x2433
#define IDP7_CLK_I_DAI_PB06_O_REG                 0x2433
#define DIT_HFCLK_I_DAI_PB06_O_REG                0x2433
#define IDP3_CLK_I_DAI_PB07_O_REG                 0x2433
#define IDP4_CLK_I_DAI_PB07_O_REG                 0x2433
#define IDP5_CLK_I_DAI_PB07_O_REG                 0x2433
#define IDP6_CLK_I_DAI_PB07_O_REG                 0x2433
#define IDP7_CLK_I_DAI_PB07_O_REG                 0x2433
#define DIT_HFCLK_I_DAI_PB07_O_REG                0x2433
#define IDP3_CLK_I_DAI_PB08_O_REG                 0x2433
#define IDP4_CLK_I_DAI_PB08_O_REG                 0x2433
#define IDP5_CLK_I_DAI_PB08_O_REG                 0x2433
#define IDP6_CLK_I_DAI_PB08_O_REG                 0x2433
#define IDP7_CLK_I_DAI_PB08_O_REG                 0x2433
#define DIT_HFCLK_I_DAI_PB08_O_REG                0x2433
#define IDP3_CLK_I_DAI_PB09_O_REG                 0x2433
#define IDP4_CLK_I_DAI_PB09_O_REG                 0x2433
#define IDP5_CLK_I_DAI_PB09_O_REG                 0x2433
#define IDP6_CLK_I_DAI_PB09_O_REG                 0x2433
#define IDP7_CLK_I_DAI_PB09_O_REG                 0x2433
#define DIT_HFCLK_I_DAI_PB09_O_REG                0x2433
#define IDP3_CLK_I_DAI_PB10_O_REG                 0x2433
#define IDP4_CLK_I_DAI_PB10_O_REG                 0x2433
#define IDP5_CLK_I_DAI_PB10_O_REG                 0x2433
#define IDP6_CLK_I_DAI_PB10_O_REG                 0x2433
#define IDP7_CLK_I_DAI_PB10_O_REG                 0x2433
#define DIT_HFCLK_I_DAI_PB10_O_REG                0x2433
#define IDP3_CLK_I_DAI_PB11_O_REG                 0x2433
#define IDP4_CLK_I_DAI_PB11_O_REG                 0x2433
#define IDP5_CLK_I_DAI_PB11_O_REG                 0x2433
#define IDP6_CLK_I_DAI_PB11_O_REG                 0x2433
#define IDP7_CLK_I_DAI_PB11_O_REG                 0x2433
#define DIT_HFCLK_I_DAI_PB11_O_REG                0x2433
#define IDP3_CLK_I_DAI_PB12_O_REG                 0x2433
#define IDP4_CLK_I_DAI_PB12_O_REG                 0x2433
#define IDP5_CLK_I_DAI_PB12_O_REG                 0x2433
#define IDP6_CLK_I_DAI_PB12_O_REG                 0x2433
#define IDP7_CLK_I_DAI_PB12_O_REG                 0x2433
#define DIT_HFCLK_I_DAI_PB12_O_REG                0x2433
#define IDP3_CLK_I_DAI_PB13_O_REG                 0x2433
#define IDP4_CLK_I_DAI_PB13_O_REG                 0x2433
#define IDP5_CLK_I_DAI_PB13_O_REG                 0x2433
#define IDP6_CLK_I_DAI_PB13_O_REG                 0x2433
#define IDP7_CLK_I_DAI_PB13_O_REG                 0x2433
#define DIT_HFCLK_I_DAI_PB13_O_REG                0x2433
#define IDP3_CLK_I_DAI_PB14_O_REG                 0x2433
#define IDP4_CLK_I_DAI_PB14_O_REG                 0x2433
#define IDP5_CLK_I_DAI_PB14_O_REG                 0x2433
#define IDP6_CLK_I_DAI_PB14_O_REG                 0x2433
#define IDP7_CLK_I_DAI_PB14_O_REG                 0x2433
#define DIT_HFCLK_I_DAI_PB14_O_REG                0x2433
#define IDP3_CLK_I_DAI_PB15_O_REG                 0x2433
#define IDP4_CLK_I_DAI_PB15_O_REG                 0x2433
#define IDP5_CLK_I_DAI_PB15_O_REG                 0x2433
#define IDP6_CLK_I_DAI_PB15_O_REG                 0x2433
#define IDP7_CLK_I_DAI_PB15_O_REG                 0x2433
#define DIT_HFCLK_I_DAI_PB15_O_REG                0x2433
#define IDP3_CLK_I_DAI_PB16_O_REG                 0x2433
#define IDP4_CLK_I_DAI_PB16_O_REG                 0x2433
#define IDP5_CLK_I_DAI_PB16_O_REG                 0x2433
#define IDP6_CLK_I_DAI_PB16_O_REG                 0x2433
#define IDP7_CLK_I_DAI_PB16_O_REG                 0x2433
#define DIT_HFCLK_I_DAI_PB16_O_REG                0x2433
#define IDP3_CLK_I_DAI_PB17_O_REG                 0x2433
#define IDP4_CLK_I_DAI_PB17_O_REG                 0x2433
#define IDP5_CLK_I_DAI_PB17_O_REG                 0x2433
#define IDP6_CLK_I_DAI_PB17_O_REG                 0x2433
#define IDP7_CLK_I_DAI_PB17_O_REG                 0x2433
#define DIT_HFCLK_I_DAI_PB17_O_REG                0x2433
#define IDP3_CLK_I_DAI_PB18_O_REG                 0x2433
#define IDP4_CLK_I_DAI_PB18_O_REG                 0x2433
#define IDP5_CLK_I_DAI_PB18_O_REG                 0x2433
#define IDP6_CLK_I_DAI_PB18_O_REG                 0x2433
#define IDP7_CLK_I_DAI_PB18_O_REG                 0x2433
#define DIT_HFCLK_I_DAI_PB18_O_REG                0x2433
#define IDP3_CLK_I_DAI_PB19_O_REG                 0x2433
#define IDP4_CLK_I_DAI_PB19_O_REG                 0x2433
#define IDP5_CLK_I_DAI_PB19_O_REG                 0x2433
#define IDP6_CLK_I_DAI_PB19_O_REG                 0x2433
#define IDP7_CLK_I_DAI_PB19_O_REG                 0x2433
#define DIT_HFCLK_I_DAI_PB19_O_REG                0x2433
#define IDP3_CLK_I_DAI_PB20_O_REG                 0x2433
#define IDP4_CLK_I_DAI_PB20_O_REG                 0x2433
#define IDP5_CLK_I_DAI_PB20_O_REG                 0x2433
#define IDP6_CLK_I_DAI_PB20_O_REG                 0x2433
#define IDP7_CLK_I_DAI_PB20_O_REG                 0x2433
#define DIT_HFCLK_I_DAI_PB20_O_REG                0x2433
#define IDP3_CLK_I_SPORT0_CLK_O_REG               0x2433
#define IDP4_CLK_I_SPORT0_CLK_O_REG               0x2433
#define IDP5_CLK_I_SPORT0_CLK_O_REG               0x2433
#define IDP6_CLK_I_SPORT0_CLK_O_REG               0x2433
#define IDP7_CLK_I_SPORT0_CLK_O_REG               0x2433
#define DIT_HFCLK_I_SPORT0_CLK_O_REG              0x2433
#define IDP3_CLK_I_SPORT1_CLK_O_REG               0x2433
#define IDP4_CLK_I_SPORT1_CLK_O_REG               0x2433
#define IDP5_CLK_I_SPORT1_CLK_O_REG               0x2433
#define IDP6_CLK_I_SPORT1_CLK_O_REG               0x2433
#define IDP7_CLK_I_SPORT1_CLK_O_REG               0x2433
#define DIT_HFCLK_I_SPORT1_CLK_O_REG              0x2433
#define IDP3_CLK_I_SPORT2_CLK_O_REG               0x2433
#define IDP4_CLK_I_SPORT2_CLK_O_REG               0x2433
#define IDP5_CLK_I_SPORT2_CLK_O_REG               0x2433
#define IDP6_CLK_I_SPORT2_CLK_O_REG               0x2433
#define IDP7_CLK_I_SPORT2_CLK_O_REG               0x2433
#define DIT_HFCLK_I_SPORT2_CLK_O_REG              0x2433
#define IDP3_CLK_I_SPORT3_CLK_O_REG               0x2433
#define IDP4_CLK_I_SPORT3_CLK_O_REG               0x2433
#define IDP5_CLK_I_SPORT3_CLK_O_REG               0x2433
#define IDP6_CLK_I_SPORT3_CLK_O_REG               0x2433
#define IDP7_CLK_I_SPORT3_CLK_O_REG               0x2433
#define DIT_HFCLK_I_SPORT3_CLK_O_REG              0x2433
#define IDP3_CLK_I_SPORT4_CLK_O_REG               0x2433
#define IDP4_CLK_I_SPORT4_CLK_O_REG               0x2433
#define IDP5_CLK_I_SPORT4_CLK_O_REG               0x2433
#define IDP6_CLK_I_SPORT4_CLK_O_REG               0x2433
#define IDP7_CLK_I_SPORT4_CLK_O_REG               0x2433
#define DIT_HFCLK_I_SPORT4_CLK_O_REG              0x2433
#define IDP3_CLK_I_SPORT5_CLK_O_REG               0x2433
#define IDP4_CLK_I_SPORT5_CLK_O_REG               0x2433
#define IDP5_CLK_I_SPORT5_CLK_O_REG               0x2433
#define IDP6_CLK_I_SPORT5_CLK_O_REG               0x2433
#define IDP7_CLK_I_SPORT5_CLK_O_REG               0x2433
#define DIT_HFCLK_I_SPORT5_CLK_O_REG              0x2433
#define IDP3_CLK_I_DIR_CLK_O_REG                  0x2433
#define IDP4_CLK_I_DIR_CLK_O_REG                  0x2433
#define IDP5_CLK_I_DIR_CLK_O_REG                  0x2433
#define IDP6_CLK_I_DIR_CLK_O_REG                  0x2433
#define IDP7_CLK_I_DIR_CLK_O_REG                  0x2433
#define DIT_HFCLK_I_DIR_TDMCLK_O_REG              0x2433
#define IDP3_CLK_I_DIR_TDMCLK_O_REG               0x2433
#define IDP4_CLK_I_DIR_TDMCLK_O_REG               0x2433
#define IDP5_CLK_I_DIR_TDMCLK_O_REG               0x2433
#define IDP6_CLK_I_DIR_TDMCLK_O_REG               0x2433
#define IDP7_CLK_I_DIR_TDMCLK_O_REG               0x2433
#define DIT_HFCLK_I_DIR_TDMCLK_O_REG              0x2433
#define IDP3_CLK_I_PCG_CLKA_O_REG                 0x2433
#define IDP4_CLK_I_PCG_CLKA_O_REG                 0x2433
#define IDP5_CLK_I_PCG_CLKA_O_REG                 0x2433
#define IDP6_CLK_I_PCG_CLKA_O_REG                 0x2433
#define IDP7_CLK_I_PCG_CLKA_O_REG                 0x2433
#define DIT_HFCLK_I_PCG_CLKA_O_REG                0x2433
#define IDP3_CLK_I_PCG_CLKB_O_REG                 0x2433
#define IDP4_CLK_I_PCG_CLKB_O_REG                 0x2433
#define IDP5_CLK_I_PCG_CLKB_O_REG                 0x2433
#define IDP6_CLK_I_PCG_CLKB_O_REG                 0x2433
#define IDP7_CLK_I_PCG_CLKB_O_REG                 0x2433
#define DIT_HFCLK_I_PCG_CLKB_O_REG                0x2433
#define IDP3_CLK_I_LOW_REG                        0x2433
#define IDP4_CLK_I_LOW_REG                        0x2433
#define IDP5_CLK_I_LOW_REG                        0x2433
#define IDP6_CLK_I_LOW_REG                        0x2433
#define IDP7_CLK_I_LOW_REG                        0x2433
#define DIT_HFCLK_I_LOW_REG                       0x2433
#define IDP3_CLK_I_HIGH_REG                       0x2433
#define IDP4_CLK_I_HIGH_REG                       0x2433
#define IDP5_CLK_I_HIGH_REG                       0x2433
#define IDP6_CLK_I_HIGH_REG                       0x2433
#define IDP7_CLK_I_HIGH_REG                       0x2433
#define DIT_HFCLK_I_HIGH_REG                      0x2433

//SRU_CLK4
#define PCG_EXTA_I_DAI_PB01_O_REG                 0x2434
#define PCG_EXTB_I_DAI_PB01_O_REG                 0x2434
#define SPDIF_PLLCLK_I_DAI_PB01_O_REG             0x2434
#define SPIB_CLK_I_DAI_PB01_O_REG                 0x2434
#define PCG_SYNC_CLKA_I_DAI_PB01_O_REG            0x2434
#define PCG_SYNC_CLKB_I_DAI_PB01_O_REG            0x2434
#define PCG_EXTA_I_DAI_PB02_O_REG                 0x2434
#define PCG_EXTB_I_DAI_PB02_O_REG                 0x2434
#define SPDIF_PLLCLK_I_DAI_PB02_O_REG             0x2434
#define SPIB_CLK_I_DAI_PB02_O_REG                 0x2434
#define PCG_SYNC_CLKA_I_DAI_PB02_O_REG            0x2434
#define PCG_SYNC_CLKB_I_DAI_PB02_O_REG            0x2434
#define PCG_EXTA_I_DAI_PB03_O_REG                 0x2434
#define PCG_EXTB_I_DAI_PB03_O_REG                 0x2434
#define SPDIF_PLLCLK_I_DAI_PB03_O_REG             0x2434
#define SPIB_CLK_I_DAI_PB03_O_REG                 0x2434
#define PCG_SYNC_CLKA_I_DAI_PB03_O_REG            0x2434
#define PCG_SYNC_CLKB_I_DAI_PB03_O_REG            0x2434
#define PCG_EXTA_I_DAI_PB04_O_REG                 0x2434
#define PCG_EXTB_I_DAI_PB04_O_REG                 0x2434
#define SPDIF_PLLCLK_I_DAI_PB04_O_REG             0x2434
#define SPIB_CLK_I_DAI_PB04_O_REG                 0x2434
#define PCG_SYNC_CLKA_I_DAI_PB04_O_REG            0x2434
#define PCG_SYNC_CLKB_I_DAI_PB04_O_REG            0x2434
#define PCG_EXTA_I_DAI_PB05_O_REG                 0x2434
#define PCG_EXTB_I_DAI_PB05_O_REG                 0x2434
#define SPDIF_PLLCLK_I_DAI_PB05_O_REG             0x2434
#define SPIB_CLK_I_DAI_PB05_O_REG                 0x2434
#define PCG_SYNC_CLKA_I_DAI_PB05_O_REG            0x2434
#define PCG_SYNC_CLKB_I_DAI_PB05_O_REG            0x2434
#define PCG_EXTA_I_DAI_PB06_O_REG                 0x2434
#define PCG_EXTB_I_DAI_PB06_O_REG                 0x2434
#define SPDIF_PLLCLK_I_DAI_PB06_O_REG             0x2434
#define SPIB_CLK_I_DAI_PB06_O_REG                 0x2434
#define PCG_SYNC_CLKA_I_DAI_PB06_O_REG            0x2434
#define PCG_SYNC_CLKB_I_DAI_PB06_O_REG            0x2434
#define PCG_EXTA_I_DAI_PB07_O_REG                 0x2434
#define PCG_EXTB_I_DAI_PB07_O_REG                 0x2434
#define SPDIF_PLLCLK_I_DAI_PB07_O_REG             0x2434
#define SPIB_CLK_I_DAI_PB07_O_REG                 0x2434
#define PCG_SYNC_CLKA_I_DAI_PB07_O_REG            0x2434
#define PCG_SYNC_CLKB_I_DAI_PB07_O_REG            0x2434
#define PCG_EXTA_I_DAI_PB08_O_REG                 0x2434
#define PCG_EXTB_I_DAI_PB08_O_REG                 0x2434
#define SPDIF_PLLCLK_I_DAI_PB08_O_REG             0x2434
#define SPIB_CLK_I_DAI_PB08_O_REG                 0x2434
#define PCG_SYNC_CLKA_I_DAI_PB08_O_REG            0x2434
#define PCG_SYNC_CLKB_I_DAI_PB08_O_REG            0x2434
#define PCG_EXTA_I_DAI_PB09_O_REG                 0x2434
#define PCG_EXTB_I_DAI_PB09_O_REG                 0x2434
#define SPDIF_PLLCLK_I_DAI_PB09_O_REG             0x2434
#define SPIB_CLK_I_DAI_PB09_O_REG                 0x2434
#define PCG_SYNC_CLKA_I_DAI_PB09_O_REG            0x2434
#define PCG_SYNC_CLKB_I_DAI_PB09_O_REG            0x2434
#define PCG_EXTA_I_DAI_PB10_O_REG                 0x2434
#define PCG_EXTB_I_DAI_PB10_O_REG                 0x2434
#define SPDIF_PLLCLK_I_DAI_PB10_O_REG             0x2434
#define SPIB_CLK_I_DAI_PB10_O_REG                 0x2434
#define PCG_SYNC_CLKA_I_DAI_PB10_O_REG            0x2434
#define PCG_SYNC_CLKB_I_DAI_PB10_O_REG            0x2434
#define PCG_EXTA_I_DAI_PB11_O_REG                 0x2434
#define PCG_EXTB_I_DAI_PB11_O_REG                 0x2434
#define SPDIF_PLLCLK_I_DAI_PB11_O_REG             0x2434
#define SPIB_CLK_I_DAI_PB11_O_REG                 0x2434
#define PCG_SYNC_CLKA_I_DAI_PB11_O_REG            0x2434
#define PCG_SYNC_CLKB_I_DAI_PB11_O_REG            0x2434
#define PCG_EXTA_I_DAI_PB12_O_REG                 0x2434
#define PCG_EXTB_I_DAI_PB12_O_REG                 0x2434
#define SPDIF_PLLCLK_I_DAI_PB12_O_REG             0x2434
#define SPIB_CLK_I_DAI_PB12_O_REG                 0x2434
#define PCG_SYNC_CLKA_I_DAI_PB12_O_REG            0x2434
#define PCG_SYNC_CLKB_I_DAI_PB12_O_REG            0x2434
#define PCG_EXTA_I_DAI_PB13_O_REG                 0x2434
#define PCG_EXTB_I_DAI_PB13_O_REG                 0x2434
#define SPDIF_PLLCLK_I_DAI_PB13_O_REG             0x2434
#define SPIB_CLK_I_DAI_PB13_O_REG                 0x2434
#define PCG_SYNC_CLKA_I_DAI_PB13_O_REG            0x2434
#define PCG_SYNC_CLKB_I_DAI_PB13_O_REG            0x2434
#define PCG_EXTA_I_DAI_PB14_O_REG                 0x2434
#define PCG_EXTB_I_DAI_PB14_O_REG                 0x2434
#define SPDIF_PLLCLK_I_DAI_PB14_O_REG             0x2434
#define SPIB_CLK_I_DAI_PB14_O_REG                 0x2434
#define PCG_SYNC_CLKA_I_DAI_PB14_O_REG            0x2434
#define PCG_SYNC_CLKB_I_DAI_PB14_O_REG            0x2434
#define PCG_EXTA_I_DAI_PB15_O_REG                 0x2434
#define PCG_EXTB_I_DAI_PB15_O_REG                 0x2434
#define SPDIF_PLLCLK_I_DAI_PB15_O_REG             0x2434
#define SPIB_CLK_I_DAI_PB15_O_REG                 0x2434
#define PCG_SYNC_CLKA_I_DAI_PB15_O_REG            0x2434
#define PCG_SYNC_CLKB_I_DAI_PB15_O_REG            0x2434
#define PCG_EXTA_I_DAI_PB16_O_REG                 0x2434
#define PCG_EXTB_I_DAI_PB16_O_REG                 0x2434
#define SPDIF_PLLCLK_I_DAI_PB16_O_REG             0x2434
#define SPIB_CLK_I_DAI_PB16_O_REG                 0x2434
#define PCG_SYNC_CLKA_I_DAI_PB16_O_REG            0x2434
#define PCG_SYNC_CLKB_I_DAI_PB16_O_REG            0x2434
#define PCG_EXTA_I_DAI_PB17_O_REG                 0x2434
#define PCG_EXTB_I_DAI_PB17_O_REG                 0x2434
#define SPDIF_PLLCLK_I_DAI_PB17_O_REG             0x2434
#define SPIB_CLK_I_DAI_PB17_O_REG                 0x2434
#define PCG_SYNC_CLKA_I_DAI_PB17_O_REG            0x2434
#define PCG_SYNC_CLKB_I_DAI_PB17_O_REG            0x2434
#define PCG_EXTA_I_DAI_PB18_O_REG                 0x2434
#define PCG_EXTB_I_DAI_PB18_O_REG                 0x2434
#define SPDIF_PLLCLK_I_DAI_PB18_O_REG             0x2434
#define SPIB_CLK_I_DAI_PB18_O_REG                 0x2434
#define PCG_SYNC_CLKA_I_DAI_PB18_O_REG            0x2434
#define PCG_SYNC_CLKB_I_DAI_PB18_O_REG            0x2434
#define PCG_EXTA_I_DAI_PB19_O_REG                 0x2434
#define PCG_EXTB_I_DAI_PB19_O_REG                 0x2434
#define SPDIF_PLLCLK_I_DAI_PB19_O_REG             0x2434
#define SPIB_CLK_I_DAI_PB19_O_REG                 0x2434
#define PCG_SYNC_CLKA_I_DAI_PB19_O_REG            0x2434
#define PCG_SYNC_CLKB_I_DAI_PB19_O_REG            0x2434
#define PCG_EXTA_I_DAI_PB20_O_REG                 0x2434
#define PCG_EXTB_I_DAI_PB20_O_REG                 0x2434
#define SPDIF_PLLCLK_I_DAI_PB20_O_REG             0x2434
#define SPIB_CLK_I_DAI_PB20_O_REG                 0x2434
#define PCG_SYNC_CLKA_I_DAI_PB20_O_REG            0x2434
#define PCG_SYNC_CLKB_I_DAI_PB20_O_REG            0x2434
#define PCG_EXTA_I_SPORT0_CLK_O_REG               0x2434
#define PCG_EXTB_I_SPORT0_CLK_O_REG               0x2434
#define SPDIF_PLLCLK_I_SPORT0_CLK_O_REG           0x2434
#define SPIB_CLK_I_SPORT0_CLK_O_REG               0x2434
#define PCG_SYNC_CLKA_I_SPORT0_CLK_O_REG          0x2434
#define PCG_SYNC_CLKB_I_SPORT0_CLK_O_REG          0x2434
#define PCG_EXTA_I_SPORT0_CLK_O_REG               0x2434
#define PCG_EXTB_I_SPORT1_CLK_O_REG               0x2434
#define SPDIF_PLLCLK_I_SPORT1_CLK_O_REG           0x2434
#define SPIB_CLK_I_SPORT1_CLK_O_REG               0x2434
#define PCG_SYNC_CLKA_I_SPORT1_CLK_O_REG          0x2434
#define PCG_SYNC_CLKB_I_SPORT1_CLK_O_REG          0x2434
#define PCG_EXTA_I_SPORT2_CLK_O_REG               0x2434
#define PCG_EXTB_I_SPORT2_CLK_O_REG               0x2434
#define SPDIF_PLLCLK_I_SPORT2_CLK_O_REG           0x2434
#define SPIB_CLK_I_SPORT2_CLK_O_REG               0x2434
#define PCG_SYNC_CLKA_I_SPORT2_CLK_O_REG          0x2434
#define PCG_SYNC_CLKB_I_SPORT2_CLK_O_REG          0x2434
#define PCG_EXTA_I_SPORT3_CLK_O_REG               0x2434
#define PCG_EXTB_I_SPORT3_CLK_O_REG               0x2434
#define SPDIF_PLLCLK_I_SPORT3_CLK_O_REG           0x2434
#define SPIB_CLK_I_SPORT3_CLK_O_REG               0x2434
#define PCG_SYNC_CLKA_I_SPORT3_CLK_O_REG          0x2434
#define PCG_SYNC_CLKB_I_SPORT3_CLK_O_REG          0x2434
#define PCG_EXTA_I_SPORT4_CLK_O_REG               0x2434
#define PCG_EXTB_I_SPORT4_CLK_O_REG               0x2434
#define SPDIF_PLLCLK_I_SPORT4_CLK_O_REG           0x2434
#define SPIB_CLK_I_SPORT4_CLK_O_REG               0x2434
#define PCG_SYNC_CLKA_I_SPORT4_CLK_O_REG          0x2434
#define PCG_SYNC_CLKB_I_SPORT4_CLK_O_REG          0x2434
#define PCG_EXTA_I_SPORT5_CLK_O_REG               0x2434
#define PCG_EXTB_I_SPORT5_CLK_O_REG               0x2434
#define SPDIF_PLLCLK_I_SPORT5_CLK_O_REG           0x2434
#define SPIB_CLK_I_SPORT5_CLK_O_REG               0x2434
#define PCG_SYNC_CLKA_I_SPORT5_CLK_O_REG          0x2434
#define PCG_SYNC_CLKB_I_SPORT5_CLK_O_REG          0x2434
#define PCG_EXTA_I_DIR_CLK_O_REG                  0x2434
#define PCG_EXTB_I_DIR_CLK_O_REG                  0x2434
#define SPDIF_PLLCLK_I_DIR_CLK_O_REG              0x2434
#define SPIB_CLK_I_DIR_CLK_O_REG                  0x2434
#define PCG_SYNC_CLKA_I_DIR_CLK_O_REG             0x2434
#define PCG_SYNC_CLKB_I_DIR_TDMCLK_O_REG          0x2434
#define PCG_EXTA_I_DIR_TDMCLK_O_REG               0x2434
#define PCG_EXTB_I_DIR_TDMCLK_O_REG               0x2434
#define SPDIF_PLLCLK_I_DIR_TDMCLK_O_REG           0x2434
#define SPIB_CLK_I_DIR_TDMCLK_O_REG               0x2434
#define PCG_SYNC_CLKA_I_DIR_TDMCLK_O_REG          0x2434
#define PCG_SYNC_CLKB_I_DIR_TDMCLK_O_REG          0x2434
#define PCG_EXTA_I_PCG_CLKA_O_REG                 0x2434
#define PCG_EXTB_I_PCG_CLKA_O_REG                 0x2434
#define SPDIF_PLLCLK_I_PCG_CLKA_O_REG             0x2434
#define SPIB_CLK_I_PCG_CLKA_O_REG                 0x2434
#define PCG_SYNC_CLKA_I_PCG_CLKA_O_REG            0x2434
#define PCG_SYNC_CLKB_I_PCG_CLKA_O_REG            0x2434
#define PCG_EXTA_I_PCG_CLKB_O_REG                 0x2434
#define PCG_EXTB_I_PCG_CLKB_O_REG                 0x2434
#define SPDIF_PLLCLK_I_PCG_CLKB_O_REG             0x2434
#define SPIB_CLK_I_PCG_CLKB_O_REG                 0x2434
#define PCG_SYNC_CLKA_I_PCG_CLKB_O_REG            0x2434
#define PCG_SYNC_CLKB_I_PCG_CLKB_O_REG            0x2434
#define PCG_EXTA_I_LOW_REG                        0x2434
#define PCG_EXTB_I_LOW_REG                        0x2434
#define SPDIF_PLLCLK_I_LOW_REG                    0x2434
#define SPIB_CLK_I_LOW_REG                        0x2434
#define PCG_SYNC_CLKA_I_LOW_REG                   0x2434
#define PCG_SYNC_CLKB_I_LOW_REG                   0x2434
#define PCG_EXTA_I_HIGH_REG                       0x2434
#define PCG_EXTB_I_HIGH_REG                       0x2434
#define SPDIF_PLLCLK_I_HIGH_REG                   0x2434
#define SPIB_CLK_I_HIGH_REG                       0x2434
#define PCG_SYNC_CLKA_I_HIGH_REG                  0x2434
#define PCG_SYNC_CLKB_I_HIGH_REG                  0x2434



/*============================================================================*
 *                                                                            *
 *      GROUP B                                                               *
 *                                                                            *
 *============================================================================*/

//------------------------------------------------------------------------------
//  Group B: Input bitfield masks & shifts
//------------------------------------------------------------------------------

//SRU_DAT0
#define SPORT0_Da_I_MASK                            0xFFFFFFC0
#define SPORT0_Da_I_SHIFT                           0
#define SPORT0_Db_I_MASK                            0xFFFFF03F
#define SPORT0_Db_I_SHIFT                           6
#define SPORT1_Da_I_MASK                            0xFFFC0FFF
#define SPORT1_Da_I_SHIFT                           12
#define SPORT1_Db_I_MASK                            0xFF03FFFF
#define SPORT1_Db_I_SHIFT                           18
#define SPORT2_Da_I_MASK                            0xC0FFFFFF
#define SPORT2_Da_I_SHIFT                           24

//SRU_DAT1
#define SPORT2_Db_I_MASK                            0xFFFFFFC0
#define SPORT2_Db_I_SHIFT                           0
#define SPORT3_Da_I_MASK                            0xFFFFF03F
#define SPORT3_Da_I_SHIFT                           6
#define SPORT3_Db_I_MASK                            0xFFFC0FFF
#define SPORT3_Db_I_SHIFT                           12
#define SPORT4_Da_I_MASK                            0xFF03FFFF
#define SPORT4_Da_I_SHIFT                           18
#define SPORT4_Db_I_MASK                            0xC0FFFFFF
#define SPORT4_Db_I_SHIFT                           24

//SRU_DAT2
#define SPORT5_Da_I_MASK                            0xFFFFFFC0
#define SPORT5_Da_I_SHIFT                           0
#define SPORT5_Db_I_MASK                            0xFFFFF03F
#define SPORT5_Db_I_SHIFT                           6
#define SRC0_DAT_IP_I_MASK                          0xFFFC0FFF
#define SRC0_DAT_IP_I_SHIFT                         12
#define SRC1_DAT_IP_I_MASK                          0xFF03FFFF
#define SRC1_DAT_IP_I_SHIFT                         18
#define SRC2_DAT_IP_I_MASK                          0xC0FFFFFF
#define SRC2_DAT_IP_I_SHIFT                         24

//SRU_DAT3
#define SRC3_DAT_IP_I_MASK                          0xFFFFFFC0
#define SRC3_DAT_IP_I_SHIFT                         0
#define SRC0_TDM_OP_I_MASK                          0xFFFFF03F
#define SRC0_TDM_OP_I_SHIFT                         6
#define SRC1_TDM_OP_I_MASK                          0xFFFC0FFF
#define SRC1_TDM_OP_I_SHIFT                         12
#define SRC2_TDM_OP_I_MASK                          0xFF03FFFF
#define SRC2_TDM_OP_I_SHIFT                         18
#define SRC3_TDM_OP_I_MASK                          0xC0FFFFFF
#define SRC3_TDM_OP_I_SHIFT                         24

//SRU_DAT4
#define DIT_DAT_IP_I_MASK                           0xFFFFFFC0
#define DIT_DAT_IP_I_SHIFT                          0
#define IDP0_DAT_I_MASK                             0xFFFFF03F
#define IDP0_DAT_I_SHIFT                            6
#define IDP1_DAT_I_MASK                             0xFFFC0FFF
#define IDP1_DAT_I_SHIFT                            12
#define IDP2_DAT_I_MASK                             0xFF03FFFF
#define IDP2_DAT_I_SHIFT                            18
#define IDP3_DAT_I_MASK                             0xC0FFFFFF
#define IDP3_DAT_I_SHIFT                            24

//SRU_DAT5
#define IDP4_DAT_I_MASK                             0xFFFFFFC0
#define IDP4_DAT_I_SHIFT                            0
#define IDP5_DAT_I_MASK                             0xFFFFF03F
#define IDP5_DAT_I_SHIFT                            6
#define IDP6_DAT_I_MASK                             0xFFFC0FFF
#define IDP6_DAT_I_SHIFT                            12
#define IDP7_DAT_I_MASK                             0xFF03FFFF
#define IDP7_DAT_I_SHIFT                            18




//------------------------------------------------------------------------------
//  Group B: Bitfield encodings
//------------------------------------------------------------------------------

#define SPORT0_Da_I_DAI_PB01_O_ENCODING             0x00
#define SPORT0_Db_I_DAI_PB01_O_ENCODING             0x00
#define SPORT1_Da_I_DAI_PB01_O_ENCODING             0x00
#define SPORT1_Db_I_DAI_PB01_O_ENCODING             0x00
#define SPORT2_Da_I_DAI_PB01_O_ENCODING             0x00
#define SPORT2_Db_I_DAI_PB01_O_ENCODING             0x00
#define SPORT3_Da_I_DAI_PB01_O_ENCODING             0x00
#define SPORT3_Db_I_DAI_PB01_O_ENCODING             0x00
#define SPORT4_Da_I_DAI_PB01_O_ENCODING             0x00
#define SPORT4_Db_I_DAI_PB01_O_ENCODING             0x00
#define SPORT5_Da_I_DAI_PB01_O_ENCODING             0x00
#define SPORT5_Db_I_DAI_PB01_O_ENCODING             0x00
#define SRC0_DAT_IP_I_DAI_PB01_O_ENCODING           0x00
#define SRC1_DAT_IP_I_DAI_PB01_O_ENCODING           0x00
#define SRC2_DAT_IP_I_DAI_PB01_O_ENCODING           0x00
#define SRC3_DAT_IP_I_DAI_PB01_O_ENCODING           0x00
#define SRC0_TDM_OP_I_DAI_PB01_O_ENCODING           0x00
#define SRC1_TDM_OP_I_DAI_PB01_O_ENCODING           0x00
#define SRC2_TDM_OP_I_DAI_PB01_O_ENCODING           0x00
#define SRC3_TDM_OP_I_DAI_PB01_O_ENCODING           0x00
#define DIT_DAT_I_DAI_PB01_O_ENCODING               0x00
#define IDP0_DAT_I_DAI_PB01_O_ENCODING              0x00
#define IDP1_DAT_I_DAI_PB01_O_ENCODING              0x00
#define IDP2_DAT_I_DAI_PB01_O_ENCODING              0x00
#define IDP3_DAT_I_DAI_PB01_O_ENCODING              0x00
#define IDP4_DAT_I_DAI_PB01_O_ENCODING              0x00
#define IDP5_DAT_I_DAI_PB01_O_ENCODING              0x00
#define IDP6_DAT_I_DAI_PB01_O_ENCODING              0x00
#define IDP7_DAT_I_DAI_PB01_O_ENCODING              0x00

#define SPORT0_Da_I_DAI_PB02_O_ENCODING             0x01
#define SPORT0_Db_I_DAI_PB02_O_ENCODING             0x01
#define SPORT1_Da_I_DAI_PB02_O_ENCODING             0x01
#define SPORT1_Db_I_DAI_PB02_O_ENCODING             0x01
#define SPORT2_Da_I_DAI_PB02_O_ENCODING             0x01
#define SPORT2_Db_I_DAI_PB02_O_ENCODING             0x01
#define SPORT3_Da_I_DAI_PB02_O_ENCODING             0x01
#define SPORT3_Db_I_DAI_PB02_O_ENCODING             0x01
#define SPORT4_Da_I_DAI_PB02_O_ENCODING             0x01
#define SPORT4_Db_I_DAI_PB02_O_ENCODING             0x01
#define SPORT5_Da_I_DAI_PB02_O_ENCODING             0x01
#define SPORT5_Db_I_DAI_PB02_O_ENCODING             0x01
#define SRC0_DAT_IP_I_DAI_PB02_O_ENCODING           0x01
#define SRC1_DAT_IP_I_DAI_PB02_O_ENCODING           0x01
#define SRC2_DAT_IP_I_DAI_PB02_O_ENCODING           0x01
#define SRC3_DAT_IP_I_DAI_PB02_O_ENCODING           0x01
#define SRC0_TDM_OP_I_DAI_PB02_O_ENCODING           0x01
#define SRC1_TDM_OP_I_DAI_PB02_O_ENCODING           0x01
#define SRC2_TDM_OP_I_DAI_PB02_O_ENCODING           0x01
#define SRC3_TDM_OP_I_DAI_PB02_O_ENCODING           0x01
#define DIT_DAT_I_DAI_PB02_O_ENCODING               0x01
#define IDP0_DAT_I_DAI_PB02_O_ENCODING              0x01
#define IDP1_DAT_I_DAI_PB02_O_ENCODING              0x01
#define IDP2_DAT_I_DAI_PB02_O_ENCODING              0x01
#define IDP3_DAT_I_DAI_PB02_O_ENCODING              0x01
#define IDP4_DAT_I_DAI_PB02_O_ENCODING              0x01
#define IDP5_DAT_I_DAI_PB02_O_ENCODING              0x01
#define IDP6_DAT_I_DAI_PB02_O_ENCODING              0x01
#define IDP7_DAT_I_DAI_PB02_O_ENCODING              0x01

#define SPORT0_Da_I_DAI_PB03_O_ENCODING             0x02
#define SPORT0_Db_I_DAI_PB03_O_ENCODING             0x02
#define SPORT1_Da_I_DAI_PB03_O_ENCODING             0x02
#define SPORT1_Db_I_DAI_PB03_O_ENCODING             0x02
#define SPORT2_Da_I_DAI_PB03_O_ENCODING             0x02
#define SPORT2_Db_I_DAI_PB03_O_ENCODING             0x02
#define SPORT3_Da_I_DAI_PB03_O_ENCODING             0x02
#define SPORT3_Db_I_DAI_PB03_O_ENCODING             0x02
#define SPORT4_Da_I_DAI_PB03_O_ENCODING             0x02
#define SPORT4_Db_I_DAI_PB03_O_ENCODING             0x02
#define SPORT5_Da_I_DAI_PB03_O_ENCODING             0x02
#define SPORT5_Db_I_DAI_PB03_O_ENCODING             0x02
#define SRC0_DAT_IP_I_DAI_PB03_O_ENCODING           0x02
#define SRC1_DAT_IP_I_DAI_PB03_O_ENCODING           0x02
#define SRC2_DAT_IP_I_DAI_PB03_O_ENCODING           0x02
#define SRC3_DAT_IP_I_DAI_PB03_O_ENCODING           0x02
#define SRC0_TDM_OP_I_DAI_PB03_O_ENCODING           0x02
#define SRC1_TDM_OP_I_DAI_PB03_O_ENCODING           0x02
#define SRC2_TDM_OP_I_DAI_PB03_O_ENCODING           0x02
#define SRC3_TDM_OP_I_DAI_PB03_O_ENCODING           0x02
#define DIT_DAT_I_DAI_PB03_O_ENCODING               0x02
#define IDP0_DAT_I_DAI_PB03_O_ENCODING              0x02
#define IDP1_DAT_I_DAI_PB03_O_ENCODING              0x02
#define IDP2_DAT_I_DAI_PB03_O_ENCODING              0x02
#define IDP3_DAT_I_DAI_PB03_O_ENCODING              0x02
#define IDP4_DAT_I_DAI_PB03_O_ENCODING              0x02
#define IDP5_DAT_I_DAI_PB03_O_ENCODING              0x02
#define IDP6_DAT_I_DAI_PB03_O_ENCODING              0x02
#define IDP7_DAT_I_DAI_PB03_O_ENCODING              0x02

#define SPORT0_Da_I_DAI_PB04_O_ENCODING             0x03
#define SPORT0_Db_I_DAI_PB04_O_ENCODING             0x03
#define SPORT1_Da_I_DAI_PB04_O_ENCODING             0x03
#define SPORT1_Db_I_DAI_PB04_O_ENCODING             0x03
#define SPORT2_Da_I_DAI_PB04_O_ENCODING             0x03
#define SPORT2_Db_I_DAI_PB04_O_ENCODING             0x03
#define SPORT3_Da_I_DAI_PB04_O_ENCODING             0x03
#define SPORT3_Db_I_DAI_PB04_O_ENCODING             0x03
#define SPORT4_Da_I_DAI_PB04_O_ENCODING             0x03
#define SPORT4_Db_I_DAI_PB04_O_ENCODING             0x03
#define SPORT5_Da_I_DAI_PB04_O_ENCODING             0x03
#define SPORT5_Db_I_DAI_PB04_O_ENCODING             0x03
#define SRC0_DAT_IP_I_DAI_PB04_O_ENCODING           0x03
#define SRC1_DAT_IP_I_DAI_PB04_O_ENCODING           0x03
#define SRC2_DAT_IP_I_DAI_PB04_O_ENCODING           0x03
#define SRC3_DAT_IP_I_DAI_PB04_O_ENCODING           0x03
#define SRC0_TDM_OP_I_DAI_PB04_O_ENCODING           0x03
#define SRC1_TDM_OP_I_DAI_PB04_O_ENCODING           0x03
#define SRC2_TDM_OP_I_DAI_PB04_O_ENCODING           0x03
#define SRC3_TDM_OP_I_DAI_PB04_O_ENCODING           0x03
#define DIT_DAT_I_DAI_PB04_O_ENCODING               0x03
#define IDP0_DAT_I_DAI_PB04_O_ENCODING              0x03
#define IDP1_DAT_I_DAI_PB04_O_ENCODING              0x03
#define IDP2_DAT_I_DAI_PB04_O_ENCODING              0x03
#define IDP3_DAT_I_DAI_PB04_O_ENCODING              0x03
#define IDP4_DAT_I_DAI_PB04_O_ENCODING              0x03
#define IDP5_DAT_I_DAI_PB04_O_ENCODING              0x03
#define IDP6_DAT_I_DAI_PB04_O_ENCODING              0x03
#define IDP7_DAT_I_DAI_PB04_O_ENCODING              0x03

#define SPORT0_Da_I_DAI_PB05_O_ENCODING             0x04
#define SPORT0_Db_I_DAI_PB05_O_ENCODING             0x04
#define SPORT1_Da_I_DAI_PB05_O_ENCODING             0x04
#define SPORT1_Db_I_DAI_PB05_O_ENCODING             0x04
#define SPORT2_Da_I_DAI_PB05_O_ENCODING             0x04
#define SPORT2_Db_I_DAI_PB05_O_ENCODING             0x04
#define SPORT3_Da_I_DAI_PB05_O_ENCODING             0x04
#define SPORT3_Db_I_DAI_PB05_O_ENCODING             0x04
#define SPORT4_Da_I_DAI_PB05_O_ENCODING             0x04
#define SPORT4_Db_I_DAI_PB05_O_ENCODING             0x04
#define SPORT5_Da_I_DAI_PB05_O_ENCODING             0x04
#define SPORT5_Db_I_DAI_PB05_O_ENCODING             0x04
#define SRC0_DAT_IP_I_DAI_PB05_O_ENCODING           0x04
#define SRC1_DAT_IP_I_DAI_PB05_O_ENCODING           0x04
#define SRC2_DAT_IP_I_DAI_PB05_O_ENCODING           0x04
#define SRC3_DAT_IP_I_DAI_PB05_O_ENCODING           0x04
#define SRC0_TDM_OP_I_DAI_PB05_O_ENCODING           0x04
#define SRC1_TDM_OP_I_DAI_PB05_O_ENCODING           0x04
#define SRC2_TDM_OP_I_DAI_PB05_O_ENCODING           0x04
#define SRC3_TDM_OP_I_DAI_PB05_O_ENCODING           0x04
#define DIT_DAT_I_DAI_PB05_O_ENCODING               0x04
#define IDP0_DAT_I_DAI_PB05_O_ENCODING              0x04
#define IDP1_DAT_I_DAI_PB05_O_ENCODING              0x04
#define IDP2_DAT_I_DAI_PB05_O_ENCODING              0x04
#define IDP3_DAT_I_DAI_PB05_O_ENCODING              0x04
#define IDP4_DAT_I_DAI_PB05_O_ENCODING              0x04
#define IDP5_DAT_I_DAI_PB05_O_ENCODING              0x04
#define IDP6_DAT_I_DAI_PB05_O_ENCODING              0x04
#define IDP7_DAT_I_DAI_PB05_O_ENCODING              0x04

#define SPORT0_Da_I_DAI_PB06_O_ENCODING             0x05
#define SPORT0_Db_I_DAI_PB06_O_ENCODING             0x05
#define SPORT1_Da_I_DAI_PB06_O_ENCODING             0x05
#define SPORT1_Db_I_DAI_PB06_O_ENCODING             0x05
#define SPORT2_Da_I_DAI_PB06_O_ENCODING             0x05
#define SPORT2_Db_I_DAI_PB06_O_ENCODING             0x05
#define SPORT3_Da_I_DAI_PB06_O_ENCODING             0x05
#define SPORT3_Db_I_DAI_PB06_O_ENCODING             0x05
#define SPORT4_Da_I_DAI_PB06_O_ENCODING             0x05
#define SPORT4_Db_I_DAI_PB06_O_ENCODING             0x05
#define SPORT5_Da_I_DAI_PB06_O_ENCODING             0x05
#define SPORT5_Db_I_DAI_PB06_O_ENCODING             0x05
#define SRC0_DAT_IP_I_DAI_PB06_O_ENCODING           0x05
#define SRC1_DAT_IP_I_DAI_PB06_O_ENCODING           0x05
#define SRC2_DAT_IP_I_DAI_PB06_O_ENCODING           0x05
#define SRC3_DAT_IP_I_DAI_PB06_O_ENCODING           0x05
#define SRC0_TDM_OP_I_DAI_PB06_O_ENCODING           0x05
#define SRC1_TDM_OP_I_DAI_PB06_O_ENCODING           0x05
#define SRC2_TDM_OP_I_DAI_PB06_O_ENCODING           0x05
#define SRC3_TDM_OP_I_DAI_PB06_O_ENCODING           0x05
#define DIT_DAT_I_DAI_PB06_O_ENCODING               0x05
#define IDP0_DAT_I_DAI_PB06_O_ENCODING              0x05
#define IDP1_DAT_I_DAI_PB06_O_ENCODING              0x05
#define IDP2_DAT_I_DAI_PB06_O_ENCODING              0x05
#define IDP3_DAT_I_DAI_PB06_O_ENCODING              0x05
#define IDP4_DAT_I_DAI_PB06_O_ENCODING              0x05
#define IDP5_DAT_I_DAI_PB06_O_ENCODING              0x05
#define IDP6_DAT_I_DAI_PB06_O_ENCODING              0x05
#define IDP7_DAT_I_DAI_PB06_O_ENCODING              0x05

#define SPORT0_Da_I_DAI_PB07_O_ENCODING             0x06
#define SPORT0_Db_I_DAI_PB07_O_ENCODING             0x06
#define SPORT1_Da_I_DAI_PB07_O_ENCODING             0x06
#define SPORT1_Db_I_DAI_PB07_O_ENCODING             0x06
#define SPORT2_Da_I_DAI_PB07_O_ENCODING             0x06
#define SPORT2_Db_I_DAI_PB07_O_ENCODING             0x06
#define SPORT3_Da_I_DAI_PB07_O_ENCODING             0x06
#define SPORT3_Db_I_DAI_PB07_O_ENCODING             0x06
#define SPORT4_Da_I_DAI_PB07_O_ENCODING             0x06
#define SPORT4_Db_I_DAI_PB07_O_ENCODING             0x06
#define SPORT5_Da_I_DAI_PB07_O_ENCODING             0x06
#define SPORT5_Db_I_DAI_PB07_O_ENCODING             0x06
#define SRC0_DAT_IP_I_DAI_PB01_O_ENCODING           0x00
#define SRC1_DAT_IP_I_DAI_PB01_O_ENCODING           0x00
#define SRC2_DAT_IP_I_DAI_PB01_O_ENCODING           0x00
#define SRC3_DAT_IP_I_DAI_PB01_O_ENCODING           0x00
#define SRC0_TDM_OP_I_DAI_PB01_O_ENCODING           0x00
#define SRC1_TDM_OP_I_DAI_PB01_O_ENCODING           0x00
#define SRC2_TDM_OP_I_DAI_PB01_O_ENCODING           0x00
#define SRC3_TDM_OP_I_DAI_PB01_O_ENCODING           0x00
#define DIT_DAT_I_DAI_PB01_O_ENCODING               0x00
#define IDP0_DAT_I_DAI_PB07_O_ENCODING              0x06
#define IDP1_DAT_I_DAI_PB07_O_ENCODING              0x06
#define IDP2_DAT_I_DAI_PB07_O_ENCODING              0x06
#define IDP3_DAT_I_DAI_PB07_O_ENCODING              0x06
#define IDP4_DAT_I_DAI_PB07_O_ENCODING              0x06
#define IDP5_DAT_I_DAI_PB07_O_ENCODING              0x06
#define IDP6_DAT_I_DAI_PB07_O_ENCODING              0x06
#define IDP7_DAT_I_DAI_PB07_O_ENCODING              0x06

#define SPORT0_Da_I_DAI_PB08_O_ENCODING             0x07
#define SPORT0_Db_I_DAI_PB08_O_ENCODING             0x07
#define SPORT1_Da_I_DAI_PB08_O_ENCODING             0x07
#define SPORT1_Db_I_DAI_PB08_O_ENCODING             0x07
#define SPORT2_Da_I_DAI_PB08_O_ENCODING             0x07
#define SPORT2_Db_I_DAI_PB08_O_ENCODING             0x07
#define SPORT3_Da_I_DAI_PB08_O_ENCODING             0x07
#define SPORT3_Db_I_DAI_PB08_O_ENCODING             0x07
#define SPORT4_Da_I_DAI_PB08_O_ENCODING             0x07
#define SPORT4_Db_I_DAI_PB08_O_ENCODING             0x07
#define SPORT5_Da_I_DAI_PB08_O_ENCODING             0x07
#define SPORT5_Db_I_DAI_PB08_O_ENCODING             0x07
#define SRC0_DAT_IP_I_DAI_PB01_O_ENCODING           0x00
#define SRC1_DAT_IP_I_DAI_PB01_O_ENCODING           0x00
#define SRC2_DAT_IP_I_DAI_PB01_O_ENCODING           0x00
#define SRC3_DAT_IP_I_DAI_PB01_O_ENCODING           0x00
#define SRC0_TDM_OP_I_DAI_PB01_O_ENCODING           0x00
#define SRC1_TDM_OP_I_DAI_PB01_O_ENCODING           0x00
#define SRC2_TDM_OP_I_DAI_PB01_O_ENCODING           0x00
#define SRC3_TDM_OP_I_DAI_PB01_O_ENCODING           0x00
#define DIT_DAT_I_DAI_PB01_O_ENCODING               0x00
#define IDP0_DAT_I_DAI_PB08_O_ENCODING              0x07
#define IDP1_DAT_I_DAI_PB08_O_ENCODING              0x07
#define IDP2_DAT_I_DAI_PB08_O_ENCODING              0x07
#define IDP3_DAT_I_DAI_PB08_O_ENCODING              0x07
#define IDP4_DAT_I_DAI_PB08_O_ENCODING              0x07
#define IDP5_DAT_I_DAI_PB08_O_ENCODING              0x07
#define IDP6_DAT_I_DAI_PB08_O_ENCODING              0x07
#define IDP7_DAT_I_DAI_PB08_O_ENCODING              0x07

#define SPORT0_Da_I_DAI_PB09_O_ENCODING             0x08
#define SPORT0_Db_I_DAI_PB09_O_ENCODING             0x08
#define SPORT1_Da_I_DAI_PB09_O_ENCODING             0x08
#define SPORT1_Db_I_DAI_PB09_O_ENCODING             0x08
#define SPORT2_Da_I_DAI_PB09_O_ENCODING             0x08
#define SPORT2_Db_I_DAI_PB09_O_ENCODING             0x08
#define SPORT3_Da_I_DAI_PB09_O_ENCODING             0x08
#define SPORT3_Db_I_DAI_PB09_O_ENCODING             0x08
#define SPORT4_Da_I_DAI_PB09_O_ENCODING             0x08
#define SPORT4_Db_I_DAI_PB09_O_ENCODING             0x08
#define SPORT5_Da_I_DAI_PB09_O_ENCODING             0x08
#define SPORT5_Db_I_DAI_PB09_O_ENCODING             0x08
#define SRC0_DAT_IP_I_DAI_PB01_O_ENCODING           0x00
#define SRC1_DAT_IP_I_DAI_PB01_O_ENCODING           0x00
#define SRC2_DAT_IP_I_DAI_PB01_O_ENCODING           0x00
#define SRC3_DAT_IP_I_DAI_PB01_O_ENCODING           0x00
#define SRC0_TDM_OP_I_DAI_PB01_O_ENCODING           0x00
#define SRC1_TDM_OP_I_DAI_PB01_O_ENCODING           0x00
#define SRC2_TDM_OP_I_DAI_PB01_O_ENCODING           0x00
#define SRC3_TDM_OP_I_DAI_PB01_O_ENCODING           0x00
#define DIT_DAT_I_DAI_PB01_O_ENCODING               0x00
#define IDP0_DAT_I_DAI_PB09_O_ENCODING              0x08
#define IDP1_DAT_I_DAI_PB09_O_ENCODING              0x08
#define IDP2_DAT_I_DAI_PB09_O_ENCODING              0x08
#define IDP3_DAT_I_DAI_PB09_O_ENCODING              0x08
#define IDP4_DAT_I_DAI_PB09_O_ENCODING              0x08
#define IDP5_DAT_I_DAI_PB09_O_ENCODING              0x08
#define IDP6_DAT_I_DAI_PB09_O_ENCODING              0x08
#define IDP7_DAT_I_DAI_PB09_O_ENCODING              0x08

#define SPORT0_Da_I_DAI_PB10_O_ENCODING             0x09
#define SPORT0_Db_I_DAI_PB10_O_ENCODING             0x09
#define SPORT1_Da_I_DAI_PB10_O_ENCODING             0x09
#define SPORT1_Db_I_DAI_PB10_O_ENCODING             0x09
#define SPORT2_Da_I_DAI_PB10_O_ENCODING             0x09
#define SPORT2_Db_I_DAI_PB10_O_ENCODING             0x09
#define SPORT3_Da_I_DAI_PB10_O_ENCODING             0x09
#define SPORT3_Db_I_DAI_PB10_O_ENCODING             0x09
#define SPORT4_Da_I_DAI_PB10_O_ENCODING             0x09
#define SPORT4_Db_I_DAI_PB10_O_ENCODING             0x09
#define SPORT5_Da_I_DAI_PB10_O_ENCODING             0x09
#define SPORT5_Db_I_DAI_PB10_O_ENCODING             0x09
#define SRC0_DAT_IP_I_DAI_PB10_O_ENCODING           0x09
#define SRC1_DAT_IP_I_DAI_PB10_O_ENCODING           0x09
#define SRC2_DAT_IP_I_DAI_PB10_O_ENCODING           0x09
#define SRC3_DAT_IP_I_DAI_PB10_O_ENCODING           0x09
#define SRC0_TDM_OP_I_DAI_PB10_O_ENCODING           0x09
#define SRC1_TDM_OP_I_DAI_PB10_O_ENCODING           0x09
#define SRC2_TDM_OP_I_DAI_PB10_O_ENCODING           0x09
#define SRC3_TDM_OP_I_DAI_PB10_O_ENCODING           0x09
#define DIT_DAT_I_DAI_PB10_O_ENCODING               0x09
#define IDP0_DAT_I_DAI_PB10_O_ENCODING              0x09
#define IDP1_DAT_I_DAI_PB10_O_ENCODING              0x09
#define IDP2_DAT_I_DAI_PB10_O_ENCODING              0x09
#define IDP3_DAT_I_DAI_PB10_O_ENCODING              0x09
#define IDP4_DAT_I_DAI_PB10_O_ENCODING              0x09
#define IDP5_DAT_I_DAI_PB10_O_ENCODING              0x09
#define IDP6_DAT_I_DAI_PB10_O_ENCODING              0x09
#define IDP7_DAT_I_DAI_PB10_O_ENCODING              0x09

#define SPORT0_Da_I_DAI_PB11_O_ENCODING             0x0A
#define SPORT0_Db_I_DAI_PB11_O_ENCODING             0x0A
#define SPORT1_Da_I_DAI_PB11_O_ENCODING             0x0A
#define SPORT1_Db_I_DAI_PB11_O_ENCODING             0x0A
#define SPORT2_Da_I_DAI_PB11_O_ENCODING             0x0A
#define SPORT2_Db_I_DAI_PB11_O_ENCODING             0x0A
#define SPORT3_Da_I_DAI_PB11_O_ENCODING             0x0A
#define SPORT3_Db_I_DAI_PB11_O_ENCODING             0x0A
#define SPORT4_Da_I_DAI_PB11_O_ENCODING             0x0A
#define SPORT4_Db_I_DAI_PB11_O_ENCODING             0x0A
#define SPORT5_Da_I_DAI_PB11_O_ENCODING             0x0A
#define SPORT5_Db_I_DAI_PB11_O_ENCODING             0x0A
#define SRC0_DAT_IP_I_DAI_PB11_O_ENCODING           0x0A
#define SRC1_DAT_IP_I_DAI_PB11_O_ENCODING           0x0A
#define SRC2_DAT_IP_I_DAI_PB11_O_ENCODING           0x0A
#define SRC3_DAT_IP_I_DAI_PB11_O_ENCODING           0x0A
#define SRC0_TDM_OP_I_DAI_PB11_O_ENCODING           0x0A
#define SRC1_TDM_OP_I_DAI_PB11_O_ENCODING           0x0A
#define SRC2_TDM_OP_I_DAI_PB11_O_ENCODING           0x0A
#define SRC3_TDM_OP_I_DAI_PB11_O_ENCODING           0x0A
#define DIT_DAT_I_DAI_PB11_O_ENCODING               0x0A
#define IDP0_DAT_I_DAI_PB11_O_ENCODING              0x0A
#define IDP1_DAT_I_DAI_PB11_O_ENCODING              0x0A
#define IDP2_DAT_I_DAI_PB11_O_ENCODING              0x0A
#define IDP3_DAT_I_DAI_PB11_O_ENCODING              0x0A
#define IDP4_DAT_I_DAI_PB11_O_ENCODING              0x0A
#define IDP5_DAT_I_DAI_PB11_O_ENCODING              0x0A
#define IDP6_DAT_I_DAI_PB11_O_ENCODING              0x0A
#define IDP7_DAT_I_DAI_PB11_O_ENCODING              0x0A

#define SPORT0_Da_I_DAI_PB12_O_ENCODING             0x0B
#define SPORT0_Db_I_DAI_PB12_O_ENCODING             0x0B
#define SPORT1_Da_I_DAI_PB12_O_ENCODING             0x0B
#define SPORT1_Db_I_DAI_PB12_O_ENCODING             0x0B
#define SPORT2_Da_I_DAI_PB12_O_ENCODING             0x0B
#define SPORT2_Db_I_DAI_PB12_O_ENCODING             0x0B
#define SPORT3_Da_I_DAI_PB12_O_ENCODING             0x0B
#define SPORT3_Db_I_DAI_PB12_O_ENCODING             0x0B
#define SPORT4_Da_I_DAI_PB12_O_ENCODING             0x0B
#define SPORT4_Db_I_DAI_PB12_O_ENCODING             0x0B
#define SPORT5_Da_I_DAI_PB12_O_ENCODING             0x0B
#define SPORT5_Db_I_DAI_PB12_O_ENCODING             0x0B
#define SRC0_DAT_IP_I_DAI_PB12_O_ENCODING           0x0B
#define SRC1_DAT_IP_I_DAI_PB12_O_ENCODING           0x0B
#define SRC2_DAT_IP_I_DAI_PB12_O_ENCODING           0x0B
#define SRC3_DAT_IP_I_DAI_PB12_O_ENCODING           0x0B
#define SRC0_TDM_OP_I_DAI_PB12_O_ENCODING           0x0B
#define SRC1_TDM_OP_I_DAI_PB12_O_ENCODING           0x0B
#define SRC2_TDM_OP_I_DAI_PB12_O_ENCODING           0x0B
#define SRC3_TDM_OP_I_DAI_PB12_O_ENCODING           0x0B
#define DIT_DAT_I_DAI_PB12_O_ENCODING               0x0B
#define IDP0_DAT_I_DAI_PB12_O_ENCODING              0x0B
#define IDP1_DAT_I_DAI_PB12_O_ENCODING              0x0B
#define IDP2_DAT_I_DAI_PB12_O_ENCODING              0x0B
#define IDP3_DAT_I_DAI_PB12_O_ENCODING              0x0B
#define IDP4_DAT_I_DAI_PB12_O_ENCODING              0x0B
#define IDP5_DAT_I_DAI_PB12_O_ENCODING              0x0B
#define IDP6_DAT_I_DAI_PB12_O_ENCODING              0x0B
#define IDP7_DAT_I_DAI_PB12_O_ENCODING              0x0B

#define SPORT0_Da_I_DAI_PB13_O_ENCODING             0x0C
#define SPORT0_Db_I_DAI_PB13_O_ENCODING             0x0C
#define SPORT1_Da_I_DAI_PB13_O_ENCODING             0x0C
#define SPORT1_Db_I_DAI_PB13_O_ENCODING             0x0C
#define SPORT2_Da_I_DAI_PB13_O_ENCODING             0x0C
#define SPORT2_Db_I_DAI_PB13_O_ENCODING             0x0C
#define SPORT3_Da_I_DAI_PB13_O_ENCODING             0x0C
#define SPORT3_Db_I_DAI_PB13_O_ENCODING             0x0C
#define SPORT4_Da_I_DAI_PB13_O_ENCODING             0x0C
#define SPORT4_Db_I_DAI_PB13_O_ENCODING             0x0C
#define SPORT5_Da_I_DAI_PB13_O_ENCODING             0x0C
#define SPORT5_Db_I_DAI_PB13_O_ENCODING             0x0C
#define SRC0_DAT_IP_I_DAI_PB13_O_ENCODING           0x0C
#define SRC1_DAT_IP_I_DAI_PB13_O_ENCODING           0x0C
#define SRC2_DAT_IP_I_DAI_PB13_O_ENCODING           0x0C
#define SRC3_DAT_IP_I_DAI_PB13_O_ENCODING           0x0C
#define SRC0_TDM_OP_I_DAI_PB13_O_ENCODING           0x0C
#define SRC1_TDM_OP_I_DAI_PB13_O_ENCODING           0x0C
#define SRC2_TDM_OP_I_DAI_PB13_O_ENCODING           0x0C
#define SRC3_TDM_OP_I_DAI_PB13_O_ENCODING           0x0C
#define DIT_DAT_I_DAI_PB13_O_ENCODING               0x0C
#define IDP0_DAT_I_DAI_PB13_O_ENCODING              0x0C
#define IDP1_DAT_I_DAI_PB13_O_ENCODING              0x0C
#define IDP2_DAT_I_DAI_PB13_O_ENCODING              0x0C
#define IDP3_DAT_I_DAI_PB13_O_ENCODING              0x0C
#define IDP4_DAT_I_DAI_PB13_O_ENCODING              0x0C
#define IDP5_DAT_I_DAI_PB13_O_ENCODING              0x0C
#define IDP6_DAT_I_DAI_PB13_O_ENCODING              0x0C
#define IDP7_DAT_I_DAI_PB13_O_ENCODING              0x0C

#define SPORT0_Da_I_DAI_PB14_O_ENCODING             0x0D
#define SPORT0_Db_I_DAI_PB14_O_ENCODING             0x0D
#define SPORT1_Da_I_DAI_PB14_O_ENCODING             0x0D
#define SPORT1_Db_I_DAI_PB14_O_ENCODING             0x0D
#define SPORT2_Da_I_DAI_PB14_O_ENCODING             0x0D
#define SPORT2_Db_I_DAI_PB14_O_ENCODING             0x0D
#define SPORT3_Da_I_DAI_PB14_O_ENCODING             0x0D
#define SPORT3_Db_I_DAI_PB14_O_ENCODING             0x0D
#define SPORT4_Da_I_DAI_PB14_O_ENCODING             0x0D
#define SPORT4_Db_I_DAI_PB14_O_ENCODING             0x0D
#define SPORT5_Da_I_DAI_PB14_O_ENCODING             0x0D
#define SPORT5_Db_I_DAI_PB14_O_ENCODING             0x0D
#define SRC0_DAT_IP_I_DAI_PB14_O_ENCODING           0x0D
#define SRC1_DAT_IP_I_DAI_PB14_O_ENCODING           0x0D
#define SRC2_DAT_IP_I_DAI_PB14_O_ENCODING           0x0D
#define SRC3_DAT_IP_I_DAI_PB14_O_ENCODING           0x0D
#define SRC0_TDM_OP_I_DAI_PB14_O_ENCODING           0x0D
#define SRC1_TDM_OP_I_DAI_PB14_O_ENCODING           0x0D
#define SRC2_TDM_OP_I_DAI_PB14_O_ENCODING           0x0D
#define SRC3_TDM_OP_I_DAI_PB14_O_ENCODING           0x0D
#define DIT_DAT_I_DAI_PB14_O_ENCODING               0x0D
#define IDP0_DAT_I_DAI_PB14_O_ENCODING              0x0D
#define IDP1_DAT_I_DAI_PB14_O_ENCODING              0x0D
#define IDP2_DAT_I_DAI_PB14_O_ENCODING              0x0D
#define IDP3_DAT_I_DAI_PB14_O_ENCODING              0x0D
#define IDP4_DAT_I_DAI_PB14_O_ENCODING              0x0D
#define IDP5_DAT_I_DAI_PB14_O_ENCODING              0x0D
#define IDP6_DAT_I_DAI_PB14_O_ENCODING              0x0D
#define IDP7_DAT_I_DAI_PB14_O_ENCODING              0x0D

#define SPORT0_Da_I_DAI_PB15_O_ENCODING             0x0E
#define SPORT0_Db_I_DAI_PB15_O_ENCODING             0x0E
#define SPORT1_Da_I_DAI_PB15_O_ENCODING             0x0E
#define SPORT1_Db_I_DAI_PB15_O_ENCODING             0x0E
#define SPORT2_Da_I_DAI_PB15_O_ENCODING             0x0E
#define SPORT2_Db_I_DAI_PB15_O_ENCODING             0x0E
#define SPORT3_Da_I_DAI_PB15_O_ENCODING             0x0E
#define SPORT3_Db_I_DAI_PB15_O_ENCODING             0x0E
#define SPORT4_Da_I_DAI_PB15_O_ENCODING             0x0E
#define SPORT4_Db_I_DAI_PB15_O_ENCODING             0x0E
#define SPORT5_Da_I_DAI_PB15_O_ENCODING             0x0E
#define SPORT5_Db_I_DAI_PB15_O_ENCODING             0x0E
#define SRC0_DAT_IP_I_DAI_PB15_O_ENCODING           0x0E
#define SRC1_DAT_IP_I_DAI_PB15_O_ENCODING           0x0E
#define SRC2_DAT_IP_I_DAI_PB15_O_ENCODING           0x0E
#define SRC3_DAT_IP_I_DAI_PB15_O_ENCODING           0x0E
#define SRC0_TDM_OP_I_DAI_PB15_O_ENCODING           0x0E
#define SRC1_TDM_OP_I_DAI_PB15_O_ENCODING           0x0E
#define SRC2_TDM_OP_I_DAI_PB15_O_ENCODING           0x0E
#define SRC3_TDM_OP_I_DAI_PB15_O_ENCODING           0x0E
#define DIT_DAT_I_DAI_PB15_O_ENCODING               0x0E
#define IDP0_DAT_I_DAI_PB15_O_ENCODING              0x0E
#define IDP1_DAT_I_DAI_PB15_O_ENCODING              0x0E
#define IDP2_DAT_I_DAI_PB15_O_ENCODING              0x0E
#define IDP3_DAT_I_DAI_PB15_O_ENCODING              0x0E
#define IDP4_DAT_I_DAI_PB15_O_ENCODING              0x0E
#define IDP5_DAT_I_DAI_PB15_O_ENCODING              0x0E
#define IDP6_DAT_I_DAI_PB15_O_ENCODING              0x0E
#define IDP7_DAT_I_DAI_PB15_O_ENCODING              0x0E

#define SPORT0_Da_I_DAI_PB16_O_ENCODING             0x0F
#define SPORT0_Db_I_DAI_PB16_O_ENCODING             0x0F
#define SPORT1_Da_I_DAI_PB16_O_ENCODING             0x0F
#define SPORT1_Db_I_DAI_PB16_O_ENCODING             0x0F
#define SPORT2_Da_I_DAI_PB16_O_ENCODING             0x0F
#define SPORT2_Db_I_DAI_PB16_O_ENCODING             0x0F
#define SPORT3_Da_I_DAI_PB16_O_ENCODING             0x0F
#define SPORT3_Db_I_DAI_PB16_O_ENCODING             0x0F
#define SPORT4_Da_I_DAI_PB16_O_ENCODING             0x0F
#define SPORT4_Db_I_DAI_PB16_O_ENCODING             0x0F
#define SPORT5_Da_I_DAI_PB16_O_ENCODING             0x0F
#define SPORT5_Db_I_DAI_PB16_O_ENCODING             0x0F
#define SRC0_DAT_IP_I_DAI_PB16_O_ENCODING           0x0F
#define SRC1_DAT_IP_I_DAI_PB16_O_ENCODING           0x0F
#define SRC2_DAT_IP_I_DAI_PB16_O_ENCODING           0x0F
#define SRC3_DAT_IP_I_DAI_PB16_O_ENCODING           0x0F
#define SRC0_TDM_OP_I_DAI_PB16_O_ENCODING           0x0F
#define SRC1_TDM_OP_I_DAI_PB16_O_ENCODING           0x0F
#define SRC2_TDM_OP_I_DAI_PB16_O_ENCODING           0x0F
#define SRC3_TDM_OP_I_DAI_PB16_O_ENCODING           0x0F
#define DIT_DAT_I_DAI_PB16_O_ENCODING               0x0F
#define IDP0_DAT_I_DAI_PB16_O_ENCODING              0x0F
#define IDP1_DAT_I_DAI_PB16_O_ENCODING              0x0F
#define IDP2_DAT_I_DAI_PB16_O_ENCODING              0x0F
#define IDP3_DAT_I_DAI_PB16_O_ENCODING              0x0F
#define IDP4_DAT_I_DAI_PB16_O_ENCODING              0x0F
#define IDP5_DAT_I_DAI_PB16_O_ENCODING              0x0F
#define IDP6_DAT_I_DAI_PB16_O_ENCODING              0x0F
#define IDP7_DAT_I_DAI_PB16_O_ENCODING              0x0F

#define SPORT0_Da_I_DAI_PB17_O_ENCODING             0x10
#define SPORT0_Db_I_DAI_PB17_O_ENCODING             0x10
#define SPORT1_Da_I_DAI_PB17_O_ENCODING             0x10
#define SPORT1_Db_I_DAI_PB17_O_ENCODING             0x10
#define SPORT2_Da_I_DAI_PB17_O_ENCODING             0x10
#define SPORT2_Db_I_DAI_PB17_O_ENCODING             0x10
#define SPORT3_Da_I_DAI_PB17_O_ENCODING             0x10
#define SPORT3_Db_I_DAI_PB17_O_ENCODING             0x10
#define SPORT4_Da_I_DAI_PB17_O_ENCODING             0x10
#define SPORT4_Db_I_DAI_PB17_O_ENCODING             0x10
#define SPORT5_Da_I_DAI_PB17_O_ENCODING             0x10
#define SPORT5_Db_I_DAI_PB17_O_ENCODING             0x10
#define SRC0_DAT_IP_I_DAI_PB17_O_ENCODING           0x10
#define SRC1_DAT_IP_I_DAI_PB17_O_ENCODING           0x10
#define SRC2_DAT_IP_I_DAI_PB17_O_ENCODING           0x10
#define SRC3_DAT_IP_I_DAI_PB17_O_ENCODING           0x10
#define SRC0_TDM_OP_I_DAI_PB17_O_ENCODING           0x10
#define SRC1_TDM_OP_I_DAI_PB17_O_ENCODING           0x10
#define SRC2_TDM_OP_I_DAI_PB17_O_ENCODING           0x10
#define SRC3_TDM_OP_I_DAI_PB17_O_ENCODING           0x10
#define DIT_DAT_I_DAI_PB17_O_ENCODING               0x10
#define IDP0_DAT_I_DAI_PB17_O_ENCODING              0x10
#define IDP1_DAT_I_DAI_PB17_O_ENCODING              0x10
#define IDP2_DAT_I_DAI_PB17_O_ENCODING              0x10
#define IDP3_DAT_I_DAI_PB17_O_ENCODING              0x10
#define IDP4_DAT_I_DAI_PB17_O_ENCODING              0x10
#define IDP5_DAT_I_DAI_PB17_O_ENCODING              0x10
#define IDP6_DAT_I_DAI_PB17_O_ENCODING              0x10
#define IDP7_DAT_I_DAI_PB17_O_ENCODING              0x10

#define SPORT0_Da_I_DAI_PB18_O_ENCODING             0x11
#define SPORT0_Db_I_DAI_PB18_O_ENCODING             0x11
#define SPORT1_Da_I_DAI_PB18_O_ENCODING             0x11
#define SPORT1_Db_I_DAI_PB18_O_ENCODING             0x11
#define SPORT2_Da_I_DAI_PB18_O_ENCODING             0x11
#define SPORT2_Db_I_DAI_PB18_O_ENCODING             0x11
#define SPORT3_Da_I_DAI_PB18_O_ENCODING             0x11
#define SPORT3_Db_I_DAI_PB18_O_ENCODING             0x11
#define SPORT4_Da_I_DAI_PB18_O_ENCODING             0x11
#define SPORT4_Db_I_DAI_PB18_O_ENCODING             0x11
#define SPORT5_Da_I_DAI_PB18_O_ENCODING             0x11
#define SPORT5_Db_I_DAI_PB18_O_ENCODING             0x11
#define SRC0_DAT_IP_I_DAI_PB18_O_ENCODING           0x11
#define SRC1_DAT_IP_I_DAI_PB18_O_ENCODING           0x11
#define SRC2_DAT_IP_I_DAI_PB18_O_ENCODING           0x11
#define SRC3_DAT_IP_I_DAI_PB18_O_ENCODING           0x11
#define SRC0_TDM_OP_I_DAI_PB18_O_ENCODING           0x11
#define SRC1_TDM_OP_I_DAI_PB18_O_ENCODING           0x11
#define SRC2_TDM_OP_I_DAI_PB18_O_ENCODING           0x11
#define SRC3_TDM_OP_I_DAI_PB18_O_ENCODING           0x11
#define DIT_DAT_I_DAI_PB18_O_ENCODING               0x11
#define IDP0_DAT_I_DAI_PB18_O_ENCODING              0x11
#define IDP1_DAT_I_DAI_PB18_O_ENCODING              0x11
#define IDP2_DAT_I_DAI_PB18_O_ENCODING              0x11
#define IDP3_DAT_I_DAI_PB18_O_ENCODING              0x11
#define IDP4_DAT_I_DAI_PB18_O_ENCODING              0x11
#define IDP5_DAT_I_DAI_PB18_O_ENCODING              0x11
#define IDP6_DAT_I_DAI_PB18_O_ENCODING              0x11
#define IDP7_DAT_I_DAI_PB18_O_ENCODING              0x11

#define SPORT0_Da_I_DAI_PB19_O_ENCODING             0x12
#define SPORT0_Db_I_DAI_PB19_O_ENCODING             0x12
#define SPORT1_Da_I_DAI_PB19_O_ENCODING             0x12
#define SPORT1_Db_I_DAI_PB19_O_ENCODING             0x12
#define SPORT2_Da_I_DAI_PB19_O_ENCODING             0x12
#define SPORT2_Db_I_DAI_PB19_O_ENCODING             0x12
#define SPORT3_Da_I_DAI_PB19_O_ENCODING             0x12
#define SPORT3_Db_I_DAI_PB19_O_ENCODING             0x12
#define SPORT4_Da_I_DAI_PB19_O_ENCODING             0x12
#define SPORT4_Db_I_DAI_PB19_O_ENCODING             0x12
#define SPORT5_Da_I_DAI_PB19_O_ENCODING             0x12
#define SPORT5_Db_I_DAI_PB19_O_ENCODING             0x12
#define SRC0_DAT_IP_I_DAI_PB19_O_ENCODING           0x12
#define SRC1_DAT_IP_I_DAI_PB19_O_ENCODING           0x12
#define SRC2_DAT_IP_I_DAI_PB19_O_ENCODING           0x12
#define SRC3_DAT_IP_I_DAI_PB19_O_ENCODING           0x12
#define SRC0_TDM_OP_I_DAI_PB19_O_ENCODING           0x12
#define SRC1_TDM_OP_I_DAI_PB19_O_ENCODING           0x12
#define SRC2_TDM_OP_I_DAI_PB19_O_ENCODING           0x12
#define SRC3_TDM_OP_I_DAI_PB19_O_ENCODING           0x12
#define DIT_DAT_I_DAI_PB19_O_ENCODING               0x12
#define IDP0_DAT_I_DAI_PB19_O_ENCODING              0x12
#define IDP1_DAT_I_DAI_PB19_O_ENCODING              0x12
#define IDP2_DAT_I_DAI_PB19_O_ENCODING              0x12
#define IDP3_DAT_I_DAI_PB19_O_ENCODING              0x12
#define IDP4_DAT_I_DAI_PB19_O_ENCODING              0x12
#define IDP5_DAT_I_DAI_PB19_O_ENCODING              0x12
#define IDP6_DAT_I_DAI_PB19_O_ENCODING              0x12
#define IDP7_DAT_I_DAI_PB19_O_ENCODING              0x12

#define SPORT0_Da_I_DAI_PB20_O_ENCODING             0x13
#define SPORT0_Db_I_DAI_PB20_O_ENCODING             0x13
#define SPORT1_Da_I_DAI_PB20_O_ENCODING             0x13
#define SPORT1_Db_I_DAI_PB20_O_ENCODING             0x13
#define SPORT2_Da_I_DAI_PB20_O_ENCODING             0x13
#define SPORT2_Db_I_DAI_PB20_O_ENCODING             0x13
#define SPORT3_Da_I_DAI_PB20_O_ENCODING             0x13
#define SPORT3_Db_I_DAI_PB20_O_ENCODING             0x13
#define SPORT4_Da_I_DAI_PB20_O_ENCODING             0x13
#define SPORT4_Db_I_DAI_PB20_O_ENCODING             0x13
#define SPORT5_Da_I_DAI_PB20_O_ENCODING             0x13
#define SPORT5_Db_I_DAI_PB20_O_ENCODING             0x13
#define SRC0_DAT_IP_I_DAI_PB20_O_ENCODING           0x13
#define SRC1_DAT_IP_I_DAI_PB20_O_ENCODING           0x13
#define SRC2_DAT_IP_I_DAI_PB20_O_ENCODING           0x13
#define SRC3_DAT_IP_I_DAI_PB20_O_ENCODING           0x13
#define SRC0_TDM_OP_I_DAI_PB20_O_ENCODING           0x13
#define SRC1_TDM_OP_I_DAI_PB20_O_ENCODING           0x13
#define SRC2_TDM_OP_I_DAI_PB20_O_ENCODING           0x13
#define SRC3_TDM_OP_I_DAI_PB20_O_ENCODING           0x13
#define DIT_DAT_I_DAI_PB20_O_ENCODING               0x13
#define IDP0_DAT_I_DAI_PB20_O_ENCODING              0x13
#define IDP1_DAT_I_DAI_PB20_O_ENCODING              0x13
#define IDP2_DAT_I_DAI_PB20_O_ENCODING              0x13
#define IDP3_DAT_I_DAI_PB20_O_ENCODING              0x13
#define IDP4_DAT_I_DAI_PB20_O_ENCODING              0x13
#define IDP5_DAT_I_DAI_PB20_O_ENCODING              0x13
#define IDP6_DAT_I_DAI_PB20_O_ENCODING              0x13
#define IDP7_DAT_I_DAI_PB20_O_ENCODING              0x13

#define SPORT0_Da_I_SPORT0_Da_O_ENCODING            0x14
#define SPORT0_Db_I_SPORT0_Da_O_ENCODING            0x14
#define SPORT1_Da_I_SPORT0_Da_O_ENCODING            0x14
#define SPORT1_Db_I_SPORT0_Da_O_ENCODING            0x14
#define SPORT2_Da_I_SPORT0_Da_O_ENCODING            0x14
#define SPORT2_Db_I_SPORT0_Da_O_ENCODING            0x14
#define SPORT3_Da_I_SPORT0_Da_O_ENCODING            0x14
#define SPORT3_Db_I_SPORT0_Da_O_ENCODING            0x14
#define SPORT4_Da_I_SPORT0_Da_O_ENCODING            0x14
#define SPORT4_Db_I_SPORT0_Da_O_ENCODING            0x14
#define SPORT5_Da_I_SPORT0_Da_O_ENCODING            0x14
#define SPORT5_Db_I_SPORT0_Da_O_ENCODING            0x14
#define SRC0_DAT_IP_I_SPORT0_Da_O_ENCODING          0x14
#define SRC1_DAT_IP_I_SPORT0_Da_O_ENCODING          0x14
#define SRC2_DAT_IP_I_SPORT0_Da_O_ENCODING          0x14
#define SRC3_DAT_IP_I_SPORT0_Da_O_ENCODING          0x14
#define SRC0_TDM_OP_I_SPORT0_Da_O_ENCODING          0x14
#define SRC1_TDM_OP_I_SPORT0_Da_O_ENCODING          0x14
#define SRC2_TDM_OP_I_SPORT0_Da_O_ENCODING          0x14
#define SRC3_TDM_OP_I_SPORT0_Da_O_ENCODING          0x14
#define DIT_DAT_I_SPORT0_Da_O_ENCODING              0x14
#define IDP0_DAT_I_SPORT0_Da_O_ENCODING             0x14
#define IDP1_DAT_I_SPORT0_Da_O_ENCODING             0x14
#define IDP2_DAT_I_SPORT0_Da_O_ENCODING             0x14
#define IDP3_DAT_I_SPORT0_Da_O_ENCODING             0x14
#define IDP4_DAT_I_SPORT0_Da_O_ENCODING             0x14
#define IDP5_DAT_I_SPORT0_Da_O_ENCODING             0x14
#define IDP6_DAT_I_SPORT0_Da_O_ENCODING             0x14
#define IDP7_DAT_I_SPORT0_Da_O_ENCODING             0x14

#define SPORT0_Da_I_SPORT0_Db_O_ENCODING            0x15
#define SPORT0_Db_I_SPORT0_Db_O_ENCODING            0x15
#define SPORT1_Da_I_SPORT0_Db_O_ENCODING            0x15
#define SPORT1_Db_I_SPORT0_Db_O_ENCODING            0x15
#define SPORT2_Da_I_SPORT0_Db_O_ENCODING            0x15
#define SPORT2_Db_I_SPORT0_Db_O_ENCODING            0x15
#define SPORT3_Da_I_SPORT0_Db_O_ENCODING            0x15
#define SPORT3_Db_I_SPORT0_Db_O_ENCODING            0x15
#define SPORT4_Da_I_SPORT0_Db_O_ENCODING            0x15
#define SPORT4_Db_I_SPORT0_Db_O_ENCODING            0x15
#define SPORT5_Da_I_SPORT0_Db_O_ENCODING            0x15
#define SPORT5_Db_I_SPORT0_Db_O_ENCODING            0x15
#define SRC0_DAT_IP_I_SPORT0_Db_O_ENCODING          0x15
#define SRC1_DAT_IP_I_SPORT0_Db_O_ENCODING          0x15
#define SRC2_DAT_IP_I_SPORT0_Db_O_ENCODING          0x15
#define SRC3_DAT_IP_I_SPORT0_Db_O_ENCODING          0x15
#define SRC0_TDM_OP_I_SPORT0_Db_O_ENCODING          0x15
#define SRC1_TDM_OP_I_SPORT0_Db_O_ENCODING          0x15
#define SRC2_TDM_OP_I_SPORT0_Db_O_ENCODING          0x15
#define SRC3_TDM_OP_I_SPORT0_Db_O_ENCODING          0x15
#define DIT_DAT_I_SPORT0_Db_O_ENCODING              0x15
#define IDP0_DAT_I_SPORT0_Db_O_ENCODING             0x15
#define IDP1_DAT_I_SPORT0_Db_O_ENCODING             0x15
#define IDP2_DAT_I_SPORT0_Db_O_ENCODING             0x15
#define IDP3_DAT_I_SPORT0_Db_O_ENCODING             0x15
#define IDP4_DAT_I_SPORT0_Db_O_ENCODING             0x15
#define IDP5_DAT_I_SPORT0_Db_O_ENCODING             0x15
#define IDP6_DAT_I_SPORT0_Db_O_ENCODING             0x15
#define IDP7_DAT_I_SPORT0_Db_O_ENCODING             0x15

#define SPORT0_Da_I_SPORT1_Da_O_ENCODING            0x16
#define SPORT0_Db_I_SPORT1_Da_O_ENCODING            0x16
#define SPORT1_Da_I_SPORT1_Da_O_ENCODING            0x16
#define SPORT1_Db_I_SPORT1_Da_O_ENCODING            0x16
#define SPORT2_Da_I_SPORT1_Da_O_ENCODING            0x16
#define SPORT2_Db_I_SPORT1_Da_O_ENCODING            0x16
#define SPORT3_Da_I_SPORT1_Da_O_ENCODING            0x16
#define SPORT3_Db_I_SPORT1_Da_O_ENCODING            0x16
#define SPORT4_Da_I_SPORT1_Da_O_ENCODING            0x16
#define SPORT4_Db_I_SPORT1_Da_O_ENCODING            0x16
#define SPORT5_Da_I_SPORT1_Da_O_ENCODING            0x16
#define SPORT5_Db_I_SPORT1_Da_O_ENCODING            0x16
#define SRC0_DAT_IP_I_SPORT1_Da_O_ENCODING          0x16
#define SRC1_DAT_IP_I_SPORT1_Da_O_ENCODING          0x16
#define SRC2_DAT_IP_I_SPORT1_Da_O_ENCODING          0x16
#define SRC3_DAT_IP_I_SPORT1_Da_O_ENCODING          0x16
#define SRC0_TDM_OP_I_SPORT1_Da_O_ENCODING          0x16
#define SRC1_TDM_OP_I_SPORT1_Da_O_ENCODING          0x16
#define SRC2_TDM_OP_I_SPORT1_Da_O_ENCODING          0x16
#define SRC3_TDM_OP_I_SPORT1_Da_O_ENCODING          0x16
#define DIT_DAT_I_SPORT1_Da_O_ENCODING              0x16
#define IDP0_DAT_I_SPORT1_Da_O_ENCODING             0x16
#define IDP1_DAT_I_SPORT1_Da_O_ENCODING             0x16
#define IDP2_DAT_I_SPORT1_Da_O_ENCODING             0x16
#define IDP3_DAT_I_SPORT1_Da_O_ENCODING             0x16
#define IDP4_DAT_I_SPORT1_Da_O_ENCODING             0x16
#define IDP5_DAT_I_SPORT1_Da_O_ENCODING             0x16
#define IDP6_DAT_I_SPORT1_Da_O_ENCODING             0x16
#define IDP7_DAT_I_SPORT1_Da_O_ENCODING             0x16

#define SPORT0_Da_I_SPORT1_Db_O_ENCODING            0x17
#define SPORT0_Db_I_SPORT1_Db_O_ENCODING            0x17
#define SPORT1_Da_I_SPORT1_Db_O_ENCODING            0x17
#define SPORT1_Db_I_SPORT1_Db_O_ENCODING            0x17
#define SPORT2_Da_I_SPORT1_Db_O_ENCODING            0x17
#define SPORT2_Db_I_SPORT1_Db_O_ENCODING            0x17
#define SPORT3_Da_I_SPORT1_Db_O_ENCODING            0x17
#define SPORT3_Db_I_SPORT1_Db_O_ENCODING            0x17
#define SPORT4_Da_I_SPORT1_Db_O_ENCODING            0x17
#define SPORT4_Db_I_SPORT1_Db_O_ENCODING            0x17
#define SPORT5_Da_I_SPORT1_Db_O_ENCODING            0x17
#define SPORT5_Db_I_SPORT1_Db_O_ENCODING            0x17
#define SRC0_DAT_IP_I_SPORT1_Db_O_ENCODING          0x17
#define SRC1_DAT_IP_I_SPORT1_Db_O_ENCODING          0x17
#define SRC2_DAT_IP_I_SPORT1_Db_O_ENCODING          0x17
#define SRC3_DAT_IP_I_SPORT1_Db_O_ENCODING          0x17
#define SRC0_TDM_OP_I_SPORT1_Db_O_ENCODING          0x17
#define SRC1_TDM_OP_I_SPORT1_Db_O_ENCODING          0x17
#define SRC2_TDM_OP_I_SPORT1_Db_O_ENCODING          0x17
#define SRC3_TDM_OP_I_SPORT1_Db_O_ENCODING          0x17
#define DIT_DAT_I_SPORT1_Db_O_ENCODING              0x17
#define IDP0_DAT_I_SPORT1_Db_O_ENCODING             0x17
#define IDP1_DAT_I_SPORT1_Db_O_ENCODING             0x17
#define IDP2_DAT_I_SPORT1_Db_O_ENCODING             0x17
#define IDP3_DAT_I_SPORT1_Db_O_ENCODING             0x17
#define IDP4_DAT_I_SPORT1_Db_O_ENCODING             0x17
#define IDP5_DAT_I_SPORT1_Db_O_ENCODING             0x17
#define IDP6_DAT_I_SPORT1_Db_O_ENCODING             0x17
#define IDP7_DAT_I_SPORT1_Db_O_ENCODING             0x17

#define SPORT0_Da_I_SPORT2_Da_O_ENCODING            0x18
#define SPORT0_Db_I_SPORT2_Da_O_ENCODING            0x18
#define SPORT1_Da_I_SPORT2_Da_O_ENCODING            0x18
#define SPORT1_Db_I_SPORT2_Da_O_ENCODING            0x18
#define SPORT2_Da_I_SPORT2_Da_O_ENCODING            0x18
#define SPORT2_Db_I_SPORT2_Da_O_ENCODING            0x18
#define SPORT3_Da_I_SPORT2_Da_O_ENCODING            0x18
#define SPORT3_Db_I_SPORT2_Da_O_ENCODING            0x18
#define SPORT4_Da_I_SPORT2_Da_O_ENCODING            0x18
#define SPORT4_Db_I_SPORT2_Da_O_ENCODING            0x18
#define SPORT5_Da_I_SPORT2_Da_O_ENCODING            0x18
#define SPORT5_Db_I_SPORT2_Da_O_ENCODING            0x18
#define SRC0_DAT_IP_I_SPORT2_Da_O_ENCODING          0x18
#define SRC1_DAT_IP_I_SPORT2_Da_O_ENCODING          0x18
#define SRC2_DAT_IP_I_SPORT2_Da_O_ENCODING          0x18
#define SRC3_DAT_IP_I_SPORT2_Da_O_ENCODING          0x18
#define SRC0_TDM_OP_I_SPORT2_Da_O_ENCODING          0x18
#define SRC1_TDM_OP_I_SPORT2_Da_O_ENCODING          0x18
#define SRC2_TDM_OP_I_SPORT2_Da_O_ENCODING          0x18
#define SRC3_TDM_OP_I_SPORT2_Da_O_ENCODING          0x18
#define DIT_DAT_I_SPORT2_Da_O_ENCODING              0x18
#define IDP0_DAT_I_SPORT2_Da_O_ENCODING             0x18
#define IDP1_DAT_I_SPORT2_Da_O_ENCODING             0x18
#define IDP2_DAT_I_SPORT2_Da_O_ENCODING             0x18
#define IDP3_DAT_I_SPORT2_Da_O_ENCODING             0x18
#define IDP4_DAT_I_SPORT2_Da_O_ENCODING             0x18
#define IDP5_DAT_I_SPORT2_Da_O_ENCODING             0x18
#define IDP6_DAT_I_SPORT2_Da_O_ENCODING             0x18
#define IDP7_DAT_I_SPORT2_Da_O_ENCODING             0x18

#define SPORT0_Da_I_SPORT2_Db_O_ENCODING            0x19
#define SPORT0_Db_I_SPORT2_Db_O_ENCODING            0x19
#define SPORT1_Da_I_SPORT2_Db_O_ENCODING            0x19
#define SPORT1_Db_I_SPORT2_Db_O_ENCODING            0x19
#define SPORT2_Da_I_SPORT2_Db_O_ENCODING            0x19
#define SPORT2_Db_I_SPORT2_Db_O_ENCODING            0x19
#define SPORT3_Da_I_SPORT2_Db_O_ENCODING            0x19
#define SPORT3_Db_I_SPORT2_Db_O_ENCODING            0x19
#define SPORT4_Da_I_SPORT2_Db_O_ENCODING            0x19
#define SPORT4_Db_I_SPORT2_Db_O_ENCODING            0x19
#define SPORT5_Da_I_SPORT2_Db_O_ENCODING            0x19
#define SPORT5_Db_I_SPORT2_Db_O_ENCODING            0x19
#define SRC0_DAT_IP_I_SPORT2_Db_O_ENCODING          0x19
#define SRC1_DAT_IP_I_SPORT2_Db_O_ENCODING          0x19
#define SRC2_DAT_IP_I_SPORT2_Db_O_ENCODING          0x19
#define SRC3_DAT_IP_I_SPORT2_Db_O_ENCODING          0x19
#define SRC0_TDM_OP_I_SPORT2_Db_O_ENCODING          0x19
#define SRC1_TDM_OP_I_SPORT2_Db_O_ENCODING          0x19
#define SRC2_TDM_OP_I_SPORT2_Db_O_ENCODING          0x19
#define SRC3_TDM_OP_I_SPORT2_Db_O_ENCODING          0x19
#define DIT_DAT_I_SPORT2_Db_O_ENCODING              0x19
#define IDP0_DAT_I_SPORT2_Db_O_ENCODING             0x19
#define IDP1_DAT_I_SPORT2_Db_O_ENCODING             0x19
#define IDP2_DAT_I_SPORT2_Db_O_ENCODING             0x19
#define IDP3_DAT_I_SPORT2_Db_O_ENCODING             0x19
#define IDP4_DAT_I_SPORT2_Db_O_ENCODING             0x19
#define IDP5_DAT_I_SPORT2_Db_O_ENCODING             0x19
#define IDP6_DAT_I_SPORT2_Db_O_ENCODING             0x19
#define IDP7_DAT_I_SPORT2_Db_O_ENCODING             0x19

#define SPORT0_Da_I_SPORT3_Da_O_ENCODING            0x1A
#define SPORT0_Db_I_SPORT3_Da_O_ENCODING            0x1A
#define SPORT1_Da_I_SPORT3_Da_O_ENCODING            0x1A
#define SPORT1_Db_I_SPORT3_Da_O_ENCODING            0x1A
#define SPORT2_Da_I_SPORT3_Da_O_ENCODING            0x1A
#define SPORT2_Db_I_SPORT3_Da_O_ENCODING            0x1A
#define SPORT3_Da_I_SPORT3_Da_O_ENCODING            0x1A
#define SPORT3_Db_I_SPORT3_Da_O_ENCODING            0x1A
#define SPORT4_Da_I_SPORT3_Da_O_ENCODING            0x1A
#define SPORT4_Db_I_SPORT3_Da_O_ENCODING            0x1A
#define SPORT5_Da_I_SPORT3_Da_O_ENCODING            0x1A
#define SPORT5_Db_I_SPORT3_Da_O_ENCODING            0x1A
#define SRC0_DAT_IP_I_SPORT3_Da_O_ENCODING          0x1A
#define SRC1_DAT_IP_I_SPORT3_Da_O_ENCODING          0x1A
#define SRC2_DAT_IP_I_SPORT3_Da_O_ENCODING          0x1A
#define SRC3_DAT_IP_I_SPORT3_Da_O_ENCODING          0x1A
#define SRC0_TDM_OP_I_SPORT3_Da_O_ENCODING          0x1A
#define SRC1_TDM_OP_I_SPORT3_Da_O_ENCODING          0x1A
#define SRC2_TDM_OP_I_SPORT3_Da_O_ENCODING          0x1A
#define SRC3_TDM_OP_I_SPORT3_Da_O_ENCODING          0x1A
#define DIT_DAT_I_SPORT3_Da_O_ENCODING              0x1A
#define IDP0_DAT_I_SPORT3_Da_O_ENCODING             0x1A
#define IDP1_DAT_I_SPORT3_Da_O_ENCODING             0x1A
#define IDP2_DAT_I_SPORT3_Da_O_ENCODING             0x1A
#define IDP3_DAT_I_SPORT3_Da_O_ENCODING             0x1A
#define IDP4_DAT_I_SPORT3_Da_O_ENCODING             0x1A
#define IDP5_DAT_I_SPORT3_Da_O_ENCODING             0x1A
#define IDP6_DAT_I_SPORT3_Da_O_ENCODING             0x1A
#define IDP7_DAT_I_SPORT3_Da_O_ENCODING             0x1A

#define SPORT0_Da_I_SPORT3_Db_O_ENCODING            0x1B
#define SPORT0_Db_I_SPORT3_Db_O_ENCODING            0x1B
#define SPORT1_Da_I_SPORT3_Db_O_ENCODING            0x1B
#define SPORT1_Db_I_SPORT3_Db_O_ENCODING            0x1B
#define SPORT2_Da_I_SPORT3_Db_O_ENCODING            0x1B
#define SPORT2_Db_I_SPORT3_Db_O_ENCODING            0x1B
#define SPORT3_Da_I_SPORT3_Db_O_ENCODING            0x1B
#define SPORT3_Db_I_SPORT3_Db_O_ENCODING            0x1B
#define SPORT4_Da_I_SPORT3_Db_O_ENCODING            0x1B
#define SPORT4_Db_I_SPORT3_Db_O_ENCODING            0x1B
#define SPORT5_Da_I_SPORT3_Db_O_ENCODING            0x1B
#define SPORT5_Db_I_SPORT3_Db_O_ENCODING            0x1B
#define SRC0_DAT_IP_I_SPORT3_Db_O_ENCODING          0x1B
#define SRC1_DAT_IP_I_SPORT3_Db_O_ENCODING          0x1B
#define SRC2_DAT_IP_I_SPORT3_Db_O_ENCODING          0x1B
#define SRC3_DAT_IP_I_SPORT3_Db_O_ENCODING          0x1B
#define SRC0_TDM_OP_I_SPORT3_Db_O_ENCODING          0x1B
#define SRC1_TDM_OP_I_SPORT3_Db_O_ENCODING          0x1B
#define SRC2_TDM_OP_I_SPORT3_Db_O_ENCODING          0x1B
#define SRC3_TDM_OP_I_SPORT3_Db_O_ENCODING          0x1B
#define DIT_DAT_I_SPORT3_Db_O_ENCODING              0x1B
#define IDP0_DAT_I_SPORT3_Db_O_ENCODING             0x1B
#define IDP1_DAT_I_SPORT3_Db_O_ENCODING             0x1B
#define IDP2_DAT_I_SPORT3_Db_O_ENCODING             0x1B
#define IDP3_DAT_I_SPORT3_Db_O_ENCODING             0x1B
#define IDP4_DAT_I_SPORT3_Db_O_ENCODING             0x1B
#define IDP5_DAT_I_SPORT3_Db_O_ENCODING             0x1B
#define IDP6_DAT_I_SPORT3_Db_O_ENCODING             0x1B
#define IDP7_DAT_I_SPORT3_Db_O_ENCODING             0x1B

#define SPORT0_Da_I_SPORT4_Da_O_ENCODING            0x1C
#define SPORT0_Db_I_SPORT4_Da_O_ENCODING            0x1C
#define SPORT1_Da_I_SPORT4_Da_O_ENCODING            0x1C
#define SPORT1_Db_I_SPORT4_Da_O_ENCODING            0x1C
#define SPORT2_Da_I_SPORT4_Da_O_ENCODING            0x1C
#define SPORT2_Db_I_SPORT4_Da_O_ENCODING            0x1C
#define SPORT3_Da_I_SPORT4_Da_O_ENCODING            0x1C
#define SPORT3_Db_I_SPORT4_Da_O_ENCODING            0x1C
#define SPORT4_Da_I_SPORT4_Da_O_ENCODING            0x1C
#define SPORT4_Db_I_SPORT4_Da_O_ENCODING            0x1C
#define SPORT5_Da_I_SPORT4_Da_O_ENCODING            0x1C
#define SPORT5_Db_I_SPORT4_Da_O_ENCODING            0x1C
#define SRC0_DAT_IP_I_SPORT4_Da_O_ENCODING          0x1C
#define SRC1_DAT_IP_I_SPORT4_Da_O_ENCODING          0x1C
#define SRC2_DAT_IP_I_SPORT4_Da_O_ENCODING          0x1C
#define SRC3_DAT_IP_I_SPORT4_Da_O_ENCODING          0x1C
#define SRC0_TDM_OP_I_SPORT4_Da_O_ENCODING          0x1C
#define SRC1_TDM_OP_I_SPORT4_Da_O_ENCODING          0x1C
#define SRC2_TDM_OP_I_SPORT4_Da_O_ENCODING          0x1C
#define SRC3_TDM_OP_I_SPORT4_Da_O_ENCODING          0x1C
#define DIT_DAT_I_SPORT4_Da_O_ENCODING              0x1C
#define IDP0_DAT_I_SPORT4_Da_O_ENCODING             0x1C
#define IDP1_DAT_I_SPORT4_Da_O_ENCODING             0x1C
#define IDP2_DAT_I_SPORT4_Da_O_ENCODING             0x1C
#define IDP3_DAT_I_SPORT4_Da_O_ENCODING             0x1C
#define IDP4_DAT_I_SPORT4_Da_O_ENCODING             0x1C
#define IDP5_DAT_I_SPORT4_Da_O_ENCODING             0x1C
#define IDP6_DAT_I_SPORT4_Da_O_ENCODING             0x1C
#define IDP7_DAT_I_SPORT4_Da_O_ENCODING             0x1C

#define SPORT0_Da_I_SPORT4_Db_O_ENCODING            0x1D
#define SPORT0_Db_I_SPORT4_Db_O_ENCODING            0x1D
#define SPORT1_Da_I_SPORT4_Db_O_ENCODING            0x1D
#define SPORT1_Db_I_SPORT4_Db_O_ENCODING            0x1D
#define SPORT2_Da_I_SPORT4_Db_O_ENCODING            0x1D
#define SPORT2_Db_I_SPORT4_Db_O_ENCODING            0x1D
#define SPORT3_Da_I_SPORT4_Db_O_ENCODING            0x1D
#define SPORT3_Db_I_SPORT4_Db_O_ENCODING            0x1D
#define SPORT4_Da_I_SPORT4_Db_O_ENCODING            0x1D
#define SPORT4_Db_I_SPORT4_Db_O_ENCODING            0x1D
#define SPORT5_Da_I_SPORT4_Db_O_ENCODING            0x1D
#define SPORT5_Db_I_SPORT4_Db_O_ENCODING            0x1D
#define SRC0_DAT_IP_I_SPORT4_Db_O_ENCODING          0x1D
#define SRC1_DAT_IP_I_SPORT4_Db_O_ENCODING          0x1D
#define SRC2_DAT_IP_I_SPORT4_Db_O_ENCODING          0x1D
#define SRC3_DAT_IP_I_SPORT4_Db_O_ENCODING          0x1D
#define SRC0_TDM_OP_I_SPORT4_Db_O_ENCODING          0x1D
#define SRC1_TDM_OP_I_SPORT4_Db_O_ENCODING          0x1D
#define SRC2_TDM_OP_I_SPORT4_Db_O_ENCODING          0x1D
#define SRC3_TDM_OP_I_SPORT4_Db_O_ENCODING          0x1D
#define DIT_DAT_I_SPORT4_Db_O_ENCODING              0x1D
#define IDP0_DAT_I_SPORT4_Db_O_ENCODING             0x1D
#define IDP1_DAT_I_SPORT4_Db_O_ENCODING             0x1D
#define IDP2_DAT_I_SPORT4_Db_O_ENCODING             0x1D
#define IDP3_DAT_I_SPORT4_Db_O_ENCODING             0x1D
#define IDP4_DAT_I_SPORT4_Db_O_ENCODING             0x1D
#define IDP5_DAT_I_SPORT4_Db_O_ENCODING             0x1D
#define IDP6_DAT_I_SPORT4_Db_O_ENCODING             0x1D
#define IDP7_DAT_I_SPORT4_Db_O_ENCODING             0x1D

#define SPORT0_Da_I_SPORT5_Da_O_ENCODING            0x1E
#define SPORT0_Db_I_SPORT5_Da_O_ENCODING            0x1E
#define SPORT1_Da_I_SPORT5_Da_O_ENCODING            0x1E
#define SPORT1_Db_I_SPORT5_Da_O_ENCODING            0x1E
#define SPORT2_Da_I_SPORT5_Da_O_ENCODING            0x1E
#define SPORT2_Db_I_SPORT5_Da_O_ENCODING            0x1E
#define SPORT3_Da_I_SPORT5_Da_O_ENCODING            0x1E
#define SPORT3_Db_I_SPORT5_Da_O_ENCODING            0x1E
#define SPORT4_Da_I_SPORT5_Da_O_ENCODING            0x1E
#define SPORT4_Db_I_SPORT5_Da_O_ENCODING            0x1E
#define SPORT5_Da_I_SPORT5_Da_O_ENCODING            0x1E
#define SPORT5_Db_I_SPORT5_Da_O_ENCODING            0x1E
#define SRC0_DAT_IP_I_SPORT5_Da_O_ENCODING          0x1E
#define SRC1_DAT_IP_I_SPORT5_Da_O_ENCODING          0x1E
#define SRC2_DAT_IP_I_SPORT5_Da_O_ENCODING          0x1E
#define SRC3_DAT_IP_I_SPORT5_Da_O_ENCODING          0x1E
#define SRC0_TDM_OP_I_SPORT5_Da_O_ENCODING          0x1E
#define SRC1_TDM_OP_I_SPORT5_Da_O_ENCODING          0x1E
#define SRC2_TDM_OP_I_SPORT5_Da_O_ENCODING          0x1E
#define SRC3_TDM_OP_I_SPORT5_Da_O_ENCODING          0x1E
#define DIT_DAT_I_SPORT5_Da_O_ENCODING              0x1E
#define IDP0_DAT_I_SPORT5_Da_O_ENCODING             0x1E
#define IDP1_DAT_I_SPORT5_Da_O_ENCODING             0x1E
#define IDP2_DAT_I_SPORT5_Da_O_ENCODING             0x1E
#define IDP3_DAT_I_SPORT5_Da_O_ENCODING             0x1E
#define IDP4_DAT_I_SPORT5_Da_O_ENCODING             0x1E
#define IDP5_DAT_I_SPORT5_Da_O_ENCODING             0x1E
#define IDP6_DAT_I_SPORT5_Da_O_ENCODING             0x1E
#define IDP7_DAT_I_SPORT5_Da_O_ENCODING             0x1E

#define SPORT0_Da_I_SPORT5_Db_O_ENCODING            0x1F
#define SPORT0_Db_I_SPORT5_Db_O_ENCODING            0x1F
#define SPORT1_Da_I_SPORT5_Db_O_ENCODING            0x1F
#define SPORT1_Db_I_SPORT5_Db_O_ENCODING            0x1F
#define SPORT2_Da_I_SPORT5_Db_O_ENCODING            0x1F
#define SPORT2_Db_I_SPORT5_Db_O_ENCODING            0x1F
#define SPORT3_Da_I_SPORT5_Db_O_ENCODING            0x1F
#define SPORT3_Db_I_SPORT5_Db_O_ENCODING            0x1F
#define SPORT4_Da_I_SPORT5_Db_O_ENCODING            0x1F
#define SPORT4_Db_I_SPORT5_Db_O_ENCODING            0x1F
#define SPORT5_Da_I_SPORT5_Db_O_ENCODING            0x1F
#define SPORT5_Db_I_SPORT5_Db_O_ENCODING            0x1F
#define SRC0_DAT_IP_I_SPORT5_Db_O_ENCODING          0x1F
#define SRC1_DAT_IP_I_SPORT5_Db_O_ENCODING          0x1F
#define SRC2_DAT_IP_I_SPORT5_Db_O_ENCODING          0x1F
#define SRC3_DAT_IP_I_SPORT5_Db_O_ENCODING          0x1F
#define SRC0_TDM_OP_I_SPORT5_Db_O_ENCODING          0x1F
#define SRC1_TDM_OP_I_SPORT5_Db_O_ENCODING          0x1F
#define SRC2_TDM_OP_I_SPORT5_Db_O_ENCODING          0x1F
#define SRC3_TDM_OP_I_SPORT5_Db_O_ENCODING          0x1F
#define DIT_DAT_I_SPORT5_Db_O_ENCODING              0x1F
#define IDP0_DAT_I_SPORT5_Db_O_ENCODING             0x1F
#define IDP1_DAT_I_SPORT5_Db_O_ENCODING             0x1F
#define IDP2_DAT_I_SPORT5_Db_O_ENCODING             0x1F
#define IDP3_DAT_I_SPORT5_Db_O_ENCODING             0x1F
#define IDP4_DAT_I_SPORT5_Db_O_ENCODING             0x1F
#define IDP5_DAT_I_SPORT5_Db_O_ENCODING             0x1F
#define IDP6_DAT_I_SPORT5_Db_O_ENCODING             0x1F
#define IDP7_DAT_I_SPORT5_Db_O_ENCODING             0x1F

#define SPORT0_Da_I_SRC0_DAT_OP_O_ENCODING            0x20
#define SPORT0_Db_I_SRC0_DAT_OP_O_ENCODING            0x20
#define SPORT1_Da_I_SRC0_DAT_OP_O_ENCODING            0x20
#define SPORT1_Db_I_SRC0_DAT_OP_O_ENCODING            0x20
#define SPORT2_Da_I_SRC0_DAT_OP_O_ENCODING            0x20
#define SPORT2_Db_I_SRC0_DAT_OP_O_ENCODING            0x20
#define SPORT3_Da_I_SRC0_DAT_OP_O_ENCODING            0x20
#define SPORT3_Db_I_SRC0_DAT_OP_O_ENCODING            0x20
#define SPORT4_Da_I_SRC0_DAT_OP_O_ENCODING            0x20
#define SPORT4_Db_I_SRC0_DAT_OP_O_ENCODING            0x20
#define SPORT5_Da_I_SRC0_DAT_OP_O_ENCODING            0x20
#define SRC0_DAT_OP_I_SRC0_DAT_OP_O_ENCODING          0x20
#define SRC0_DAT_IP_I_SRC0_DAT_OP_O_ENCODING          0x20
#define SRC1_DAT_IP_I_SRC0_DAT_OP_O_ENCODING          0x20
#define SRC2_DAT_IP_I_SRC0_DAT_OP_O_ENCODING          0x20
#define SRC3_DAT_IP_I_SRC0_DAT_OP_O_ENCODING          0x20
#define SRC0_TDM_OP_I_SRC0_DAT_OP_O_ENCODING          0x20
#define SRC1_TDM_OP_I_SRC0_DAT_OP_O_ENCODING          0x20
#define SRC2_TDM_OP_I_SRC0_DAT_OP_O_ENCODING          0x20
#define SRC3_TDM_OP_I_SRC0_DAT_OP_O_ENCODING          0x20
#define DIT_DAT_I_SRC0_DAT_OP_O_ENCODING              0x20
#define IDP0_DAT_I_SRC0_DAT_OP_O_ENCODING             0x20
#define IDP1_DAT_I_SRC0_DAT_OP_O_ENCODING             0x20
#define IDP2_DAT_I_SRC0_DAT_OP_O_ENCODING             0x20
#define IDP3_DAT_I_SRC0_DAT_OP_O_ENCODING             0x20
#define IDP4_DAT_I_SRC0_DAT_OP_O_ENCODING             0x20
#define IDP5_DAT_I_SRC0_DAT_OP_O_ENCODING             0x20
#define IDP6_DAT_I_SRC0_DAT_OP_O_ENCODING             0x20
#define IDP7_DAT_I_SRC0_DAT_OP_O_ENCODING             0x20

#define SPORT0_Da_I_SRC1_DAT_OP_O_ENCODING            0x21
#define SPORT0_Db_I_SRC1_DAT_OP_O_ENCODING            0x21
#define SPORT1_Da_I_SRC1_DAT_OP_O_ENCODING            0x21
#define SPORT1_Db_I_SRC1_DAT_OP_O_ENCODING            0x21
#define SPORT2_Da_I_SRC1_DAT_OP_O_ENCODING            0x21
#define SPORT2_Db_I_SRC1_DAT_OP_O_ENCODING            0x21
#define SPORT3_Da_I_SRC1_DAT_OP_O_ENCODING            0x21
#define SPORT3_Db_I_SRC1_DAT_OP_O_ENCODING            0x21
#define SPORT4_Da_I_SRC1_DAT_OP_O_ENCODING            0x21
#define SPORT4_Db_I_SRC1_DAT_OP_O_ENCODING            0x21
#define SPORT5_Da_I_SRC1_DAT_OP_O_ENCODING            0x21
#define SRC0_DAT_OP_I_SRC1_DAT_OP_O_ENCODING          0x21
#define SRC0_DAT_IP_I_SRC1_DAT_OP_O_ENCODING          0x21
#define SRC1_DAT_IP_I_SRC1_DAT_OP_O_ENCODING          0x21
#define SRC2_DAT_IP_I_SRC1_DAT_OP_O_ENCODING          0x21
#define SRC3_DAT_IP_I_SRC1_DAT_OP_O_ENCODING          0x21
#define SRC0_TDM_OP_I_SRC1_DAT_OP_O_ENCODING          0x21
#define SRC1_TDM_OP_I_SRC1_DAT_OP_O_ENCODING          0x21
#define SRC2_TDM_OP_I_SRC1_DAT_OP_O_ENCODING          0x21
#define SRC3_TDM_OP_I_SRC1_DAT_OP_O_ENCODING          0x21
#define DIT_DAT_I_SRC1_DAT_OP_O_ENCODING              0x21
#define IDP0_DAT_I_SRC1_DAT_OP_O_ENCODING             0x21
#define IDP1_DAT_I_SRC1_DAT_OP_O_ENCODING             0x21
#define IDP2_DAT_I_SRC1_DAT_OP_O_ENCODING             0x21
#define IDP3_DAT_I_SRC1_DAT_OP_O_ENCODING             0x21
#define IDP4_DAT_I_SRC1_DAT_OP_O_ENCODING             0x21
#define IDP5_DAT_I_SRC1_DAT_OP_O_ENCODING             0x21
#define IDP6_DAT_I_SRC1_DAT_OP_O_ENCODING             0x21
#define IDP7_DAT_I_SRC1_DAT_OP_O_ENCODING             0x21

#define SPORT0_Da_I_SRC2_DAT_OP_O_ENCODING            0x22
#define SPORT0_Db_I_SRC2_DAT_OP_O_ENCODING            0x22
#define SPORT1_Da_I_SRC2_DAT_OP_O_ENCODING            0x22
#define SPORT1_Db_I_SRC2_DAT_OP_O_ENCODING            0x22
#define SPORT2_Da_I_SRC2_DAT_OP_O_ENCODING            0x22
#define SPORT2_Db_I_SRC2_DAT_OP_O_ENCODING            0x22
#define SPORT3_Da_I_SRC2_DAT_OP_O_ENCODING            0x22
#define SPORT3_Db_I_SRC2_DAT_OP_O_ENCODING            0x22
#define SPORT4_Da_I_SRC2_DAT_OP_O_ENCODING            0x22
#define SPORT4_Db_I_SRC2_DAT_OP_O_ENCODING            0x22
#define SPORT5_Da_I_SRC2_DAT_OP_O_ENCODING            0x22
#define SRC0_DAT_OP_I_SRC2_DAT_OP_O_ENCODING          0x22
#define SRC0_DAT_IP_I_SRC2_DAT_OP_O_ENCODING          0x22
#define SRC1_DAT_IP_I_SRC2_DAT_OP_O_ENCODING          0x22
#define SRC2_DAT_IP_I_SRC2_DAT_OP_O_ENCODING          0x22
#define SRC3_DAT_IP_I_SRC2_DAT_OP_O_ENCODING          0x22
#define SRC0_TDM_OP_I_SRC2_DAT_OP_O_ENCODING          0x22
#define SRC1_TDM_OP_I_SRC2_DAT_OP_O_ENCODING          0x22
#define SRC2_TDM_OP_I_SRC2_DAT_OP_O_ENCODING          0x22
#define SRC3_TDM_OP_I_SRC2_DAT_OP_O_ENCODING          0x22
#define DIT_DAT_I_SRC2_DAT_OP_O_ENCODING              0x22
#define IDP0_DAT_I_SRC2_DAT_OP_O_ENCODING             0x22
#define IDP1_DAT_I_SRC2_DAT_OP_O_ENCODING             0x22
#define IDP2_DAT_I_SRC2_DAT_OP_O_ENCODING             0x22
#define IDP3_DAT_I_SRC2_DAT_OP_O_ENCODING             0x22
#define IDP4_DAT_I_SRC2_DAT_OP_O_ENCODING             0x22
#define IDP5_DAT_I_SRC2_DAT_OP_O_ENCODING             0x22
#define IDP6_DAT_I_SRC2_DAT_OP_O_ENCODING             0x22
#define IDP7_DAT_I_SRC2_DAT_OP_O_ENCODING             0x22

#define SPORT0_Da_I_SRC3_DAT_OP_O_ENCODING            0x23
#define SPORT0_Db_I_SRC3_DAT_OP_O_ENCODING            0x23
#define SPORT1_Da_I_SRC3_DAT_OP_O_ENCODING            0x23
#define SPORT1_Db_I_SRC3_DAT_OP_O_ENCODING            0x23
#define SPORT2_Da_I_SRC3_DAT_OP_O_ENCODING            0x23
#define SPORT2_Db_I_SRC3_DAT_OP_O_ENCODING            0x23
#define SPORT3_Da_I_SRC3_DAT_OP_O_ENCODING            0x23
#define SPORT3_Db_I_SRC3_DAT_OP_O_ENCODING            0x23
#define SPORT4_Da_I_SRC3_DAT_OP_O_ENCODING            0x23
#define SPORT4_Db_I_SRC3_DAT_OP_O_ENCODING            0x23
#define SPORT5_Da_I_SRC3_DAT_OP_O_ENCODING            0x23
#define SRC0_DAT_OP_I_SRC3_DAT_OP_O_ENCODING          0x23
#define SRC0_DAT_IP_I_SRC3_DAT_OP_O_ENCODING          0x23
#define SRC1_DAT_IP_I_SRC3_DAT_OP_O_ENCODING          0x23
#define SRC2_DAT_IP_I_SRC3_DAT_OP_O_ENCODING          0x23
#define SRC3_DAT_IP_I_SRC3_DAT_OP_O_ENCODING          0x23
#define SRC0_TDM_OP_I_SRC3_DAT_OP_O_ENCODING          0x23
#define SRC1_TDM_OP_I_SRC3_DAT_OP_O_ENCODING          0x23
#define SRC2_TDM_OP_I_SRC3_DAT_OP_O_ENCODING          0x23
#define SRC3_TDM_OP_I_SRC3_DAT_OP_O_ENCODING          0x23
#define DIT_DAT_I_SRC3_DAT_OP_O_ENCODING              0x23
#define IDP0_DAT_I_SRC3_DAT_OP_O_ENCODING             0x23
#define IDP1_DAT_I_SRC3_DAT_OP_O_ENCODING             0x23
#define IDP2_DAT_I_SRC3_DAT_OP_O_ENCODING             0x23
#define IDP3_DAT_I_SRC3_DAT_OP_O_ENCODING             0x23
#define IDP4_DAT_I_SRC3_DAT_OP_O_ENCODING             0x23
#define IDP5_DAT_I_SRC3_DAT_OP_O_ENCODING             0x23
#define IDP6_DAT_I_SRC3_DAT_OP_O_ENCODING             0x23
#define IDP7_DAT_I_SRC3_DAT_OP_O_ENCODING             0x23

#define SPORT0_Da_I_SRC0_TDM_IP_O_ENCODING            0x24
#define SPORT0_Db_I_SRC0_TDM_IP_O_ENCODING            0x24
#define SPORT1_Da_I_SRC0_TDM_IP_O_ENCODING            0x24
#define SPORT1_Db_I_SRC0_TDM_IP_O_ENCODING            0x24
#define SPORT2_Da_I_SRC0_TDM_IP_O_ENCODING            0x24
#define SPORT2_Db_I_SRC0_TDM_IP_O_ENCODING            0x24
#define SPORT3_Da_I_SRC0_TDM_IP_O_ENCODING            0x24
#define SPORT3_Db_I_SRC0_TDM_IP_O_ENCODING            0x24
#define SPORT4_Da_I_SRC0_TDM_IP_O_ENCODING            0x24
#define SPORT4_Db_I_SRC0_TDM_IP_O_ENCODING            0x24
#define SPORT5_Da_I_SRC0_TDM_IP_O_ENCODING            0x24
#define SRC0_DAT_OP_I_SRC0_TDM_IP_O_ENCODING          0x24
#define SRC0_DAT_IP_I_SRC0_TDM_IP_O_ENCODING          0x24
#define SRC1_DAT_IP_I_SRC0_TDM_IP_O_ENCODING          0x24
#define SRC2_DAT_IP_I_SRC0_TDM_IP_O_ENCODING          0x24
#define SRC3_DAT_IP_I_SRC0_TDM_IP_O_ENCODING          0x24
#define SRC0_TDM_OP_I_SRC0_TDM_IP_O_ENCODING          0x24
#define SRC1_TDM_OP_I_SRC0_TDM_IP_O_ENCODING          0x24
#define SRC2_TDM_OP_I_SRC0_TDM_IP_O_ENCODING          0x24
#define SRC3_TDM_OP_I_SRC0_TDM_IP_O_ENCODING          0x24
#define DIT_DAT_I_SRC0_TDM_IP_O_ENCODING              0x24
#define IDP0_DAT_I_SRC0_TDM_IP_O_ENCODING             0x24
#define IDP1_DAT_I_SRC0_TDM_IP_O_ENCODING             0x24
#define IDP2_DAT_I_SRC0_TDM_IP_O_ENCODING             0x24
#define IDP3_DAT_I_SRC0_TDM_IP_O_ENCODING             0x24
#define IDP4_DAT_I_SRC0_TDM_IP_O_ENCODING             0x24
#define IDP5_DAT_I_SRC0_TDM_IP_O_ENCODING             0x24
#define IDP6_DAT_I_SRC0_TDM_IP_O_ENCODING             0x24
#define IDP7_DAT_I_SRC0_TDM_IP_O_ENCODING             0x24

#define SPORT0_Da_I_SRC1_TDM_IP_O_ENCODING            0x25
#define SPORT0_Db_I_SRC1_TDM_IP_O_ENCODING            0x25
#define SPORT1_Da_I_SRC1_TDM_IP_O_ENCODING            0x25
#define SPORT1_Db_I_SRC1_TDM_IP_O_ENCODING            0x25
#define SPORT2_Da_I_SRC1_TDM_IP_O_ENCODING            0x25
#define SPORT2_Db_I_SRC1_TDM_IP_O_ENCODING            0x25
#define SPORT3_Da_I_SRC1_TDM_IP_O_ENCODING            0x25
#define SPORT3_Db_I_SRC1_TDM_IP_O_ENCODING            0x25
#define SPORT4_Da_I_SRC1_TDM_IP_O_ENCODING            0x25
#define SPORT4_Db_I_SRC1_TDM_IP_O_ENCODING            0x25
#define SPORT5_Da_I_SRC1_TDM_IP_O_ENCODING            0x25
#define SRC0_DAT_OP_I_SRC1_TDM_IP_O_ENCODING          0x25
#define SRC0_DAT_IP_I_SRC1_TDM_IP_O_ENCODING          0x25
#define SRC1_DAT_IP_I_SRC1_TDM_IP_O_ENCODING          0x25
#define SRC2_DAT_IP_I_SRC1_TDM_IP_O_ENCODING          0x25
#define SRC3_DAT_IP_I_SRC1_TDM_IP_O_ENCODING          0x25
#define SRC0_TDM_OP_I_SRC1_TDM_IP_O_ENCODING          0x25
#define SRC1_TDM_OP_I_SRC1_TDM_IP_O_ENCODING          0x25
#define SRC2_TDM_OP_I_SRC1_TDM_IP_O_ENCODING          0x25
#define SRC3_TDM_OP_I_SRC1_TDM_IP_O_ENCODING          0x25
#define DIT_DAT_I_SRC1_TDM_IP_O_ENCODING              0x25
#define IDP0_DAT_I_SRC1_TDM_IP_O_ENCODING             0x25
#define IDP1_DAT_I_SRC1_TDM_IP_O_ENCODING             0x25
#define IDP2_DAT_I_SRC1_TDM_IP_O_ENCODING             0x25
#define IDP3_DAT_I_SRC1_TDM_IP_O_ENCODING             0x25
#define IDP4_DAT_I_SRC1_TDM_IP_O_ENCODING             0x25
#define IDP5_DAT_I_SRC1_TDM_IP_O_ENCODING             0x25
#define IDP6_DAT_I_SRC1_TDM_IP_O_ENCODING             0x25
#define IDP7_DAT_I_SRC1_TDM_IP_O_ENCODING             0x25

#define SPORT0_Da_I_SRC2_TDM_IP_O_ENCODING            0x26
#define SPORT0_Db_I_SRC2_TDM_IP_O_ENCODING            0x26
#define SPORT1_Da_I_SRC2_TDM_IP_O_ENCODING            0x26
#define SPORT1_Db_I_SRC2_TDM_IP_O_ENCODING            0x26
#define SPORT2_Da_I_SRC2_TDM_IP_O_ENCODING            0x26
#define SPORT2_Db_I_SRC2_TDM_IP_O_ENCODING            0x26
#define SPORT3_Da_I_SRC2_TDM_IP_O_ENCODING            0x26
#define SPORT3_Db_I_SRC2_TDM_IP_O_ENCODING            0x26
#define SPORT4_Da_I_SRC2_TDM_IP_O_ENCODING            0x26
#define SPORT4_Db_I_SRC2_TDM_IP_O_ENCODING            0x26
#define SPORT5_Da_I_SRC2_TDM_IP_O_ENCODING            0x26
#define SRC0_DAT_OP_I_SRC2_TDM_IP_O_ENCODING          0x26
#define SRC0_DAT_IP_I_SRC2_TDM_IP_O_ENCODING          0x26
#define SRC1_DAT_IP_I_SRC2_TDM_IP_O_ENCODING          0x26
#define SRC2_DAT_IP_I_SRC2_TDM_IP_O_ENCODING          0x26
#define SRC3_DAT_IP_I_SRC2_TDM_IP_O_ENCODING          0x26
#define SRC0_TDM_OP_I_SRC2_TDM_IP_O_ENCODING          0x26
#define SRC1_TDM_OP_I_SRC2_TDM_IP_O_ENCODING          0x26
#define SRC2_TDM_OP_I_SRC2_TDM_IP_O_ENCODING          0x26
#define SRC3_TDM_OP_I_SRC2_TDM_IP_O_ENCODING          0x26
#define DIT_DAT_I_SRC2_TDM_IP_O_ENCODING              0x26
#define IDP0_DAT_I_SRC2_TDM_IP_O_ENCODING             0x26
#define IDP1_DAT_I_SRC2_TDM_IP_O_ENCODING             0x26
#define IDP2_DAT_I_SRC2_TDM_IP_O_ENCODING             0x26
#define IDP3_DAT_I_SRC2_TDM_IP_O_ENCODING             0x26
#define IDP4_DAT_I_SRC2_TDM_IP_O_ENCODING             0x26
#define IDP5_DAT_I_SRC2_TDM_IP_O_ENCODING             0x26
#define IDP6_DAT_I_SRC2_TDM_IP_O_ENCODING             0x26
#define IDP7_DAT_I_SRC2_TDM_IP_O_ENCODING             0x26

#define SPORT0_Da_I_SRC3_TDM_IP_O_ENCODING            0x27
#define SPORT0_Db_I_SRC3_TDM_IP_O_ENCODING            0x27
#define SPORT1_Da_I_SRC3_TDM_IP_O_ENCODING            0x27
#define SPORT1_Db_I_SRC3_TDM_IP_O_ENCODING            0x27
#define SPORT2_Da_I_SRC3_TDM_IP_O_ENCODING            0x27
#define SPORT2_Db_I_SRC3_TDM_IP_O_ENCODING            0x27
#define SPORT3_Da_I_SRC3_TDM_IP_O_ENCODING            0x27
#define SPORT3_Db_I_SRC3_TDM_IP_O_ENCODING            0x27
#define SPORT4_Da_I_SRC3_TDM_IP_O_ENCODING            0x27
#define SPORT4_Db_I_SRC3_TDM_IP_O_ENCODING            0x27
#define SPORT5_Da_I_SRC3_TDM_IP_O_ENCODING            0x27
#define SRC0_DAT_OP_I_SRC3_TDM_IP_O_ENCODING          0x27
#define SRC0_DAT_IP_I_SRC3_TDM_IP_O_ENCODING          0x27
#define SRC1_DAT_IP_I_SRC3_TDM_IP_O_ENCODING          0x27
#define SRC2_DAT_IP_I_SRC3_TDM_IP_O_ENCODING          0x27
#define SRC3_DAT_IP_I_SRC3_TDM_IP_O_ENCODING          0x27
#define SRC0_TDM_OP_I_SRC3_TDM_IP_O_ENCODING          0x27
#define SRC1_TDM_OP_I_SRC3_TDM_IP_O_ENCODING          0x27
#define SRC2_TDM_OP_I_SRC3_TDM_IP_O_ENCODING          0x27
#define SRC3_TDM_OP_I_SRC3_TDM_IP_O_ENCODING          0x27
#define DIT_DAT_I_SRC3_TDM_IP_O_ENCODING              0x27
#define IDP0_DAT_I_SRC3_TDM_IP_O_ENCODING             0x27
#define IDP1_DAT_I_SRC3_TDM_IP_O_ENCODING             0x27
#define IDP2_DAT_I_SRC3_TDM_IP_O_ENCODING             0x27
#define IDP3_DAT_I_SRC3_TDM_IP_O_ENCODING             0x27
#define IDP4_DAT_I_SRC3_TDM_IP_O_ENCODING             0x27
#define IDP5_DAT_I_SRC3_TDM_IP_O_ENCODING             0x27
#define IDP6_DAT_I_SRC3_TDM_IP_O_ENCODING             0x27
#define IDP7_DAT_I_SRC3_TDM_IP_O_ENCODING             0x27

#define SPORT0_Da_I_DIR_DAT_O_ENCODING            0x28
#define SPORT0_Db_I_DIR_DAT_O_ENCODING            0x28
#define SPORT1_Da_I_DIR_DAT_O_ENCODING            0x28
#define SPORT1_Db_I_DIR_DAT_O_ENCODING            0x28
#define SPORT2_Da_I_DIR_DAT_O_ENCODING            0x28
#define SPORT2_Db_I_DIR_DAT_O_ENCODING            0x28
#define SPORT3_Da_I_DIR_DAT_O_ENCODING            0x28
#define SPORT3_Db_I_DIR_DAT_O_ENCODING            0x28
#define SPORT4_Da_I_DIR_DAT_O_ENCODING            0x28
#define SPORT4_Db_I_DIR_DAT_O_ENCODING            0x28
#define SPORT5_Da_I_DIR_DAT_O_ENCODING            0x28
#define SRC0_DAT_OP_I_DIR_DAT_O_ENCODING          0x28
#define SRC0_DAT_IP_I_DIR_DAT_O_ENCODING          0x28
#define SRC1_DAT_IP_I_DIR_DAT_O_ENCODING          0x28
#define SRC2_DAT_IP_I_DIR_DAT_O_ENCODING          0x28
#define SRC3_DAT_IP_I_DIR_DAT_O_ENCODING          0x28
#define SRC0_TDM_OP_I_DIR_DAT_O_ENCODING          0x28
#define SRC1_TDM_OP_I_DIR_DAT_O_ENCODING          0x28
#define SRC2_TDM_OP_I_DIR_DAT_O_ENCODING          0x28
#define SRC3_TDM_OP_I_DIR_DAT_O_ENCODING          0x28
#define DIT_DAT_I_DIR_DAT_O_ENCODING              0x28
#define IDP0_DAT_I_DIR_DAT_O_ENCODING             0x28
#define IDP1_DAT_I_DIR_DAT_O_ENCODING             0x28
#define IDP2_DAT_I_DIR_DAT_O_ENCODING             0x28
#define IDP3_DAT_I_DIR_DAT_O_ENCODING             0x28
#define IDP4_DAT_I_DIR_DAT_O_ENCODING             0x28
#define IDP5_DAT_I_DIR_DAT_O_ENCODING             0x28
#define IDP6_DAT_I_DIR_DAT_O_ENCODING             0x28
#define IDP7_DAT_I_DIR_DAT_O_ENCODING             0x28
/*
// The low encoding for group B sources is defined 
// as both 0x29 and 0x2A.  Because we cannot define it 
// twice in this header file we've chosen ox2A

#define SPORT0_Da_I_LOW_ENCODING            0x29
#define SPORT0_DB_I_LOW_ENCODING            0x29
#define SPORT1_Da_I_LOW_ENCODING            0x29
#define SPORT1_DB_I_LOW_ENCODING            0x29
#define SPORT2_Da_I_LOW_ENCODING            0x29
#define SPORT2_DB_I_LOW_ENCODING            0x29
#define SPORT3_Da_I_LOW_ENCODING            0x29
#define SPORT3_DB_I_LOW_ENCODING            0x29
#define SPORT4_Da_I_LOW_ENCODING            0x29
#define SPORT4_DB_I_LOW_ENCODING            0x29
#define SPORT5_Da_I_LOW_ENCODING            0x29
#define SRC0_DAT_OP_I_LOW_ENCODING          0x29
#define SRC0_DAT_IP_I_LOW_ENCODING          0x29
#define SRC1_DAT_IP_I_LOW_ENCODING          0x29
#define SRC2_DAT_IP_I_LOW_ENCODING          0x29
#define SRC3_DAT_IP_I_LOW_ENCODING          0x29
#define SRC0_TDM_OP_I_LOW_ENCODING          0x29
#define SRC1_TDM_OP_I_LOW_ENCODING          0x29
#define SRC2_TDM_OP_I_LOW_ENCODING          0x29
#define SRC3_TDM_OP_I_LOW_ENCODING          0x29
#define DIT_DAT_I_LOW_ENCODING              0x29
#define IDP0_DAT_I_LOW_ENCODING             0x29
#define IDP1_DAT_I_LOW_ENCODING             0x29
#define IDP2_DAT_I_LOW_ENCODING             0x29
#define IDP3_DAT_I_LOW_ENCODING             0x29
#define IDP4_DAT_I_LOW_ENCODING             0x29
#define IDP5_DAT_I_LOW_ENCODING             0x29
#define IDP6_DAT_I_LOW_ENCODING             0x29
#define IDP7_DAT_I_LOW_ENCODING             0x29
*/
#define SPORT0_Da_I_LOW_ENCODING            0x2A
#define SPORT0_Db_I_LOW_ENCODING            0x2A
#define SPORT1_Da_I_LOW_ENCODING            0x2A
#define SPORT1_Db_I_LOW_ENCODING            0x2A
#define SPORT2_Da_I_LOW_ENCODING            0x2A
#define SPORT2_Db_I_LOW_ENCODING            0x2A
#define SPORT3_Da_I_LOW_ENCODING            0x2A
#define SPORT3_Db_I_LOW_ENCODING            0x2A
#define SPORT4_Da_I_LOW_ENCODING            0x2A
#define SPORT4_Db_I_LOW_ENCODING            0x2A
#define SPORT5_Da_I_LOW_ENCODING            0x2A
#define SRC0_DAT_OP_I_LOW_ENCODING          0x2A
#define SRC0_DAT_IP_I_LOW_ENCODING          0x2A
#define SRC1_DAT_IP_I_LOW_ENCODING          0x2A
#define SRC2_DAT_IP_I_LOW_ENCODING          0x2A
#define SRC3_DAT_IP_I_LOW_ENCODING          0x2A
#define SRC0_TDM_OP_I_LOW_ENCODING          0x2A
#define SRC1_TDM_OP_I_LOW_ENCODING          0x2A
#define SRC2_TDM_OP_I_LOW_ENCODING          0x2A
#define SRC3_TDM_OP_I_LOW_ENCODING          0x2A
#define DIT_DAT_I_LOW_ENCODING              0x2A
#define IDP0_DAT_I_LOW_ENCODING             0x2A
#define IDP1_DAT_I_LOW_ENCODING             0x2A
#define IDP2_DAT_I_LOW_ENCODING             0x2A
#define IDP3_DAT_I_LOW_ENCODING             0x2A
#define IDP4_DAT_I_LOW_ENCODING             0x2A
#define IDP5_DAT_I_LOW_ENCODING             0x2A
#define IDP6_DAT_I_LOW_ENCODING             0x2A
#define IDP7_DAT_I_LOW_ENCODING             0x2A

#define SPORT0_Da_I_HIGH_ENCODING            0x2B
#define SPORT0_Db_I_HIGH_ENCODING            0x2B
#define SPORT1_Da_I_HIGH_ENCODING            0x2B
#define SPORT1_Db_I_HIGH_ENCODING            0x2B
#define SPORT2_Da_I_HIGH_ENCODING            0x2B
#define SPORT2_Db_I_HIGH_ENCODING            0x2B
#define SPORT3_Da_I_HIGH_ENCODING            0x2B
#define SPORT3_Db_I_HIGH_ENCODING            0x2B
#define SPORT4_Da_I_HIGH_ENCODING            0x2B
#define SPORT4_Db_I_HIGH_ENCODING            0x2B
#define SPORT5_Da_I_HIGH_ENCODING            0x2B
#define SRC0_DAT_OP_I_HIGH_ENCODING          0x2B
#define SRC0_DAT_IP_I_HIGH_ENCODING          0x2B
#define SRC1_DAT_IP_I_HIGH_ENCODING          0x2B
#define SRC2_DAT_IP_I_HIGH_ENCODING          0x2B
#define SRC3_DAT_IP_I_HIGH_ENCODING          0x2B
#define SRC0_TDM_OP_I_HIGH_ENCODING          0x2B
#define SRC1_TDM_OP_I_HIGH_ENCODING          0x2B
#define SRC2_TDM_OP_I_HIGH_ENCODING          0x2B
#define SRC3_TDM_OP_I_HIGH_ENCODING          0x2B
#define DIT_DAT_I_HIGH_ENCODING              0x2B
#define IDP0_DAT_I_HIGH_ENCODING             0x2B
#define IDP1_DAT_I_HIGH_ENCODING             0x2B
#define IDP2_DAT_I_HIGH_ENCODING             0x2B
#define IDP3_DAT_I_HIGH_ENCODING             0x2B
#define IDP4_DAT_I_HIGH_ENCODING             0x2B
#define IDP5_DAT_I_HIGH_ENCODING             0x2B
#define IDP6_DAT_I_HIGH_ENCODING             0x2B
#define IDP7_DAT_I_HIGH_ENCODING             0x2B



//------------------------------------------------------------------------------
//  Group B: Register associations
//------------------------------------------------------------------------------

//SRU_DAT0
#define SPORT0_Da_I_DAI_PB01_O_REG       0x2440    
#define SPORT0_Db_I_DAI_PB01_O_REG       0x2440    
#define SPORT1_Da_I_DAI_PB01_O_REG       0x2440    
#define SPORT1_Db_I_DAI_PB01_O_REG       0x2440    
#define SPORT2_Da_I_DAI_PB01_O_REG       0x2440    
#define SPORT0_Da_I_DAI_PB02_O_REG       0x2440    
#define SPORT0_Db_I_DAI_PB02_O_REG       0x2440    
#define SPORT1_Da_I_DAI_PB02_O_REG       0x2440    
#define SPORT1_Db_I_DAI_PB02_O_REG       0x2440    
#define SPORT2_Da_I_DAI_PB02_O_REG       0x2440    
#define SPORT0_Da_I_DAI_PB03_O_REG       0x2440    
#define SPORT0_Db_I_DAI_PB03_O_REG       0x2440    
#define SPORT1_Da_I_DAI_PB03_O_REG       0x2440    
#define SPORT1_Db_I_DAI_PB03_O_REG       0x2440    
#define SPORT2_Da_I_DAI_PB03_O_REG       0x2440    
#define SPORT0_Da_I_DAI_PB04_O_REG       0x2440    
#define SPORT0_Db_I_DAI_PB04_O_REG       0x2440    
#define SPORT1_Da_I_DAI_PB04_O_REG       0x2440    
#define SPORT1_Db_I_DAI_PB04_O_REG       0x2440    
#define SPORT2_Da_I_DAI_PB04_O_REG       0x2440    
#define SPORT0_Da_I_DAI_PB05_O_REG       0x2440    
#define SPORT0_Db_I_DAI_PB05_O_REG       0x2440    
#define SPORT1_Da_I_DAI_PB05_O_REG       0x2440    
#define SPORT1_Db_I_DAI_PB05_O_REG       0x2440    
#define SPORT2_Da_I_DAI_PB05_O_REG       0x2440    
#define SPORT0_Da_I_DAI_PB06_O_REG       0x2440    
#define SPORT0_Db_I_DAI_PB06_O_REG       0x2440    
#define SPORT1_Da_I_DAI_PB06_O_REG       0x2440    
#define SPORT1_Db_I_DAI_PB06_O_REG       0x2440    
#define SPORT2_Da_I_DAI_PB06_O_REG       0x2440    
#define SPORT0_Da_I_DAI_PB07_O_REG       0x2440    
#define SPORT0_Db_I_DAI_PB07_O_REG       0x2440    
#define SPORT1_Da_I_DAI_PB07_O_REG       0x2440    
#define SPORT1_Db_I_DAI_PB07_O_REG       0x2440    
#define SPORT2_Da_I_DAI_PB07_O_REG       0x2440    
#define SPORT0_Da_I_DAI_PB08_O_REG       0x2440    
#define SPORT0_Db_I_DAI_PB08_O_REG       0x2440    
#define SPORT1_Da_I_DAI_PB08_O_REG       0x2440    
#define SPORT1_Db_I_DAI_PB08_O_REG       0x2440    
#define SPORT2_Da_I_DAI_PB08_O_REG       0x2440    
#define SPORT0_Da_I_DAI_PB09_O_REG       0x2440    
#define SPORT0_Db_I_DAI_PB09_O_REG       0x2440    
#define SPORT1_Da_I_DAI_PB09_O_REG       0x2440    
#define SPORT1_Db_I_DAI_PB09_O_REG       0x2440    
#define SPORT2_Da_I_DAI_PB09_O_REG       0x2440    
#define SPORT0_Da_I_DAI_PB10_O_REG       0x2440    
#define SPORT0_Db_I_DAI_PB10_O_REG       0x2440    
#define SPORT1_Da_I_DAI_PB10_O_REG       0x2440    
#define SPORT1_Db_I_DAI_PB10_O_REG       0x2440    
#define SPORT2_Da_I_DAI_PB10_O_REG       0x2440    
#define SPORT0_Da_I_DAI_PB11_O_REG       0x2440    
#define SPORT0_Db_I_DAI_PB11_O_REG       0x2440    
#define SPORT1_Da_I_DAI_PB11_O_REG       0x2440    
#define SPORT1_Db_I_DAI_PB11_O_REG       0x2440    
#define SPORT2_Da_I_DAI_PB11_O_REG       0x2440    
#define SPORT0_Da_I_DAI_PB12_O_REG       0x2440    
#define SPORT0_Db_I_DAI_PB12_O_REG       0x2440    
#define SPORT1_Da_I_DAI_PB12_O_REG       0x2440    
#define SPORT1_Db_I_DAI_PB12_O_REG       0x2440    
#define SPORT2_Da_I_DAI_PB12_O_REG       0x2440    
#define SPORT0_Da_I_DAI_PB13_O_REG       0x2440    
#define SPORT0_Db_I_DAI_PB13_O_REG       0x2440    
#define SPORT1_Da_I_DAI_PB13_O_REG       0x2440    
#define SPORT1_Db_I_DAI_PB13_O_REG       0x2440    
#define SPORT2_Da_I_DAI_PB13_O_REG       0x2440    
#define SPORT0_Da_I_DAI_PB14_O_REG       0x2440    
#define SPORT0_Db_I_DAI_PB14_O_REG       0x2440    
#define SPORT1_Da_I_DAI_PB14_O_REG       0x2440    
#define SPORT1_Db_I_DAI_PB14_O_REG       0x2440    
#define SPORT2_Da_I_DAI_PB14_O_REG       0x2440    
#define SPORT0_Da_I_DAI_PB15_O_REG       0x2440    
#define SPORT0_Db_I_DAI_PB15_O_REG       0x2440    
#define SPORT1_Da_I_DAI_PB15_O_REG       0x2440    
#define SPORT1_Db_I_DAI_PB15_O_REG       0x2440    
#define SPORT2_Da_I_DAI_PB15_O_REG       0x2440    
#define SPORT0_Da_I_DAI_PB16_O_REG       0x2440    
#define SPORT0_Db_I_DAI_PB16_O_REG       0x2440    
#define SPORT1_Da_I_DAI_PB16_O_REG       0x2440    
#define SPORT1_Db_I_DAI_PB16_O_REG       0x2440    
#define SPORT2_Da_I_DAI_PB16_O_REG       0x2440    
#define SPORT0_Da_I_DAI_PB17_O_REG       0x2440    
#define SPORT0_Db_I_DAI_PB17_O_REG       0x2440    
#define SPORT1_Da_I_DAI_PB17_O_REG       0x2440    
#define SPORT1_Db_I_DAI_PB17_O_REG       0x2440    
#define SPORT2_Da_I_DAI_PB17_O_REG       0x2440    
#define SPORT0_Da_I_DAI_PB18_O_REG       0x2440    
#define SPORT0_Db_I_DAI_PB18_O_REG       0x2440    
#define SPORT1_Da_I_DAI_PB18_O_REG       0x2440    
#define SPORT1_Db_I_DAI_PB18_O_REG       0x2440    
#define SPORT2_Da_I_DAI_PB18_O_REG       0x2440    
#define SPORT0_Da_I_DAI_PB19_O_REG       0x2440    
#define SPORT0_Db_I_DAI_PB19_O_REG       0x2440    
#define SPORT1_Da_I_DAI_PB19_O_REG       0x2440    
#define SPORT1_Db_I_DAI_PB19_O_REG       0x2440    
#define SPORT2_Da_I_DAI_PB19_O_REG       0x2440    
#define SPORT0_Da_I_DAI_PB20_O_REG       0x2440    
#define SPORT0_Db_I_DAI_PB20_O_REG       0x2440    
#define SPORT1_Da_I_DAI_PB20_O_REG       0x2440    
#define SPORT1_Db_I_DAI_PB20_O_REG       0x2440    
#define SPORT2_Da_I_DAI_PB20_O_REG       0x2440    
#define SPORT0_Da_I_SPORT0_Da_O_REG      0x2440    
#define SPORT0_Db_I_SPORT0_Da_O_REG      0x2440    
#define SPORT1_Da_I_SPORT0_Da_O_REG      0x2440    
#define SPORT1_Db_I_SPORT0_Da_O_REG      0x2440    
#define SPORT2_Da_I_SPORT0_Da_O_REG      0x2440    
#define SPORT0_Da_I_SPORT0_Db_O_REG      0x2440    
#define SPORT0_Db_I_SPORT0_Db_O_REG      0x2440    
#define SPORT1_Da_I_SPORT0_Db_O_REG      0x2440    
#define SPORT1_Db_I_SPORT0_Db_O_REG      0x2440    
#define SPORT2_Da_I_SPORT0_Db_O_REG      0x2440    
#define SPORT0_Da_I_SPORT1_Da_O_REG      0x2440    
#define SPORT0_Db_I_SPORT1_Da_O_REG      0x2440    
#define SPORT1_Da_I_SPORT1_Da_O_REG      0x2440    
#define SPORT1_Db_I_SPORT1_Da_O_REG      0x2440    
#define SPORT2_Da_I_SPORT1_Da_O_REG      0x2440    
#define SPORT0_Da_I_SPORT1_Db_O_REG      0x2440    
#define SPORT0_Db_I_SPORT1_Db_O_REG      0x2440    
#define SPORT1_Da_I_SPORT1_Db_O_REG      0x2440    
#define SPORT1_Db_I_SPORT1_Db_O_REG      0x2440    
#define SPORT2_Da_I_SPORT1_Db_O_REG      0x2440    
#define SPORT0_Da_I_SPORT2_Da_O_REG      0x2440    
#define SPORT0_Db_I_SPORT2_Da_O_REG      0x2440    
#define SPORT1_Da_I_SPORT2_Da_O_REG      0x2440    
#define SPORT1_Db_I_SPORT2_Da_O_REG      0x2440    
#define SPORT2_Da_I_SPORT2_Da_O_REG      0x2440    
#define SPORT0_Da_I_SPORT2_Db_O_REG      0x2440    
#define SPORT0_Db_I_SPORT2_Db_O_REG      0x2440    
#define SPORT1_Da_I_SPORT2_Db_O_REG      0x2440    
#define SPORT1_Db_I_SPORT2_Db_O_REG      0x2440    
#define SPORT2_Da_I_SPORT2_Db_O_REG      0x2440    
#define SPORT0_Da_I_SPORT3_Da_O_REG      0x2440    
#define SPORT0_Db_I_SPORT3_Da_O_REG      0x2440    
#define SPORT1_Da_I_SPORT3_Da_O_REG      0x2440    
#define SPORT1_Db_I_SPORT3_Da_O_REG      0x2440    
#define SPORT2_Da_I_SPORT3_Da_O_REG      0x2440    
#define SPORT0_Da_I_SPORT3_Db_O_REG      0x2440    
#define SPORT0_Db_I_SPORT3_Db_O_REG      0x2440    
#define SPORT1_Da_I_SPORT3_Db_O_REG      0x2440    
#define SPORT1_Db_I_SPORT3_Db_O_REG      0x2440    
#define SPORT2_Da_I_SPORT3_Db_O_REG      0x2440    
#define SPORT0_Da_I_SPORT4_Da_O_REG      0x2440    
#define SPORT0_Db_I_SPORT4_Da_O_REG      0x2440    
#define SPORT1_Da_I_SPORT4_Da_O_REG      0x2440    
#define SPORT1_Db_I_SPORT4_Da_O_REG      0x2440    
#define SPORT2_Da_I_SPORT4_Da_O_REG      0x2440    
#define SPORT0_Da_I_SPORT4_Db_O_REG      0x2440    
#define SPORT0_Db_I_SPORT4_Db_O_REG      0x2440    
#define SPORT1_Da_I_SPORT4_Db_O_REG      0x2440    
#define SPORT1_Db_I_SPORT4_Db_O_REG      0x2440    
#define SPORT2_Da_I_SPORT4_Db_O_REG      0x2440    
#define SPORT0_Da_I_SPORT5_Da_O_REG      0x2440    
#define SPORT0_Db_I_SPORT5_Da_O_REG      0x2440    
#define SPORT1_Da_I_SPORT5_Da_O_REG      0x2440    
#define SPORT1_Db_I_SPORT5_Da_O_REG      0x2440    
#define SPORT2_Da_I_SPORT5_Da_O_REG      0x2440    
#define SPORT0_Da_I_SPORT5_Db_O_REG      0x2440    
#define SPORT0_Db_I_SPORT5_Db_O_REG      0x2440    
#define SPORT1_Da_I_SPORT5_Db_O_REG      0x2440    
#define SPORT1_Db_I_SPORT5_Db_O_REG      0x2440    
#define SPORT2_Da_I_SPORT5_Db_O_REG      0x2440
#define SPORT0_Da_I_SRC0_DAT_OP_O_REG    0x2440    
#define SPORT0_Db_I_SRC0_DAT_OP_O_REG    0x2440    
#define SPORT1_Da_I_SRC0_DAT_OP_O_REG    0x2440    
#define SPORT1_Db_I_SRC0_DAT_OP_O_REG    0x2440    
#define SPORT2_Da_I_SRC0_DAT_OP_O_REG    0x2440     
#define SPORT0_Da_I_SRC1_DAT_OP_O_REG    0x2440    
#define SPORT0_Db_I_SRC1_DAT_OP_O_REG    0x2440    
#define SPORT1_Da_I_SRC1_DAT_OP_O_REG    0x2440    
#define SPORT1_Db_I_SRC1_DAT_OP_O_REG    0x2440    
#define SPORT2_Da_I_SRC1_DAT_OP_O_REG    0x2440 
#define SPORT0_Da_I_SRC2_DAT_OP_O_REG    0x2440    
#define SPORT0_Db_I_SRC2_DAT_OP_O_REG    0x2440    
#define SPORT1_Da_I_SRC2_DAT_OP_O_REG    0x2440    
#define SPORT1_Db_I_SRC2_DAT_OP_O_REG    0x2440    
#define SPORT2_Da_I_SRC2_DAT_OP_O_REG    0x2440 
#define SPORT0_Da_I_SRC3_DAT_OP_O_REG    0x2440    
#define SPORT0_Db_I_SRC3_DAT_OP_O_REG    0x2440    
#define SPORT1_Da_I_SRC3_DAT_OP_O_REG    0x2440    
#define SPORT1_Db_I_SRC3_DAT_OP_O_REG    0x2440    
#define SPORT2_Da_I_SRC3_DAT_OP_O_REG    0x2440 
#define SPORT0_Da_I_SRC0_TDM_IP_O_REG    0x2440    
#define SPORT0_Db_I_SRC0_TDM_IP_O_REG    0x2440    
#define SPORT1_Da_I_SRC0_TDM_IP_O_REG    0x2440    
#define SPORT1_Db_I_SRC0_TDM_IP_O_REG    0x2440    
#define SPORT2_Da_I_SRC0_TDM_IP_O_REG    0x2440     
#define SPORT0_Da_I_SRC1_TDM_IP_O_REG    0x2440    
#define SPORT0_Db_I_SRC1_TDM_IP_O_REG    0x2440    
#define SPORT1_Da_I_SRC1_TDM_IP_O_REG    0x2440    
#define SPORT1_Db_I_SRC1_TDM_IP_O_REG    0x2440    
#define SPORT2_Da_I_SRC1_TDM_IP_O_REG    0x2440 
#define SPORT0_Da_I_SRC2_TDM_IP_O_REG    0x2440    
#define SPORT0_Db_I_SRC2_TDM_IP_O_REG    0x2440    
#define SPORT1_Da_I_SRC2_TDM_IP_O_REG    0x2440    
#define SPORT1_Db_I_SRC2_TDM_IP_O_REG    0x2440    
#define SPORT2_Da_I_SRC2_TDM_IP_O_REG    0x2440 
#define SPORT0_Da_I_SRC3_TDM_IP_O_REG    0x2440    
#define SPORT0_Db_I_SRC3_TDM_IP_O_REG    0x2440    
#define SPORT1_Da_I_SRC3_TDM_IP_O_REG    0x2440    
#define SPORT1_Db_I_SRC3_TDM_IP_O_REG    0x2440    
#define SPORT2_Da_I_SRC3_TDM_IP_O_REG    0x2440 
#define SPORT0_Da_I_DIR_DAT_O_REG        0x2440    
#define SPORT0_Db_I_DIR_DAT_O_REG        0x2440    
#define SPORT1_Da_I_DIR_DAT_O_REG        0x2440    
#define SPORT1_Db_I_DIR_DAT_O_REG        0x2440    
#define SPORT2_Da_I_DIR_DAT_O_REG        0x2440
#define SPORT0_Da_I_LOW_REG              0x2440    
#define SPORT0_Db_I_LOW_REG              0x2440    
#define SPORT1_Da_I_LOW_REG              0x2440    
#define SPORT1_Db_I_LOW_REG              0x2440    
#define SPORT2_Da_I_LOW_REG              0x2440
#define SPORT0_Da_I_HIGH_REG             0x2440    
#define SPORT0_Db_I_HIGH_REG             0x2440    
#define SPORT1_Da_I_HIGH_REG             0x2440    
#define SPORT1_Db_I_HIGH_REG             0x2440    
#define SPORT2_Da_I_HIGH_REG             0x2440


//SRU_DAT1
#define SPORT2_Db_I_DAI_PB01_O_REG       0x2441    
#define SPORT3_Da_I_DAI_PB01_O_REG       0x2441    
#define SPORT3_Db_I_DAI_PB01_O_REG       0x2441    
#define SPORT4_Da_I_DAI_PB01_O_REG       0x2441    
#define SPORT4_Db_I_DAI_PB01_O_REG       0x2441    
#define SPORT2_Db_I_DAI_PB02_O_REG       0x2441    
#define SPORT3_Da_I_DAI_PB02_O_REG       0x2441    
#define SPORT3_Db_I_DAI_PB02_O_REG       0x2441    
#define SPORT4_Da_I_DAI_PB02_O_REG       0x2441    
#define SPORT4_Db_I_DAI_PB02_O_REG       0x2441    
#define SPORT2_Db_I_DAI_PB03_O_REG       0x2441    
#define SPORT3_Da_I_DAI_PB03_O_REG       0x2441    
#define SPORT3_Db_I_DAI_PB03_O_REG       0x2441    
#define SPORT4_Da_I_DAI_PB03_O_REG       0x2441    
#define SPORT4_Db_I_DAI_PB03_O_REG       0x2441    
#define SPORT2_Db_I_DAI_PB04_O_REG       0x2441    
#define SPORT3_Da_I_DAI_PB04_O_REG       0x2441    
#define SPORT3_Db_I_DAI_PB04_O_REG       0x2441    
#define SPORT4_Da_I_DAI_PB04_O_REG       0x2441    
#define SPORT4_Db_I_DAI_PB04_O_REG       0x2441    
#define SPORT2_Db_I_DAI_PB05_O_REG       0x2441    
#define SPORT3_Da_I_DAI_PB05_O_REG       0x2441    
#define SPORT3_Db_I_DAI_PB05_O_REG       0x2441    
#define SPORT4_Da_I_DAI_PB05_O_REG       0x2441    
#define SPORT4_Db_I_DAI_PB05_O_REG       0x2441    
#define SPORT2_Db_I_DAI_PB06_O_REG       0x2441    
#define SPORT3_Da_I_DAI_PB06_O_REG       0x2441    
#define SPORT3_Db_I_DAI_PB06_O_REG       0x2441    
#define SPORT4_Da_I_DAI_PB06_O_REG       0x2441    
#define SPORT4_Db_I_DAI_PB06_O_REG       0x2441    
#define SPORT2_Db_I_DAI_PB07_O_REG       0x2441    
#define SPORT3_Da_I_DAI_PB07_O_REG       0x2441    
#define SPORT3_Db_I_DAI_PB07_O_REG       0x2441    
#define SPORT4_Da_I_DAI_PB07_O_REG       0x2441    
#define SPORT4_Db_I_DAI_PB07_O_REG       0x2441    
#define SPORT2_Db_I_DAI_PB08_O_REG       0x2441    
#define SPORT3_Da_I_DAI_PB08_O_REG       0x2441    
#define SPORT3_Db_I_DAI_PB08_O_REG       0x2441    
#define SPORT4_Da_I_DAI_PB08_O_REG       0x2441    
#define SPORT4_Db_I_DAI_PB08_O_REG       0x2441    
#define SPORT2_Db_I_DAI_PB09_O_REG       0x2441    
#define SPORT3_Da_I_DAI_PB09_O_REG       0x2441    
#define SPORT3_Db_I_DAI_PB09_O_REG       0x2441    
#define SPORT4_Da_I_DAI_PB09_O_REG       0x2441    
#define SPORT4_Db_I_DAI_PB09_O_REG       0x2441    
#define SPORT2_Db_I_DAI_PB10_O_REG       0x2441    
#define SPORT3_Da_I_DAI_PB10_O_REG       0x2441    
#define SPORT3_Db_I_DAI_PB10_O_REG       0x2441    
#define SPORT4_Da_I_DAI_PB10_O_REG       0x2441    
#define SPORT4_Db_I_DAI_PB10_O_REG       0x2441    
#define SPORT2_Db_I_DAI_PB11_O_REG       0x2441    
#define SPORT3_Da_I_DAI_PB11_O_REG       0x2441    
#define SPORT3_Db_I_DAI_PB11_O_REG       0x2441    
#define SPORT4_Da_I_DAI_PB11_O_REG       0x2441    
#define SPORT4_Db_I_DAI_PB11_O_REG       0x2441    
#define SPORT2_Db_I_DAI_PB12_O_REG       0x2441    
#define SPORT3_Da_I_DAI_PB12_O_REG       0x2441    
#define SPORT3_Db_I_DAI_PB12_O_REG       0x2441    
#define SPORT4_Da_I_DAI_PB12_O_REG       0x2441    
#define SPORT4_Db_I_DAI_PB12_O_REG       0x2441    
#define SPORT2_Db_I_DAI_PB13_O_REG       0x2441    
#define SPORT3_Da_I_DAI_PB13_O_REG       0x2441    
#define SPORT3_Db_I_DAI_PB13_O_REG       0x2441    
#define SPORT4_Da_I_DAI_PB13_O_REG       0x2441    
#define SPORT4_Db_I_DAI_PB13_O_REG       0x2441    
#define SPORT2_Db_I_DAI_PB14_O_REG       0x2441    
#define SPORT3_Da_I_DAI_PB14_O_REG       0x2441    
#define SPORT3_Db_I_DAI_PB14_O_REG       0x2441    
#define SPORT4_Da_I_DAI_PB14_O_REG       0x2441    
#define SPORT4_Db_I_DAI_PB14_O_REG       0x2441    
#define SPORT2_Db_I_DAI_PB15_O_REG       0x2441    
#define SPORT3_Da_I_DAI_PB15_O_REG       0x2441    
#define SPORT3_Db_I_DAI_PB15_O_REG       0x2441    
#define SPORT4_Da_I_DAI_PB15_O_REG       0x2441    
#define SPORT4_Db_I_DAI_PB15_O_REG       0x2441    
#define SPORT2_Db_I_DAI_PB16_O_REG       0x2441    
#define SPORT3_Da_I_DAI_PB16_O_REG       0x2441    
#define SPORT3_Db_I_DAI_PB16_O_REG       0x2441    
#define SPORT4_Da_I_DAI_PB16_O_REG       0x2441    
#define SPORT4_Db_I_DAI_PB16_O_REG       0x2441    
#define SPORT2_Db_I_DAI_PB17_O_REG       0x2441    
#define SPORT3_Da_I_DAI_PB17_O_REG       0x2441    
#define SPORT3_Db_I_DAI_PB17_O_REG       0x2441    
#define SPORT4_Da_I_DAI_PB17_O_REG       0x2441    
#define SPORT4_Db_I_DAI_PB17_O_REG       0x2441    
#define SPORT2_Db_I_DAI_PB18_O_REG       0x2441    
#define SPORT3_Da_I_DAI_PB18_O_REG       0x2441    
#define SPORT3_Db_I_DAI_PB18_O_REG       0x2441    
#define SPORT4_Da_I_DAI_PB18_O_REG       0x2441    
#define SPORT4_Db_I_DAI_PB18_O_REG       0x2441    
#define SPORT2_Db_I_DAI_PB19_O_REG       0x2441    
#define SPORT3_Da_I_DAI_PB19_O_REG       0x2441    
#define SPORT3_Db_I_DAI_PB19_O_REG       0x2441    
#define SPORT4_Da_I_DAI_PB19_O_REG       0x2441    
#define SPORT4_Db_I_DAI_PB19_O_REG       0x2441    
#define SPORT2_Db_I_DAI_PB20_O_REG       0x2441    
#define SPORT3_Da_I_DAI_PB20_O_REG       0x2441    
#define SPORT3_Db_I_DAI_PB20_O_REG       0x2441    
#define SPORT4_Da_I_DAI_PB20_O_REG       0x2441    
#define SPORT4_Db_I_DAI_PB20_O_REG       0x2441    
#define SPORT2_Db_I_SPORT0_Da_O_REG      0x2441    
#define SPORT3_Da_I_SPORT0_Da_O_REG      0x2441    
#define SPORT3_Db_I_SPORT0_Da_O_REG      0x2441    
#define SPORT4_Da_I_SPORT0_Da_O_REG      0x2441    
#define SPORT4_Db_I_SPORT0_Da_O_REG      0x2441    
#define SPORT2_Db_I_SPORT0_Db_O_REG      0x2441    
#define SPORT3_Da_I_SPORT0_Db_O_REG      0x2441    
#define SPORT3_Db_I_SPORT0_Db_O_REG      0x2441    
#define SPORT4_Da_I_SPORT0_Db_O_REG      0x2441    
#define SPORT4_Db_I_SPORT0_Db_O_REG      0x2441    
#define SPORT2_Db_I_SPORT1_Da_O_REG      0x2441    
#define SPORT3_Da_I_SPORT1_Da_O_REG      0x2441    
#define SPORT3_Db_I_SPORT1_Da_O_REG      0x2441    
#define SPORT4_Da_I_SPORT1_Da_O_REG      0x2441    
#define SPORT4_Db_I_SPORT1_Da_O_REG      0x2441    
#define SPORT2_Db_I_SPORT1_Db_O_REG      0x2441    
#define SPORT3_Da_I_SPORT1_Db_O_REG      0x2441    
#define SPORT3_Db_I_SPORT1_Db_O_REG      0x2441    
#define SPORT4_Da_I_SPORT1_Db_O_REG      0x2441    
#define SPORT4_Db_I_SPORT1_Db_O_REG      0x2441    
#define SPORT2_Db_I_SPORT2_Da_O_REG      0x2441    
#define SPORT3_Da_I_SPORT2_Da_O_REG      0x2441    
#define SPORT3_Db_I_SPORT2_Da_O_REG      0x2441    
#define SPORT4_Da_I_SPORT2_Da_O_REG      0x2441    
#define SPORT4_Db_I_SPORT2_Da_O_REG      0x2441    
#define SPORT2_Db_I_SPORT2_Db_O_REG      0x2441    
#define SPORT3_Da_I_SPORT2_Db_O_REG      0x2441    
#define SPORT3_Db_I_SPORT2_Db_O_REG      0x2441    
#define SPORT4_Da_I_SPORT2_Db_O_REG      0x2441    
#define SPORT4_Db_I_SPORT2_Db_O_REG      0x2441    
#define SPORT2_Db_I_SPORT3_Da_O_REG      0x2441    
#define SPORT3_Da_I_SPORT3_Da_O_REG      0x2441    
#define SPORT3_Db_I_SPORT3_Da_O_REG      0x2441    
#define SPORT4_Da_I_SPORT3_Da_O_REG      0x2441    
#define SPORT4_Db_I_SPORT3_Da_O_REG      0x2441    
#define SPORT2_Db_I_SPORT3_Db_O_REG      0x2441    
#define SPORT3_Da_I_SPORT3_Db_O_REG      0x2441    
#define SPORT3_Db_I_SPORT3_Db_O_REG      0x2441    
#define SPORT4_Da_I_SPORT3_Db_O_REG      0x2441    
#define SPORT4_Db_I_SPORT3_Db_O_REG      0x2441    
#define SPORT2_Db_I_SPORT4_Da_O_REG      0x2441    
#define SPORT3_Da_I_SPORT4_Da_O_REG      0x2441    
#define SPORT3_Db_I_SPORT4_Da_O_REG      0x2441    
#define SPORT4_Da_I_SPORT4_Da_O_REG      0x2441    
#define SPORT4_Db_I_SPORT4_Da_O_REG      0x2441    
#define SPORT2_Db_I_SPORT4_Db_O_REG      0x2441    
#define SPORT3_Da_I_SPORT4_Db_O_REG      0x2441    
#define SPORT3_Db_I_SPORT4_Db_O_REG      0x2441    
#define SPORT4_Da_I_SPORT4_Db_O_REG      0x2441    
#define SPORT4_Db_I_SPORT4_Db_O_REG      0x2441    
#define SPORT2_Db_I_SPORT5_Da_O_REG      0x2441    
#define SPORT3_Da_I_SPORT5_Da_O_REG      0x2441    
#define SPORT3_Db_I_SPORT5_Da_O_REG      0x2441    
#define SPORT4_Da_I_SPORT5_Da_O_REG      0x2441    
#define SPORT4_Db_I_SPORT5_Da_O_REG      0x2441    
#define SPORT2_Db_I_SPORT5_Db_O_REG      0x2441    
#define SPORT3_Da_I_SPORT5_Db_O_REG      0x2441    
#define SPORT3_Db_I_SPORT5_Db_O_REG      0x2441    
#define SPORT4_Da_I_SPORT5_Db_O_REG      0x2441    
#define SPORT4_Db_I_SPORT5_Db_O_REG      0x2441 
#define SPORT2_Db_I_SRC0_DAT_OP_O_REG    0x2441    
#define SPORT3_Da_I_SRC0_DAT_OP_O_REG    0x2441    
#define SPORT3_Db_I_SRC0_DAT_OP_O_REG    0x2441    
#define SPORT4_Da_I_SRC0_DAT_OP_O_REG    0x2441    
#define SPORT4_Db_I_SRC0_DAT_OP_O_REG    0x2441     
#define SPORT2_Db_I_SRC1_DAT_OP_O_REG    0x2441    
#define SPORT3_Da_I_SRC1_DAT_OP_O_REG    0x2441    
#define SPORT3_Db_I_SRC1_DAT_OP_O_REG    0x2441    
#define SPORT4_Da_I_SRC1_DAT_OP_O_REG    0x2441    
#define SPORT4_Db_I_SRC1_DAT_OP_O_REG    0x2441 
#define SPORT2_Db_I_SRC2_DAT_OP_O_REG    0x2441    
#define SPORT3_Da_I_SRC2_DAT_OP_O_REG    0x2441    
#define SPORT3_Db_I_SRC2_DAT_OP_O_REG    0x2441    
#define SPORT4_Da_I_SRC2_DAT_OP_O_REG    0x2441    
#define SPORT4_Db_I_SRC2_DAT_OP_O_REG    0x2441 
#define SPORT2_Db_I_SRC3_DAT_OP_O_REG    0x2441    
#define SPORT3_Da_I_SRC3_DAT_OP_O_REG    0x2441    
#define SPORT3_Db_I_SRC3_DAT_OP_O_REG    0x2441    
#define SPORT4_Da_I_SRC3_DAT_OP_O_REG    0x2441    
#define SPORT4_Db_I_SRC3_DAT_OP_O_REG    0x2441 
#define SPORT2_Db_I_SRC0_TDM_IP_O_REG    0x2441    
#define SPORT3_Da_I_SRC0_TDM_IP_O_REG    0x2441    
#define SPORT3_Db_I_SRC0_TDM_IP_O_REG    0x2441    
#define SPORT4_Da_I_SRC0_TDM_IP_O_REG    0x2441    
#define SPORT4_Db_I_SRC0_TDM_IP_O_REG    0x2441     
#define SPORT2_Db_I_SRC1_TDM_IP_O_REG    0x2441    
#define SPORT3_Da_I_SRC1_TDM_IP_O_REG    0x2441    
#define SPORT3_Db_I_SRC1_TDM_IP_O_REG    0x2441    
#define SPORT4_Da_I_SRC1_TDM_IP_O_REG    0x2441    
#define SPORT4_Db_I_SRC1_TDM_IP_O_REG    0x2441 
#define SPORT2_Db_I_SRC2_TDM_IP_O_REG    0x2441    
#define SPORT3_Da_I_SRC2_TDM_IP_O_REG    0x2441    
#define SPORT3_Db_I_SRC2_TDM_IP_O_REG    0x2441    
#define SPORT4_Da_I_SRC2_TDM_IP_O_REG    0x2441    
#define SPORT4_Db_I_SRC2_TDM_IP_O_REG    0x2441 
#define SPORT2_Db_I_SRC3_TDM_IP_O_REG    0x2441    
#define SPORT3_Da_I_SRC3_TDM_IP_O_REG    0x2441    
#define SPORT3_Db_I_SRC3_TDM_IP_O_REG    0x2441    
#define SPORT4_Da_I_SRC3_TDM_IP_O_REG    0x2441    
#define SPORT4_Db_I_SRC3_TDM_IP_O_REG    0x2441 
#define SPORT2_Db_I_DIR_DAT_O_REG        0x2441    
#define SPORT3_Da_I_DIR_DAT_O_REG        0x2441    
#define SPORT3_Db_I_DIR_DAT_O_REG        0x2441    
#define SPORT4_Da_I_DIR_DAT_O_REG        0x2441    
#define SPORT4_Db_I_DIR_DAT_O_REG        0x2441
#define SPORT2_Db_I_LOW_REG              0x2441    
#define SPORT3_Da_I_LOW_REG              0x2441    
#define SPORT3_Db_I_LOW_REG              0x2441    
#define SPORT4_Da_I_LOW_REG              0x2441    
#define SPORT4_Db_I_LOW_REG              0x2441
#define SPORT2_Db_I_HIGH_REG             0x2441    
#define SPORT3_Da_I_HIGH_REG             0x2441    
#define SPORT3_Db_I_HIGH_REG             0x2441    
#define SPORT4_Da_I_HIGH_REG             0x2441    
#define SPORT4_Db_I_HIGH_REG             0x2441   

//SRU_DAT2
#define SPORT5_Da_I_DAI_PB01_O_REG       0x2442    
#define SPORT5_Db_I_DAI_PB01_O_REG       0x2442    
#define SRC0_DAT_IP_I_DAI_PB01_O_REG     0x2442  
#define SRC1_DAT_IP_I_DAI_PB01_O_REG     0x2442  
#define SRC2_DAT_IP_I_DAI_PB01_O_REG     0x2442  
#define SPORT5_Da_I_DAI_PB02_O_REG       0x2442    
#define SPORT5_Db_I_DAI_PB02_O_REG       0x2442    
#define SRC0_DAT_IP_I_DAI_PB02_O_REG     0x2442  
#define SRC1_DAT_IP_I_DAI_PB02_O_REG     0x2442  
#define SRC2_DAT_IP_I_DAI_PB02_O_REG     0x2442 
#define SPORT5_Da_I_DAI_PB03_O_REG       0x2442    
#define SPORT5_Db_I_DAI_PB03_O_REG       0x2442 
#define SRC0_DAT_IP_I_DAI_PB03_O_REG     0x2442  
#define SRC1_DAT_IP_I_DAI_PB03_O_REG     0x2442  
#define SRC2_DAT_IP_I_DAI_PB03_O_REG     0x2442    
#define SPORT5_Da_I_DAI_PB04_O_REG       0x2442    
#define SPORT5_Db_I_DAI_PB04_O_REG       0x2442 
#define SRC0_DAT_IP_I_DAI_PB04_O_REG     0x2442  
#define SRC1_DAT_IP_I_DAI_PB04_O_REG     0x2442  
#define SRC2_DAT_IP_I_DAI_PB04_O_REG     0x2442    
#define SPORT5_Da_I_DAI_PB05_O_REG       0x2442    
#define SPORT5_Db_I_DAI_PB05_O_REG       0x2442 
#define SRC0_DAT_IP_I_DAI_PB05_O_REG     0x2442  
#define SRC1_DAT_IP_I_DAI_PB05_O_REG     0x2442  
#define SRC2_DAT_IP_I_DAI_PB05_O_REG     0x2442    
#define SPORT5_Da_I_DAI_PB06_O_REG       0x2442    
#define SPORT5_Db_I_DAI_PB06_O_REG       0x2442 
#define SRC0_DAT_IP_I_DAI_PB06_O_REG     0x2442  
#define SRC1_DAT_IP_I_DAI_PB06_O_REG     0x2442  
#define SRC2_DAT_IP_I_DAI_PB06_O_REG     0x2442    
#define SPORT5_Da_I_DAI_PB07_O_REG       0x2442    
#define SPORT5_Db_I_DAI_PB07_O_REG       0x2442 
#define SRC0_DAT_IP_I_DAI_PB07_O_REG     0x2442  
#define SRC1_DAT_IP_I_DAI_PB07_O_REG     0x2442  
#define SRC2_DAT_IP_I_DAI_PB07_O_REG     0x2442    
#define SPORT5_Da_I_DAI_PB08_O_REG       0x2442    
#define SPORT5_Db_I_DAI_PB08_O_REG       0x2442 
#define SRC0_DAT_IP_I_DAI_PB08_O_REG     0x2442  
#define SRC1_DAT_IP_I_DAI_PB08_O_REG     0x2442  
#define SRC2_DAT_IP_I_DAI_PB08_O_REG     0x2442    
#define SPORT5_Da_I_DAI_PB09_O_REG       0x2442    
#define SPORT5_Db_I_DAI_PB09_O_REG       0x2442 
#define SRC0_DAT_IP_I_DAI_PB09_O_REG     0x2442  
#define SRC1_DAT_IP_I_DAI_PB09_O_REG     0x2442  
#define SRC2_DAT_IP_I_DAI_PB09_O_REG     0x2442    
#define SPORT5_Da_I_DAI_PB10_O_REG       0x2442    
#define SPORT5_Db_I_DAI_PB10_O_REG       0x2442 
#define SRC0_DAT_IP_I_DAI_PB10_O_REG     0x2442  
#define SRC1_DAT_IP_I_DAI_PB10_O_REG     0x2442  
#define SRC2_DAT_IP_I_DAI_PB10_O_REG     0x2442    
#define SPORT5_Da_I_DAI_PB11_O_REG       0x2442    
#define SPORT5_Db_I_DAI_PB11_O_REG       0x2442 
#define SRC0_DAT_IP_I_DAI_PB11_O_REG     0x2442  
#define SRC1_DAT_IP_I_DAI_PB11_O_REG     0x2442  
#define SRC2_DAT_IP_I_DAI_PB11_O_REG     0x2442    
#define SPORT5_Da_I_DAI_PB12_O_REG       0x2442    
#define SPORT5_Db_I_DAI_PB12_O_REG       0x2442 
#define SRC0_DAT_IP_I_DAI_PB12_O_REG     0x2442  
#define SRC1_DAT_IP_I_DAI_PB12_O_REG     0x2442  
#define SRC2_DAT_IP_I_DAI_PB12_O_REG     0x2442    
#define SPORT5_Da_I_DAI_PB13_O_REG       0x2442    
#define SPORT5_Db_I_DAI_PB13_O_REG       0x2442 
#define SRC0_DAT_IP_I_DAI_PB13_O_REG     0x2442  
#define SRC1_DAT_IP_I_DAI_PB13_O_REG     0x2442  
#define SRC2_DAT_IP_I_DAI_PB13_O_REG     0x2442    
#define SPORT5_Da_I_DAI_PB14_O_REG       0x2442    
#define SPORT5_Db_I_DAI_PB14_O_REG       0x2442 
#define SRC0_DAT_IP_I_DAI_PB14_O_REG     0x2442  
#define SRC1_DAT_IP_I_DAI_PB14_O_REG     0x2442  
#define SRC2_DAT_IP_I_DAI_PB14_O_REG     0x2442    
#define SPORT5_Da_I_DAI_PB15_O_REG       0x2442    
#define SPORT5_Db_I_DAI_PB15_O_REG       0x2442 
#define SRC0_DAT_IP_I_DAI_PB15_O_REG     0x2442  
#define SRC1_DAT_IP_I_DAI_PB15_O_REG     0x2442  
#define SRC2_DAT_IP_I_DAI_PB15_O_REG     0x2442    
#define SPORT5_Da_I_DAI_PB16_O_REG       0x2442    
#define SPORT5_Db_I_DAI_PB16_O_REG       0x2442 
#define SRC0_DAT_IP_I_DAI_PB16_O_REG     0x2442  
#define SRC1_DAT_IP_I_DAI_PB16_O_REG     0x2442  
#define SRC2_DAT_IP_I_DAI_PB16_O_REG     0x2442    
#define SPORT5_Da_I_DAI_PB17_O_REG       0x2442    
#define SPORT5_Db_I_DAI_PB17_O_REG       0x2442 
#define SRC0_DAT_IP_I_DAI_PB17_O_REG     0x2442  
#define SRC1_DAT_IP_I_DAI_PB17_O_REG     0x2442  
#define SRC2_DAT_IP_I_DAI_PB17_O_REG     0x2442    
#define SPORT5_Da_I_DAI_PB18_O_REG       0x2442    
#define SPORT5_Db_I_DAI_PB18_O_REG       0x2442 
#define SRC0_DAT_IP_I_DAI_PB18_O_REG     0x2442  
#define SRC1_DAT_IP_I_DAI_PB18_O_REG     0x2442  
#define SRC2_DAT_IP_I_DAI_PB18_O_REG     0x2442    
#define SPORT5_Da_I_DAI_PB19_O_REG       0x2442    
#define SPORT5_Db_I_DAI_PB19_O_REG       0x2442 
#define SRC0_DAT_IP_I_DAI_PB19_O_REG     0x2442  
#define SRC1_DAT_IP_I_DAI_PB19_O_REG     0x2442  
#define SRC2_DAT_IP_I_DAI_PB19_O_REG     0x2442    
#define SPORT5_Da_I_DAI_PB20_O_REG       0x2442    
#define SPORT5_Db_I_DAI_PB20_O_REG       0x2442 
#define SRC0_DAT_IP_I_DAI_PB20_O_REG     0x2442  
#define SRC1_DAT_IP_I_DAI_PB20_O_REG     0x2442  
#define SRC2_DAT_IP_I_DAI_PB20_O_REG     0x2442    
#define SPORT5_Da_I_SPORT0_Da_O_REG      0x2442    
#define SPORT5_Db_I_SPORT0_Da_O_REG      0x2442    
#define SRC0_DAT_IP_I_SPORT0_Da_O_REG    0x2442  
#define SRC1_DAT_IP_I_SPORT0_Da_O_REG    0x2442  
#define SRC2_DAT_IP_I_SPORT0_Da_O_REG    0x2442    
#define SPORT5_Da_I_SPORT0_Db_O_REG      0x2442    
#define SPORT5_Db_I_SPORT0_Db_O_REG      0x2442    
#define SRC0_DAT_IP_I_SPORT0_Db_O_REG    0x2442  
#define SRC1_DAT_IP_I_SPORT0_Db_O_REG    0x2442  
#define SRC2_DAT_IP_I_SPORT0_Db_O_REG    0x2442 
#define SPORT5_Da_I_SPORT1_Da_O_REG      0x2442    
#define SPORT5_Db_I_SPORT1_Da_O_REG      0x2442    
#define SRC0_DAT_IP_I_SPORT1_Da_O_REG    0x2442  
#define SRC1_DAT_IP_I_SPORT1_Da_O_REG    0x2442  
#define SRC2_DAT_IP_I_SPORT1_Da_O_REG    0x2442 
#define SPORT5_Da_I_SPORT1_Db_O_REG      0x2442    
#define SPORT5_Db_I_SPORT1_Db_O_REG      0x2442   
#define SRC0_DAT_IP_I_SPORT1_Db_O_REG    0x2442  
#define SRC1_DAT_IP_I_SPORT1_Db_O_REG    0x2442  
#define SRC2_DAT_IP_I_SPORT1_Db_O_REG    0x2442  
#define SPORT5_Da_I_SPORT2_Da_O_REG      0x2442    
#define SPORT5_Db_I_SPORT2_Da_O_REG      0x2442 
#define SRC0_DAT_IP_I_SPORT2_Da_O_REG    0x2442  
#define SRC1_DAT_IP_I_SPORT2_Da_O_REG    0x2442  
#define SRC2_DAT_IP_I_SPORT2_Da_O_REG    0x2442  
#define SPORT5_Da_I_SPORT2_Db_O_REG      0x2442    
#define SPORT5_Db_I_SPORT2_Db_O_REG      0x2442 
#define SRC0_DAT_IP_I_SPORT2_Db_O_REG    0x2442  
#define SRC1_DAT_IP_I_SPORT2_Db_O_REG    0x2442  
#define SRC2_DAT_IP_I_SPORT2_Db_O_REG    0x2442    
#define SPORT5_Da_I_SPORT3_Da_O_REG      0x2442    
#define SPORT5_Db_I_SPORT3_Da_O_REG      0x2442 
#define SRC0_DAT_IP_I_SPORT3_Da_O_REG    0x2442  
#define SRC1_DAT_IP_I_SPORT3_Da_O_REG    0x2442  
#define SRC2_DAT_IP_I_SPORT3_Da_O_REG    0x2442    
#define SPORT5_Da_I_SPORT3_Db_O_REG      0x2442    
#define SPORT5_Db_I_SPORT3_Db_O_REG      0x2442 
#define SRC0_DAT_IP_I_SPORT3_Db_O_REG    0x2442  
#define SRC1_DAT_IP_I_SPORT3_Db_O_REG    0x2442  
#define SRC2_DAT_IP_I_SPORT3_Db_O_REG    0x2442    
#define SPORT5_Da_I_SPORT4_Da_O_REG      0x2442    
#define SPORT5_Db_I_SPORT4_Da_O_REG      0x2442 
#define SRC0_DAT_IP_I_SPORT4_Da_O_REG    0x2442  
#define SRC1_DAT_IP_I_SPORT4_Da_O_REG    0x2442  
#define SRC2_DAT_IP_I_SPORT4_Da_O_REG    0x2442    
#define SPORT5_Da_I_SPORT4_Db_O_REG      0x2442    
#define SPORT5_Db_I_SPORT4_Db_O_REG      0x2442 
#define SRC0_DAT_IP_I_SPORT4_Db_O_REG    0x2442  
#define SRC1_DAT_IP_I_SPORT4_Db_O_REG    0x2442  
#define SRC2_DAT_IP_I_SPORT4_Db_O_REG    0x2442    
#define SPORT5_Da_I_SPORT5_Da_O_REG      0x2442    
#define SPORT5_Db_I_SPORT5_Da_O_REG      0x2442 
#define SRC0_DAT_IP_I_SPORT5_Da_O_REG    0x2442  
#define SRC1_DAT_IP_I_SPORT5_Da_O_REG    0x2442  
#define SRC2_DAT_IP_I_SPORT5_Da_O_REG    0x2442    
#define SPORT5_Da_I_SPORT5_Db_O_REG      0x2442    
#define SPORT5_Db_I_SPORT5_Db_O_REG      0x2442 
#define SRC0_DAT_IP_I_SPORT5_Db_O_REG    0x2442  
#define SRC1_DAT_IP_I_SPORT5_Db_O_REG    0x2442  
#define SRC2_DAT_IP_I_SPORT5_Db_O_REG    0x2442 
#define SPORT5_Da_I_SRC0_DAT_OP_O_REG    0x2442
#define SPORT5_Db_I_SRC0_DAT_OP_O_REG    0x2442
#define SRC0_DAT_IP_I_SRC0_DAT_OP_O_REG  0x2442
#define SRC1_DAT_IP_I_SRC0_DAT_OP_O_REG  0x2442
#define SRC2_DAT_IP_I_SRC0_DAT_OP_O_REG  0x2442
#define SPORT5_Da_I_SRC1_DAT_OP_O_REG    0x2442
#define SPORT5_Db_I_SRC1_DAT_OP_O_REG    0x2442
#define SRC0_DAT_IP_I_SRC1_DAT_OP_O_REG  0x2442
#define SRC1_DAT_IP_I_SRC1_DAT_OP_O_REG  0x2442
#define SRC2_DAT_IP_I_SRC1_DAT_OP_O_REG  0x2442
#define SPORT5_Da_I_SRC2_DAT_OP_O_REG    0x2442
#define SPORT5_Db_I_SRC2_DAT_OP_O_REG    0x2442
#define SRC0_DAT_IP_I_SRC2_DAT_OP_O_REG  0x2442
#define SRC1_DAT_IP_I_SRC2_DAT_OP_O_REG  0x2442
#define SRC2_DAT_IP_I_SRC2_DAT_OP_O_REG  0x2442
#define SPORT5_Da_I_SRC3_DAT_OP_O_REG    0x2442
#define SPORT5_Db_I_SRC3_DAT_OP_O_REG    0x2442
#define SRC0_DAT_IP_I_SRC3_DAT_OP_O_REG  0x2442
#define SRC1_DAT_IP_I_SRC3_DAT_OP_O_REG  0x2442
#define SRC2_DAT_IP_I_SRC3_DAT_OP_O_REG  0x2442
#define SPORT5_Da_I_SRC0_TDM_IP_O_REG    0x2442
#define SPORT5_Db_I_SRC0_TDM_IP_O_REG    0x2442
#define SRC0_DAT_IP_I_SRC0_TDM_IP_O_REG  0x2442
#define SRC1_DAT_IP_I_SRC0_TDM_IP_O_REG  0x2442
#define SRC2_DAT_IP_I_SRC0_TDM_IP_O_REG  0x2442
#define SPORT5_Da_I_SRC1_TDM_IP_O_REG    0x2442
#define SPORT5_Db_I_SRC1_TDM_IP_O_REG    0x2442
#define SRC0_DAT_IP_I_SRC1_TDM_IP_O_REG  0x2442
#define SRC1_DAT_IP_I_SRC1_TDM_IP_O_REG  0x2442
#define SRC2_DAT_IP_I_SRC1_TDM_IP_O_REG  0x2442
#define SPORT5_Da_I_SRC2_TDM_IP_O_REG    0x2442
#define SPORT5_Db_I_SRC2_TDM_IP_O_REG    0x2442
#define SRC0_DAT_IP_I_SRC2_TDM_IP_O_REG  0x2442
#define SRC1_DAT_IP_I_SRC2_TDM_IP_O_REG  0x2442
#define SRC2_DAT_IP_I_SRC2_TDM_IP_O_REG  0x2442
#define SPORT5_Da_I_SRC3_TDM_IP_O_REG    0x2442
#define SPORT5_Db_I_SRC3_TDM_IP_O_REG    0x2442
#define SRC0_DAT_IP_I_SRC3_TDM_IP_O_REG  0x2442
#define SRC1_DAT_IP_I_SRC3_TDM_IP_O_REG  0x2442
#define SRC2_DAT_IP_I_SRC3_TDM_IP_O_REG  0x2442
#define SPORT5_Da_I_DIR_DAT_O_REG        0x2442
#define SPORT5_Db_I_DIR_DAT_O_REG        0x2442
#define SRC0_DAT_IP_I_DIR_DAT_O_REG      0x2442
#define SRC1_DAT_IP_I_DIR_DAT_O_REG      0x2442
#define SRC2_DAT_IP_I_DIR_DAT_O_REG      0x2442
#define SPORT5_Da_I_LOW_REG              0x2442
#define SPORT5_Db_I_LOW_REG              0x2442
#define SRC0_DAT_IP_I_LOW_REG            0x2442
#define SRC1_DAT_IP_I_LOW_REG            0x2442
#define SRC2_DAT_IP_I_LOW_REG            0x2442
#define SPORT5_Da_I_HIGH_REG             0x2442
#define SPORT5_Db_I_HIGH_REG             0x2442
#define SRC0_DAT_IP_I_HIGH_REG           0x2442
#define SRC1_DAT_IP_I_HIGH_REG           0x2442
#define SRC2_DAT_IP_I_HIGH_REG           0x2442

//SRU_DAT3
#define SRC3_DAT_IP_I_DAI_PB01_O_REG       0x2443    
#define SRC0_TDM_OP_I_DAI_PB01_O_REG       0x2443    
#define SRC1_TDM_OP_I_DAI_PB01_O_REG       0x2443    
#define SRC2_TDM_OP_I_DAI_PB01_O_REG       0x2443    
#define SRC3_TDM_OP_I_DAI_PB01_O_REG       0x2443    
#define SRC3_DAT_IP_I_DAI_PB02_O_REG       0x2443    
#define SRC0_TDM_OP_I_DAI_PB02_O_REG       0x2443    
#define SRC1_TDM_OP_I_DAI_PB02_O_REG       0x2443    
#define SRC2_TDM_OP_I_DAI_PB02_O_REG       0x2443    
#define SRC3_TDM_OP_I_DAI_PB02_O_REG       0x2443    
#define SRC3_DAT_IP_I_DAI_PB03_O_REG       0x2443    
#define SRC0_TDM_OP_I_DAI_PB03_O_REG       0x2443    
#define SRC1_TDM_OP_I_DAI_PB03_O_REG       0x2443    
#define SRC2_TDM_OP_I_DAI_PB03_O_REG       0x2443    
#define SRC3_TDM_OP_I_DAI_PB03_O_REG       0x2443    
#define SRC3_DAT_IP_I_DAI_PB04_O_REG       0x2443    
#define SRC0_TDM_OP_I_DAI_PB04_O_REG       0x2443    
#define SRC1_TDM_OP_I_DAI_PB04_O_REG       0x2443    
#define SRC2_TDM_OP_I_DAI_PB04_O_REG       0x2443    
#define SRC3_TDM_OP_I_DAI_PB04_O_REG       0x2443    
#define SRC3_DAT_IP_I_DAI_PB05_O_REG       0x2443    
#define SRC0_TDM_OP_I_DAI_PB05_O_REG       0x2443    
#define SRC1_TDM_OP_I_DAI_PB05_O_REG       0x2443    
#define SRC2_TDM_OP_I_DAI_PB05_O_REG       0x2443    
#define SRC3_TDM_OP_I_DAI_PB05_O_REG       0x2443    
#define SRC3_DAT_IP_I_DAI_PB06_O_REG       0x2443    
#define SRC0_TDM_OP_I_DAI_PB06_O_REG       0x2443    
#define SRC1_TDM_OP_I_DAI_PB06_O_REG       0x2443    
#define SRC2_TDM_OP_I_DAI_PB06_O_REG       0x2443    
#define SRC3_TDM_OP_I_DAI_PB06_O_REG       0x2443    
#define SRC3_DAT_IP_I_DAI_PB07_O_REG       0x2443    
#define SRC0_TDM_OP_I_DAI_PB07_O_REG       0x2443    
#define SRC1_TDM_OP_I_DAI_PB07_O_REG       0x2443    
#define SRC2_TDM_OP_I_DAI_PB07_O_REG       0x2443    
#define SRC3_TDM_OP_I_DAI_PB07_O_REG       0x2443    
#define SRC3_DAT_IP_I_DAI_PB08_O_REG       0x2443    
#define SRC0_TDM_OP_I_DAI_PB08_O_REG       0x2443    
#define SRC1_TDM_OP_I_DAI_PB08_O_REG       0x2443    
#define SRC2_TDM_OP_I_DAI_PB08_O_REG       0x2443    
#define SRC3_TDM_OP_I_DAI_PB08_O_REG       0x2443    
#define SRC3_DAT_IP_I_DAI_PB09_O_REG       0x2443    
#define SRC0_TDM_OP_I_DAI_PB09_O_REG       0x2443    
#define SRC1_TDM_OP_I_DAI_PB09_O_REG       0x2443    
#define SRC2_TDM_OP_I_DAI_PB09_O_REG       0x2443    
#define SRC3_TDM_OP_I_DAI_PB09_O_REG       0x2443    
#define SRC3_DAT_IP_I_DAI_PB10_O_REG       0x2443    
#define SRC0_TDM_OP_I_DAI_PB10_O_REG       0x2443    
#define SRC1_TDM_OP_I_DAI_PB10_O_REG       0x2443    
#define SRC2_TDM_OP_I_DAI_PB10_O_REG       0x2443    
#define SRC3_TDM_OP_I_DAI_PB10_O_REG       0x2443    
#define SRC3_DAT_IP_I_DAI_PB11_O_REG       0x2443    
#define SRC0_TDM_OP_I_DAI_PB11_O_REG       0x2443    
#define SRC1_TDM_OP_I_DAI_PB11_O_REG       0x2443    
#define SRC2_TDM_OP_I_DAI_PB11_O_REG       0x2443    
#define SRC3_TDM_OP_I_DAI_PB11_O_REG       0x2443    
#define SRC3_DAT_IP_I_DAI_PB12_O_REG       0x2443    
#define SRC0_TDM_OP_I_DAI_PB12_O_REG       0x2443    
#define SRC1_TDM_OP_I_DAI_PB12_O_REG       0x2443    
#define SRC2_TDM_OP_I_DAI_PB12_O_REG       0x2443    
#define SRC3_TDM_OP_I_DAI_PB12_O_REG       0x2443    
#define SRC3_DAT_IP_I_DAI_PB13_O_REG       0x2443    
#define SRC0_TDM_OP_I_DAI_PB13_O_REG       0x2443    
#define SRC1_TDM_OP_I_DAI_PB13_O_REG       0x2443    
#define SRC2_TDM_OP_I_DAI_PB13_O_REG       0x2443    
#define SRC3_TDM_OP_I_DAI_PB13_O_REG       0x2443    
#define SRC3_DAT_IP_I_DAI_PB14_O_REG       0x2443    
#define SRC0_TDM_OP_I_DAI_PB14_O_REG       0x2443    
#define SRC1_TDM_OP_I_DAI_PB14_O_REG       0x2443    
#define SRC2_TDM_OP_I_DAI_PB14_O_REG       0x2443    
#define SRC3_TDM_OP_I_DAI_PB14_O_REG       0x2443    
#define SRC3_DAT_IP_I_DAI_PB15_O_REG       0x2443    
#define SRC0_TDM_OP_I_DAI_PB15_O_REG       0x2443    
#define SRC1_TDM_OP_I_DAI_PB15_O_REG       0x2443    
#define SRC2_TDM_OP_I_DAI_PB15_O_REG       0x2443    
#define SRC3_TDM_OP_I_DAI_PB15_O_REG       0x2443    
#define SRC3_DAT_IP_I_DAI_PB16_O_REG       0x2443    
#define SRC0_TDM_OP_I_DAI_PB16_O_REG       0x2443    
#define SRC1_TDM_OP_I_DAI_PB16_O_REG       0x2443    
#define SRC2_TDM_OP_I_DAI_PB16_O_REG       0x2443    
#define SRC3_TDM_OP_I_DAI_PB16_O_REG       0x2443    
#define SRC3_DAT_IP_I_DAI_PB17_O_REG       0x2443    
#define SRC0_TDM_OP_I_DAI_PB17_O_REG       0x2443    
#define SRC1_TDM_OP_I_DAI_PB17_O_REG       0x2443    
#define SRC2_TDM_OP_I_DAI_PB17_O_REG       0x2443    
#define SRC3_TDM_OP_I_DAI_PB17_O_REG       0x2443    
#define SRC3_DAT_IP_I_DAI_PB18_O_REG       0x2443    
#define SRC0_TDM_OP_I_DAI_PB18_O_REG       0x2443    
#define SRC1_TDM_OP_I_DAI_PB18_O_REG       0x2443    
#define SRC2_TDM_OP_I_DAI_PB18_O_REG       0x2443    
#define SRC3_TDM_OP_I_DAI_PB18_O_REG       0x2443    
#define SRC3_DAT_IP_I_DAI_PB19_O_REG       0x2443    
#define SRC0_TDM_OP_I_DAI_PB19_O_REG       0x2443    
#define SRC1_TDM_OP_I_DAI_PB19_O_REG       0x2443    
#define SRC2_TDM_OP_I_DAI_PB19_O_REG       0x2443    
#define SRC3_TDM_OP_I_DAI_PB19_O_REG       0x2443    
#define SRC3_DAT_IP_I_DAI_PB20_O_REG       0x2443    
#define SRC0_TDM_OP_I_DAI_PB20_O_REG       0x2443    
#define SRC1_TDM_OP_I_DAI_PB20_O_REG       0x2443    
#define SRC2_TDM_OP_I_DAI_PB20_O_REG       0x2443    
#define SRC3_TDM_OP_I_DAI_PB20_O_REG       0x2443    
#define SRC3_DAT_IP_I_SPORT0_Da_O_REG      0x2443    
#define SRC0_TDM_OP_I_SPORT0_Da_O_REG      0x2443    
#define SRC1_TDM_OP_I_SPORT0_Da_O_REG      0x2443    
#define SRC2_TDM_OP_I_SPORT0_Da_O_REG      0x2443    
#define SRC3_TDM_OP_I_SPORT0_Da_O_REG      0x2443    
#define SRC3_DAT_IP_I_SPORT0_Db_O_REG      0x2443    
#define SRC0_TDM_OP_I_SPORT0_Db_O_REG      0x2443    
#define SRC1_TDM_OP_I_SPORT0_Db_O_REG      0x2443    
#define SRC2_TDM_OP_I_SPORT0_Db_O_REG      0x2443    
#define SRC3_TDM_OP_I_SPORT0_Db_O_REG      0x2443    
#define SRC3_DAT_IP_I_SPORT1_Da_O_REG      0x2443    
#define SRC0_TDM_OP_I_SPORT1_Da_O_REG      0x2443    
#define SRC1_TDM_OP_I_SPORT1_Da_O_REG      0x2443    
#define SRC2_TDM_OP_I_SPORT1_Da_O_REG      0x2443    
#define SRC3_TDM_OP_I_SPORT1_Da_O_REG      0x2443    
#define SRC3_DAT_IP_I_SPORT1_Db_O_REG      0x2443    
#define SRC0_TDM_OP_I_SPORT1_Db_O_REG      0x2443    
#define SRC1_TDM_OP_I_SPORT1_Db_O_REG      0x2443    
#define SRC2_TDM_OP_I_SPORT1_Db_O_REG      0x2443    
#define SRC3_TDM_OP_I_SPORT1_Db_O_REG      0x2443    
#define SRC3_DAT_IP_I_SPORT2_Da_O_REG      0x2443    
#define SRC0_TDM_OP_I_SPORT2_Da_O_REG      0x2443    
#define SRC1_TDM_OP_I_SPORT2_Da_O_REG      0x2443    
#define SRC2_TDM_OP_I_SPORT2_Da_O_REG      0x2443    
#define SRC3_TDM_OP_I_SPORT2_Da_O_REG      0x2443    
#define SRC3_DAT_IP_I_SPORT2_Db_O_REG      0x2443    
#define SRC0_TDM_OP_I_SPORT2_Db_O_REG      0x2443    
#define SRC1_TDM_OP_I_SPORT2_Db_O_REG      0x2443    
#define SRC2_TDM_OP_I_SPORT2_Db_O_REG      0x2443    
#define SRC3_TDM_OP_I_SPORT2_Db_O_REG      0x2443    
#define SRC3_DAT_IP_I_SPORT3_Da_O_REG      0x2443    
#define SRC0_TDM_OP_I_SPORT3_Da_O_REG      0x2443    
#define SRC1_TDM_OP_I_SPORT3_Da_O_REG      0x2443    
#define SRC2_TDM_OP_I_SPORT3_Da_O_REG      0x2443    
#define SRC3_TDM_OP_I_SPORT3_Da_O_REG      0x2443    
#define SRC3_DAT_IP_I_SPORT3_Db_O_REG      0x2443    
#define SRC0_TDM_OP_I_SPORT3_Db_O_REG      0x2443    
#define SRC1_TDM_OP_I_SPORT3_Db_O_REG      0x2443    
#define SRC2_TDM_OP_I_SPORT3_Db_O_REG      0x2443    
#define SRC3_TDM_OP_I_SPORT3_Db_O_REG      0x2443    
#define SRC3_DAT_IP_I_SPORT4_Da_O_REG      0x2443    
#define SRC0_TDM_OP_I_SPORT4_Da_O_REG      0x2443    
#define SRC1_TDM_OP_I_SPORT4_Da_O_REG      0x2443    
#define SRC2_TDM_OP_I_SPORT4_Da_O_REG      0x2443    
#define SRC3_TDM_OP_I_SPORT4_Da_O_REG      0x2443    
#define SRC3_DAT_IP_I_SPORT4_Db_O_REG      0x2443    
#define SRC0_TDM_OP_I_SPORT4_Db_O_REG      0x2443    
#define SRC1_TDM_OP_I_SPORT4_Db_O_REG      0x2443    
#define SRC2_TDM_OP_I_SPORT4_Db_O_REG      0x2443    
#define SRC3_TDM_OP_I_SPORT4_Db_O_REG      0x2443    
#define SRC3_DAT_IP_I_SPORT5_Da_O_REG      0x2443    
#define SRC0_TDM_OP_I_SPORT5_Da_O_REG      0x2443    
#define SRC1_TDM_OP_I_SPORT5_Da_O_REG      0x2443    
#define SRC2_TDM_OP_I_SPORT5_Da_O_REG      0x2443    
#define SRC3_TDM_OP_I_SPORT5_Da_O_REG      0x2443    
#define SRC3_DAT_IP_I_SPORT5_Db_O_REG      0x2443    
#define SRC0_TDM_OP_I_SPORT5_Db_O_REG      0x2443    
#define SRC1_TDM_OP_I_SPORT5_Db_O_REG      0x2443    
#define SRC2_TDM_OP_I_SPORT5_Db_O_REG      0x2443    
#define SRC3_TDM_OP_I_SPORT5_Db_O_REG      0x2443
#define SRC3_DAT_IP_I_SRC0_DAT_OP_O_REG    0x2443    
#define SRC0_TDM_OP_I_SRC0_DAT_OP_O_REG    0x2443    
#define SRC1_TDM_OP_I_SRC0_DAT_OP_O_REG    0x2443    
#define SRC2_TDM_OP_I_SRC0_DAT_OP_O_REG    0x2443    
#define SRC3_TDM_OP_I_SRC0_DAT_OP_O_REG    0x2443     
#define SRC3_DAT_IP_I_SRC1_DAT_OP_O_REG    0x2443    
#define SRC0_TDM_OP_I_SRC1_DAT_OP_O_REG    0x2443    
#define SRC1_TDM_OP_I_SRC1_DAT_OP_O_REG    0x2443    
#define SRC2_TDM_OP_I_SRC1_DAT_OP_O_REG    0x2443    
#define SRC3_TDM_OP_I_SRC1_DAT_OP_O_REG    0x2443 
#define SRC3_DAT_IP_I_SRC2_DAT_OP_O_REG    0x2443    
#define SRC0_TDM_OP_I_SRC2_DAT_OP_O_REG    0x2443    
#define SRC1_TDM_OP_I_SRC2_DAT_OP_O_REG    0x2443    
#define SRC2_TDM_OP_I_SRC2_DAT_OP_O_REG    0x2443    
#define SRC3_TDM_OP_I_SRC2_DAT_OP_O_REG    0x2443 
#define SRC3_DAT_IP_I_SRC3_DAT_OP_O_REG    0x2443    
#define SRC0_TDM_OP_I_SRC3_DAT_OP_O_REG    0x2443    
#define SRC1_TDM_OP_I_SRC3_DAT_OP_O_REG    0x2443    
#define SRC2_TDM_OP_I_SRC3_DAT_OP_O_REG    0x2443    
#define SRC3_TDM_OP_I_SRC3_DAT_OP_O_REG    0x2443 
#define SRC3_DAT_IP_I_SRC0_TDM_IP_O_REG    0x2443    
#define SRC0_TDM_OP_I_SRC0_TDM_IP_O_REG    0x2443    
#define SRC1_TDM_OP_I_SRC0_TDM_IP_O_REG    0x2443    
#define SRC2_TDM_OP_I_SRC0_TDM_IP_O_REG    0x2443    
#define SRC3_TDM_OP_I_SRC0_TDM_IP_O_REG    0x2443     
#define SRC3_DAT_IP_I_SRC1_TDM_IP_O_REG    0x2443    
#define SRC0_TDM_OP_I_SRC1_TDM_IP_O_REG    0x2443    
#define SRC1_TDM_OP_I_SRC1_TDM_IP_O_REG    0x2443    
#define SRC2_TDM_OP_I_SRC1_TDM_IP_O_REG    0x2443    
#define SRC3_TDM_OP_I_SRC1_TDM_IP_O_REG    0x2443 
#define SRC3_DAT_IP_I_SRC2_TDM_IP_O_REG    0x2443    
#define SRC0_TDM_OP_I_SRC2_TDM_IP_O_REG    0x2443    
#define SRC1_TDM_OP_I_SRC2_TDM_IP_O_REG    0x2443    
#define SRC2_TDM_OP_I_SRC2_TDM_IP_O_REG    0x2443    
#define SRC3_TDM_OP_I_SRC2_TDM_IP_O_REG    0x2443 
#define SRC3_DAT_IP_I_SRC3_TDM_IP_O_REG    0x2443    
#define SRC0_TDM_OP_I_SRC3_TDM_IP_O_REG    0x2443    
#define SRC1_TDM_OP_I_SRC3_TDM_IP_O_REG    0x2443    
#define SRC2_TDM_OP_I_SRC3_TDM_IP_O_REG    0x2443    
#define SRC3_TDM_OP_I_SRC3_TDM_IP_O_REG    0x2443 
#define SRC3_DAT_IP_I_DIR_DAT_O_REG        0x2443    
#define SRC0_TDM_OP_I_DIR_DAT_O_REG        0x2443    
#define SRC1_TDM_OP_I_DIR_DAT_O_REG        0x2443    
#define SRC2_TDM_OP_I_DIR_DAT_O_REG        0x2443    
#define SRC3_TDM_OP_I_DIR_DAT_O_REG        0x2443
#define SRC3_DAT_IP_I_LOW_REG              0x2443    
#define SRC0_TDM_OP_I_LOW_REG              0x2443    
#define SRC1_TDM_OP_I_LOW_REG              0x2443    
#define SRC2_TDM_OP_I_LOW_REG              0x2443    
#define SRC3_TDM_OP_I_LOW_REG              0x2443
#define SRC3_DAT_IP_I_HIGH_REG             0x2443    
#define SRC0_TDM_OP_I_HIGH_REG             0x2443    
#define SRC1_TDM_OP_I_HIGH_REG             0x2443    
#define SRC2_TDM_OP_I_HIGH_REG             0x2443    
#define SRC3_TDM_OP_I_HIGH_REG             0x2443
   
//SRU_DAT4
#define DIT_DAT_I_DAI_PB01_O_REG         0x2444    
#define IDP0_DAT_I_DAI_PB01_O_REG        0x2444    
#define IDP1_DAT_I_DAI_PB01_O_REG        0x2444    
#define IDP2_DAT_I_DAI_PB01_O_REG        0x2444    
#define IDP3_DAT_I_DAI_PB01_O_REG        0x2444    
#define DAT_DAT_I_DAI_PB02_O_REG         0x2444    
#define IDP0_DAT_I_DAI_PB02_O_REG        0x2444    
#define IDP1_DAT_I_DAI_PB02_O_REG        0x2444    
#define IDP2_DAT_I_DAI_PB02_O_REG        0x2444    
#define IDP3_DAT_I_DAI_PB02_O_REG        0x2444    
#define DIT_DAT_I_DAI_PB03_O_REG         0x2444    
#define IDP0_DAT_I_DAI_PB03_O_REG        0x2444    
#define IDP1_DAT_I_DAI_PB03_O_REG        0x2444    
#define IDP2_DAT_I_DAI_PB03_O_REG        0x2444    
#define IDP3_DAT_I_DAI_PB03_O_REG        0x2444    
#define DIT_DAT_I_DAI_PB04_O_REG         0x2444    
#define IDP0_DAT_I_DAI_PB04_O_REG        0x2444    
#define IDP1_DAT_I_DAI_PB04_O_REG        0x2444    
#define IDP2_DAT_I_DAI_PB04_O_REG        0x2444    
#define IDP3_DAT_I_DAI_PB04_O_REG        0x2444    
#define DIT_DAT_I_DAI_PB05_O_REG         0x2444    
#define IDP0_DAT_I_DAI_PB05_O_REG        0x2444    
#define IDP1_DAT_I_DAI_PB05_O_REG        0x2444    
#define IDP2_DAT_I_DAI_PB05_O_REG        0x2444    
#define IDP3_DAT_I_DAI_PB05_O_REG        0x2444    
#define DIT_DAT_I_DAI_PB06_O_REG         0x2444    
#define IDP0_DAT_I_DAI_PB06_O_REG        0x2444    
#define IDP1_DAT_I_DAI_PB06_O_REG        0x2444    
#define IDP2_DAT_I_DAI_PB06_O_REG        0x2444    
#define IDP3_DAT_I_DAI_PB06_O_REG        0x2444    
#define DIT_DAT_I_DAI_PB07_O_REG         0x2444    
#define IDP0_DAT_I_DAI_PB07_O_REG        0x2444    
#define IDP1_DAT_I_DAI_PB07_O_REG        0x2444    
#define IDP2_DAT_I_DAI_PB07_O_REG        0x2444    
#define IDP3_DAT_I_DAI_PB07_O_REG        0x2444    
#define DIT_DAT_I_DAI_PB08_O_REG         0x2444    
#define IDP0_DAT_I_DAI_PB08_O_REG        0x2444    
#define IDP1_DAT_I_DAI_PB08_O_REG        0x2444    
#define IDP2_DAT_I_DAI_PB08_O_REG        0x2444    
#define IDP3_DAT_I_DAI_PB08_O_REG        0x2444    
#define DIT_DAT_I_DAI_PB09_O_REG         0x2444    
#define IDP0_DAT_I_DAI_PB09_O_REG        0x2444    
#define IDP1_DAT_I_DAI_PB09_O_REG        0x2444    
#define IDP2_DAT_I_DAI_PB09_O_REG        0x2444    
#define IDP3_DAT_I_DAI_PB09_O_REG        0x2444    
#define DIT_DAT_I_DAI_PB10_O_REG         0x2444    
#define IDP0_DAT_I_DAI_PB10_O_REG        0x2444    
#define IDP1_DAT_I_DAI_PB10_O_REG        0x2444    
#define IDP2_DAT_I_DAI_PB10_O_REG        0x2444    
#define IDP3_DAT_I_DAI_PB10_O_REG        0x2444    
#define DIT_DAT_I_DAI_PB11_O_REG         0x2444    
#define IDP0_DAT_I_DAI_PB11_O_REG        0x2444    
#define IDP1_DAT_I_DAI_PB11_O_REG        0x2444    
#define IDP2_DAT_I_DAI_PB11_O_REG        0x2444    
#define IDP3_DAT_I_DAI_PB11_O_REG        0x2444    
#define DIT_DAT_I_DAI_PB12_O_REG         0x2444    
#define IDP0_DAT_I_DAI_PB12_O_REG        0x2444    
#define IDP1_DAT_I_DAI_PB12_O_REG        0x2444    
#define IDP2_DAT_I_DAI_PB12_O_REG        0x2444    
#define IDP3_DAT_I_DAI_PB12_O_REG        0x2444    
#define DIT_DAT_I_DAI_PB13_O_REG         0x2444    
#define IDP0_DAT_I_DAI_PB13_O_REG        0x2444    
#define IDP1_DAT_I_DAI_PB13_O_REG        0x2444    
#define IDP2_DAT_I_DAI_PB13_O_REG        0x2444    
#define IDP3_DAT_I_DAI_PB13_O_REG        0x2444    
#define DIT_DAT_I_DAI_PB14_O_REG         0x2444    
#define IDP0_DAT_I_DAI_PB14_O_REG        0x2444    
#define IDP1_DAT_I_DAI_PB14_O_REG        0x2444    
#define IDP2_DAT_I_DAI_PB14_O_REG        0x2444    
#define IDP3_DAT_I_DAI_PB14_O_REG        0x2444    
#define DIT_DAT_I_DAI_PB15_O_REG         0x2444    
#define IDP0_DAT_I_DAI_PB15_O_REG        0x2444    
#define IDP1_DAT_I_DAI_PB15_O_REG        0x2444    
#define IDP2_DAT_I_DAI_PB15_O_REG        0x2444    
#define IDP3_DAT_I_DAI_PB15_O_REG        0x2444    
#define DIT_DAT_I_DAI_PB16_O_REG         0x2444    
#define IDP0_DAT_I_DAI_PB16_O_REG        0x2444    
#define IDP1_DAT_I_DAI_PB16_O_REG        0x2444    
#define IDP2_DAT_I_DAI_PB16_O_REG        0x2444    
#define IDP3_DAT_I_DAI_PB16_O_REG        0x2444    
#define DIT_DAT_I_DAI_PB17_O_REG         0x2444    
#define IDP0_DAT_I_DAI_PB17_O_REG        0x2444    
#define IDP1_DAT_I_DAI_PB17_O_REG        0x2444    
#define IDP2_DAT_I_DAI_PB17_O_REG        0x2444    
#define IDP3_DAT_I_DAI_PB17_O_REG        0x2444    
#define DIT_DAT_I_DAI_PB18_O_REG         0x2444    
#define IDP0_DAT_I_DAI_PB18_O_REG        0x2444    
#define IDP1_DAT_I_DAI_PB18_O_REG        0x2444    
#define IDP2_DAT_I_DAI_PB18_O_REG        0x2444    
#define IDP3_DAT_I_DAI_PB18_O_REG        0x2444    
#define DIT_DAT_I_DAI_PB19_O_REG         0x2444    
#define IDP0_DAT_I_DAI_PB19_O_REG        0x2444    
#define IDP1_DAT_I_DAI_PB19_O_REG        0x2444    
#define IDP2_DAT_I_DAI_PB19_O_REG        0x2444    
#define IDP3_DAT_I_DAI_PB19_O_REG        0x2444    
#define DIT_DAT_I_DAI_PB20_O_REG         0x2444    
#define IDP0_DAT_I_DAI_PB20_O_REG        0x2444    
#define IDP1_DAT_I_DAI_PB20_O_REG        0x2444    
#define IDP2_DAT_I_DAI_PB20_O_REG        0x2444    
#define IDP3_DAT_I_DAI_PB20_O_REG        0x2444    
#define DIT_DAT_I_SPORT0_Da_O_REG        0x2444    
#define IDP0_DAT_I_SPORT0_Da_O_REG       0x2444    
#define IDP1_DAT_I_SPORT0_Da_O_REG       0x2444    
#define IDP2_DAT_I_SPORT0_Da_O_REG       0x2444    
#define IDP3_DAT_I_SPORT0_Da_O_REG       0x2444    
#define DIT_DAT_I_SPORT0_Db_O_REG        0x2444    
#define IDP0_DAT_I_SPORT0_Db_O_REG       0x2444    
#define IDP1_DAT_I_SPORT0_Db_O_REG       0x2444    
#define IDP2_DAT_I_SPORT0_Db_O_REG       0x2444    
#define IDP3_DAT_I_SPORT0_Db_O_REG       0x2444    
#define DIT_DAT_I_SPORT1_Da_O_REG        0x2444    
#define IDP0_DAT_I_SPORT1_Da_O_REG       0x2444    
#define IDP1_DAT_I_SPORT1_Da_O_REG       0x2444    
#define IDP2_DAT_I_SPORT1_Da_O_REG       0x2444    
#define IDP3_DAT_I_SPORT1_Da_O_REG       0x2444    
#define DIT_DAT_I_SPORT1_Db_O_REG        0x2444    
#define IDP0_DAT_I_SPORT1_Db_O_REG       0x2444    
#define IDP1_DAT_I_SPORT1_Db_O_REG       0x2444    
#define IDP2_DAT_I_SPORT1_Db_O_REG       0x2444    
#define IDP3_DAT_I_SPORT1_Db_O_REG       0x2444    
#define DIT_DAT_I_SPORT2_Da_O_REG        0x2444    
#define IDP0_DAT_I_SPORT2_Da_O_REG       0x2444    
#define IDP1_DAT_I_SPORT2_Da_O_REG       0x2444    
#define IDP2_DAT_I_SPORT2_Da_O_REG       0x2444    
#define IDP3_DAT_I_SPORT2_Da_O_REG       0x2444    
#define DIT_DAT_I_SPORT2_Db_O_REG        0x2444    
#define IDP0_DAT_I_SPORT2_Db_O_REG       0x2444    
#define IDP1_DAT_I_SPORT2_Db_O_REG       0x2444    
#define IDP2_DAT_I_SPORT2_Db_O_REG       0x2444    
#define IDP3_DAT_I_SPORT2_Db_O_REG       0x2444    
#define DIT_DAT_I_SPORT3_Da_O_REG        0x2444    
#define IDP0_DAT_I_SPORT3_Da_O_REG       0x2444    
#define IDP1_DAT_I_SPORT3_Da_O_REG       0x2444    
#define IDP2_DAT_I_SPORT3_Da_O_REG       0x2444    
#define IDP3_DAT_I_SPORT3_Da_O_REG       0x2444    
#define DIT_DAT_I_SPORT3_Db_O_REG        0x2444    
#define IDP0_DAT_I_SPORT3_Db_O_REG       0x2444    
#define IDP1_DAT_I_SPORT3_Db_O_REG       0x2444    
#define IDP2_DAT_I_SPORT3_Db_O_REG       0x2444    
#define IDP3_DAT_I_SPORT3_Db_O_REG       0x2444    
#define DIT_DAT_I_SPORT4_Da_O_REG        0x2444    
#define IDP0_DAT_I_SPORT4_Da_O_REG       0x2444    
#define IDP1_DAT_I_SPORT4_Da_O_REG       0x2444    
#define IDP2_DAT_I_SPORT4_Da_O_REG       0x2444    
#define IDP3_DAT_I_SPORT4_Da_O_REG       0x2444    
#define DIT_DAT_I_SPORT4_Db_O_REG        0x2444    
#define IDP0_DAT_I_SPORT4_Db_O_REG       0x2444    
#define IDP1_DAT_I_SPORT4_Db_O_REG       0x2444    
#define IDP2_DAT_I_SPORT4_Db_O_REG       0x2444    
#define IDP3_DAT_I_SPORT4_Db_O_REG       0x2444    
#define DIT_DAT_I_SPORT5_Da_O_REG        0x2444    
#define IDP0_DAT_I_SPORT5_Da_O_REG       0x2444    
#define IDP1_DAT_I_SPORT5_Da_O_REG       0x2444    
#define IDP2_DAT_I_SPORT5_Da_O_REG       0x2444    
#define IDP3_DAT_I_SPORT5_Da_O_REG       0x2444    
#define DIT_DAT_I_SPORT5_Db_O_REG        0x2444    
#define IDP0_DAT_I_SPORT5_Db_O_REG       0x2444    
#define IDP1_DAT_I_SPORT5_Db_O_REG       0x2444    
#define IDP2_DAT_I_SPORT5_Db_O_REG       0x2444    
#define IDP3_DAT_I_SPORT5_Db_O_REG       0x2444    
#define DIT_DAT_I_SRC0_DAT_OP_O_REG      0x2444    
#define IDP0_DAT_I_SRC0_DAT_OP_O_REG     0x2444    
#define IDP1_DAT_I_SRC0_DAT_OP_O_REG     0x2444    
#define IDP2_DAT_I_SRC0_DAT_OP_O_REG     0x2444    
#define IDP3_DAT_I_SRC0_DAT_OP_O_REG     0x2444 
#define DIT_DAT_I_SRC1_DAT_OP_O_REG      0x2444    
#define IDP0_DAT_I_SRC1_DAT_OP_O_REG     0x2444    
#define IDP1_DAT_I_SRC1_DAT_OP_O_REG     0x2444    
#define IDP2_DAT_I_SRC1_DAT_OP_O_REG     0x2444    
#define IDP3_DAT_I_SRC1_DAT_OP_O_REG     0x2444
#define DIT_DAT_I_SRC2_DAT_OP_O_REG      0x2444    
#define IDP0_DAT_I_SRC2_DAT_OP_O_REG     0x2444    
#define IDP1_DAT_I_SRC2_DAT_OP_O_REG     0x2444    
#define IDP2_DAT_I_SRC2_DAT_OP_O_REG     0x2444    
#define IDP3_DAT_I_SRC2_DAT_OP_O_REG     0x2444
#define DIT_DAT_I_SRC3_DAT_OP_O_REG      0x2444    
#define IDP0_DAT_I_SRC3_DAT_OP_O_REG     0x2444    
#define IDP1_DAT_I_SRC3_DAT_OP_O_REG     0x2444    
#define IDP2_DAT_I_SRC3_DAT_OP_O_REG     0x2444    
#define IDP3_DAT_I_SRC3_DAT_OP_O_REG     0x2444
#define DIT_DAT_I_SRC0_TDM_IP_O_REG      0x2444    
#define IDP0_DAT_I_SRC0_TDM_IP_O_REG     0x2444    
#define IDP1_DAT_I_SRC0_TDM_IP_O_REG     0x2444    
#define IDP2_DAT_I_SRC0_TDM_IP_O_REG     0x2444    
#define IDP3_DAT_I_SRC0_TDM_IP_O_REG     0x2444
#define DIT_DAT_I_SRC1_TDM_IP_O_REG      0x2444    
#define IDP0_DAT_I_SRC1_TDM_IP_O_REG     0x2444    
#define IDP1_DAT_I_SRC1_TDM_IP_O_REG     0x2444    
#define IDP2_DAT_I_SRC1_TDM_IP_O_REG     0x2444    
#define IDP3_DAT_I_SRC1_TDM_IP_O_REG     0x2444
#define DIT_DAT_I_SRC2_TDM_IP_O_REG      0x2444    
#define IDP0_DAT_I_SRC2_TDM_IP_O_REG     0x2444    
#define IDP1_DAT_I_SRC2_TDM_IP_O_REG     0x2444    
#define IDP2_DAT_I_SRC2_TDM_IP_O_REG     0x2444    
#define IDP3_DAT_I_SRC2_TDM_IP_O_REG     0x2444
#define DIT_DAT_I_SRC3_TDM_IP_O_REG      0x2444    
#define IDP0_DAT_I_SRC3_TDM_IP_O_REG     0x2444    
#define IDP1_DAT_I_SRC3_TDM_IP_O_REG     0x2444    
#define IDP2_DAT_I_SRC3_TDM_IP_O_REG     0x2444    
#define IDP3_DAT_I_SRC3_TDM_IP_O_REG     0x2444
#define DIT_DAT_I_DIR_DAT_O_REG          0x2444    
#define IDP0_DAT_I_DIR_DAT_O_REG         0x2444    
#define IDP1_DAT_I_DIR_DAT_O_REG         0x2444    
#define IDP2_DAT_I_DIR_DAT_O_REG         0x2444    
#define IDP3_DAT_I_DIR_DAT_O_REG         0x2444
#define DIT_DAT_I_LOW_REG                0x2444    
#define IDP0_DAT_I_LOW_REG               0x2444    
#define IDP1_DAT_I_LOW_REG               0x2444    
#define IDP2_DAT_I_LOW_REG               0x2444    
#define IDP3_DAT_I_LOW_REG               0x2444
#define DIT_DAT_I_HIGH_REG               0x2444    
#define IDP0_DAT_I_HIGH_REG              0x2444    
#define IDP1_DAT_I_HIGH_REG              0x2444    
#define IDP2_DAT_I_HIGH_REG              0x2444    
#define IDP3_DAT_I_HIGH_REG              0x2444

//SRU_DAT5
#define IDP4_DAT_I_DAI_PB01_O_REG        0x2445    
#define IDP5_DAT_I_DAI_PB01_O_REG        0x2445    
#define IDP6_DAT_I_DAI_PB01_O_REG        0x2445    
#define IDP7_DAT_I_DAI_PB01_O_REG        0x2445    
#define IDP4_DAT_I_DAI_PB02_O_REG        0x2445    
#define IDP5_DAT_I_DAI_PB02_O_REG        0x2445    
#define IDP6_DAT_I_DAI_PB02_O_REG        0x2445    
#define IDP7_DAT_I_DAI_PB02_O_REG        0x2445    
#define IDP4_DAT_I_DAI_PB03_O_REG        0x2445    
#define IDP5_DAT_I_DAI_PB03_O_REG        0x2445    
#define IDP6_DAT_I_DAI_PB03_O_REG        0x2445    
#define IDP7_DAT_I_DAI_PB03_O_REG        0x2445    
#define IDP4_DAT_I_DAI_PB04_O_REG        0x2445    
#define IDP5_DAT_I_DAI_PB04_O_REG        0x2445    
#define IDP6_DAT_I_DAI_PB04_O_REG        0x2445    
#define IDP7_DAT_I_DAI_PB04_O_REG        0x2445    
#define IDP4_DAT_I_DAI_PB05_O_REG        0x2445    
#define IDP5_DAT_I_DAI_PB05_O_REG        0x2445    
#define IDP6_DAT_I_DAI_PB05_O_REG        0x2445    
#define IDP7_DAT_I_DAI_PB05_O_REG        0x2445    
#define IDP4_DAT_I_DAI_PB06_O_REG        0x2445    
#define IDP5_DAT_I_DAI_PB06_O_REG        0x2445    
#define IDP6_DAT_I_DAI_PB06_O_REG        0x2445    
#define IDP7_DAT_I_DAI_PB06_O_REG        0x2445    
#define IDP4_DAT_I_DAI_PB07_O_REG        0x2445    
#define IDP5_DAT_I_DAI_PB07_O_REG        0x2445    
#define IDP6_DAT_I_DAI_PB07_O_REG        0x2445    
#define IDP7_DAT_I_DAI_PB07_O_REG        0x2445    
#define IDP4_DAT_I_DAI_PB08_O_REG        0x2445    
#define IDP5_DAT_I_DAI_PB08_O_REG        0x2445    
#define IDP6_DAT_I_DAI_PB08_O_REG        0x2445    
#define IDP7_DAT_I_DAI_PB08_O_REG        0x2445    
#define IDP4_DAT_I_DAI_PB09_O_REG        0x2445    
#define IDP5_DAT_I_DAI_PB09_O_REG        0x2445    
#define IDP6_DAT_I_DAI_PB09_O_REG        0x2445    
#define IDP7_DAT_I_DAI_PB09_O_REG        0x2445    
#define IDP4_DAT_I_DAI_PB10_O_REG        0x2445    
#define IDP5_DAT_I_DAI_PB10_O_REG        0x2445    
#define IDP6_DAT_I_DAI_PB10_O_REG        0x2445    
#define IDP7_DAT_I_DAI_PB10_O_REG        0x2445    
#define IDP4_DAT_I_DAI_PB11_O_REG        0x2445    
#define IDP5_DAT_I_DAI_PB11_O_REG        0x2445    
#define IDP6_DAT_I_DAI_PB11_O_REG        0x2445    
#define IDP7_DAT_I_DAI_PB11_O_REG        0x2445    
#define IDP4_DAT_I_DAI_PB12_O_REG        0x2445    
#define IDP5_DAT_I_DAI_PB12_O_REG        0x2445    
#define IDP6_DAT_I_DAI_PB12_O_REG        0x2445    
#define IDP7_DAT_I_DAI_PB12_O_REG        0x2445    
#define IDP4_DAT_I_DAI_PB13_O_REG        0x2445    
#define IDP5_DAT_I_DAI_PB13_O_REG        0x2445    
#define IDP6_DAT_I_DAI_PB13_O_REG        0x2445    
#define IDP7_DAT_I_DAI_PB13_O_REG        0x2445    
#define IDP4_DAT_I_DAI_PB14_O_REG        0x2445    
#define IDP5_DAT_I_DAI_PB14_O_REG        0x2445    
#define IDP6_DAT_I_DAI_PB14_O_REG        0x2445    
#define IDP7_DAT_I_DAI_PB14_O_REG        0x2445    
#define IDP4_DAT_I_DAI_PB15_O_REG        0x2445    
#define IDP5_DAT_I_DAI_PB15_O_REG        0x2445    
#define IDP6_DAT_I_DAI_PB15_O_REG        0x2445    
#define IDP7_DAT_I_DAI_PB15_O_REG        0x2445    
#define IDP4_DAT_I_DAI_PB16_O_REG        0x2445    
#define IDP5_DAT_I_DAI_PB16_O_REG        0x2445    
#define IDP6_DAT_I_DAI_PB16_O_REG        0x2445    
#define IDP7_DAT_I_DAI_PB16_O_REG        0x2445    
#define IDP4_DAT_I_DAI_PB17_O_REG        0x2445    
#define IDP5_DAT_I_DAI_PB17_O_REG        0x2445    
#define IDP6_DAT_I_DAI_PB17_O_REG        0x2445    
#define IDP7_DAT_I_DAI_PB17_O_REG        0x2445    
#define IDP4_DAT_I_DAI_PB18_O_REG        0x2445    
#define IDP5_DAT_I_DAI_PB18_O_REG        0x2445    
#define IDP6_DAT_I_DAI_PB18_O_REG        0x2445    
#define IDP7_DAT_I_DAI_PB18_O_REG        0x2445    
#define IDP4_DAT_I_DAI_PB19_O_REG        0x2445    
#define IDP5_DAT_I_DAI_PB19_O_REG        0x2445    
#define IDP6_DAT_I_DAI_PB19_O_REG        0x2445    
#define IDP7_DAT_I_DAI_PB19_O_REG        0x2445    
#define IDP4_DAT_I_DAI_PB20_O_REG        0x2445    
#define IDP5_DAT_I_DAI_PB20_O_REG        0x2445    
#define IDP6_DAT_I_DAI_PB20_O_REG        0x2445    
#define IDP7_DAT_I_DAI_PB20_O_REG        0x2445    
#define IDP4_DAT_I_SPORT0_Da_O_REG       0x2445    
#define IDP5_DAT_I_SPORT0_Da_O_REG       0x2445    
#define IDP6_DAT_I_SPORT0_Da_O_REG       0x2445    
#define IDP7_DAT_I_SPORT0_Da_O_REG       0x2445    
#define IDP4_DAT_I_SPORT0_Db_O_REG       0x2445    
#define IDP5_DAT_I_SPORT0_Db_O_REG       0x2445    
#define IDP6_DAT_I_SPORT0_Db_O_REG       0x2445    
#define IDP7_DAT_I_SPORT0_Db_O_REG       0x2445    
#define IDP4_DAT_I_SPORT1_Da_O_REG       0x2445    
#define IDP5_DAT_I_SPORT1_Da_O_REG       0x2445    
#define IDP6_DAT_I_SPORT1_Da_O_REG       0x2445    
#define IDP7_DAT_I_SPORT1_Da_O_REG       0x2445    
#define IDP4_DAT_I_SPORT1_Db_O_REG       0x2445    
#define IDP5_DAT_I_SPORT1_Db_O_REG       0x2445    
#define IDP6_DAT_I_SPORT1_Db_O_REG       0x2445    
#define IDP7_DAT_I_SPORT1_Db_O_REG       0x2445    
#define IDP4_DAT_I_SPORT2_Da_O_REG       0x2445    
#define IDP5_DAT_I_SPORT2_Da_O_REG       0x2445    
#define IDP6_DAT_I_SPORT2_Da_O_REG       0x2445    
#define IDP7_DAT_I_SPORT2_Da_O_REG       0x2445    
#define IDP4_DAT_I_SPORT2_Db_O_REG       0x2445    
#define IDP5_DAT_I_SPORT2_Db_O_REG       0x2445    
#define IDP6_DAT_I_SPORT2_Db_O_REG       0x2445    
#define IDP7_DAT_I_SPORT2_Db_O_REG       0x2445    
#define IDP4_DAT_I_SPORT3_Da_O_REG       0x2445    
#define IDP5_DAT_I_SPORT3_Da_O_REG       0x2445    
#define IDP6_DAT_I_SPORT3_Da_O_REG       0x2445    
#define IDP7_DAT_I_SPORT3_Da_O_REG       0x2445    
#define IDP4_DAT_I_SPORT3_Db_O_REG       0x2445    
#define IDP5_DAT_I_SPORT3_Db_O_REG       0x2445    
#define IDP6_DAT_I_SPORT3_Db_O_REG       0x2445    
#define IDP7_DAT_I_SPORT3_Db_O_REG       0x2445    
#define IDP4_DAT_I_SPORT4_Da_O_REG       0x2445    
#define IDP5_DAT_I_SPORT4_Da_O_REG       0x2445    
#define IDP6_DAT_I_SPORT4_Da_O_REG       0x2445    
#define IDP7_DAT_I_SPORT4_Da_O_REG       0x2445    
#define IDP4_DAT_I_SPORT4_Db_O_REG       0x2445    
#define IDP5_DAT_I_SPORT4_Db_O_REG       0x2445    
#define IDP6_DAT_I_SPORT4_Db_O_REG       0x2445    
#define IDP7_DAT_I_SPORT4_Db_O_REG       0x2445    
#define IDP4_DAT_I_SPORT5_Da_O_REG       0x2445    
#define IDP5_DAT_I_SPORT5_Da_O_REG       0x2445    
#define IDP6_DAT_I_SPORT5_Da_O_REG       0x2445    
#define IDP7_DAT_I_SPORT5_Da_O_REG       0x2445    
#define IDP4_DAT_I_SPORT5_Db_O_REG       0x2445    
#define IDP5_DAT_I_SPORT5_Db_O_REG       0x2445    
#define IDP6_DAT_I_SPORT5_Db_O_REG       0x2445    
#define IDP7_DAT_I_SPORT5_Db_O_REG       0x2445    
#define IDP4_DAT_I_SRC0_DAT_OP_O_REG     0x2445    
#define IDP5_DAT_I_SRC0_DAT_OP_O_REG     0x2445    
#define IDP6_DAT_I_SRC0_DAT_OP_O_REG     0x2445    
#define IDP7_DAT_I_SRC0_DAT_OP_O_REG     0x2445     
#define IDP4_DAT_I_SRC1_DAT_OP_O_REG     0x2445    
#define IDP5_DAT_I_SRC1_DAT_OP_O_REG     0x2445    
#define IDP6_DAT_I_SRC1_DAT_OP_O_REG     0x2445    
#define IDP7_DAT_I_SRC1_DAT_OP_O_REG     0x2445 
#define IDP4_DAT_I_SRC2_DAT_OP_O_REG     0x2445    
#define IDP5_DAT_I_SRC2_DAT_OP_O_REG     0x2445    
#define IDP6_DAT_I_SRC2_DAT_OP_O_REG     0x2445    
#define IDP7_DAT_I_SRC2_DAT_OP_O_REG     0x2445 
#define IDP4_DAT_I_SRC3_DAT_OP_O_REG     0x2445    
#define IDP5_DAT_I_SRC3_DAT_OP_O_REG     0x2445    
#define IDP6_DAT_I_SRC3_DAT_OP_O_REG     0x2445    
#define IDP7_DAT_I_SRC3_DAT_OP_O_REG     0x2445 
#define IDP4_DAT_I_SRC0_TDM_IP_O_REG     0x2445    
#define IDP5_DAT_I_SRC0_TDM_IP_O_REG     0x2445    
#define IDP6_DAT_I_SRC0_TDM_IP_O_REG     0x2445    
#define IDP7_DAT_I_SRC0_TDM_IP_O_REG     0x2445     
#define IDP4_DAT_I_SRC1_TDM_IP_O_REG     0x2445    
#define IDP5_DAT_I_SRC1_TDM_IP_O_REG     0x2445    
#define IDP6_DAT_I_SRC1_TDM_IP_O_REG     0x2445    
#define IDP7_DAT_I_SRC1_TDM_IP_O_REG     0x2445 
#define IDP4_DAT_I_SRC2_TDM_IP_O_REG     0x2445    
#define IDP5_DAT_I_SRC2_TDM_IP_O_REG     0x2445    
#define IDP6_DAT_I_SRC2_TDM_IP_O_REG     0x2445    
#define IDP7_DAT_I_SRC2_TDM_IP_O_REG     0x2445 
#define IDP4_DAT_I_SRC3_TDM_IP_O_REG     0x2445    
#define IDP5_DAT_I_SRC3_TDM_IP_O_REG     0x2445    
#define IDP6_DAT_I_SRC3_TDM_IP_O_REG     0x2445    
#define IDP7_DAT_I_SRC3_TDM_IP_O_REG     0x2445 
#define IDP4_DAT_I_DIR_DAT_O_REG         0x2445    
#define IDP5_DAT_I_DIR_DAT_O_REG         0x2445    
#define IDP6_DAT_I_DIR_DAT_O_REG         0x2445    
#define IDP7_DAT_I_DIR_DAT_O_REG         0x2445
#define IDP4_DAT_I_LOW_REG               0x2445    
#define IDP5_DAT_I_LOW_REG               0x2445    
#define IDP6_DAT_I_LOW_REG               0x2445    
#define IDP7_DAT_I_LOW_REG               0x2445
#define IDP4_DAT_I_HIGH_REG              0x2445    
#define IDP5_DAT_I_HIGH_REG              0x2445    
#define IDP6_DAT_I_HIGH_REG              0x2445    
#define IDP7_DAT_I_HIGH_REG              0x2445
                   
                   
/*============================================================================*
 *                                                                                                                                                    *
 *      GROUP C                                                                   *
 *                                                                                                                                                        *
 *============================================================================*/
                   
//------------------------------------------------------------------------------
//  Group C: Input bitfield masks & shifts
//------------------------------------------------------------------------------

//SRU_FS0
  #define    SPORT0_FS_I_MASK                       0xFFFFFFE0
  #define    SPORT0_FS_I_SHIFT                      0
  #define    SPORT1_FS_I_MASK                       0xFFFFFC1F
  #define    SPORT1_FS_I_SHIFT                      5
  #define    SPORT2_FS_I_MASK                       0xFFFF83FF
  #define    SPORT2_FS_I_SHIFT                      10
  #define    SPORT3_FS_I_MASK                       0xFFF07FFF
  #define    SPORT3_FS_I_SHIFT                      15
  #define    SPORT4_FS_I_MASK                       0xFE0FFFFF
  #define    SPORT4_FS_I_SHIFT                      20
  #define    SPORT5_FS_I_MASK                       0xC1FFFFFF
  #define    SPORT5_FS_I_SHIFT                      25

//SRU_FS1
  #define    SRC0_FS_IP_I_MASK                      0xFFFFFFE0
  #define    SRC0_FS_IP_I_SHIFT                     0
  #define    SRC0_FS_OP_I_MASK                      0xFFFFFC1F
  #define    SRC0_FS_OP_I_SHIFT                     5
  #define    SRC1_FS_IP_I_MASK                      0xFFFF83FF
  #define    SRC1_FS_IP_I_SHIFT                     10
  #define    SRC1_FS_OP_I_MASK                      0xFFF07FFF
  #define    SRC1_FS_OP_I_SHIFT                     15
  #define    SRC2_FS_IP_I_MASK                      0xFE0FFFFF
  #define    SRC2_FS_IP_I_SHIFT                     20
  #define    SRC2_FS_OP_I_MASK                      0xC1FFFFFF
  #define    SRC2_FS_OP_I_SHIFT                     25

//SRU_FS2
  #define    SRC3_FS_IP_I_MASK                      0xFFFFFFE0
  #define    SRC3_FS_IP_I_SHIFT                     0
  #define    SRC3_FS_OP_I_MASK                      0xFFFFFC1F
  #define    SRC3_FS_OP_I_SHIFT                     5
  #define    DIT_I_MASK                          0xFFFF83FF
  #define    DIT_I_SHIFT                         10
  #define    IDP0_FS_I_MASK                         0xFFF07FFF
  #define    IDP0_FS_I_SHIFT                        15
  #define    PDAP_HOLD_I_MASK                       0xFFF07FFF
  #define    PDAP_HOLD_I_SHIFT                      15
  #define    IDP1_FS_I_MASK                         0xFE0FFFFF
  #define    IDP1_FS_I_SHIFT                        20
  #define    IDP2_FS_I_MASK                         0xC1FFFFFF
  #define    IDP2_FS_I_SHIFT                        25
  
//SRU_FS3  
  #define    IDP3_FS_I_MASK                         0xFFFFFFE0
  #define    IDP3_FS_I_SHIFT                        0
  #define    IDP4_FS_I_MASK                         0xFFFFFC1F
  #define    IDP4_FS_I_SHIFT                        5
  #define    IDP5_FS_I_MASK                         0xFFFF83FF
  #define    IDP5_FS_I_SHIFT                        10
  #define    IDP6_FS_I_MASK                         0xFFF07FFF
  #define    IDP6_FS_I_SHIFT                        15
  #define    IDP7_FS_I_MASK                         0xFE0FFFFF
  #define    IDP7_FS_I_SHIFT                        20
  #define    DIR_FS_I_MASK                          0xC1FFFFFF
  #define    DIR_FS_I_SHIFT                         25


//------------------------------------------------------------------------------
//  Group C: Bitfield encodings
//------------------------------------------------------------------------------

#define SPORT0_FS_I_DAI_PB01_O_ENCODING             0x00
#define SPORT1_FS_I_DAI_PB01_O_ENCODING             0x00
#define SPORT2_FS_I_DAI_PB01_O_ENCODING             0x00
#define SPORT3_FS_I_DAI_PB01_O_ENCODING             0x00
#define SPORT4_FS_I_DAI_PB01_O_ENCODING             0x00
#define SPORT5_FS_I_DAI_PB01_O_ENCODING             0x00
#define IDP0_FS_I_DAI_PB01_O_ENCODING               0x00
#define PDAP_HOLD_I_DAI_PB01_O_ENCODING             0x00
#define IDP1_FS_I_DAI_PB01_O_ENCODING               0x00
#define IDP2_FS_I_DAI_PB01_O_ENCODING               0x00
#define IDP3_FS_I_DAI_PB01_O_ENCODING               0x00
#define IDP4_FS_I_DAI_PB01_O_ENCODING               0x00
#define IDP5_FS_I_DAI_PB01_O_ENCODING               0x00
#define IDP6_FS_I_DAI_PB01_O_ENCODING               0x00
#define IDP7_FS_I_DAI_PB01_O_ENCODING               0x00
#define SRC0_FS_IP_I_DAI_PB01_O_ENCODING            0x00
#define SRC0_FS_0P_I_DAI_PB01_O_ENCODING            0x00
#define SRC1_FS_IP_I_DAI_PB01_O_ENCODING            0x00
#define SRC1_FS_OP_I_DAI_PB01_O_ENCODING            0x00
#define SRC2_FS_IP_I_DAI_PB01_O_ENCODING            0x00
#define SRC2_FS_OP_I_DAI_PB01_O_ENCODING            0x00
#define SRC3_FS_IP_I_DAI_PB01_O_ENCODING            0x00
#define SRC3_FS_OP_I_DAI_PB01_O_ENCODING            0x00
#define DIT_I_DAI_PB01_O_ENCODING                0x00
#define DIR_FS_I_DAI_PB01_O_ENCODING                0x00


#define SPORT0_FS_I_DAI_PB02_O_ENCODING             0x01
#define SPORT1_FS_I_DAI_PB02_O_ENCODING             0x01
#define SPORT2_FS_I_DAI_PB02_O_ENCODING             0x01
#define SPORT3_FS_I_DAI_PB02_O_ENCODING             0x01
#define SPORT4_FS_I_DAI_PB02_O_ENCODING             0x01
#define SPORT5_FS_I_DAI_PB02_O_ENCODING             0x01
#define IDP0_FS_I_DAI_PB02_O_ENCODING               0x01
#define PDAP_HOLD_I_DAI_PB02_O_ENCODING             0x01
#define IDP1_FS_I_DAI_PB02_O_ENCODING               0x01
#define IDP2_FS_I_DAI_PB02_O_ENCODING               0x01
#define IDP3_FS_I_DAI_PB02_O_ENCODING               0x01
#define IDP4_FS_I_DAI_PB02_O_ENCODING               0x01
#define IDP5_FS_I_DAI_PB02_O_ENCODING               0x01
#define IDP6_FS_I_DAI_PB02_O_ENCODING               0x01
#define IDP7_FS_I_DAI_PB02_O_ENCODING               0x01
#define SRC0_FS_IP_I_DAI_PB02_O_ENCODING            0x01
#define SRC0_FS_0P_I_DAI_PB02_O_ENCODING            0x01
#define SRC1_FS_IP_I_DAI_PB02_O_ENCODING            0x01
#define SRC1_FS_OP_I_DAI_PB02_O_ENCODING            0x01
#define SRC2_FS_IP_I_DAI_PB02_O_ENCODING            0x01
#define SRC2_FS_OP_I_DAI_PB02_O_ENCODING            0x01
#define SRC3_FS_IP_I_DAI_PB02_O_ENCODING            0x01
#define SRC3_FS_OP_I_DAI_PB02_O_ENCODING            0x01
#define DIT_I_DAI_PB02_O_ENCODING                0x01
#define DIR_FS_I_DAI_PB02_O_ENCODING                0x01

#define SPORT0_FS_I_DAI_PB03_O_ENCODING             0x02
#define SPORT1_FS_I_DAI_PB03_O_ENCODING             0x02
#define SPORT2_FS_I_DAI_PB03_O_ENCODING             0x02
#define SPORT3_FS_I_DAI_PB03_O_ENCODING             0x02
#define SPORT4_FS_I_DAI_PB03_O_ENCODING             0x02
#define SPORT5_FS_I_DAI_PB03_O_ENCODING             0x02
#define IDP0_FS_I_DAI_PB03_O_ENCODING               0x02
#define PDAP_HOLD_I_DAI_PB03_O_ENCODING             0x02
#define IDP1_FS_I_DAI_PB03_O_ENCODING               0x02
#define IDP2_FS_I_DAI_PB03_O_ENCODING               0x02
#define IDP3_FS_I_DAI_PB03_O_ENCODING               0x02
#define IDP4_FS_I_DAI_PB03_O_ENCODING               0x02
#define IDP5_FS_I_DAI_PB03_O_ENCODING               0x02
#define IDP6_FS_I_DAI_PB03_O_ENCODING               0x02
#define IDP7_FS_I_DAI_PB03_O_ENCODING               0x02
#define SRC0_FS_IP_I_DAI_PB03_O_ENCODING            0x02
#define SRC0_FS_0P_I_DAI_PB03_O_ENCODING            0x02
#define SRC1_FS_IP_I_DAI_PB03_O_ENCODING            0x02
#define SRC1_FS_OP_I_DAI_PB03_O_ENCODING            0x02
#define SRC2_FS_IP_I_DAI_PB03_O_ENCODING            0x02
#define SRC2_FS_OP_I_DAI_PB03_O_ENCODING            0x02
#define SRC3_FS_IP_I_DAI_PB03_O_ENCODING            0x02
#define SRC3_FS_OP_I_DAI_PB03_O_ENCODING            0x02
#define DIT_I_DAI_PB03_O_ENCODING                0x02
#define DIR_FS_I_DAI_PB03_O_ENCODING                0x02

#define SPORT0_FS_I_DAI_PB04_O_ENCODING             0x03
#define SPORT1_FS_I_DAI_PB04_O_ENCODING             0x03
#define SPORT2_FS_I_DAI_PB04_O_ENCODING             0x03
#define SPORT3_FS_I_DAI_PB04_O_ENCODING             0x03
#define SPORT4_FS_I_DAI_PB04_O_ENCODING             0x03
#define SPORT5_FS_I_DAI_PB04_O_ENCODING             0x03
#define IDP0_FS_I_DAI_PB04_O_ENCODING               0x03
#define PDAP_HOLD_I_DAI_PB04_O_ENCODING             0x03
#define IDP1_FS_I_DAI_PB04_O_ENCODING               0x03
#define IDP2_FS_I_DAI_PB04_O_ENCODING               0x03
#define IDP3_FS_I_DAI_PB04_O_ENCODING               0x03
#define IDP4_FS_I_DAI_PB04_O_ENCODING               0x03
#define IDP5_FS_I_DAI_PB04_O_ENCODING               0x03
#define IDP6_FS_I_DAI_PB04_O_ENCODING               0x03
#define IDP7_FS_I_DAI_PB04_O_ENCODING               0x03
#define SRC0_FS_IP_I_DAI_PB04_O_ENCODING            0x03
#define SRC0_FS_0P_I_DAI_PB04_O_ENCODING            0x03
#define SRC1_FS_IP_I_DAI_PB04_O_ENCODING            0x03
#define SRC1_FS_OP_I_DAI_PB04_O_ENCODING            0x03
#define SRC2_FS_IP_I_DAI_PB04_O_ENCODING            0x03
#define SRC2_FS_OP_I_DAI_PB04_O_ENCODING            0x03
#define SRC3_FS_IP_I_DAI_PB04_O_ENCODING            0x03
#define SRC3_FS_OP_I_DAI_PB04_O_ENCODING            0x03
#define DIT_I_DAI_PB04_O_ENCODING                0x03
#define DIR_FS_I_DAI_PB04_O_ENCODING                0x03

#define SPORT0_FS_I_DAI_PB05_O_ENCODING             0x04
#define SPORT1_FS_I_DAI_PB05_O_ENCODING             0x04
#define SPORT2_FS_I_DAI_PB05_O_ENCODING             0x04
#define SPORT3_FS_I_DAI_PB05_O_ENCODING             0x04
#define SPORT4_FS_I_DAI_PB05_O_ENCODING             0x04
#define SPORT5_FS_I_DAI_PB05_O_ENCODING             0x04
#define IDP0_FS_I_DAI_PB05_O_ENCODING               0x04
#define PDAP_HOLD_I_DAI_PB05_O_ENCODING             0x04
#define IDP1_FS_I_DAI_PB05_O_ENCODING               0x04
#define IDP2_FS_I_DAI_PB05_O_ENCODING               0x04
#define IDP3_FS_I_DAI_PB05_O_ENCODING               0x04
#define IDP4_FS_I_DAI_PB05_O_ENCODING               0x04
#define IDP5_FS_I_DAI_PB05_O_ENCODING               0x04
#define IDP6_FS_I_DAI_PB05_O_ENCODING               0x04
#define IDP7_FS_I_DAI_PB05_O_ENCODING               0x04
#define SRC0_FS_IP_I_DAI_PB05_O_ENCODING            0x04
#define SRC0_FS_0P_I_DAI_PB05_O_ENCODING            0x04
#define SRC1_FS_IP_I_DAI_PB05_O_ENCODING            0x04
#define SRC1_FS_OP_I_DAI_PB05_O_ENCODING            0x04
#define SRC2_FS_IP_I_DAI_PB05_O_ENCODING            0x04
#define SRC2_FS_OP_I_DAI_PB05_O_ENCODING            0x04
#define SRC3_FS_IP_I_DAI_PB05_O_ENCODING            0x04
#define SRC3_FS_OP_I_DAI_PB05_O_ENCODING            0x04
#define DIT_I_DAI_PB05_O_ENCODING                0x04
#define DIR_FS_I_DAI_PB05_O_ENCODING                0x04

#define SPORT0_FS_I_DAI_PB06_O_ENCODING             0x05
#define SPORT1_FS_I_DAI_PB06_O_ENCODING             0x05
#define SPORT2_FS_I_DAI_PB06_O_ENCODING             0x05
#define SPORT3_FS_I_DAI_PB06_O_ENCODING             0x05
#define SPORT4_FS_I_DAI_PB06_O_ENCODING             0x05
#define SPORT5_FS_I_DAI_PB06_O_ENCODING             0x05
#define IDP0_FS_I_DAI_PB06_O_ENCODING               0x05
#define PDAP_HOLD_I_DAI_PB06_O_ENCODING             0x05
#define IDP1_FS_I_DAI_PB06_O_ENCODING               0x05
#define IDP2_FS_I_DAI_PB06_O_ENCODING               0x05
#define IDP3_FS_I_DAI_PB06_O_ENCODING               0x05
#define IDP4_FS_I_DAI_PB06_O_ENCODING               0x05
#define IDP5_FS_I_DAI_PB06_O_ENCODING               0x05
#define IDP6_FS_I_DAI_PB06_O_ENCODING               0x05
#define IDP7_FS_I_DAI_PB06_O_ENCODING               0x05
#define SRC0_FS_IP_I_DAI_PB06_O_ENCODING            0x05
#define SRC0_FS_0P_I_DAI_PB06_O_ENCODING            0x05
#define SRC1_FS_IP_I_DAI_PB06_O_ENCODING            0x05
#define SRC1_FS_OP_I_DAI_PB06_O_ENCODING            0x05
#define SRC2_FS_IP_I_DAI_PB06_O_ENCODING            0x05
#define SRC2_FS_OP_I_DAI_PB06_O_ENCODING            0x05
#define SRC3_FS_IP_I_DAI_PB06_O_ENCODING            0x05
#define SRC3_FS_OP_I_DAI_PB06_O_ENCODING            0x05
#define DIT_I_DAI_PB06_O_ENCODING                0x05
#define DIR_FS_I_DAI_PB06_O_ENCODING                0x05

#define SPORT0_FS_I_DAI_PB07_O_ENCODING             0x06
#define SPORT1_FS_I_DAI_PB07_O_ENCODING             0x06
#define SPORT2_FS_I_DAI_PB07_O_ENCODING             0x06
#define SPORT3_FS_I_DAI_PB07_O_ENCODING             0x06
#define SPORT4_FS_I_DAI_PB07_O_ENCODING             0x06
#define SPORT5_FS_I_DAI_PB07_O_ENCODING             0x06
#define IDP0_FS_I_DAI_PB07_O_ENCODING               0x06
#define PDAP_HOLD_I_DAI_PB07_O_ENCODING             0x06
#define IDP1_FS_I_DAI_PB07_O_ENCODING               0x06
#define IDP2_FS_I_DAI_PB07_O_ENCODING               0x06
#define IDP3_FS_I_DAI_PB07_O_ENCODING               0x06
#define IDP4_FS_I_DAI_PB07_O_ENCODING               0x06
#define IDP5_FS_I_DAI_PB07_O_ENCODING               0x06
#define IDP6_FS_I_DAI_PB07_O_ENCODING               0x06
#define IDP7_FS_I_DAI_PB07_O_ENCODING               0x06
#define SRC0_FS_IP_I_DAI_PB07_O_ENCODING            0x06
#define SRC0_FS_0P_I_DAI_PB07_O_ENCODING            0x06
#define SRC1_FS_IP_I_DAI_PB07_O_ENCODING            0x06
#define SRC1_FS_OP_I_DAI_PB07_O_ENCODING            0x06
#define SRC2_FS_IP_I_DAI_PB07_O_ENCODING            0x06
#define SRC2_FS_OP_I_DAI_PB07_O_ENCODING            0x06
#define SRC3_FS_IP_I_DAI_PB07_O_ENCODING            0x06
#define SRC3_FS_OP_I_DAI_PB07_O_ENCODING            0x06
#define DIT_I_DAI_PB07_O_ENCODING                0x06
#define DIR_FS_I_DAI_PB07_O_ENCODING                0x06

#define SPORT0_FS_I_DAI_PB08_O_ENCODING             0x07
#define SPORT1_FS_I_DAI_PB08_O_ENCODING             0x07
#define SPORT2_FS_I_DAI_PB08_O_ENCODING             0x07
#define SPORT3_FS_I_DAI_PB08_O_ENCODING             0x07
#define SPORT4_FS_I_DAI_PB08_O_ENCODING             0x07
#define SPORT5_FS_I_DAI_PB08_O_ENCODING             0x07
#define IDP0_FS_I_DAI_PB08_O_ENCODING               0x07
#define PDAP_HOLD_I_DAI_PB08_O_ENCODING             0x07
#define IDP1_FS_I_DAI_PB08_O_ENCODING               0x07
#define IDP2_FS_I_DAI_PB08_O_ENCODING               0x07
#define IDP3_FS_I_DAI_PB08_O_ENCODING               0x07
#define IDP4_FS_I_DAI_PB08_O_ENCODING               0x07
#define IDP5_FS_I_DAI_PB08_O_ENCODING               0x07
#define IDP6_FS_I_DAI_PB08_O_ENCODING               0x07
#define IDP7_FS_I_DAI_PB08_O_ENCODING               0x07
#define SRC0_FS_IP_I_DAI_PB08_O_ENCODING            0x07
#define SRC0_FS_0P_I_DAI_PB08_O_ENCODING            0x07
#define SRC1_FS_IP_I_DAI_PB08_O_ENCODING            0x07
#define SRC1_FS_OP_I_DAI_PB08_O_ENCODING            0x07
#define SRC2_FS_IP_I_DAI_PB08_O_ENCODING            0x07
#define SRC2_FS_OP_I_DAI_PB08_O_ENCODING            0x07
#define SRC3_FS_IP_I_DAI_PB08_O_ENCODING            0x07
#define SRC3_FS_OP_I_DAI_PB08_O_ENCODING            0x07
#define DIT_I_DAI_PB08_O_ENCODING                0x07
#define DIR_FS_I_DAI_PB08_O_ENCODING                0x07

#define SPORT0_FS_I_DAI_PB09_O_ENCODING             0x08
#define SPORT1_FS_I_DAI_PB09_O_ENCODING             0x08
#define SPORT2_FS_I_DAI_PB09_O_ENCODING             0x08
#define SPORT3_FS_I_DAI_PB09_O_ENCODING             0x08
#define SPORT4_FS_I_DAI_PB09_O_ENCODING             0x08
#define SPORT5_FS_I_DAI_PB09_O_ENCODING             0x08
#define IDP0_FS_I_DAI_PB09_O_ENCODING               0x08
#define PDAP_HOLD_I_DAI_PB09_O_ENCODING             0x08
#define IDP1_FS_I_DAI_PB09_O_ENCODING               0x08
#define IDP2_FS_I_DAI_PB09_O_ENCODING               0x08
#define IDP3_FS_I_DAI_PB09_O_ENCODING               0x08
#define IDP4_FS_I_DAI_PB09_O_ENCODING               0x08
#define IDP5_FS_I_DAI_PB09_O_ENCODING               0x08
#define IDP6_FS_I_DAI_PB09_O_ENCODING               0x08
#define IDP7_FS_I_DAI_PB09_O_ENCODING               0x08
#define SRC0_FS_IP_I_DAI_PB09_O_ENCODING            0x08
#define SRC0_FS_0P_I_DAI_PB09_O_ENCODING            0x08
#define SRC1_FS_IP_I_DAI_PB09_O_ENCODING            0x08
#define SRC1_FS_OP_I_DAI_PB09_O_ENCODING            0x08
#define SRC2_FS_IP_I_DAI_PB09_O_ENCODING            0x08
#define SRC2_FS_OP_I_DAI_PB09_O_ENCODING            0x08
#define SRC3_FS_IP_I_DAI_PB09_O_ENCODING            0x08
#define SRC3_FS_OP_I_DAI_PB09_O_ENCODING            0x08
#define DIT_I_DAI_PB09_O_ENCODING                0x08
#define DIR_FS_I_DAI_PB09_O_ENCODING                0x08

#define SPORT0_FS_I_DAI_PB10_O_ENCODING             0x09
#define SPORT1_FS_I_DAI_PB10_O_ENCODING             0x09
#define SPORT2_FS_I_DAI_PB10_O_ENCODING             0x09
#define SPORT3_FS_I_DAI_PB10_O_ENCODING             0x09
#define SPORT4_FS_I_DAI_PB10_O_ENCODING             0x09
#define SPORT5_FS_I_DAI_PB10_O_ENCODING             0x09
#define IDP0_FS_I_DAI_PB10_O_ENCODING               0x09
#define PDAP_HOLD_I_DAI_PB10_O_ENCODING             0x09
#define IDP1_FS_I_DAI_PB10_O_ENCODING               0x09
#define IDP2_FS_I_DAI_PB10_O_ENCODING               0x09
#define IDP3_FS_I_DAI_PB10_O_ENCODING               0x09
#define IDP4_FS_I_DAI_PB10_O_ENCODING               0x09
#define IDP5_FS_I_DAI_PB10_O_ENCODING               0x09
#define IDP6_FS_I_DAI_PB10_O_ENCODING               0x09
#define IDP7_FS_I_DAI_PB10_O_ENCODING               0x09
#define SRC0_FS_IP_I_DAI_PB10_O_ENCODING            0x09
#define SRC0_FS_0P_I_DAI_PB10_O_ENCODING            0x09
#define SRC1_FS_IP_I_DAI_PB10_O_ENCODING            0x09
#define SRC1_FS_OP_I_DAI_PB10_O_ENCODING            0x09
#define SRC2_FS_IP_I_DAI_PB10_O_ENCODING            0x09
#define SRC2_FS_OP_I_DAI_PB10_O_ENCODING            0x09
#define SRC3_FS_IP_I_DAI_PB10_O_ENCODING            0x09
#define SRC3_FS_OP_I_DAI_PB10_O_ENCODING            0x09
#define DIT_I_DAI_PB10_O_ENCODING                0x09
#define DIR_FS_I_DAI_PB10_O_ENCODING                0x09

#define SPORT0_FS_I_DAI_PB11_O_ENCODING             0x0A
#define SPORT1_FS_I_DAI_PB11_O_ENCODING             0x0A
#define SPORT2_FS_I_DAI_PB11_O_ENCODING             0x0A
#define SPORT3_FS_I_DAI_PB11_O_ENCODING             0x0A
#define SPORT4_FS_I_DAI_PB11_O_ENCODING             0x0A
#define SPORT5_FS_I_DAI_PB11_O_ENCODING             0x0A
#define IDP0_FS_I_DAI_PB11_O_ENCODING               0x0A
#define PDAP_HOLD_I_DAI_PB11_O_ENCODING             0x0A
#define IDP1_FS_I_DAI_PB11_O_ENCODING               0x0A
#define IDP2_FS_I_DAI_PB11_O_ENCODING               0x0A
#define IDP3_FS_I_DAI_PB11_O_ENCODING               0x0A
#define IDP4_FS_I_DAI_PB11_O_ENCODING               0x0A
#define IDP5_FS_I_DAI_PB11_O_ENCODING               0x0A
#define IDP6_FS_I_DAI_PB11_O_ENCODING               0x0A
#define IDP7_FS_I_DAI_PB11_O_ENCODING               0x0A
#define SRC0_FS_IP_I_DAI_PB11_O_ENCODING            0x0A
#define SRC0_FS_0P_I_DAI_PB11_O_ENCODING            0x0A
#define SRC1_FS_IP_I_DAI_PB11_O_ENCODING            0x0A
#define SRC1_FS_OP_I_DAI_PB11_O_ENCODING            0x0A
#define SRC2_FS_IP_I_DAI_PB11_O_ENCODING            0x0A
#define SRC2_FS_OP_I_DAI_PB11_O_ENCODING            0x0A
#define SRC3_FS_IP_I_DAI_PB11_O_ENCODING            0x0A
#define SRC3_FS_OP_I_DAI_PB11_O_ENCODING            0x0A
#define DIT_I_DAI_PB11_O_ENCODING                0x0A
#define DIR_FS_I_DAI_PB11_O_ENCODING                0x0A

#define SPORT0_FS_I_DAI_PB12_O_ENCODING             0x0B
#define SPORT1_FS_I_DAI_PB12_O_ENCODING             0x0B
#define SPORT2_FS_I_DAI_PB12_O_ENCODING             0x0B
#define SPORT3_FS_I_DAI_PB12_O_ENCODING             0x0B
#define SPORT4_FS_I_DAI_PB12_O_ENCODING             0x0B
#define SPORT5_FS_I_DAI_PB12_O_ENCODING             0x0B
#define IDP0_FS_I_DAI_PB12_O_ENCODING               0x0B
#define PDAP_HOLD_I_DAI_PB12_O_ENCODING             0x0B
#define IDP1_FS_I_DAI_PB12_O_ENCODING               0x0B
#define IDP2_FS_I_DAI_PB12_O_ENCODING               0x0B
#define IDP3_FS_I_DAI_PB12_O_ENCODING               0x0B
#define IDP4_FS_I_DAI_PB12_O_ENCODING               0x0B
#define IDP5_FS_I_DAI_PB12_O_ENCODING               0x0B
#define IDP6_FS_I_DAI_PB12_O_ENCODING               0x0B
#define IDP7_FS_I_DAI_PB12_O_ENCODING               0x0B
#define SRC0_FS_IP_I_DAI_PB12_O_ENCODING            0x0B
#define SRC0_FS_0P_I_DAI_PB12_O_ENCODING            0x0B
#define SRC1_FS_IP_I_DAI_PB12_O_ENCODING            0x0B
#define SRC1_FS_OP_I_DAI_PB12_O_ENCODING            0x0B
#define SRC2_FS_IP_I_DAI_PB12_O_ENCODING            0x0B
#define SRC2_FS_OP_I_DAI_PB12_O_ENCODING            0x0B
#define SRC3_FS_IP_I_DAI_PB12_O_ENCODING            0x0B
#define SRC3_FS_OP_I_DAI_PB12_O_ENCODING            0x0B
#define DIT_I_DAI_PB12_O_ENCODING                0x0B
#define DIR_FS_I_DAI_PB12_O_ENCODING                0x0B

#define SPORT0_FS_I_DAI_PB13_O_ENCODING             0x0C
#define SPORT1_FS_I_DAI_PB13_O_ENCODING             0x0C
#define SPORT2_FS_I_DAI_PB13_O_ENCODING             0x0C
#define SPORT3_FS_I_DAI_PB13_O_ENCODING             0x0C
#define SPORT4_FS_I_DAI_PB13_O_ENCODING             0x0C
#define SPORT5_FS_I_DAI_PB13_O_ENCODING             0x0C
#define IDP0_FS_I_DAI_PB13_O_ENCODING               0x0C
#define PDAP_HOLD_I_DAI_PB13_O_ENCODING             0x0C
#define IDP1_FS_I_DAI_PB13_O_ENCODING               0x0C
#define IDP2_FS_I_DAI_PB13_O_ENCODING               0x0C
#define IDP3_FS_I_DAI_PB13_O_ENCODING               0x0C
#define IDP4_FS_I_DAI_PB13_O_ENCODING               0x0C
#define IDP5_FS_I_DAI_PB13_O_ENCODING               0x0C
#define IDP6_FS_I_DAI_PB13_O_ENCODING               0x0C
#define IDP7_FS_I_DAI_PB13_O_ENCODING               0x0C
#define SRC0_FS_IP_I_DAI_PB13_O_ENCODING            0x0C
#define SRC0_FS_0P_I_DAI_PB13_O_ENCODING            0x0C
#define SRC1_FS_IP_I_DAI_PB13_O_ENCODING            0x0C
#define SRC1_FS_OP_I_DAI_PB13_O_ENCODING            0x0C
#define SRC2_FS_IP_I_DAI_PB13_O_ENCODING            0x0C
#define SRC2_FS_OP_I_DAI_PB13_O_ENCODING            0x0C
#define SRC3_FS_IP_I_DAI_PB13_O_ENCODING            0x0C
#define SRC3_FS_OP_I_DAI_PB13_O_ENCODING            0x0C
#define DIT_I_DAI_PB13_O_ENCODING                0x0C
#define DIR_FS_I_DAI_PB13_O_ENCODING                0x0C

#define SPORT0_FS_I_DAI_PB14_O_ENCODING             0x0D
#define SPORT1_FS_I_DAI_PB14_O_ENCODING             0x0D
#define SPORT2_FS_I_DAI_PB14_O_ENCODING             0x0D
#define SPORT3_FS_I_DAI_PB14_O_ENCODING             0x0D
#define SPORT4_FS_I_DAI_PB14_O_ENCODING             0x0D
#define SPORT5_FS_I_DAI_PB14_O_ENCODING             0x0D
#define IDP0_FS_I_DAI_PB14_O_ENCODING               0x0D
#define PDAP_HOLD_I_DAI_PB14_O_ENCODING             0x0D
#define IDP1_FS_I_DAI_PB14_O_ENCODING               0x0D
#define IDP2_FS_I_DAI_PB14_O_ENCODING               0x0D
#define IDP3_FS_I_DAI_PB14_O_ENCODING               0x0D
#define IDP4_FS_I_DAI_PB14_O_ENCODING               0x0D
#define IDP5_FS_I_DAI_PB14_O_ENCODING               0x0D
#define IDP6_FS_I_DAI_PB14_O_ENCODING               0x0D
#define IDP7_FS_I_DAI_PB14_O_ENCODING               0x0D
#define SRC0_FS_IP_I_DAI_PB14_O_ENCODING            0x0D
#define SRC0_FS_0P_I_DAI_PB14_O_ENCODING            0x0D
#define SRC1_FS_IP_I_DAI_PB14_O_ENCODING            0x0D
#define SRC1_FS_OP_I_DAI_PB14_O_ENCODING            0x0D
#define SRC2_FS_IP_I_DAI_PB14_O_ENCODING            0x0D
#define SRC2_FS_OP_I_DAI_PB14_O_ENCODING            0x0D
#define SRC3_FS_IP_I_DAI_PB14_O_ENCODING            0x0D
#define SRC3_FS_OP_I_DAI_PB14_O_ENCODING            0x0D
#define DIT_I_DAI_PB14_O_ENCODING                0x0D
#define DIR_FS_I_DAI_PB14_O_ENCODING                0x0D

#define SPORT0_FS_I_DAI_PB15_O_ENCODING             0x0E
#define SPORT1_FS_I_DAI_PB15_O_ENCODING             0x0E
#define SPORT2_FS_I_DAI_PB15_O_ENCODING             0x0E
#define SPORT3_FS_I_DAI_PB15_O_ENCODING             0x0E
#define SPORT4_FS_I_DAI_PB15_O_ENCODING             0x0E
#define SPORT5_FS_I_DAI_PB15_O_ENCODING             0x0E
#define IDP0_FS_I_DAI_PB15_O_ENCODING               0x0E
#define PDAP_HOLD_I_DAI_PB15_O_ENCODING             0x0E
#define IDP1_FS_I_DAI_PB15_O_ENCODING               0x0E
#define IDP2_FS_I_DAI_PB15_O_ENCODING               0x0E
#define IDP3_FS_I_DAI_PB15_O_ENCODING               0x0E
#define IDP4_FS_I_DAI_PB15_O_ENCODING               0x0E
#define IDP5_FS_I_DAI_PB15_O_ENCODING               0x0E
#define IDP6_FS_I_DAI_PB15_O_ENCODING               0x0E
#define IDP7_FS_I_DAI_PB15_O_ENCODING               0x0E
#define SRC0_FS_IP_I_DAI_PB15_O_ENCODING            0x0E
#define SRC0_FS_0P_I_DAI_PB15_O_ENCODING            0x0E
#define SRC1_FS_IP_I_DAI_PB15_O_ENCODING            0x0E
#define SRC1_FS_OP_I_DAI_PB15_O_ENCODING            0x0E
#define SRC2_FS_IP_I_DAI_PB15_O_ENCODING            0x0E
#define SRC2_FS_OP_I_DAI_PB15_O_ENCODING            0x0E
#define SRC3_FS_IP_I_DAI_PB15_O_ENCODING            0x0E
#define SRC3_FS_OP_I_DAI_PB15_O_ENCODING            0x0E
#define DIT_I_DAI_PB15_O_ENCODING                0x0E
#define DIR_FS_I_DAI_PB15_O_ENCODING                0x0E

#define SPORT0_FS_I_DAI_PB16_O_ENCODING             0x0F
#define SPORT1_FS_I_DAI_PB16_O_ENCODING             0x0F
#define SPORT2_FS_I_DAI_PB16_O_ENCODING             0x0F
#define SPORT3_FS_I_DAI_PB16_O_ENCODING             0x0F
#define SPORT4_FS_I_DAI_PB16_O_ENCODING             0x0F
#define SPORT5_FS_I_DAI_PB16_O_ENCODING             0x0F
#define IDP0_FS_I_DAI_PB16_O_ENCODING               0x0F
#define PDAP_HOLD_I_DAI_PB16_O_ENCODING             0x0F
#define IDP1_FS_I_DAI_PB16_O_ENCODING               0x0F
#define IDP2_FS_I_DAI_PB16_O_ENCODING               0x0F
#define IDP3_FS_I_DAI_PB16_O_ENCODING               0x0F
#define IDP4_FS_I_DAI_PB16_O_ENCODING               0x0F
#define IDP5_FS_I_DAI_PB16_O_ENCODING               0x0F
#define IDP6_FS_I_DAI_PB16_O_ENCODING               0x0F
#define IDP7_FS_I_DAI_PB16_O_ENCODING               0x0F
#define SRC0_FS_IP_I_DAI_PB16_O_ENCODING            0x0F
#define SRC0_FS_0P_I_DAI_PB16_O_ENCODING            0x0F
#define SRC1_FS_IP_I_DAI_PB16_O_ENCODING            0x0F
#define SRC1_FS_OP_I_DAI_PB16_O_ENCODING            0x0F
#define SRC2_FS_IP_I_DAI_PB16_O_ENCODING            0x0F
#define SRC2_FS_OP_I_DAI_PB16_O_ENCODING            0x0F
#define SRC3_FS_IP_I_DAI_PB16_O_ENCODING            0x0F
#define SRC3_FS_OP_I_DAI_PB16_O_ENCODING            0x0F
#define DIT_I_DAI_PB16_O_ENCODING                0x0F
#define DIR_FS_I_DAI_PB16_O_ENCODING                0x0F

#define SPORT0_FS_I_DAI_PB17_O_ENCODING             0x10
#define SPORT1_FS_I_DAI_PB17_O_ENCODING             0x10
#define SPORT2_FS_I_DAI_PB17_O_ENCODING             0x10
#define SPORT3_FS_I_DAI_PB17_O_ENCODING             0x10
#define SPORT4_FS_I_DAI_PB17_O_ENCODING             0x10
#define SPORT5_FS_I_DAI_PB17_O_ENCODING             0x10
#define IDP0_FS_I_DAI_PB17_O_ENCODING               0x10
#define PDAP_HOLD_I_DAI_PB17_O_ENCODING             0x10
#define IDP1_FS_I_DAI_PB17_O_ENCODING               0x10
#define IDP2_FS_I_DAI_PB17_O_ENCODING               0x10
#define IDP3_FS_I_DAI_PB17_O_ENCODING               0x10
#define IDP4_FS_I_DAI_PB17_O_ENCODING               0x10
#define IDP5_FS_I_DAI_PB17_O_ENCODING               0x10
#define IDP6_FS_I_DAI_PB17_O_ENCODING               0x10
#define IDP7_FS_I_DAI_PB17_O_ENCODING               0x10
#define SRC0_FS_IP_I_DAI_PB17_O_ENCODING            0x10
#define SRC0_FS_0P_I_DAI_PB17_O_ENCODING            0x10
#define SRC1_FS_IP_I_DAI_PB17_O_ENCODING            0x10
#define SRC1_FS_OP_I_DAI_PB17_O_ENCODING            0x10
#define SRC2_FS_IP_I_DAI_PB17_O_ENCODING            0x10
#define SRC2_FS_OP_I_DAI_PB17_O_ENCODING            0x10
#define SRC3_FS_IP_I_DAI_PB17_O_ENCODING            0x10
#define SRC3_FS_OP_I_DAI_PB17_O_ENCODING            0x10
#define DIT_I_DAI_PB17_O_ENCODING                0x10
#define DIR_FS_I_DAI_PB17_O_ENCODING                0x10

#define SPORT0_FS_I_DAI_PB18_O_ENCODING             0x11
#define SPORT1_FS_I_DAI_PB18_O_ENCODING             0x11
#define SPORT2_FS_I_DAI_PB18_O_ENCODING             0x11
#define SPORT3_FS_I_DAI_PB18_O_ENCODING             0x11
#define SPORT4_FS_I_DAI_PB18_O_ENCODING             0x11
#define SPORT5_FS_I_DAI_PB18_O_ENCODING             0x11
#define IDP0_FS_I_DAI_PB18_O_ENCODING               0x11
#define PDAP_HOLD_I_DAI_PB18_O_ENCODING             0x11
#define IDP1_FS_I_DAI_PB18_O_ENCODING               0x11
#define IDP2_FS_I_DAI_PB18_O_ENCODING               0x11
#define IDP3_FS_I_DAI_PB18_O_ENCODING               0x11
#define IDP4_FS_I_DAI_PB18_O_ENCODING               0x11
#define IDP5_FS_I_DAI_PB18_O_ENCODING               0x11
#define IDP6_FS_I_DAI_PB18_O_ENCODING               0x11
#define IDP7_FS_I_DAI_PB18_O_ENCODING               0x11
#define SRC0_FS_IP_I_DAI_PB18_O_ENCODING            0x11
#define SRC0_FS_0P_I_DAI_PB18_O_ENCODING            0x11
#define SRC1_FS_IP_I_DAI_PB18_O_ENCODING            0x11
#define SRC1_FS_OP_I_DAI_PB18_O_ENCODING            0x11
#define SRC2_FS_IP_I_DAI_PB18_O_ENCODING            0x11
#define SRC2_FS_OP_I_DAI_PB18_O_ENCODING            0x11
#define SRC3_FS_IP_I_DAI_PB18_O_ENCODING            0x11
#define SRC3_FS_OP_I_DAI_PB18_O_ENCODING            0x11
#define DIT_I_DAI_PB18_O_ENCODING                0x11
#define DIR_FS_I_DAI_PB18_O_ENCODING                0x11

#define SPORT0_FS_I_DAI_PB19_O_ENCODING             0x12
#define SPORT1_FS_I_DAI_PB19_O_ENCODING             0x12
#define SPORT2_FS_I_DAI_PB19_O_ENCODING             0x12
#define SPORT3_FS_I_DAI_PB19_O_ENCODING             0x12
#define SPORT4_FS_I_DAI_PB19_O_ENCODING             0x12
#define SPORT5_FS_I_DAI_PB19_O_ENCODING             0x12
#define IDP0_FS_I_DAI_PB19_O_ENCODING               0x12
#define PDAP_HOLD_I_DAI_PB19_O_ENCODING             0x12
#define IDP1_FS_I_DAI_PB19_O_ENCODING               0x12
#define IDP2_FS_I_DAI_PB19_O_ENCODING               0x12
#define IDP3_FS_I_DAI_PB19_O_ENCODING               0x12
#define IDP4_FS_I_DAI_PB19_O_ENCODING               0x12
#define IDP5_FS_I_DAI_PB19_O_ENCODING               0x12
#define IDP6_FS_I_DAI_PB19_O_ENCODING               0x12
#define IDP7_FS_I_DAI_PB19_O_ENCODING               0x12
#define SRC0_FS_IP_I_DAI_PB19_O_ENCODING            0x12
#define SRC0_FS_0P_I_DAI_PB19_O_ENCODING            0x12
#define SRC1_FS_IP_I_DAI_PB19_O_ENCODING            0x12
#define SRC1_FS_OP_I_DAI_PB19_O_ENCODING            0x12
#define SRC2_FS_IP_I_DAI_PB19_O_ENCODING            0x12
#define SRC2_FS_OP_I_DAI_PB19_O_ENCODING            0x12
#define SRC3_FS_IP_I_DAI_PB19_O_ENCODING            0x12
#define SRC3_FS_OP_I_DAI_PB19_O_ENCODING            0x12
#define DIT_I_DAI_PB19_O_ENCODING                0x12
#define DIR_FS_I_DAI_PB19_O_ENCODING                0x12

#define SPORT0_FS_I_DAI_PB20_O_ENCODING             0x13
#define SPORT1_FS_I_DAI_PB20_O_ENCODING             0x13
#define SPORT2_FS_I_DAI_PB20_O_ENCODING             0x13
#define SPORT3_FS_I_DAI_PB20_O_ENCODING             0x13
#define SPORT4_FS_I_DAI_PB20_O_ENCODING             0x13
#define SPORT5_FS_I_DAI_PB20_O_ENCODING             0x13
#define IDP0_FS_I_DAI_PB20_O_ENCODING               0x13
#define PDAP_HOLD_I_DAI_PB20_O_ENCODING             0x13
#define IDP1_FS_I_DAI_PB20_O_ENCODING               0x13
#define IDP2_FS_I_DAI_PB20_O_ENCODING               0x13
#define IDP3_FS_I_DAI_PB20_O_ENCODING               0x13
#define IDP4_FS_I_DAI_PB20_O_ENCODING               0x13
#define IDP5_FS_I_DAI_PB20_O_ENCODING               0x13
#define IDP6_FS_I_DAI_PB20_O_ENCODING               0x13
#define IDP7_FS_I_DAI_PB20_O_ENCODING               0x13
#define SRC0_FS_IP_I_DAI_PB20_O_ENCODING            0x13
#define SRC0_FS_0P_I_DAI_PB20_O_ENCODING            0x13
#define SRC1_FS_IP_I_DAI_PB20_O_ENCODING            0x13
#define SRC1_FS_OP_I_DAI_PB20_O_ENCODING            0x13
#define SRC2_FS_IP_I_DAI_PB20_O_ENCODING            0x13
#define SRC2_FS_OP_I_DAI_PB20_O_ENCODING            0x13
#define SRC3_FS_IP_I_DAI_PB20_O_ENCODING            0x13
#define SRC3_FS_OP_I_DAI_PB20_O_ENCODING            0x13
#define DIT_I_DAI_PB20_O_ENCODING                0x13
#define DIR_FS_I_DAI_PB20_O_ENCODING                0x13

#define SPORT0_FS_I_SPORT0_FS_O_ENCODING            0x14
#define SPORT1_FS_I_SPORT0_FS_O_ENCODING            0x14
#define SPORT2_FS_I_SPORT0_FS_O_ENCODING            0x14
#define SPORT3_FS_I_SPORT0_FS_O_ENCODING            0x14
#define SPORT4_FS_I_SPORT0_FS_O_ENCODING            0x14
#define SPORT5_FS_I_SPORT0_FS_O_ENCODING            0x14
#define IDP0_FS_I_SPORT0_FS_O_ENCODING              0x14
#define PDAP_HOLD_I_SPORT0_FS_O_ENCODING            0x14
#define IDP1_FS_I_SPORT0_FS_O_ENCODING              0x14
#define IDP2_FS_I_SPORT0_FS_O_ENCODING              0x14
#define IDP3_FS_I_SPORT0_FS_O_ENCODING              0x14
#define IDP4_FS_I_SPORT0_FS_O_ENCODING              0x14
#define IDP5_FS_I_SPORT0_FS_O_ENCODING              0x14
#define IDP6_FS_I_SPORT0_FS_O_ENCODING              0x14
#define IDP7_FS_I_SPORT0_FS_O_ENCODING              0x14
#define SRC0_FS_IP_I_SPORT0_FS_O_ENCODING           0x14
#define SRC0_FS_0P_I_SPORT0_FS_O_ENCODING           0x14
#define SRC1_FS_IP_I_SPORT0_FS_O_ENCODING           0x14
#define SRC1_FS_OP_I_SPORT0_FS_O_ENCODING           0x14
#define SRC2_FS_IP_I_SPORT0_FS_O_ENCODING           0x14
#define SRC2_FS_OP_I_SPORT0_FS_O_ENCODING           0x14
#define SRC3_FS_IP_I_SPORT0_FS_O_ENCODING           0x14
#define SRC3_FS_OP_I_SPORT0_FS_O_ENCODING           0x14
#define DIT_I_SPORT0_FS_O_ENCODING               0x14
#define DIR_FS_I_SPORT0_FS_O_ENCODING               0x14

#define SPORT0_FS_I_SPORT1_FS_O_ENCODING            0x15
#define SPORT1_FS_I_SPORT1_FS_O_ENCODING            0x15
#define SPORT2_FS_I_SPORT1_FS_O_ENCODING            0x15
#define SPORT3_FS_I_SPORT1_FS_O_ENCODING            0x15
#define SPORT4_FS_I_SPORT1_FS_O_ENCODING            0x15
#define SPORT5_FS_I_SPORT1_FS_O_ENCODING            0x15
#define IDP0_FS_I_SPORT1_FS_O_ENCODING              0x15
#define PDAP_HOLD_I_SPORT1_FS_O_ENCODING            0x15
#define IDP1_FS_I_SPORT1_FS_O_ENCODING              0x15
#define IDP2_FS_I_SPORT1_FS_O_ENCODING              0x15
#define IDP3_FS_I_SPORT1_FS_O_ENCODING              0x15
#define IDP4_FS_I_SPORT1_FS_O_ENCODING              0x15
#define IDP5_FS_I_SPORT1_FS_O_ENCODING              0x15
#define IDP6_FS_I_SPORT1_FS_O_ENCODING              0x15
#define IDP7_FS_I_SPORT1_FS_O_ENCODING              0x15
#define SRC0_FS_IP_I_SPORT1_FS_O_ENCODING           0x15
#define SRC0_FS_0P_I_SPORT1_FS_O_ENCODING           0x15
#define SRC1_FS_IP_I_SPORT1_FS_O_ENCODING           0x15
#define SRC1_FS_OP_I_SPORT1_FS_O_ENCODING           0x15
#define SRC2_FS_IP_I_SPORT1_FS_O_ENCODING           0x15
#define SRC2_FS_OP_I_SPORT1_FS_O_ENCODING           0x15
#define SRC3_FS_IP_I_SPORT1_FS_O_ENCODING           0x15
#define SRC3_FS_OP_I_SPORT1_FS_O_ENCODING           0x15
#define DIT_I_SPORT1_FS_O_ENCODING               0x15
#define DIR_FS_I_SPORT1_FS_O_ENCODING               0x15

#define SPORT0_FS_I_SPORT2_FS_O_ENCODING            0x16
#define SPORT1_FS_I_SPORT2_FS_O_ENCODING            0x16
#define SPORT2_FS_I_SPORT2_FS_O_ENCODING            0x16
#define SPORT3_FS_I_SPORT2_FS_O_ENCODING            0x16
#define SPORT4_FS_I_SPORT2_FS_O_ENCODING            0x16
#define SPORT5_FS_I_SPORT2_FS_O_ENCODING            0x16
#define IDP0_FS_I_SPORT2_FS_O_ENCODING              0x16
#define PDAP_HOLD_I_SPORT2_FS_O_ENCODING            0x16
#define IDP1_FS_I_SPORT2_FS_O_ENCODING              0x16
#define IDP2_FS_I_SPORT2_FS_O_ENCODING              0x16
#define IDP3_FS_I_SPORT2_FS_O_ENCODING              0x16
#define IDP4_FS_I_SPORT2_FS_O_ENCODING              0x16
#define IDP5_FS_I_SPORT2_FS_O_ENCODING              0x16
#define IDP6_FS_I_SPORT2_FS_O_ENCODING              0x16
#define IDP7_FS_I_SPORT2_FS_O_ENCODING              0x16
#define SRC0_FS_IP_I_SPORT2_FS_O_ENCODING           0x16
#define SRC0_FS_0P_I_SPORT2_FS_O_ENCODING           0x16
#define SRC1_FS_IP_I_SPORT2_FS_O_ENCODING           0x16
#define SRC1_FS_OP_I_SPORT2_FS_O_ENCODING           0x16
#define SRC2_FS_IP_I_SPORT2_FS_O_ENCODING           0x16
#define SRC2_FS_OP_I_SPORT2_FS_O_ENCODING           0x16
#define SRC3_FS_IP_I_SPORT2_FS_O_ENCODING           0x16
#define SRC3_FS_OP_I_SPORT2_FS_O_ENCODING           0x16
#define DIT_I_SPORT2_FS_O_ENCODING               0x16
#define DIR_FS_I_SPORT2_FS_O_ENCODING               0x16

#define SPORT0_FS_I_SPORT3_FS_O_ENCODING            0x17
#define SPORT1_FS_I_SPORT3_FS_O_ENCODING            0x17
#define SPORT2_FS_I_SPORT3_FS_O_ENCODING            0x17
#define SPORT3_FS_I_SPORT3_FS_O_ENCODING            0x17
#define SPORT4_FS_I_SPORT3_FS_O_ENCODING            0x17
#define SPORT5_FS_I_SPORT3_FS_O_ENCODING            0x17
#define IDP0_FS_I_SPORT3_FS_O_ENCODING              0x17
#define PDAP_HOLD_I_SPORT3_FS_O_ENCODING            0x17
#define IDP1_FS_I_SPORT3_FS_O_ENCODING              0x17
#define IDP2_FS_I_SPORT3_FS_O_ENCODING              0x17
#define IDP3_FS_I_SPORT3_FS_O_ENCODING              0x17
#define IDP4_FS_I_SPORT3_FS_O_ENCODING              0x17
#define IDP5_FS_I_SPORT3_FS_O_ENCODING              0x17
#define IDP6_FS_I_SPORT3_FS_O_ENCODING              0x17
#define IDP7_FS_I_SPORT3_FS_O_ENCODING              0x17
#define SRC0_FS_IP_I_SPORT3_FS_O_ENCODING           0x17
#define SRC0_FS_0P_I_SPORT3_FS_O_ENCODING           0x17
#define SRC1_FS_IP_I_SPORT3_FS_O_ENCODING           0x17
#define SRC1_FS_OP_I_SPORT3_FS_O_ENCODING           0x17
#define SRC2_FS_IP_I_SPORT3_FS_O_ENCODING           0x17
#define SRC2_FS_OP_I_SPORT3_FS_O_ENCODING           0x17
#define SRC3_FS_IP_I_SPORT3_FS_O_ENCODING           0x17
#define SRC3_FS_OP_I_SPORT3_FS_O_ENCODING           0x17
#define DIT_I_SPORT3_FS_O_ENCODING               0x17
#define DIR_FS_I_SPORT3_FS_O_ENCODING               0x17

#define SPORT0_FS_I_SPORT4_FS_O_ENCODING            0x18
#define SPORT1_FS_I_SPORT4_FS_O_ENCODING            0x18
#define SPORT2_FS_I_SPORT4_FS_O_ENCODING            0x18
#define SPORT3_FS_I_SPORT4_FS_O_ENCODING            0x18
#define SPORT4_FS_I_SPORT4_FS_O_ENCODING            0x18
#define SPORT5_FS_I_SPORT4_FS_O_ENCODING            0x18
#define IDP0_FS_I_SPORT4_FS_O_ENCODING              0x18
#define PDAP_HOLD_I_SPORT4_FS_O_ENCODING            0x18
#define IDP1_FS_I_SPORT4_FS_O_ENCODING              0x18
#define IDP2_FS_I_SPORT4_FS_O_ENCODING              0x18
#define IDP3_FS_I_SPORT4_FS_O_ENCODING              0x18
#define IDP4_FS_I_SPORT4_FS_O_ENCODING              0x18
#define IDP5_FS_I_SPORT4_FS_O_ENCODING              0x18
#define IDP6_FS_I_SPORT4_FS_O_ENCODING              0x18
#define IDP7_FS_I_SPORT4_FS_O_ENCODING              0x18
#define SRC0_FS_IP_I_SPORT4_FS_O_ENCODING           0x18
#define SRC0_FS_0P_I_SPORT4_FS_O_ENCODING           0x18
#define SRC1_FS_IP_I_SPORT4_FS_O_ENCODING           0x18
#define SRC1_FS_OP_I_SPORT4_FS_O_ENCODING           0x18
#define SRC2_FS_IP_I_SPORT4_FS_O_ENCODING           0x18
#define SRC2_FS_OP_I_SPORT4_FS_O_ENCODING           0x18
#define SRC3_FS_IP_I_SPORT4_FS_O_ENCODING           0x18
#define SRC3_FS_OP_I_SPORT4_FS_O_ENCODING           0x18
#define DIT_I_SPORT4_FS_O_ENCODING               0x18
#define DIR_FS_I_SPORT4_FS_O_ENCODING               0x18

#define SPORT0_FS_I_SPORT5_FS_O_ENCODING            0x19
#define SPORT1_FS_I_SPORT5_FS_O_ENCODING            0x19
#define SPORT2_FS_I_SPORT5_FS_O_ENCODING            0x19
#define SPORT3_FS_I_SPORT5_FS_O_ENCODING            0x19
#define SPORT4_FS_I_SPORT5_FS_O_ENCODING            0x19
#define SPORT5_FS_I_SPORT5_FS_O_ENCODING            0x19
#define IDP0_FS_I_SPORT5_FS_O_ENCODING              0x19
#define PDAP_HOLD_I_SPORT5_FS_O_ENCODING            0x19
#define IDP1_FS_I_SPORT5_FS_O_ENCODING              0x19
#define IDP2_FS_I_SPORT5_FS_O_ENCODING              0x19
#define IDP3_FS_I_SPORT5_FS_O_ENCODING              0x19
#define IDP4_FS_I_SPORT5_FS_O_ENCODING              0x19
#define IDP5_FS_I_SPORT5_FS_O_ENCODING              0x19
#define IDP6_FS_I_SPORT5_FS_O_ENCODING              0x19
#define IDP7_FS_I_SPORT5_FS_O_ENCODING              0x19
#define SRC0_FS_IP_I_SPORT5_FS_O_ENCODING           0x19
#define SRC0_FS_0P_I_SPORT5_FS_O_ENCODING           0x19
#define SRC1_FS_IP_I_SPORT5_FS_O_ENCODING           0x19
#define SRC1_FS_OP_I_SPORT5_FS_O_ENCODING           0x19
#define SRC2_FS_IP_I_SPORT5_FS_O_ENCODING           0x19
#define SRC2_FS_OP_I_SPORT5_FS_O_ENCODING           0x19
#define SRC3_FS_IP_I_SPORT5_FS_O_ENCODING           0x19
#define SRC3_FS_OP_I_SPORT5_FS_O_ENCODING           0x19
#define DIT_I_SPORT5_FS_O_ENCODING               0x19
#define DIR_FS_I_SPORT5_FS_O_ENCODING               0x19

#define SPORT0_FS_I_DIR_FS_O_ENCODING               0x1A
#define SPORT1_FS_I_DIR_FS_O_ENCODING               0x1A
#define SPORT2_FS_I_DIR_FS_O_ENCODING               0x1A
#define SPORT3_FS_I_DIR_FS_O_ENCODING               0x1A
#define SPORT4_FS_I_DIR_FS_O_ENCODING               0x1A
#define SPORT5_FS_I_DIR_FS_O_ENCODING               0x1A
#define IDP0_FS_I_DIR_FS_O_ENCODING                 0x1A
#define PDAP_HOLD_I_DIR_FS_O_ENCODING               0x1A
#define IDP1_FS_I_DIR_FS_O_ENCODING                 0x1A
#define IDP2_FS_I_DIR_FS_O_ENCODING                 0x1A
#define IDP3_FS_I_DIR_FS_O_ENCODING                 0x1A
#define IDP4_FS_I_DIR_FS_O_ENCODING                 0x1A
#define IDP5_FS_I_DIR_FS_O_ENCODING                 0x1A
#define IDP6_FS_I_DIR_FS_O_ENCODING                 0x1A
#define IDP7_FS_I_DIR_FS_O_ENCODING                 0x1A
#define SRC0_FS_IP_I_DIR_FS_O_ENCODING              0x1A
#define SRC0_FS_0P_I_DIR_FS_O_ENCODING              0x1A
#define SRC1_FS_IP_I_DIR_FS_O_ENCODING              0x1A
#define SRC1_FS_OP_I_DIR_FS_O_ENCODING              0x1A
#define SRC2_FS_IP_I_DIR_FS_O_ENCODING              0x1A
#define SRC2_FS_OP_I_DIR_FS_O_ENCODING              0x1A
#define SRC3_FS_IP_I_DIR_FS_O_ENCODING              0x1A
#define SRC3_FS_OP_I_DIR_FS_O_ENCODING              0x1A
#define DIT_I_DIR_FS_O_ENCODING                  0x1A
#define DIR_FS_I_DIR_FS_O_ENCODING                  0x1A
                                                
#define SPORT0_FS_I_DIT_O_ENCODING               0x1B
#define SPORT1_FS_I_DIT_O_ENCODING               0x1B
#define SPORT2_FS_I_DIT_O_ENCODING               0x1B
#define SPORT3_FS_I_DIT_O_ENCODING               0x1B
#define SPORT4_FS_I_DIT_O_ENCODING               0x1B
#define SPORT5_FS_I_DIT_O_ENCODING               0x1B
#define IDP0_FS_I_DIT_O_ENCODING                 0x1B
#define PDAP_HOLD_I_DIT_O_ENCODING               0x1B
#define IDP1_FS_I_DIT_O_ENCODING                 0x1B
#define IDP2_FS_I_DIT_O_ENCODING                 0x1B
#define IDP3_FS_I_DIT_O_ENCODING                 0x1B
#define IDP4_FS_I_DIT_O_ENCODING                 0x1B
#define IDP5_FS_I_DIT_O_ENCODING                 0x1B
#define IDP6_FS_I_DIT_O_ENCODING                 0x1B
#define IDP7_FS_I_DIT_O_ENCODING                 0x1B
#define SRC0_FS_IP_I_DIT_O_ENCODING              0x1B
#define SRC0_FS_0P_I_DIT_O_ENCODING              0x1B
#define SRC1_FS_IP_I_DIT_O_ENCODING              0x1B
#define SRC1_FS_OP_I_DIT_O_ENCODING              0x1B
#define SRC2_FS_IP_I_DIT_O_ENCODING              0x1B
#define SRC2_FS_OP_I_DIT_O_ENCODING              0x1B
#define SRC3_FS_IP_I_DIT_O_ENCODING              0x1B
#define SRC3_FS_OP_I_DIT_O_ENCODING              0x1B
#define DIT_I_DIT_O_ENCODING                  0x1B
#define DIR_FS_I_DIT_O_ENCODING                  0x1B

#define SPORT0_FS_I_PCG_FSA_O_ENCODING              0x1C
#define SPORT1_FS_I_PCG_FSA_O_ENCODING              0x1C
#define SPORT2_FS_I_PCG_FSA_O_ENCODING              0x1C
#define SPORT3_FS_I_PCG_FSA_O_ENCODING              0x1C
#define SPORT4_FS_I_PCG_FSA_O_ENCODING              0x1C
#define SPORT5_FS_I_PCG_FSA_O_ENCODING              0x1C
#define IDP0_FS_I_PCG_FSA_O_ENCODING                0x1C
#define PDAP_HOLD_I_PCG_FSA_O_ENCODING              0x1C
#define IDP1_FS_I_PCG_FSA_O_ENCODING                0x1C
#define IDP2_FS_I_PCG_FSA_O_ENCODING                0x1C
#define IDP3_FS_I_PCG_FSA_O_ENCODING                0x1C
#define IDP4_FS_I_PCG_FSA_O_ENCODING                0x1C
#define IDP5_FS_I_PCG_FSA_O_ENCODING                0x1C
#define IDP6_FS_I_PCG_FSA_O_ENCODING                0x1C
#define IDP7_FS_I_PCG_FSA_O_ENCODING                0x1C
#define SRC0_FS_IP_I_PCG_FSA_O_ENCODING             0x1C
#define SRC0_FS_0P_I_PCG_FSA_O_ENCODING             0x1C
#define SRC1_FS_IP_I_PCG_FSA_O_ENCODING             0x1C
#define SRC1_FS_OP_I_PCG_FSA_O_ENCODING             0x1C
#define SRC2_FS_IP_I_PCG_FSA_O_ENCODING             0x1C
#define SRC2_FS_OP_I_PCG_FSA_O_ENCODING             0x1C
#define SRC3_FS_IP_I_PCG_FSA_O_ENCODING             0x1C
#define SRC3_FS_OP_I_PCG_FSA_O_ENCODING             0x1C
#define DIT_I_PCG_FSA_O_ENCODING                 0x1C
#define DIR_FS_I_PCG_FSA_O_ENCODING                 0x1C

#define SPORT0_FS_I_PCG_FSB_O_ENCODING              0x1D
#define SPORT1_FS_I_PCG_FSB_O_ENCODING              0x1D
#define SPORT2_FS_I_PCG_FSB_O_ENCODING              0x1D
#define SPORT3_FS_I_PCG_FSB_O_ENCODING              0x1D
#define SPORT4_FS_I_PCG_FSB_O_ENCODING              0x1D
#define SPORT5_FS_I_PCG_FSB_O_ENCODING              0x1D
#define IDP0_FS_I_PCG_FSB_O_ENCODING                0x1D
#define PDAP_HOLD_I_PCG_FSB_O_ENCODING              0x1D
#define IDP1_FS_I_PCG_FSB_O_ENCODING                0x1D
#define IDP2_FS_I_PCG_FSB_O_ENCODING                0x1D
#define IDP3_FS_I_PCG_FSB_O_ENCODING                0x1D
#define IDP4_FS_I_PCG_FSB_O_ENCODING                0x1D
#define IDP5_FS_I_PCG_FSB_O_ENCODING                0x1D
#define IDP6_FS_I_PCG_FSB_O_ENCODING                0x1D
#define IDP7_FS_I_PCG_FSB_O_ENCODING                0x1D
#define SRC0_FS_IP_I_PCG_FSB_O_ENCODING             0x1D
#define SRC0_FS_0P_I_PCG_FSB_O_ENCODING             0x1D
#define SRC1_FS_IP_I_PCG_FSB_O_ENCODING             0x1D
#define SRC1_FS_OP_I_PCG_FSB_O_ENCODING             0x1D
#define SRC2_FS_IP_I_PCG_FSB_O_ENCODING             0x1D
#define SRC2_FS_OP_I_PCG_FSB_O_ENCODING             0x1D
#define SRC3_FS_IP_I_PCG_FSB_O_ENCODING             0x1D
#define SRC3_FS_OP_I_PCG_FSB_O_ENCODING             0x1D
#define DIT_I_PCG_FSB_O_ENCODING                 0x1D
#define DIR_FS_I_PCG_FSB_O_ENCODING                 0x1D

#define SPORT0_FS_I_LOW_ENCODING                    0x1E
#define SPORT1_FS_I_LOW_ENCODING                    0x1E
#define SPORT2_FS_I_LOW_ENCODING                    0x1E
#define SPORT3_FS_I_LOW_ENCODING                    0x1E
#define SPORT4_FS_I_LOW_ENCODING                    0x1E
#define SPORT5_FS_I_LOW_ENCODING                    0x1E
#define IDP0_FS_I_LOW_ENCODING                      0x1E
#define PDAP_HOLD_I_LOW_ENCODING                    0x1E
#define IDP1_FS_I_LOW_ENCODING                      0x1E
#define IDP2_FS_I_LOW_ENCODING                      0x1E
#define IDP3_FS_I_LOW_ENCODING                      0x1E
#define IDP4_FS_I_LOW_ENCODING                      0x1E
#define IDP5_FS_I_LOW_ENCODING                      0x1E
#define IDP6_FS_I_LOW_ENCODING                      0x1E
#define IDP7_FS_I_LOW_ENCODING                      0x1E
#define SRC0_FS_IP_I_LOW_ENCODING       	    0x1D
#define SRC0_FS_0P_I_LOW_ENCODING       	    0x1D
#define SRC1_FS_IP_I_LOW_ENCODING       	    0x1D
#define SRC1_FS_OP_I_LOW_ENCODING       	    0x1D
#define SRC2_FS_IP_I_LOW_ENCODING       	    0x1D
#define SRC2_FS_OP_I_LOW_ENCODING       	    0x1D
#define SRC3_FS_IP_I_LOW_ENCODING       	    0x1D
#define SRC3_FS_OP_I_LOW_ENCODING       	    0x1D
#define DIT_I_LOW_O_ENCODING                     0x1D
#define DIR_FS_I_LOW_O_ENCODING                     0x1D

#define SPORT0_FS_I_HIGH_ENCODING                   0x1F
#define SPORT1_FS_I_HIGH_ENCODING                   0x1F
#define SPORT2_FS_I_HIGH_ENCODING                   0x1F
#define SPORT3_FS_I_HIGH_ENCODING                   0x1F
#define SPORT4_FS_I_HIGH_ENCODING                   0x1F
#define SPORT5_FS_I_HIGH_ENCODING                   0x1F
#define IDP0_FS_I_HIGH_ENCODING                     0x1F 
#define PDAP_HOLD_I_HIGH_ENCODING                   0x1F
#define IDP1_FS_I_HIGH_ENCODING                     0x1F
#define IDP2_FS_I_HIGH_ENCODING                     0x1F
#define IDP3_FS_I_HIGH_ENCODING                     0x1F
#define IDP4_FS_I_HIGH_ENCODING                     0x1F
#define IDP5_FS_I_HIGH_ENCODING                     0x1F
#define IDP6_FS_I_HIGH_ENCODING                     0x1F
#define IDP7_FS_I_HIGH_ENCODING                     0x1F
#define SRC0_FS_IP_I_HIGH_ENCODING                  0x1F
#define SRC0_FS_0P_I_HIGH_ENCODING                  0x1F
#define SRC1_FS_IP_I_HIGH_ENCODING                  0x1F
#define SRC1_FS_OP_I_HIGH_ENCODING                  0x1F
#define SRC2_FS_IP_I_HIGH_ENCODING                  0x1F
#define SRC2_FS_OP_I_HIGH_ENCODING                  0x1F
#define SRC3_FS_IP_I_HIGH_ENCODING                  0x1F
#define SRC3_FS_OP_I_HIGH_ENCODING                  0x1F
#define DIT_I_HIGH_ENCODING                      0x1F
#define DIR_FS_I_HIGH_ENCODING                      0x1F


//------------------------------------------------------------------------------
//  Group C: Register associations
//------------------------------------------------------------------------------

#define SPORT0_FS_I_DAI_PB01_O_REG        0x2450    /* SRU_FS0 */
#define SPORT1_FS_I_DAI_PB01_O_REG        0x2450    
#define SPORT2_FS_I_DAI_PB01_O_REG        0x2450    
#define SPORT3_FS_I_DAI_PB01_O_REG        0x2450    
#define SPORT4_FS_I_DAI_PB01_O_REG        0x2450    
#define SPORT5_FS_I_DAI_PB01_O_REG        0x2450    
#define SRC0_FS_IP_I_DAI_PB01_O_REG       0x2451    /* SRU_FS1 */
#define SRC0_FS_0P_I_DAI_PB01_O_REG       0x2451    
#define SRC1_FS_IP_I_DAI_PB01_O_REG       0x2451    
#define SRC1_FS_OP_I_DAI_PB01_O_REG       0x2451    
#define SRC2_FS_IP_I_DAI_PB01_O_REG       0x2451    
#define SRC2_FS_OP_I_DAI_PB01_O_REG       0x2451    
#define SRC3_FS_IP_I_DAI_PB01_O_REG       0x2452    /* SRU_FS2 */
#define SRC3_FS_OP_I_DAI_PB01_O_REG       0x2452    
#define DIT_I_DAI_PB01_O_REG           0x2452    
#define PDAP_HOLD_DAI_PB01_O_REG          0x2452    
#define IDP0_FS_I_DAI_PB01_O_REG          0x2452    
#define IDP1_FS_I_DAI_PB01_O_REG          0x2452    
#define IDP2_FS_I_DAI_PB01_O_REG          0x2452    
#define IDP3_FS_I_DAI_PB01_O_REG          0x2453    /* SRU_FS3 */
#define IDP4_FS_I_DAI_PB01_O_REG          0x2453    
#define IDP5_FS_I_DAI_PB01_O_REG          0x2453    
#define IDP6_FS_I_DAI_PB01_O_REG          0x2453    
#define IDP7_FS_I_DAI_PB01_O_REG          0x2453        
#define DIR_FS_I_DAI_PB01_O_REG           0x2453

#define SPORT0_FS_I_DAI_PB02_O_REG        0x2450    /* SRU_FS0 */
#define SPORT1_FS_I_DAI_PB02_O_REG        0x2450    
#define SPORT2_FS_I_DAI_PB02_O_REG        0x2450    
#define SPORT3_FS_I_DAI_PB02_O_REG        0x2450    
#define SPORT4_FS_I_DAI_PB02_O_REG        0x2450    
#define SPORT5_FS_I_DAI_PB02_O_REG        0x2450    
#define SRC0_FS_IP_I_DAI_PB02_O_REG       0x2451    /* SRU_FS1 */
#define SRC0_FS_0P_I_DAI_PB02_O_REG       0x2451    
#define SRC1_FS_IP_I_DAI_PB02_O_REG       0x2451    
#define SRC1_FS_OP_I_DAI_PB02_O_REG       0x2451    
#define SRC2_FS_IP_I_DAI_PB02_O_REG       0x2451    
#define SRC2_FS_OP_I_DAI_PB02_O_REG       0x2451    
#define SRC3_FS_IP_I_DAI_PB02_O_REG       0x2452    /* SRU_FS2 */
#define SRC3_FS_OP_I_DAI_PB02_O_REG       0x2452    
#define DIT_I_DAI_PB02_O_REG           0x2452    
#define PDAP_HOLD_DAI_PB02_O_REG          0x2452    
#define IDP0_FS_I_DAI_PB02_O_REG          0x2452    
#define IDP1_FS_I_DAI_PB02_O_REG          0x2452    
#define IDP2_FS_I_DAI_PB02_O_REG          0x2452    
#define IDP3_FS_I_DAI_PB02_O_REG          0x2453    /* SRU_FS3 */
#define IDP4_FS_I_DAI_PB02_O_REG          0x2453    
#define IDP5_FS_I_DAI_PB02_O_REG          0x2453    
#define IDP6_FS_I_DAI_PB02_O_REG          0x2453    
#define IDP7_FS_I_DAI_PB02_O_REG          0x2453        
#define DIR_FS_I_DAI_PB02_O_REG           0x2453

#define SPORT0_FS_I_DAI_PB03_O_REG        0x2450    /* SRU_FS0 */
#define SPORT1_FS_I_DAI_PB03_O_REG        0x2450    
#define SPORT2_FS_I_DAI_PB03_O_REG        0x2450    
#define SPORT3_FS_I_DAI_PB03_O_REG        0x2450    
#define SPORT4_FS_I_DAI_PB03_O_REG        0x2450    
#define SPORT5_FS_I_DAI_PB03_O_REG        0x2450    
#define SRC0_FS_IP_I_DAI_PB03_O_REG       0x2451    /* SRU_FS1 */
#define SRC0_FS_0P_I_DAI_PB03_O_REG       0x2451    
#define SRC1_FS_IP_I_DAI_PB03_O_REG       0x2451    
#define SRC1_FS_OP_I_DAI_PB03_O_REG       0x2451    
#define SRC2_FS_IP_I_DAI_PB03_O_REG       0x2451    
#define SRC2_FS_OP_I_DAI_PB03_O_REG       0x2451    
#define SRC3_FS_IP_I_DAI_PB03_O_REG       0x2452    /* SRU_FS2 */
#define SRC3_FS_OP_I_DAI_PB03_O_REG       0x2452    
#define DIT_I_DAI_PB03_O_REG           0x2452    
#define PDAP_HOLD_DAI_PB03_O_REG          0x2452    
#define IDP0_FS_I_DAI_PB03_O_REG          0x2452    
#define IDP1_FS_I_DAI_PB03_O_REG          0x2452    
#define IDP2_FS_I_DAI_PB03_O_REG          0x2452    
#define IDP3_FS_I_DAI_PB03_O_REG          0x2453    /* SRU_FS3 */
#define IDP4_FS_I_DAI_PB03_O_REG          0x2453    
#define IDP5_FS_I_DAI_PB03_O_REG          0x2453    
#define IDP6_FS_I_DAI_PB03_O_REG          0x2453    
#define IDP7_FS_I_DAI_PB03_O_REG          0x2453        
#define DIR_FS_I_DAI_PB03_O_REG           0x2453

#define SPORT0_FS_I_DAI_PB04_O_REG        0x2450    /* SRU_FS0 */
#define SPORT1_FS_I_DAI_PB04_O_REG        0x2450    
#define SPORT2_FS_I_DAI_PB04_O_REG        0x2450    
#define SPORT3_FS_I_DAI_PB04_O_REG        0x2450    
#define SPORT4_FS_I_DAI_PB04_O_REG        0x2450    
#define SPORT5_FS_I_DAI_PB04_O_REG        0x2450    
#define SRC0_FS_IP_I_DAI_PB04_O_REG       0x2451    /* SRU_FS1 */
#define SRC0_FS_0P_I_DAI_PB04_O_REG       0x2451    
#define SRC1_FS_IP_I_DAI_PB04_O_REG       0x2451    
#define SRC1_FS_OP_I_DAI_PB04_O_REG       0x2451    
#define SRC2_FS_IP_I_DAI_PB04_O_REG       0x2451    
#define SRC2_FS_OP_I_DAI_PB04_O_REG       0x2451    
#define SRC3_FS_IP_I_DAI_PB04_O_REG       0x2452    /* SRU_FS2 */
#define SRC3_FS_OP_I_DAI_PB04_O_REG       0x2452    
#define DIT_I_DAI_PB04_O_REG           0x2452    
#define PDAP_HOLD_DAI_PB04_O_REG          0x2452    
#define IDP0_FS_I_DAI_PB04_O_REG          0x2452    
#define IDP1_FS_I_DAI_PB04_O_REG          0x2452    
#define IDP2_FS_I_DAI_PB04_O_REG          0x2452    
#define IDP3_FS_I_DAI_PB04_O_REG          0x2453    /* SRU_FS3 */
#define IDP4_FS_I_DAI_PB04_O_REG          0x2453    
#define IDP5_FS_I_DAI_PB04_O_REG          0x2453    
#define IDP6_FS_I_DAI_PB04_O_REG          0x2453    
#define IDP7_FS_I_DAI_PB04_O_REG          0x2453        
#define DIR_FS_I_DAI_PB04_O_REG           0x2453

#define SPORT0_FS_I_DAI_PB05_O_REG        0x2450    /* SRU_FS0 */
#define SPORT1_FS_I_DAI_PB05_O_REG        0x2450    
#define SPORT2_FS_I_DAI_PB05_O_REG        0x2450    
#define SPORT3_FS_I_DAI_PB05_O_REG        0x2450    
#define SPORT4_FS_I_DAI_PB05_O_REG        0x2450    
#define SPORT5_FS_I_DAI_PB05_O_REG        0x2450    
#define SRC0_FS_IP_I_DAI_PB05_O_REG       0x2451    /* SRU_FS1 */
#define SRC0_FS_0P_I_DAI_PB05_O_REG       0x2451    
#define SRC1_FS_IP_I_DAI_PB05_O_REG       0x2451    
#define SRC1_FS_OP_I_DAI_PB05_O_REG       0x2451    
#define SRC2_FS_IP_I_DAI_PB05_O_REG       0x2451    
#define SRC2_FS_OP_I_DAI_PB05_O_REG       0x2451    
#define SRC3_FS_IP_I_DAI_PB05_O_REG       0x2452    /* SRU_FS2 */
#define SRC3_FS_OP_I_DAI_PB05_O_REG       0x2452    
#define DIT_I_DAI_PB05_O_REG           0x2452    
#define PDAP_HOLD_DAI_PB05_O_REG          0x2452    
#define IDP0_FS_I_DAI_PB05_O_REG          0x2452    
#define IDP1_FS_I_DAI_PB05_O_REG          0x2452    
#define IDP2_FS_I_DAI_PB05_O_REG          0x2452    
#define IDP3_FS_I_DAI_PB05_O_REG          0x2453    /* SRU_FS3 */
#define IDP4_FS_I_DAI_PB05_O_REG          0x2453    
#define IDP5_FS_I_DAI_PB05_O_REG          0x2453    
#define IDP6_FS_I_DAI_PB05_O_REG          0x2453    
#define IDP7_FS_I_DAI_PB05_O_REG          0x2453        
#define DIR_FS_I_DAI_PB05_O_REG           0x2453

#define SPORT0_FS_I_DAI_PB06_O_REG        0x2450    /* SRU_FS0 */
#define SPORT1_FS_I_DAI_PB06_O_REG        0x2450    
#define SPORT2_FS_I_DAI_PB06_O_REG        0x2450    
#define SPORT3_FS_I_DAI_PB06_O_REG        0x2450    
#define SPORT4_FS_I_DAI_PB06_O_REG        0x2450    
#define SPORT5_FS_I_DAI_PB06_O_REG        0x2450    
#define SRC0_FS_IP_I_DAI_PB06_O_REG       0x2451    /* SRU_FS1 */
#define SRC0_FS_0P_I_DAI_PB06_O_REG       0x2451    
#define SRC1_FS_IP_I_DAI_PB06_O_REG       0x2451    
#define SRC1_FS_OP_I_DAI_PB06_O_REG       0x2451    
#define SRC2_FS_IP_I_DAI_PB06_O_REG       0x2451    
#define SRC2_FS_OP_I_DAI_PB06_O_REG       0x2451    
#define SRC3_FS_IP_I_DAI_PB06_O_REG       0x2452    /* SRU_FS2 */
#define SRC3_FS_OP_I_DAI_PB06_O_REG       0x2452    
#define DIT_I_DAI_PB06_O_REG           0x2452    
#define PDAP_HOLD_DAI_PB06_O_REG          0x2452    
#define IDP0_FS_I_DAI_PB06_O_REG          0x2452    
#define IDP1_FS_I_DAI_PB06_O_REG          0x2452    
#define IDP2_FS_I_DAI_PB06_O_REG          0x2452    
#define IDP3_FS_I_DAI_PB06_O_REG          0x2453    /* SRU_FS3 */
#define IDP4_FS_I_DAI_PB06_O_REG          0x2453    
#define IDP5_FS_I_DAI_PB06_O_REG          0x2453    
#define IDP6_FS_I_DAI_PB06_O_REG          0x2453    
#define IDP7_FS_I_DAI_PB06_O_REG          0x2453        
#define DIR_FS_I_DAI_PB06_O_REG           0x2453

#define SPORT0_FS_I_DAI_PB07_O_REG        0x2450    /* SRU_FS0 */
#define SPORT1_FS_I_DAI_PB07_O_REG        0x2450    
#define SPORT2_FS_I_DAI_PB07_O_REG        0x2450    
#define SPORT3_FS_I_DAI_PB07_O_REG        0x2450    
#define SPORT4_FS_I_DAI_PB07_O_REG        0x2450    
#define SPORT5_FS_I_DAI_PB07_O_REG        0x2450    
#define SRC0_FS_IP_I_DAI_PB07_O_REG       0x2451    /* SRU_FS1 */
#define SRC0_FS_0P_I_DAI_PB07_O_REG       0x2451    
#define SRC1_FS_IP_I_DAI_PB07_O_REG       0x2451    
#define SRC1_FS_OP_I_DAI_PB07_O_REG       0x2451    
#define SRC2_FS_IP_I_DAI_PB07_O_REG       0x2451    
#define SRC2_FS_OP_I_DAI_PB07_O_REG       0x2451    
#define SRC3_FS_IP_I_DAI_PB07_O_REG       0x2452    /* SRU_FS2 */
#define SRC3_FS_OP_I_DAI_PB07_O_REG       0x2452    
#define DIT_I_DAI_PB07_O_REG           0x2452    
#define PDAP_HOLD_DAI_PB07_O_REG          0x2452    
#define IDP0_FS_I_DAI_PB07_O_REG          0x2452    
#define IDP1_FS_I_DAI_PB07_O_REG          0x2452    
#define IDP2_FS_I_DAI_PB07_O_REG          0x2452    
#define IDP3_FS_I_DAI_PB07_O_REG          0x2453    /* SRU_FS3 */
#define IDP4_FS_I_DAI_PB07_O_REG          0x2453    
#define IDP5_FS_I_DAI_PB07_O_REG          0x2453    
#define IDP6_FS_I_DAI_PB07_O_REG          0x2453    
#define IDP7_FS_I_DAI_PB07_O_REG          0x2453        
#define DIR_FS_I_DAI_PB07_O_REG           0x2453

#define SPORT0_FS_I_DAI_PB08_O_REG        0x2450    /* SRU_FS0 */
#define SPORT1_FS_I_DAI_PB08_O_REG        0x2450    
#define SPORT2_FS_I_DAI_PB08_O_REG        0x2450    
#define SPORT3_FS_I_DAI_PB08_O_REG        0x2450    
#define SPORT4_FS_I_DAI_PB08_O_REG        0x2450    
#define SPORT5_FS_I_DAI_PB08_O_REG        0x2450    
#define SRC0_FS_IP_I_DAI_PB08_O_REG       0x2451    /* SRU_FS1 */
#define SRC0_FS_0P_I_DAI_PB08_O_REG       0x2451    
#define SRC1_FS_IP_I_DAI_PB08_O_REG       0x2451    
#define SRC1_FS_OP_I_DAI_PB08_O_REG       0x2451    
#define SRC2_FS_IP_I_DAI_PB08_O_REG       0x2451    
#define SRC2_FS_OP_I_DAI_PB08_O_REG       0x2451    
#define SRC3_FS_IP_I_DAI_PB08_O_REG       0x2452    /* SRU_FS2 */
#define SRC3_FS_OP_I_DAI_PB08_O_REG       0x2452    
#define DIT_I_DAI_PB08_O_REG           0x2452    
#define PDAP_HOLD_DAI_PB08_O_REG          0x2452    
#define IDP0_FS_I_DAI_PB08_O_REG          0x2452    
#define IDP1_FS_I_DAI_PB08_O_REG          0x2452    
#define IDP2_FS_I_DAI_PB08_O_REG          0x2452    
#define IDP3_FS_I_DAI_PB08_O_REG          0x2453    /* SRU_FS3 */
#define IDP4_FS_I_DAI_PB08_O_REG          0x2453    
#define IDP5_FS_I_DAI_PB08_O_REG          0x2453    
#define IDP6_FS_I_DAI_PB08_O_REG          0x2453    
#define IDP7_FS_I_DAI_PB08_O_REG          0x2453        
#define DIR_FS_I_DAI_PB08_O_REG           0x2453

#define SPORT0_FS_I_DAI_PB09_O_REG        0x2450    /* SRU_FS0 */
#define SPORT1_FS_I_DAI_PB09_O_REG        0x2450    
#define SPORT2_FS_I_DAI_PB09_O_REG        0x2450    
#define SPORT3_FS_I_DAI_PB09_O_REG        0x2450    
#define SPORT4_FS_I_DAI_PB09_O_REG        0x2450    
#define SPORT5_FS_I_DAI_PB09_O_REG        0x2450    
#define SRC0_FS_IP_I_DAI_PB09_O_REG       0x2451    /* SRU_FS1 */
#define SRC0_FS_0P_I_DAI_PB09_O_REG       0x2451    
#define SRC1_FS_IP_I_DAI_PB09_O_REG       0x2451    
#define SRC1_FS_OP_I_DAI_PB09_O_REG       0x2451    
#define SRC2_FS_IP_I_DAI_PB09_O_REG       0x2451    
#define SRC2_FS_OP_I_DAI_PB09_O_REG       0x2451    
#define SRC3_FS_IP_I_DAI_PB09_O_REG       0x2452    /* SRU_FS2 */
#define SRC3_FS_OP_I_DAI_PB09_O_REG       0x2452    
#define DIT_I_DAI_PB09_O_REG           0x2452    
#define PDAP_HOLD_DAI_PB09_O_REG          0x2452    
#define IDP0_FS_I_DAI_PB09_O_REG          0x2452    
#define IDP1_FS_I_DAI_PB09_O_REG          0x2452    
#define IDP2_FS_I_DAI_PB09_O_REG          0x2452    
#define IDP3_FS_I_DAI_PB09_O_REG          0x2453    /* SRU_FS3 */
#define IDP4_FS_I_DAI_PB09_O_REG          0x2453    
#define IDP5_FS_I_DAI_PB09_O_REG          0x2453    
#define IDP6_FS_I_DAI_PB09_O_REG          0x2453    
#define IDP7_FS_I_DAI_PB09_O_REG          0x2453        
#define DIR_FS_I_DAI_PB09_O_REG           0x2453

#define SPORT0_FS_I_DAI_PB10_O_REG        0x2450    /* SRU_FS0 */
#define SPORT1_FS_I_DAI_PB10_O_REG        0x2450    
#define SPORT2_FS_I_DAI_PB10_O_REG        0x2450    
#define SPORT3_FS_I_DAI_PB10_O_REG        0x2450    
#define SPORT4_FS_I_DAI_PB10_O_REG        0x2450    
#define SPORT5_FS_I_DAI_PB10_O_REG        0x2450    
#define SRC0_FS_IP_I_DAI_PB10_O_REG       0x2451    /* SRU_FS1 */
#define SRC0_FS_0P_I_DAI_PB10_O_REG       0x2451    
#define SRC1_FS_IP_I_DAI_PB10_O_REG       0x2451    
#define SRC1_FS_OP_I_DAI_PB10_O_REG       0x2451    
#define SRC2_FS_IP_I_DAI_PB10_O_REG       0x2451    
#define SRC2_FS_OP_I_DAI_PB10_O_REG       0x2451    
#define SRC3_FS_IP_I_DAI_PB10_O_REG       0x2452    /* SRU_FS2 */
#define SRC3_FS_OP_I_DAI_PB10_O_REG       0x2452    
#define DIT_I_DAI_PB10_O_REG           0x2452    
#define PDAP_HOLD_DAI_PB10_O_REG          0x2452    
#define IDP0_FS_I_DAI_PB10_O_REG          0x2452    
#define IDP1_FS_I_DAI_PB10_O_REG          0x2452    
#define IDP2_FS_I_DAI_PB10_O_REG          0x2452    
#define IDP3_FS_I_DAI_PB10_O_REG          0x2453    /* SRU_FS3 */
#define IDP4_FS_I_DAI_PB10_O_REG          0x2453    
#define IDP5_FS_I_DAI_PB10_O_REG          0x2453    
#define IDP6_FS_I_DAI_PB10_O_REG          0x2453    
#define IDP7_FS_I_DAI_PB10_O_REG          0x2453        
#define DIR_FS_I_DAI_PB10_O_REG           0x2453

#define SPORT0_FS_I_DAI_PB11_O_REG        0x2450    /* SRU_FS0 */
#define SPORT1_FS_I_DAI_PB11_O_REG        0x2450    
#define SPORT2_FS_I_DAI_PB11_O_REG        0x2450    
#define SPORT3_FS_I_DAI_PB11_O_REG        0x2450    
#define SPORT4_FS_I_DAI_PB11_O_REG        0x2450    
#define SPORT5_FS_I_DAI_PB11_O_REG        0x2450    
#define SRC0_FS_IP_I_DAI_PB11_O_REG       0x2451    /* SRU_FS1 */
#define SRC0_FS_0P_I_DAI_PB11_O_REG       0x2451    
#define SRC1_FS_IP_I_DAI_PB11_O_REG       0x2451    
#define SRC1_FS_OP_I_DAI_PB11_O_REG       0x2451    
#define SRC2_FS_IP_I_DAI_PB11_O_REG       0x2451    
#define SRC2_FS_OP_I_DAI_PB11_O_REG       0x2451    
#define SRC3_FS_IP_I_DAI_PB11_O_REG       0x2452    /* SRU_FS2 */
#define SRC3_FS_OP_I_DAI_PB11_O_REG       0x2452    
#define DIT_I_DAI_PB11_O_REG           0x2452    
#define PDAP_HOLD_DAI_PB11_O_REG          0x2452    
#define IDP0_FS_I_DAI_PB11_O_REG          0x2452    
#define IDP1_FS_I_DAI_PB11_O_REG          0x2452    
#define IDP2_FS_I_DAI_PB11_O_REG          0x2452    
#define IDP3_FS_I_DAI_PB11_O_REG          0x2453    /* SRU_FS3 */
#define IDP4_FS_I_DAI_PB11_O_REG          0x2453    
#define IDP5_FS_I_DAI_PB11_O_REG          0x2453    
#define IDP6_FS_I_DAI_PB11_O_REG          0x2453    
#define IDP7_FS_I_DAI_PB11_O_REG          0x2453        
#define DIR_FS_I_DAI_PB11_O_REG           0x2453

#define SPORT0_FS_I_DAI_PB12_O_REG        0x2450    /* SRU_FS0 */
#define SPORT1_FS_I_DAI_PB12_O_REG        0x2450    
#define SPORT2_FS_I_DAI_PB12_O_REG        0x2450    
#define SPORT3_FS_I_DAI_PB12_O_REG        0x2450    
#define SPORT4_FS_I_DAI_PB12_O_REG        0x2450    
#define SPORT5_FS_I_DAI_PB12_O_REG        0x2450    
#define SRC0_FS_IP_I_DAI_PB12_O_REG       0x2451    /* SRU_FS1 */
#define SRC0_FS_0P_I_DAI_PB12_O_REG       0x2451    
#define SRC1_FS_IP_I_DAI_PB12_O_REG       0x2451    
#define SRC1_FS_OP_I_DAI_PB12_O_REG       0x2451    
#define SRC2_FS_IP_I_DAI_PB12_O_REG       0x2451    
#define SRC2_FS_OP_I_DAI_PB12_O_REG       0x2451    
#define SRC3_FS_IP_I_DAI_PB12_O_REG       0x2452    /* SRU_FS2 */
#define SRC3_FS_OP_I_DAI_PB12_O_REG       0x2452    
#define DIT_I_DAI_PB12_O_REG           0x2452    
#define PDAP_HOLD_DAI_PB12_O_REG          0x2452    
#define IDP0_FS_I_DAI_PB12_O_REG          0x2452    
#define IDP1_FS_I_DAI_PB12_O_REG          0x2452    
#define IDP2_FS_I_DAI_PB12_O_REG          0x2452    
#define IDP3_FS_I_DAI_PB12_O_REG          0x2453    /* SRU_FS3 */
#define IDP4_FS_I_DAI_PB12_O_REG          0x2453    
#define IDP5_FS_I_DAI_PB12_O_REG          0x2453    
#define IDP6_FS_I_DAI_PB12_O_REG          0x2453    
#define IDP7_FS_I_DAI_PB12_O_REG          0x2453        
#define DIR_FS_I_DAI_PB12_O_REG           0x2453

#define SPORT0_FS_I_DAI_PB13_O_REG        0x2450    /* SRU_FS0 */
#define SPORT1_FS_I_DAI_PB13_O_REG        0x2450    
#define SPORT2_FS_I_DAI_PB13_O_REG        0x2450    
#define SPORT3_FS_I_DAI_PB13_O_REG        0x2450    
#define SPORT4_FS_I_DAI_PB13_O_REG        0x2450    
#define SPORT5_FS_I_DAI_PB13_O_REG        0x2450    
#define SRC0_FS_IP_I_DAI_PB13_O_REG       0x2451    /* SRU_FS1 */
#define SRC0_FS_0P_I_DAI_PB13_O_REG       0x2451    
#define SRC1_FS_IP_I_DAI_PB13_O_REG       0x2451    
#define SRC1_FS_OP_I_DAI_PB13_O_REG       0x2451    
#define SRC2_FS_IP_I_DAI_PB13_O_REG       0x2451    
#define SRC2_FS_OP_I_DAI_PB13_O_REG       0x2451    
#define SRC3_FS_IP_I_DAI_PB13_O_REG       0x2452    /* SRU_FS2 */
#define SRC3_FS_OP_I_DAI_PB13_O_REG       0x2452    
#define DIT_I_DAI_PB13_O_REG           0x2452    
#define PDAP_HOLD_DAI_PB13_O_REG          0x2452    
#define IDP0_FS_I_DAI_PB13_O_REG          0x2452    
#define IDP1_FS_I_DAI_PB13_O_REG          0x2452    
#define IDP2_FS_I_DAI_PB13_O_REG          0x2452    
#define IDP3_FS_I_DAI_PB13_O_REG          0x2453    /* SRU_FS3 */
#define IDP4_FS_I_DAI_PB13_O_REG          0x2453    
#define IDP5_FS_I_DAI_PB13_O_REG          0x2453    
#define IDP6_FS_I_DAI_PB13_O_REG          0x2453    
#define IDP7_FS_I_DAI_PB13_O_REG          0x2453        
#define DIR_FS_I_DAI_PB13_O_REG           0x2453

#define SPORT0_FS_I_DAI_PB14_O_REG        0x2450    /* SRU_FS0 */
#define SPORT1_FS_I_DAI_PB14_O_REG        0x2450    
#define SPORT2_FS_I_DAI_PB14_O_REG        0x2450    
#define SPORT3_FS_I_DAI_PB14_O_REG        0x2450    
#define SPORT4_FS_I_DAI_PB14_O_REG        0x2450    
#define SPORT5_FS_I_DAI_PB14_O_REG        0x2450    
#define SRC0_FS_IP_I_DAI_PB14_O_REG       0x2451    /* SRU_FS1 */
#define SRC0_FS_0P_I_DAI_PB14_O_REG       0x2451    
#define SRC1_FS_IP_I_DAI_PB14_O_REG       0x2451    
#define SRC1_FS_OP_I_DAI_PB14_O_REG       0x2451    
#define SRC2_FS_IP_I_DAI_PB14_O_REG       0x2451    
#define SRC2_FS_OP_I_DAI_PB14_O_REG       0x2451    
#define SRC3_FS_IP_I_DAI_PB14_O_REG       0x2452    /* SRU_FS2 */
#define SRC3_FS_OP_I_DAI_PB14_O_REG       0x2452    
#define DIT_I_DAI_PB14_O_REG           0x2452    
#define PDAP_HOLD_DAI_PB14_O_REG          0x2452    
#define IDP0_FS_I_DAI_PB14_O_REG          0x2452    
#define IDP1_FS_I_DAI_PB14_O_REG          0x2452    
#define IDP2_FS_I_DAI_PB14_O_REG          0x2452    
#define IDP3_FS_I_DAI_PB14_O_REG          0x2453    /* SRU_FS3 */
#define IDP4_FS_I_DAI_PB14_O_REG          0x2453    
#define IDP5_FS_I_DAI_PB14_O_REG          0x2453    
#define IDP6_FS_I_DAI_PB14_O_REG          0x2453    
#define IDP7_FS_I_DAI_PB14_O_REG          0x2453        
#define DIR_FS_I_DAI_PB14_O_REG           0x2453

#define SPORT0_FS_I_DAI_PB15_O_REG        0x2450    /* SRU_FS0 */
#define SPORT1_FS_I_DAI_PB15_O_REG        0x2450    
#define SPORT2_FS_I_DAI_PB15_O_REG        0x2450    
#define SPORT3_FS_I_DAI_PB15_O_REG        0x2450    
#define SPORT4_FS_I_DAI_PB15_O_REG        0x2450    
#define SPORT5_FS_I_DAI_PB15_O_REG        0x2450    
#define SRC0_FS_IP_I_DAI_PB15_O_REG       0x2451    /* SRU_FS1 */
#define SRC0_FS_0P_I_DAI_PB15_O_REG       0x2451    
#define SRC1_FS_IP_I_DAI_PB15_O_REG       0x2451    
#define SRC1_FS_OP_I_DAI_PB15_O_REG       0x2451    
#define SRC2_FS_IP_I_DAI_PB15_O_REG       0x2451    
#define SRC2_FS_OP_I_DAI_PB15_O_REG       0x2451    
#define SRC3_FS_IP_I_DAI_PB15_O_REG       0x2452    /* SRU_FS2 */
#define SRC3_FS_OP_I_DAI_PB15_O_REG       0x2452    
#define DIT_I_DAI_PB15_O_REG           0x2452    
#define PDAP_HOLD_DAI_PB15_O_REG          0x2452    
#define IDP0_FS_I_DAI_PB15_O_REG          0x2452    
#define IDP1_FS_I_DAI_PB15_O_REG          0x2452    
#define IDP2_FS_I_DAI_PB15_O_REG          0x2452    
#define IDP3_FS_I_DAI_PB15_O_REG          0x2453    /* SRU_FS3 */
#define IDP4_FS_I_DAI_PB15_O_REG          0x2453    
#define IDP5_FS_I_DAI_PB15_O_REG          0x2453    
#define IDP6_FS_I_DAI_PB15_O_REG          0x2453    
#define IDP7_FS_I_DAI_PB15_O_REG          0x2453        
#define DIR_FS_I_DAI_PB15_O_REG           0x2453

#define SPORT0_FS_I_DAI_PB16_O_REG        0x2450    /* SRU_FS0 */
#define SPORT1_FS_I_DAI_PB16_O_REG        0x2450    
#define SPORT2_FS_I_DAI_PB16_O_REG        0x2450    
#define SPORT3_FS_I_DAI_PB16_O_REG        0x2450    
#define SPORT4_FS_I_DAI_PB16_O_REG        0x2450    
#define SPORT5_FS_I_DAI_PB16_O_REG        0x2450    
#define SRC0_FS_IP_I_DAI_PB16_O_REG       0x2451    /* SRU_FS1 */
#define SRC0_FS_0P_I_DAI_PB16_O_REG       0x2451    
#define SRC1_FS_IP_I_DAI_PB16_O_REG       0x2451    
#define SRC1_FS_OP_I_DAI_PB16_O_REG       0x2451    
#define SRC2_FS_IP_I_DAI_PB16_O_REG       0x2451    
#define SRC2_FS_OP_I_DAI_PB16_O_REG       0x2451    
#define SRC3_FS_IP_I_DAI_PB16_O_REG       0x2452    /* SRU_FS2 */
#define SRC3_FS_OP_I_DAI_PB16_O_REG       0x2452    
#define DIT_I_DAI_PB16_O_REG           0x2452    
#define PDAP_HOLD_DAI_PB16_O_REG          0x2452    
#define IDP0_FS_I_DAI_PB16_O_REG          0x2452    
#define IDP1_FS_I_DAI_PB16_O_REG          0x2452    
#define IDP2_FS_I_DAI_PB16_O_REG          0x2452    
#define IDP3_FS_I_DAI_PB16_O_REG          0x2453    /* SRU_FS3 */
#define IDP4_FS_I_DAI_PB16_O_REG          0x2453    
#define IDP5_FS_I_DAI_PB16_O_REG          0x2453    
#define IDP6_FS_I_DAI_PB16_O_REG          0x2453    
#define IDP7_FS_I_DAI_PB16_O_REG          0x2453        
#define DIR_FS_I_DAI_PB16_O_REG           0x2453

#define SPORT0_FS_I_DAI_PB17_O_REG        0x2450    /* SRU_FS0 */
#define SPORT1_FS_I_DAI_PB17_O_REG        0x2450    
#define SPORT2_FS_I_DAI_PB17_O_REG        0x2450    
#define SPORT3_FS_I_DAI_PB17_O_REG        0x2450    
#define SPORT4_FS_I_DAI_PB17_O_REG        0x2450    
#define SPORT5_FS_I_DAI_PB17_O_REG        0x2450    
#define SRC0_FS_IP_I_DAI_PB17_O_REG       0x2451    /* SRU_FS1 */
#define SRC0_FS_0P_I_DAI_PB17_O_REG       0x2451    
#define SRC1_FS_IP_I_DAI_PB17_O_REG       0x2451    
#define SRC1_FS_OP_I_DAI_PB17_O_REG       0x2451    
#define SRC2_FS_IP_I_DAI_PB17_O_REG       0x2451    
#define SRC2_FS_OP_I_DAI_PB17_O_REG       0x2451    
#define SRC3_FS_IP_I_DAI_PB17_O_REG       0x2452    /* SRU_FS2 */
#define SRC3_FS_OP_I_DAI_PB17_O_REG       0x2452    
#define DIT_I_DAI_PB17_O_REG           0x2452    
#define PDAP_HOLD_DAI_PB17_O_REG          0x2452    
#define IDP0_FS_I_DAI_PB17_O_REG          0x2452    
#define IDP1_FS_I_DAI_PB17_O_REG          0x2452    
#define IDP2_FS_I_DAI_PB17_O_REG          0x2452    
#define IDP3_FS_I_DAI_PB17_O_REG          0x2453    /* SRU_FS3 */
#define IDP4_FS_I_DAI_PB17_O_REG          0x2453    
#define IDP5_FS_I_DAI_PB17_O_REG          0x2453    
#define IDP6_FS_I_DAI_PB17_O_REG          0x2453    
#define IDP7_FS_I_DAI_PB17_O_REG          0x2453        
#define DIR_FS_I_DAI_PB17_O_REG           0x2453

#define SPORT0_FS_I_DAI_PB18_O_REG        0x2450    /* SRU_FS0 */
#define SPORT1_FS_I_DAI_PB18_O_REG        0x2450    
#define SPORT2_FS_I_DAI_PB18_O_REG        0x2450    
#define SPORT3_FS_I_DAI_PB18_O_REG        0x2450    
#define SPORT4_FS_I_DAI_PB18_O_REG        0x2450    
#define SPORT5_FS_I_DAI_PB18_O_REG        0x2450    
#define SRC0_FS_IP_I_DAI_PB18_O_REG       0x2451    /* SRU_FS1 */
#define SRC0_FS_0P_I_DAI_PB18_O_REG       0x2451    
#define SRC1_FS_IP_I_DAI_PB18_O_REG       0x2451    
#define SRC1_FS_OP_I_DAI_PB18_O_REG       0x2451    
#define SRC2_FS_IP_I_DAI_PB18_O_REG       0x2451    
#define SRC2_FS_OP_I_DAI_PB18_O_REG       0x2451    
#define SRC3_FS_IP_I_DAI_PB18_O_REG       0x2452    /* SRU_FS2 */
#define SRC3_FS_OP_I_DAI_PB18_O_REG       0x2452    
#define DIT_I_DAI_PB18_O_REG           0x2452    
#define PDAP_HOLD_DAI_PB18_O_REG          0x2452    
#define IDP0_FS_I_DAI_PB18_O_REG          0x2452    
#define IDP1_FS_I_DAI_PB18_O_REG          0x2452    
#define IDP2_FS_I_DAI_PB18_O_REG          0x2452    
#define IDP3_FS_I_DAI_PB18_O_REG          0x2453    /* SRU_FS3 */
#define IDP4_FS_I_DAI_PB18_O_REG          0x2453    
#define IDP5_FS_I_DAI_PB18_O_REG          0x2453    
#define IDP6_FS_I_DAI_PB18_O_REG          0x2453    
#define IDP7_FS_I_DAI_PB18_O_REG          0x2453        
#define DIR_FS_I_DAI_PB18_O_REG           0x2453

#define SPORT0_FS_I_DAI_PB19_O_REG        0x2450    /* SRU_FS0 */
#define SPORT1_FS_I_DAI_PB19_O_REG        0x2450    
#define SPORT2_FS_I_DAI_PB19_O_REG        0x2450    
#define SPORT3_FS_I_DAI_PB19_O_REG        0x2450    
#define SPORT4_FS_I_DAI_PB19_O_REG        0x2450    
#define SPORT5_FS_I_DAI_PB19_O_REG        0x2450    
#define SRC0_FS_IP_I_DAI_PB19_O_REG       0x2451    /* SRU_FS1 */
#define SRC0_FS_0P_I_DAI_PB19_O_REG       0x2451    
#define SRC1_FS_IP_I_DAI_PB19_O_REG       0x2451    
#define SRC1_FS_OP_I_DAI_PB19_O_REG       0x2451    
#define SRC2_FS_IP_I_DAI_PB19_O_REG       0x2451    
#define SRC2_FS_OP_I_DAI_PB19_O_REG       0x2451    
#define SRC3_FS_IP_I_DAI_PB19_O_REG       0x2452    /* SRU_FS2 */
#define SRC3_FS_OP_I_DAI_PB19_O_REG       0x2452    
#define DIT_I_DAI_PB19_O_REG           0x2452    
#define PDAP_HOLD_DAI_PB19_O_REG          0x2452    
#define IDP0_FS_I_DAI_PB19_O_REG          0x2452    
#define IDP1_FS_I_DAI_PB19_O_REG          0x2452    
#define IDP2_FS_I_DAI_PB19_O_REG          0x2452    
#define IDP3_FS_I_DAI_PB19_O_REG          0x2453    /* SRU_FS3 */
#define IDP4_FS_I_DAI_PB19_O_REG          0x2453    
#define IDP5_FS_I_DAI_PB19_O_REG          0x2453    
#define IDP6_FS_I_DAI_PB19_O_REG          0x2453    
#define IDP7_FS_I_DAI_PB19_O_REG          0x2453        
#define DIR_FS_I_DAI_PB19_O_REG           0x2453

#define SPORT0_FS_I_DAI_PB20_O_REG        0x2450    /* SRU_FS0 */
#define SPORT1_FS_I_DAI_PB20_O_REG        0x2450    
#define SPORT2_FS_I_DAI_PB20_O_REG        0x2450    
#define SPORT3_FS_I_DAI_PB20_O_REG        0x2450    
#define SPORT4_FS_I_DAI_PB20_O_REG        0x2450    
#define SPORT5_FS_I_DAI_PB20_O_REG        0x2450    
#define SRC0_FS_IP_I_DAI_PB20_O_REG       0x2451    /* SRU_FS1 */
#define SRC0_FS_0P_I_DAI_PB20_O_REG       0x2451    
#define SRC1_FS_IP_I_DAI_PB20_O_REG       0x2451    
#define SRC1_FS_OP_I_DAI_PB20_O_REG       0x2451    
#define SRC2_FS_IP_I_DAI_PB20_O_REG       0x2451    
#define SRC2_FS_OP_I_DAI_PB20_O_REG       0x2451    
#define SRC3_FS_IP_I_DAI_PB20_O_REG       0x2452    /* SRU_FS2 */
#define SRC3_FS_OP_I_DAI_PB20_O_REG       0x2452    
#define DIT_I_DAI_PB20_O_REG           0x2452    
#define PDAP_HOLD_DAI_PB20_O_REG          0x2452    
#define IDP0_FS_I_DAI_PB20_O_REG          0x2452    
#define IDP1_FS_I_DAI_PB20_O_REG          0x2452    
#define IDP2_FS_I_DAI_PB20_O_REG          0x2452    
#define IDP3_FS_I_DAI_PB20_O_REG          0x2453    /* SRU_FS3 */
#define IDP4_FS_I_DAI_PB20_O_REG          0x2453    
#define IDP5_FS_I_DAI_PB20_O_REG          0x2453    
#define IDP6_FS_I_DAI_PB20_O_REG          0x2453    
#define IDP7_FS_I_DAI_PB20_O_REG          0x2453        
#define DIR_FS_I_DAI_PB20_O_REG           0x2453

#define SPORT0_FS_I_SPORT0_FS_O_REG        0x2450    /* SRU_FS0 */
#define SPORT1_FS_I_SPORT0_FS_O_REG        0x2450    
#define SPORT2_FS_I_SPORT0_FS_O_REG        0x2450    
#define SPORT3_FS_I_SPORT0_FS_O_REG        0x2450    
#define SPORT4_FS_I_SPORT0_FS_O_REG        0x2450    
#define SPORT5_FS_I_SPORT0_FS_O_REG        0x2450    
#define SRC0_FS_IP_I_SPORT0_FS_O_REG       0x2451    /* SRU_FS1 */
#define SRC0_FS_0P_I_SPORT0_FS_O_REG       0x2451    
#define SRC1_FS_IP_I_SPORT0_FS_O_REG       0x2451    
#define SRC1_FS_OP_I_SPORT0_FS_O_REG       0x2451    
#define SRC2_FS_IP_I_SPORT0_FS_O_REG       0x2451    
#define SRC2_FS_OP_I_SPORT0_FS_O_REG       0x2451    
#define SRC3_FS_IP_I_SPORT0_FS_O_REG       0x2452    /* SRU_FS2 */
#define SRC3_FS_OP_I_SPORT0_FS_O_REG       0x2452    
#define DIT_I_SPORT0_FS_O_REG           0x2452    
#define PDAP_HOLD_I_SPORT0_FS_O_REG        0x2452    
#define IDP0_FS_I_SPORT0_FS_O_REG          0x2452    
#define IDP1_FS_I_SPORT0_FS_O_REG          0x2452    
#define IDP2_FS_I_SPORT0_FS_O_REG          0x2452    
#define IDP3_FS_I_SPORT0_FS_O_REG          0x2453    /* SRU_FS3 */
#define IDP4_FS_I_SPORT0_FS_O_REG          0x2453    
#define IDP5_FS_I_SPORT0_FS_O_REG          0x2453    
#define IDP6_FS_I_SPORT0_FS_O_REG          0x2453    
#define IDP7_FS_I_SPORT0_FS_O_REG          0x2453        
#define DIR_FS_I_SPORT0_FS_O_REG           0x2453

#define SPORT0_FS_I_SPORT1_FS_O_REG        0x2450    /* SRU_FS0 */
#define SPORT1_FS_I_SPORT1_FS_O_REG        0x2450    
#define SPORT2_FS_I_SPORT1_FS_O_REG        0x2450    
#define SPORT3_FS_I_SPORT1_FS_O_REG        0x2450    
#define SPORT4_FS_I_SPORT1_FS_O_REG        0x2450    
#define SPORT5_FS_I_SPORT1_FS_O_REG        0x2450    
#define SRC0_FS_IP_I_SPORT1_FS_O_REG       0x2451    /* SRU_FS1 */
#define SRC0_FS_0P_I_SPORT1_FS_O_REG       0x2451    
#define SRC1_FS_IP_I_SPORT1_FS_O_REG       0x2451    
#define SRC1_FS_OP_I_SPORT1_FS_O_REG       0x2451    
#define SRC2_FS_IP_I_SPORT1_FS_O_REG       0x2451    
#define SRC2_FS_OP_I_SPORT1_FS_O_REG       0x2451    
#define SRC3_FS_IP_I_SPORT1_FS_O_REG       0x2452    /* SRU_FS2 */
#define SRC3_FS_OP_I_SPORT1_FS_O_REG       0x2452    
#define DIT_I_SPORT1_FS_O_REG           0x2452    
#define PDAP_HOLD_I_SPORT1_FS_O_REG        0x2452    
#define IDP0_FS_I_SPORT1_FS_O_REG          0x2452    
#define IDP1_FS_I_SPORT1_FS_O_REG          0x2452    
#define IDP2_FS_I_SPORT1_FS_O_REG          0x2452    
#define IDP3_FS_I_SPORT1_FS_O_REG          0x2453    /* SRU_FS3 */
#define IDP4_FS_I_SPORT1_FS_O_REG          0x2453    
#define IDP5_FS_I_SPORT1_FS_O_REG          0x2453    
#define IDP6_FS_I_SPORT1_FS_O_REG          0x2453    
#define IDP7_FS_I_SPORT1_FS_O_REG          0x2453        
#define DIR_FS_I_SPORT1_FS_O_REG           0x2453

#define SPORT0_FS_I_SPORT2_FS_O_REG        0x2450    /* SRU_FS0 */
#define SPORT1_FS_I_SPORT2_FS_O_REG        0x2450    
#define SPORT2_FS_I_SPORT2_FS_O_REG        0x2450    
#define SPORT3_FS_I_SPORT2_FS_O_REG        0x2450    
#define SPORT4_FS_I_SPORT2_FS_O_REG        0x2450    
#define SPORT5_FS_I_SPORT2_FS_O_REG        0x2450    
#define SRC0_FS_IP_I_SPORT2_FS_O_REG       0x2451    /* SRU_FS1 */
#define SRC0_FS_0P_I_SPORT2_FS_O_REG       0x2451    
#define SRC1_FS_IP_I_SPORT2_FS_O_REG       0x2451    
#define SRC1_FS_OP_I_SPORT2_FS_O_REG       0x2451    
#define SRC2_FS_IP_I_SPORT2_FS_O_REG       0x2451    
#define SRC2_FS_OP_I_SPORT2_FS_O_REG       0x2451    
#define SRC3_FS_IP_I_SPORT2_FS_O_REG       0x2452    /* SRU_FS2 */
#define SRC3_FS_OP_I_SPORT2_FS_O_REG       0x2452    
#define DIT_I_SPORT2_FS_O_REG           0x2452    
#define PDAP_HOLD_I_SPORT2_FS_O_REG        0x2452    
#define IDP0_FS_I_SPORT2_FS_O_REG          0x2452    
#define IDP1_FS_I_SPORT2_FS_O_REG          0x2452    
#define IDP2_FS_I_SPORT2_FS_O_REG          0x2452    
#define IDP3_FS_I_SPORT2_FS_O_REG          0x2453    /* SRU_FS3 */
#define IDP4_FS_I_SPORT2_FS_O_REG          0x2453    
#define IDP5_FS_I_SPORT2_FS_O_REG          0x2453    
#define IDP6_FS_I_SPORT2_FS_O_REG          0x2453    
#define IDP7_FS_I_SPORT2_FS_O_REG          0x2453        
#define DIR_FS_I_SPORT2_FS_O_REG           0x2453

#define SPORT0_FS_I_SPORT3_FS_O_REG        0x2450    /* SRU_FS0 */
#define SPORT1_FS_I_SPORT3_FS_O_REG        0x2450    
#define SPORT2_FS_I_SPORT3_FS_O_REG        0x2450    
#define SPORT3_FS_I_SPORT3_FS_O_REG        0x2450    
#define SPORT4_FS_I_SPORT3_FS_O_REG        0x2450    
#define SPORT5_FS_I_SPORT3_FS_O_REG        0x2450    
#define SRC0_FS_IP_I_SPORT3_FS_O_REG       0x2451    /* SRU_FS1 */
#define SRC0_FS_0P_I_SPORT3_FS_O_REG       0x2451    
#define SRC1_FS_IP_I_SPORT3_FS_O_REG       0x2451    
#define SRC1_FS_OP_I_SPORT3_FS_O_REG       0x2451    
#define SRC2_FS_IP_I_SPORT3_FS_O_REG       0x2451    
#define SRC2_FS_OP_I_SPORT3_FS_O_REG       0x2451    
#define SRC3_FS_IP_I_SPORT3_FS_O_REG       0x2452    /* SRU_FS2 */
#define SRC3_FS_OP_I_SPORT3_FS_O_REG       0x2452    
#define DIT_I_SPORT3_FS_O_REG           0x2452    
#define PDAP_HOLD_I_SPORT3_FS_O_REG        0x2452    
#define IDP0_FS_I_SPORT3_FS_O_REG          0x2452    
#define IDP1_FS_I_SPORT3_FS_O_REG          0x2452    
#define IDP2_FS_I_SPORT3_FS_O_REG          0x2452    
#define IDP3_FS_I_SPORT3_FS_O_REG          0x2453    /* SRU_FS3 */
#define IDP4_FS_I_SPORT3_FS_O_REG          0x2453    
#define IDP5_FS_I_SPORT3_FS_O_REG          0x2453    
#define IDP6_FS_I_SPORT3_FS_O_REG          0x2453    
#define IDP7_FS_I_SPORT3_FS_O_REG          0x2453        
#define DIR_FS_I_SPORT3_FS_O_REG           0x2453

#define SPORT0_FS_I_SPORT4_FS_O_REG        0x2450    /* SRU_FS0 */
#define SPORT1_FS_I_SPORT4_FS_O_REG        0x2450    
#define SPORT2_FS_I_SPORT4_FS_O_REG        0x2450    
#define SPORT3_FS_I_SPORT4_FS_O_REG        0x2450    
#define SPORT4_FS_I_SPORT4_FS_O_REG        0x2450    
#define SPORT5_FS_I_SPORT4_FS_O_REG        0x2450    
#define SRC0_FS_IP_I_SPORT4_FS_O_REG       0x2451    /* SRU_FS1 */
#define SRC0_FS_0P_I_SPORT4_FS_O_REG       0x2451    
#define SRC1_FS_IP_I_SPORT4_FS_O_REG       0x2451    
#define SRC1_FS_OP_I_SPORT4_FS_O_REG       0x2451    
#define SRC2_FS_IP_I_SPORT4_FS_O_REG       0x2451    
#define SRC2_FS_OP_I_SPORT4_FS_O_REG       0x2451    
#define SRC3_FS_IP_I_SPORT4_FS_O_REG       0x2452    /* SRU_FS2 */
#define SRC3_FS_OP_I_SPORT4_FS_O_REG       0x2452    
#define DIT_I_SPORT4_FS_O_REG           0x2452    
#define PDAP_HOLD_I_SPORT4_FS_O_REG        0x2452    
#define IDP0_FS_I_SPORT4_FS_O_REG          0x2452    
#define IDP1_FS_I_SPORT4_FS_O_REG          0x2452    
#define IDP2_FS_I_SPORT4_FS_O_REG          0x2452    
#define IDP3_FS_I_SPORT4_FS_O_REG          0x2453    /* SRU_FS3 */
#define IDP4_FS_I_SPORT4_FS_O_REG          0x2453    
#define IDP5_FS_I_SPORT4_FS_O_REG          0x2453    
#define IDP6_FS_I_SPORT4_FS_O_REG          0x2453    
#define IDP7_FS_I_SPORT4_FS_O_REG          0x2453        
#define DIR_FS_I_SPORT4_FS_O_REG           0x2453

#define SPORT0_FS_I_SPORT5_FS_O_REG        0x2450    /* SRU_FS0 */
#define SPORT1_FS_I_SPORT5_FS_O_REG        0x2450    
#define SPORT2_FS_I_SPORT5_FS_O_REG        0x2450    
#define SPORT3_FS_I_SPORT5_FS_O_REG        0x2450    
#define SPORT4_FS_I_SPORT5_FS_O_REG        0x2450    
#define SPORT5_FS_I_SPORT5_FS_O_REG        0x2450    
#define SRC0_FS_IP_I_SPORT5_FS_O_REG       0x2451    /* SRU_FS1 */
#define SRC0_FS_0P_I_SPORT5_FS_O_REG       0x2451    
#define SRC1_FS_IP_I_SPORT5_FS_O_REG       0x2451    
#define SRC1_FS_OP_I_SPORT5_FS_O_REG       0x2451    
#define SRC2_FS_IP_I_SPORT5_FS_O_REG       0x2451    
#define SRC2_FS_OP_I_SPORT5_FS_O_REG       0x2451    
#define SRC3_FS_IP_I_SPORT5_FS_O_REG       0x2452    /* SRU_FS2 */
#define SRC3_FS_OP_I_SPORT5_FS_O_REG       0x2452    
#define DIT_I_SPORT5_FS_O_REG           0x2452    
#define PDAP_HOLD_I_SPORT5_FS_O_REG        0x2452    
#define IDP0_FS_I_SPORT5_FS_O_REG          0x2452    
#define IDP1_FS_I_SPORT5_FS_O_REG          0x2452    
#define IDP2_FS_I_SPORT5_FS_O_REG          0x2452    
#define IDP3_FS_I_SPORT5_FS_O_REG          0x2453    /* SRU_FS3 */
#define IDP4_FS_I_SPORT5_FS_O_REG          0x2453    
#define IDP5_FS_I_SPORT5_FS_O_REG          0x2453    
#define IDP6_FS_I_SPORT5_FS_O_REG          0x2453    
#define IDP7_FS_I_SPORT5_FS_O_REG          0x2453        
#define DIR_FS_I_SPORT5_FS_O_REG           0x2453

#define SPORT0_FS_I_DIR_FS_O_REG        0x2450    /* SRU_FS0 */
#define SPORT1_FS_I_DIR_FS_O_REG        0x2450    
#define SPORT2_FS_I_DIR_FS_O_REG        0x2450    
#define SPORT3_FS_I_DIR_FS_O_REG        0x2450    
#define SPORT4_FS_I_DIR_FS_O_REG        0x2450    
#define SPORT5_FS_I_DIR_FS_O_REG        0x2450    
#define SRC0_FS_IP_I_DIR_FS_O_REG       0x2451    /* SRU_FS1 */
#define SRC0_FS_0P_I_DIR_FS_O_REG       0x2451    
#define SRC1_FS_IP_I_DIR_FS_O_REG       0x2451    
#define SRC1_FS_OP_I_DIR_FS_O_REG       0x2451    
#define SRC2_FS_IP_I_DIR_FS_O_REG       0x2451    
#define SRC2_FS_OP_I_DIR_FS_O_REG       0x2451    
#define SRC3_FS_IP_I_DIR_FS_O_REG       0x2452    /* SRU_FS2 */
#define SRC3_FS_OP_I_DIR_FS_O_REG       0x2452    
#define DIT_I_DIR_FS_O_REG           0x2452    
#define PDAP_HOLD_I_DIR_FS_O_REG        0x2452    
#define IDP0_FS_I_DIR_FS_O_REG          0x2452    
#define IDP1_FS_I_DIR_FS_O_REG          0x2452    
#define IDP2_FS_I_DIR_FS_O_REG          0x2452    
#define IDP3_FS_I_DIR_FS_O_REG          0x2453    /* SRU_FS3 */
#define IDP4_FS_I_DIR_FS_O_REG          0x2453    
#define IDP5_FS_I_DIR_FS_O_REG          0x2453    
#define IDP6_FS_I_DIR_FS_O_REG          0x2453    
#define IDP7_FS_I_DIR_FS_O_REG          0x2453        
#define DIR_FS_I_DIR_FS_O_REG           0x2453

#define SPORT0_FS_I_DIT_O_REG        0x2450    /* SRU_FS0 */
#define SPORT1_FS_I_DIT_O_REG        0x2450    
#define SPORT2_FS_I_DIT_O_REG        0x2450    
#define SPORT3_FS_I_DIT_O_REG        0x2450    
#define SPORT4_FS_I_DIT_O_REG        0x2450    
#define SPORT5_FS_I_DIT_O_REG        0x2450    
#define SRC0_FS_IP_I_DIT_O_REG       0x2451    /* SRU_FS1 */
#define SRC0_FS_0P_I_DIT_O_REG       0x2451    
#define SRC1_FS_IP_I_DIT_O_REG       0x2451    
#define SRC1_FS_OP_I_DIT_O_REG       0x2451    
#define SRC2_FS_IP_I_DIT_O_REG       0x2451    
#define SRC2_FS_OP_I_DIT_O_REG       0x2451    
#define SRC3_FS_IP_I_DIT_O_REG       0x2452    /* SRU_FS2 */
#define SRC3_FS_OP_I_DIT_O_REG       0x2452    
#define DIT_I_DIT_O_REG           0x2452    
#define PDAP_HOLD_I_DIT_O_REG        0x2452    
#define IDP0_FS_I_DIT_O_REG          0x2452    
#define IDP1_FS_I_DIT_O_REG          0x2452    
#define IDP2_FS_I_DIT_O_REG          0x2452    
#define IDP3_FS_I_DIT_O_REG          0x2453    /* SRU_FS3 */
#define IDP4_FS_I_DIT_O_REG          0x2453    
#define IDP5_FS_I_DIT_O_REG          0x2453    
#define IDP6_FS_I_DIT_O_REG          0x2453    
#define IDP7_FS_I_DIT_O_REG          0x2453        
#define DIR_FS_I_DIT_O_REG           0x2453


#define SPORT0_FS_I_PCG_FSA_O_REG        0x2450    /* SRU_FS0 */
#define SPORT1_FS_I_PCG_FSA_O_REG        0x2450    
#define SPORT2_FS_I_PCG_FSA_O_REG        0x2450    
#define SPORT3_FS_I_PCG_FSA_O_REG        0x2450    
#define SPORT4_FS_I_PCG_FSA_O_REG        0x2450    
#define SPORT5_FS_I_PCG_FSA_O_REG        0x2450    
#define SRC0_FS_IP_I_PCG_FSA_O_REG       0x2451    /* SRU_FS1 */
#define SRC0_FS_0P_I_PCG_FSA_O_REG       0x2451    
#define SRC1_FS_IP_I_PCG_FSA_O_REG       0x2451    
#define SRC1_FS_OP_I_PCG_FSA_O_REG       0x2451    
#define SRC2_FS_IP_I_PCG_FSA_O_REG       0x2451    
#define SRC2_FS_OP_I_PCG_FSA_O_REG       0x2451    
#define SRC3_FS_IP_I_PCG_FSA_O_REG       0x2452    /* SRU_FS2 */
#define SRC3_FS_OP_I_PCG_FSA_O_REG       0x2452    
#define DIT_I_PCG_FSA_O_REG           0x2452    
#define PDAP_HOLD_I_PCG_FSA_O_REG        0x2452    
#define IDP0_FS_I_PCG_FSA_O_REG          0x2452    
#define IDP1_FS_I_PCG_FSA_O_REG          0x2452    
#define IDP2_FS_I_PCG_FSA_O_REG          0x2452    
#define IDP3_FS_I_PCG_FSA_O_REG          0x2453    /* SRU_FS3 */
#define IDP4_FS_I_PCG_FSA_O_REG          0x2453    
#define IDP5_FS_I_PCG_FSA_O_REG          0x2453    
#define IDP6_FS_I_PCG_FSA_O_REG          0x2453    
#define IDP7_FS_I_PCG_FSA_O_REG          0x2453        
#define DIR_FS_I_PCG_FSA_O_REG           0x2453

#define SPORT0_FS_I_PCG_FSB_O_REG        0x2450    /* SRU_FS0 */
#define SPORT1_FS_I_PCG_FSB_O_REG        0x2450    
#define SPORT2_FS_I_PCG_FSB_O_REG        0x2450    
#define SPORT3_FS_I_PCG_FSB_O_REG        0x2450    
#define SPORT4_FS_I_PCG_FSB_O_REG        0x2450    
#define SPORT5_FS_I_PCG_FSB_O_REG        0x2450    
#define SRC0_FS_IP_I_PCG_FSB_O_REG       0x2451    /* SRU_FS1 */
#define SRC0_FS_0P_I_PCG_FSB_O_REG       0x2451    
#define SRC1_FS_IP_I_PCG_FSB_O_REG       0x2451    
#define SRC1_FS_OP_I_PCG_FSB_O_REG       0x2451    
#define SRC2_FS_IP_I_PCG_FSB_O_REG       0x2451    
#define SRC2_FS_OP_I_PCG_FSB_O_REG       0x2451    
#define SRC3_FS_IP_I_PCG_FSB_O_REG       0x2452    /* SRU_FS2 */
#define SRC3_FS_OP_I_PCG_FSB_O_REG       0x2452    
#define DIT_I_PCG_FSB_O_REG           0x2452    
#define PDAP_HOLD_I_PCG_FSB_O_REG        0x2452    
#define IDP0_FS_I_PCG_FSB_O_REG          0x2452    
#define IDP1_FS_I_PCG_FSB_O_REG          0x2452    
#define IDP2_FS_I_PCG_FSB_O_REG          0x2452    
#define IDP3_FS_I_PCG_FSB_O_REG          0x2453    /* SRU_FS3 */
#define IDP4_FS_I_PCG_FSB_O_REG          0x2453    
#define IDP5_FS_I_PCG_FSB_O_REG          0x2453    
#define IDP6_FS_I_PCG_FSB_O_REG          0x2453    
#define IDP7_FS_I_PCG_FSB_O_REG          0x2453        
#define DIR_FS_I_PCG_FSB_O_REG           0x2453

#define SPORT0_FS_I_LOW_REG        0x2450    /* SRU_FS0 */
#define SPORT1_FS_I_LOW_REG        0x2450    
#define SPORT2_FS_I_LOW_REG        0x2450    
#define SPORT3_FS_I_LOW_REG        0x2450    
#define SPORT4_FS_I_LOW_REG        0x2450    
#define SPORT5_FS_I_LOW_REG        0x2450    
#define SRC0_FS_IP_I_LOW_REG       0x2451    /* SRU_FS1 */
#define SRC0_FS_0P_I_LOW_REG       0x2451    
#define SRC1_FS_IP_I_LOW_REG       0x2451    
#define SRC1_FS_OP_I_LOW_REG       0x2451    
#define SRC2_FS_IP_I_LOW_REG       0x2451    
#define SRC2_FS_OP_I_LOW_REG       0x2451    
#define SRC3_FS_IP_I_LOW_REG       0x2452    /* SRU_FS2 */
#define SRC3_FS_OP_I_LOW_REG       0x2452    
#define DIT_I_LOW_REG           0x2452    
#define PDAP_HOLD_I_LOW_REG        0x2452    
#define IDP0_FS_I_LOW_REG          0x2452    
#define IDP1_FS_I_LOW_REG          0x2452    
#define IDP2_FS_I_LOW_REG          0x2452    
#define IDP3_FS_I_LOW_REG          0x2453    /* SRU_FS3 */
#define IDP4_FS_I_LOW_REG          0x2453    
#define IDP5_FS_I_LOW_REG          0x2453    
#define IDP6_FS_I_LOW_REG          0x2453    
#define IDP7_FS_I_LOW_REG          0x2453        
#define DIR_FS_I_LOW_REG           0x2453

#define SPORT0_FS_I_HIGH_REG        0x2450    /* SRU_FS0 */
#define SPORT1_FS_I_HIGH_REG        0x2450    
#define SPORT2_FS_I_HIGH_REG        0x2450    
#define SPORT3_FS_I_HIGH_REG        0x2450    
#define SPORT4_FS_I_HIGH_REG        0x2450    
#define SPORT5_FS_I_HIGH_REG        0x2450    
#define SRC0_FS_IP_I_HIGH_REG       0x2451    /* SRU_FS1 */
#define SRC0_FS_0P_I_HIGH_REG       0x2451    
#define SRC1_FS_IP_I_HIGH_REG       0x2451    
#define SRC1_FS_OP_I_HIGH_REG       0x2451    
#define SRC2_FS_IP_I_HIGH_REG       0x2451    
#define SRC2_FS_OP_I_HIGH_REG       0x2451    
#define SRC3_FS_IP_I_HIGH_REG       0x2452    /* SRU_FS2 */
#define SRC3_FS_OP_I_HIGH_REG       0x2452    
#define DIT_I_HIGH_REG           0x2452    
#define PDAP_HOLD_I_HIGH_REG        0x2452    
#define IDP0_FS_I_HIGH_REG          0x2452    
#define IDP1_FS_I_HIGH_REG          0x2452    
#define IDP2_FS_I_HIGH_REG          0x2452    
#define IDP3_FS_I_HIGH_REG          0x2453    /* SRU_FS3 */
#define IDP4_FS_I_HIGH_REG          0x2453    
#define IDP5_FS_I_HIGH_REG          0x2453    
#define IDP6_FS_I_HIGH_REG          0x2453    
#define IDP7_FS_I_HIGH_REG          0x2453        
#define DIR_FS_I_HIGH_REG           0x2453





/*============================================================================*
 *                                                                                                                                                    *
 *      GROUP D                                                                   *
 *                                                                                                                                                        *
 *============================================================================*/

//------------------------------------------------------------------------------
//  Group D: Input bitfield masks & shifts
//------------------------------------------------------------------------------

//SRU_PIN0
  #define    DAI_PB01_I_MASK                  0xFFFFF80
  #define    DAI_PB01_I_SHIFT                 0
  #define    DAI_PB02_I_MASK                  0xFFFC07F
  #define    DAI_PB02_I_SHIFT                 7
  #define    DAI_PB03_I_MASK                  0xFE03FFF
  #define    DAI_PB03_I_SHIFT                 14
  #define    DAI_PB04_I_MASK                  0x1FFFFF
  #define    DAI_PB04_I_SHIFT                 21

//SRU_PIN1 
  #define    DAI_PB05_I_MASK                  0xFFFFF80
  #define    DAI_PB05_I_SHIFT                 0        
  #define    DAI_PB06_I_MASK                  0xFFFC07F
  #define    DAI_PB06_I_SHIFT                 7        
  #define    DAI_PB07_I_MASK                  0xFE03FFF
  #define    DAI_PB07_I_SHIFT                 14       
  #define    DAI_PB08_I_MASK                  0x1FFFFF 
  #define    DAI_PB08_I_SHIFT                 21       

//SRU_PIN2
  #define    DAI_PB09_I_MASK                  0xFFFFF80
  #define    DAI_PB09_I_SHIFT                 0        
  #define    DAI_PB10_I_MASK                  0xFFFC07F
  #define    DAI_PB10_I_SHIFT                 7        
  #define    DAI_PB11_I_MASK                  0xFE03FFF
  #define    DAI_PB11_I_SHIFT                 14       
  #define    DAI_PB12_I_MASK                  0x1FFFFF 
  #define    DAI_PB12_I_SHIFT                 21       

//SRU_PIN3
  #define    DAI_PB13_I_MASK                  0xFFFFF80
  #define    DAI_PB13_I_SHIFT                 0        
  #define    DAI_PB14_I_MASK                  0xFFFC07F
  #define    DAI_PB14_I_SHIFT                 7        
  #define    DAI_PB15_I_MASK                  0xFE03FFF
  #define    DAI_PB15_I_SHIFT                 14       
  #define    DAI_PB16_I_MASK                  0x1FFFFF 
  #define    DAI_PB16_I_SHIFT                 21       

//SRU_PIN4
  #define    DAI_PB17_I_MASK                  0x3FFFFF80 
  #define    DAI_PB17_I_SHIFT                 0        
  #define    DAI_PB18_I_MASK                  0x3FFFC07F
  #define    DAI_PB18_I_SHIFT                 7        
  #define    DAI_PB19_I_MASK                  0x3FE03FFF
  #define    DAI_PB19_I_SHIFT                 14       
  #define    DAI_PB20_I_MASK                  0x301FFFFF 
  #define    DAI_PB20_I_SHIFT                 21       
  #define    INV_DAI_PB19_SHIFT		      0x2FFFFFFF
  #define    INV_DAI_PB19_MASK 		      28
  #define    INV_DAI_PB20_SHIFT		      0x1FFFFFFF
  #define    INV_DAI_PB20_MASK 		      29
  
//------------------------------------------------------------------------------
//  Group D: Bitfield encodings
//------------------------------------------------------------------------------

#define DAI_PB01_I_DAI_PB01_O_ENCODING              0x00
#define DAI_PB02_I_DAI_PB01_O_ENCODING              0x00
#define DAI_PB03_I_DAI_PB01_O_ENCODING              0x00
#define DAI_PB04_I_DAI_PB01_O_ENCODING              0x00
#define DAI_PB05_I_DAI_PB01_O_ENCODING              0x00
#define DAI_PB06_I_DAI_PB01_O_ENCODING              0x00
#define DAI_PB07_I_DAI_PB01_O_ENCODING              0x00
#define DAI_PB08_I_DAI_PB01_O_ENCODING              0x00
#define DAI_PB09_I_DAI_PB01_O_ENCODING              0x00
#define DAI_PB10_I_DAI_PB01_O_ENCODING              0x00
#define DAI_PB11_I_DAI_PB01_O_ENCODING              0x00
#define DAI_PB12_I_DAI_PB01_O_ENCODING              0x00
#define DAI_PB13_I_DAI_PB01_O_ENCODING              0x00
#define DAI_PB14_I_DAI_PB01_O_ENCODING              0x00
#define DAI_PB15_I_DAI_PB01_O_ENCODING              0x00
#define DAI_PB16_I_DAI_PB01_O_ENCODING              0x00
#define DAI_PB17_I_DAI_PB01_O_ENCODING              0x00
#define DAI_PB18_I_DAI_PB01_O_ENCODING              0x00
#define DAI_PB19_I_DAI_PB01_O_ENCODING              0x00
#define DAI_PB20_I_DAI_PB01_O_ENCODING              0x00

#define DAI_PB01_I_DAI_PB02_O_ENCODING              0x01
#define DAI_PB02_I_DAI_PB02_O_ENCODING              0x01
#define DAI_PB03_I_DAI_PB02_O_ENCODING              0x01
#define DAI_PB04_I_DAI_PB02_O_ENCODING              0x01
#define DAI_PB05_I_DAI_PB02_O_ENCODING              0x01
#define DAI_PB06_I_DAI_PB02_O_ENCODING              0x01
#define DAI_PB07_I_DAI_PB02_O_ENCODING              0x01
#define DAI_PB08_I_DAI_PB02_O_ENCODING              0x01
#define DAI_PB09_I_DAI_PB02_O_ENCODING              0x01
#define DAI_PB10_I_DAI_PB02_O_ENCODING              0x01
#define DAI_PB11_I_DAI_PB02_O_ENCODING              0x01
#define DAI_PB12_I_DAI_PB02_O_ENCODING              0x01
#define DAI_PB13_I_DAI_PB02_O_ENCODING              0x01
#define DAI_PB14_I_DAI_PB02_O_ENCODING              0x01
#define DAI_PB15_I_DAI_PB02_O_ENCODING              0x01
#define DAI_PB16_I_DAI_PB02_O_ENCODING              0x01
#define DAI_PB17_I_DAI_PB02_O_ENCODING              0x01
#define DAI_PB18_I_DAI_PB02_O_ENCODING              0x01
#define DAI_PB19_I_DAI_PB02_O_ENCODING              0x01
#define DAI_PB20_I_DAI_PB02_O_ENCODING              0x01

#define DAI_PB01_I_DAI_PB03_O_ENCODING              0x02
#define DAI_PB02_I_DAI_PB03_O_ENCODING              0x02
#define DAI_PB03_I_DAI_PB03_O_ENCODING              0x02
#define DAI_PB04_I_DAI_PB03_O_ENCODING              0x02
#define DAI_PB05_I_DAI_PB03_O_ENCODING              0x02
#define DAI_PB06_I_DAI_PB03_O_ENCODING              0x02
#define DAI_PB07_I_DAI_PB03_O_ENCODING              0x02
#define DAI_PB08_I_DAI_PB03_O_ENCODING              0x02
#define DAI_PB09_I_DAI_PB03_O_ENCODING              0x02
#define DAI_PB10_I_DAI_PB03_O_ENCODING              0x02
#define DAI_PB11_I_DAI_PB03_O_ENCODING              0x02
#define DAI_PB12_I_DAI_PB03_O_ENCODING              0x02
#define DAI_PB13_I_DAI_PB03_O_ENCODING              0x02
#define DAI_PB14_I_DAI_PB03_O_ENCODING              0x02
#define DAI_PB15_I_DAI_PB03_O_ENCODING              0x02
#define DAI_PB16_I_DAI_PB03_O_ENCODING              0x02
#define DAI_PB17_I_DAI_PB03_O_ENCODING              0x02
#define DAI_PB18_I_DAI_PB03_O_ENCODING              0x02
#define DAI_PB19_I_DAI_PB03_O_ENCODING              0x02
#define DAI_PB20_I_DAI_PB03_O_ENCODING              0x02

#define DAI_PB01_I_DAI_PB04_O_ENCODING              0x03
#define DAI_PB02_I_DAI_PB04_O_ENCODING              0x03
#define DAI_PB03_I_DAI_PB04_O_ENCODING              0x03
#define DAI_PB04_I_DAI_PB04_O_ENCODING              0x03
#define DAI_PB05_I_DAI_PB04_O_ENCODING              0x03
#define DAI_PB06_I_DAI_PB04_O_ENCODING              0x03
#define DAI_PB07_I_DAI_PB04_O_ENCODING              0x03
#define DAI_PB08_I_DAI_PB04_O_ENCODING              0x03
#define DAI_PB09_I_DAI_PB04_O_ENCODING              0x03
#define DAI_PB10_I_DAI_PB04_O_ENCODING              0x03
#define DAI_PB11_I_DAI_PB04_O_ENCODING              0x03
#define DAI_PB12_I_DAI_PB04_O_ENCODING              0x03
#define DAI_PB13_I_DAI_PB04_O_ENCODING              0x03
#define DAI_PB14_I_DAI_PB04_O_ENCODING              0x03
#define DAI_PB15_I_DAI_PB04_O_ENCODING              0x03
#define DAI_PB16_I_DAI_PB04_O_ENCODING              0x03
#define DAI_PB17_I_DAI_PB04_O_ENCODING              0x03
#define DAI_PB18_I_DAI_PB04_O_ENCODING              0x03
#define DAI_PB19_I_DAI_PB04_O_ENCODING              0x03
#define DAI_PB20_I_DAI_PB04_O_ENCODING              0x03

#define DAI_PB01_I_DAI_PB05_O_ENCODING              0x04
#define DAI_PB02_I_DAI_PB05_O_ENCODING              0x04
#define DAI_PB03_I_DAI_PB05_O_ENCODING              0x04
#define DAI_PB04_I_DAI_PB05_O_ENCODING              0x04
#define DAI_PB05_I_DAI_PB05_O_ENCODING              0x04
#define DAI_PB06_I_DAI_PB05_O_ENCODING              0x04
#define DAI_PB07_I_DAI_PB05_O_ENCODING              0x04
#define DAI_PB08_I_DAI_PB05_O_ENCODING              0x04
#define DAI_PB09_I_DAI_PB05_O_ENCODING              0x04
#define DAI_PB10_I_DAI_PB05_O_ENCODING              0x04
#define DAI_PB11_I_DAI_PB05_O_ENCODING              0x04
#define DAI_PB12_I_DAI_PB05_O_ENCODING              0x04
#define DAI_PB13_I_DAI_PB05_O_ENCODING              0x04
#define DAI_PB14_I_DAI_PB05_O_ENCODING              0x04
#define DAI_PB15_I_DAI_PB05_O_ENCODING              0x04
#define DAI_PB16_I_DAI_PB05_O_ENCODING              0x04
#define DAI_PB17_I_DAI_PB05_O_ENCODING              0x04
#define DAI_PB18_I_DAI_PB05_O_ENCODING              0x04
#define DAI_PB19_I_DAI_PB05_O_ENCODING              0x04
#define DAI_PB20_I_DAI_PB05_O_ENCODING              0x04

#define DAI_PB01_I_DAI_PB06_O_ENCODING              0x05
#define DAI_PB02_I_DAI_PB06_O_ENCODING              0x05
#define DAI_PB03_I_DAI_PB06_O_ENCODING              0x05
#define DAI_PB04_I_DAI_PB06_O_ENCODING              0x05
#define DAI_PB05_I_DAI_PB06_O_ENCODING              0x05
#define DAI_PB06_I_DAI_PB06_O_ENCODING              0x05
#define DAI_PB07_I_DAI_PB06_O_ENCODING              0x05
#define DAI_PB08_I_DAI_PB06_O_ENCODING              0x05
#define DAI_PB09_I_DAI_PB06_O_ENCODING              0x05
#define DAI_PB10_I_DAI_PB06_O_ENCODING              0x05
#define DAI_PB11_I_DAI_PB06_O_ENCODING              0x05
#define DAI_PB12_I_DAI_PB06_O_ENCODING              0x05
#define DAI_PB13_I_DAI_PB06_O_ENCODING              0x05
#define DAI_PB14_I_DAI_PB06_O_ENCODING              0x05
#define DAI_PB15_I_DAI_PB06_O_ENCODING              0x05
#define DAI_PB16_I_DAI_PB06_O_ENCODING              0x05
#define DAI_PB17_I_DAI_PB06_O_ENCODING              0x05
#define DAI_PB18_I_DAI_PB06_O_ENCODING              0x05
#define DAI_PB19_I_DAI_PB06_O_ENCODING              0x05
#define DAI_PB20_I_DAI_PB06_O_ENCODING              0x05

#define DAI_PB01_I_DAI_PB07_O_ENCODING              0x06
#define DAI_PB02_I_DAI_PB07_O_ENCODING              0x06
#define DAI_PB03_I_DAI_PB07_O_ENCODING              0x06
#define DAI_PB04_I_DAI_PB07_O_ENCODING              0x06
#define DAI_PB05_I_DAI_PB07_O_ENCODING              0x06
#define DAI_PB06_I_DAI_PB07_O_ENCODING              0x06
#define DAI_PB07_I_DAI_PB07_O_ENCODING              0x06
#define DAI_PB08_I_DAI_PB07_O_ENCODING              0x06
#define DAI_PB09_I_DAI_PB07_O_ENCODING              0x06
#define DAI_PB10_I_DAI_PB07_O_ENCODING              0x06
#define DAI_PB11_I_DAI_PB07_O_ENCODING              0x06
#define DAI_PB12_I_DAI_PB07_O_ENCODING              0x06
#define DAI_PB13_I_DAI_PB07_O_ENCODING              0x06
#define DAI_PB14_I_DAI_PB07_O_ENCODING              0x06
#define DAI_PB15_I_DAI_PB07_O_ENCODING              0x06
#define DAI_PB16_I_DAI_PB07_O_ENCODING              0x06
#define DAI_PB17_I_DAI_PB07_O_ENCODING              0x06
#define DAI_PB18_I_DAI_PB07_O_ENCODING              0x06
#define DAI_PB19_I_DAI_PB07_O_ENCODING              0x06
#define DAI_PB20_I_DAI_PB07_O_ENCODING              0x06

#define DAI_PB01_I_DAI_PB08_O_ENCODING              0x07
#define DAI_PB02_I_DAI_PB08_O_ENCODING              0x07
#define DAI_PB03_I_DAI_PB08_O_ENCODING              0x07
#define DAI_PB04_I_DAI_PB08_O_ENCODING              0x07
#define DAI_PB05_I_DAI_PB08_O_ENCODING              0x07
#define DAI_PB06_I_DAI_PB08_O_ENCODING              0x07
#define DAI_PB07_I_DAI_PB08_O_ENCODING              0x07
#define DAI_PB08_I_DAI_PB08_O_ENCODING              0x07
#define DAI_PB09_I_DAI_PB08_O_ENCODING              0x07
#define DAI_PB10_I_DAI_PB08_O_ENCODING              0x07
#define DAI_PB11_I_DAI_PB08_O_ENCODING              0x07
#define DAI_PB12_I_DAI_PB08_O_ENCODING              0x07
#define DAI_PB13_I_DAI_PB08_O_ENCODING              0x07
#define DAI_PB14_I_DAI_PB08_O_ENCODING              0x07
#define DAI_PB15_I_DAI_PB08_O_ENCODING              0x07
#define DAI_PB16_I_DAI_PB08_O_ENCODING              0x07
#define DAI_PB17_I_DAI_PB08_O_ENCODING              0x07
#define DAI_PB18_I_DAI_PB08_O_ENCODING              0x07
#define DAI_PB19_I_DAI_PB08_O_ENCODING              0x07
#define DAI_PB20_I_DAI_PB08_O_ENCODING              0x07

#define DAI_PB01_I_DAI_PB09_O_ENCODING              0x08
#define DAI_PB02_I_DAI_PB09_O_ENCODING              0x08
#define DAI_PB03_I_DAI_PB09_O_ENCODING              0x08
#define DAI_PB04_I_DAI_PB09_O_ENCODING              0x08
#define DAI_PB05_I_DAI_PB09_O_ENCODING              0x08
#define DAI_PB06_I_DAI_PB09_O_ENCODING              0x08
#define DAI_PB07_I_DAI_PB09_O_ENCODING              0x08
#define DAI_PB08_I_DAI_PB09_O_ENCODING              0x08
#define DAI_PB09_I_DAI_PB09_O_ENCODING              0x08
#define DAI_PB10_I_DAI_PB09_O_ENCODING              0x08
#define DAI_PB11_I_DAI_PB09_O_ENCODING              0x08
#define DAI_PB12_I_DAI_PB09_O_ENCODING              0x08
#define DAI_PB13_I_DAI_PB09_O_ENCODING              0x08
#define DAI_PB14_I_DAI_PB09_O_ENCODING              0x08
#define DAI_PB15_I_DAI_PB09_O_ENCODING              0x08
#define DAI_PB16_I_DAI_PB09_O_ENCODING              0x08
#define DAI_PB17_I_DAI_PB09_O_ENCODING              0x08
#define DAI_PB18_I_DAI_PB09_O_ENCODING              0x08
#define DAI_PB19_I_DAI_PB09_O_ENCODING              0x08
#define DAI_PB20_I_DAI_PB09_O_ENCODING              0x08

#define DAI_PB01_I_DAI_PB10_O_ENCODING              0x09
#define DAI_PB02_I_DAI_PB10_O_ENCODING              0x09
#define DAI_PB03_I_DAI_PB10_O_ENCODING              0x09
#define DAI_PB04_I_DAI_PB10_O_ENCODING              0x09
#define DAI_PB05_I_DAI_PB10_O_ENCODING              0x09
#define DAI_PB06_I_DAI_PB10_O_ENCODING              0x09
#define DAI_PB07_I_DAI_PB10_O_ENCODING              0x09
#define DAI_PB08_I_DAI_PB10_O_ENCODING              0x09
#define DAI_PB09_I_DAI_PB10_O_ENCODING              0x09
#define DAI_PB10_I_DAI_PB10_O_ENCODING              0x09
#define DAI_PB11_I_DAI_PB10_O_ENCODING              0x09
#define DAI_PB12_I_DAI_PB10_O_ENCODING              0x09
#define DAI_PB13_I_DAI_PB10_O_ENCODING              0x09
#define DAI_PB14_I_DAI_PB10_O_ENCODING              0x09
#define DAI_PB15_I_DAI_PB10_O_ENCODING              0x09
#define DAI_PB16_I_DAI_PB10_O_ENCODING              0x09
#define DAI_PB17_I_DAI_PB10_O_ENCODING              0x09
#define DAI_PB18_I_DAI_PB10_O_ENCODING              0x09
#define DAI_PB19_I_DAI_PB10_O_ENCODING              0x09
#define DAI_PB20_I_DAI_PB10_O_ENCODING              0x09

#define DAI_PB01_I_DAI_PB11_O_ENCODING              0x0A
#define DAI_PB02_I_DAI_PB11_O_ENCODING              0x0A
#define DAI_PB03_I_DAI_PB11_O_ENCODING              0x0A
#define DAI_PB04_I_DAI_PB11_O_ENCODING              0x0A
#define DAI_PB05_I_DAI_PB11_O_ENCODING              0x0A
#define DAI_PB06_I_DAI_PB11_O_ENCODING              0x0A
#define DAI_PB07_I_DAI_PB11_O_ENCODING              0x0A
#define DAI_PB08_I_DAI_PB11_O_ENCODING              0x0A
#define DAI_PB09_I_DAI_PB11_O_ENCODING              0x0A
#define DAI_PB10_I_DAI_PB11_O_ENCODING              0x0A
#define DAI_PB11_I_DAI_PB11_O_ENCODING              0x0A
#define DAI_PB12_I_DAI_PB11_O_ENCODING              0x0A
#define DAI_PB13_I_DAI_PB11_O_ENCODING              0x0A
#define DAI_PB14_I_DAI_PB11_O_ENCODING              0x0A
#define DAI_PB15_I_DAI_PB11_O_ENCODING              0x0A
#define DAI_PB16_I_DAI_PB11_O_ENCODING              0x0A
#define DAI_PB17_I_DAI_PB11_O_ENCODING              0x0A
#define DAI_PB18_I_DAI_PB11_O_ENCODING              0x0A
#define DAI_PB19_I_DAI_PB11_O_ENCODING              0x0A
#define DAI_PB20_I_DAI_PB11_O_ENCODING              0x0A

#define DAI_PB01_I_DAI_PB12_O_ENCODING              0x0B
#define DAI_PB02_I_DAI_PB12_O_ENCODING              0x0B
#define DAI_PB03_I_DAI_PB12_O_ENCODING              0x0B
#define DAI_PB04_I_DAI_PB12_O_ENCODING              0x0B
#define DAI_PB05_I_DAI_PB12_O_ENCODING              0x0B
#define DAI_PB06_I_DAI_PB12_O_ENCODING              0x0B
#define DAI_PB07_I_DAI_PB12_O_ENCODING              0x0B
#define DAI_PB08_I_DAI_PB12_O_ENCODING              0x0B
#define DAI_PB09_I_DAI_PB12_O_ENCODING              0x0B
#define DAI_PB10_I_DAI_PB12_O_ENCODING              0x0B
#define DAI_PB11_I_DAI_PB12_O_ENCODING              0x0B
#define DAI_PB12_I_DAI_PB12_O_ENCODING              0x0B
#define DAI_PB13_I_DAI_PB12_O_ENCODING              0x0B
#define DAI_PB14_I_DAI_PB12_O_ENCODING              0x0B
#define DAI_PB15_I_DAI_PB12_O_ENCODING              0x0B
#define DAI_PB16_I_DAI_PB12_O_ENCODING              0x0B
#define DAI_PB17_I_DAI_PB12_O_ENCODING              0x0B
#define DAI_PB18_I_DAI_PB12_O_ENCODING              0x0B
#define DAI_PB19_I_DAI_PB12_O_ENCODING              0x0B
#define DAI_PB20_I_DAI_PB12_O_ENCODING              0x0B

#define DAI_PB01_I_DAI_PB13_O_ENCODING              0x0C
#define DAI_PB02_I_DAI_PB13_O_ENCODING              0x0C
#define DAI_PB03_I_DAI_PB13_O_ENCODING              0x0C
#define DAI_PB04_I_DAI_PB13_O_ENCODING              0x0C
#define DAI_PB05_I_DAI_PB13_O_ENCODING              0x0C
#define DAI_PB06_I_DAI_PB13_O_ENCODING              0x0C
#define DAI_PB07_I_DAI_PB13_O_ENCODING              0x0C
#define DAI_PB08_I_DAI_PB13_O_ENCODING              0x0C
#define DAI_PB09_I_DAI_PB13_O_ENCODING              0x0C
#define DAI_PB10_I_DAI_PB13_O_ENCODING              0x0C
#define DAI_PB11_I_DAI_PB13_O_ENCODING              0x0C
#define DAI_PB12_I_DAI_PB13_O_ENCODING              0x0C
#define DAI_PB13_I_DAI_PB13_O_ENCODING              0x0C
#define DAI_PB14_I_DAI_PB13_O_ENCODING              0x0C
#define DAI_PB15_I_DAI_PB13_O_ENCODING              0x0C
#define DAI_PB16_I_DAI_PB13_O_ENCODING              0x0C
#define DAI_PB17_I_DAI_PB13_O_ENCODING              0x0C
#define DAI_PB18_I_DAI_PB13_O_ENCODING              0x0C
#define DAI_PB19_I_DAI_PB13_O_ENCODING              0x0C
#define DAI_PB20_I_DAI_PB13_O_ENCODING              0x0C

#define DAI_PB01_I_DAI_PB14_O_ENCODING              0x0D
#define DAI_PB02_I_DAI_PB14_O_ENCODING              0x0D
#define DAI_PB03_I_DAI_PB14_O_ENCODING              0x0D
#define DAI_PB04_I_DAI_PB14_O_ENCODING              0x0D
#define DAI_PB05_I_DAI_PB14_O_ENCODING              0x0D
#define DAI_PB06_I_DAI_PB14_O_ENCODING              0x0D
#define DAI_PB07_I_DAI_PB14_O_ENCODING              0x0D
#define DAI_PB08_I_DAI_PB14_O_ENCODING              0x0D
#define DAI_PB09_I_DAI_PB14_O_ENCODING              0x0D
#define DAI_PB10_I_DAI_PB14_O_ENCODING              0x0D
#define DAI_PB11_I_DAI_PB14_O_ENCODING              0x0D
#define DAI_PB12_I_DAI_PB14_O_ENCODING              0x0D
#define DAI_PB13_I_DAI_PB14_O_ENCODING              0x0D
#define DAI_PB14_I_DAI_PB14_O_ENCODING              0x0D
#define DAI_PB15_I_DAI_PB14_O_ENCODING              0x0D
#define DAI_PB16_I_DAI_PB14_O_ENCODING              0x0D
#define DAI_PB17_I_DAI_PB14_O_ENCODING              0x0D
#define DAI_PB18_I_DAI_PB14_O_ENCODING              0x0D
#define DAI_PB19_I_DAI_PB14_O_ENCODING              0x0D
#define DAI_PB20_I_DAI_PB14_O_ENCODING              0x0D

#define DAI_PB01_I_DAI_PB15_O_ENCODING              0x0E
#define DAI_PB02_I_DAI_PB15_O_ENCODING              0x0E
#define DAI_PB03_I_DAI_PB15_O_ENCODING              0x0E
#define DAI_PB04_I_DAI_PB15_O_ENCODING              0x0E
#define DAI_PB05_I_DAI_PB15_O_ENCODING              0x0E
#define DAI_PB06_I_DAI_PB15_O_ENCODING              0x0E
#define DAI_PB07_I_DAI_PB15_O_ENCODING              0x0E
#define DAI_PB08_I_DAI_PB15_O_ENCODING              0x0E
#define DAI_PB09_I_DAI_PB15_O_ENCODING              0x0E
#define DAI_PB10_I_DAI_PB15_O_ENCODING              0x0E
#define DAI_PB11_I_DAI_PB15_O_ENCODING              0x0E
#define DAI_PB12_I_DAI_PB15_O_ENCODING              0x0E
#define DAI_PB13_I_DAI_PB15_O_ENCODING              0x0E
#define DAI_PB14_I_DAI_PB15_O_ENCODING              0x0E
#define DAI_PB15_I_DAI_PB15_O_ENCODING              0x0E
#define DAI_PB16_I_DAI_PB15_O_ENCODING              0x0E
#define DAI_PB17_I_DAI_PB15_O_ENCODING              0x0E
#define DAI_PB18_I_DAI_PB15_O_ENCODING              0x0E
#define DAI_PB19_I_DAI_PB15_O_ENCODING              0x0E
#define DAI_PB20_I_DAI_PB15_O_ENCODING              0x0E

#define DAI_PB01_I_DAI_PB16_O_ENCODING              0x0F
#define DAI_PB02_I_DAI_PB16_O_ENCODING              0x0F
#define DAI_PB03_I_DAI_PB16_O_ENCODING              0x0F
#define DAI_PB04_I_DAI_PB16_O_ENCODING              0x0F
#define DAI_PB05_I_DAI_PB16_O_ENCODING              0x0F
#define DAI_PB06_I_DAI_PB16_O_ENCODING              0x0F
#define DAI_PB07_I_DAI_PB16_O_ENCODING              0x0F
#define DAI_PB08_I_DAI_PB16_O_ENCODING              0x0F
#define DAI_PB09_I_DAI_PB16_O_ENCODING              0x0F
#define DAI_PB10_I_DAI_PB16_O_ENCODING              0x0F
#define DAI_PB11_I_DAI_PB16_O_ENCODING              0x0F
#define DAI_PB12_I_DAI_PB16_O_ENCODING              0x0F
#define DAI_PB13_I_DAI_PB16_O_ENCODING              0x0F
#define DAI_PB14_I_DAI_PB16_O_ENCODING              0x0F
#define DAI_PB15_I_DAI_PB16_O_ENCODING              0x0F
#define DAI_PB16_I_DAI_PB16_O_ENCODING              0x0F
#define DAI_PB17_I_DAI_PB16_O_ENCODING              0x0F
#define DAI_PB18_I_DAI_PB16_O_ENCODING              0x0F
#define DAI_PB19_I_DAI_PB16_O_ENCODING              0x0F
#define DAI_PB20_I_DAI_PB16_O_ENCODING              0x0F

#define DAI_PB01_I_DAI_PB17_O_ENCODING              0x10
#define DAI_PB02_I_DAI_PB17_O_ENCODING              0x10
#define DAI_PB03_I_DAI_PB17_O_ENCODING              0x10
#define DAI_PB04_I_DAI_PB17_O_ENCODING              0x10
#define DAI_PB05_I_DAI_PB17_O_ENCODING              0x10
#define DAI_PB06_I_DAI_PB17_O_ENCODING              0x10
#define DAI_PB07_I_DAI_PB17_O_ENCODING              0x10
#define DAI_PB08_I_DAI_PB17_O_ENCODING              0x10
#define DAI_PB09_I_DAI_PB17_O_ENCODING              0x10
#define DAI_PB10_I_DAI_PB17_O_ENCODING              0x10
#define DAI_PB11_I_DAI_PB17_O_ENCODING              0x10
#define DAI_PB12_I_DAI_PB17_O_ENCODING              0x10
#define DAI_PB13_I_DAI_PB17_O_ENCODING              0x10
#define DAI_PB14_I_DAI_PB17_O_ENCODING              0x10
#define DAI_PB15_I_DAI_PB17_O_ENCODING              0x10
#define DAI_PB16_I_DAI_PB17_O_ENCODING              0x10
#define DAI_PB17_I_DAI_PB17_O_ENCODING              0x10
#define DAI_PB18_I_DAI_PB17_O_ENCODING              0x10
#define DAI_PB19_I_DAI_PB17_O_ENCODING              0x10
#define DAI_PB20_I_DAI_PB17_O_ENCODING              0x10

#define DAI_PB01_I_DAI_PB18_O_ENCODING              0x11
#define DAI_PB02_I_DAI_PB18_O_ENCODING              0x11
#define DAI_PB03_I_DAI_PB18_O_ENCODING              0x11
#define DAI_PB04_I_DAI_PB18_O_ENCODING              0x11
#define DAI_PB05_I_DAI_PB18_O_ENCODING              0x11
#define DAI_PB06_I_DAI_PB18_O_ENCODING              0x11
#define DAI_PB07_I_DAI_PB18_O_ENCODING              0x11
#define DAI_PB08_I_DAI_PB18_O_ENCODING              0x11
#define DAI_PB09_I_DAI_PB18_O_ENCODING              0x11
#define DAI_PB10_I_DAI_PB18_O_ENCODING              0x11
#define DAI_PB11_I_DAI_PB18_O_ENCODING              0x11
#define DAI_PB12_I_DAI_PB18_O_ENCODING              0x11
#define DAI_PB13_I_DAI_PB18_O_ENCODING              0x11
#define DAI_PB14_I_DAI_PB18_O_ENCODING              0x11
#define DAI_PB15_I_DAI_PB18_O_ENCODING              0x11
#define DAI_PB16_I_DAI_PB18_O_ENCODING              0x11
#define DAI_PB17_I_DAI_PB18_O_ENCODING              0x11
#define DAI_PB18_I_DAI_PB18_O_ENCODING              0x11
#define DAI_PB19_I_DAI_PB18_O_ENCODING              0x11
#define DAI_PB20_I_DAI_PB18_O_ENCODING              0x11

#define DAI_PB01_I_DAI_PB19_O_ENCODING              0x12
#define DAI_PB02_I_DAI_PB19_O_ENCODING              0x12
#define DAI_PB03_I_DAI_PB19_O_ENCODING              0x12
#define DAI_PB04_I_DAI_PB19_O_ENCODING              0x12
#define DAI_PB05_I_DAI_PB19_O_ENCODING              0x12
#define DAI_PB06_I_DAI_PB19_O_ENCODING              0x12
#define DAI_PB07_I_DAI_PB19_O_ENCODING              0x12
#define DAI_PB08_I_DAI_PB19_O_ENCODING              0x12
#define DAI_PB09_I_DAI_PB19_O_ENCODING              0x12
#define DAI_PB10_I_DAI_PB19_O_ENCODING              0x12
#define DAI_PB11_I_DAI_PB19_O_ENCODING              0x12
#define DAI_PB12_I_DAI_PB19_O_ENCODING              0x12
#define DAI_PB13_I_DAI_PB19_O_ENCODING              0x12
#define DAI_PB14_I_DAI_PB19_O_ENCODING              0x12
#define DAI_PB15_I_DAI_PB19_O_ENCODING              0x12
#define DAI_PB16_I_DAI_PB19_O_ENCODING              0x12
#define DAI_PB17_I_DAI_PB19_O_ENCODING              0x12
#define DAI_PB18_I_DAI_PB19_O_ENCODING              0x12
#define DAI_PB19_I_DAI_PB19_O_ENCODING              0x12
#define DAI_PB20_I_DAI_PB19_O_ENCODING              0x12

#define DAI_PB01_I_DAI_PB20_O_ENCODING              0x13
#define DAI_PB02_I_DAI_PB20_O_ENCODING              0x13
#define DAI_PB03_I_DAI_PB20_O_ENCODING              0x13
#define DAI_PB04_I_DAI_PB20_O_ENCODING              0x13
#define DAI_PB05_I_DAI_PB20_O_ENCODING              0x13
#define DAI_PB06_I_DAI_PB20_O_ENCODING              0x13
#define DAI_PB07_I_DAI_PB20_O_ENCODING              0x13
#define DAI_PB08_I_DAI_PB20_O_ENCODING              0x13
#define DAI_PB09_I_DAI_PB20_O_ENCODING              0x13
#define DAI_PB10_I_DAI_PB20_O_ENCODING              0x13
#define DAI_PB11_I_DAI_PB20_O_ENCODING              0x13
#define DAI_PB12_I_DAI_PB20_O_ENCODING              0x13
#define DAI_PB13_I_DAI_PB20_O_ENCODING              0x13
#define DAI_PB14_I_DAI_PB20_O_ENCODING              0x13
#define DAI_PB15_I_DAI_PB20_O_ENCODING              0x13
#define DAI_PB16_I_DAI_PB20_O_ENCODING              0x13
#define DAI_PB17_I_DAI_PB20_O_ENCODING              0x13
#define DAI_PB18_I_DAI_PB20_O_ENCODING              0x13
#define DAI_PB19_I_DAI_PB20_O_ENCODING              0x13
#define DAI_PB20_I_DAI_PB20_O_ENCODING              0x13

#define DAI_PB01_I_SPORT0_Da_O_ENCODING          0x14
#define DAI_PB02_I_SPORT0_Da_O_ENCODING          0x14
#define DAI_PB03_I_SPORT0_Da_O_ENCODING          0x14
#define DAI_PB04_I_SPORT0_Da_O_ENCODING          0x14
#define DAI_PB05_I_SPORT0_Da_O_ENCODING          0x14
#define DAI_PB06_I_SPORT0_Da_O_ENCODING          0x14
#define DAI_PB07_I_SPORT0_Da_O_ENCODING          0x14
#define DAI_PB08_I_SPORT0_Da_O_ENCODING          0x14
#define DAI_PB09_I_SPORT0_Da_O_ENCODING          0x14
#define DAI_PB10_I_SPORT0_Da_O_ENCODING          0x14
#define DAI_PB11_I_SPORT0_Da_O_ENCODING          0x14
#define DAI_PB12_I_SPORT0_Da_O_ENCODING          0x14
#define DAI_PB13_I_SPORT0_Da_O_ENCODING          0x14
#define DAI_PB14_I_SPORT0_Da_O_ENCODING          0x14
#define DAI_PB15_I_SPORT0_Da_O_ENCODING          0x14
#define DAI_PB16_I_SPORT0_Da_O_ENCODING          0x14
#define DAI_PB17_I_SPORT0_Da_O_ENCODING          0x14
#define DAI_PB18_I_SPORT0_Da_O_ENCODING          0x14
#define DAI_PB19_I_SPORT0_Da_O_ENCODING          0x14
#define DAI_PB20_I_SPORT0_Da_O_ENCODING          0x14

#define DAI_PB01_I_SPORT0_Db_O_ENCODING          0x15
#define DAI_PB02_I_SPORT0_Db_O_ENCODING          0x15
#define DAI_PB03_I_SPORT0_Db_O_ENCODING          0x15
#define DAI_PB04_I_SPORT0_Db_O_ENCODING          0x15
#define DAI_PB05_I_SPORT0_Db_O_ENCODING          0x15
#define DAI_PB06_I_SPORT0_Db_O_ENCODING          0x15
#define DAI_PB07_I_SPORT0_Db_O_ENCODING          0x15
#define DAI_PB08_I_SPORT0_Db_O_ENCODING          0x15
#define DAI_PB09_I_SPORT0_Db_O_ENCODING          0x15
#define DAI_PB10_I_SPORT0_Db_O_ENCODING          0x15
#define DAI_PB11_I_SPORT0_Db_O_ENCODING          0x15
#define DAI_PB12_I_SPORT0_Db_O_ENCODING          0x15
#define DAI_PB13_I_SPORT0_Db_O_ENCODING          0x15
#define DAI_PB14_I_SPORT0_Db_O_ENCODING          0x15
#define DAI_PB15_I_SPORT0_Db_O_ENCODING          0x15
#define DAI_PB16_I_SPORT0_Db_O_ENCODING          0x15
#define DAI_PB17_I_SPORT0_Db_O_ENCODING          0x15
#define DAI_PB18_I_SPORT0_Db_O_ENCODING          0x15
#define DAI_PB19_I_SPORT0_Db_O_ENCODING          0x15
#define DAI_PB20_I_SPORT0_Db_O_ENCODING          0x15

#define DAI_PB01_I_SPORT1_Da_O_ENCODING          0x16
#define DAI_PB02_I_SPORT1_Da_O_ENCODING          0x16
#define DAI_PB03_I_SPORT1_Da_O_ENCODING          0x16
#define DAI_PB04_I_SPORT1_Da_O_ENCODING          0x16
#define DAI_PB05_I_SPORT1_Da_O_ENCODING          0x16
#define DAI_PB06_I_SPORT1_Da_O_ENCODING          0x16
#define DAI_PB07_I_SPORT1_Da_O_ENCODING          0x16
#define DAI_PB08_I_SPORT1_Da_O_ENCODING          0x16
#define DAI_PB09_I_SPORT1_Da_O_ENCODING          0x16
#define DAI_PB10_I_SPORT1_Da_O_ENCODING          0x16
#define DAI_PB11_I_SPORT1_Da_O_ENCODING          0x16
#define DAI_PB12_I_SPORT1_Da_O_ENCODING          0x16
#define DAI_PB13_I_SPORT1_Da_O_ENCODING          0x16
#define DAI_PB14_I_SPORT1_Da_O_ENCODING          0x16
#define DAI_PB15_I_SPORT1_Da_O_ENCODING          0x16
#define DAI_PB16_I_SPORT1_Da_O_ENCODING          0x16
#define DAI_PB17_I_SPORT1_Da_O_ENCODING          0x16
#define DAI_PB18_I_SPORT1_Da_O_ENCODING          0x16
#define DAI_PB19_I_SPORT1_Da_O_ENCODING          0x16
#define DAI_PB20_I_SPORT1_Da_O_ENCODING          0x16

#define DAI_PB01_I_SPORT1_Db_O_ENCODING          0x17
#define DAI_PB02_I_SPORT1_Db_O_ENCODING          0x17
#define DAI_PB03_I_SPORT1_Db_O_ENCODING          0x17
#define DAI_PB04_I_SPORT1_Db_O_ENCODING          0x17
#define DAI_PB05_I_SPORT1_Db_O_ENCODING          0x17
#define DAI_PB06_I_SPORT1_Db_O_ENCODING          0x17
#define DAI_PB07_I_SPORT1_Db_O_ENCODING          0x17
#define DAI_PB08_I_SPORT1_Db_O_ENCODING          0x17
#define DAI_PB09_I_SPORT1_Db_O_ENCODING          0x17
#define DAI_PB10_I_SPORT1_Db_O_ENCODING          0x17
#define DAI_PB11_I_SPORT1_Db_O_ENCODING          0x17
#define DAI_PB12_I_SPORT1_Db_O_ENCODING          0x17
#define DAI_PB13_I_SPORT1_Db_O_ENCODING          0x17
#define DAI_PB14_I_SPORT1_Db_O_ENCODING          0x17
#define DAI_PB15_I_SPORT1_Db_O_ENCODING          0x17
#define DAI_PB16_I_SPORT1_Db_O_ENCODING          0x17
#define DAI_PB17_I_SPORT1_Db_O_ENCODING          0x17
#define DAI_PB18_I_SPORT1_Db_O_ENCODING          0x17
#define DAI_PB19_I_SPORT1_Db_O_ENCODING          0x17
#define DAI_PB20_I_SPORT1_Db_O_ENCODING          0x17

#define DAI_PB01_I_SPORT2_Da_O_ENCODING          0x18
#define DAI_PB02_I_SPORT2_Da_O_ENCODING          0x18
#define DAI_PB03_I_SPORT2_Da_O_ENCODING          0x18
#define DAI_PB04_I_SPORT2_Da_O_ENCODING          0x18
#define DAI_PB05_I_SPORT2_Da_O_ENCODING          0x18
#define DAI_PB06_I_SPORT2_Da_O_ENCODING          0x18
#define DAI_PB07_I_SPORT2_Da_O_ENCODING          0x18
#define DAI_PB08_I_SPORT2_Da_O_ENCODING          0x18
#define DAI_PB09_I_SPORT2_Da_O_ENCODING          0x18
#define DAI_PB10_I_SPORT2_Da_O_ENCODING          0x18
#define DAI_PB11_I_SPORT2_Da_O_ENCODING          0x18
#define DAI_PB12_I_SPORT2_Da_O_ENCODING          0x18
#define DAI_PB13_I_SPORT2_Da_O_ENCODING          0x18
#define DAI_PB14_I_SPORT2_Da_O_ENCODING          0x18
#define DAI_PB15_I_SPORT2_Da_O_ENCODING          0x18
#define DAI_PB16_I_SPORT2_Da_O_ENCODING          0x18
#define DAI_PB17_I_SPORT2_Da_O_ENCODING          0x18
#define DAI_PB18_I_SPORT2_Da_O_ENCODING          0x18
#define DAI_PB19_I_SPORT2_Da_O_ENCODING          0x18
#define DAI_PB20_I_SPORT2_Da_O_ENCODING          0x18

#define DAI_PB01_I_SPORT2_Db_O_ENCODING          0x19
#define DAI_PB02_I_SPORT2_Db_O_ENCODING          0x19
#define DAI_PB03_I_SPORT2_Db_O_ENCODING          0x19
#define DAI_PB04_I_SPORT2_Db_O_ENCODING          0x19
#define DAI_PB05_I_SPORT2_Db_O_ENCODING          0x19
#define DAI_PB06_I_SPORT2_Db_O_ENCODING          0x19
#define DAI_PB07_I_SPORT2_Db_O_ENCODING          0x19
#define DAI_PB08_I_SPORT2_Db_O_ENCODING          0x19
#define DAI_PB09_I_SPORT2_Db_O_ENCODING          0x19
#define DAI_PB10_I_SPORT2_Db_O_ENCODING          0x19
#define DAI_PB11_I_SPORT2_Db_O_ENCODING          0x19
#define DAI_PB12_I_SPORT2_Db_O_ENCODING          0x19
#define DAI_PB13_I_SPORT2_Db_O_ENCODING          0x19
#define DAI_PB14_I_SPORT2_Db_O_ENCODING          0x19
#define DAI_PB15_I_SPORT2_Db_O_ENCODING          0x19
#define DAI_PB16_I_SPORT2_Db_O_ENCODING          0x19
#define DAI_PB17_I_SPORT2_Db_O_ENCODING          0x19
#define DAI_PB18_I_SPORT2_Db_O_ENCODING          0x19
#define DAI_PB19_I_SPORT2_Db_O_ENCODING          0x19
#define DAI_PB20_I_SPORT2_Db_O_ENCODING          0x19

#define DAI_PB01_I_SPORT3_Da_O_ENCODING          0x1A
#define DAI_PB02_I_SPORT3_Da_O_ENCODING          0x1A
#define DAI_PB03_I_SPORT3_Da_O_ENCODING          0x1A
#define DAI_PB04_I_SPORT3_Da_O_ENCODING          0x1A
#define DAI_PB05_I_SPORT3_Da_O_ENCODING          0x1A
#define DAI_PB06_I_SPORT3_Da_O_ENCODING          0x1A
#define DAI_PB07_I_SPORT3_Da_O_ENCODING          0x1A
#define DAI_PB08_I_SPORT3_Da_O_ENCODING          0x1A
#define DAI_PB09_I_SPORT3_Da_O_ENCODING          0x1A
#define DAI_PB10_I_SPORT3_Da_O_ENCODING          0x1A
#define DAI_PB11_I_SPORT3_Da_O_ENCODING          0x1A
#define DAI_PB12_I_SPORT3_Da_O_ENCODING          0x1A
#define DAI_PB13_I_SPORT3_Da_O_ENCODING          0x1A
#define DAI_PB14_I_SPORT3_Da_O_ENCODING          0x1A
#define DAI_PB15_I_SPORT3_Da_O_ENCODING          0x1A
#define DAI_PB16_I_SPORT3_Da_O_ENCODING          0x1A
#define DAI_PB17_I_SPORT3_Da_O_ENCODING          0x1A
#define DAI_PB18_I_SPORT3_Da_O_ENCODING          0x1A
#define DAI_PB19_I_SPORT3_Da_O_ENCODING          0x1A
#define DAI_PB20_I_SPORT3_Da_O_ENCODING          0x1A

#define DAI_PB01_I_SPORT3_Db_O_ENCODING          0x1B
#define DAI_PB02_I_SPORT3_Db_O_ENCODING          0x1B
#define DAI_PB03_I_SPORT3_Db_O_ENCODING          0x1B
#define DAI_PB04_I_SPORT3_Db_O_ENCODING          0x1B
#define DAI_PB05_I_SPORT3_Db_O_ENCODING          0x1B
#define DAI_PB06_I_SPORT3_Db_O_ENCODING          0x1B
#define DAI_PB07_I_SPORT3_Db_O_ENCODING          0x1B
#define DAI_PB08_I_SPORT3_Db_O_ENCODING          0x1B
#define DAI_PB09_I_SPORT3_Db_O_ENCODING          0x1B
#define DAI_PB10_I_SPORT3_Db_O_ENCODING          0x1B
#define DAI_PB11_I_SPORT3_Db_O_ENCODING          0x1B
#define DAI_PB12_I_SPORT3_Db_O_ENCODING          0x1B
#define DAI_PB13_I_SPORT3_Db_O_ENCODING          0x1B
#define DAI_PB14_I_SPORT3_Db_O_ENCODING          0x1B
#define DAI_PB15_I_SPORT3_Db_O_ENCODING          0x1B
#define DAI_PB16_I_SPORT3_Db_O_ENCODING          0x1B
#define DAI_PB17_I_SPORT3_Db_O_ENCODING          0x1B
#define DAI_PB18_I_SPORT3_Db_O_ENCODING          0x1B
#define DAI_PB19_I_SPORT3_Db_O_ENCODING          0x1B
#define DAI_PB20_I_SPORT3_Db_O_ENCODING          0x1B

#define DAI_PB01_I_SPORT4_Da_O_ENCODING          0x1C
#define DAI_PB02_I_SPORT4_Da_O_ENCODING          0x1C
#define DAI_PB03_I_SPORT4_Da_O_ENCODING          0x1C
#define DAI_PB04_I_SPORT4_Da_O_ENCODING          0x1C
#define DAI_PB05_I_SPORT4_Da_O_ENCODING          0x1C
#define DAI_PB06_I_SPORT4_Da_O_ENCODING          0x1C
#define DAI_PB07_I_SPORT4_Da_O_ENCODING          0x1C
#define DAI_PB08_I_SPORT4_Da_O_ENCODING          0x1C
#define DAI_PB09_I_SPORT4_Da_O_ENCODING          0x1C
#define DAI_PB10_I_SPORT4_Da_O_ENCODING          0x1C
#define DAI_PB11_I_SPORT4_Da_O_ENCODING          0x1C
#define DAI_PB12_I_SPORT4_Da_O_ENCODING          0x1C
#define DAI_PB13_I_SPORT4_Da_O_ENCODING          0x1C
#define DAI_PB14_I_SPORT4_Da_O_ENCODING          0x1C
#define DAI_PB15_I_SPORT4_Da_O_ENCODING          0x1C
#define DAI_PB16_I_SPORT4_Da_O_ENCODING          0x1C
#define DAI_PB17_I_SPORT4_Da_O_ENCODING          0x1C
#define DAI_PB18_I_SPORT4_Da_O_ENCODING          0x1C
#define DAI_PB19_I_SPORT4_Da_O_ENCODING          0x1C
#define DAI_PB20_I_SPORT4_Da_O_ENCODING          0x1C

#define DAI_PB01_I_SPORT4_Db_O_ENCODING          0x1D
#define DAI_PB02_I_SPORT4_Db_O_ENCODING          0x1D
#define DAI_PB03_I_SPORT4_Db_O_ENCODING          0x1D
#define DAI_PB04_I_SPORT4_Db_O_ENCODING          0x1D
#define DAI_PB05_I_SPORT4_Db_O_ENCODING          0x1D
#define DAI_PB06_I_SPORT4_Db_O_ENCODING          0x1D
#define DAI_PB07_I_SPORT4_Db_O_ENCODING          0x1D
#define DAI_PB08_I_SPORT4_Db_O_ENCODING          0x1D
#define DAI_PB09_I_SPORT4_Db_O_ENCODING          0x1D
#define DAI_PB10_I_SPORT4_Db_O_ENCODING          0x1D
#define DAI_PB11_I_SPORT4_Db_O_ENCODING          0x1D
#define DAI_PB12_I_SPORT4_Db_O_ENCODING          0x1D
#define DAI_PB13_I_SPORT4_Db_O_ENCODING          0x1D
#define DAI_PB14_I_SPORT4_Db_O_ENCODING          0x1D
#define DAI_PB15_I_SPORT4_Db_O_ENCODING          0x1D
#define DAI_PB16_I_SPORT4_Db_O_ENCODING          0x1D
#define DAI_PB17_I_SPORT4_Db_O_ENCODING          0x1D
#define DAI_PB18_I_SPORT4_Db_O_ENCODING          0x1D
#define DAI_PB19_I_SPORT4_Db_O_ENCODING          0x1D
#define DAI_PB20_I_SPORT4_Db_O_ENCODING          0x1D

#define DAI_PB01_I_SPORT5_Da_O_ENCODING          0x1E
#define DAI_PB02_I_SPORT5_Da_O_ENCODING          0x1E
#define DAI_PB03_I_SPORT5_Da_O_ENCODING          0x1E
#define DAI_PB04_I_SPORT5_Da_O_ENCODING          0x1E
#define DAI_PB05_I_SPORT5_Da_O_ENCODING          0x1E
#define DAI_PB06_I_SPORT5_Da_O_ENCODING          0x1E
#define DAI_PB07_I_SPORT5_Da_O_ENCODING          0x1E
#define DAI_PB08_I_SPORT5_Da_O_ENCODING          0x1E
#define DAI_PB09_I_SPORT5_Da_O_ENCODING          0x1E
#define DAI_PB10_I_SPORT5_Da_O_ENCODING          0x1E
#define DAI_PB11_I_SPORT5_Da_O_ENCODING          0x1E
#define DAI_PB12_I_SPORT5_Da_O_ENCODING          0x1E
#define DAI_PB13_I_SPORT5_Da_O_ENCODING          0x1E
#define DAI_PB14_I_SPORT5_Da_O_ENCODING          0x1E
#define DAI_PB15_I_SPORT5_Da_O_ENCODING          0x1E
#define DAI_PB16_I_SPORT5_Da_O_ENCODING          0x1E
#define DAI_PB17_I_SPORT5_Da_O_ENCODING          0x1E
#define DAI_PB18_I_SPORT5_Da_O_ENCODING          0x1E
#define DAI_PB19_I_SPORT5_Da_O_ENCODING          0x1E
#define DAI_PB20_I_SPORT5_Da_O_ENCODING          0x1E

#define DAI_PB01_I_SPORT5_Db_O_ENCODING          0x1F
#define DAI_PB02_I_SPORT5_Db_O_ENCODING          0x1F
#define DAI_PB03_I_SPORT5_Db_O_ENCODING          0x1F
#define DAI_PB04_I_SPORT5_Db_O_ENCODING          0x1F
#define DAI_PB05_I_SPORT5_Db_O_ENCODING          0x1F
#define DAI_PB06_I_SPORT5_Db_O_ENCODING          0x1F
#define DAI_PB07_I_SPORT5_Db_O_ENCODING          0x1F
#define DAI_PB08_I_SPORT5_Db_O_ENCODING          0x1F
#define DAI_PB09_I_SPORT5_Db_O_ENCODING          0x1F
#define DAI_PB10_I_SPORT5_Db_O_ENCODING          0x1F
#define DAI_PB11_I_SPORT5_Db_O_ENCODING          0x1F
#define DAI_PB12_I_SPORT5_Db_O_ENCODING          0x1F
#define DAI_PB13_I_SPORT5_Db_O_ENCODING          0x1F
#define DAI_PB14_I_SPORT5_Db_O_ENCODING          0x1F
#define DAI_PB15_I_SPORT5_Db_O_ENCODING          0x1F
#define DAI_PB16_I_SPORT5_Db_O_ENCODING          0x1F
#define DAI_PB17_I_SPORT5_Db_O_ENCODING          0x1F
#define DAI_PB18_I_SPORT5_Db_O_ENCODING          0x1F
#define DAI_PB19_I_SPORT5_Db_O_ENCODING          0x1F
#define DAI_PB20_I_SPORT5_Db_O_ENCODING          0x1F

#define DAI_PB01_I_SPORT0_CLK_O_ENCODING         0x20
#define DAI_PB02_I_SPORT0_CLK_O_ENCODING         0x20
#define DAI_PB03_I_SPORT0_CLK_O_ENCODING         0x20
#define DAI_PB04_I_SPORT0_CLK_O_ENCODING         0x20
#define DAI_PB05_I_SPORT0_CLK_O_ENCODING         0x20
#define DAI_PB06_I_SPORT0_CLK_O_ENCODING         0x20
#define DAI_PB07_I_SPORT0_CLK_O_ENCODING         0x20
#define DAI_PB08_I_SPORT0_CLK_O_ENCODING         0x20
#define DAI_PB09_I_SPORT0_CLK_O_ENCODING         0x20
#define DAI_PB10_I_SPORT0_CLK_O_ENCODING         0x20
#define DAI_PB11_I_SPORT0_CLK_O_ENCODING         0x20
#define DAI_PB12_I_SPORT0_CLK_O_ENCODING         0x20
#define DAI_PB13_I_SPORT0_CLK_O_ENCODING         0x20
#define DAI_PB14_I_SPORT0_CLK_O_ENCODING         0x20
#define DAI_PB15_I_SPORT0_CLK_O_ENCODING         0x20
#define DAI_PB16_I_SPORT0_CLK_O_ENCODING         0x20
#define DAI_PB17_I_SPORT0_CLK_O_ENCODING         0x20
#define DAI_PB18_I_SPORT0_CLK_O_ENCODING         0x20
#define DAI_PB19_I_SPORT0_CLK_O_ENCODING         0x20
#define DAI_PB20_I_SPORT0_CLK_O_ENCODING         0x20

#define DAI_PB01_I_SPORT1_CLK_O_ENCODING         0x21
#define DAI_PB02_I_SPORT1_CLK_O_ENCODING         0x21
#define DAI_PB03_I_SPORT1_CLK_O_ENCODING         0x21
#define DAI_PB04_I_SPORT1_CLK_O_ENCODING         0x21
#define DAI_PB05_I_SPORT1_CLK_O_ENCODING         0x21
#define DAI_PB06_I_SPORT1_CLK_O_ENCODING         0x21
#define DAI_PB07_I_SPORT1_CLK_O_ENCODING         0x21
#define DAI_PB08_I_SPORT1_CLK_O_ENCODING         0x21
#define DAI_PB09_I_SPORT1_CLK_O_ENCODING         0x21
#define DAI_PB10_I_SPORT1_CLK_O_ENCODING         0x21
#define DAI_PB11_I_SPORT1_CLK_O_ENCODING         0x21
#define DAI_PB12_I_SPORT1_CLK_O_ENCODING         0x21
#define DAI_PB13_I_SPORT1_CLK_O_ENCODING         0x21
#define DAI_PB14_I_SPORT1_CLK_O_ENCODING         0x21
#define DAI_PB15_I_SPORT1_CLK_O_ENCODING         0x21
#define DAI_PB16_I_SPORT1_CLK_O_ENCODING         0x21
#define DAI_PB17_I_SPORT1_CLK_O_ENCODING         0x21
#define DAI_PB18_I_SPORT1_CLK_O_ENCODING         0x21
#define DAI_PB19_I_SPORT1_CLK_O_ENCODING         0x21
#define DAI_PB20_I_SPORT1_CLK_O_ENCODING         0x21

#define DAI_PB01_I_SPORT2_CLK_O_ENCODING         0x22
#define DAI_PB02_I_SPORT2_CLK_O_ENCODING         0x22
#define DAI_PB03_I_SPORT2_CLK_O_ENCODING         0x22
#define DAI_PB04_I_SPORT2_CLK_O_ENCODING         0x22
#define DAI_PB05_I_SPORT2_CLK_O_ENCODING         0x22
#define DAI_PB06_I_SPORT2_CLK_O_ENCODING         0x22
#define DAI_PB07_I_SPORT2_CLK_O_ENCODING         0x22
#define DAI_PB08_I_SPORT2_CLK_O_ENCODING         0x22
#define DAI_PB09_I_SPORT2_CLK_O_ENCODING         0x22
#define DAI_PB10_I_SPORT2_CLK_O_ENCODING         0x22
#define DAI_PB11_I_SPORT2_CLK_O_ENCODING         0x22
#define DAI_PB12_I_SPORT2_CLK_O_ENCODING         0x22
#define DAI_PB13_I_SPORT2_CLK_O_ENCODING         0x22
#define DAI_PB14_I_SPORT2_CLK_O_ENCODING         0x22
#define DAI_PB15_I_SPORT2_CLK_O_ENCODING         0x22
#define DAI_PB16_I_SPORT2_CLK_O_ENCODING         0x22
#define DAI_PB17_I_SPORT2_CLK_O_ENCODING         0x22
#define DAI_PB18_I_SPORT2_CLK_O_ENCODING         0x22
#define DAI_PB19_I_SPORT2_CLK_O_ENCODING         0x22
#define DAI_PB20_I_SPORT2_CLK_O_ENCODING         0x22

#define DAI_PB01_I_SPORT3_CLK_O_ENCODING         0x23
#define DAI_PB02_I_SPORT3_CLK_O_ENCODING         0x23
#define DAI_PB03_I_SPORT3_CLK_O_ENCODING         0x23
#define DAI_PB04_I_SPORT3_CLK_O_ENCODING         0x23
#define DAI_PB05_I_SPORT3_CLK_O_ENCODING         0x23
#define DAI_PB06_I_SPORT3_CLK_O_ENCODING         0x23
#define DAI_PB07_I_SPORT3_CLK_O_ENCODING         0x23
#define DAI_PB08_I_SPORT3_CLK_O_ENCODING         0x23
#define DAI_PB09_I_SPORT3_CLK_O_ENCODING         0x23
#define DAI_PB10_I_SPORT3_CLK_O_ENCODING         0x23
#define DAI_PB11_I_SPORT3_CLK_O_ENCODING         0x23
#define DAI_PB12_I_SPORT3_CLK_O_ENCODING         0x23
#define DAI_PB13_I_SPORT3_CLK_O_ENCODING         0x23
#define DAI_PB14_I_SPORT3_CLK_O_ENCODING         0x23
#define DAI_PB15_I_SPORT3_CLK_O_ENCODING         0x23
#define DAI_PB16_I_SPORT3_CLK_O_ENCODING         0x23
#define DAI_PB17_I_SPORT3_CLK_O_ENCODING         0x23
#define DAI_PB18_I_SPORT3_CLK_O_ENCODING         0x23
#define DAI_PB19_I_SPORT3_CLK_O_ENCODING         0x23
#define DAI_PB20_I_SPORT3_CLK_O_ENCODING         0x23

#define DAI_PB01_I_SPORT4_CLK_O_ENCODING         0x24
#define DAI_PB02_I_SPORT4_CLK_O_ENCODING         0x24
#define DAI_PB03_I_SPORT4_CLK_O_ENCODING         0x24
#define DAI_PB04_I_SPORT4_CLK_O_ENCODING         0x24
#define DAI_PB05_I_SPORT4_CLK_O_ENCODING         0x24
#define DAI_PB06_I_SPORT4_CLK_O_ENCODING         0x24
#define DAI_PB07_I_SPORT4_CLK_O_ENCODING         0x24
#define DAI_PB08_I_SPORT4_CLK_O_ENCODING         0x24
#define DAI_PB09_I_SPORT4_CLK_O_ENCODING         0x24
#define DAI_PB10_I_SPORT4_CLK_O_ENCODING         0x24
#define DAI_PB11_I_SPORT4_CLK_O_ENCODING         0x24
#define DAI_PB12_I_SPORT4_CLK_O_ENCODING         0x24
#define DAI_PB13_I_SPORT4_CLK_O_ENCODING         0x24
#define DAI_PB14_I_SPORT4_CLK_O_ENCODING         0x24
#define DAI_PB15_I_SPORT4_CLK_O_ENCODING         0x24
#define DAI_PB16_I_SPORT4_CLK_O_ENCODING         0x24
#define DAI_PB17_I_SPORT4_CLK_O_ENCODING         0x24
#define DAI_PB18_I_SPORT4_CLK_O_ENCODING         0x24
#define DAI_PB19_I_SPORT4_CLK_O_ENCODING         0x24
#define DAI_PB20_I_SPORT4_CLK_O_ENCODING         0x24

#define DAI_PB01_I_SPORT5_CLK_O_ENCODING         0x25
#define DAI_PB02_I_SPORT5_CLK_O_ENCODING         0x25
#define DAI_PB03_I_SPORT5_CLK_O_ENCODING         0x25
#define DAI_PB04_I_SPORT5_CLK_O_ENCODING         0x25
#define DAI_PB05_I_SPORT5_CLK_O_ENCODING         0x25
#define DAI_PB06_I_SPORT5_CLK_O_ENCODING         0x25
#define DAI_PB07_I_SPORT5_CLK_O_ENCODING         0x25
#define DAI_PB08_I_SPORT5_CLK_O_ENCODING         0x25
#define DAI_PB09_I_SPORT5_CLK_O_ENCODING         0x25
#define DAI_PB10_I_SPORT5_CLK_O_ENCODING         0x25
#define DAI_PB11_I_SPORT5_CLK_O_ENCODING         0x25
#define DAI_PB12_I_SPORT5_CLK_O_ENCODING         0x25
#define DAI_PB13_I_SPORT5_CLK_O_ENCODING         0x25
#define DAI_PB14_I_SPORT5_CLK_O_ENCODING         0x25
#define DAI_PB15_I_SPORT5_CLK_O_ENCODING         0x25
#define DAI_PB16_I_SPORT5_CLK_O_ENCODING         0x25
#define DAI_PB17_I_SPORT5_CLK_O_ENCODING         0x25
#define DAI_PB18_I_SPORT5_CLK_O_ENCODING         0x25
#define DAI_PB19_I_SPORT5_CLK_O_ENCODING         0x25
#define DAI_PB20_I_SPORT5_CLK_O_ENCODING         0x25

#define DAI_PB01_I_SPORT0_FS_O_ENCODING          0x26
#define DAI_PB02_I_SPORT0_FS_O_ENCODING          0x26
#define DAI_PB03_I_SPORT0_FS_O_ENCODING          0x26
#define DAI_PB04_I_SPORT0_FS_O_ENCODING          0x26
#define DAI_PB05_I_SPORT0_FS_O_ENCODING          0x26
#define DAI_PB06_I_SPORT0_FS_O_ENCODING          0x26
#define DAI_PB07_I_SPORT0_FS_O_ENCODING          0x26
#define DAI_PB08_I_SPORT0_FS_O_ENCODING          0x26
#define DAI_PB09_I_SPORT0_FS_O_ENCODING          0x26
#define DAI_PB10_I_SPORT0_FS_O_ENCODING          0x26
#define DAI_PB11_I_SPORT0_FS_O_ENCODING          0x26
#define DAI_PB12_I_SPORT0_FS_O_ENCODING          0x26
#define DAI_PB13_I_SPORT0_FS_O_ENCODING          0x26
#define DAI_PB14_I_SPORT0_FS_O_ENCODING          0x26
#define DAI_PB15_I_SPORT0_FS_O_ENCODING          0x26
#define DAI_PB16_I_SPORT0_FS_O_ENCODING          0x26
#define DAI_PB17_I_SPORT0_FS_O_ENCODING          0x26
#define DAI_PB18_I_SPORT0_FS_O_ENCODING          0x26
#define DAI_PB19_I_SPORT0_FS_O_ENCODING          0x26
#define DAI_PB20_I_SPORT0_FS_O_ENCODING          0x26

#define DAI_PB01_I_SPORT1_FS_O_ENCODING          0x27
#define DAI_PB02_I_SPORT1_FS_O_ENCODING          0x27
#define DAI_PB03_I_SPORT1_FS_O_ENCODING          0x27
#define DAI_PB04_I_SPORT1_FS_O_ENCODING          0x27
#define DAI_PB05_I_SPORT1_FS_O_ENCODING          0x27
#define DAI_PB06_I_SPORT1_FS_O_ENCODING          0x27
#define DAI_PB07_I_SPORT1_FS_O_ENCODING          0x27
#define DAI_PB08_I_SPORT1_FS_O_ENCODING          0x27
#define DAI_PB09_I_SPORT1_FS_O_ENCODING          0x27
#define DAI_PB10_I_SPORT1_FS_O_ENCODING          0x27
#define DAI_PB11_I_SPORT1_FS_O_ENCODING          0x27
#define DAI_PB12_I_SPORT1_FS_O_ENCODING          0x27
#define DAI_PB13_I_SPORT1_FS_O_ENCODING          0x27
#define DAI_PB14_I_SPORT1_FS_O_ENCODING          0x27
#define DAI_PB15_I_SPORT1_FS_O_ENCODING          0x27
#define DAI_PB16_I_SPORT1_FS_O_ENCODING          0x27
#define DAI_PB17_I_SPORT1_FS_O_ENCODING          0x27
#define DAI_PB18_I_SPORT1_FS_O_ENCODING          0x27
#define DAI_PB19_I_SPORT1_FS_O_ENCODING          0x27
#define DAI_PB20_I_SPORT1_FS_O_ENCODING          0x27

#define DAI_PB01_I_SPORT2_FS_O_ENCODING          0x28
#define DAI_PB02_I_SPORT2_FS_O_ENCODING          0x28
#define DAI_PB03_I_SPORT2_FS_O_ENCODING          0x28
#define DAI_PB04_I_SPORT2_FS_O_ENCODING          0x28
#define DAI_PB05_I_SPORT2_FS_O_ENCODING          0x28
#define DAI_PB06_I_SPORT2_FS_O_ENCODING          0x28
#define DAI_PB07_I_SPORT2_FS_O_ENCODING          0x28
#define DAI_PB08_I_SPORT2_FS_O_ENCODING          0x28
#define DAI_PB09_I_SPORT2_FS_O_ENCODING          0x28
#define DAI_PB10_I_SPORT2_FS_O_ENCODING          0x28
#define DAI_PB11_I_SPORT2_FS_O_ENCODING          0x28
#define DAI_PB12_I_SPORT2_FS_O_ENCODING          0x28
#define DAI_PB13_I_SPORT2_FS_O_ENCODING          0x28
#define DAI_PB14_I_SPORT2_FS_O_ENCODING          0x28
#define DAI_PB15_I_SPORT2_FS_O_ENCODING          0x28
#define DAI_PB16_I_SPORT2_FS_O_ENCODING          0x28
#define DAI_PB17_I_SPORT2_FS_O_ENCODING          0x28
#define DAI_PB18_I_SPORT2_FS_O_ENCODING          0x28
#define DAI_PB19_I_SPORT2_FS_O_ENCODING          0x28
#define DAI_PB20_I_SPORT2_FS_O_ENCODING          0x28

#define DAI_PB01_I_SPORT3_FS_O_ENCODING          0x29
#define DAI_PB02_I_SPORT3_FS_O_ENCODING          0x29
#define DAI_PB03_I_SPORT3_FS_O_ENCODING          0x29
#define DAI_PB04_I_SPORT3_FS_O_ENCODING          0x29
#define DAI_PB05_I_SPORT3_FS_O_ENCODING          0x29
#define DAI_PB06_I_SPORT3_FS_O_ENCODING          0x29
#define DAI_PB07_I_SPORT3_FS_O_ENCODING          0x29
#define DAI_PB08_I_SPORT3_FS_O_ENCODING          0x29
#define DAI_PB09_I_SPORT3_FS_O_ENCODING          0x29
#define DAI_PB10_I_SPORT3_FS_O_ENCODING          0x29
#define DAI_PB11_I_SPORT3_FS_O_ENCODING          0x29
#define DAI_PB12_I_SPORT3_FS_O_ENCODING          0x29
#define DAI_PB13_I_SPORT3_FS_O_ENCODING          0x29
#define DAI_PB14_I_SPORT3_FS_O_ENCODING          0x29
#define DAI_PB15_I_SPORT3_FS_O_ENCODING          0x29
#define DAI_PB16_I_SPORT3_FS_O_ENCODING          0x29
#define DAI_PB17_I_SPORT3_FS_O_ENCODING          0x29
#define DAI_PB18_I_SPORT3_FS_O_ENCODING          0x29
#define DAI_PB19_I_SPORT3_FS_O_ENCODING          0x29
#define DAI_PB20_I_SPORT3_FS_O_ENCODING          0x29

#define DAI_PB01_I_SPORT4_FS_O_ENCODING          0x2A
#define DAI_PB02_I_SPORT4_FS_O_ENCODING          0x2A
#define DAI_PB03_I_SPORT4_FS_O_ENCODING          0x2A
#define DAI_PB04_I_SPORT4_FS_O_ENCODING          0x2A
#define DAI_PB05_I_SPORT4_FS_O_ENCODING          0x2A
#define DAI_PB06_I_SPORT4_FS_O_ENCODING          0x2A
#define DAI_PB07_I_SPORT4_FS_O_ENCODING          0x2A
#define DAI_PB08_I_SPORT4_FS_O_ENCODING          0x2A
#define DAI_PB09_I_SPORT4_FS_O_ENCODING          0x2A
#define DAI_PB10_I_SPORT4_FS_O_ENCODING          0x2A
#define DAI_PB11_I_SPORT4_FS_O_ENCODING          0x2A
#define DAI_PB12_I_SPORT4_FS_O_ENCODING          0x2A
#define DAI_PB13_I_SPORT4_FS_O_ENCODING          0x2A
#define DAI_PB14_I_SPORT4_FS_O_ENCODING          0x2A
#define DAI_PB15_I_SPORT4_FS_O_ENCODING          0x2A
#define DAI_PB16_I_SPORT4_FS_O_ENCODING          0x2A
#define DAI_PB17_I_SPORT4_FS_O_ENCODING          0x2A
#define DAI_PB18_I_SPORT4_FS_O_ENCODING          0x2A
#define DAI_PB19_I_SPORT4_FS_O_ENCODING          0x2A
#define DAI_PB20_I_SPORT4_FS_O_ENCODING          0x2A

#define DAI_PB01_I_SPORT5_FS_O_ENCODING          0x2B
#define DAI_PB02_I_SPORT5_FS_O_ENCODING          0x2B
#define DAI_PB03_I_SPORT5_FS_O_ENCODING          0x2B
#define DAI_PB04_I_SPORT5_FS_O_ENCODING          0x2B
#define DAI_PB05_I_SPORT5_FS_O_ENCODING          0x2B
#define DAI_PB06_I_SPORT5_FS_O_ENCODING          0x2B
#define DAI_PB07_I_SPORT5_FS_O_ENCODING          0x2B
#define DAI_PB08_I_SPORT5_FS_O_ENCODING          0x2B
#define DAI_PB09_I_SPORT5_FS_O_ENCODING          0x2B
#define DAI_PB10_I_SPORT5_FS_O_ENCODING          0x2B
#define DAI_PB11_I_SPORT5_FS_O_ENCODING          0x2B
#define DAI_PB12_I_SPORT5_FS_O_ENCODING          0x2B
#define DAI_PB13_I_SPORT5_FS_O_ENCODING          0x2B
#define DAI_PB14_I_SPORT5_FS_O_ENCODING          0x2B
#define DAI_PB15_I_SPORT5_FS_O_ENCODING          0x2B
#define DAI_PB16_I_SPORT5_FS_O_ENCODING          0x2B
#define DAI_PB17_I_SPORT5_FS_O_ENCODING          0x2B
#define DAI_PB18_I_SPORT5_FS_O_ENCODING          0x2B
#define DAI_PB19_I_SPORT5_FS_O_ENCODING          0x2B
#define DAI_PB20_I_SPORT5_FS_O_ENCODING          0x2B

#define DAI_PB01_I_TIMER0_O_ENCODING             0x2C
#define DAI_PB02_I_TIMER0_O_ENCODING             0x2C
#define DAI_PB03_I_TIMER0_O_ENCODING             0x2C
#define DAI_PB04_I_TIMER0_O_ENCODING             0x2C
#define DAI_PB05_I_TIMER0_O_ENCODING             0x2C
#define DAI_PB06_I_TIMER0_O_ENCODING             0x2C
#define DAI_PB07_I_TIMER0_O_ENCODING             0x2C
#define DAI_PB08_I_TIMER0_O_ENCODING             0x2C
#define DAI_PB09_I_TIMER0_O_ENCODING             0x2C
#define DAI_PB10_I_TIMER0_O_ENCODING             0x2C
#define DAI_PB11_I_TIMER0_O_ENCODING             0x2C
#define DAI_PB12_I_TIMER0_O_ENCODING             0x2C
#define DAI_PB13_I_TIMER0_O_ENCODING             0x2C
#define DAI_PB14_I_TIMER0_O_ENCODING             0x2C
#define DAI_PB15_I_TIMER0_O_ENCODING             0x2C
#define DAI_PB16_I_TIMER0_O_ENCODING             0x2C
#define DAI_PB17_I_TIMER0_O_ENCODING             0x2C
#define DAI_PB18_I_TIMER0_O_ENCODING             0x2C
#define DAI_PB19_I_TIMER0_O_ENCODING             0x2C
#define DAI_PB20_I_TIMER0_O_ENCODING             0x2C

#define DAI_PB01_I_TIMER1_O_ENCODING             0x2D
#define DAI_PB02_I_TIMER1_O_ENCODING             0x2D
#define DAI_PB03_I_TIMER1_O_ENCODING             0x2D
#define DAI_PB04_I_TIMER1_O_ENCODING             0x2D
#define DAI_PB05_I_TIMER1_O_ENCODING             0x2D
#define DAI_PB06_I_TIMER1_O_ENCODING             0x2D
#define DAI_PB07_I_TIMER1_O_ENCODING             0x2D
#define DAI_PB08_I_TIMER1_O_ENCODING             0x2D
#define DAI_PB09_I_TIMER1_O_ENCODING             0x2D
#define DAI_PB10_I_TIMER1_O_ENCODING             0x2D
#define DAI_PB11_I_TIMER1_O_ENCODING             0x2D
#define DAI_PB12_I_TIMER1_O_ENCODING             0x2D
#define DAI_PB13_I_TIMER1_O_ENCODING             0x2D
#define DAI_PB14_I_TIMER1_O_ENCODING             0x2D
#define DAI_PB15_I_TIMER1_O_ENCODING             0x2D
#define DAI_PB16_I_TIMER1_O_ENCODING             0x2D
#define DAI_PB17_I_TIMER1_O_ENCODING             0x2D
#define DAI_PB18_I_TIMER1_O_ENCODING             0x2D
#define DAI_PB19_I_TIMER1_O_ENCODING             0x2D
#define DAI_PB20_I_TIMER1_O_ENCODING             0x2D

#define DAI_PB01_I_TIMER2_O_ENCODING             0x2E
#define DAI_PB02_I_TIMER2_O_ENCODING             0x2E
#define DAI_PB03_I_TIMER2_O_ENCODING             0x2E
#define DAI_PB04_I_TIMER2_O_ENCODING             0x2E
#define DAI_PB05_I_TIMER2_O_ENCODING             0x2E
#define DAI_PB06_I_TIMER2_O_ENCODING             0x2E
#define DAI_PB07_I_TIMER2_O_ENCODING             0x2E
#define DAI_PB08_I_TIMER2_O_ENCODING             0x2E
#define DAI_PB09_I_TIMER2_O_ENCODING             0x2E
#define DAI_PB10_I_TIMER2_O_ENCODING             0x2E
#define DAI_PB11_I_TIMER2_O_ENCODING             0x2E
#define DAI_PB12_I_TIMER2_O_ENCODING             0x2E
#define DAI_PB13_I_TIMER2_O_ENCODING             0x2E
#define DAI_PB14_I_TIMER2_O_ENCODING             0x2E
#define DAI_PB15_I_TIMER2_O_ENCODING             0x2E
#define DAI_PB16_I_TIMER2_O_ENCODING             0x2E
#define DAI_PB17_I_TIMER2_O_ENCODING             0x2E
#define DAI_PB18_I_TIMER2_O_ENCODING             0x2E
#define DAI_PB19_I_TIMER2_O_ENCODING             0x2E
#define DAI_PB20_I_TIMER2_O_ENCODING             0x2E

#define DAI_PB01_I_FLAG10_O_ENCODING             0x2F
#define DAI_PB02_I_FLAG10_O_ENCODING             0x2F
#define DAI_PB03_I_FLAG10_O_ENCODING             0x2F
#define DAI_PB04_I_FLAG10_O_ENCODING             0x2F
#define DAI_PB05_I_FLAG10_O_ENCODING             0x2F
#define DAI_PB06_I_FLAG10_O_ENCODING             0x2F
#define DAI_PB07_I_FLAG10_O_ENCODING             0x2F
#define DAI_PB08_I_FLAG10_O_ENCODING             0x2F
#define DAI_PB09_I_FLAG10_O_ENCODING             0x2F
#define DAI_PB10_I_FLAG10_O_ENCODING             0x2F
#define DAI_PB11_I_FLAG10_O_ENCODING             0x2F
#define DAI_PB12_I_FLAG10_O_ENCODING             0x2F
#define DAI_PB13_I_FLAG10_O_ENCODING             0x2F
#define DAI_PB14_I_FLAG10_O_ENCODING             0x2F
#define DAI_PB15_I_FLAG10_O_ENCODING             0x2F
#define DAI_PB16_I_FLAG10_O_ENCODING             0x2F
#define DAI_PB17_I_FLAG10_O_ENCODING             0x2F
#define DAI_PB18_I_FLAG10_O_ENCODING             0x2F
#define DAI_PB19_I_FLAG10_O_ENCODING             0x2F
#define DAI_PB20_I_FLAG10_O_ENCODING             0x2F

#define DAI_PB01_I_PDAP_STB_O_ENCODING             0x30
#define DAI_PB02_I_PDAP_STB_O_ENCODING             0x30
#define DAI_PB03_I_PDAP_STB_O_ENCODING             0x30
#define DAI_PB04_I_PDAP_STB_O_ENCODING             0x30
#define DAI_PB05_I_PDAP_STB_O_ENCODING             0x30
#define DAI_PB06_I_PDAP_STB_O_ENCODING             0x30
#define DAI_PB07_I_PDAP_STB_O_ENCODING             0x30
#define DAI_PB08_I_PDAP_STB_O_ENCODING             0x30
#define DAI_PB09_I_PDAP_STB_O_ENCODING             0x30
#define DAI_PB10_I_PDAP_STB_O_ENCODING             0x30
#define DAI_PB11_I_PDAP_STB_O_ENCODING             0x30
#define DAI_PB12_I_PDAP_STB_O_ENCODING             0x30
#define DAI_PB13_I_PDAP_STB_O_ENCODING             0x30
#define DAI_PB14_I_PDAP_STB_O_ENCODING             0x30
#define DAI_PB15_I_PDAP_STB_O_ENCODING             0x30
#define DAI_PB16_I_PDAP_STB_O_ENCODING             0x30
#define DAI_PB17_I_PDAP_STB_O_ENCODING             0x30
#define DAI_PB18_I_PDAP_STB_O_ENCODING             0x30
#define DAI_PB19_I_PDAP_STB_O_ENCODING             0x30
#define DAI_PB20_I_PDAP_STB_O_ENCODING             0x30

#define DAI_PB01_I_FLAG11_O_ENCODING             0x34
#define DAI_PB02_I_FLAG11_O_ENCODING             0x34
#define DAI_PB03_I_FLAG11_O_ENCODING             0x34
#define DAI_PB04_I_FLAG11_O_ENCODING             0x34
#define DAI_PB05_I_FLAG11_O_ENCODING             0x34
#define DAI_PB06_I_FLAG11_O_ENCODING             0x34
#define DAI_PB07_I_FLAG11_O_ENCODING             0x34
#define DAI_PB08_I_FLAG11_O_ENCODING             0x34
#define DAI_PB09_I_FLAG11_O_ENCODING             0x34
#define DAI_PB10_I_FLAG11_O_ENCODING             0x34
#define DAI_PB11_I_FLAG11_O_ENCODING             0x34
#define DAI_PB12_I_FLAG11_O_ENCODING             0x34
#define DAI_PB13_I_FLAG11_O_ENCODING             0x34
#define DAI_PB14_I_FLAG11_O_ENCODING             0x34
#define DAI_PB15_I_FLAG11_O_ENCODING             0x34
#define DAI_PB16_I_FLAG11_O_ENCODING             0x34
#define DAI_PB17_I_FLAG11_O_ENCODING             0x34
#define DAI_PB18_I_FLAG11_O_ENCODING             0x34
#define DAI_PB19_I_FLAG11_O_ENCODING             0x34
#define DAI_PB20_I_FLAG11_O_ENCODING             0x34

#define DAI_PB01_I_FLAG12_O_ENCODING             0x35
#define DAI_PB02_I_FLAG12_O_ENCODING             0x35
#define DAI_PB03_I_FLAG12_O_ENCODING             0x35
#define DAI_PB04_I_FLAG12_O_ENCODING             0x35
#define DAI_PB05_I_FLAG12_O_ENCODING             0x35
#define DAI_PB06_I_FLAG12_O_ENCODING             0x35
#define DAI_PB07_I_FLAG12_O_ENCODING             0x35
#define DAI_PB08_I_FLAG12_O_ENCODING             0x35
#define DAI_PB09_I_FLAG12_O_ENCODING             0x35
#define DAI_PB10_I_FLAG12_O_ENCODING             0x35
#define DAI_PB11_I_FLAG12_O_ENCODING             0x35
#define DAI_PB12_I_FLAG12_O_ENCODING             0x35
#define DAI_PB13_I_FLAG12_O_ENCODING             0x35
#define DAI_PB14_I_FLAG12_O_ENCODING             0x35
#define DAI_PB15_I_FLAG12_O_ENCODING             0x35
#define DAI_PB16_I_FLAG12_O_ENCODING             0x35
#define DAI_PB17_I_FLAG12_O_ENCODING             0x35
#define DAI_PB18_I_FLAG12_O_ENCODING             0x35
#define DAI_PB19_I_FLAG12_O_ENCODING             0x35
#define DAI_PB20_I_FLAG12_O_ENCODING             0x35

#define DAI_PB01_I_FLAG13_O_ENCODING             0x36
#define DAI_PB02_I_FLAG13_O_ENCODING             0x36
#define DAI_PB03_I_FLAG13_O_ENCODING             0x36
#define DAI_PB04_I_FLAG13_O_ENCODING             0x36
#define DAI_PB05_I_FLAG13_O_ENCODING             0x36
#define DAI_PB06_I_FLAG13_O_ENCODING             0x36
#define DAI_PB07_I_FLAG13_O_ENCODING             0x36
#define DAI_PB08_I_FLAG13_O_ENCODING             0x36
#define DAI_PB09_I_FLAG13_O_ENCODING             0x36
#define DAI_PB10_I_FLAG13_O_ENCODING             0x36
#define DAI_PB11_I_FLAG13_O_ENCODING             0x36
#define DAI_PB12_I_FLAG13_O_ENCODING             0x36
#define DAI_PB13_I_FLAG13_O_ENCODING             0x36
#define DAI_PB14_I_FLAG13_O_ENCODING             0x36
#define DAI_PB15_I_FLAG13_O_ENCODING             0x36
#define DAI_PB16_I_FLAG13_O_ENCODING             0x36
#define DAI_PB17_I_FLAG13_O_ENCODING             0x36
#define DAI_PB18_I_FLAG13_O_ENCODING             0x36
#define DAI_PB19_I_FLAG13_O_ENCODING             0x36
#define DAI_PB20_I_FLAG13_O_ENCODING             0x36

#define DAI_PB01_I_FLAG14_O_ENCODING             0x37
#define DAI_PB02_I_FLAG14_O_ENCODING             0x37
#define DAI_PB03_I_FLAG14_O_ENCODING             0x37
#define DAI_PB04_I_FLAG14_O_ENCODING             0x37
#define DAI_PB05_I_FLAG14_O_ENCODING             0x37
#define DAI_PB06_I_FLAG14_O_ENCODING             0x37
#define DAI_PB07_I_FLAG14_O_ENCODING             0x37
#define DAI_PB08_I_FLAG14_O_ENCODING             0x37
#define DAI_PB09_I_FLAG14_O_ENCODING             0x37
#define DAI_PB10_I_FLAG14_O_ENCODING             0x37
#define DAI_PB11_I_FLAG14_O_ENCODING             0x37
#define DAI_PB12_I_FLAG14_O_ENCODING             0x37
#define DAI_PB13_I_FLAG14_O_ENCODING             0x37
#define DAI_PB14_I_FLAG14_O_ENCODING             0x37
#define DAI_PB15_I_FLAG14_O_ENCODING             0x37
#define DAI_PB16_I_FLAG14_O_ENCODING             0x37
#define DAI_PB17_I_FLAG14_O_ENCODING             0x37
#define DAI_PB18_I_FLAG14_O_ENCODING             0x37
#define DAI_PB19_I_FLAG14_O_ENCODING             0x37
#define DAI_PB20_I_FLAG14_O_ENCODING             0x37

#define DAI_PB01_I_PCG_CLKA_O_ENCODING           0x38
#define DAI_PB02_I_PCG_CLKA_O_ENCODING           0x38
#define DAI_PB03_I_PCG_CLKA_O_ENCODING           0x38
#define DAI_PB04_I_PCG_CLKA_O_ENCODING           0x38
#define DAI_PB05_I_PCG_CLKA_O_ENCODING           0x38
#define DAI_PB06_I_PCG_CLKA_O_ENCODING           0x38
#define DAI_PB07_I_PCG_CLKA_O_ENCODING           0x38
#define DAI_PB08_I_PCG_CLKA_O_ENCODING           0x38
#define DAI_PB09_I_PCG_CLKA_O_ENCODING           0x38
#define DAI_PB10_I_PCG_CLKA_O_ENCODING           0x38
#define DAI_PB11_I_PCG_CLKA_O_ENCODING           0x38
#define DAI_PB12_I_PCG_CLKA_O_ENCODING           0x38
#define DAI_PB13_I_PCG_CLKA_O_ENCODING           0x38
#define DAI_PB14_I_PCG_CLKA_O_ENCODING           0x38
#define DAI_PB15_I_PCG_CLKA_O_ENCODING           0x38
#define DAI_PB16_I_PCG_CLKA_O_ENCODING           0x38
#define DAI_PB17_I_PCG_CLKA_O_ENCODING           0x38
#define DAI_PB18_I_PCG_CLKA_O_ENCODING           0x38
#define DAI_PB19_I_PCG_CLKA_O_ENCODING           0x38
#define DAI_PB20_I_PCG_CLKA_O_ENCODING           0x38

#define DAI_PB01_I_PCG_CLKB_O_ENCODING           0x39
#define DAI_PB02_I_PCG_CLKB_O_ENCODING           0x39
#define DAI_PB03_I_PCG_CLKB_O_ENCODING           0x39
#define DAI_PB04_I_PCG_CLKB_O_ENCODING           0x39
#define DAI_PB05_I_PCG_CLKB_O_ENCODING           0x39
#define DAI_PB06_I_PCG_CLKB_O_ENCODING           0x39
#define DAI_PB07_I_PCG_CLKB_O_ENCODING           0x39
#define DAI_PB08_I_PCG_CLKB_O_ENCODING           0x39
#define DAI_PB09_I_PCG_CLKB_O_ENCODING           0x39
#define DAI_PB10_I_PCG_CLKB_O_ENCODING           0x39
#define DAI_PB11_I_PCG_CLKB_O_ENCODING           0x39
#define DAI_PB12_I_PCG_CLKB_O_ENCODING           0x39
#define DAI_PB13_I_PCG_CLKB_O_ENCODING           0x39
#define DAI_PB14_I_PCG_CLKB_O_ENCODING           0x39
#define DAI_PB15_I_PCG_CLKB_O_ENCODING           0x39
#define DAI_PB16_I_PCG_CLKB_O_ENCODING           0x39
#define DAI_PB17_I_PCG_CLKB_O_ENCODING           0x39
#define DAI_PB18_I_PCG_CLKB_O_ENCODING           0x39
#define DAI_PB19_I_PCG_CLKB_O_ENCODING           0x39
#define DAI_PB20_I_PCG_CLKB_O_ENCODING           0x39

#define DAI_PB01_I_PCG_FSA_O_ENCODING            0x3A
#define DAI_PB02_I_PCG_FSA_O_ENCODING            0x3A
#define DAI_PB03_I_PCG_FSA_O_ENCODING            0x3A
#define DAI_PB04_I_PCG_FSA_O_ENCODING            0x3A
#define DAI_PB05_I_PCG_FSA_O_ENCODING            0x3A
#define DAI_PB06_I_PCG_FSA_O_ENCODING            0x3A
#define DAI_PB07_I_PCG_FSA_O_ENCODING            0x3A
#define DAI_PB08_I_PCG_FSA_O_ENCODING            0x3A
#define DAI_PB09_I_PCG_FSA_O_ENCODING            0x3A
#define DAI_PB10_I_PCG_FSA_O_ENCODING            0x3A
#define DAI_PB11_I_PCG_FSA_O_ENCODING            0x3A
#define DAI_PB12_I_PCG_FSA_O_ENCODING            0x3A
#define DAI_PB13_I_PCG_FSA_O_ENCODING            0x3A
#define DAI_PB14_I_PCG_FSA_O_ENCODING            0x3A
#define DAI_PB15_I_PCG_FSA_O_ENCODING            0x3A
#define DAI_PB16_I_PCG_FSA_O_ENCODING            0x3A
#define DAI_PB17_I_PCG_FSA_O_ENCODING            0x3A
#define DAI_PB18_I_PCG_FSA_O_ENCODING            0x3A
#define DAI_PB19_I_PCG_FSA_O_ENCODING            0x3A
#define DAI_PB20_I_PCG_FSA_O_ENCODING            0x3A

#define DAI_PB01_I_PCG_FSB_O_ENCODING            0x3B
#define DAI_PB02_I_PCG_FSB_O_ENCODING            0x3B
#define DAI_PB03_I_PCG_FSB_O_ENCODING            0x3B
#define DAI_PB04_I_PCG_FSB_O_ENCODING            0x3B
#define DAI_PB05_I_PCG_FSB_O_ENCODING            0x3B
#define DAI_PB06_I_PCG_FSB_O_ENCODING            0x3B
#define DAI_PB07_I_PCG_FSB_O_ENCODING            0x3B
#define DAI_PB08_I_PCG_FSB_O_ENCODING            0x3B
#define DAI_PB09_I_PCG_FSB_O_ENCODING            0x3B
#define DAI_PB10_I_PCG_FSB_O_ENCODING            0x3B
#define DAI_PB11_I_PCG_FSB_O_ENCODING            0x3B
#define DAI_PB12_I_PCG_FSB_O_ENCODING            0x3B
#define DAI_PB13_I_PCG_FSB_O_ENCODING            0x3B
#define DAI_PB14_I_PCG_FSB_O_ENCODING            0x3B
#define DAI_PB15_I_PCG_FSB_O_ENCODING            0x3B
#define DAI_PB16_I_PCG_FSB_O_ENCODING            0x3B
#define DAI_PB17_I_PCG_FSB_O_ENCODING            0x3B
#define DAI_PB18_I_PCG_FSB_O_ENCODING            0x3B
#define DAI_PB19_I_PCG_FSB_O_ENCODING            0x3B
#define DAI_PB20_I_PCG_FSB_O_ENCODING            0x3B

#define DAI_PB01_I_FLAG15_O_ENCODING             0x3C
#define DAI_PB02_I_FLAG15_O_ENCODING             0x3C
#define DAI_PB03_I_FLAG15_O_ENCODING             0x3C
#define DAI_PB04_I_FLAG15_O_ENCODING             0x3C
#define DAI_PB05_I_FLAG15_O_ENCODING             0x3C
#define DAI_PB06_I_FLAG15_O_ENCODING             0x3C
#define DAI_PB07_I_FLAG15_O_ENCODING             0x3C
#define DAI_PB08_I_FLAG15_O_ENCODING             0x3C
#define DAI_PB09_I_FLAG15_O_ENCODING             0x3C
#define DAI_PB10_I_FLAG15_O_ENCODING             0x3C
#define DAI_PB11_I_FLAG15_O_ENCODING             0x3C
#define DAI_PB12_I_FLAG15_O_ENCODING             0x3C
#define DAI_PB13_I_FLAG15_O_ENCODING             0x3C
#define DAI_PB14_I_FLAG15_O_ENCODING             0x3C
#define DAI_PB15_I_FLAG15_O_ENCODING             0x3C
#define DAI_PB16_I_FLAG15_O_ENCODING             0x3C
#define DAI_PB17_I_FLAG15_O_ENCODING             0x3C
#define DAI_PB18_I_FLAG15_O_ENCODING             0x3C
#define DAI_PB19_I_FLAG15_O_ENCODING             0x3C
#define DAI_PB20_I_FLAG15_O_ENCODING             0x3C

#define DAI_PB01_I_SRC0_DAT_OP_O_ENCODING             0x3D
#define DAI_PB02_I_SRC0_DAT_OP_O_ENCODING             0x3D
#define DAI_PB03_I_SRC0_DAT_OP_O_ENCODING             0x3D
#define DAI_PB04_I_SRC0_DAT_OP_O_ENCODING             0x3D
#define DAI_PB05_I_SRC0_DAT_OP_O_ENCODING             0x3D
#define DAI_PB06_I_SRC0_DAT_OP_O_ENCODING             0x3D
#define DAI_PB07_I_SRC0_DAT_OP_O_ENCODING             0x3D
#define DAI_PB08_I_SRC0_DAT_OP_O_ENCODING             0x3D
#define DAI_PB09_I_SRC0_DAT_OP_O_ENCODING             0x3D
#define DAI_PB10_I_SRC0_DAT_OP_O_ENCODING             0x3D
#define DAI_PB11_I_SRC0_DAT_OP_O_ENCODING             0x3D
#define DAI_PB12_I_SRC0_DAT_OP_O_ENCODING             0x3D
#define DAI_PB13_I_SRC0_DAT_OP_O_ENCODING             0x3D
#define DAI_PB14_I_SRC0_DAT_OP_O_ENCODING             0x3D
#define DAI_PB15_I_SRC0_DAT_OP_O_ENCODING             0x3D
#define DAI_PB16_I_SRC0_DAT_OP_O_ENCODING             0x3D
#define DAI_PB17_I_SRC0_DAT_OP_O_ENCODING             0x3D
#define DAI_PB18_I_SRC0_DAT_OP_O_ENCODING             0x3D
#define DAI_PB19_I_SRC0_DAT_OP_O_ENCODING             0x3D
#define DAI_PB20_I_SRC0_DAT_OP_O_ENCODING             0x3D

#define DAI_PB01_I_SRC1_DAT_OP_O_ENCODING             0x3E
#define DAI_PB02_I_SRC1_DAT_OP_O_ENCODING             0x3E
#define DAI_PB03_I_SRC1_DAT_OP_O_ENCODING             0x3E
#define DAI_PB04_I_SRC1_DAT_OP_O_ENCODING             0x3E
#define DAI_PB05_I_SRC1_DAT_OP_O_ENCODING             0x3E
#define DAI_PB06_I_SRC1_DAT_OP_O_ENCODING             0x3E
#define DAI_PB07_I_SRC1_DAT_OP_O_ENCODING             0x3E
#define DAI_PB08_I_SRC1_DAT_OP_O_ENCODING             0x3E
#define DAI_PB09_I_SRC1_DAT_OP_O_ENCODING             0x3E
#define DAI_PB10_I_SRC1_DAT_OP_O_ENCODING             0x3E
#define DAI_PB11_I_SRC1_DAT_OP_O_ENCODING             0x3E
#define DAI_PB12_I_SRC1_DAT_OP_O_ENCODING             0x3E
#define DAI_PB13_I_SRC1_DAT_OP_O_ENCODING             0x3E
#define DAI_PB14_I_SRC1_DAT_OP_O_ENCODING             0x3E
#define DAI_PB15_I_SRC1_DAT_OP_O_ENCODING             0x3E
#define DAI_PB16_I_SRC1_DAT_OP_O_ENCODING             0x3E
#define DAI_PB17_I_SRC1_DAT_OP_O_ENCODING             0x3E
#define DAI_PB18_I_SRC1_DAT_OP_O_ENCODING             0x3E
#define DAI_PB19_I_SRC1_DAT_OP_O_ENCODING             0x3E
#define DAI_PB20_I_SRC1_DAT_OP_O_ENCODING             0x3E

#define DAI_PB01_I_SRC2_DAT_OP_O_ENCODING             0x3F
#define DAI_PB02_I_SRC2_DAT_OP_O_ENCODING             0x3F
#define DAI_PB03_I_SRC2_DAT_OP_O_ENCODING             0x3F
#define DAI_PB04_I_SRC2_DAT_OP_O_ENCODING             0x3F
#define DAI_PB05_I_SRC2_DAT_OP_O_ENCODING             0x3F
#define DAI_PB06_I_SRC2_DAT_OP_O_ENCODING             0x3F
#define DAI_PB07_I_SRC2_DAT_OP_O_ENCODING             0x3F
#define DAI_PB08_I_SRC2_DAT_OP_O_ENCODING             0x3F
#define DAI_PB09_I_SRC2_DAT_OP_O_ENCODING             0x3F
#define DAI_PB10_I_SRC2_DAT_OP_O_ENCODING             0x3F
#define DAI_PB11_I_SRC2_DAT_OP_O_ENCODING             0x3F
#define DAI_PB12_I_SRC2_DAT_OP_O_ENCODING             0x3F
#define DAI_PB13_I_SRC2_DAT_OP_O_ENCODING             0x3F
#define DAI_PB14_I_SRC2_DAT_OP_O_ENCODING             0x3F
#define DAI_PB15_I_SRC2_DAT_OP_O_ENCODING             0x3F
#define DAI_PB16_I_SRC2_DAT_OP_O_ENCODING             0x3F
#define DAI_PB17_I_SRC2_DAT_OP_O_ENCODING             0x3F
#define DAI_PB18_I_SRC2_DAT_OP_O_ENCODING             0x3F
#define DAI_PB19_I_SRC2_DAT_OP_O_ENCODING             0x3F
#define DAI_PB20_I_SRC2_DAT_OP_O_ENCODING             0x3F

#define DAI_PB01_I_SRC3_DAT_OP_O_ENCODING             0x40
#define DAI_PB02_I_SRC3_DAT_OP_O_ENCODING             0x40
#define DAI_PB03_I_SRC3_DAT_OP_O_ENCODING             0x40
#define DAI_PB04_I_SRC3_DAT_OP_O_ENCODING             0x40
#define DAI_PB05_I_SRC3_DAT_OP_O_ENCODING             0x40
#define DAI_PB06_I_SRC3_DAT_OP_O_ENCODING             0x40
#define DAI_PB07_I_SRC3_DAT_OP_O_ENCODING             0x40
#define DAI_PB08_I_SRC3_DAT_OP_O_ENCODING             0x40
#define DAI_PB09_I_SRC3_DAT_OP_O_ENCODING             0x40
#define DAI_PB10_I_SRC3_DAT_OP_O_ENCODING             0x40
#define DAI_PB11_I_SRC3_DAT_OP_O_ENCODING             0x40
#define DAI_PB12_I_SRC3_DAT_OP_O_ENCODING             0x40
#define DAI_PB13_I_SRC3_DAT_OP_O_ENCODING             0x40
#define DAI_PB14_I_SRC3_DAT_OP_O_ENCODING             0x40
#define DAI_PB15_I_SRC3_DAT_OP_O_ENCODING             0x40
#define DAI_PB16_I_SRC3_DAT_OP_O_ENCODING             0x40
#define DAI_PB17_I_SRC3_DAT_OP_O_ENCODING             0x40
#define DAI_PB18_I_SRC3_DAT_OP_O_ENCODING             0x40
#define DAI_PB19_I_SRC3_DAT_OP_O_ENCODING             0x40
#define DAI_PB20_I_SRC3_DAT_OP_O_ENCODING             0x40

#define DAI_PB01_I_DIR_DAT_O_ENCODING             0x41
#define DAI_PB02_I_DIR_DAT_O_ENCODING             0x41
#define DAI_PB03_I_DIR_DAT_O_ENCODING             0x41
#define DAI_PB04_I_DIR_DAT_O_ENCODING             0x41
#define DAI_PB05_I_DIR_DAT_O_ENCODING             0x41
#define DAI_PB06_I_DIR_DAT_O_ENCODING             0x41
#define DAI_PB07_I_DIR_DAT_O_ENCODING             0x41
#define DAI_PB08_I_DIR_DAT_O_ENCODING             0x41
#define DAI_PB09_I_DIR_DAT_O_ENCODING             0x41
#define DAI_PB10_I_DIR_DAT_O_ENCODING             0x41
#define DAI_PB11_I_DIR_DAT_O_ENCODING             0x41
#define DAI_PB12_I_DIR_DAT_O_ENCODING             0x41
#define DAI_PB13_I_DIR_DAT_O_ENCODING             0x41
#define DAI_PB14_I_DIR_DAT_O_ENCODING             0x41
#define DAI_PB15_I_DIR_DAT_O_ENCODING             0x41
#define DAI_PB16_I_DIR_DAT_O_ENCODING             0x41
#define DAI_PB17_I_DIR_DAT_O_ENCODING             0x41
#define DAI_PB18_I_DIR_DAT_O_ENCODING             0x41
#define DAI_PB19_I_DIR_DAT_O_ENCODING             0x41
#define DAI_PB20_I_DIR_DAT_O_ENCODING             0x41

#define DAI_PB01_I_DIR_FS_O_ENCODING             0x42
#define DAI_PB02_I_DIR_FS_O_ENCODING             0x42
#define DAI_PB03_I_DIR_FS_O_ENCODING             0x42
#define DAI_PB04_I_DIR_FS_O_ENCODING             0x42
#define DAI_PB05_I_DIR_FS_O_ENCODING             0x42
#define DAI_PB06_I_DIR_FS_O_ENCODING             0x42
#define DAI_PB07_I_DIR_FS_O_ENCODING             0x42
#define DAI_PB08_I_DIR_FS_O_ENCODING             0x42
#define DAI_PB09_I_DIR_FS_O_ENCODING             0x42
#define DAI_PB10_I_DIR_FS_O_ENCODING             0x42
#define DAI_PB11_I_DIR_FS_O_ENCODING             0x42
#define DAI_PB12_I_DIR_FS_O_ENCODING             0x42
#define DAI_PB13_I_DIR_FS_O_ENCODING             0x42
#define DAI_PB14_I_DIR_FS_O_ENCODING             0x42
#define DAI_PB15_I_DIR_FS_O_ENCODING             0x42
#define DAI_PB16_I_DIR_FS_O_ENCODING             0x42
#define DAI_PB17_I_DIR_FS_O_ENCODING             0x42
#define DAI_PB18_I_DIR_FS_O_ENCODING             0x42
#define DAI_PB19_I_DIR_FS_O_ENCODING             0x42
#define DAI_PB20_I_DIR_FS_O_ENCODING             0x42

#define DAI_PB01_I_DIR_CLK_O_ENCODING             0x43
#define DAI_PB02_I_DIR_CLK_O_ENCODING             0x43
#define DAI_PB03_I_DIR_CLK_O_ENCODING             0x43
#define DAI_PB04_I_DIR_CLK_O_ENCODING             0x43
#define DAI_PB05_I_DIR_CLK_O_ENCODING             0x43
#define DAI_PB06_I_DIR_CLK_O_ENCODING             0x43
#define DAI_PB07_I_DIR_CLK_O_ENCODING             0x43
#define DAI_PB08_I_DIR_CLK_O_ENCODING             0x43
#define DAI_PB09_I_DIR_CLK_O_ENCODING             0x43
#define DAI_PB10_I_DIR_CLK_O_ENCODING             0x43
#define DAI_PB11_I_DIR_CLK_O_ENCODING             0x43
#define DAI_PB12_I_DIR_CLK_O_ENCODING             0x43
#define DAI_PB13_I_DIR_CLK_O_ENCODING             0x43
#define DAI_PB14_I_DIR_CLK_O_ENCODING             0x43
#define DAI_PB15_I_DIR_CLK_O_ENCODING             0x43
#define DAI_PB16_I_DIR_CLK_O_ENCODING             0x43
#define DAI_PB17_I_DIR_CLK_O_ENCODING             0x43
#define DAI_PB18_I_DIR_CLK_O_ENCODING             0x43
#define DAI_PB19_I_DIR_CLK_O_ENCODING             0x43
#define DAI_PB20_I_DIR_CLK_O_ENCODING             0x43

#define DAI_PB01_I_DIR_TDMCLK_O_ENCODING             0x44
#define DAI_PB02_I_DIR_TDMCLK_O_ENCODING             0x44
#define DAI_PB03_I_DIR_TDMCLK_O_ENCODING             0x44
#define DAI_PB04_I_DIR_TDMCLK_O_ENCODING             0x44
#define DAI_PB05_I_DIR_TDMCLK_O_ENCODING             0x44
#define DAI_PB06_I_DIR_TDMCLK_O_ENCODING             0x44
#define DAI_PB07_I_DIR_TDMCLK_O_ENCODING             0x44
#define DAI_PB08_I_DIR_TDMCLK_O_ENCODING             0x44
#define DAI_PB09_I_DIR_TDMCLK_O_ENCODING             0x44
#define DAI_PB10_I_DIR_TDMCLK_O_ENCODING             0x44
#define DAI_PB11_I_DIR_TDMCLK_O_ENCODING             0x44
#define DAI_PB12_I_DIR_TDMCLK_O_ENCODING             0x44
#define DAI_PB13_I_DIR_TDMCLK_O_ENCODING             0x44
#define DAI_PB14_I_DIR_TDMCLK_O_ENCODING             0x44
#define DAI_PB15_I_DIR_TDMCLK_O_ENCODING             0x44
#define DAI_PB16_I_DIR_TDMCLK_O_ENCODING             0x44
#define DAI_PB17_I_DIR_TDMCLK_O_ENCODING             0x44
#define DAI_PB18_I_DIR_TDMCLK_O_ENCODING             0x44
#define DAI_PB19_I_DIR_TDMCLK_O_ENCODING             0x44
#define DAI_PB20_I_DIR_TDMCLK_O_ENCODING             0x44

#define DAI_PB01_I_DIT_DAT_O_ENCODING             0x45
#define DAI_PB02_I_DIT_DAT_O_ENCODING             0x45
#define DAI_PB03_I_DIT_DAT_O_ENCODING             0x45
#define DAI_PB04_I_DIT_DAT_O_ENCODING             0x45
#define DAI_PB05_I_DIT_DAT_O_ENCODING             0x45
#define DAI_PB06_I_DIT_DAT_O_ENCODING             0x45
#define DAI_PB07_I_DIT_DAT_O_ENCODING             0x45
#define DAI_PB08_I_DIT_DAT_O_ENCODING             0x45
#define DAI_PB09_I_DIT_DAT_O_ENCODING             0x45
#define DAI_PB10_I_DIT_DAT_O_ENCODING             0x45
#define DAI_PB11_I_DIT_DAT_O_ENCODING             0x45
#define DAI_PB12_I_DIT_DAT_O_ENCODING             0x45
#define DAI_PB13_I_DIT_DAT_O_ENCODING             0x45
#define DAI_PB14_I_DIT_DAT_O_ENCODING             0x45
#define DAI_PB15_I_DIT_DAT_O_ENCODING             0x45
#define DAI_PB16_I_DIT_DAT_O_ENCODING             0x45
#define DAI_PB17_I_DIT_DAT_O_ENCODING             0x45
#define DAI_PB18_I_DIT_DAT_O_ENCODING             0x45
#define DAI_PB19_I_DIT_DAT_O_ENCODING             0x45
#define DAI_PB20_I_DIT_DAT_O_ENCODING             0x45
 
#define DAI_PB01_I_SPIB_MISO_O_ENCODING             0x46
#define DAI_PB02_I_SPIB_MISO_O_ENCODING             0x46
#define DAI_PB03_I_SPIB_MISO_O_ENCODING             0x46
#define DAI_PB04_I_SPIB_MISO_O_ENCODING             0x46
#define DAI_PB05_I_SPIB_MISO_O_ENCODING             0x46
#define DAI_PB06_I_SPIB_MISO_O_ENCODING             0x46
#define DAI_PB07_I_SPIB_MISO_O_ENCODING             0x46
#define DAI_PB08_I_SPIB_MISO_O_ENCODING             0x46
#define DAI_PB09_I_SPIB_MISO_O_ENCODING             0x46
#define DAI_PB10_I_SPIB_MISO_O_ENCODING             0x46
#define DAI_PB11_I_SPIB_MISO_O_ENCODING             0x46
#define DAI_PB12_I_SPIB_MISO_O_ENCODING             0x46
#define DAI_PB13_I_SPIB_MISO_O_ENCODING             0x46
#define DAI_PB14_I_SPIB_MISO_O_ENCODING             0x46
#define DAI_PB15_I_SPIB_MISO_O_ENCODING             0x46
#define DAI_PB16_I_SPIB_MISO_O_ENCODING             0x46
#define DAI_PB17_I_SPIB_MISO_O_ENCODING             0x46
#define DAI_PB18_I_SPIB_MISO_O_ENCODING             0x46
#define DAI_PB19_I_SPIB_MISO_O_ENCODING             0x46
#define DAI_PB20_I_SPIB_MISO_O_ENCODING             0x46

#define DAI_PB01_I_SPIB_MOSI_O_ENCODING             0x47
#define DAI_PB02_I_SPIB_MOSI_O_ENCODING             0x47
#define DAI_PB03_I_SPIB_MOSI_O_ENCODING             0x47
#define DAI_PB04_I_SPIB_MOSI_O_ENCODING             0x47
#define DAI_PB05_I_SPIB_MOSI_O_ENCODING             0x47
#define DAI_PB06_I_SPIB_MOSI_O_ENCODING             0x47
#define DAI_PB07_I_SPIB_MOSI_O_ENCODING             0x47
#define DAI_PB08_I_SPIB_MOSI_O_ENCODING             0x47
#define DAI_PB09_I_SPIB_MOSI_O_ENCODING             0x47
#define DAI_PB10_I_SPIB_MOSI_O_ENCODING             0x47
#define DAI_PB11_I_SPIB_MOSI_O_ENCODING             0x47
#define DAI_PB12_I_SPIB_MOSI_O_ENCODING             0x47
#define DAI_PB13_I_SPIB_MOSI_O_ENCODING             0x47
#define DAI_PB14_I_SPIB_MOSI_O_ENCODING             0x47
#define DAI_PB15_I_SPIB_MOSI_O_ENCODING             0x47
#define DAI_PB16_I_SPIB_MOSI_O_ENCODING             0x47
#define DAI_PB17_I_SPIB_MOSI_O_ENCODING             0x47
#define DAI_PB18_I_SPIB_MOSI_O_ENCODING             0x47
#define DAI_PB19_I_SPIB_MOSI_O_ENCODING             0x47
#define DAI_PB20_I_SPIB_MOSI_O_ENCODING             0x47

#define DAI_PB01_I_SPIB_CLK_O_ENCODING             0x48
#define DAI_PB02_I_SPIB_CLK_O_ENCODING             0x48
#define DAI_PB03_I_SPIB_CLK_O_ENCODING             0x48
#define DAI_PB04_I_SPIB_CLK_O_ENCODING             0x48
#define DAI_PB05_I_SPIB_CLK_O_ENCODING             0x48
#define DAI_PB06_I_SPIB_CLK_O_ENCODING             0x48
#define DAI_PB07_I_SPIB_CLK_O_ENCODING             0x48
#define DAI_PB08_I_SPIB_CLK_O_ENCODING             0x48
#define DAI_PB09_I_SPIB_CLK_O_ENCODING             0x48
#define DAI_PB10_I_SPIB_CLK_O_ENCODING             0x48
#define DAI_PB11_I_SPIB_CLK_O_ENCODING             0x48
#define DAI_PB12_I_SPIB_CLK_O_ENCODING             0x48
#define DAI_PB13_I_SPIB_CLK_O_ENCODING             0x48
#define DAI_PB14_I_SPIB_CLK_O_ENCODING             0x48
#define DAI_PB15_I_SPIB_CLK_O_ENCODING             0x48
#define DAI_PB16_I_SPIB_CLK_O_ENCODING             0x48
#define DAI_PB17_I_SPIB_CLK_O_ENCODING             0x48
#define DAI_PB18_I_SPIB_CLK_O_ENCODING             0x48
#define DAI_PB19_I_SPIB_CLK_O_ENCODING             0x48
#define DAI_PB20_I_SPIB_CLK_O_ENCODING             0x48

#define DAI_PB01_I_SPIB_FLG0_O_ENCODING             0x49
#define DAI_PB02_I_SPIB_FLG0_O_ENCODING             0x49
#define DAI_PB03_I_SPIB_FLG0_O_ENCODING             0x49
#define DAI_PB04_I_SPIB_FLG0_O_ENCODING             0x49
#define DAI_PB05_I_SPIB_FLG0_O_ENCODING             0x49
#define DAI_PB06_I_SPIB_FLG0_O_ENCODING             0x49
#define DAI_PB07_I_SPIB_FLG0_O_ENCODING             0x49
#define DAI_PB08_I_SPIB_FLG0_O_ENCODING             0x49
#define DAI_PB09_I_SPIB_FLG0_O_ENCODING             0x49
#define DAI_PB10_I_SPIB_FLG0_O_ENCODING             0x49
#define DAI_PB11_I_SPIB_FLG0_O_ENCODING             0x49
#define DAI_PB12_I_SPIB_FLG0_O_ENCODING             0x49
#define DAI_PB13_I_SPIB_FLG0_O_ENCODING             0x49
#define DAI_PB14_I_SPIB_FLG0_O_ENCODING             0x49
#define DAI_PB15_I_SPIB_FLG0_O_ENCODING             0x49
#define DAI_PB16_I_SPIB_FLG0_O_ENCODING             0x49
#define DAI_PB17_I_SPIB_FLG0_O_ENCODING             0x49
#define DAI_PB18_I_SPIB_FLG0_O_ENCODING             0x49
#define DAI_PB19_I_SPIB_FLG0_O_ENCODING             0x49
#define DAI_PB20_I_SPIB_FLG0_O_ENCODING             0x49

#define DAI_PB01_I_SPIB_FLG1_O_ENCODING             0x4A
#define DAI_PB02_I_SPIB_FLG1_O_ENCODING             0x4A
#define DAI_PB03_I_SPIB_FLG1_O_ENCODING             0x4A
#define DAI_PB04_I_SPIB_FLG1_O_ENCODING             0x4A
#define DAI_PB05_I_SPIB_FLG1_O_ENCODING             0x4A
#define DAI_PB06_I_SPIB_FLG1_O_ENCODING             0x4A
#define DAI_PB07_I_SPIB_FLG1_O_ENCODING             0x4A
#define DAI_PB08_I_SPIB_FLG1_O_ENCODING             0x4A
#define DAI_PB09_I_SPIB_FLG1_O_ENCODING             0x4A
#define DAI_PB10_I_SPIB_FLG1_O_ENCODING             0x4A
#define DAI_PB11_I_SPIB_FLG1_O_ENCODING             0x4A
#define DAI_PB12_I_SPIB_FLG1_O_ENCODING             0x4A
#define DAI_PB13_I_SPIB_FLG1_O_ENCODING             0x4A
#define DAI_PB14_I_SPIB_FLG1_O_ENCODING             0x4A
#define DAI_PB15_I_SPIB_FLG1_O_ENCODING             0x4A
#define DAI_PB16_I_SPIB_FLG1_O_ENCODING             0x4A
#define DAI_PB17_I_SPIB_FLG1_O_ENCODING             0x4A
#define DAI_PB18_I_SPIB_FLG1_O_ENCODING             0x4A
#define DAI_PB19_I_SPIB_FLG1_O_ENCODING             0x4A
#define DAI_PB20_I_SPIB_FLG1_O_ENCODING             0x4A

#define DAI_PB01_I_SPIB_FLG2_O_ENCODING             0x4B
#define DAI_PB02_I_SPIB_FLG2_O_ENCODING             0x4B
#define DAI_PB03_I_SPIB_FLG2_O_ENCODING             0x4B
#define DAI_PB04_I_SPIB_FLG2_O_ENCODING             0x4B
#define DAI_PB05_I_SPIB_FLG2_O_ENCODING             0x4B
#define DAI_PB06_I_SPIB_FLG2_O_ENCODING             0x4B
#define DAI_PB07_I_SPIB_FLG2_O_ENCODING             0x4B
#define DAI_PB08_I_SPIB_FLG2_O_ENCODING             0x4B
#define DAI_PB09_I_SPIB_FLG2_O_ENCODING             0x4B
#define DAI_PB10_I_SPIB_FLG2_O_ENCODING             0x4B
#define DAI_PB11_I_SPIB_FLG2_O_ENCODING             0x4B
#define DAI_PB12_I_SPIB_FLG2_O_ENCODING             0x4B
#define DAI_PB13_I_SPIB_FLG2_O_ENCODING             0x4B
#define DAI_PB14_I_SPIB_FLG2_O_ENCODING             0x4B
#define DAI_PB15_I_SPIB_FLG2_O_ENCODING             0x4B
#define DAI_PB16_I_SPIB_FLG2_O_ENCODING             0x4B
#define DAI_PB17_I_SPIB_FLG2_O_ENCODING             0x4B
#define DAI_PB18_I_SPIB_FLG2_O_ENCODING             0x4B
#define DAI_PB19_I_SPIB_FLG2_O_ENCODING             0x4B
#define DAI_PB20_I_SPIB_FLG2_O_ENCODING             0x4B

#define DAI_PB01_I_SPIB_FLG3_O_ENCODING             0x4C
#define DAI_PB02_I_SPIB_FLG3_O_ENCODING             0x4C
#define DAI_PB03_I_SPIB_FLG3_O_ENCODING             0x4C
#define DAI_PB04_I_SPIB_FLG3_O_ENCODING             0x4C
#define DAI_PB05_I_SPIB_FLG3_O_ENCODING             0x4C
#define DAI_PB06_I_SPIB_FLG3_O_ENCODING             0x4C
#define DAI_PB07_I_SPIB_FLG3_O_ENCODING             0x4C
#define DAI_PB08_I_SPIB_FLG3_O_ENCODING             0x4C
#define DAI_PB09_I_SPIB_FLG3_O_ENCODING             0x4C
#define DAI_PB10_I_SPIB_FLG3_O_ENCODING             0x4C
#define DAI_PB11_I_SPIB_FLG3_O_ENCODING             0x4C
#define DAI_PB12_I_SPIB_FLG3_O_ENCODING             0x4C
#define DAI_PB13_I_SPIB_FLG3_O_ENCODING             0x4C
#define DAI_PB14_I_SPIB_FLG3_O_ENCODING             0x4C
#define DAI_PB15_I_SPIB_FLG3_O_ENCODING             0x4C
#define DAI_PB16_I_SPIB_FLG3_O_ENCODING             0x4C
#define DAI_PB17_I_SPIB_FLG3_O_ENCODING             0x4C
#define DAI_PB18_I_SPIB_FLG3_O_ENCODING             0x4C
#define DAI_PB19_I_SPIB_FLG3_O_ENCODING             0x4C
#define DAI_PB20_I_SPIB_FLG3_O_ENCODING             0x4C

#define DAI_PB01_I_DIR_LRCLK_FB_O_ENCODING             0x4D
#define DAI_PB02_I_DIR_LRCLK_FB_O_ENCODING             0x4D
#define DAI_PB03_I_DIR_LRCLK_FB_O_ENCODING             0x4D
#define DAI_PB04_I_DIR_LRCLK_FB_O_ENCODING             0x4D
#define DAI_PB05_I_DIR_LRCLK_FB_O_ENCODING             0x4D
#define DAI_PB06_I_DIR_LRCLK_FB_O_ENCODING             0x4D
#define DAI_PB07_I_DIR_LRCLK_FB_O_ENCODING             0x4D
#define DAI_PB08_I_DIR_LRCLK_FB_O_ENCODING             0x4D
#define DAI_PB09_I_DIR_LRCLK_FB_O_ENCODING             0x4D
#define DAI_PB10_I_DIR_LRCLK_FB_O_ENCODING             0x4D
#define DAI_PB11_I_DIR_LRCLK_FB_O_ENCODING             0x4D
#define DAI_PB12_I_DIR_LRCLK_FB_O_ENCODING             0x4D
#define DAI_PB13_I_DIR_LRCLK_FB_O_ENCODING             0x4D
#define DAI_PB14_I_DIR_LRCLK_FB_O_ENCODING             0x4D
#define DAI_PB15_I_DIR_LRCLK_FB_O_ENCODING             0x4D
#define DAI_PB16_I_DIR_LRCLK_FB_O_ENCODING             0x4D
#define DAI_PB17_I_DIR_LRCLK_FB_O_ENCODING             0x4D
#define DAI_PB18_I_DIR_LRCLK_FB_O_ENCODING             0x4D
#define DAI_PB19_I_DIR_LRCLK_FB_O_ENCODING             0x4D
#define DAI_PB20_I_DIR_LRCLK_FB_O_ENCODING             0x4D

#define DAI_PB01_DIR_LRCLK_REF_O_ENCODING             0x4E
#define DAI_PB02_DIR_LRCLK_REF_O_ENCODING             0x4E
#define DAI_PB03_DIR_LRCLK_REF_O_ENCODING             0x4E
#define DAI_PB04_DIR_LRCLK_REF_O_ENCODING             0x4E
#define DAI_PB05_DIR_LRCLK_REF_O_ENCODING             0x4E
#define DAI_PB06_DIR_LRCLK_REF_O_ENCODING             0x4E
#define DAI_PB07_DIR_LRCLK_REF_O_ENCODING             0x4E
#define DAI_PB08_DIR_LRCLK_REF_O_ENCODING             0x4E
#define DAI_PB09_DIR_LRCLK_REF_O_ENCODING             0x4E
#define DAI_PB10_DIR_LRCLK_REF_O_ENCODING             0x4E
#define DAI_PB11_DIR_LRCLK_REF_O_ENCODING             0x4E
#define DAI_PB12_DIR_LRCLK_REF_O_ENCODING             0x4E
#define DAI_PB13_DIR_LRCLK_REF_O_ENCODING             0x4E
#define DAI_PB14_DIR_LRCLK_REF_O_ENCODING             0x4E
#define DAI_PB15_DIR_LRCLK_REF_O_ENCODING             0x4E
#define DAI_PB16_DIR_LRCLK_REF_O_ENCODING             0x4E
#define DAI_PB17_DIR_LRCLK_REF_O_ENCODING             0x4E
#define DAI_PB18_DIR_LRCLK_REF_O_ENCODING             0x4E
#define DAI_PB19_DIR_LRCLK_REF_O_ENCODING             0x4E
#define DAI_PB20_DIR_LRCLK_REF_O_ENCODING             0x4E
 
#define DAI_PB01_I_LOW_ENCODING                     0x50
#define DAI_PB02_I_LOW_ENCODING                     0x50
#define DAI_PB03_I_LOW_ENCODING                     0x50
#define DAI_PB04_I_LOW_ENCODING                     0x50
#define DAI_PB05_I_LOW_ENCODING                     0x50
#define DAI_PB06_I_LOW_ENCODING                     0x50
#define DAI_PB07_I_LOW_ENCODING                     0x50
#define DAI_PB08_I_LOW_ENCODING                     0x50
#define DAI_PB09_I_LOW_ENCODING                     0x50
#define DAI_PB10_I_LOW_ENCODING                     0x50
#define DAI_PB11_I_LOW_ENCODING                     0x50
#define DAI_PB12_I_LOW_ENCODING                     0x50
#define DAI_PB13_I_LOW_ENCODING                     0x50
#define DAI_PB14_I_LOW_ENCODING                     0x50
#define DAI_PB15_I_LOW_ENCODING                     0x50
#define DAI_PB16_I_LOW_ENCODING                     0x50
#define DAI_PB17_I_LOW_ENCODING                     0x50
#define DAI_PB18_I_LOW_ENCODING                     0x50
#define DAI_PB19_I_LOW_ENCODING                     0x50
#define DAI_PB20_I_LOW_ENCODING                     0x50

#define INV_DAI_PB19_LOW_ENCODING                   0x0
#define INV_DAI_PB20_LOW_ENCODING                   0x0
                         
#define DAI_PB01_I_HIGH_ENCODING                    0x51
#define DAI_PB02_I_HIGH_ENCODING                    0x51
#define DAI_PB03_I_HIGH_ENCODING                    0x51
#define DAI_PB04_I_HIGH_ENCODING                    0x51
#define DAI_PB05_I_HIGH_ENCODING                    0x51
#define DAI_PB06_I_HIGH_ENCODING                    0x51
#define DAI_PB07_I_HIGH_ENCODING                    0x51
#define DAI_PB08_I_HIGH_ENCODING                    0x51
#define DAI_PB09_I_HIGH_ENCODING                    0x51
#define DAI_PB10_I_HIGH_ENCODING                    0x51
#define DAI_PB11_I_HIGH_ENCODING                    0x51
#define DAI_PB12_I_HIGH_ENCODING                    0x51
#define DAI_PB13_I_HIGH_ENCODING                    0x51
#define DAI_PB14_I_HIGH_ENCODING                    0x51
#define DAI_PB15_I_HIGH_ENCODING                    0x51
#define DAI_PB16_I_HIGH_ENCODING                    0x51
#define DAI_PB17_I_HIGH_ENCODING                    0x51
#define DAI_PB18_I_HIGH_ENCODING                    0x51
#define DAI_PB19_I_HIGH_ENCODING                    0x51
#define DAI_PB20_I_HIGH_ENCODING                    0x51

#define INV_DAI_PB19_HIGH_ENCODING                  0x1
#define INV_DAI_PB20_HIGH_ENCODING                  0x1

//------------------------------------------------------------------------------
//  Group D: Register associations
//------------------------------------------------------------------------------

#define DAI_PB01_I_DAI_PB01_O_REG          0x2460     /* SRU_PIN0 */
#define DAI_PB02_I_DAI_PB01_O_REG          0x2460     
#define DAI_PB03_I_DAI_PB01_O_REG          0x2460     
#define DAI_PB04_I_DAI_PB01_O_REG          0x2460     
#define DAI_PB05_I_DAI_PB01_O_REG          0x2461     /* SRU_PIN1 */
#define DAI_PB06_I_DAI_PB01_O_REG          0x2461     
#define DAI_PB07_I_DAI_PB01_O_REG          0x2461     
#define DAI_PB08_I_DAI_PB01_O_REG          0x2461     
#define DAI_PB09_I_DAI_PB01_O_REG          0x2462     /* SRU_PIN2 */
#define DAI_PB10_I_DAI_PB01_O_REG          0x2462     
#define DAI_PB11_I_DAI_PB01_O_REG          0x2462     
#define DAI_PB12_I_DAI_PB01_O_REG          0x2462     
#define DAI_PB13_I_DAI_PB01_O_REG          0x2463     /* SRU_PIN3 */   
#define DAI_PB14_I_DAI_PB01_O_REG          0x2463     
#define DAI_PB15_I_DAI_PB01_O_REG          0x2463     
#define DAI_PB16_I_DAI_PB01_O_REG          0x2463     
#define DAI_PB17_I_DAI_PB01_O_REG          0x2464     /* SRU_PIN4 */
#define DAI_PB18_I_DAI_PB01_O_REG          0x2464     
#define DAI_PB19_I_DAI_PB01_O_REG          0x2464     
#define DAI_PB20_I_DAI_PB01_O_REG          0x2464     

#define DAI_PB01_I_DAI_PB02_O_REG          0x2460     /* SRU_PIN0 */
#define DAI_PB02_I_DAI_PB02_O_REG          0x2460                   
#define DAI_PB03_I_DAI_PB02_O_REG          0x2460                   
#define DAI_PB04_I_DAI_PB02_O_REG          0x2460                   
#define DAI_PB05_I_DAI_PB02_O_REG          0x2461     /* SRU_PIN1 */
#define DAI_PB06_I_DAI_PB02_O_REG          0x2461                   
#define DAI_PB07_I_DAI_PB02_O_REG          0x2461                   
#define DAI_PB08_I_DAI_PB02_O_REG          0x2461                   
#define DAI_PB09_I_DAI_PB02_O_REG          0x2462     /* SRU_PIN2 */
#define DAI_PB10_I_DAI_PB02_O_REG          0x2462                   
#define DAI_PB11_I_DAI_PB02_O_REG          0x2462                   
#define DAI_PB12_I_DAI_PB02_O_REG          0x2462                   
#define DAI_PB13_I_DAI_PB02_O_REG          0x2463     /* SRU_PIN3 */
#define DAI_PB14_I_DAI_PB02_O_REG          0x2463                   
#define DAI_PB15_I_DAI_PB02_O_REG          0x2463                   
#define DAI_PB16_I_DAI_PB02_O_REG          0x2463                   
#define DAI_PB17_I_DAI_PB02_O_REG          0x2464     /* SRU_PIN4 */
#define DAI_PB18_I_DAI_PB02_O_REG          0x2464                   
#define DAI_PB19_I_DAI_PB02_O_REG          0x2464                   
#define DAI_PB20_I_DAI_PB02_O_REG          0x2464                   

#define DAI_PB01_I_DAI_PB03_O_REG          0x2460     /* SRU_PIN0 */
#define DAI_PB02_I_DAI_PB03_O_REG          0x2460                   
#define DAI_PB03_I_DAI_PB03_O_REG          0x2460                   
#define DAI_PB04_I_DAI_PB03_O_REG          0x2460                   
#define DAI_PB05_I_DAI_PB03_O_REG          0x2461     /* SRU_PIN1 */
#define DAI_PB06_I_DAI_PB03_O_REG          0x2461                   
#define DAI_PB07_I_DAI_PB03_O_REG          0x2461                   
#define DAI_PB08_I_DAI_PB03_O_REG          0x2461                   
#define DAI_PB09_I_DAI_PB03_O_REG          0x2462     /* SRU_PIN2 */
#define DAI_PB10_I_DAI_PB03_O_REG          0x2462                   
#define DAI_PB11_I_DAI_PB03_O_REG          0x2462                   
#define DAI_PB12_I_DAI_PB03_O_REG          0x2462                   
#define DAI_PB13_I_DAI_PB03_O_REG          0x2463     /* SRU_PIN3 */
#define DAI_PB14_I_DAI_PB03_O_REG          0x2463                   
#define DAI_PB15_I_DAI_PB03_O_REG          0x2463                   
#define DAI_PB16_I_DAI_PB03_O_REG          0x2463                   
#define DAI_PB17_I_DAI_PB03_O_REG          0x2464     /* SRU_PIN4 */
#define DAI_PB18_I_DAI_PB03_O_REG          0x2464                   
#define DAI_PB19_I_DAI_PB03_O_REG          0x2464                   
#define DAI_PB20_I_DAI_PB03_O_REG          0x2464                   

#define DAI_PB01_I_DAI_PB04_O_REG          0x2460     /* SRU_PIN0 */
#define DAI_PB02_I_DAI_PB04_O_REG          0x2460                   
#define DAI_PB03_I_DAI_PB04_O_REG          0x2460                   
#define DAI_PB04_I_DAI_PB04_O_REG          0x2460                   
#define DAI_PB05_I_DAI_PB04_O_REG          0x2461     /* SRU_PIN1 */
#define DAI_PB06_I_DAI_PB04_O_REG          0x2461                   
#define DAI_PB07_I_DAI_PB04_O_REG          0x2461                   
#define DAI_PB08_I_DAI_PB04_O_REG          0x2461                   
#define DAI_PB09_I_DAI_PB04_O_REG          0x2462     /* SRU_PIN2 */
#define DAI_PB10_I_DAI_PB04_O_REG          0x2462                   
#define DAI_PB11_I_DAI_PB04_O_REG          0x2462                   
#define DAI_PB12_I_DAI_PB04_O_REG          0x2462                   
#define DAI_PB13_I_DAI_PB04_O_REG          0x2463     /* SRU_PIN3 */
#define DAI_PB14_I_DAI_PB04_O_REG          0x2463                   
#define DAI_PB15_I_DAI_PB04_O_REG          0x2463                   
#define DAI_PB16_I_DAI_PB04_O_REG          0x2463                   
#define DAI_PB17_I_DAI_PB04_O_REG          0x2464     /* SRU_PIN4 */
#define DAI_PB18_I_DAI_PB04_O_REG          0x2464                   
#define DAI_PB19_I_DAI_PB04_O_REG          0x2464                   
#define DAI_PB20_I_DAI_PB04_O_REG          0x2464                   

#define DAI_PB01_I_DAI_PB05_O_REG          0x2460     /* SRU_PIN0 */
#define DAI_PB02_I_DAI_PB05_O_REG          0x2460                   
#define DAI_PB03_I_DAI_PB05_O_REG          0x2460                   
#define DAI_PB04_I_DAI_PB05_O_REG          0x2460                   
#define DAI_PB05_I_DAI_PB05_O_REG          0x2461     /* SRU_PIN1 */
#define DAI_PB06_I_DAI_PB05_O_REG          0x2461                   
#define DAI_PB07_I_DAI_PB05_O_REG          0x2461                   
#define DAI_PB08_I_DAI_PB05_O_REG          0x2461                   
#define DAI_PB09_I_DAI_PB05_O_REG          0x2462     /* SRU_PIN2 */
#define DAI_PB10_I_DAI_PB05_O_REG          0x2462                   
#define DAI_PB11_I_DAI_PB05_O_REG          0x2462                   
#define DAI_PB12_I_DAI_PB05_O_REG          0x2462                   
#define DAI_PB13_I_DAI_PB05_O_REG          0x2463     /* SRU_PIN3 */
#define DAI_PB14_I_DAI_PB05_O_REG          0x2463                   
#define DAI_PB15_I_DAI_PB05_O_REG          0x2463                   
#define DAI_PB16_I_DAI_PB05_O_REG          0x2463                   
#define DAI_PB17_I_DAI_PB05_O_REG          0x2464     /* SRU_PIN4 */
#define DAI_PB18_I_DAI_PB05_O_REG          0x2464                   
#define DAI_PB19_I_DAI_PB05_O_REG          0x2464                   
#define DAI_PB20_I_DAI_PB05_O_REG          0x2464                   

#define DAI_PB01_I_DAI_PB06_O_REG          0x2460     /* SRU_PIN0 */
#define DAI_PB02_I_DAI_PB06_O_REG          0x2460                   
#define DAI_PB03_I_DAI_PB06_O_REG          0x2460                   
#define DAI_PB04_I_DAI_PB06_O_REG          0x2460                   
#define DAI_PB05_I_DAI_PB06_O_REG          0x2461     /* SRU_PIN1 */
#define DAI_PB06_I_DAI_PB06_O_REG          0x2461                   
#define DAI_PB07_I_DAI_PB06_O_REG          0x2461                   
#define DAI_PB08_I_DAI_PB06_O_REG          0x2461                   
#define DAI_PB09_I_DAI_PB06_O_REG          0x2462     /* SRU_PIN2 */
#define DAI_PB10_I_DAI_PB06_O_REG          0x2462                   
#define DAI_PB11_I_DAI_PB06_O_REG          0x2462                   
#define DAI_PB12_I_DAI_PB06_O_REG          0x2462                   
#define DAI_PB13_I_DAI_PB06_O_REG          0x2463     /* SRU_PIN3 */
#define DAI_PB14_I_DAI_PB06_O_REG          0x2463                   
#define DAI_PB15_I_DAI_PB06_O_REG          0x2463                   
#define DAI_PB16_I_DAI_PB06_O_REG          0x2463                   
#define DAI_PB17_I_DAI_PB06_O_REG          0x2464     /* SRU_PIN4 */
#define DAI_PB18_I_DAI_PB06_O_REG          0x2464                   
#define DAI_PB19_I_DAI_PB06_O_REG          0x2464                   
#define DAI_PB20_I_DAI_PB06_O_REG          0x2464                   

#define DAI_PB01_I_DAI_PB07_O_REG          0x2460     /* SRU_PIN0 */
#define DAI_PB02_I_DAI_PB07_O_REG          0x2460                   
#define DAI_PB03_I_DAI_PB07_O_REG          0x2460                   
#define DAI_PB04_I_DAI_PB07_O_REG          0x2460                   
#define DAI_PB05_I_DAI_PB07_O_REG          0x2461     /* SRU_PIN1 */
#define DAI_PB06_I_DAI_PB07_O_REG          0x2461                   
#define DAI_PB07_I_DAI_PB07_O_REG          0x2461                   
#define DAI_PB08_I_DAI_PB07_O_REG          0x2461                   
#define DAI_PB09_I_DAI_PB07_O_REG          0x2462     /* SRU_PIN2 */
#define DAI_PB10_I_DAI_PB07_O_REG          0x2462                   
#define DAI_PB11_I_DAI_PB07_O_REG          0x2462                   
#define DAI_PB12_I_DAI_PB07_O_REG          0x2462                   
#define DAI_PB13_I_DAI_PB07_O_REG          0x2463     /* SRU_PIN3 */
#define DAI_PB14_I_DAI_PB07_O_REG          0x2463                   
#define DAI_PB15_I_DAI_PB07_O_REG          0x2463                   
#define DAI_PB16_I_DAI_PB07_O_REG          0x2463                   
#define DAI_PB17_I_DAI_PB07_O_REG          0x2464     /* SRU_PIN4 */
#define DAI_PB18_I_DAI_PB07_O_REG          0x2464                   
#define DAI_PB19_I_DAI_PB07_O_REG          0x2464                   
#define DAI_PB20_I_DAI_PB07_O_REG          0x2464                   

#define DAI_PB01_I_DAI_PB08_O_REG          0x2460     /* SRU_PIN0 */
#define DAI_PB02_I_DAI_PB08_O_REG          0x2460                   
#define DAI_PB03_I_DAI_PB08_O_REG          0x2460                   
#define DAI_PB04_I_DAI_PB08_O_REG          0x2460                   
#define DAI_PB05_I_DAI_PB08_O_REG          0x2461     /* SRU_PIN1 */
#define DAI_PB06_I_DAI_PB08_O_REG          0x2461                   
#define DAI_PB07_I_DAI_PB08_O_REG          0x2461                   
#define DAI_PB08_I_DAI_PB08_O_REG          0x2461                   
#define DAI_PB09_I_DAI_PB08_O_REG          0x2462     /* SRU_PIN2 */
#define DAI_PB10_I_DAI_PB08_O_REG          0x2462                   
#define DAI_PB11_I_DAI_PB08_O_REG          0x2462                   
#define DAI_PB12_I_DAI_PB08_O_REG          0x2462                   
#define DAI_PB13_I_DAI_PB08_O_REG          0x2463     /* SRU_PIN3 */
#define DAI_PB14_I_DAI_PB08_O_REG          0x2463                   
#define DAI_PB15_I_DAI_PB08_O_REG          0x2463                   
#define DAI_PB16_I_DAI_PB08_O_REG          0x2463                   
#define DAI_PB17_I_DAI_PB08_O_REG          0x2464     /* SRU_PIN4 */
#define DAI_PB18_I_DAI_PB08_O_REG          0x2464                   
#define DAI_PB19_I_DAI_PB08_O_REG          0x2464                   
#define DAI_PB20_I_DAI_PB08_O_REG          0x2464                   

#define DAI_PB01_I_DAI_PB09_O_REG          0x2460     /* SRU_PIN0 */
#define DAI_PB02_I_DAI_PB09_O_REG          0x2460                   
#define DAI_PB03_I_DAI_PB09_O_REG          0x2460                   
#define DAI_PB04_I_DAI_PB09_O_REG          0x2460                   
#define DAI_PB05_I_DAI_PB09_O_REG          0x2461     /* SRU_PIN1 */
#define DAI_PB06_I_DAI_PB09_O_REG          0x2461                   
#define DAI_PB07_I_DAI_PB09_O_REG          0x2461                   
#define DAI_PB08_I_DAI_PB09_O_REG          0x2461                   
#define DAI_PB09_I_DAI_PB09_O_REG          0x2462     /* SRU_PIN2 */
#define DAI_PB10_I_DAI_PB09_O_REG          0x2462                   
#define DAI_PB11_I_DAI_PB09_O_REG          0x2462                   
#define DAI_PB12_I_DAI_PB09_O_REG          0x2462                   
#define DAI_PB13_I_DAI_PB09_O_REG          0x2463     /* SRU_PIN3 */
#define DAI_PB14_I_DAI_PB09_O_REG          0x2463                   
#define DAI_PB15_I_DAI_PB09_O_REG          0x2463                   
#define DAI_PB16_I_DAI_PB09_O_REG          0x2463                   
#define DAI_PB17_I_DAI_PB09_O_REG          0x2464     /* SRU_PIN4 */
#define DAI_PB18_I_DAI_PB09_O_REG          0x2464                   
#define DAI_PB19_I_DAI_PB09_O_REG          0x2464                   
#define DAI_PB20_I_DAI_PB09_O_REG          0x2464                   

#define DAI_PB01_I_DAI_PB10_O_REG          0x2460     /* SRU_PIN0 */
#define DAI_PB02_I_DAI_PB10_O_REG          0x2460                   
#define DAI_PB03_I_DAI_PB10_O_REG          0x2460                   
#define DAI_PB04_I_DAI_PB10_O_REG          0x2460                   
#define DAI_PB05_I_DAI_PB10_O_REG          0x2461     /* SRU_PIN1 */
#define DAI_PB06_I_DAI_PB10_O_REG          0x2461                   
#define DAI_PB07_I_DAI_PB10_O_REG          0x2461                   
#define DAI_PB08_I_DAI_PB10_O_REG          0x2461                   
#define DAI_PB09_I_DAI_PB10_O_REG          0x2462     /* SRU_PIN2 */
#define DAI_PB10_I_DAI_PB10_O_REG          0x2462                   
#define DAI_PB11_I_DAI_PB10_O_REG          0x2462                   
#define DAI_PB12_I_DAI_PB10_O_REG          0x2462                   
#define DAI_PB13_I_DAI_PB10_O_REG          0x2463     /* SRU_PIN3 */
#define DAI_PB14_I_DAI_PB10_O_REG          0x2463                   
#define DAI_PB15_I_DAI_PB10_O_REG          0x2463                   
#define DAI_PB16_I_DAI_PB10_O_REG          0x2463                   
#define DAI_PB17_I_DAI_PB10_O_REG          0x2464     /* SRU_PIN4 */
#define DAI_PB18_I_DAI_PB10_O_REG          0x2464                   
#define DAI_PB19_I_DAI_PB10_O_REG          0x2464                   
#define DAI_PB20_I_DAI_PB10_O_REG          0x2464                   

#define DAI_PB01_I_DAI_PB11_O_REG          0x2460     /* SRU_PIN0 */
#define DAI_PB02_I_DAI_PB11_O_REG          0x2460                   
#define DAI_PB03_I_DAI_PB11_O_REG          0x2460                   
#define DAI_PB04_I_DAI_PB11_O_REG          0x2460                   
#define DAI_PB05_I_DAI_PB11_O_REG          0x2461     /* SRU_PIN1 */
#define DAI_PB06_I_DAI_PB11_O_REG          0x2461                   
#define DAI_PB07_I_DAI_PB11_O_REG          0x2461                   
#define DAI_PB08_I_DAI_PB11_O_REG          0x2461                   
#define DAI_PB09_I_DAI_PB11_O_REG          0x2462     /* SRU_PIN2 */
#define DAI_PB10_I_DAI_PB11_O_REG          0x2462                   
#define DAI_PB11_I_DAI_PB11_O_REG          0x2462                   
#define DAI_PB12_I_DAI_PB11_O_REG          0x2462                   
#define DAI_PB13_I_DAI_PB11_O_REG          0x2463     /* SRU_PIN3 */
#define DAI_PB14_I_DAI_PB11_O_REG          0x2463                   
#define DAI_PB15_I_DAI_PB11_O_REG          0x2463                   
#define DAI_PB16_I_DAI_PB11_O_REG          0x2463                   
#define DAI_PB17_I_DAI_PB11_O_REG          0x2464     /* SRU_PIN4 */
#define DAI_PB18_I_DAI_PB11_O_REG          0x2464                   
#define DAI_PB19_I_DAI_PB11_O_REG          0x2464                   
#define DAI_PB20_I_DAI_PB11_O_REG          0x2464                   

#define DAI_PB01_I_DAI_PB12_O_REG          0x2460     /* SRU_PIN0 */
#define DAI_PB02_I_DAI_PB12_O_REG          0x2460                   
#define DAI_PB03_I_DAI_PB12_O_REG          0x2460                   
#define DAI_PB04_I_DAI_PB12_O_REG          0x2460                   
#define DAI_PB05_I_DAI_PB12_O_REG          0x2461     /* SRU_PIN1 */
#define DAI_PB06_I_DAI_PB12_O_REG          0x2461                   
#define DAI_PB07_I_DAI_PB12_O_REG          0x2461                   
#define DAI_PB08_I_DAI_PB12_O_REG          0x2461                   
#define DAI_PB09_I_DAI_PB12_O_REG          0x2462     /* SRU_PIN2 */
#define DAI_PB10_I_DAI_PB12_O_REG          0x2462                   
#define DAI_PB11_I_DAI_PB12_O_REG          0x2462                   
#define DAI_PB12_I_DAI_PB12_O_REG          0x2462                   
#define DAI_PB13_I_DAI_PB12_O_REG          0x2463     /* SRU_PIN3 */
#define DAI_PB14_I_DAI_PB12_O_REG          0x2463                   
#define DAI_PB15_I_DAI_PB12_O_REG          0x2463                   
#define DAI_PB16_I_DAI_PB12_O_REG          0x2463                   
#define DAI_PB17_I_DAI_PB12_O_REG          0x2464     /* SRU_PIN4 */
#define DAI_PB18_I_DAI_PB12_O_REG          0x2464                   
#define DAI_PB19_I_DAI_PB12_O_REG          0x2464                   
#define DAI_PB20_I_DAI_PB12_O_REG          0x2464                   

#define DAI_PB01_I_DAI_PB13_O_REG          0x2460     /* SRU_PIN0 */
#define DAI_PB02_I_DAI_PB13_O_REG          0x2460                   
#define DAI_PB03_I_DAI_PB13_O_REG          0x2460                   
#define DAI_PB04_I_DAI_PB13_O_REG          0x2460                   
#define DAI_PB05_I_DAI_PB13_O_REG          0x2461     /* SRU_PIN1 */
#define DAI_PB06_I_DAI_PB13_O_REG          0x2461                   
#define DAI_PB07_I_DAI_PB13_O_REG          0x2461                   
#define DAI_PB08_I_DAI_PB13_O_REG          0x2461                   
#define DAI_PB09_I_DAI_PB13_O_REG          0x2462     /* SRU_PIN2 */
#define DAI_PB10_I_DAI_PB13_O_REG          0x2462                   
#define DAI_PB11_I_DAI_PB13_O_REG          0x2462                   
#define DAI_PB12_I_DAI_PB13_O_REG          0x2462                   
#define DAI_PB13_I_DAI_PB13_O_REG          0x2463     /* SRU_PIN3 */
#define DAI_PB14_I_DAI_PB13_O_REG          0x2463                   
#define DAI_PB15_I_DAI_PB13_O_REG          0x2463                   
#define DAI_PB16_I_DAI_PB13_O_REG          0x2463                   
#define DAI_PB17_I_DAI_PB13_O_REG          0x2464     /* SRU_PIN4 */
#define DAI_PB18_I_DAI_PB13_O_REG          0x2464                   
#define DAI_PB19_I_DAI_PB13_O_REG          0x2464                   
#define DAI_PB20_I_DAI_PB13_O_REG          0x2464                   

#define DAI_PB01_I_DAI_PB14_O_REG          0x2460     /* SRU_PIN0 */
#define DAI_PB02_I_DAI_PB14_O_REG          0x2460                   
#define DAI_PB03_I_DAI_PB14_O_REG          0x2460                   
#define DAI_PB04_I_DAI_PB14_O_REG          0x2460                   
#define DAI_PB05_I_DAI_PB14_O_REG          0x2461     /* SRU_PIN1 */
#define DAI_PB06_I_DAI_PB14_O_REG          0x2461                   
#define DAI_PB07_I_DAI_PB14_O_REG          0x2461                   
#define DAI_PB08_I_DAI_PB14_O_REG          0x2461                   
#define DAI_PB09_I_DAI_PB14_O_REG          0x2462     /* SRU_PIN2 */
#define DAI_PB10_I_DAI_PB14_O_REG          0x2462                   
#define DAI_PB11_I_DAI_PB14_O_REG          0x2462                   
#define DAI_PB12_I_DAI_PB14_O_REG          0x2462                   
#define DAI_PB13_I_DAI_PB14_O_REG          0x2463     /* SRU_PIN3 */
#define DAI_PB14_I_DAI_PB14_O_REG          0x2463                   
#define DAI_PB15_I_DAI_PB14_O_REG          0x2463                   
#define DAI_PB16_I_DAI_PB14_O_REG          0x2463                   
#define DAI_PB17_I_DAI_PB14_O_REG          0x2464     /* SRU_PIN4 */
#define DAI_PB18_I_DAI_PB14_O_REG          0x2464                   
#define DAI_PB19_I_DAI_PB14_O_REG          0x2464                   
#define DAI_PB20_I_DAI_PB14_O_REG          0x2464                   

#define DAI_PB01_I_DAI_PB15_O_REG          0x2460     /* SRU_PIN0 */
#define DAI_PB02_I_DAI_PB15_O_REG          0x2460                   
#define DAI_PB03_I_DAI_PB15_O_REG          0x2460                   
#define DAI_PB04_I_DAI_PB15_O_REG          0x2460                   
#define DAI_PB05_I_DAI_PB15_O_REG          0x2461     /* SRU_PIN1 */
#define DAI_PB06_I_DAI_PB15_O_REG          0x2461                   
#define DAI_PB07_I_DAI_PB15_O_REG          0x2461                   
#define DAI_PB08_I_DAI_PB15_O_REG          0x2461                   
#define DAI_PB09_I_DAI_PB15_O_REG          0x2462     /* SRU_PIN2 */
#define DAI_PB10_I_DAI_PB15_O_REG          0x2462                   
#define DAI_PB11_I_DAI_PB15_O_REG          0x2462                   
#define DAI_PB12_I_DAI_PB15_O_REG          0x2462                   
#define DAI_PB13_I_DAI_PB15_O_REG          0x2463     /* SRU_PIN3 */
#define DAI_PB14_I_DAI_PB15_O_REG          0x2463                   
#define DAI_PB15_I_DAI_PB15_O_REG          0x2463                   
#define DAI_PB16_I_DAI_PB15_O_REG          0x2463                   
#define DAI_PB17_I_DAI_PB15_O_REG          0x2464     /* SRU_PIN4 */
#define DAI_PB18_I_DAI_PB15_O_REG          0x2464                   
#define DAI_PB19_I_DAI_PB15_O_REG          0x2464                   
#define DAI_PB20_I_DAI_PB15_O_REG          0x2464                   

#define DAI_PB01_I_DAI_PB16_O_REG          0x2460     /* SRU_PIN0 */
#define DAI_PB02_I_DAI_PB16_O_REG          0x2460                   
#define DAI_PB03_I_DAI_PB16_O_REG          0x2460                   
#define DAI_PB04_I_DAI_PB16_O_REG          0x2460                   
#define DAI_PB05_I_DAI_PB16_O_REG          0x2461     /* SRU_PIN1 */
#define DAI_PB06_I_DAI_PB16_O_REG          0x2461                   
#define DAI_PB07_I_DAI_PB16_O_REG          0x2461                   
#define DAI_PB08_I_DAI_PB16_O_REG          0x2461                   
#define DAI_PB09_I_DAI_PB16_O_REG          0x2462     /* SRU_PIN2 */
#define DAI_PB10_I_DAI_PB16_O_REG          0x2462                   
#define DAI_PB11_I_DAI_PB16_O_REG          0x2462                   
#define DAI_PB12_I_DAI_PB16_O_REG          0x2462                   
#define DAI_PB13_I_DAI_PB16_O_REG          0x2463     /* SRU_PIN3 */
#define DAI_PB14_I_DAI_PB16_O_REG          0x2463                   
#define DAI_PB15_I_DAI_PB16_O_REG          0x2463                   
#define DAI_PB16_I_DAI_PB16_O_REG          0x2463                   
#define DAI_PB17_I_DAI_PB16_O_REG          0x2464     /* SRU_PIN4 */
#define DAI_PB18_I_DAI_PB16_O_REG          0x2464                   
#define DAI_PB19_I_DAI_PB16_O_REG          0x2464                   
#define DAI_PB20_I_DAI_PB16_O_REG          0x2464                   

#define DAI_PB01_I_DAI_PB17_O_REG          0x2460     /* SRU_PIN0 */
#define DAI_PB02_I_DAI_PB17_O_REG          0x2460                   
#define DAI_PB03_I_DAI_PB17_O_REG          0x2460                   
#define DAI_PB04_I_DAI_PB17_O_REG          0x2460                   
#define DAI_PB05_I_DAI_PB17_O_REG          0x2461     /* SRU_PIN1 */
#define DAI_PB06_I_DAI_PB17_O_REG          0x2461                   
#define DAI_PB07_I_DAI_PB17_O_REG          0x2461                   
#define DAI_PB08_I_DAI_PB17_O_REG          0x2461                   
#define DAI_PB09_I_DAI_PB17_O_REG          0x2462     /* SRU_PIN2 */
#define DAI_PB10_I_DAI_PB17_O_REG          0x2462                   
#define DAI_PB11_I_DAI_PB17_O_REG          0x2462                   
#define DAI_PB12_I_DAI_PB17_O_REG          0x2462                   
#define DAI_PB13_I_DAI_PB17_O_REG          0x2463     /* SRU_PIN3 */
#define DAI_PB14_I_DAI_PB17_O_REG          0x2463                   
#define DAI_PB15_I_DAI_PB17_O_REG          0x2463                   
#define DAI_PB16_I_DAI_PB17_O_REG          0x2463                   
#define DAI_PB17_I_DAI_PB17_O_REG          0x2464     /* SRU_PIN4 */
#define DAI_PB18_I_DAI_PB17_O_REG          0x2464                   
#define DAI_PB19_I_DAI_PB17_O_REG          0x2464                   
#define DAI_PB20_I_DAI_PB17_O_REG          0x2464                   

#define DAI_PB01_I_DAI_PB18_O_REG          0x2460     /* SRU_PIN0 */
#define DAI_PB02_I_DAI_PB18_O_REG          0x2460                   
#define DAI_PB03_I_DAI_PB18_O_REG          0x2460                   
#define DAI_PB04_I_DAI_PB18_O_REG          0x2460                   
#define DAI_PB05_I_DAI_PB18_O_REG          0x2461     /* SRU_PIN1 */
#define DAI_PB06_I_DAI_PB18_O_REG          0x2461                   
#define DAI_PB07_I_DAI_PB18_O_REG          0x2461                   
#define DAI_PB08_I_DAI_PB18_O_REG          0x2461                   
#define DAI_PB09_I_DAI_PB18_O_REG          0x2462     /* SRU_PIN2 */
#define DAI_PB10_I_DAI_PB18_O_REG          0x2462                   
#define DAI_PB11_I_DAI_PB18_O_REG          0x2462                   
#define DAI_PB12_I_DAI_PB18_O_REG          0x2462                   
#define DAI_PB13_I_DAI_PB18_O_REG          0x2463     /* SRU_PIN3 */
#define DAI_PB14_I_DAI_PB18_O_REG          0x2463                   
#define DAI_PB15_I_DAI_PB18_O_REG          0x2463                   
#define DAI_PB16_I_DAI_PB18_O_REG          0x2463                   
#define DAI_PB17_I_DAI_PB18_O_REG          0x2464     /* SRU_PIN4 */
#define DAI_PB18_I_DAI_PB18_O_REG          0x2464                   
#define DAI_PB19_I_DAI_PB18_O_REG          0x2464                   
#define DAI_PB20_I_DAI_PB18_O_REG          0x2464                   

#define DAI_PB01_I_DAI_PB19_O_REG          0x2460     /* SRU_PIN0 */
#define DAI_PB02_I_DAI_PB19_O_REG          0x2460                   
#define DAI_PB03_I_DAI_PB19_O_REG          0x2460                   
#define DAI_PB04_I_DAI_PB19_O_REG          0x2460                   
#define DAI_PB05_I_DAI_PB19_O_REG          0x2461     /* SRU_PIN1 */
#define DAI_PB06_I_DAI_PB19_O_REG          0x2461                   
#define DAI_PB07_I_DAI_PB19_O_REG          0x2461                   
#define DAI_PB08_I_DAI_PB19_O_REG          0x2461                   
#define DAI_PB09_I_DAI_PB19_O_REG          0x2462     /* SRU_PIN2 */
#define DAI_PB10_I_DAI_PB19_O_REG          0x2462                   
#define DAI_PB11_I_DAI_PB19_O_REG          0x2462                   
#define DAI_PB12_I_DAI_PB19_O_REG          0x2462                   
#define DAI_PB13_I_DAI_PB19_O_REG          0x2463     /* SRU_PIN3 */
#define DAI_PB14_I_DAI_PB19_O_REG          0x2463                   
#define DAI_PB15_I_DAI_PB19_O_REG          0x2463                   
#define DAI_PB16_I_DAI_PB19_O_REG          0x2463                   
#define DAI_PB17_I_DAI_PB19_O_REG          0x2464     /* SRU_PIN4 */
#define DAI_PB18_I_DAI_PB19_O_REG          0x2464                   
#define DAI_PB19_I_DAI_PB19_O_REG          0x2464                   
#define DAI_PB20_I_DAI_PB19_O_REG          0x2464                   

#define DAI_PB01_I_DAI_PB20_O_REG          0x2460     /* SRU_PIN0 */
#define DAI_PB02_I_DAI_PB20_O_REG          0x2460                   
#define DAI_PB03_I_DAI_PB20_O_REG          0x2460                   
#define DAI_PB04_I_DAI_PB20_O_REG          0x2460                   
#define DAI_PB05_I_DAI_PB20_O_REG          0x2461     /* SRU_PIN1 */
#define DAI_PB06_I_DAI_PB20_O_REG          0x2461                   
#define DAI_PB07_I_DAI_PB20_O_REG          0x2461                   
#define DAI_PB08_I_DAI_PB20_O_REG          0x2461                   
#define DAI_PB09_I_DAI_PB20_O_REG          0x2462     /* SRU_PIN2 */
#define DAI_PB10_I_DAI_PB20_O_REG          0x2462                   
#define DAI_PB11_I_DAI_PB20_O_REG          0x2462                   
#define DAI_PB12_I_DAI_PB20_O_REG          0x2462                   
#define DAI_PB13_I_DAI_PB20_O_REG          0x2463     /* SRU_PIN3 */
#define DAI_PB14_I_DAI_PB20_O_REG          0x2463                   
#define DAI_PB15_I_DAI_PB20_O_REG          0x2463                   
#define DAI_PB16_I_DAI_PB20_O_REG          0x2463                   
#define DAI_PB17_I_DAI_PB20_O_REG          0x2464     /* SRU_PIN4 */
#define DAI_PB18_I_DAI_PB20_O_REG          0x2464                   
#define DAI_PB19_I_DAI_PB20_O_REG          0x2464                   
#define DAI_PB20_I_DAI_PB20_O_REG          0x2464                   

#define DAI_PB01_I_SPORT0_Da_O_REG          0x2460     /* SRU_PIN0 */
#define DAI_PB02_I_SPORT0_Da_O_REG          0x2460                   
#define DAI_PB03_I_SPORT0_Da_O_REG          0x2460                   
#define DAI_PB04_I_SPORT0_Da_O_REG          0x2460                   
#define DAI_PB05_I_SPORT0_Da_O_REG          0x2461     /* SRU_PIN1 */
#define DAI_PB06_I_SPORT0_Da_O_REG          0x2461                   
#define DAI_PB07_I_SPORT0_Da_O_REG          0x2461                   
#define DAI_PB08_I_SPORT0_Da_O_REG          0x2461                   
#define DAI_PB09_I_SPORT0_Da_O_REG          0x2462     /* SRU_PIN2 */
#define DAI_PB10_I_SPORT0_Da_O_REG          0x2462                   
#define DAI_PB11_I_SPORT0_Da_O_REG          0x2462                   
#define DAI_PB12_I_SPORT0_Da_O_REG          0x2462                   
#define DAI_PB13_I_SPORT0_Da_O_REG          0x2463     /* SRU_PIN3 */
#define DAI_PB14_I_SPORT0_Da_O_REG          0x2463                   
#define DAI_PB15_I_SPORT0_Da_O_REG          0x2463                   
#define DAI_PB16_I_SPORT0_Da_O_REG          0x2463                   
#define DAI_PB17_I_SPORT0_Da_O_REG          0x2464     /* SRU_PIN4 */
#define DAI_PB18_I_SPORT0_Da_O_REG          0x2464                   
#define DAI_PB19_I_SPORT0_Da_O_REG          0x2464                   
#define DAI_PB20_I_SPORT0_Da_O_REG          0x2464                   

#define DAI_PB01_I_SPORT0_Db_O_REG          0x2460     /* SRU_PIN0 */
#define DAI_PB02_I_SPORT0_Db_O_REG          0x2460                   
#define DAI_PB03_I_SPORT0_Db_O_REG          0x2460                   
#define DAI_PB04_I_SPORT0_Db_O_REG          0x2460                   
#define DAI_PB05_I_SPORT0_Db_O_REG          0x2461     /* SRU_PIN1 */
#define DAI_PB06_I_SPORT0_Db_O_REG          0x2461                   
#define DAI_PB07_I_SPORT0_Db_O_REG          0x2461                   
#define DAI_PB08_I_SPORT0_Db_O_REG          0x2461                   
#define DAI_PB09_I_SPORT0_Db_O_REG          0x2462     /* SRU_PIN2 */
#define DAI_PB10_I_SPORT0_Db_O_REG          0x2462                   
#define DAI_PB11_I_SPORT0_Db_O_REG          0x2462                   
#define DAI_PB12_I_SPORT0_Db_O_REG          0x2462                   
#define DAI_PB13_I_SPORT0_Db_O_REG          0x2463     /* SRU_PIN3 */
#define DAI_PB14_I_SPORT0_Db_O_REG          0x2463                   
#define DAI_PB15_I_SPORT0_Db_O_REG          0x2463                   
#define DAI_PB16_I_SPORT0_Db_O_REG          0x2463                   
#define DAI_PB17_I_SPORT0_Db_O_REG          0x2464     /* SRU_PIN4 */
#define DAI_PB18_I_SPORT0_Db_O_REG          0x2464                   
#define DAI_PB19_I_SPORT0_Db_O_REG          0x2464                   
#define DAI_PB20_I_SPORT0_Db_O_REG          0x2464                   

#define DAI_PB01_I_SPORT1_Da_O_REG          0x2460     /* SRU_PIN0 */
#define DAI_PB02_I_SPORT1_Da_O_REG          0x2460                   
#define DAI_PB03_I_SPORT1_Da_O_REG          0x2460                   
#define DAI_PB04_I_SPORT1_Da_O_REG          0x2460                   
#define DAI_PB05_I_SPORT1_Da_O_REG          0x2461     /* SRU_PIN1 */
#define DAI_PB06_I_SPORT1_Da_O_REG          0x2461                   
#define DAI_PB07_I_SPORT1_Da_O_REG          0x2461                   
#define DAI_PB08_I_SPORT1_Da_O_REG          0x2461                   
#define DAI_PB09_I_SPORT1_Da_O_REG          0x2462     /* SRU_PIN2 */
#define DAI_PB10_I_SPORT1_Da_O_REG          0x2462                   
#define DAI_PB11_I_SPORT1_Da_O_REG          0x2462                   
#define DAI_PB12_I_SPORT1_Da_O_REG          0x2462                   
#define DAI_PB13_I_SPORT1_Da_O_REG          0x2463     /* SRU_PIN3 */
#define DAI_PB14_I_SPORT1_Da_O_REG          0x2463                   
#define DAI_PB15_I_SPORT1_Da_O_REG          0x2463                   
#define DAI_PB16_I_SPORT1_Da_O_REG          0x2463                   
#define DAI_PB17_I_SPORT1_Da_O_REG          0x2464     /* SRU_PIN4 */
#define DAI_PB18_I_SPORT1_Da_O_REG          0x2464                   
#define DAI_PB19_I_SPORT1_Da_O_REG          0x2464                   
#define DAI_PB20_I_SPORT1_Da_O_REG          0x2464                   

#define DAI_PB01_I_SPORT1_Db_O_REG          0x2460     /* SRU_PIN0 */
#define DAI_PB02_I_SPORT1_Db_O_REG          0x2460                   
#define DAI_PB03_I_SPORT1_Db_O_REG          0x2460                   
#define DAI_PB04_I_SPORT1_Db_O_REG          0x2460                   
#define DAI_PB05_I_SPORT1_Db_O_REG          0x2461     /* SRU_PIN1 */
#define DAI_PB06_I_SPORT1_Db_O_REG          0x2461                   
#define DAI_PB07_I_SPORT1_Db_O_REG          0x2461                   
#define DAI_PB08_I_SPORT1_Db_O_REG          0x2461                   
#define DAI_PB09_I_SPORT1_Db_O_REG          0x2462     /* SRU_PIN2 */
#define DAI_PB10_I_SPORT1_Db_O_REG          0x2462                   
#define DAI_PB11_I_SPORT1_Db_O_REG          0x2462                   
#define DAI_PB12_I_SPORT1_Db_O_REG          0x2462                   
#define DAI_PB13_I_SPORT1_Db_O_REG          0x2463     /* SRU_PIN3 */
#define DAI_PB14_I_SPORT1_Db_O_REG          0x2463                   
#define DAI_PB15_I_SPORT1_Db_O_REG          0x2463                   
#define DAI_PB16_I_SPORT1_Db_O_REG          0x2463                   
#define DAI_PB17_I_SPORT1_Db_O_REG          0x2464     /* SRU_PIN4 */
#define DAI_PB18_I_SPORT1_Db_O_REG          0x2464                   
#define DAI_PB19_I_SPORT1_Db_O_REG          0x2464                   
#define DAI_PB20_I_SPORT1_Db_O_REG          0x2464                   

#define DAI_PB01_I_SPORT2_Da_O_REG          0x2460     /* SRU_PIN0 */
#define DAI_PB02_I_SPORT2_Da_O_REG          0x2460                   
#define DAI_PB03_I_SPORT2_Da_O_REG          0x2460                   
#define DAI_PB04_I_SPORT2_Da_O_REG          0x2460                   
#define DAI_PB05_I_SPORT2_Da_O_REG          0x2461     /* SRU_PIN1 */
#define DAI_PB06_I_SPORT2_Da_O_REG          0x2461                   
#define DAI_PB07_I_SPORT2_Da_O_REG          0x2461                   
#define DAI_PB08_I_SPORT2_Da_O_REG          0x2461                   
#define DAI_PB09_I_SPORT2_Da_O_REG          0x2462     /* SRU_PIN2 */
#define DAI_PB10_I_SPORT2_Da_O_REG          0x2462                   
#define DAI_PB11_I_SPORT2_Da_O_REG          0x2462                   
#define DAI_PB12_I_SPORT2_Da_O_REG          0x2462                   
#define DAI_PB13_I_SPORT2_Da_O_REG          0x2463     /* SRU_PIN3 */
#define DAI_PB14_I_SPORT2_Da_O_REG          0x2463                   
#define DAI_PB15_I_SPORT2_Da_O_REG          0x2463                   
#define DAI_PB16_I_SPORT2_Da_O_REG          0x2463                   
#define DAI_PB17_I_SPORT2_Da_O_REG          0x2464     /* SRU_PIN4 */
#define DAI_PB18_I_SPORT2_Da_O_REG          0x2464                   
#define DAI_PB19_I_SPORT2_Da_O_REG          0x2464                   
#define DAI_PB20_I_SPORT2_Da_O_REG          0x2464                   

#define DAI_PB01_I_SPORT2_Db_O_REG          0x2460     /* SRU_PIN0 */
#define DAI_PB02_I_SPORT2_Db_O_REG          0x2460                   
#define DAI_PB03_I_SPORT2_Db_O_REG          0x2460                   
#define DAI_PB04_I_SPORT2_Db_O_REG          0x2460                   
#define DAI_PB05_I_SPORT2_Db_O_REG          0x2461     /* SRU_PIN1 */
#define DAI_PB06_I_SPORT2_Db_O_REG          0x2461                   
#define DAI_PB07_I_SPORT2_Db_O_REG          0x2461                   
#define DAI_PB08_I_SPORT2_Db_O_REG          0x2461                   
#define DAI_PB09_I_SPORT2_Db_O_REG          0x2462     /* SRU_PIN2 */
#define DAI_PB10_I_SPORT2_Db_O_REG          0x2462                   
#define DAI_PB11_I_SPORT2_Db_O_REG          0x2462                   
#define DAI_PB12_I_SPORT2_Db_O_REG          0x2462                   
#define DAI_PB13_I_SPORT2_Db_O_REG          0x2463     /* SRU_PIN3 */
#define DAI_PB14_I_SPORT2_Db_O_REG          0x2463                   
#define DAI_PB15_I_SPORT2_Db_O_REG          0x2463                   
#define DAI_PB16_I_SPORT2_Db_O_REG          0x2463                   
#define DAI_PB17_I_SPORT2_Db_O_REG          0x2464     /* SRU_PIN4 */
#define DAI_PB18_I_SPORT2_Db_O_REG          0x2464                   
#define DAI_PB19_I_SPORT2_Db_O_REG          0x2464                   
#define DAI_PB20_I_SPORT2_Db_O_REG          0x2464                   

#define DAI_PB01_I_SPORT3_Da_O_REG          0x2460     /* SRU_PIN0 */
#define DAI_PB02_I_SPORT3_Da_O_REG          0x2460                   
#define DAI_PB03_I_SPORT3_Da_O_REG          0x2460                   
#define DAI_PB04_I_SPORT3_Da_O_REG          0x2460                   
#define DAI_PB05_I_SPORT3_Da_O_REG          0x2461     /* SRU_PIN1 */
#define DAI_PB06_I_SPORT3_Da_O_REG          0x2461                   
#define DAI_PB07_I_SPORT3_Da_O_REG          0x2461                   
#define DAI_PB08_I_SPORT3_Da_O_REG          0x2461                   
#define DAI_PB09_I_SPORT3_Da_O_REG          0x2462     /* SRU_PIN2 */
#define DAI_PB10_I_SPORT3_Da_O_REG          0x2462                   
#define DAI_PB11_I_SPORT3_Da_O_REG          0x2462                   
#define DAI_PB12_I_SPORT3_Da_O_REG          0x2462                   
#define DAI_PB13_I_SPORT3_Da_O_REG          0x2463     /* SRU_PIN3 */
#define DAI_PB14_I_SPORT3_Da_O_REG          0x2463                   
#define DAI_PB15_I_SPORT3_Da_O_REG          0x2463                   
#define DAI_PB16_I_SPORT3_Da_O_REG          0x2463                   
#define DAI_PB17_I_SPORT3_Da_O_REG          0x2464     /* SRU_PIN4 */
#define DAI_PB18_I_SPORT3_Da_O_REG          0x2464                   
#define DAI_PB19_I_SPORT3_Da_O_REG          0x2464                   
#define DAI_PB20_I_SPORT3_Da_O_REG          0x2464                   

#define DAI_PB01_I_SPORT3_Db_O_REG          0x2460     /* SRU_PIN0 */
#define DAI_PB02_I_SPORT3_Db_O_REG          0x2460                   
#define DAI_PB03_I_SPORT3_Db_O_REG          0x2460                   
#define DAI_PB04_I_SPORT3_Db_O_REG          0x2460                   
#define DAI_PB05_I_SPORT3_Db_O_REG          0x2461     /* SRU_PIN1 */
#define DAI_PB06_I_SPORT3_Db_O_REG          0x2461                   
#define DAI_PB07_I_SPORT3_Db_O_REG          0x2461                   
#define DAI_PB08_I_SPORT3_Db_O_REG          0x2461                   
#define DAI_PB09_I_SPORT3_Db_O_REG          0x2462     /* SRU_PIN2 */
#define DAI_PB10_I_SPORT3_Db_O_REG          0x2462                   
#define DAI_PB11_I_SPORT3_Db_O_REG          0x2462                   
#define DAI_PB12_I_SPORT3_Db_O_REG          0x2462                   
#define DAI_PB13_I_SPORT3_Db_O_REG          0x2463     /* SRU_PIN3 */
#define DAI_PB14_I_SPORT3_Db_O_REG          0x2463                   
#define DAI_PB15_I_SPORT3_Db_O_REG          0x2463                   
#define DAI_PB16_I_SPORT3_Db_O_REG          0x2463                   
#define DAI_PB17_I_SPORT3_Db_O_REG          0x2464     /* SRU_PIN4 */
#define DAI_PB18_I_SPORT3_Db_O_REG          0x2464                   
#define DAI_PB19_I_SPORT3_Db_O_REG          0x2464                   
#define DAI_PB20_I_SPORT3_Db_O_REG          0x2464                   

#define DAI_PB01_I_SPORT4_Da_O_REG          0x2460     /* SRU_PIN0 */
#define DAI_PB02_I_SPORT4_Da_O_REG          0x2460                   
#define DAI_PB03_I_SPORT4_Da_O_REG          0x2460                   
#define DAI_PB04_I_SPORT4_Da_O_REG          0x2460                   
#define DAI_PB05_I_SPORT4_Da_O_REG          0x2461     /* SRU_PIN1 */
#define DAI_PB06_I_SPORT4_Da_O_REG          0x2461                   
#define DAI_PB07_I_SPORT4_Da_O_REG          0x2461                   
#define DAI_PB08_I_SPORT4_Da_O_REG          0x2461                   
#define DAI_PB09_I_SPORT4_Da_O_REG          0x2462     /* SRU_PIN2 */
#define DAI_PB10_I_SPORT4_Da_O_REG          0x2462                   
#define DAI_PB11_I_SPORT4_Da_O_REG          0x2462                   
#define DAI_PB12_I_SPORT4_Da_O_REG          0x2462                   
#define DAI_PB13_I_SPORT4_Da_O_REG          0x2463     /* SRU_PIN3 */
#define DAI_PB14_I_SPORT4_Da_O_REG          0x2463                   
#define DAI_PB15_I_SPORT4_Da_O_REG          0x2463                   
#define DAI_PB16_I_SPORT4_Da_O_REG          0x2463                   
#define DAI_PB17_I_SPORT4_Da_O_REG          0x2464     /* SRU_PIN4 */
#define DAI_PB18_I_SPORT4_Da_O_REG          0x2464                   
#define DAI_PB19_I_SPORT4_Da_O_REG          0x2464                   
#define DAI_PB20_I_SPORT4_Da_O_REG          0x2464                   

#define DAI_PB01_I_SPORT4_Db_O_REG          0x2460     /* SRU_PIN0 */
#define DAI_PB02_I_SPORT4_Db_O_REG          0x2460                   
#define DAI_PB03_I_SPORT4_Db_O_REG          0x2460                   
#define DAI_PB04_I_SPORT4_Db_O_REG          0x2460                   
#define DAI_PB05_I_SPORT4_Db_O_REG          0x2461     /* SRU_PIN1 */
#define DAI_PB06_I_SPORT4_Db_O_REG          0x2461                   
#define DAI_PB07_I_SPORT4_Db_O_REG          0x2461                   
#define DAI_PB08_I_SPORT4_Db_O_REG          0x2461                   
#define DAI_PB09_I_SPORT4_Db_O_REG          0x2462     /* SRU_PIN2 */
#define DAI_PB10_I_SPORT4_Db_O_REG          0x2462                   
#define DAI_PB11_I_SPORT4_Db_O_REG          0x2462                   
#define DAI_PB12_I_SPORT4_Db_O_REG          0x2462                   
#define DAI_PB13_I_SPORT4_Db_O_REG          0x2463     /* SRU_PIN3 */
#define DAI_PB14_I_SPORT4_Db_O_REG          0x2463                   
#define DAI_PB15_I_SPORT4_Db_O_REG          0x2463                   
#define DAI_PB16_I_SPORT4_Db_O_REG          0x2463                   
#define DAI_PB17_I_SPORT4_Db_O_REG          0x2464     /* SRU_PIN4 */
#define DAI_PB18_I_SPORT4_Db_O_REG          0x2464                   
#define DAI_PB19_I_SPORT4_Db_O_REG          0x2464                   
#define DAI_PB20_I_SPORT4_Db_O_REG          0x2464                   

#define DAI_PB01_I_SPORT5_Da_O_REG          0x2460     /* SRU_PIN0 */
#define DAI_PB02_I_SPORT5_Da_O_REG          0x2460                   
#define DAI_PB03_I_SPORT5_Da_O_REG          0x2460                   
#define DAI_PB04_I_SPORT5_Da_O_REG          0x2460                   
#define DAI_PB05_I_SPORT5_Da_O_REG          0x2461     /* SRU_PIN1 */
#define DAI_PB06_I_SPORT5_Da_O_REG          0x2461                   
#define DAI_PB07_I_SPORT5_Da_O_REG          0x2461                   
#define DAI_PB08_I_SPORT5_Da_O_REG          0x2461                   
#define DAI_PB09_I_SPORT5_Da_O_REG          0x2462     /* SRU_PIN2 */
#define DAI_PB10_I_SPORT5_Da_O_REG          0x2462                   
#define DAI_PB11_I_SPORT5_Da_O_REG          0x2462                   
#define DAI_PB12_I_SPORT5_Da_O_REG          0x2462                   
#define DAI_PB13_I_SPORT5_Da_O_REG          0x2463     /* SRU_PIN3 */
#define DAI_PB14_I_SPORT5_Da_O_REG          0x2463                   
#define DAI_PB15_I_SPORT5_Da_O_REG          0x2463                   
#define DAI_PB16_I_SPORT5_Da_O_REG          0x2463                   
#define DAI_PB17_I_SPORT5_Da_O_REG          0x2464     /* SRU_PIN4 */
#define DAI_PB18_I_SPORT5_Da_O_REG          0x2464                   
#define DAI_PB19_I_SPORT5_Da_O_REG          0x2464                   
#define DAI_PB20_I_SPORT5_Da_O_REG          0x2464                   

#define DAI_PB01_I_SPORT5_Db_O_REG          0x2460     /* SRU_PIN0 */
#define DAI_PB02_I_SPORT5_Db_O_REG          0x2460                   
#define DAI_PB03_I_SPORT5_Db_O_REG          0x2460                   
#define DAI_PB04_I_SPORT5_Db_O_REG          0x2460                   
#define DAI_PB05_I_SPORT5_Db_O_REG          0x2461     /* SRU_PIN1 */
#define DAI_PB06_I_SPORT5_Db_O_REG          0x2461                   
#define DAI_PB07_I_SPORT5_Db_O_REG          0x2461                   
#define DAI_PB08_I_SPORT5_Db_O_REG          0x2461                   
#define DAI_PB09_I_SPORT5_Db_O_REG          0x2462     /* SRU_PIN2 */
#define DAI_PB10_I_SPORT5_Db_O_REG          0x2462                   
#define DAI_PB11_I_SPORT5_Db_O_REG          0x2462                   
#define DAI_PB12_I_SPORT5_Db_O_REG          0x2462                   
#define DAI_PB13_I_SPORT5_Db_O_REG          0x2463     /* SRU_PIN3 */
#define DAI_PB14_I_SPORT5_Db_O_REG          0x2463                   
#define DAI_PB15_I_SPORT5_Db_O_REG          0x2463                   
#define DAI_PB16_I_SPORT5_Db_O_REG          0x2463                   
#define DAI_PB17_I_SPORT5_Db_O_REG          0x2464     /* SRU_PIN4 */
#define DAI_PB18_I_SPORT5_Db_O_REG          0x2464                   
#define DAI_PB19_I_SPORT5_Db_O_REG          0x2464                   
#define DAI_PB20_I_SPORT5_Db_O_REG          0x2464                   

#define DAI_PB01_I_SPORT0_CLK_O_REG          0x2460     /* SRU_PIN0 */
#define DAI_PB02_I_SPORT0_CLK_O_REG          0x2460                   
#define DAI_PB03_I_SPORT0_CLK_O_REG          0x2460                   
#define DAI_PB04_I_SPORT0_CLK_O_REG          0x2460                   
#define DAI_PB05_I_SPORT0_CLK_O_REG          0x2461     /* SRU_PIN1 */
#define DAI_PB06_I_SPORT0_CLK_O_REG          0x2461                   
#define DAI_PB07_I_SPORT0_CLK_O_REG          0x2461                   
#define DAI_PB08_I_SPORT0_CLK_O_REG          0x2461                   
#define DAI_PB09_I_SPORT0_CLK_O_REG          0x2462     /* SRU_PIN2 */
#define DAI_PB10_I_SPORT0_CLK_O_REG          0x2462                   
#define DAI_PB11_I_SPORT0_CLK_O_REG          0x2462                   
#define DAI_PB12_I_SPORT0_CLK_O_REG          0x2462                   
#define DAI_PB13_I_SPORT0_CLK_O_REG          0x2463     /* SRU_PIN3 */
#define DAI_PB14_I_SPORT0_CLK_O_REG          0x2463                   
#define DAI_PB15_I_SPORT0_CLK_O_REG          0x2463                   
#define DAI_PB16_I_SPORT0_CLK_O_REG          0x2463                   
#define DAI_PB17_I_SPORT0_CLK_O_REG          0x2464     /* SRU_PIN4 */
#define DAI_PB18_I_SPORT0_CLK_O_REG          0x2464                   
#define DAI_PB19_I_SPORT0_CLK_O_REG          0x2464                   
#define DAI_PB20_I_SPORT0_CLK_O_REG          0x2464                   

#define DAI_PB01_I_SPORT1_CLK_O_REG          0x2460     /* SRU_PIN0 */
#define DAI_PB02_I_SPORT1_CLK_O_REG          0x2460                   
#define DAI_PB03_I_SPORT1_CLK_O_REG          0x2460                   
#define DAI_PB04_I_SPORT1_CLK_O_REG          0x2460                   
#define DAI_PB05_I_SPORT1_CLK_O_REG          0x2461     /* SRU_PIN1 */
#define DAI_PB06_I_SPORT1_CLK_O_REG          0x2461                   
#define DAI_PB07_I_SPORT1_CLK_O_REG          0x2461                   
#define DAI_PB08_I_SPORT1_CLK_O_REG          0x2461                   
#define DAI_PB09_I_SPORT1_CLK_O_REG          0x2462     /* SRU_PIN2 */
#define DAI_PB10_I_SPORT1_CLK_O_REG          0x2462                   
#define DAI_PB11_I_SPORT1_CLK_O_REG          0x2462                   
#define DAI_PB12_I_SPORT1_CLK_O_REG          0x2462                   
#define DAI_PB13_I_SPORT1_CLK_O_REG          0x2463     /* SRU_PIN3 */
#define DAI_PB14_I_SPORT1_CLK_O_REG          0x2463                   
#define DAI_PB15_I_SPORT1_CLK_O_REG          0x2463                   
#define DAI_PB16_I_SPORT1_CLK_O_REG          0x2463                   
#define DAI_PB17_I_SPORT1_CLK_O_REG          0x2464     /* SRU_PIN4 */
#define DAI_PB18_I_SPORT1_CLK_O_REG          0x2464                   
#define DAI_PB19_I_SPORT1_CLK_O_REG          0x2464                   
#define DAI_PB20_I_SPORT1_CLK_O_REG          0x2464                   

#define DAI_PB01_I_SPORT2_CLK_O_REG          0x2460     /* SRU_PIN0 */
#define DAI_PB02_I_SPORT2_CLK_O_REG          0x2460                   
#define DAI_PB03_I_SPORT2_CLK_O_REG          0x2460                   
#define DAI_PB04_I_SPORT2_CLK_O_REG          0x2460                   
#define DAI_PB05_I_SPORT2_CLK_O_REG          0x2461     /* SRU_PIN1 */
#define DAI_PB06_I_SPORT2_CLK_O_REG          0x2461                   
#define DAI_PB07_I_SPORT2_CLK_O_REG          0x2461                   
#define DAI_PB08_I_SPORT2_CLK_O_REG          0x2461                   
#define DAI_PB09_I_SPORT2_CLK_O_REG          0x2462     /* SRU_PIN2 */
#define DAI_PB10_I_SPORT2_CLK_O_REG          0x2462                   
#define DAI_PB11_I_SPORT2_CLK_O_REG          0x2462                   
#define DAI_PB12_I_SPORT2_CLK_O_REG          0x2462                   
#define DAI_PB13_I_SPORT2_CLK_O_REG          0x2463     /* SRU_PIN3 */
#define DAI_PB14_I_SPORT2_CLK_O_REG          0x2463                   
#define DAI_PB15_I_SPORT2_CLK_O_REG          0x2463                   
#define DAI_PB16_I_SPORT2_CLK_O_REG          0x2463                   
#define DAI_PB17_I_SPORT2_CLK_O_REG          0x2464     /* SRU_PIN4 */
#define DAI_PB18_I_SPORT2_CLK_O_REG          0x2464                   
#define DAI_PB19_I_SPORT2_CLK_O_REG          0x2464                   
#define DAI_PB20_I_SPORT2_CLK_O_REG          0x2464                   

#define DAI_PB01_I_SPORT3_CLK_O_REG          0x2460     /* SRU_PIN0 */
#define DAI_PB02_I_SPORT3_CLK_O_REG          0x2460                   
#define DAI_PB03_I_SPORT3_CLK_O_REG          0x2460                   
#define DAI_PB04_I_SPORT3_CLK_O_REG          0x2460                   
#define DAI_PB05_I_SPORT3_CLK_O_REG          0x2461     /* SRU_PIN1 */
#define DAI_PB06_I_SPORT3_CLK_O_REG          0x2461                   
#define DAI_PB07_I_SPORT3_CLK_O_REG          0x2461                   
#define DAI_PB08_I_SPORT3_CLK_O_REG          0x2461                   
#define DAI_PB09_I_SPORT3_CLK_O_REG          0x2462     /* SRU_PIN2 */
#define DAI_PB10_I_SPORT3_CLK_O_REG          0x2462                   
#define DAI_PB11_I_SPORT3_CLK_O_REG          0x2462                   
#define DAI_PB12_I_SPORT3_CLK_O_REG          0x2462                   
#define DAI_PB13_I_SPORT3_CLK_O_REG          0x2463     /* SRU_PIN3 */
#define DAI_PB14_I_SPORT3_CLK_O_REG          0x2463                   
#define DAI_PB15_I_SPORT3_CLK_O_REG          0x2463                   
#define DAI_PB16_I_SPORT3_CLK_O_REG          0x2463                   
#define DAI_PB17_I_SPORT3_CLK_O_REG          0x2464     /* SRU_PIN4 */
#define DAI_PB18_I_SPORT3_CLK_O_REG          0x2464                   
#define DAI_PB19_I_SPORT3_CLK_O_REG          0x2464                   
#define DAI_PB20_I_SPORT3_CLK_O_REG          0x2464                   

#define DAI_PB01_I_SPORT4_CLK_O_REG          0x2460     /* SRU_PIN0 */
#define DAI_PB02_I_SPORT4_CLK_O_REG          0x2460                   
#define DAI_PB03_I_SPORT4_CLK_O_REG          0x2460                   
#define DAI_PB04_I_SPORT4_CLK_O_REG          0x2460                   
#define DAI_PB05_I_SPORT4_CLK_O_REG          0x2461     /* SRU_PIN1 */
#define DAI_PB06_I_SPORT4_CLK_O_REG          0x2461                   
#define DAI_PB07_I_SPORT4_CLK_O_REG          0x2461                   
#define DAI_PB08_I_SPORT4_CLK_O_REG          0x2461                   
#define DAI_PB09_I_SPORT4_CLK_O_REG          0x2462     /* SRU_PIN2 */
#define DAI_PB10_I_SPORT4_CLK_O_REG          0x2462                   
#define DAI_PB11_I_SPORT4_CLK_O_REG          0x2462                   
#define DAI_PB12_I_SPORT4_CLK_O_REG          0x2462                   
#define DAI_PB13_I_SPORT4_CLK_O_REG          0x2463     /* SRU_PIN3 */
#define DAI_PB14_I_SPORT4_CLK_O_REG          0x2463                   
#define DAI_PB15_I_SPORT4_CLK_O_REG          0x2463                   
#define DAI_PB16_I_SPORT4_CLK_O_REG          0x2463                   
#define DAI_PB17_I_SPORT4_CLK_O_REG          0x2464     /* SRU_PIN4 */
#define DAI_PB18_I_SPORT4_CLK_O_REG          0x2464                   
#define DAI_PB19_I_SPORT4_CLK_O_REG          0x2464                   
#define DAI_PB20_I_SPORT4_CLK_O_REG          0x2464                   

#define DAI_PB01_I_SPORT5_CLK_O_REG          0x2460     /* SRU_PIN0 */
#define DAI_PB02_I_SPORT5_CLK_O_REG          0x2460                   
#define DAI_PB03_I_SPORT5_CLK_O_REG          0x2460                   
#define DAI_PB04_I_SPORT5_CLK_O_REG          0x2460                   
#define DAI_PB05_I_SPORT5_CLK_O_REG          0x2461     /* SRU_PIN1 */
#define DAI_PB06_I_SPORT5_CLK_O_REG          0x2461                   
#define DAI_PB07_I_SPORT5_CLK_O_REG          0x2461                   
#define DAI_PB08_I_SPORT5_CLK_O_REG          0x2461                   
#define DAI_PB09_I_SPORT5_CLK_O_REG          0x2462     /* SRU_PIN2 */
#define DAI_PB10_I_SPORT5_CLK_O_REG          0x2462                   
#define DAI_PB11_I_SPORT5_CLK_O_REG          0x2462                   
#define DAI_PB12_I_SPORT5_CLK_O_REG          0x2462                   
#define DAI_PB13_I_SPORT5_CLK_O_REG          0x2463     /* SRU_PIN3 */
#define DAI_PB14_I_SPORT5_CLK_O_REG          0x2463                   
#define DAI_PB15_I_SPORT5_CLK_O_REG          0x2463                   
#define DAI_PB16_I_SPORT5_CLK_O_REG          0x2463                   
#define DAI_PB17_I_SPORT5_CLK_O_REG          0x2464     /* SRU_PIN4 */
#define DAI_PB18_I_SPORT5_CLK_O_REG          0x2464                   
#define DAI_PB19_I_SPORT5_CLK_O_REG          0x2464                   
#define DAI_PB20_I_SPORT5_CLK_O_REG          0x2464                   

#define DAI_PB01_I_SPORT0_FS_O_REG          0x2460     /* SRU_PIN0 */
#define DAI_PB02_I_SPORT0_FS_O_REG          0x2460                   
#define DAI_PB03_I_SPORT0_FS_O_REG          0x2460                   
#define DAI_PB04_I_SPORT0_FS_O_REG          0x2460                   
#define DAI_PB05_I_SPORT0_FS_O_REG          0x2461     /* SRU_PIN1 */
#define DAI_PB06_I_SPORT0_FS_O_REG          0x2461                   
#define DAI_PB07_I_SPORT0_FS_O_REG          0x2461                   
#define DAI_PB08_I_SPORT0_FS_O_REG          0x2461                   
#define DAI_PB09_I_SPORT0_FS_O_REG          0x2462     /* SRU_PIN2 */
#define DAI_PB10_I_SPORT0_FS_O_REG          0x2462                   
#define DAI_PB11_I_SPORT0_FS_O_REG          0x2462                   
#define DAI_PB12_I_SPORT0_FS_O_REG          0x2462                   
#define DAI_PB13_I_SPORT0_FS_O_REG          0x2463     /* SRU_PIN3 */
#define DAI_PB14_I_SPORT0_FS_O_REG          0x2463                   
#define DAI_PB15_I_SPORT0_FS_O_REG          0x2463                   
#define DAI_PB16_I_SPORT0_FS_O_REG          0x2463                   
#define DAI_PB17_I_SPORT0_FS_O_REG          0x2464     /* SRU_PIN4 */
#define DAI_PB18_I_SPORT0_FS_O_REG          0x2464                   
#define DAI_PB19_I_SPORT0_FS_O_REG          0x2464                   
#define DAI_PB20_I_SPORT0_FS_O_REG          0x2464                   

#define DAI_PB01_I_SPORT1_FS_O_REG          0x2460     /* SRU_PIN0 */
#define DAI_PB02_I_SPORT1_FS_O_REG          0x2460                   
#define DAI_PB03_I_SPORT1_FS_O_REG          0x2460                   
#define DAI_PB04_I_SPORT1_FS_O_REG          0x2460                   
#define DAI_PB05_I_SPORT1_FS_O_REG          0x2461     /* SRU_PIN1 */
#define DAI_PB06_I_SPORT1_FS_O_REG          0x2461                   
#define DAI_PB07_I_SPORT1_FS_O_REG          0x2461                   
#define DAI_PB08_I_SPORT1_FS_O_REG          0x2461                   
#define DAI_PB09_I_SPORT1_FS_O_REG          0x2462     /* SRU_PIN2 */
#define DAI_PB10_I_SPORT1_FS_O_REG          0x2462                   
#define DAI_PB11_I_SPORT1_FS_O_REG          0x2462                   
#define DAI_PB12_I_SPORT1_FS_O_REG          0x2462                   
#define DAI_PB13_I_SPORT1_FS_O_REG          0x2463     /* SRU_PIN3 */
#define DAI_PB14_I_SPORT1_FS_O_REG          0x2463                   
#define DAI_PB15_I_SPORT1_FS_O_REG          0x2463                   
#define DAI_PB16_I_SPORT1_FS_O_REG          0x2463                   
#define DAI_PB17_I_SPORT1_FS_O_REG          0x2464     /* SRU_PIN4 */
#define DAI_PB18_I_SPORT1_FS_O_REG          0x2464                   
#define DAI_PB19_I_SPORT1_FS_O_REG          0x2464                   
#define DAI_PB20_I_SPORT1_FS_O_REG          0x2464                   

#define DAI_PB01_I_SPORT2_FS_O_REG          0x2460     /* SRU_PIN0 */
#define DAI_PB02_I_SPORT2_FS_O_REG          0x2460                   
#define DAI_PB03_I_SPORT2_FS_O_REG          0x2460                   
#define DAI_PB04_I_SPORT2_FS_O_REG          0x2460                   
#define DAI_PB05_I_SPORT2_FS_O_REG          0x2461     /* SRU_PIN1 */
#define DAI_PB06_I_SPORT2_FS_O_REG          0x2461                   
#define DAI_PB07_I_SPORT2_FS_O_REG          0x2461                   
#define DAI_PB08_I_SPORT2_FS_O_REG          0x2461                   
#define DAI_PB09_I_SPORT2_FS_O_REG          0x2462     /* SRU_PIN2 */
#define DAI_PB10_I_SPORT2_FS_O_REG          0x2462                   
#define DAI_PB11_I_SPORT2_FS_O_REG          0x2462                   
#define DAI_PB12_I_SPORT2_FS_O_REG          0x2462                   
#define DAI_PB13_I_SPORT2_FS_O_REG          0x2463     /* SRU_PIN3 */
#define DAI_PB14_I_SPORT2_FS_O_REG          0x2463                   
#define DAI_PB15_I_SPORT2_FS_O_REG          0x2463                   
#define DAI_PB16_I_SPORT2_FS_O_REG          0x2463                   
#define DAI_PB17_I_SPORT2_FS_O_REG          0x2464     /* SRU_PIN4 */
#define DAI_PB18_I_SPORT2_FS_O_REG          0x2464                   
#define DAI_PB19_I_SPORT2_FS_O_REG          0x2464                   
#define DAI_PB20_I_SPORT2_FS_O_REG          0x2464                   

#define DAI_PB01_I_SPORT3_FS_O_REG          0x2460     /* SRU_PIN0 */
#define DAI_PB02_I_SPORT3_FS_O_REG          0x2460                   
#define DAI_PB03_I_SPORT3_FS_O_REG          0x2460                   
#define DAI_PB04_I_SPORT3_FS_O_REG          0x2460                   
#define DAI_PB05_I_SPORT3_FS_O_REG          0x2461     /* SRU_PIN1 */
#define DAI_PB06_I_SPORT3_FS_O_REG          0x2461                   
#define DAI_PB07_I_SPORT3_FS_O_REG          0x2461                   
#define DAI_PB08_I_SPORT3_FS_O_REG          0x2461                   
#define DAI_PB09_I_SPORT3_FS_O_REG          0x2462     /* SRU_PIN2 */
#define DAI_PB10_I_SPORT3_FS_O_REG          0x2462                   
#define DAI_PB11_I_SPORT3_FS_O_REG          0x2462                   
#define DAI_PB12_I_SPORT3_FS_O_REG          0x2462                   
#define DAI_PB13_I_SPORT3_FS_O_REG          0x2463     /* SRU_PIN3 */
#define DAI_PB14_I_SPORT3_FS_O_REG          0x2463                   
#define DAI_PB15_I_SPORT3_FS_O_REG          0x2463                   
#define DAI_PB16_I_SPORT3_FS_O_REG          0x2463                   
#define DAI_PB17_I_SPORT3_FS_O_REG          0x2464     /* SRU_PIN4 */
#define DAI_PB18_I_SPORT3_FS_O_REG          0x2464                   
#define DAI_PB19_I_SPORT3_FS_O_REG          0x2464                   
#define DAI_PB20_I_SPORT3_FS_O_REG          0x2464                   

#define DAI_PB01_I_SPORT4_FS_O_REG          0x2460     /* SRU_PIN0 */
#define DAI_PB02_I_SPORT4_FS_O_REG          0x2460                   
#define DAI_PB03_I_SPORT4_FS_O_REG          0x2460                   
#define DAI_PB04_I_SPORT4_FS_O_REG          0x2460                   
#define DAI_PB05_I_SPORT4_FS_O_REG          0x2461     /* SRU_PIN1 */
#define DAI_PB06_I_SPORT4_FS_O_REG          0x2461                   
#define DAI_PB07_I_SPORT4_FS_O_REG          0x2461                   
#define DAI_PB08_I_SPORT4_FS_O_REG          0x2461                   
#define DAI_PB09_I_SPORT4_FS_O_REG          0x2462     /* SRU_PIN2 */
#define DAI_PB10_I_SPORT4_FS_O_REG          0x2462                   
#define DAI_PB11_I_SPORT4_FS_O_REG          0x2462                   
#define DAI_PB12_I_SPORT4_FS_O_REG          0x2462                   
#define DAI_PB13_I_SPORT4_FS_O_REG          0x2463     /* SRU_PIN3 */
#define DAI_PB14_I_SPORT4_FS_O_REG          0x2463                   
#define DAI_PB15_I_SPORT4_FS_O_REG          0x2463                   
#define DAI_PB16_I_SPORT4_FS_O_REG          0x2463                   
#define DAI_PB17_I_SPORT4_FS_O_REG          0x2464     /* SRU_PIN4 */
#define DAI_PB18_I_SPORT4_FS_O_REG          0x2464                   
#define DAI_PB19_I_SPORT4_FS_O_REG          0x2464                   
#define DAI_PB20_I_SPORT4_FS_O_REG          0x2464                   

#define DAI_PB01_I_SPORT5_FS_O_REG          0x2460     /* SRU_PIN0 */
#define DAI_PB02_I_SPORT5_FS_O_REG          0x2460                   
#define DAI_PB03_I_SPORT5_FS_O_REG          0x2460                   
#define DAI_PB04_I_SPORT5_FS_O_REG          0x2460                   
#define DAI_PB05_I_SPORT5_FS_O_REG          0x2461     /* SRU_PIN1 */
#define DAI_PB06_I_SPORT5_FS_O_REG          0x2461                   
#define DAI_PB07_I_SPORT5_FS_O_REG          0x2461                   
#define DAI_PB08_I_SPORT5_FS_O_REG          0x2461                   
#define DAI_PB09_I_SPORT5_FS_O_REG          0x2462     /* SRU_PIN2 */
#define DAI_PB10_I_SPORT5_FS_O_REG          0x2462                   
#define DAI_PB11_I_SPORT5_FS_O_REG          0x2462                   
#define DAI_PB12_I_SPORT5_FS_O_REG          0x2462                   
#define DAI_PB13_I_SPORT5_FS_O_REG          0x2463     /* SRU_PIN3 */
#define DAI_PB14_I_SPORT5_FS_O_REG          0x2463                   
#define DAI_PB15_I_SPORT5_FS_O_REG          0x2463                   
#define DAI_PB16_I_SPORT5_FS_O_REG          0x2463                   
#define DAI_PB17_I_SPORT5_FS_O_REG          0x2464     /* SRU_PIN4 */
#define DAI_PB18_I_SPORT5_FS_O_REG          0x2464                   
#define DAI_PB19_I_SPORT5_FS_O_REG          0x2464                   
#define DAI_PB20_I_SPORT5_FS_O_REG          0x2464                   

#define DAI_PB01_I_TIMER0_O_REG          0x2460     /* SRU_PIN0 */
#define DAI_PB02_I_TIMER0_O_REG          0x2460                   
#define DAI_PB03_I_TIMER0_O_REG          0x2460                   
#define DAI_PB04_I_TIMER0_O_REG          0x2460                   
#define DAI_PB05_I_TIMER0_O_REG          0x2461     /* SRU_PIN1 */
#define DAI_PB06_I_TIMER0_O_REG          0x2461                   
#define DAI_PB07_I_TIMER0_O_REG          0x2461                   
#define DAI_PB08_I_TIMER0_O_REG          0x2461                   
#define DAI_PB09_I_TIMER0_O_REG          0x2462     /* SRU_PIN2 */
#define DAI_PB10_I_TIMER0_O_REG          0x2462                   
#define DAI_PB11_I_TIMER0_O_REG          0x2462                   
#define DAI_PB12_I_TIMER0_O_REG          0x2462                   
#define DAI_PB13_I_TIMER0_O_REG          0x2463     /* SRU_PIN3 */
#define DAI_PB14_I_TIMER0_O_REG          0x2463                   
#define DAI_PB15_I_TIMER0_O_REG          0x2463                   
#define DAI_PB16_I_TIMER0_O_REG          0x2463                   
#define DAI_PB17_I_TIMER0_O_REG          0x2464     /* SRU_PIN4 */
#define DAI_PB18_I_TIMER0_O_REG          0x2464                   
#define DAI_PB19_I_TIMER0_O_REG          0x2464                   
#define DAI_PB20_I_TIMER0_O_REG          0x2464                   

#define DAI_PB01_I_TIMER1_O_REG          0x2460     /* SRU_PIN0 */
#define DAI_PB02_I_TIMER1_O_REG          0x2460                   
#define DAI_PB03_I_TIMER1_O_REG          0x2460                   
#define DAI_PB04_I_TIMER1_O_REG          0x2460                   
#define DAI_PB05_I_TIMER1_O_REG          0x2461     /* SRU_PIN1 */
#define DAI_PB06_I_TIMER1_O_REG          0x2461                   
#define DAI_PB07_I_TIMER1_O_REG          0x2461                   
#define DAI_PB08_I_TIMER1_O_REG          0x2461                   
#define DAI_PB09_I_TIMER1_O_REG          0x2462     /* SRU_PIN2 */
#define DAI_PB10_I_TIMER1_O_REG          0x2462                   
#define DAI_PB11_I_TIMER1_O_REG          0x2462                   
#define DAI_PB12_I_TIMER1_O_REG          0x2462                   
#define DAI_PB13_I_TIMER1_O_REG          0x2463     /* SRU_PIN3 */
#define DAI_PB14_I_TIMER1_O_REG          0x2463                   
#define DAI_PB15_I_TIMER1_O_REG          0x2463                   
#define DAI_PB16_I_TIMER1_O_REG          0x2463                   
#define DAI_PB17_I_TIMER1_O_REG          0x2464     /* SRU_PIN4 */
#define DAI_PB18_I_TIMER1_O_REG          0x2464                   
#define DAI_PB19_I_TIMER1_O_REG          0x2464                   
#define DAI_PB20_I_TIMER1_O_REG          0x2464                   

#define DAI_PB01_I_TIMER2_O_REG          0x2460     /* SRU_PIN0 */
#define DAI_PB02_I_TIMER2_O_REG          0x2460                   
#define DAI_PB03_I_TIMER2_O_REG          0x2460                   
#define DAI_PB04_I_TIMER2_O_REG          0x2460                   
#define DAI_PB05_I_TIMER2_O_REG          0x2461     /* SRU_PIN1 */
#define DAI_PB06_I_TIMER2_O_REG          0x2461                   
#define DAI_PB07_I_TIMER2_O_REG          0x2461                   
#define DAI_PB08_I_TIMER2_O_REG          0x2461                   
#define DAI_PB09_I_TIMER2_O_REG          0x2462     /* SRU_PIN2 */
#define DAI_PB10_I_TIMER2_O_REG          0x2462                   
#define DAI_PB11_I_TIMER2_O_REG          0x2462                   
#define DAI_PB12_I_TIMER2_O_REG          0x2462                   
#define DAI_PB13_I_TIMER2_O_REG          0x2463     /* SRU_PIN3 */
#define DAI_PB14_I_TIMER2_O_REG          0x2463                   
#define DAI_PB15_I_TIMER2_O_REG          0x2463                   
#define DAI_PB16_I_TIMER2_O_REG          0x2463                   
#define DAI_PB17_I_TIMER2_O_REG          0x2464     /* SRU_PIN4 */
#define DAI_PB18_I_TIMER2_O_REG          0x2464                   
#define DAI_PB19_I_TIMER2_O_REG          0x2464                   
#define DAI_PB20_I_TIMER2_O_REG          0x2464                   

#define DAI_PB01_I_FLAG10_O_REG          0x2460     /* SRU_PIN0 */
#define DAI_PB02_I_FLAG10_O_REG          0x2460                   
#define DAI_PB03_I_FLAG10_O_REG          0x2460                   
#define DAI_PB04_I_FLAG10_O_REG          0x2460                   
#define DAI_PB05_I_FLAG10_O_REG          0x2461     /* SRU_PIN1 */
#define DAI_PB06_I_FLAG10_O_REG          0x2461                   
#define DAI_PB07_I_FLAG10_O_REG          0x2461                   
#define DAI_PB08_I_FLAG10_O_REG          0x2461                   
#define DAI_PB09_I_FLAG10_O_REG          0x2462     /* SRU_PIN2 */
#define DAI_PB10_I_FLAG10_O_REG          0x2462                   
#define DAI_PB11_I_FLAG10_O_REG          0x2462                   
#define DAI_PB12_I_FLAG10_O_REG          0x2462                   
#define DAI_PB13_I_FLAG10_O_REG          0x2463     /* SRU_PIN3 */
#define DAI_PB14_I_FLAG10_O_REG          0x2463                   
#define DAI_PB15_I_FLAG10_O_REG          0x2463                   
#define DAI_PB16_I_FLAG10_O_REG          0x2463                   
#define DAI_PB17_I_FLAG10_O_REG          0x2464     /* SRU_PIN4 */
#define DAI_PB18_I_FLAG10_O_REG          0x2464                   
#define DAI_PB19_I_FLAG10_O_REG          0x2464                   
#define DAI_PB20_I_FLAG10_O_REG          0x2464                   

#define DAI_PB01_I_PDAP_STB_O_REG          0x2460     /* SRU_PIN0 */
#define DAI_PB02_I_PDAP_STB_O_REG          0x2460                   
#define DAI_PB03_I_PDAP_STB_O_REG          0x2460                   
#define DAI_PB04_I_PDAP_STB_O_REG          0x2460                   
#define DAI_PB05_I_PDAP_STB_O_REG          0x2461     /* SRU_PIN1 */
#define DAI_PB06_I_PDAP_STB_O_REG          0x2461                   
#define DAI_PB07_I_PDAP_STB_O_REG          0x2461                   
#define DAI_PB08_I_PDAP_STB_O_REG          0x2461                   
#define DAI_PB09_I_PDAP_STB_O_REG          0x2462     /* SRU_PIN2 */
#define DAI_PB10_I_PDAP_STB_O_REG          0x2462                   
#define DAI_PB11_I_PDAP_STB_O_REG          0x2462                   
#define DAI_PB12_I_PDAP_STB_O_REG          0x2462                   
#define DAI_PB13_I_PDAP_STB_O_REG          0x2463     /* SRU_PIN3 */
#define DAI_PB14_I_PDAP_STB_O_REG          0x2463                   
#define DAI_PB15_I_PDAP_STB_O_REG          0x2463                   
#define DAI_PB16_I_PDAP_STB_O_REG          0x2463                   
#define DAI_PB17_I_PDAP_STB_O_REG          0x2464     /* SRU_PIN4 */
#define DAI_PB18_I_PDAP_STB_O_REG          0x2464                   
#define DAI_PB19_I_PDAP_STB_O_REG          0x2464                   
#define DAI_PB20_I_PDAP_STB_O_REG          0x2464                   

#define DAI_PB01_I_FLAG11_O_REG          0x2460     /* SRU_PIN0 */
#define DAI_PB02_I_FLAG11_O_REG          0x2460                   
#define DAI_PB03_I_FLAG11_O_REG          0x2460                   
#define DAI_PB04_I_FLAG11_O_REG          0x2460                   
#define DAI_PB05_I_FLAG11_O_REG          0x2461     /* SRU_PIN1 */
#define DAI_PB06_I_FLAG11_O_REG          0x2461                   
#define DAI_PB07_I_FLAG11_O_REG          0x2461                   
#define DAI_PB08_I_FLAG11_O_REG          0x2461                   
#define DAI_PB09_I_FLAG11_O_REG          0x2462     /* SRU_PIN2 */
#define DAI_PB10_I_FLAG11_O_REG          0x2462                   
#define DAI_PB11_I_FLAG11_O_REG          0x2462                   
#define DAI_PB12_I_FLAG11_O_REG          0x2462                   
#define DAI_PB13_I_FLAG11_O_REG          0x2463     /* SRU_PIN3 */
#define DAI_PB14_I_FLAG11_O_REG          0x2463                   
#define DAI_PB15_I_FLAG11_O_REG          0x2463                   
#define DAI_PB16_I_FLAG11_O_REG          0x2463                   
#define DAI_PB17_I_FLAG11_O_REG          0x2464     /* SRU_PIN4 */
#define DAI_PB18_I_FLAG11_O_REG          0x2464                   
#define DAI_PB19_I_FLAG11_O_REG          0x2464                   
#define DAI_PB20_I_FLAG11_O_REG          0x2464                   

#define DAI_PB01_I_FLAG12_O_REG          0x2460     /* SRU_PIN0 */
#define DAI_PB02_I_FLAG12_O_REG          0x2460                   
#define DAI_PB03_I_FLAG12_O_REG          0x2460                   
#define DAI_PB04_I_FLAG12_O_REG          0x2460                   
#define DAI_PB05_I_FLAG12_O_REG          0x2461     /* SRU_PIN1 */
#define DAI_PB06_I_FLAG12_O_REG          0x2461                   
#define DAI_PB07_I_FLAG12_O_REG          0x2461                   
#define DAI_PB08_I_FLAG12_O_REG          0x2461                   
#define DAI_PB09_I_FLAG12_O_REG          0x2462     /* SRU_PIN2 */
#define DAI_PB10_I_FLAG12_O_REG          0x2462                   
#define DAI_PB11_I_FLAG12_O_REG          0x2462                   
#define DAI_PB12_I_FLAG12_O_REG          0x2462                   
#define DAI_PB13_I_FLAG12_O_REG          0x2463     /* SRU_PIN3 */
#define DAI_PB14_I_FLAG12_O_REG          0x2463                   
#define DAI_PB15_I_FLAG12_O_REG          0x2463                   
#define DAI_PB16_I_FLAG12_O_REG          0x2463                   
#define DAI_PB17_I_FLAG12_O_REG          0x2464     /* SRU_PIN4 */
#define DAI_PB18_I_FLAG12_O_REG          0x2464                   
#define DAI_PB19_I_FLAG12_O_REG          0x2464                   
#define DAI_PB20_I_FLAG12_O_REG          0x2464                   

#define DAI_PB01_I_FLAG13_O_REG          0x2460     /* SRU_PIN0 */
#define DAI_PB02_I_FLAG13_O_REG          0x2460                   
#define DAI_PB03_I_FLAG13_O_REG          0x2460                   
#define DAI_PB04_I_FLAG13_O_REG          0x2460                   
#define DAI_PB05_I_FLAG13_O_REG          0x2461     /* SRU_PIN1 */
#define DAI_PB06_I_FLAG13_O_REG          0x2461                   
#define DAI_PB07_I_FLAG13_O_REG          0x2461                   
#define DAI_PB08_I_FLAG13_O_REG          0x2461                   
#define DAI_PB09_I_FLAG13_O_REG          0x2462     /* SRU_PIN2 */
#define DAI_PB10_I_FLAG13_O_REG          0x2462                   
#define DAI_PB11_I_FLAG13_O_REG          0x2462                   
#define DAI_PB12_I_FLAG13_O_REG          0x2462                   
#define DAI_PB13_I_FLAG13_O_REG          0x2463     /* SRU_PIN3 */
#define DAI_PB14_I_FLAG13_O_REG          0x2463                   
#define DAI_PB15_I_FLAG13_O_REG          0x2463                   
#define DAI_PB16_I_FLAG13_O_REG          0x2463                   
#define DAI_PB17_I_FLAG13_O_REG          0x2464     /* SRU_PIN4 */
#define DAI_PB18_I_FLAG13_O_REG          0x2464                   
#define DAI_PB19_I_FLAG13_O_REG          0x2464                   
#define DAI_PB20_I_FLAG13_O_REG          0x2464                   

#define DAI_PB01_I_FLAG14_O_REG          0x2460     /* SRU_PIN0 */
#define DAI_PB02_I_FLAG14_O_REG          0x2460                   
#define DAI_PB03_I_FLAG14_O_REG          0x2460                   
#define DAI_PB04_I_FLAG14_O_REG          0x2460                   
#define DAI_PB05_I_FLAG14_O_REG          0x2461     /* SRU_PIN1 */
#define DAI_PB06_I_FLAG14_O_REG          0x2461                   
#define DAI_PB07_I_FLAG14_O_REG          0x2461                   
#define DAI_PB08_I_FLAG14_O_REG          0x2461                   
#define DAI_PB09_I_FLAG14_O_REG          0x2462     /* SRU_PIN2 */
#define DAI_PB10_I_FLAG14_O_REG          0x2462                   
#define DAI_PB11_I_FLAG14_O_REG          0x2462                   
#define DAI_PB12_I_FLAG14_O_REG          0x2462                   
#define DAI_PB13_I_FLAG14_O_REG          0x2463     /* SRU_PIN3 */
#define DAI_PB14_I_FLAG14_O_REG          0x2463                   
#define DAI_PB15_I_FLAG14_O_REG          0x2463                   
#define DAI_PB16_I_FLAG14_O_REG          0x2463                   
#define DAI_PB17_I_FLAG14_O_REG          0x2464     /* SRU_PIN4 */
#define DAI_PB18_I_FLAG14_O_REG          0x2464                   
#define DAI_PB19_I_FLAG14_O_REG          0x2464                   
#define DAI_PB20_I_FLAG14_O_REG          0x2464                   

#define DAI_PB01_I_PCG_CLKA_O_REG          0x2460     /* SRU_PIN0 */
#define DAI_PB02_I_PCG_CLKA_O_REG          0x2460                   
#define DAI_PB03_I_PCG_CLKA_O_REG          0x2460                   
#define DAI_PB04_I_PCG_CLKA_O_REG          0x2460                   
#define DAI_PB05_I_PCG_CLKA_O_REG          0x2461     /* SRU_PIN1 */
#define DAI_PB06_I_PCG_CLKA_O_REG          0x2461                   
#define DAI_PB07_I_PCG_CLKA_O_REG          0x2461                   
#define DAI_PB08_I_PCG_CLKA_O_REG          0x2461                   
#define DAI_PB09_I_PCG_CLKA_O_REG          0x2462     /* SRU_PIN2 */
#define DAI_PB10_I_PCG_CLKA_O_REG          0x2462                   
#define DAI_PB11_I_PCG_CLKA_O_REG          0x2462                   
#define DAI_PB12_I_PCG_CLKA_O_REG          0x2462                   
#define DAI_PB13_I_PCG_CLKA_O_REG          0x2463     /* SRU_PIN3 */
#define DAI_PB14_I_PCG_CLKA_O_REG          0x2463                   
#define DAI_PB15_I_PCG_CLKA_O_REG          0x2463                   
#define DAI_PB16_I_PCG_CLKA_O_REG          0x2463                   
#define DAI_PB17_I_PCG_CLKA_O_REG          0x2464     /* SRU_PIN4 */
#define DAI_PB18_I_PCG_CLKA_O_REG          0x2464                   
#define DAI_PB19_I_PCG_CLKA_O_REG          0x2464                   
#define DAI_PB20_I_PCG_CLKA_O_REG          0x2464                   

#define DAI_PB01_I_PCG_CLKB_O_REG          0x2460     /* SRU_PIN0 */
#define DAI_PB02_I_PCG_CLKB_O_REG          0x2460                   
#define DAI_PB03_I_PCG_CLKB_O_REG          0x2460                   
#define DAI_PB04_I_PCG_CLKB_O_REG          0x2460                   
#define DAI_PB05_I_PCG_CLKB_O_REG          0x2461     /* SRU_PIN1 */
#define DAI_PB06_I_PCG_CLKB_O_REG          0x2461                   
#define DAI_PB07_I_PCG_CLKB_O_REG          0x2461                   
#define DAI_PB08_I_PCG_CLKB_O_REG          0x2461                   
#define DAI_PB09_I_PCG_CLKB_O_REG          0x2462     /* SRU_PIN2 */
#define DAI_PB10_I_PCG_CLKB_O_REG          0x2462                   
#define DAI_PB11_I_PCG_CLKB_O_REG          0x2462                   
#define DAI_PB12_I_PCG_CLKB_O_REG          0x2462                   
#define DAI_PB13_I_PCG_CLKB_O_REG          0x2463     /* SRU_PIN3 */
#define DAI_PB14_I_PCG_CLKB_O_REG          0x2463                   
#define DAI_PB15_I_PCG_CLKB_O_REG          0x2463                   
#define DAI_PB16_I_PCG_CLKB_O_REG          0x2463                   
#define DAI_PB17_I_PCG_CLKB_O_REG          0x2464     /* SRU_PIN4 */
#define DAI_PB18_I_PCG_CLKB_O_REG          0x2464                   
#define DAI_PB19_I_PCG_CLKB_O_REG          0x2464                   
#define DAI_PB20_I_PCG_CLKB_O_REG          0x2464                   

#define DAI_PB01_I_PCG_FSA_O_REG          0x2460     /* SRU_PIN0 */
#define DAI_PB02_I_PCG_FSA_O_REG          0x2460                   
#define DAI_PB03_I_PCG_FSA_O_REG          0x2460                   
#define DAI_PB04_I_PCG_FSA_O_REG          0x2460                   
#define DAI_PB05_I_PCG_FSA_O_REG          0x2461     /* SRU_PIN1 */
#define DAI_PB06_I_PCG_FSA_O_REG          0x2461                   
#define DAI_PB07_I_PCG_FSA_O_REG          0x2461                   
#define DAI_PB08_I_PCG_FSA_O_REG          0x2461                   
#define DAI_PB09_I_PCG_FSA_O_REG          0x2462     /* SRU_PIN2 */
#define DAI_PB10_I_PCG_FSA_O_REG          0x2462                   
#define DAI_PB11_I_PCG_FSA_O_REG          0x2462                   
#define DAI_PB12_I_PCG_FSA_O_REG          0x2462                   
#define DAI_PB13_I_PCG_FSA_O_REG          0x2463     /* SRU_PIN3 */
#define DAI_PB14_I_PCG_FSA_O_REG          0x2463                   
#define DAI_PB15_I_PCG_FSA_O_REG          0x2463                   
#define DAI_PB16_I_PCG_FSA_O_REG          0x2463                   
#define DAI_PB17_I_PCG_FSA_O_REG          0x2464     /* SRU_PIN4 */
#define DAI_PB18_I_PCG_FSA_O_REG          0x2464                   
#define DAI_PB19_I_PCG_FSA_O_REG          0x2464                   
#define DAI_PB20_I_PCG_FSA_O_REG          0x2464                   

#define DAI_PB01_I_PCG_FSB_O_REG          0x2460     /* SRU_PIN0 */
#define DAI_PB02_I_PCG_FSB_O_REG          0x2460                   
#define DAI_PB03_I_PCG_FSB_O_REG          0x2460                   
#define DAI_PB04_I_PCG_FSB_O_REG          0x2460                   
#define DAI_PB05_I_PCG_FSB_O_REG          0x2461     /* SRU_PIN1 */
#define DAI_PB06_I_PCG_FSB_O_REG          0x2461                   
#define DAI_PB07_I_PCG_FSB_O_REG          0x2461                   
#define DAI_PB08_I_PCG_FSB_O_REG          0x2461                   
#define DAI_PB09_I_PCG_FSB_O_REG          0x2462     /* SRU_PIN2 */
#define DAI_PB10_I_PCG_FSB_O_REG          0x2462                   
#define DAI_PB11_I_PCG_FSB_O_REG          0x2462                   
#define DAI_PB12_I_PCG_FSB_O_REG          0x2462                   
#define DAI_PB13_I_PCG_FSB_O_REG          0x2463     /* SRU_PIN3 */
#define DAI_PB14_I_PCG_FSB_O_REG          0x2463                   
#define DAI_PB15_I_PCG_FSB_O_REG          0x2463                   
#define DAI_PB16_I_PCG_FSB_O_REG          0x2463                   
#define DAI_PB17_I_PCG_FSB_O_REG          0x2464     /* SRU_PIN4 */
#define DAI_PB18_I_PCG_FSB_O_REG          0x2464                   
#define DAI_PB19_I_PCG_FSB_O_REG          0x2464                   
#define DAI_PB20_I_PCG_FSB_O_REG          0x2464                   

#define DAI_PB01_I_FLAG15_O_REG          0x2460     /* SRU_PIN0 */
#define DAI_PB02_I_FLAG15_O_REG          0x2460                   
#define DAI_PB03_I_FLAG15_O_REG          0x2460                   
#define DAI_PB04_I_FLAG15_O_REG          0x2460                   
#define DAI_PB05_I_FLAG15_O_REG          0x2461     /* SRU_PIN1 */
#define DAI_PB06_I_FLAG15_O_REG          0x2461                   
#define DAI_PB07_I_FLAG15_O_REG          0x2461                   
#define DAI_PB08_I_FLAG15_O_REG          0x2461                   
#define DAI_PB09_I_FLAG15_O_REG          0x2462     /* SRU_PIN2 */
#define DAI_PB10_I_FLAG15_O_REG          0x2462                   
#define DAI_PB11_I_FLAG15_O_REG          0x2462                   
#define DAI_PB12_I_FLAG15_O_REG          0x2462                   
#define DAI_PB13_I_FLAG15_O_REG          0x2463     /* SRU_PIN3 */
#define DAI_PB14_I_FLAG15_O_REG          0x2463                   
#define DAI_PB15_I_FLAG15_O_REG          0x2463                   
#define DAI_PB16_I_FLAG15_O_REG          0x2463                   
#define DAI_PB17_I_FLAG15_O_REG          0x2464     /* SRU_PIN4 */
#define DAI_PB18_I_FLAG15_O_REG          0x2464                   
#define DAI_PB19_I_FLAG15_O_REG          0x2464                   
#define DAI_PB20_I_FLAG15_O_REG          0x2464                   

#define DAI_PB01_I_SRC0_DAT_OP_O_REG          0x2460     /* SRU_PIN0 */
#define DAI_PB02_I_SRC0_DAT_OP_O_REG          0x2460                   
#define DAI_PB03_I_SRC0_DAT_OP_O_REG          0x2460                   
#define DAI_PB04_I_SRC0_DAT_OP_O_REG          0x2460                   
#define DAI_PB05_I_SRC0_DAT_OP_O_REG          0x2461     /* SRU_PIN1 */
#define DAI_PB06_I_SRC0_DAT_OP_O_REG          0x2461                   
#define DAI_PB07_I_SRC0_DAT_OP_O_REG          0x2461                   
#define DAI_PB08_I_SRC0_DAT_OP_O_REG          0x2461                   
#define DAI_PB09_I_SRC0_DAT_OP_O_REG          0x2462     /* SRU_PIN2 */
#define DAI_PB10_I_SRC0_DAT_OP_O_REG          0x2462                   
#define DAI_PB11_I_SRC0_DAT_OP_O_REG          0x2462                   
#define DAI_PB12_I_SRC0_DAT_OP_O_REG          0x2462                   
#define DAI_PB13_I_SRC0_DAT_OP_O_REG          0x2463     /* SRU_PIN3 */
#define DAI_PB14_I_SRC0_DAT_OP_O_REG          0x2463                   
#define DAI_PB15_I_SRC0_DAT_OP_O_REG          0x2463                   
#define DAI_PB16_I_SRC0_DAT_OP_O_REG          0x2463                   
#define DAI_PB17_I_SRC0_DAT_OP_O_REG          0x2464     /* SRU_PIN4 */
#define DAI_PB18_I_SRC0_DAT_OP_O_REG          0x2464                   
#define DAI_PB19_I_SRC0_DAT_OP_O_REG          0x2464                   
#define DAI_PB20_I_SRC0_DAT_OP_O_REG          0x2464                   

#define DAI_PB01_I_SRC1_DAT_OP_O_REG          0x2460     /* SRU_PIN0 */
#define DAI_PB02_I_SRC1_DAT_OP_O_REG          0x2460                   
#define DAI_PB03_I_SRC1_DAT_OP_O_REG          0x2460                   
#define DAI_PB04_I_SRC1_DAT_OP_O_REG          0x2460                   
#define DAI_PB05_I_SRC1_DAT_OP_O_REG          0x2461     /* SRU_PIN1 */
#define DAI_PB06_I_SRC1_DAT_OP_O_REG          0x2461                   
#define DAI_PB07_I_SRC1_DAT_OP_O_REG          0x2461                   
#define DAI_PB08_I_SRC1_DAT_OP_O_REG          0x2461                   
#define DAI_PB09_I_SRC1_DAT_OP_O_REG          0x2462     /* SRU_PIN2 */
#define DAI_PB10_I_SRC1_DAT_OP_O_REG          0x2462                   
#define DAI_PB11_I_SRC1_DAT_OP_O_REG          0x2462                   
#define DAI_PB12_I_SRC1_DAT_OP_O_REG          0x2462                   
#define DAI_PB13_I_SRC1_DAT_OP_O_REG          0x2463     /* SRU_PIN3 */
#define DAI_PB14_I_SRC1_DAT_OP_O_REG          0x2463                   
#define DAI_PB15_I_SRC1_DAT_OP_O_REG          0x2463                   
#define DAI_PB16_I_SRC1_DAT_OP_O_REG          0x2463                   
#define DAI_PB17_I_SRC1_DAT_OP_O_REG          0x2464     /* SRU_PIN4 */
#define DAI_PB18_I_SRC1_DAT_OP_O_REG          0x2464                   
#define DAI_PB19_I_SRC1_DAT_OP_O_REG          0x2464                   
#define DAI_PB20_I_SRC1_DAT_OP_O_REG          0x2464                   

#define DAI_PB01_I_SRC2_DAT_OP_O_REG          0x2460     /* SRU_PIN0 */
#define DAI_PB02_I_SRC2_DAT_OP_O_REG          0x2460                   
#define DAI_PB03_I_SRC2_DAT_OP_O_REG          0x2460                   
#define DAI_PB04_I_SRC2_DAT_OP_O_REG          0x2460                   
#define DAI_PB05_I_SRC2_DAT_OP_O_REG          0x2461     /* SRU_PIN1 */
#define DAI_PB06_I_SRC2_DAT_OP_O_REG          0x2461                   
#define DAI_PB07_I_SRC2_DAT_OP_O_REG          0x2461                   
#define DAI_PB08_I_SRC2_DAT_OP_O_REG          0x2461                   
#define DAI_PB09_I_SRC2_DAT_OP_O_REG          0x2462     /* SRU_PIN2 */
#define DAI_PB10_I_SRC2_DAT_OP_O_REG          0x2462                   
#define DAI_PB11_I_SRC2_DAT_OP_O_REG          0x2462                   
#define DAI_PB12_I_SRC2_DAT_OP_O_REG          0x2462                   
#define DAI_PB13_I_SRC2_DAT_OP_O_REG          0x2463     /* SRU_PIN3 */
#define DAI_PB14_I_SRC2_DAT_OP_O_REG          0x2463                   
#define DAI_PB15_I_SRC2_DAT_OP_O_REG          0x2463                   
#define DAI_PB16_I_SRC2_DAT_OP_O_REG          0x2463                   
#define DAI_PB17_I_SRC2_DAT_OP_O_REG          0x2464     /* SRU_PIN4 */
#define DAI_PB18_I_SRC2_DAT_OP_O_REG          0x2464                   
#define DAI_PB19_I_SRC2_DAT_OP_O_REG          0x2464                   
#define DAI_PB20_I_SRC2_DAT_OP_O_REG          0x2464                   

#define DAI_PB01_I_SRC3_DAT_OP_O_REG          0x2460     /* SRU_PIN0 */
#define DAI_PB02_I_SRC3_DAT_OP_O_REG          0x2460                   
#define DAI_PB03_I_SRC3_DAT_OP_O_REG          0x2460                   
#define DAI_PB04_I_SRC3_DAT_OP_O_REG          0x2460                   
#define DAI_PB05_I_SRC3_DAT_OP_O_REG          0x2461     /* SRU_PIN1 */
#define DAI_PB06_I_SRC3_DAT_OP_O_REG          0x2461                   
#define DAI_PB07_I_SRC3_DAT_OP_O_REG          0x2461                   
#define DAI_PB08_I_SRC3_DAT_OP_O_REG          0x2461                   
#define DAI_PB09_I_SRC3_DAT_OP_O_REG          0x2462     /* SRU_PIN2 */
#define DAI_PB10_I_SRC3_DAT_OP_O_REG          0x2462                   
#define DAI_PB11_I_SRC3_DAT_OP_O_REG          0x2462                   
#define DAI_PB12_I_SRC3_DAT_OP_O_REG          0x2462                   
#define DAI_PB13_I_SRC3_DAT_OP_O_REG          0x2463     /* SRU_PIN3 */
#define DAI_PB14_I_SRC3_DAT_OP_O_REG          0x2463                   
#define DAI_PB15_I_SRC3_DAT_OP_O_REG          0x2463                   
#define DAI_PB16_I_SRC3_DAT_OP_O_REG          0x2463                   
#define DAI_PB17_I_SRC3_DAT_OP_O_REG          0x2464     /* SRU_PIN4 */
#define DAI_PB18_I_SRC3_DAT_OP_O_REG          0x2464                   
#define DAI_PB19_I_SRC3_DAT_OP_O_REG          0x2464                   
#define DAI_PB20_I_SRC3_DAT_OP_O_REG          0x2464                   

#define DAI_PB01_I_DIR_DAT_O_REG          0x2460     /* SRU_PIN0 */
#define DAI_PB02_I_DIR_DAT_O_REG          0x2460                   
#define DAI_PB03_I_DIR_DAT_O_REG          0x2460                   
#define DAI_PB04_I_DIR_DAT_O_REG          0x2460                   
#define DAI_PB05_I_DIR_DAT_O_REG          0x2461     /* SRU_PIN1 */
#define DAI_PB06_I_DIR_DAT_O_REG          0x2461                   
#define DAI_PB07_I_DIR_DAT_O_REG          0x2461                   
#define DAI_PB08_I_DIR_DAT_O_REG          0x2461                   
#define DAI_PB09_I_DIR_DAT_O_REG          0x2462     /* SRU_PIN2 */
#define DAI_PB10_I_DIR_DAT_O_REG          0x2462                   
#define DAI_PB11_I_DIR_DAT_O_REG          0x2462                   
#define DAI_PB12_I_DIR_DAT_O_REG          0x2462                   
#define DAI_PB13_I_DIR_DAT_O_REG          0x2463     /* SRU_PIN3 */
#define DAI_PB14_I_DIR_DAT_O_REG          0x2463                   
#define DAI_PB15_I_DIR_DAT_O_REG          0x2463                   
#define DAI_PB16_I_DIR_DAT_O_REG          0x2463                   
#define DAI_PB17_I_DIR_DAT_O_REG          0x2464     /* SRU_PIN4 */
#define DAI_PB18_I_DIR_DAT_O_REG          0x2464                   
#define DAI_PB19_I_DIR_DAT_O_REG          0x2464                   
#define DAI_PB20_I_DIR_DAT_O_REG          0x2464                   

#define DAI_PB01_I_DIR_FS_O_REG          0x2460     /* SRU_PIN0 */
#define DAI_PB02_I_DIR_FS_O_REG          0x2460                   
#define DAI_PB03_I_DIR_FS_O_REG          0x2460                   
#define DAI_PB04_I_DIR_FS_O_REG          0x2460                   
#define DAI_PB05_I_DIR_FS_O_REG          0x2461     /* SRU_PIN1 */
#define DAI_PB06_I_DIR_FS_O_REG          0x2461                   
#define DAI_PB07_I_DIR_FS_O_REG          0x2461                   
#define DAI_PB08_I_DIR_FS_O_REG          0x2461                   
#define DAI_PB09_I_DIR_FS_O_REG          0x2462     /* SRU_PIN2 */
#define DAI_PB10_I_DIR_FS_O_REG          0x2462                   
#define DAI_PB11_I_DIR_FS_O_REG          0x2462                   
#define DAI_PB12_I_DIR_FS_O_REG          0x2462                   
#define DAI_PB13_I_DIR_FS_O_REG          0x2463     /* SRU_PIN3 */
#define DAI_PB14_I_DIR_FS_O_REG          0x2463                   
#define DAI_PB15_I_DIR_FS_O_REG          0x2463                   
#define DAI_PB16_I_DIR_FS_O_REG          0x2463                   
#define DAI_PB17_I_DIR_FS_O_REG          0x2464     /* SRU_PIN4 */
#define DAI_PB18_I_DIR_FS_O_REG          0x2464                   
#define DAI_PB19_I_DIR_FS_O_REG          0x2464                   
#define DAI_PB20_I_DIR_FS_O_REG          0x2464                   

#define DAI_PB01_I_DIR_CLK_O_REG          0x2460     /* SRU_PIN0 */
#define DAI_PB02_I_DIR_CLK_O_REG          0x2460                   
#define DAI_PB03_I_DIR_CLK_O_REG          0x2460                   
#define DAI_PB04_I_DIR_CLK_O_REG          0x2460                   
#define DAI_PB05_I_DIR_CLK_O_REG          0x2461     /* SRU_PIN1 */
#define DAI_PB06_I_DIR_CLK_O_REG          0x2461                   
#define DAI_PB07_I_DIR_CLK_O_REG          0x2461                   
#define DAI_PB08_I_DIR_CLK_O_REG          0x2461                   
#define DAI_PB09_I_DIR_CLK_O_REG          0x2462     /* SRU_PIN2 */
#define DAI_PB10_I_DIR_CLK_O_REG          0x2462                   
#define DAI_PB11_I_DIR_CLK_O_REG          0x2462                   
#define DAI_PB12_I_DIR_CLK_O_REG          0x2462                   
#define DAI_PB13_I_DIR_CLK_O_REG          0x2463     /* SRU_PIN3 */
#define DAI_PB14_I_DIR_CLK_O_REG          0x2463                   
#define DAI_PB15_I_DIR_CLK_O_REG          0x2463                   
#define DAI_PB16_I_DIR_CLK_O_REG          0x2463                   
#define DAI_PB17_I_DIR_CLK_O_REG          0x2464     /* SRU_PIN4 */
#define DAI_PB18_I_DIR_CLK_O_REG          0x2464                   
#define DAI_PB19_I_DIR_CLK_O_REG          0x2464                   
#define DAI_PB20_I_DIR_CLK_O_REG          0x2464                   

#define DAI_PB01_I_DIR_TDMCLK_O_REG          0x2460     /* SRU_PIN0 */
#define DAI_PB02_I_DIR_TDMCLK_O_REG          0x2460                   
#define DAI_PB03_I_DIR_TDMCLK_O_REG          0x2460                   
#define DAI_PB04_I_DIR_TDMCLK_O_REG          0x2460                   
#define DAI_PB05_I_DIR_TDMCLK_O_REG          0x2461     /* SRU_PIN1 */
#define DAI_PB06_I_DIR_TDMCLK_O_REG          0x2461                   
#define DAI_PB07_I_DIR_TDMCLK_O_REG          0x2461                   
#define DAI_PB08_I_DIR_TDMCLK_O_REG          0x2461                   
#define DAI_PB09_I_DIR_TDMCLK_O_REG          0x2462     /* SRU_PIN2 */
#define DAI_PB10_I_DIR_TDMCLK_O_REG          0x2462                   
#define DAI_PB11_I_DIR_TDMCLK_O_REG          0x2462                   
#define DAI_PB12_I_DIR_TDMCLK_O_REG          0x2462                   
#define DAI_PB13_I_DIR_TDMCLK_O_REG          0x2463     /* SRU_PIN3 */
#define DAI_PB14_I_DIR_TDMCLK_O_REG          0x2463                   
#define DAI_PB15_I_DIR_TDMCLK_O_REG          0x2463                   
#define DAI_PB16_I_DIR_TDMCLK_O_REG          0x2463                   
#define DAI_PB17_I_DIR_TDMCLK_O_REG          0x2464     /* SRU_PIN4 */
#define DAI_PB18_I_DIR_TDMCLK_O_REG          0x2464                   
#define DAI_PB19_I_DIR_TDMCLK_O_REG          0x2464                   
#define DAI_PB20_I_DIR_TDMCLK_O_REG          0x2464                   

#define DAI_PB01_I_DIT_DAT_O_REG          0x2460     /* SRU_PIN0 */
#define DAI_PB02_I_DIT_DAT_O_REG          0x2460                   
#define DAI_PB03_I_DIT_DAT_O_REG          0x2460                   
#define DAI_PB04_I_DIT_DAT_O_REG          0x2460                   
#define DAI_PB05_I_DIT_DAT_O_REG          0x2461     /* SRU_PIN1 */
#define DAI_PB06_I_DIT_DAT_O_REG          0x2461                   
#define DAI_PB07_I_DIT_DAT_O_REG          0x2461                   
#define DAI_PB08_I_DIT_DAT_O_REG          0x2461                   
#define DAI_PB09_I_DIT_DAT_O_REG          0x2462     /* SRU_PIN2 */
#define DAI_PB10_I_DIT_DAT_O_REG          0x2462                   
#define DAI_PB11_I_DIT_DAT_O_REG          0x2462                   
#define DAI_PB12_I_DIT_DAT_O_REG          0x2462                   
#define DAI_PB13_I_DIT_DAT_O_REG          0x2463     /* SRU_PIN3 */
#define DAI_PB14_I_DIT_DAT_O_REG          0x2463                   
#define DAI_PB15_I_DIT_DAT_O_REG          0x2463                   
#define DAI_PB16_I_DIT_DAT_O_REG          0x2463                   
#define DAI_PB17_I_DIT_DAT_O_REG          0x2464     /* SRU_PIN4 */
#define DAI_PB18_I_DIT_DAT_O_REG          0x2464                   
#define DAI_PB19_I_DIT_DAT_O_REG          0x2464                   
#define DAI_PB20_I_DIT_DAT_O_REG          0x2464                   
 
#define DAI_PB01_I_SPIB_MISO_O_REG          0x2460     /* SRU_PIN0 */
#define DAI_PB02_I_SPIB_MISO_O_REG          0x2460                   
#define DAI_PB03_I_SPIB_MISO_O_REG          0x2460                   
#define DAI_PB04_I_SPIB_MISO_O_REG          0x2460                   
#define DAI_PB05_I_SPIB_MISO_O_REG          0x2461     /* SRU_PIN1 */
#define DAI_PB06_I_SPIB_MISO_O_REG          0x2461                   
#define DAI_PB07_I_SPIB_MISO_O_REG          0x2461                   
#define DAI_PB08_I_SPIB_MISO_O_REG          0x2461                   
#define DAI_PB09_I_SPIB_MISO_O_REG          0x2462     /* SRU_PIN2 */
#define DAI_PB10_I_SPIB_MISO_O_REG          0x2462                   
#define DAI_PB11_I_SPIB_MISO_O_REG          0x2462                   
#define DAI_PB12_I_SPIB_MISO_O_REG          0x2462                   
#define DAI_PB13_I_SPIB_MISO_O_REG          0x2463     /* SRU_PIN3 */
#define DAI_PB14_I_SPIB_MISO_O_REG          0x2463                   
#define DAI_PB15_I_SPIB_MISO_O_REG          0x2463                   
#define DAI_PB16_I_SPIB_MISO_O_REG          0x2463                   
#define DAI_PB17_I_SPIB_MISO_O_REG          0x2464     /* SRU_PIN4 */
#define DAI_PB18_I_SPIB_MISO_O_REG          0x2464                   
#define DAI_PB19_I_SPIB_MISO_O_REG          0x2464                   
#define DAI_PB20_I_SPIB_MISO_O_REG          0x2464                   

#define DAI_PB01_I_SPIB_MOSI_O_REG          0x2460     /* SRU_PIN0 */
#define DAI_PB02_I_SPIB_MOSI_O_REG          0x2460                   
#define DAI_PB03_I_SPIB_MOSI_O_REG          0x2460                   
#define DAI_PB04_I_SPIB_MOSI_O_REG          0x2460                   
#define DAI_PB05_I_SPIB_MOSI_O_REG          0x2461     /* SRU_PIN1 */
#define DAI_PB06_I_SPIB_MOSI_O_REG          0x2461                   
#define DAI_PB07_I_SPIB_MOSI_O_REG          0x2461                   
#define DAI_PB08_I_SPIB_MOSI_O_REG          0x2461                   
#define DAI_PB09_I_SPIB_MOSI_O_REG          0x2462     /* SRU_PIN2 */
#define DAI_PB10_I_SPIB_MOSI_O_REG          0x2462                   
#define DAI_PB11_I_SPIB_MOSI_O_REG          0x2462                   
#define DAI_PB12_I_SPIB_MOSI_O_REG          0x2462                   
#define DAI_PB13_I_SPIB_MOSI_O_REG          0x2463     /* SRU_PIN3 */
#define DAI_PB14_I_SPIB_MOSI_O_REG          0x2463                   
#define DAI_PB15_I_SPIB_MOSI_O_REG          0x2463                   
#define DAI_PB16_I_SPIB_MOSI_O_REG          0x2463                   
#define DAI_PB17_I_SPIB_MOSI_O_REG          0x2464     /* SRU_PIN4 */
#define DAI_PB18_I_SPIB_MOSI_O_REG          0x2464                   
#define DAI_PB19_I_SPIB_MOSI_O_REG          0x2464                   
#define DAI_PB20_I_SPIB_MOSI_O_REG          0x2464                   

#define DAI_PB01_I_SPIB_CLK_O_REG          0x2460     /* SRU_PIN0 */
#define DAI_PB02_I_SPIB_CLK_O_REG          0x2460                   
#define DAI_PB03_I_SPIB_CLK_O_REG          0x2460                   
#define DAI_PB04_I_SPIB_CLK_O_REG          0x2460                   
#define DAI_PB05_I_SPIB_CLK_O_REG          0x2461     /* SRU_PIN1 */
#define DAI_PB06_I_SPIB_CLK_O_REG          0x2461                   
#define DAI_PB07_I_SPIB_CLK_O_REG          0x2461                   
#define DAI_PB08_I_SPIB_CLK_O_REG          0x2461                   
#define DAI_PB09_I_SPIB_CLK_O_REG          0x2462     /* SRU_PIN2 */
#define DAI_PB10_I_SPIB_CLK_O_REG          0x2462                   
#define DAI_PB11_I_SPIB_CLK_O_REG          0x2462                   
#define DAI_PB12_I_SPIB_CLK_O_REG          0x2462                   
#define DAI_PB13_I_SPIB_CLK_O_REG          0x2463     /* SRU_PIN3 */
#define DAI_PB14_I_SPIB_CLK_O_REG          0x2463                   
#define DAI_PB15_I_SPIB_CLK_O_REG          0x2463                   
#define DAI_PB16_I_SPIB_CLK_O_REG          0x2463                   
#define DAI_PB17_I_SPIB_CLK_O_REG          0x2464     /* SRU_PIN4 */
#define DAI_PB18_I_SPIB_CLK_O_REG          0x2464                   
#define DAI_PB19_I_SPIB_CLK_O_REG          0x2464                   
#define DAI_PB20_I_SPIB_CLK_O_REG          0x2464                   

#define DAI_PB01_I_SPIB_FLG0_O_REG          0x2460     /* SRU_PIN0 */
#define DAI_PB02_I_SPIB_FLG0_O_REG          0x2460                   
#define DAI_PB03_I_SPIB_FLG0_O_REG          0x2460                   
#define DAI_PB04_I_SPIB_FLG0_O_REG          0x2460                   
#define DAI_PB05_I_SPIB_FLG0_O_REG          0x2461     /* SRU_PIN1 */
#define DAI_PB06_I_SPIB_FLG0_O_REG          0x2461                   
#define DAI_PB07_I_SPIB_FLG0_O_REG          0x2461                   
#define DAI_PB08_I_SPIB_FLG0_O_REG          0x2461                   
#define DAI_PB09_I_SPIB_FLG0_O_REG          0x2462     /* SRU_PIN2 */
#define DAI_PB10_I_SPIB_FLG0_O_REG          0x2462                   
#define DAI_PB11_I_SPIB_FLG0_O_REG          0x2462                   
#define DAI_PB12_I_SPIB_FLG0_O_REG          0x2462                   
#define DAI_PB13_I_SPIB_FLG0_O_REG          0x2463     /* SRU_PIN3 */
#define DAI_PB14_I_SPIB_FLG0_O_REG          0x2463                   
#define DAI_PB15_I_SPIB_FLG0_O_REG          0x2463                   
#define DAI_PB16_I_SPIB_FLG0_O_REG          0x2463                   
#define DAI_PB17_I_SPIB_FLG0_O_REG          0x2464     /* SRU_PIN4 */
#define DAI_PB18_I_SPIB_FLG0_O_REG          0x2464                   
#define DAI_PB19_I_SPIB_FLG0_O_REG          0x2464                   
#define DAI_PB20_I_SPIB_FLG0_O_REG          0x2464                   

#define DAI_PB01_I_SPIB_FLG1_O_REG          0x2460     /* SRU_PIN0 */
#define DAI_PB02_I_SPIB_FLG1_O_REG          0x2460                   
#define DAI_PB03_I_SPIB_FLG1_O_REG          0x2460                   
#define DAI_PB04_I_SPIB_FLG1_O_REG          0x2460                   
#define DAI_PB05_I_SPIB_FLG1_O_REG          0x2461     /* SRU_PIN1 */
#define DAI_PB06_I_SPIB_FLG1_O_REG          0x2461                   
#define DAI_PB07_I_SPIB_FLG1_O_REG          0x2461                   
#define DAI_PB08_I_SPIB_FLG1_O_REG          0x2461                   
#define DAI_PB09_I_SPIB_FLG1_O_REG          0x2462     /* SRU_PIN2 */
#define DAI_PB10_I_SPIB_FLG1_O_REG          0x2462                   
#define DAI_PB11_I_SPIB_FLG1_O_REG          0x2462                   
#define DAI_PB12_I_SPIB_FLG1_O_REG          0x2462                   
#define DAI_PB13_I_SPIB_FLG1_O_REG          0x2463     /* SRU_PIN3 */
#define DAI_PB14_I_SPIB_FLG1_O_REG          0x2463                   
#define DAI_PB15_I_SPIB_FLG1_O_REG          0x2463                   
#define DAI_PB16_I_SPIB_FLG1_O_REG          0x2463                   
#define DAI_PB17_I_SPIB_FLG1_O_REG          0x2464     /* SRU_PIN4 */
#define DAI_PB18_I_SPIB_FLG1_O_REG          0x2464                   
#define DAI_PB19_I_SPIB_FLG1_O_REG          0x2464                   
#define DAI_PB20_I_SPIB_FLG1_O_REG          0x2464                   

#define DAI_PB01_I_SPIB_FLG2_O_REG          0x2460     /* SRU_PIN0 */
#define DAI_PB02_I_SPIB_FLG2_O_REG          0x2460                   
#define DAI_PB03_I_SPIB_FLG2_O_REG          0x2460                   
#define DAI_PB04_I_SPIB_FLG2_O_REG          0x2460                   
#define DAI_PB05_I_SPIB_FLG2_O_REG          0x2461     /* SRU_PIN1 */
#define DAI_PB06_I_SPIB_FLG2_O_REG          0x2461                   
#define DAI_PB07_I_SPIB_FLG2_O_REG          0x2461                   
#define DAI_PB08_I_SPIB_FLG2_O_REG          0x2461                   
#define DAI_PB09_I_SPIB_FLG2_O_REG          0x2462     /* SRU_PIN2 */
#define DAI_PB10_I_SPIB_FLG2_O_REG          0x2462                   
#define DAI_PB11_I_SPIB_FLG2_O_REG          0x2462                   
#define DAI_PB12_I_SPIB_FLG2_O_REG          0x2462                   
#define DAI_PB13_I_SPIB_FLG2_O_REG          0x2463     /* SRU_PIN3 */
#define DAI_PB14_I_SPIB_FLG2_O_REG          0x2463                   
#define DAI_PB15_I_SPIB_FLG2_O_REG          0x2463                   
#define DAI_PB16_I_SPIB_FLG2_O_REG          0x2463                   
#define DAI_PB17_I_SPIB_FLG2_O_REG          0x2464     /* SRU_PIN4 */
#define DAI_PB18_I_SPIB_FLG2_O_REG          0x2464                   
#define DAI_PB19_I_SPIB_FLG2_O_REG          0x2464                   
#define DAI_PB20_I_SPIB_FLG2_O_REG          0x2464                   

#define DAI_PB01_I_SPIB_FLG3_O_REG          0x2460     /* SRU_PIN0 */
#define DAI_PB02_I_SPIB_FLG3_O_REG          0x2460                   
#define DAI_PB03_I_SPIB_FLG3_O_REG          0x2460                   
#define DAI_PB04_I_SPIB_FLG3_O_REG          0x2460                   
#define DAI_PB05_I_SPIB_FLG3_O_REG          0x2461     /* SRU_PIN1 */
#define DAI_PB06_I_SPIB_FLG3_O_REG          0x2461                   
#define DAI_PB07_I_SPIB_FLG3_O_REG          0x2461                   
#define DAI_PB08_I_SPIB_FLG3_O_REG          0x2461                   
#define DAI_PB09_I_SPIB_FLG3_O_REG          0x2462     /* SRU_PIN2 */
#define DAI_PB10_I_SPIB_FLG3_O_REG          0x2462                   
#define DAI_PB11_I_SPIB_FLG3_O_REG          0x2462                   
#define DAI_PB12_I_SPIB_FLG3_O_REG          0x2462                   
#define DAI_PB13_I_SPIB_FLG3_O_REG          0x2463     /* SRU_PIN3 */
#define DAI_PB14_I_SPIB_FLG3_O_REG          0x2463                   
#define DAI_PB15_I_SPIB_FLG3_O_REG          0x2463                   
#define DAI_PB16_I_SPIB_FLG3_O_REG          0x2463                   
#define DAI_PB17_I_SPIB_FLG3_O_REG          0x2464     /* SRU_PIN4 */
#define DAI_PB18_I_SPIB_FLG3_O_REG          0x2464                   
#define DAI_PB19_I_SPIB_FLG3_O_REG          0x2464                   
#define DAI_PB20_I_SPIB_FLG3_O_REG          0x2464                   

#define DAI_PB01_I_DIR_LRCLK_FB_O_REG          0x2460     /* SRU_PIN0 */
#define DAI_PB02_I_DIR_LRCLK_FB_O_REG          0x2460                   
#define DAI_PB03_I_DIR_LRCLK_FB_O_REG          0x2460                   
#define DAI_PB04_I_DIR_LRCLK_FB_O_REG          0x2460                   
#define DAI_PB05_I_DIR_LRCLK_FB_O_REG          0x2461     /* SRU_PIN1 */
#define DAI_PB06_I_DIR_LRCLK_FB_O_REG          0x2461                   
#define DAI_PB07_I_DIR_LRCLK_FB_O_REG          0x2461                   
#define DAI_PB08_I_DIR_LRCLK_FB_O_REG          0x2461                   
#define DAI_PB09_I_DIR_LRCLK_FB_O_REG          0x2462     /* SRU_PIN2 */
#define DAI_PB10_I_DIR_LRCLK_FB_O_REG          0x2462                   
#define DAI_PB11_I_DIR_LRCLK_FB_O_REG          0x2462                   
#define DAI_PB12_I_DIR_LRCLK_FB_O_REG          0x2462                   
#define DAI_PB13_I_DIR_LRCLK_FB_O_REG          0x2463     /* SRU_PIN3 */
#define DAI_PB14_I_DIR_LRCLK_FB_O_REG          0x2463                   
#define DAI_PB15_I_DIR_LRCLK_FB_O_REG          0x2463                   
#define DAI_PB16_I_DIR_LRCLK_FB_O_REG          0x2463                   
#define DAI_PB17_I_DIR_LRCLK_FB_O_REG          0x2464     /* SRU_PIN4 */
#define DAI_PB18_I_DIR_LRCLK_FB_O_REG          0x2464                   
#define DAI_PB19_I_DIR_LRCLK_FB_O_REG          0x2464                   
#define DAI_PB20_I_DIR_LRCLK_FB_O_REG          0x2464                   

#define DAI_PB01_DIR_LRCLK_REF_O_REG          0x2460     /* SRU_PIN0 */
#define DAI_PB02_DIR_LRCLK_REF_O_REG          0x2460                   
#define DAI_PB03_DIR_LRCLK_REF_O_REG          0x2460                   
#define DAI_PB04_DIR_LRCLK_REF_O_REG          0x2460                   
#define DAI_PB05_DIR_LRCLK_REF_O_REG          0x2461     /* SRU_PIN1 */
#define DAI_PB06_DIR_LRCLK_REF_O_REG          0x2461                   
#define DAI_PB07_DIR_LRCLK_REF_O_REG          0x2461                   
#define DAI_PB08_DIR_LRCLK_REF_O_REG          0x2461                   
#define DAI_PB09_DIR_LRCLK_REF_O_REG          0x2462     /* SRU_PIN2 */
#define DAI_PB10_DIR_LRCLK_REF_O_REG          0x2462                   
#define DAI_PB11_DIR_LRCLK_REF_O_REG          0x2462                   
#define DAI_PB12_DIR_LRCLK_REF_O_REG          0x2462                   
#define DAI_PB13_DIR_LRCLK_REF_O_REG          0x2463     /* SRU_PIN3 */
#define DAI_PB14_DIR_LRCLK_REF_O_REG          0x2463                   
#define DAI_PB15_DIR_LRCLK_REF_O_REG          0x2463                   
#define DAI_PB16_DIR_LRCLK_REF_O_REG          0x2463                   
#define DAI_PB17_DIR_LRCLK_REF_O_REG          0x2464     /* SRU_PIN4 */
#define DAI_PB18_DIR_LRCLK_REF_O_REG          0x2464                   
#define DAI_PB19_DIR_LRCLK_REF_O_REG          0x2464                   
#define DAI_PB20_DIR_LRCLK_REF_O_REG          0x2464                   
 
#define DAI_PB01_I_LOW_REG          0x2460     /* SRU_PIN0 */
#define DAI_PB02_I_LOW_REG          0x2460                   
#define DAI_PB03_I_LOW_REG          0x2460                   
#define DAI_PB04_I_LOW_REG          0x2460                   
#define DAI_PB05_I_LOW_REG          0x2461     /* SRU_PIN1 */
#define DAI_PB06_I_LOW_REG          0x2461                   
#define DAI_PB07_I_LOW_REG          0x2461                   
#define DAI_PB08_I_LOW_REG          0x2461                   
#define DAI_PB09_I_LOW_REG          0x2462     /* SRU_PIN2 */
#define DAI_PB10_I_LOW_REG          0x2462                   
#define DAI_PB11_I_LOW_REG          0x2462                   
#define DAI_PB12_I_LOW_REG          0x2462                   
#define DAI_PB13_I_LOW_REG          0x2463     /* SRU_PIN3 */
#define DAI_PB14_I_LOW_REG          0x2463                   
#define DAI_PB15_I_LOW_REG          0x2463                   
#define DAI_PB16_I_LOW_REG          0x2463                   
#define DAI_PB17_I_LOW_REG          0x2464     /* SRU_PIN4 */
#define DAI_PB18_I_LOW_REG          0x2464                   
#define DAI_PB19_I_LOW_REG          0x2464                   
#define DAI_PB20_I_LOW_REG          0x2464                   

#define INV_DAI_PB19_LOW_REG          0x2464     /* SRU_PIN4 */
#define INV_DAI_PB20_LOW_REG          0x2464                   
                         
#define DAI_PB01_I_HIGH_REG          0x2460     /* SRU_PIN0 */
#define DAI_PB02_I_HIGH_REG          0x2460                   
#define DAI_PB03_I_HIGH_REG          0x2460                   
#define DAI_PB04_I_HIGH_REG          0x2460                   
#define DAI_PB05_I_HIGH_REG          0x2461     /* SRU_PIN1 */
#define DAI_PB06_I_HIGH_REG          0x2461                   
#define DAI_PB07_I_HIGH_REG          0x2461                   
#define DAI_PB08_I_HIGH_REG          0x2461                   
#define DAI_PB09_I_HIGH_REG          0x2462     /* SRU_PIN2 */
#define DAI_PB10_I_HIGH_REG          0x2462                   
#define DAI_PB11_I_HIGH_REG          0x2462                   
#define DAI_PB12_I_HIGH_REG          0x2462                   
#define DAI_PB13_I_HIGH_REG          0x2463     /* SRU_PIN3 */
#define DAI_PB14_I_HIGH_REG          0x2463                   
#define DAI_PB15_I_HIGH_REG          0x2463                   
#define DAI_PB16_I_HIGH_REG          0x2463                   
#define DAI_PB17_I_HIGH_REG          0x2464     /* SRU_PIN4 */
#define DAI_PB18_I_HIGH_REG          0x2464                   
#define DAI_PB19_I_HIGH_REG          0x2464                   
#define DAI_PB20_I_HIGH_REG          0x2464                   

#define INV_DAI_PB19_HIGH_REG          0x2464     /* SRU_PIN4 */
#define INV_DAI_PB20_HIGH_REG          0x2464                   


/*============================================================================*
 *                                                                                                                                                    *
 *      GROUP E                                                                   *
 *                                                                                                                                                        *
 *============================================================================*/

//------------------------------------------------------------------------------
//  Group E: Input bitfield masks & shifts
//------------------------------------------------------------------------------

//SRU_EXT_MISCA
#define MISCA0_I_MASK                               0xFFFFFFE0
#define MISCA0_I_SHIFT                              0
#define FLAG13_I_MASK                               0xFFFFFFE0
#define FLAG13_I_SHIFT                              0
#define DAI_INT_28_I_MASK                           0xFFFFFFE0
#define DAI_INT_28_I_SHIFT                          0


#define MISCA1_I_MASK                               0xFFFFFC1F
#define MISCA1_I_SHIFT                              5
#define FLAG14_I_MASK                               0xFFFFFC1F
#define FLAG14_I_SHIFT                              5
#define DAI_INT_29_I_MASK                           0xFFFFFC1F
#define DAI_INT_29_I_SHIFT                          5

#define MISCA2_I_MASK                               0xFFFF83FF
#define MISCA2_I_SHIFT                              10
#define FLAG15_I_MASK                               0xFFFF83FF
#define FLAG15_I_SHIFT                              10
#define DAI_INT_30_I_MASK                           0xFFFF83FF
#define DAI_INT_30_I_SHIFT                          10

#define MISCA3_I_MASK                               0xFFF07FFF
#define MISCA3_I_SHIFT                              15
#define DAI_INT_31_I_MASK                           0xFFF07FFF
#define DAI_INT_31_I_SHIFT                          15
#define SPIB_DS_I_MASK                              0xFFF07FFF
#define SPIB_DS_I_SHIFT                             15

#define MISCA4_I_MASK                               0xFE0FFFFF
#define MISCA4_I_SHIFT                              20
#define MISCA5_I_MASK                               0xC1FFFFFF
#define MISCA5_I_SHIFT                              25
#define SPIB_MISO_I_MASK                            0xC1FFFFFF
#define SPIB_MISO_I_SHIFT                           25

#define INV_MISCA4_I_MASK                           0x7FFFFFFF
#define INV_MISCA4_I_SHIFT                          30
#define INV_MISCA5_I_MASK                           0xBFFFFFFF
#define INV_MISCA5_I_SHIFT                          31
#define SPIB_MOSI_I_MASK                            0xBFFFFFFF
#define SPIB_MOSI_I_SHIFT                           31

//SRU_EXT_MISCB
#define MISCB0_I_MASK                               0xFFFFFFE0
#define MISCB0_I_SHIFT                              0
#define TIMER0_I_MASK                               0xFFFFFFE0
#define TIMER0_I_SHIFT                              0
#define DAI_INT_22_I_MASK                           0xFFFFFFE0
#define DAI_INT_22_I_SHIFT                          0

#define MISCB1_I_MASK                               0xFFFFFC1F
#define MISCB1_I_SHIFT                              5
#define TIMER1_I_MASK                               0xFFFFFC1F
#define TIMER1_I_SHIFT                              5
#define DAI_INT_23_I_MASK                           0xFFFFFC1F
#define DAI_INT_23_I_SHIFT                          5

#define MISCB2_I_MASK                               0xFFFF83FF
#define MISCB2_I_SHIFT                              10
#define TIMER2_I_MASK                               0xFFFF83FF
#define TIMER2_I_SHIFT                              10
#define DAI_INT_24_I_MASK                           0xFFFF83FF
#define DAI_INT_24_I_SHIFT                          10

#define MISCB3_I_MASK                               0xFFF07FFF
#define MISCB3_I_SHIFT                              15
#define FLAG10_I_MASK                               0xFFF07FFF
#define FLAG10_I_SHIFT                              15
#define DAI_INT_25_I_MASK                           0xFFF07FFF
#define DAI_INT_25_I_SHIFT                          15

#define MISCB4_I_MASK                               0xFE0FFFFF
#define MISCB4_I_SHIFT                              20
#define FLAG11_I_MASK                               0xFE0FFFFF
#define FLAG11_I_SHIFT                              20
#define DAI_INT_26_I_MASK                           0xFE0FFFFF
#define DAI_INT_26_I_SHIFT                          20

#define MISCB5_I_MASK                               0xC1FFFFFF
#define MISCB5_I_SHIFT                              25
#define FLAG12_I_MASK                               0xC1FFFFFF
#define FLAG12_I_SHIFT                              25
#define DAI_INT_27_I_MASK                           0xC1FFFFFF
#define DAI_INT_27_I_SHIFT                          25




//------------------------------------------------------------------------------
//  Group E: Bitfield encodings
//------------------------------------------------------------------------------

#define MISCA0_I_DAI_PB01_O_ENCODING                0x00
#define FLAG13_I_DAI_PB01_O_ENCODING                0x00
#define DAI_INT_28_I_DAI_PB01_O_ENCODING            0x00
#define MISCA1_I_DAI_PB01_O_ENCODING                0x00
#define FLAG14_I_DAI_PB01_O_ENCODING                0x00
#define DAI_INT_29_I_DAI_PB01_O_ENCODING            0x00
#define MISCA2_I_DAI_PB01_O_ENCODING                0x00
#define FLAG15_I_DAI_PB01_O_ENCODING                0x00
#define DAI_INT_30_I_DAI_PB01_O_ENCODING            0x00
#define MISCA3_I_DAI_PB01_O_ENCODING                0x00
#define DAI_INT_31_I_DAI_PB01_O_ENCODING            0x00
#define MISCA4_I_DAI_PB01_O_ENCODING                0x00
#define MISCA5_I_DAI_PB01_O_ENCODING                0x00
#define INV_MISCA4_I_DAI_PB01_O_ENCODING            0x00
#define INV_MISCA5_I_DAI_PB01_O_ENCODING            0x00
#define MISCB0_I_DAI_PB01_O_ENCODING                0x00
#define TIMER0_I_DAI_PB01_O_ENCODING                0x00
#define DAI_INT_22_I_DAI_PB01_O_ENCODING            0x00
#define MISCB1_I_DAI_PB01_O_ENCODING                0x00
#define TIMER1_I_DAI_PB01_O_ENCODING                0x00
#define DAI_INT_23_I_DAI_PB01_O_ENCODING            0x00
#define MISCB2_I_DAI_PB01_O_ENCODING                0x00
#define TIMER2_I_DAI_PB01_O_ENCODING                0x00
#define DAI_INT_24_I_DAI_PB01_O_ENCODING            0x00
#define MISCB3_I_DAI_PB01_O_ENCODING                0x00
#define FLAG10_I_DAI_PB01_O_ENCODING                0x00
#define DAI_INT_25_I_DAI_PB01_O_ENCODING            0x00
#define MISCB4_I_DAI_PB01_O_ENCODING                0x00
#define FLAG11_I_DAI_PB01_O_ENCODING                0x00
#define DAI_INT_26_I_DAI_PB01_O_ENCODING            0x00
#define MISCB5_I_DAI_PB01_O_ENCODING                0x00
#define FLAG12_I_DAI_PB01_O_ENCODING                0x00
#define DAI_INT_27_I_DAI_PB01_O_ENCODING            0x00
#define SPIB_DS_I_DAI_PB01_O_ENCODING               0x00
#define SPIB_MOSI_I_DAI_PB01_O_ENCODING             0x00
#define SPIB_MISO_I_DAI_PB01_O_ENCODING             0x00

#define MISCA0_I_DAI_PB02_O_ENCODING                0x01
#define FLAG13_I_DAI_PB02_O_ENCODING                0x01
#define DAI_INT_28_I_DAI_PB02_O_ENCODING            0x01
#define MISCA1_I_DAI_PB02_O_ENCODING                0x01
#define FLAG14_I_DAI_PB02_O_ENCODING                0x01
#define DAI_INT_29_I_DAI_PB02_O_ENCODING            0x01
#define MISCA2_I_DAI_PB02_O_ENCODING                0x01
#define FLAG15_I_DAI_PB02_O_ENCODING                0x01
#define DAI_INT_30_I_DAI_PB02_O_ENCODING            0x01
#define MISCA3_I_DAI_PB02_O_ENCODING                0x01
#define DAI_INT_31_I_DAI_PB02_O_ENCODING            0x01
#define MISCA4_I_DAI_PB02_O_ENCODING                0x01
#define MISCA5_I_DAI_PB02_O_ENCODING                0x01
#define INV_MISCA4_I_DAI_PB02_O_ENCODING            0x01
#define INV_MISCA5_I_DAI_PB02_O_ENCODING            0x01
#define MISCB0_I_DAI_PB02_O_ENCODING                0x01
#define TIMER0_I_DAI_PB02_O_ENCODING                0x01
#define DAI_INT_22_I_DAI_PB02_O_ENCODING            0x01
#define MISCB1_I_DAI_PB02_O_ENCODING                0x01
#define TIMER1_I_DAI_PB02_O_ENCODING                0x01
#define DAI_INT_23_I_DAI_PB02_O_ENCODING            0x01
#define MISCB2_I_DAI_PB02_O_ENCODING                0x01
#define TIMER2_I_DAI_PB02_O_ENCODING                0x01
#define DAI_INT_24_I_DAI_PB02_O_ENCODING            0x01
#define MISCB3_I_DAI_PB02_O_ENCODING                0x01
#define FLAG10_I_DAI_PB02_O_ENCODING                0x01
#define DAI_INT_25_I_DAI_PB02_O_ENCODING            0x01
#define MISCB4_I_DAI_PB02_O_ENCODING                0x01
#define FLAG11_I_DAI_PB02_O_ENCODING                0x01
#define DAI_INT_26_I_DAI_PB02_O_ENCODING            0x01
#define MISCB5_I_DAI_PB02_O_ENCODING                0x01
#define FLAG12_I_DAI_PB02_O_ENCODING                0x01
#define DAI_INT_27_I_DAI_PB02_O_ENCODING            0x01
#define SPIB_DS_I_DAI_PB02_O_ENCODING               0x01
#define SPIB_MOSI_I_DAI_PB02_O_ENCODING             0x01
#define SPIB_MISO_I_DAI_PB02_O_ENCODING             0x01

#define MISCA0_I_DAI_PB03_O_ENCODING                0x02
#define FLAG13_I_DAI_PB03_O_ENCODING                0x02
#define DAI_INT_28_I_DAI_PB03_O_ENCODING            0x02
#define MISCA1_I_DAI_PB03_O_ENCODING                0x02
#define FLAG14_I_DAI_PB03_O_ENCODING                0x02
#define DAI_INT_29_I_DAI_PB03_O_ENCODING            0x02
#define MISCA2_I_DAI_PB03_O_ENCODING                0x02
#define FLAG15_I_DAI_PB03_O_ENCODING                0x02
#define DAI_INT_30_I_DAI_PB03_O_ENCODING            0x02
#define MISCA3_I_DAI_PB03_O_ENCODING                0x02
#define DAI_INT_31_I_DAI_PB03_O_ENCODING            0x02
#define MISCA4_I_DAI_PB03_O_ENCODING                0x02
#define MISCA5_I_DAI_PB03_O_ENCODING                0x02
#define INV_MISCA4_I_DAI_PB03_O_ENCODING            0x02
#define INV_MISCA5_I_DAI_PB03_O_ENCODING            0x02
#define MISCB0_I_DAI_PB03_O_ENCODING                0x02
#define TIMER0_I_DAI_PB03_O_ENCODING                0x02
#define DAI_INT_22_I_DAI_PB03_O_ENCODING            0x02
#define MISCB1_I_DAI_PB03_O_ENCODING                0x02
#define TIMER1_I_DAI_PB03_O_ENCODING                0x02
#define DAI_INT_23_I_DAI_PB03_O_ENCODING            0x02
#define MISCB2_I_DAI_PB03_O_ENCODING                0x02
#define TIMER2_I_DAI_PB03_O_ENCODING                0x02
#define DAI_INT_24_I_DAI_PB03_O_ENCODING            0x02
#define MISCB3_I_DAI_PB03_O_ENCODING                0x02
#define FLAG10_I_DAI_PB03_O_ENCODING                0x02
#define DAI_INT_25_I_DAI_PB03_O_ENCODING            0x02
#define MISCB4_I_DAI_PB03_O_ENCODING                0x02
#define FLAG11_I_DAI_PB03_O_ENCODING                0x02
#define DAI_INT_26_I_DAI_PB03_O_ENCODING            0x02
#define MISCB5_I_DAI_PB03_O_ENCODING                0x02
#define FLAG12_I_DAI_PB03_O_ENCODING                0x02
#define DAI_INT_27_I_DAI_PB03_O_ENCODING            0x02
#define SPIB_DS_I_DAI_PB03_O_ENCODING               0x02
#define SPIB_MOSI_I_DAI_PB03_O_ENCODING             0x02
#define SPIB_MISO_I_DAI_PB03_O_ENCODING             0x02

#define MISCA0_I_DAI_PB04_O_ENCODING                0x03
#define FLAG13_I_DAI_PB04_O_ENCODING                0x03
#define DAI_INT_28_I_DAI_PB04_O_ENCODING            0x03
#define MISCA1_I_DAI_PB04_O_ENCODING                0x03
#define FLAG14_I_DAI_PB04_O_ENCODING                0x03
#define DAI_INT_29_I_DAI_PB04_O_ENCODING            0x03
#define MISCA2_I_DAI_PB04_O_ENCODING                0x03
#define FLAG15_I_DAI_PB04_O_ENCODING                0x03
#define DAI_INT_30_I_DAI_PB04_O_ENCODING            0x03
#define MISCA3_I_DAI_PB04_O_ENCODING                0x03
#define DAI_INT_31_I_DAI_PB04_O_ENCODING            0x03
#define MISCA4_I_DAI_PB04_O_ENCODING                0x03
#define MISCA5_I_DAI_PB04_O_ENCODING                0x03
#define INV_MISCA4_I_DAI_PB04_O_ENCODING            0x03
#define INV_MISCA5_I_DAI_PB04_O_ENCODING            0x03
#define MISCB0_I_DAI_PB04_O_ENCODING                0x03
#define TIMER0_I_DAI_PB04_O_ENCODING                0x03
#define DAI_INT_22_I_DAI_PB04_O_ENCODING            0x03
#define MISCB1_I_DAI_PB04_O_ENCODING                0x03
#define TIMER1_I_DAI_PB04_O_ENCODING                0x03
#define DAI_INT_23_I_DAI_PB04_O_ENCODING            0x03
#define MISCB2_I_DAI_PB04_O_ENCODING                0x03
#define TIMER2_I_DAI_PB04_O_ENCODING                0x03
#define DAI_INT_24_I_DAI_PB04_O_ENCODING            0x03
#define MISCB3_I_DAI_PB04_O_ENCODING                0x03
#define FLAG10_I_DAI_PB04_O_ENCODING                0x03
#define DAI_INT_25_I_DAI_PB04_O_ENCODING            0x03
#define MISCB4_I_DAI_PB04_O_ENCODING                0x03
#define FLAG11_I_DAI_PB04_O_ENCODING                0x03
#define DAI_INT_26_I_DAI_PB04_O_ENCODING            0x03
#define MISCB5_I_DAI_PB04_O_ENCODING                0x03
#define FLAG12_I_DAI_PB04_O_ENCODING                0x03
#define DAI_INT_27_I_DAI_PB04_O_ENCODING            0x03
#define SPIB_DS_I_DAI_PB04_O_ENCODING               0x03
#define SPIB_MOSI_I_DAI_PB04_O_ENCODING             0x03
#define SPIB_MISO_I_DAI_PB04_O_ENCODING             0x03

#define MISCA0_I_DAI_PB05_O_ENCODING                0x04
#define FLAG13_I_DAI_PB05_O_ENCODING                0x04
#define DAI_INT_28_I_DAI_PB05_O_ENCODING            0x04
#define MISCA1_I_DAI_PB05_O_ENCODING                0x04
#define FLAG14_I_DAI_PB05_O_ENCODING                0x04
#define DAI_INT_29_I_DAI_PB05_O_ENCODING            0x04
#define MISCA2_I_DAI_PB05_O_ENCODING                0x04
#define FLAG15_I_DAI_PB05_O_ENCODING                0x04
#define DAI_INT_30_I_DAI_PB05_O_ENCODING            0x04
#define MISCA3_I_DAI_PB05_O_ENCODING                0x04
#define DAI_INT_31_I_DAI_PB05_O_ENCODING            0x04
#define MISCA4_I_DAI_PB05_O_ENCODING                0x04
#define MISCA5_I_DAI_PB05_O_ENCODING                0x04
#define INV_MISCA4_I_DAI_PB05_O_ENCODING            0x04
#define INV_MISCA5_I_DAI_PB05_O_ENCODING            0x04
#define MISCB0_I_DAI_PB05_O_ENCODING                0x04
#define TIMER0_I_DAI_PB05_O_ENCODING                0x04
#define DAI_INT_22_I_DAI_PB05_O_ENCODING            0x04
#define MISCB1_I_DAI_PB05_O_ENCODING                0x04
#define TIMER1_I_DAI_PB05_O_ENCODING                0x04
#define DAI_INT_23_I_DAI_PB05_O_ENCODING            0x04
#define MISCB2_I_DAI_PB05_O_ENCODING                0x04
#define TIMER2_I_DAI_PB05_O_ENCODING                0x04
#define DAI_INT_24_I_DAI_PB05_O_ENCODING            0x04
#define MISCB3_I_DAI_PB05_O_ENCODING                0x04
#define FLAG10_I_DAI_PB05_O_ENCODING                0x04
#define DAI_INT_25_I_DAI_PB05_O_ENCODING            0x04
#define MISCB4_I_DAI_PB05_O_ENCODING                0x04
#define FLAG11_I_DAI_PB05_O_ENCODING                0x04
#define DAI_INT_26_I_DAI_PB05_O_ENCODING            0x04
#define MISCB5_I_DAI_PB05_O_ENCODING                0x04
#define FLAG12_I_DAI_PB05_O_ENCODING                0x04
#define DAI_INT_27_I_DAI_PB05_O_ENCODING            0x04
#define SPIB_DS_I_DAI_PB05_O_ENCODING               0x04
#define SPIB_MOSI_I_DAI_PB05_O_ENCODING             0x04
#define SPIB_MISO_I_DAI_PB05_O_ENCODING             0x04

#define MISCA0_I_DAI_PB06_O_ENCODING                0x05
#define FLAG13_I_DAI_PB06_O_ENCODING                0x05
#define DAI_INT_28_I_DAI_PB06_O_ENCODING            0x05
#define MISCA1_I_DAI_PB06_O_ENCODING                0x05
#define FLAG14_I_DAI_PB06_O_ENCODING                0x05
#define DAI_INT_29_I_DAI_PB06_O_ENCODING            0x05
#define MISCA2_I_DAI_PB06_O_ENCODING                0x05
#define FLAG15_I_DAI_PB06_O_ENCODING                0x05
#define DAI_INT_30_I_DAI_PB06_O_ENCODING            0x05
#define MISCA3_I_DAI_PB06_O_ENCODING                0x05
#define DAI_INT_31_I_DAI_PB06_O_ENCODING            0x05
#define MISCA4_I_DAI_PB06_O_ENCODING                0x05
#define MISCA5_I_DAI_PB06_O_ENCODING                0x05
#define INV_MISCA4_I_DAI_PB06_O_ENCODING            0x05
#define INV_MISCA5_I_DAI_PB06_O_ENCODING            0x05
#define MISCB0_I_DAI_PB06_O_ENCODING                0x05
#define TIMER0_I_DAI_PB06_O_ENCODING                0x05
#define DAI_INT_22_I_DAI_PB06_O_ENCODING            0x05
#define MISCB1_I_DAI_PB06_O_ENCODING                0x05
#define TIMER1_I_DAI_PB06_O_ENCODING                0x05
#define DAI_INT_23_I_DAI_PB06_O_ENCODING            0x05
#define MISCB2_I_DAI_PB06_O_ENCODING                0x05
#define TIMER2_I_DAI_PB06_O_ENCODING                0x05
#define DAI_INT_24_I_DAI_PB06_O_ENCODING            0x05
#define MISCB3_I_DAI_PB06_O_ENCODING                0x05
#define FLAG10_I_DAI_PB06_O_ENCODING                0x05
#define DAI_INT_25_I_DAI_PB06_O_ENCODING            0x05
#define MISCB4_I_DAI_PB06_O_ENCODING                0x05
#define FLAG11_I_DAI_PB06_O_ENCODING                0x05
#define DAI_INT_26_I_DAI_PB06_O_ENCODING            0x05
#define MISCB5_I_DAI_PB06_O_ENCODING                0x05
#define FLAG12_I_DAI_PB06_O_ENCODING                0x05
#define DAI_INT_27_I_DAI_PB06_O_ENCODING            0x05
#define SPIB_DS_I_DAI_PB06_O_ENCODING               0x05
#define SPIB_MOSI_I_DAI_PB06_O_ENCODING             0x05
#define SPIB_MISO_I_DAI_PB06_O_ENCODING             0x05

#define MISCA0_I_DAI_PB07_O_ENCODING                0x06
#define FLAG13_I_DAI_PB07_O_ENCODING                0x06
#define DAI_INT_28_I_DAI_PB07_O_ENCODING            0x06
#define MISCA1_I_DAI_PB07_O_ENCODING                0x06
#define FLAG14_I_DAI_PB07_O_ENCODING                0x06
#define DAI_INT_29_I_DAI_PB07_O_ENCODING            0x06
#define MISCA2_I_DAI_PB07_O_ENCODING                0x06
#define FLAG15_I_DAI_PB07_O_ENCODING                0x06
#define DAI_INT_30_I_DAI_PB07_O_ENCODING            0x06
#define MISCA3_I_DAI_PB07_O_ENCODING                0x06
#define DAI_INT_31_I_DAI_PB07_O_ENCODING            0x06
#define MISCA4_I_DAI_PB07_O_ENCODING                0x06
#define MISCA5_I_DAI_PB07_O_ENCODING                0x06
#define INV_MISCA4_I_DAI_PB07_O_ENCODING            0x06
#define INV_MISCA5_I_DAI_PB07_O_ENCODING            0x06
#define MISCB0_I_DAI_PB07_O_ENCODING                0x06
#define TIMER0_I_DAI_PB07_O_ENCODING                0x06
#define DAI_INT_22_I_DAI_PB07_O_ENCODING            0x06
#define MISCB1_I_DAI_PB07_O_ENCODING                0x06
#define TIMER1_I_DAI_PB07_O_ENCODING                0x06
#define DAI_INT_23_I_DAI_PB07_O_ENCODING            0x06
#define MISCB2_I_DAI_PB07_O_ENCODING                0x06
#define TIMER2_I_DAI_PB07_O_ENCODING                0x06
#define DAI_INT_24_I_DAI_PB07_O_ENCODING            0x06
#define MISCB3_I_DAI_PB07_O_ENCODING                0x06
#define FLAG10_I_DAI_PB07_O_ENCODING                0x06
#define DAI_INT_25_I_DAI_PB07_O_ENCODING            0x06
#define MISCB4_I_DAI_PB07_O_ENCODING                0x06
#define FLAG11_I_DAI_PB07_O_ENCODING                0x06
#define DAI_INT_26_I_DAI_PB07_O_ENCODING            0x06
#define MISCB5_I_DAI_PB07_O_ENCODING                0x06
#define FLAG12_I_DAI_PB07_O_ENCODING                0x06
#define DAI_INT_27_I_DAI_PB07_O_ENCODING            0x06
#define SPIB_DS_I_DAI_PB07_O_ENCODING               0x06
#define SPIB_MOSI_I_DAI_PB07_O_ENCODING             0x06
#define SPIB_MISO_I_DAI_PB07_O_ENCODING             0x06

#define MISCA0_I_DAI_PB08_O_ENCODING                0x07
#define FLAG13_I_DAI_PB08_O_ENCODING                0x07
#define DAI_INT_28_I_DAI_PB08_O_ENCODING            0x07
#define MISCA1_I_DAI_PB08_O_ENCODING                0x07
#define FLAG14_I_DAI_PB08_O_ENCODING                0x07
#define DAI_INT_29_I_DAI_PB08_O_ENCODING            0x07
#define MISCA2_I_DAI_PB08_O_ENCODING                0x07
#define FLAG15_I_DAI_PB08_O_ENCODING                0x07
#define DAI_INT_30_I_DAI_PB08_O_ENCODING            0x07
#define MISCA3_I_DAI_PB08_O_ENCODING                0x07
#define DAI_INT_31_I_DAI_PB08_O_ENCODING            0x07
#define MISCA4_I_DAI_PB08_O_ENCODING                0x07
#define MISCA5_I_DAI_PB08_O_ENCODING                0x07
#define INV_MISCA4_I_DAI_PB08_O_ENCODING            0x07
#define INV_MISCA5_I_DAI_PB08_O_ENCODING            0x07
#define MISCB0_I_DAI_PB08_O_ENCODING                0x07
#define TIMER0_I_DAI_PB08_O_ENCODING                0x07
#define DAI_INT_22_I_DAI_PB08_O_ENCODING            0x07
#define MISCB1_I_DAI_PB08_O_ENCODING                0x07
#define TIMER1_I_DAI_PB08_O_ENCODING                0x07
#define DAI_INT_23_I_DAI_PB08_O_ENCODING            0x07
#define MISCB2_I_DAI_PB08_O_ENCODING                0x07
#define TIMER2_I_DAI_PB08_O_ENCODING                0x07
#define DAI_INT_24_I_DAI_PB08_O_ENCODING            0x07
#define MISCB3_I_DAI_PB08_O_ENCODING                0x07
#define FLAG10_I_DAI_PB08_O_ENCODING                0x07
#define DAI_INT_25_I_DAI_PB08_O_ENCODING            0x07
#define MISCB4_I_DAI_PB08_O_ENCODING                0x07
#define FLAG11_I_DAI_PB08_O_ENCODING                0x07
#define DAI_INT_26_I_DAI_PB08_O_ENCODING            0x07
#define MISCB5_I_DAI_PB08_O_ENCODING                0x07
#define FLAG12_I_DAI_PB08_O_ENCODING                0x07
#define DAI_INT_27_I_DAI_PB08_O_ENCODING            0x07
#define SPIB_DS_I_DAI_PB08_O_ENCODING               0x07
#define SPIB_MOSI_I_DAI_PB08_O_ENCODING             0x07
#define SPIB_MISO_I_DAI_PB08_O_ENCODING             0x07

#define MISCA0_I_DAI_PB09_O_ENCODING                0x08
#define FLAG13_I_DAI_PB09_O_ENCODING                0x08
#define DAI_INT_28_I_DAI_PB09_O_ENCODING            0x08
#define MISCA1_I_DAI_PB09_O_ENCODING                0x08
#define FLAG14_I_DAI_PB09_O_ENCODING                0x08
#define DAI_INT_29_I_DAI_PB09_O_ENCODING            0x08
#define MISCA2_I_DAI_PB09_O_ENCODING                0x08
#define FLAG15_I_DAI_PB09_O_ENCODING                0x08
#define DAI_INT_30_I_DAI_PB09_O_ENCODING            0x08
#define MISCA3_I_DAI_PB09_O_ENCODING                0x08
#define DAI_INT_31_I_DAI_PB09_O_ENCODING            0x08
#define MISCA4_I_DAI_PB09_O_ENCODING                0x08
#define MISCA5_I_DAI_PB09_O_ENCODING                0x08
#define INV_MISCA4_I_DAI_PB09_O_ENCODING            0x08
#define INV_MISCA5_I_DAI_PB09_O_ENCODING            0x08
#define MISCB0_I_DAI_PB09_O_ENCODING                0x08
#define TIMER0_I_DAI_PB09_O_ENCODING                0x08
#define DAI_INT_22_I_DAI_PB09_O_ENCODING            0x08
#define MISCB1_I_DAI_PB09_O_ENCODING                0x08
#define TIMER1_I_DAI_PB09_O_ENCODING                0x08
#define DAI_INT_23_I_DAI_PB09_O_ENCODING            0x08
#define MISCB2_I_DAI_PB09_O_ENCODING                0x08
#define TIMER2_I_DAI_PB09_O_ENCODING                0x08
#define DAI_INT_24_I_DAI_PB09_O_ENCODING            0x08
#define MISCB3_I_DAI_PB09_O_ENCODING                0x08
#define FLAG10_I_DAI_PB09_O_ENCODING                0x08
#define DAI_INT_25_I_DAI_PB09_O_ENCODING            0x08
#define MISCB4_I_DAI_PB09_O_ENCODING                0x08
#define FLAG11_I_DAI_PB09_O_ENCODING                0x08
#define DAI_INT_26_I_DAI_PB09_O_ENCODING            0x08
#define MISCB5_I_DAI_PB09_O_ENCODING                0x08
#define FLAG12_I_DAI_PB09_O_ENCODING                0x08
#define DAI_INT_27_I_DAI_PB09_O_ENCODING            0x08
#define SPIB_DS_I_DAI_PB09_O_ENCODING               0x08
#define SPIB_MOSI_I_DAI_PB09_O_ENCODING             0x08
#define SPIB_MISO_I_DAI_PB09_O_ENCODING             0x08

#define MISCA0_I_DAI_PB10_O_ENCODING                0x09
#define FLAG13_I_DAI_PB10_O_ENCODING                0x09
#define DAI_INT_28_I_DAI_PB10_O_ENCODING            0x09
#define MISCA1_I_DAI_PB10_O_ENCODING                0x09
#define FLAG14_I_DAI_PB10_O_ENCODING                0x09
#define DAI_INT_29_I_DAI_PB10_O_ENCODING            0x09
#define MISCA2_I_DAI_PB10_O_ENCODING                0x09
#define FLAG15_I_DAI_PB10_O_ENCODING                0x09
#define DAI_INT_30_I_DAI_PB10_O_ENCODING            0x09
#define MISCA3_I_DAI_PB10_O_ENCODING                0x09
#define DAI_INT_31_I_DAI_PB10_O_ENCODING            0x09
#define MISCA4_I_DAI_PB10_O_ENCODING                0x09
#define MISCA5_I_DAI_PB10_O_ENCODING                0x09
#define INV_MISCA4_I_DAI_PB10_O_ENCODING            0x09
#define INV_MISCA5_I_DAI_PB10_O_ENCODING            0x09
#define MISCB0_I_DAI_PB10_O_ENCODING                0x09
#define TIMER0_I_DAI_PB10_O_ENCODING                0x09
#define DAI_INT_22_I_DAI_PB10_O_ENCODING            0x09
#define MISCB1_I_DAI_PB10_O_ENCODING                0x09
#define TIMER1_I_DAI_PB10_O_ENCODING                0x09
#define DAI_INT_23_I_DAI_PB10_O_ENCODING            0x09
#define MISCB2_I_DAI_PB10_O_ENCODING                0x09
#define TIMER2_I_DAI_PB10_O_ENCODING                0x09
#define DAI_INT_24_I_DAI_PB10_O_ENCODING            0x09
#define MISCB3_I_DAI_PB10_O_ENCODING                0x09
#define FLAG10_I_DAI_PB10_O_ENCODING                0x09
#define DAI_INT_25_I_DAI_PB10_O_ENCODING            0x09
#define MISCB4_I_DAI_PB10_O_ENCODING                0x09
#define FLAG11_I_DAI_PB10_O_ENCODING                0x09
#define DAI_INT_26_I_DAI_PB10_O_ENCODING            0x09
#define MISCB5_I_DAI_PB10_O_ENCODING                0x09
#define FLAG12_I_DAI_PB10_O_ENCODING                0x09
#define DAI_INT_27_I_DAI_PB10_O_ENCODING            0x09
#define SPIB_DS_I_DAI_PB10_O_ENCODING               0x09
#define SPIB_MOSI_I_DAI_PB10_O_ENCODING             0x09
#define SPIB_MISO_I_DAI_PB10_O_ENCODING             0x09

#define MISCA0_I_DAI_PB11_O_ENCODING                0x0A
#define FLAG13_I_DAI_PB11_O_ENCODING                0x0A
#define DAI_INT_28_I_DAI_PB11_O_ENCODING            0x0A
#define MISCA1_I_DAI_PB11_O_ENCODING                0x0A
#define FLAG14_I_DAI_PB11_O_ENCODING                0x0A
#define DAI_INT_29_I_DAI_PB11_O_ENCODING            0x0A
#define MISCA2_I_DAI_PB11_O_ENCODING                0x0A
#define FLAG15_I_DAI_PB11_O_ENCODING                0x0A
#define DAI_INT_30_I_DAI_PB11_O_ENCODING            0x0A
#define MISCA3_I_DAI_PB11_O_ENCODING                0x0A
#define DAI_INT_31_I_DAI_PB11_O_ENCODING            0x0A
#define MISCA4_I_DAI_PB11_O_ENCODING                0x0A
#define MISCA5_I_DAI_PB11_O_ENCODING                0x0A
#define INV_MISCA4_I_DAI_PB11_O_ENCODING            0x0A
#define INV_MISCA5_I_DAI_PB11_O_ENCODING            0x0A
#define MISCB0_I_DAI_PB11_O_ENCODING                0x0A
#define TIMER0_I_DAI_PB11_O_ENCODING                0x0A
#define DAI_INT_22_I_DAI_PB11_O_ENCODING            0x0A
#define MISCB1_I_DAI_PB11_O_ENCODING                0x0A
#define TIMER1_I_DAI_PB11_O_ENCODING                0x0A
#define DAI_INT_23_I_DAI_PB11_O_ENCODING            0x0A
#define MISCB2_I_DAI_PB11_O_ENCODING                0x0A
#define TIMER2_I_DAI_PB11_O_ENCODING                0x0A
#define DAI_INT_24_I_DAI_PB11_O_ENCODING            0x0A
#define MISCB3_I_DAI_PB11_O_ENCODING                0x0A
#define FLAG10_I_DAI_PB11_O_ENCODING                0x0A
#define DAI_INT_25_I_DAI_PB11_O_ENCODING            0x0A
#define MISCB4_I_DAI_PB11_O_ENCODING                0x0A
#define FLAG11_I_DAI_PB11_O_ENCODING                0x0A
#define DAI_INT_26_I_DAI_PB11_O_ENCODING            0x0A
#define MISCB5_I_DAI_PB11_O_ENCODING                0x0A
#define FLAG12_I_DAI_PB11_O_ENCODING                0x0A
#define DAI_INT_27_I_DAI_PB11_O_ENCODING            0x0A
#define SPIB_DS_I_DAI_PB11_O_ENCODING               0x0A
#define SPIB_MOSI_I_DAI_PB11_O_ENCODING             0x0A
#define SPIB_MISO_I_DAI_PB11_O_ENCODING             0x0A

#define MISCA0_I_DAI_PB12_O_ENCODING                0x0B
#define FLAG13_I_DAI_PB12_O_ENCODING                0x0B
#define DAI_INT_28_I_DAI_PB12_O_ENCODING            0x0B
#define MISCA1_I_DAI_PB12_O_ENCODING                0x0B
#define FLAG14_I_DAI_PB12_O_ENCODING                0x0B
#define DAI_INT_29_I_DAI_PB12_O_ENCODING            0x0B
#define MISCA2_I_DAI_PB12_O_ENCODING                0x0B
#define FLAG15_I_DAI_PB12_O_ENCODING                0x0B
#define DAI_INT_30_I_DAI_PB12_O_ENCODING            0x0B
#define MISCA3_I_DAI_PB12_O_ENCODING                0x0B
#define DAI_INT_31_I_DAI_PB12_O_ENCODING            0x0B
#define MISCA4_I_DAI_PB12_O_ENCODING                0x0B
#define MISCA5_I_DAI_PB12_O_ENCODING                0x0B
#define INV_MISCA4_I_DAI_PB12_O_ENCODING            0x0B
#define INV_MISCA5_I_DAI_PB12_O_ENCODING            0x0B
#define MISCB0_I_DAI_PB12_O_ENCODING                0x0B
#define TIMER0_I_DAI_PB12_O_ENCODING                0x0B
#define DAI_INT_22_I_DAI_PB12_O_ENCODING            0x0B
#define MISCB1_I_DAI_PB12_O_ENCODING                0x0B
#define TIMER1_I_DAI_PB12_O_ENCODING                0x0B
#define DAI_INT_23_I_DAI_PB12_O_ENCODING            0x0B
#define MISCB2_I_DAI_PB12_O_ENCODING                0x0B
#define TIMER2_I_DAI_PB12_O_ENCODING                0x0B
#define DAI_INT_24_I_DAI_PB12_O_ENCODING            0x0B
#define MISCB3_I_DAI_PB12_O_ENCODING                0x0B
#define FLAG10_I_DAI_PB12_O_ENCODING                0x0B
#define DAI_INT_25_I_DAI_PB12_O_ENCODING            0x0B
#define MISCB4_I_DAI_PB12_O_ENCODING                0x0B
#define FLAG11_I_DAI_PB12_O_ENCODING                0x0B
#define DAI_INT_26_I_DAI_PB12_O_ENCODING            0x0B
#define MISCB5_I_DAI_PB12_O_ENCODING                0x0B
#define FLAG12_I_DAI_PB12_O_ENCODING                0x0B
#define DAI_INT_27_I_DAI_PB12_O_ENCODING            0x0B
#define SPIB_DS_I_DAI_PB12_O_ENCODING               0x0B
#define SPIB_MOSI_I_DAI_PB12_O_ENCODING             0x0B
#define SPIB_MISO_I_DAI_PB12_O_ENCODING             0x0B

#define MISCA0_I_DAI_PB13_O_ENCODING                0x0C
#define FLAG13_I_DAI_PB13_O_ENCODING                0x0C
#define DAI_INT_28_I_DAI_PB13_O_ENCODING            0x0C
#define MISCA1_I_DAI_PB13_O_ENCODING                0x0C
#define FLAG14_I_DAI_PB13_O_ENCODING                0x0C
#define DAI_INT_29_I_DAI_PB13_O_ENCODING            0x0C
#define MISCA2_I_DAI_PB13_O_ENCODING                0x0C
#define FLAG15_I_DAI_PB13_O_ENCODING                0x0C
#define DAI_INT_30_I_DAI_PB13_O_ENCODING            0x0C
#define MISCA3_I_DAI_PB13_O_ENCODING                0x0C
#define DAI_INT_31_I_DAI_PB13_O_ENCODING            0x0C
#define MISCA4_I_DAI_PB13_O_ENCODING                0x0C
#define MISCA5_I_DAI_PB13_O_ENCODING                0x0C
#define INV_MISCA4_I_DAI_PB13_O_ENCODING            0x0C
#define INV_MISCA5_I_DAI_PB13_O_ENCODING            0x0C
#define MISCB0_I_DAI_PB13_O_ENCODING                0x0C
#define TIMER0_I_DAI_PB13_O_ENCODING                0x0C
#define DAI_INT_22_I_DAI_PB13_O_ENCODING            0x0C
#define MISCB1_I_DAI_PB13_O_ENCODING                0x0C
#define TIMER1_I_DAI_PB13_O_ENCODING                0x0C
#define DAI_INT_23_I_DAI_PB13_O_ENCODING            0x0C
#define MISCB2_I_DAI_PB13_O_ENCODING                0x0C
#define TIMER2_I_DAI_PB13_O_ENCODING                0x0C
#define DAI_INT_24_I_DAI_PB13_O_ENCODING            0x0C
#define MISCB3_I_DAI_PB13_O_ENCODING                0x0C
#define FLAG10_I_DAI_PB13_O_ENCODING                0x0C
#define DAI_INT_25_I_DAI_PB13_O_ENCODING            0x0C
#define MISCB4_I_DAI_PB13_O_ENCODING                0x0C
#define FLAG11_I_DAI_PB13_O_ENCODING                0x0C
#define DAI_INT_26_I_DAI_PB13_O_ENCODING            0x0C
#define MISCB5_I_DAI_PB13_O_ENCODING                0x0C
#define FLAG12_I_DAI_PB13_O_ENCODING                0x0C
#define DAI_INT_27_I_DAI_PB13_O_ENCODING            0x0C
#define SPIB_DS_I_DAI_PB13_O_ENCODING               0x0C
#define SPIB_MOSI_I_DAI_PB13_O_ENCODING             0x0C
#define SPIB_MISO_I_DAI_PB13_O_ENCODING             0x0C

#define MISCA0_I_DAI_PB14_O_ENCODING                0x0D
#define FLAG13_I_DAI_PB14_O_ENCODING                0x0D
#define DAI_INT_28_I_DAI_PB14_O_ENCODING            0x0D
#define MISCA1_I_DAI_PB14_O_ENCODING                0x0D
#define FLAG14_I_DAI_PB14_O_ENCODING                0x0D
#define DAI_INT_29_I_DAI_PB14_O_ENCODING            0x0D
#define MISCA2_I_DAI_PB14_O_ENCODING                0x0D
#define FLAG15_I_DAI_PB14_O_ENCODING                0x0D
#define DAI_INT_30_I_DAI_PB14_O_ENCODING            0x0D
#define MISCA3_I_DAI_PB14_O_ENCODING                0x0D
#define DAI_INT_31_I_DAI_PB14_O_ENCODING            0x0D
#define MISCA4_I_DAI_PB14_O_ENCODING                0x0D
#define MISCA5_I_DAI_PB14_O_ENCODING                0x0D
#define INV_MISCA4_I_DAI_PB14_O_ENCODING            0x0D
#define INV_MISCA5_I_DAI_PB14_O_ENCODING            0x0D
#define MISCB0_I_DAI_PB14_O_ENCODING                0x0D
#define TIMER0_I_DAI_PB14_O_ENCODING                0x0D
#define DAI_INT_22_I_DAI_PB14_O_ENCODING            0x0D
#define MISCB1_I_DAI_PB14_O_ENCODING                0x0D
#define TIMER1_I_DAI_PB14_O_ENCODING                0x0D
#define DAI_INT_23_I_DAI_PB14_O_ENCODING            0x0D
#define MISCB2_I_DAI_PB14_O_ENCODING                0x0D
#define TIMER2_I_DAI_PB14_O_ENCODING                0x0D
#define DAI_INT_24_I_DAI_PB14_O_ENCODING            0x0D
#define MISCB3_I_DAI_PB14_O_ENCODING                0x0D
#define FLAG10_I_DAI_PB14_O_ENCODING                0x0D
#define DAI_INT_25_I_DAI_PB14_O_ENCODING            0x0D
#define MISCB4_I_DAI_PB14_O_ENCODING                0x0D
#define FLAG11_I_DAI_PB14_O_ENCODING                0x0D
#define DAI_INT_26_I_DAI_PB14_O_ENCODING            0x0D
#define MISCB5_I_DAI_PB14_O_ENCODING                0x0D
#define FLAG12_I_DAI_PB14_O_ENCODING                0x0D
#define DAI_INT_27_I_DAI_PB14_O_ENCODING            0x0D
#define SPIB_DS_I_DAI_PB14_O_ENCODING               0x0D
#define SPIB_MOSI_I_DAI_PB14_O_ENCODING             0x0D
#define SPIB_MISO_I_DAI_PB14_O_ENCODING             0x0D

#define MISCA0_I_DAI_PB15_O_ENCODING                0x0E
#define FLAG13_I_DAI_PB15_O_ENCODING                0x0E
#define DAI_INT_28_I_DAI_PB15_O_ENCODING            0x0E
#define MISCA1_I_DAI_PB15_O_ENCODING                0x0E
#define FLAG14_I_DAI_PB15_O_ENCODING                0x0E
#define DAI_INT_29_I_DAI_PB15_O_ENCODING            0x0E
#define MISCA2_I_DAI_PB15_O_ENCODING                0x0E
#define FLAG15_I_DAI_PB15_O_ENCODING                0x0E
#define DAI_INT_30_I_DAI_PB15_O_ENCODING            0x0E
#define MISCA3_I_DAI_PB15_O_ENCODING                0x0E
#define DAI_INT_31_I_DAI_PB15_O_ENCODING            0x0E
#define MISCA4_I_DAI_PB15_O_ENCODING                0x0E
#define MISCA5_I_DAI_PB15_O_ENCODING                0x0E
#define INV_MISCA4_I_DAI_PB15_O_ENCODING            0x0E
#define INV_MISCA5_I_DAI_PB15_O_ENCODING            0x0E
#define MISCB0_I_DAI_PB15_O_ENCODING                0x0E
#define TIMER0_I_DAI_PB15_O_ENCODING                0x0E
#define DAI_INT_22_I_DAI_PB15_O_ENCODING            0x0E
#define MISCB1_I_DAI_PB15_O_ENCODING                0x0E
#define TIMER1_I_DAI_PB15_O_ENCODING                0x0E
#define DAI_INT_23_I_DAI_PB15_O_ENCODING            0x0E
#define MISCB2_I_DAI_PB15_O_ENCODING                0x0E
#define TIMER2_I_DAI_PB15_O_ENCODING                0x0E
#define DAI_INT_24_I_DAI_PB15_O_ENCODING            0x0E
#define MISCB3_I_DAI_PB15_O_ENCODING                0x0E
#define FLAG10_I_DAI_PB15_O_ENCODING                0x0E
#define DAI_INT_25_I_DAI_PB15_O_ENCODING            0x0E
#define MISCB4_I_DAI_PB15_O_ENCODING                0x0E
#define FLAG11_I_DAI_PB15_O_ENCODING                0x0E
#define DAI_INT_26_I_DAI_PB15_O_ENCODING            0x0E
#define MISCB5_I_DAI_PB15_O_ENCODING                0x0E
#define FLAG12_I_DAI_PB15_O_ENCODING                0x0E
#define DAI_INT_27_I_DAI_PB15_O_ENCODING            0x0E
#define SPIB_DS_I_DAI_PB15_O_ENCODING               0x0E
#define SPIB_MOSI_I_DAI_PB15_O_ENCODING             0x0E
#define SPIB_MISO_I_DAI_PB15_O_ENCODING             0x0E

#define MISCA0_I_DAI_PB16_O_ENCODING                0x0F
#define FLAG13_I_DAI_PB16_O_ENCODING                0x0F
#define DAI_INT_28_I_DAI_PB16_O_ENCODING            0x0F
#define MISCA1_I_DAI_PB16_O_ENCODING                0x0F
#define FLAG14_I_DAI_PB16_O_ENCODING                0x0F
#define DAI_INT_29_I_DAI_PB16_O_ENCODING            0x0F
#define MISCA2_I_DAI_PB16_O_ENCODING                0x0F
#define FLAG15_I_DAI_PB16_O_ENCODING                0x0F
#define DAI_INT_30_I_DAI_PB16_O_ENCODING            0x0F
#define MISCA3_I_DAI_PB16_O_ENCODING                0x0F
#define DAI_INT_31_I_DAI_PB16_O_ENCODING            0x0F
#define MISCA4_I_DAI_PB16_O_ENCODING                0x0F
#define MISCA5_I_DAI_PB16_O_ENCODING                0x0F
#define INV_MISCA4_I_DAI_PB16_O_ENCODING            0x0F
#define INV_MISCA5_I_DAI_PB16_O_ENCODING            0x0F
#define MISCB0_I_DAI_PB16_O_ENCODING                0x0F
#define TIMER0_I_DAI_PB16_O_ENCODING                0x0F
#define DAI_INT_22_I_DAI_PB16_O_ENCODING            0x0F
#define MISCB1_I_DAI_PB16_O_ENCODING                0x0F
#define TIMER1_I_DAI_PB16_O_ENCODING                0x0F
#define DAI_INT_23_I_DAI_PB16_O_ENCODING            0x0F
#define MISCB2_I_DAI_PB16_O_ENCODING                0x0F
#define TIMER2_I_DAI_PB16_O_ENCODING                0x0F
#define DAI_INT_24_I_DAI_PB16_O_ENCODING            0x0F
#define MISCB3_I_DAI_PB16_O_ENCODING                0x0F
#define FLAG10_I_DAI_PB16_O_ENCODING                0x0F
#define DAI_INT_25_I_DAI_PB16_O_ENCODING            0x0F
#define MISCB4_I_DAI_PB16_O_ENCODING                0x0F
#define FLAG11_I_DAI_PB16_O_ENCODING                0x0F
#define DAI_INT_26_I_DAI_PB16_O_ENCODING            0x0F
#define MISCB5_I_DAI_PB16_O_ENCODING                0x0F
#define FLAG12_I_DAI_PB16_O_ENCODING                0x0F
#define DAI_INT_27_I_DAI_PB16_O_ENCODING            0x0F
#define SPIB_DS_I_DAI_PB16_O_ENCODING               0x0F
#define SPIB_MOSI_I_DAI_PB16_O_ENCODING             0x0F
#define SPIB_MISO_I_DAI_PB16_O_ENCODING             0x0F

#define MISCA0_I_DAI_PB17_O_ENCODING                0x10
#define FLAG13_I_DAI_PB17_O_ENCODING                0x10
#define DAI_INT_28_I_DAI_PB17_O_ENCODING            0x10
#define MISCA1_I_DAI_PB17_O_ENCODING                0x10
#define FLAG14_I_DAI_PB17_O_ENCODING                0x10
#define DAI_INT_29_I_DAI_PB17_O_ENCODING            0x10
#define MISCA2_I_DAI_PB17_O_ENCODING                0x10
#define FLAG15_I_DAI_PB17_O_ENCODING                0x10
#define DAI_INT_30_I_DAI_PB17_O_ENCODING            0x10
#define MISCA3_I_DAI_PB17_O_ENCODING                0x10
#define DAI_INT_31_I_DAI_PB17_O_ENCODING            0x10
#define MISCA4_I_DAI_PB17_O_ENCODING                0x10
#define MISCA5_I_DAI_PB17_O_ENCODING                0x10
#define INV_MISCA4_I_DAI_PB17_O_ENCODING            0x10
#define INV_MISCA5_I_DAI_PB17_O_ENCODING            0x10
#define MISCB0_I_DAI_PB17_O_ENCODING                0x10
#define TIMER0_I_DAI_PB17_O_ENCODING                0x10
#define DAI_INT_22_I_DAI_PB17_O_ENCODING            0x10
#define MISCB1_I_DAI_PB17_O_ENCODING                0x10
#define TIMER1_I_DAI_PB17_O_ENCODING                0x10
#define DAI_INT_23_I_DAI_PB17_O_ENCODING            0x10
#define MISCB2_I_DAI_PB17_O_ENCODING                0x10
#define TIMER2_I_DAI_PB17_O_ENCODING                0x10
#define DAI_INT_24_I_DAI_PB17_O_ENCODING            0x10
#define MISCB3_I_DAI_PB17_O_ENCODING                0x10
#define FLAG10_I_DAI_PB17_O_ENCODING                0x10
#define DAI_INT_25_I_DAI_PB17_O_ENCODING            0x10
#define MISCB4_I_DAI_PB17_O_ENCODING                0x10
#define FLAG11_I_DAI_PB17_O_ENCODING                0x10
#define DAI_INT_26_I_DAI_PB17_O_ENCODING            0x10
#define MISCB5_I_DAI_PB17_O_ENCODING                0x10
#define FLAG12_I_DAI_PB17_O_ENCODING                0x10
#define DAI_INT_27_I_DAI_PB17_O_ENCODING            0x10
#define SPIB_DS_I_DAI_PB17_O_ENCODING               0x10
#define SPIB_MOSI_I_DAI_PB17_O_ENCODING             0x10
#define SPIB_MISO_I_DAI_PB17_O_ENCODING             0x10

#define MISCA0_I_DAI_PB18_O_ENCODING                0x11
#define FLAG13_I_DAI_PB18_O_ENCODING                0x11
#define DAI_INT_28_I_DAI_PB18_O_ENCODING            0x11
#define MISCA1_I_DAI_PB18_O_ENCODING                0x11
#define FLAG14_I_DAI_PB18_O_ENCODING                0x11
#define DAI_INT_29_I_DAI_PB18_O_ENCODING            0x11
#define MISCA2_I_DAI_PB18_O_ENCODING                0x11
#define FLAG15_I_DAI_PB18_O_ENCODING                0x11
#define DAI_INT_30_I_DAI_PB18_O_ENCODING            0x11
#define MISCA3_I_DAI_PB18_O_ENCODING                0x11
#define DAI_INT_31_I_DAI_PB18_O_ENCODING            0x11
#define MISCA4_I_DAI_PB18_O_ENCODING                0x11
#define MISCA5_I_DAI_PB18_O_ENCODING                0x11
#define INV_MISCA4_I_DAI_PB18_O_ENCODING            0x11
#define INV_MISCA5_I_DAI_PB18_O_ENCODING            0x11
#define MISCB0_I_DAI_PB18_O_ENCODING                0x11
#define TIMER0_I_DAI_PB18_O_ENCODING                0x11
#define DAI_INT_22_I_DAI_PB18_O_ENCODING            0x11
#define MISCB1_I_DAI_PB18_O_ENCODING                0x11
#define TIMER1_I_DAI_PB18_O_ENCODING                0x11
#define DAI_INT_23_I_DAI_PB18_O_ENCODING            0x11
#define MISCB2_I_DAI_PB18_O_ENCODING                0x11
#define TIMER2_I_DAI_PB18_O_ENCODING                0x11
#define DAI_INT_24_I_DAI_PB18_O_ENCODING            0x11
#define MISCB3_I_DAI_PB18_O_ENCODING                0x11
#define FLAG10_I_DAI_PB18_O_ENCODING                0x11
#define DAI_INT_25_I_DAI_PB18_O_ENCODING            0x11
#define MISCB4_I_DAI_PB18_O_ENCODING                0x11
#define FLAG11_I_DAI_PB18_O_ENCODING                0x11
#define DAI_INT_26_I_DAI_PB18_O_ENCODING            0x11
#define MISCB5_I_DAI_PB18_O_ENCODING                0x11
#define FLAG12_I_DAI_PB18_O_ENCODING                0x11
#define DAI_INT_27_I_DAI_PB18_O_ENCODING            0x11
#define SPIB_DS_I_DAI_PB18_O_ENCODING               0x01
#define SPIB_MOSI_I_DAI_PB18_O_ENCODING             0x01
#define SPIB_MISO_I_DAI_PB18_O_ENCODING             0x01

#define MISCA0_I_DAI_PB19_O_ENCODING                0x12
#define FLAG13_I_DAI_PB19_O_ENCODING                0x12
#define DAI_INT_28_I_DAI_PB19_O_ENCODING            0x12
#define MISCA1_I_DAI_PB19_O_ENCODING                0x12
#define FLAG14_I_DAI_PB19_O_ENCODING                0x12
#define DAI_INT_29_I_DAI_PB19_O_ENCODING            0x12
#define MISCA2_I_DAI_PB19_O_ENCODING                0x12
#define FLAG15_I_DAI_PB19_O_ENCODING                0x12
#define DAI_INT_30_I_DAI_PB19_O_ENCODING            0x12
#define MISCA3_I_DAI_PB19_O_ENCODING                0x12
#define DAI_INT_31_I_DAI_PB19_O_ENCODING            0x12
#define MISCA4_I_DAI_PB19_O_ENCODING                0x12
#define MISCA5_I_DAI_PB19_O_ENCODING                0x12
#define INV_MISCA4_I_DAI_PB19_O_ENCODING            0x12
#define INV_MISCA5_I_DAI_PB19_O_ENCODING            0x12
#define MISCB0_I_DAI_PB19_O_ENCODING                0x12
#define TIMER0_I_DAI_PB19_O_ENCODING                0x12
#define DAI_INT_22_I_DAI_PB19_O_ENCODING            0x12
#define MISCB1_I_DAI_PB19_O_ENCODING                0x12
#define TIMER1_I_DAI_PB19_O_ENCODING                0x12
#define DAI_INT_23_I_DAI_PB19_O_ENCODING            0x12
#define MISCB2_I_DAI_PB19_O_ENCODING                0x12
#define TIMER2_I_DAI_PB19_O_ENCODING                0x12
#define DAI_INT_24_I_DAI_PB19_O_ENCODING            0x12
#define MISCB3_I_DAI_PB19_O_ENCODING                0x12
#define FLAG10_I_DAI_PB19_O_ENCODING                0x12
#define DAI_INT_25_I_DAI_PB19_O_ENCODING            0x12
#define MISCB4_I_DAI_PB19_O_ENCODING                0x12
#define FLAG11_I_DAI_PB19_O_ENCODING                0x12
#define DAI_INT_26_I_DAI_PB19_O_ENCODING            0x12
#define MISCB5_I_DAI_PB19_O_ENCODING                0x12
#define FLAG12_I_DAI_PB19_O_ENCODING                0x12
#define DAI_INT_27_I_DAI_PB19_O_ENCODING            0x12
#define SPIB_DS_I_DAI_PB19_O_ENCODING               0x12
#define SPIB_MOSI_I_DAI_PB19_O_ENCODING             0x12
#define SPIB_MISO_I_DAI_PB19_O_ENCODING             0x12

#define MISCA0_I_DAI_PB20_O_ENCODING                0x13
#define FLAG13_I_DAI_PB20_O_ENCODING                0x13
#define DAI_INT_28_I_DAI_PB20_O_ENCODING            0x13
#define MISCA1_I_DAI_PB20_O_ENCODING                0x13
#define FLAG14_I_DAI_PB20_O_ENCODING                0x13
#define DAI_INT_29_I_DAI_PB20_O_ENCODING            0x13
#define MISCA2_I_DAI_PB20_O_ENCODING                0x13
#define FLAG15_I_DAI_PB20_O_ENCODING                0x13
#define DAI_INT_30_I_DAI_PB20_O_ENCODING            0x13
#define MISCA3_I_DAI_PB20_O_ENCODING                0x13
#define DAI_INT_31_I_DAI_PB20_O_ENCODING            0x13
#define MISCA4_I_DAI_PB20_O_ENCODING                0x13
#define MISCA5_I_DAI_PB20_O_ENCODING                0x13
#define INV_MISCA4_I_DAI_PB20_O_ENCODING            0x13
#define INV_MISCA5_I_DAI_PB20_O_ENCODING            0x13
#define MISCB0_I_DAI_PB20_O_ENCODING                0x13
#define TIMER0_I_DAI_PB20_O_ENCODING                0x13
#define DAI_INT_22_I_DAI_PB20_O_ENCODING            0x13
#define MISCB1_I_DAI_PB20_O_ENCODING                0x13
#define TIMER1_I_DAI_PB20_O_ENCODING                0x13
#define DAI_INT_23_I_DAI_PB20_O_ENCODING            0x13
#define MISCB2_I_DAI_PB20_O_ENCODING                0x13
#define TIMER2_I_DAI_PB20_O_ENCODING                0x13
#define DAI_INT_24_I_DAI_PB20_O_ENCODING            0x13
#define MISCB3_I_DAI_PB20_O_ENCODING                0x13
#define FLAG10_I_DAI_PB20_O_ENCODING                0x13
#define DAI_INT_25_I_DAI_PB20_O_ENCODING            0x13
#define MISCB4_I_DAI_PB20_O_ENCODING                0x13
#define FLAG11_I_DAI_PB20_O_ENCODING                0x13
#define DAI_INT_26_I_DAI_PB20_O_ENCODING            0x13
#define MISCB5_I_DAI_PB20_O_ENCODING                0x13
#define FLAG12_I_DAI_PB20_O_ENCODING                0x13
#define DAI_INT_27_I_DAI_PB20_O_ENCODING            0x13
#define SPIB_DS_I_DAI_PB20_O_ENCODING               0x13
#define SPIB_MOSI_I_DAI_PB20_O_ENCODING             0x13
#define SPIB_MISO_I_DAI_PB20_O_ENCODING             0x13

#define MISCA0_I_TIMER0_MISC_O_ENCODING             0x14
#define FLAG13_I_TIMER0_MISC_O_ENCODING             0x14
#define DAI_INT_28_I_TIMER0_MISC_O_ENCODING         0x14
#define MISCA1_I_TIMER0_MISC_O_ENCODING             0x14
#define FLAG14_I_TIMER0_MISC_O_ENCODING             0x14
#define DAI_INT_29_I_TIMER0_MISC_O_ENCODING         0x14
#define MISCA2_I_TIMER0_MISC_O_ENCODING             0x14
#define FLAG15_I_TIMER0_MISC_O_ENCODING             0x14
#define DAI_INT_30_I_TIMER0_MISC_O_ENCODING         0x14
#define MISCA3_I_TIMER0_MISC_O_ENCODING             0x14
#define DAI_INT_31_I_TIMER0_MISC_O_ENCODING         0x14
#define MISCA4_I_TIMER0_MISC_O_ENCODING             0x14
#define MISCA5_I_TIMER0_MISC_O_ENCODING             0x14
#define INV_MISCA4_I_TIMER0_MISC_O_ENCODING         0x14
#define INV_MISCA5_I_TIMER0_MISC_O_ENCODING         0x14
#define MISCB0_I_TIMER0_MISC_O_ENCODING             0x14
#define TIMER0_I_TIMER0_MISC_O_ENCODING             0x14
#define DAI_INT_22_I_TIMER0_MISC_O_ENCODING         0x14
#define MISCB1_I_TIMER0_MISC_O_ENCODING             0x14
#define TIMER1_I_TIMER0_MISC_O_ENCODING             0x14
#define DAI_INT_23_I_TIMER0_MISC_O_ENCODING         0x14
#define MISCB2_I_TIMER0_MISC_O_ENCODING             0x14
#define TIMER2_I_TIMER0_MISC_O_ENCODING             0x14
#define DAI_INT_24_I_TIMER0_MISC_O_ENCODING         0x14
#define MISCB3_I_TIMER0_MISC_O_ENCODING             0x14
#define FLAG10_I_TIMER0_MISC_O_ENCODING             0x14
#define DAI_INT_25_I_TIMER0_MISC_O_ENCODING         0x14
#define MISCB4_I_TIMER0_MISC_O_ENCODING             0x14
#define FLAG11_I_TIMER0_MISC_O_ENCODING             0x14
#define DAI_INT_26_I_TIMER0_MISC_O_ENCODING         0x14
#define MISCB5_I_TIMER0_MISC_O_ENCODING             0x14
#define FLAG12_I_TIMER0_MISC_O_ENCODING             0x14
#define DAI_INT_27_I_TIMER0_MISC_O_ENCODING         0x14
#define SPIB_DS_I_TIMER0_MISC_O_ENCODING            0x14
#define SPIB_MOSI_I_TIMER0_MISC_O_ENCODING          0x14
#define SPIB_MISO_I_TIMER0_MISC_O_ENCODING          0x14

#define MISCA0_I_TIMER1_MISC_O_ENCODING             0x15
#define FLAG13_I_TIMER1_MISC_O_ENCODING             0x15
#define DAI_INT_28_I_TIMER1_MISC_O_ENCODING         0x15
#define MISCA1_I_TIMER1_MISC_O_ENCODING             0x15
#define FLAG14_I_TIMER1_MISC_O_ENCODING             0x15
#define DAI_INT_29_I_TIMER1_MISC_O_ENCODING         0x15
#define MISCA2_I_TIMER1_MISC_O_ENCODING             0x15
#define FLAG15_I_TIMER1_MISC_O_ENCODING             0x15
#define DAI_INT_30_I_TIMER1_MISC_O_ENCODING         0x15
#define MISCA3_I_TIMER1_MISC_O_ENCODING             0x15
#define DAI_INT_31_I_TIMER1_MISC_O_ENCODING         0x15
#define MISCA4_I_TIMER1_MISC_O_ENCODING             0x15
#define MISCA5_I_TIMER1_MISC_O_ENCODING             0x15
#define INV_MISCA4_I_TIMER1_MISC_O_ENCODING         0x15
#define INV_MISCA5_I_TIMER1_MISC_O_ENCODING         0x15
#define MISCB0_I_TIMER1_MISC_O_ENCODING             0x15
#define TIMER0_I_TIMER1_MISC_O_ENCODING             0x15
#define DAI_INT_22_I_TIMER1_MISC_O_ENCODING         0x15
#define MISCB1_I_TIMER1_MISC_O_ENCODING             0x15
#define TIMER1_I_TIMER1_MISC_O_ENCODING             0x15
#define DAI_INT_23_I_TIMER1_MISC_O_ENCODING         0x15
#define MISCB2_I_TIMER1_MISC_O_ENCODING             0x15
#define TIMER2_I_TIMER1_MISC_O_ENCODING             0x15
#define DAI_INT_24_I_TIMER1_MISC_O_ENCODING         0x15
#define MISCB3_I_TIMER1_MISC_O_ENCODING             0x15
#define FLAG10_I_TIMER1_MISC_O_ENCODING             0x15
#define DAI_INT_25_I_TIMER1_MISC_O_ENCODING         0x15
#define MISCB4_I_TIMER1_MISC_O_ENCODING             0x15
#define FLAG11_I_TIMER1_MISC_O_ENCODING             0x15
#define DAI_INT_26_I_TIMER1_MISC_O_ENCODING         0x15
#define MISCB5_I_TIMER1_MISC_O_ENCODING             0x15
#define FLAG12_I_TIMER1_MISC_O_ENCODING             0x15
#define DAI_INT_27_I_TIMER1_MISC_O_ENCODING         0x15
#define SPIB_DS_I_TIMER1_MISC_O_ENCODING            0x15
#define SPIB_MOSI_I_TIMER1_MISC_O_ENCODING          0x15
#define SPIB_MISO_I_TIMER1_MISC_O_ENCODING          0x15

#define MISCA0_I_TIMER2_MISC_O_ENCODING             0x16
#define FLAG13_I_TIMER2_MISC_O_ENCODING             0x16
#define DAI_INT_28_I_TIMER2_MISC_O_ENCODING         0x16
#define MISCA1_I_TIMER2_MISC_O_ENCODING             0x16
#define FLAG14_I_TIMER2_MISC_O_ENCODING             0x16
#define DAI_INT_29_I_TIMER2_MISC_O_ENCODING         0x16
#define MISCA2_I_TIMER2_MISC_O_ENCODING             0x16
#define FLAG15_I_TIMER2_MISC_O_ENCODING             0x16
#define DAI_INT_30_I_TIMER2_MISC_O_ENCODING         0x16
#define MISCA3_I_TIMER2_MISC_O_ENCODING             0x16
#define DAI_INT_31_I_TIMER2_MISC_O_ENCODING         0x16
#define MISCA4_I_TIMER2_MISC_O_ENCODING             0x16
#define MISCA5_I_TIMER2_MISC_O_ENCODING             0x16
#define INV_MISCA4_I_TIMER2_MISC_O_ENCODING         0x16
#define INV_MISCA5_I_TIMER2_MISC_O_ENCODING         0x16
#define MISCB0_I_TIMER2_MISC_O_ENCODING             0x16
#define TIMER0_I_TIMER2_MISC_O_ENCODING             0x16
#define DAI_INT_22_I_TIMER2_MISC_O_ENCODING         0x16
#define MISCB1_I_TIMER2_MISC_O_ENCODING             0x16
#define TIMER1_I_TIMER2_MISC_O_ENCODING             0x16
#define DAI_INT_23_I_TIMER2_MISC_O_ENCODING         0x16
#define MISCB2_I_TIMER2_MISC_O_ENCODING             0x16
#define TIMER2_I_TIMER2_MISC_O_ENCODING             0x16
#define DAI_INT_24_I_TIMER2_MISC_O_ENCODING         0x16
#define MISCB3_I_TIMER2_MISC_O_ENCODING             0x16
#define FLAG10_I_TIMER2_MISC_O_ENCODING             0x16
#define DAI_INT_25_I_TIMER2_MISC_O_ENCODING         0x16
#define MISCB4_I_TIMER2_MISC_O_ENCODING             0x16
#define FLAG11_I_TIMER2_MISC_O_ENCODING             0x16
#define DAI_INT_26_I_TIMER2_MISC_O_ENCODING         0x16
#define MISCB5_I_TIMER2_MISC_O_ENCODING             0x16
#define FLAG12_I_TIMER2_MISC_O_ENCODING             0x16
#define DAI_INT_27_I_TIMER2_MISC_O_ENCODING         0x16
#define SPIB_DS_I_TIMER2_MISC_O_ENCODING            0x16
#define SPIB_MOSI_I_TIMER2_MISC_O_ENCODING          0x16
#define SPIB_MISO_I_TIMER2_MISC_O_ENCODING          0x16

#define MISCA0_I_PDAP_STRB_O_ENCODING               0x19
#define FLAG13_I_PDAP_STRB_O_ENCODING               0x19
#define DAI_INT_28_I_PDAP_STRB_O_ENCODING           0x19
#define MISCA1_I_PDAP_STRB_O_ENCODING               0x19
#define FLAG14_I_PDAP_STRB_O_ENCODING               0x19
#define DAI_INT_29_I_PDAP_STRB_O_ENCODING           0x19
#define MISCA2_I_PDAP_STRB_O_ENCODING               0x19
#define FLAG15_I_PDAP_STRB_O_ENCODING               0x19
#define DAI_INT_30_I_PDAP_STRB_O_ENCODING           0x19
#define MISCA3_I_PDAP_STRB_O_ENCODING               0x19
#define DAI_INT_31_I_PDAP_STRB_O_ENCODING           0x19
#define MISCA4_I_PDAP_STRB_O_ENCODING               0x19
#define MISCA5_I_PDAP_STRB_O_ENCODING               0x19
#define INV_MISCA4_I_PDAP_STRB_O_ENCODING           0x19
#define INV_MISCA5_I_PDAP_STRB_O_ENCODING           0x19
#define MISCB0_I_PDAP_STRB_O_ENCODING               0x19
#define TIMER0_I_PDAP_STRB_O_ENCODING               0x19
#define DAI_INT_22_I_PDAP_STRB_O_ENCODING           0x19
#define MISCB1_I_PDAP_STRB_O_ENCODING               0x19
#define TIMER1_I_PDAP_STRB_O_ENCODING               0x19
#define DAI_INT_23_I_PDAP_STRB_O_ENCODING           0x19
#define MISCB2_I_PDAP_STRB_O_ENCODING               0x19
#define TIMER2_I_PDAP_STRB_O_ENCODING               0x19
#define DAI_INT_24_I_PDAP_STRB_O_ENCODING           0x19
#define MISCB3_I_PDAP_STRB_O_ENCODING               0x19
#define FLAG10_I_PDAP_STRB_O_ENCODING               0x19
#define DAI_INT_25_I_PDAP_STRB_O_ENCODING           0x19
#define MISCB4_I_PDAP_STRB_O_ENCODING               0x19
#define FLAG11_I_PDAP_STRB_O_ENCODING               0x19
#define DAI_INT_26_I_PDAP_STRB_O_ENCODING           0x19
#define MISCB5_I_PDAP_STRB_O_ENCODING               0x19
#define FLAG12_I_PDAP_STRB_O_ENCODING               0x19
#define DAI_INT_27_I_PDAP_STRB_O_ENCODING           0x19
#define SPIB_DS_I_PDAP_STRB_O_ENCODING              0x19
#define SPIB_MOSI_I_PDAP_STRB_O_ENCODING            0x19
#define SPIB_MISO_I_PDAP_STRB_O_ENCODING            0x19     

#define MISCA0_I_PCG_CLKA_O_ENCODING                0x1A
#define FLAG13_I_PCG_CLKA_O_ENCODING                0x1A
#define DAI_INT_28_I_PCG_CLKA_O_ENCODING            0x1A
#define MISCA1_I_PCG_CLKA_O_ENCODING                0x1A
#define FLAG14_I_PCG_CLKA_O_ENCODING                0x1A
#define DAI_INT_29_I_PCG_CLKA_O_ENCODING            0x1A
#define MISCA2_I_PCG_CLKA_O_ENCODING                0x1A
#define FLAG15_I_PCG_CLKA_O_ENCODING                0x1A
#define DAI_INT_30_I_PCG_CLKA_O_ENCODING            0x1A
#define MISCA3_I_PCG_CLKA_O_ENCODING                0x1A
#define DAI_INT_31_I_PCG_CLKA_O_ENCODING            0x1A
#define MISCA4_I_PCG_CLKA_O_ENCODING                0x1A
#define MISCA5_I_PCG_CLKA_O_ENCODING                0x1A
#define INV_MISCA4_I_PCG_CLKA_O_ENCODING            0x1A
#define INV_MISCA5_I_PCG_CLKA_O_ENCODING            0x1A
#define MISCB0_I_PCG_CLKA_O_ENCODING                0x1A
#define TIMER0_I_PCG_CLKA_O_ENCODING                0x1A
#define DAI_INT_22_I_PCG_CLKA_O_ENCODING            0x1A
#define MISCB1_I_PCG_CLKA_O_ENCODING                0x1A
#define TIMER1_I_PCG_CLKA_O_ENCODING                0x1A
#define DAI_INT_23_I_PCG_CLKA_O_ENCODING            0x1A
#define MISCB2_I_PCG_CLKA_O_ENCODING                0x1A
#define TIMER2_I_PCG_CLKA_O_ENCODING                0x1A
#define DAI_INT_24_I_PCG_CLKA_O_ENCODING            0x1A
#define MISCB3_I_PCG_CLKA_O_ENCODING                0x1A
#define FLAG10_I_PCG_CLKA_O_ENCODING                0x1A
#define DAI_INT_25_I_PCG_CLKA_O_ENCODING            0x1A
#define MISCB4_I_PCG_CLKA_O_ENCODING                0x1A
#define FLAG11_I_PCG_CLKA_O_ENCODING                0x1A
#define DAI_INT_26_I_PCG_CLKA_O_ENCODING            0x1A
#define MISCB5_I_PCG_CLKA_O_ENCODING                0x1A
#define FLAG12_I_PCG_CLKA_O_ENCODING                0x1A
#define DAI_INT_27_I_PCG_CLKA_O_ENCODING            0x1A
#define SPIB_DS_I_PCG_CLKA_O_ENCODING		    0x1A
#define SPIB_MOSI_I_PCG_CLKA_O_ENCODING             0x1A
#define SPIB_MISO_I_PCG_CLKA_O_ENCODING             0x1A

#define MISCA0_I_PCG_FSA_O_ENCODING                 0x1B
#define FLAG13_I_PCG_FSA_O_ENCODING                 0x1B
#define DAI_INT_28_I_PCG_FSA_O_ENCODING             0x1B
#define MISCA1_I_PCG_FSA_O_ENCODING                 0x1B
#define FLAG14_I_PCG_FSA_O_ENCODING                 0x1B
#define DAI_INT_29_I_PCG_FSA_O_ENCODING             0x1B
#define MISCA2_I_PCG_FSA_O_ENCODING                 0x1B
#define FLAG15_I_PCG_FSA_O_ENCODING                 0x1B
#define DAI_INT_30_I_PCG_FSA_O_ENCODING             0x1B
#define MISCA3_I_PCG_FSA_O_ENCODING                 0x1B
#define DAI_INT_31_I_PCG_FSA_O_ENCODING             0x1B
#define MISCA4_I_PCG_FSA_O_ENCODING                 0x1B
#define MISCA5_I_PCG_FSA_O_ENCODING                 0x1B
#define INV_MISCA4_I_PCG_FSA_O_ENCODING             0x1B
#define INV_MISCA5_I_PCG_FSA_O_ENCODING             0x1B
#define MISCB0_I_PCG_FSA_O_ENCODING                 0x1B
#define TIMER0_I_PCG_FSA_O_ENCODING                 0x1B
#define DAI_INT_22_I_PCG_FSA_O_ENCODING             0x1B
#define MISCB1_I_PCG_FSA_O_ENCODING                 0x1B
#define TIMER1_I_PCG_FSA_O_ENCODING                 0x1B
#define DAI_INT_23_I_PCG_FSA_O_ENCODING             0x1B
#define MISCB2_I_PCG_FSA_O_ENCODING                 0x1B
#define TIMER2_I_PCG_FSA_O_ENCODING                 0x1B
#define DAI_INT_24_I_PCG_FSA_O_ENCODING             0x1B
#define MISCB3_I_PCG_FSA_O_ENCODING                 0x1B
#define FLAG10_I_PCG_FSA_O_ENCODING                 0x1B
#define DAI_INT_25_I_PCG_FSA_O_ENCODING             0x1B
#define MISCB4_I_PCG_FSA_O_ENCODING                 0x1B
#define FLAG11_I_PCG_FSA_O_ENCODING                 0x1B
#define DAI_INT_26_I_PCG_FSA_O_ENCODING             0x1B
#define MISCB5_I_PCG_FSA_O_ENCODING                 0x1B
#define FLAG12_I_PCG_FSA_O_ENCODING                 0x1B
#define DAI_INT_27_I_PCG_FSA_O_ENCODING             0x1B
#define SPIB_DS_I_PCG_FSA_O_ENCODING		    0x1B
#define SPIB_MOSI_I_PCG_FSA_O_ENCODING              0x1B
#define SPIB_MISO_I_PCG_FSA_O_ENCODING              0x1B

#define MISCA0_I_PCG_CLKB_O_ENCODING                0x1C
#define FLAG13_I_PCG_CLKB_O_ENCODING                0x1C
#define DAI_INT_28_I_PCG_CLKB_O_ENCODING            0x1C
#define MISCA1_I_PCG_CLKB_O_ENCODING                0x1C
#define FLAG14_I_PCG_CLKB_O_ENCODING                0x1C
#define DAI_INT_29_I_PCG_CLKB_O_ENCODING            0x1C
#define MISCA2_I_PCG_CLKB_O_ENCODING                0x1C
#define FLAG15_I_PCG_CLKB_O_ENCODING                0x1C
#define DAI_INT_30_I_PCG_CLKB_O_ENCODING            0x1C
#define MISCA3_I_PCG_CLKB_O_ENCODING                0x1C
#define DAI_INT_31_I_PCG_CLKB_O_ENCODING            0x1C
#define MISCA4_I_PCG_CLKB_O_ENCODING                0x1C
#define MISCA5_I_PCG_CLKB_O_ENCODING                0x1C
#define INV_MISCA4_I_PCG_CLKB_O_ENCODING            0x1C
#define INV_MISCA5_I_PCG_CLKB_O_ENCODING            0x1C
#define MISCB0_I_PCG_CLKB_O_ENCODING                0x1C
#define TIMER0_I_PCG_CLKB_O_ENCODING                0x1C
#define DAI_INT_22_I_PCG_CLKB_O_ENCODING            0x1C
#define MISCB1_I_PCG_CLKB_O_ENCODING                0x1C
#define TIMER1_I_PCG_CLKB_O_ENCODING                0x1C
#define DAI_INT_23_I_PCG_CLKB_O_ENCODING            0x1C
#define MISCB2_I_PCG_CLKB_O_ENCODING                0x1C
#define TIMER2_I_PCG_CLKB_O_ENCODING                0x1C
#define DAI_INT_24_I_PCG_CLKB_O_ENCODING            0x1C
#define MISCB3_I_PCG_CLKB_O_ENCODING                0x1C
#define FLAG10_I_PCG_CLKB_O_ENCODING                0x1C
#define DAI_INT_25_I_PCG_CLKB_O_ENCODING            0x1C
#define MISCB4_I_PCG_CLKB_O_ENCODING                0x1C
#define FLAG11_I_PCG_CLKB_O_ENCODING                0x1C
#define DAI_INT_26_I_PCG_CLKB_O_ENCODING            0x1C
#define MISCB5_I_PCG_CLKB_O_ENCODING                0x1C
#define FLAG12_I_PCG_CLKB_O_ENCODING                0x1C
#define DAI_INT_27_I_PCG_CLKB_O_ENCODING            0x1C
#define SPIB_DS_I_PCG_CLKB_O_ENCODING		    0x1C
#define SPIB_MOSI_I_PCG_CLKB_O_ENCODING             0x1C
#define SPIB_MISO_I_PCG_CLKB_O_ENCODING             0x1C

#define MISCA0_I_PCG_FSB_O_ENCODING                 0x1D
#define FLAG13_I_PCG_FSB_O_ENCODING                 0x1D
#define DAI_INT_28_I_PCG_FSB_O_ENCODING             0x1D
#define MISCA1_I_PCG_FSB_O_ENCODING                 0x1D
#define FLAG14_I_PCG_FSB_O_ENCODING                 0x1D
#define DAI_INT_29_I_PCG_FSB_O_ENCODING             0x1D
#define MISCA2_I_PCG_FSB_O_ENCODING                 0x1D
#define FLAG15_I_PCG_FSB_O_ENCODING                 0x1D
#define DAI_INT_30_I_PCG_FSB_O_ENCODING             0x1D
#define MISCA3_I_PCG_FSB_O_ENCODING                 0x1D
#define DAI_INT_31_I_PCG_FSB_O_ENCODING             0x1D
#define MISCA4_I_PCG_FSB_O_ENCODING                 0x1D
#define MISCA5_I_PCG_FSB_O_ENCODING                 0x1D
#define INV_MISCA4_I_PCG_FSB_O_ENCODING             0x1D
#define INV_MISCA5_I_PCG_FSB_O_ENCODING             0x1D
#define MISCB0_I_PCG_FSB_O_ENCODING                 0x1D
#define TIMER0_I_PCG_FSB_O_ENCODING                 0x1D
#define DAI_INT_22_I_PCG_FSB_O_ENCODING             0x1D
#define MISCB1_I_PCG_FSB_O_ENCODING                 0x1D
#define TIMER1_I_PCG_FSB_O_ENCODING                 0x1D
#define DAI_INT_23_I_PCG_FSB_O_ENCODING             0x1D
#define MISCB2_I_PCG_FSB_O_ENCODING                 0x1D
#define TIMER2_I_PCG_FSB_O_ENCODING                 0x1D
#define DAI_INT_24_I_PCG_FSB_O_ENCODING             0x1D
#define MISCB3_I_PCG_FSB_O_ENCODING                 0x1D
#define FLAG10_I_PCG_FSB_O_ENCODING                 0x1D
#define DAI_INT_25_I_PCG_FSB_O_ENCODING             0x1D
#define MISCB4_I_PCG_FSB_O_ENCODING                 0x1D
#define FLAG11_I_PCG_FSB_O_ENCODING                 0x1D
#define DAI_INT_26_I_PCG_FSB_O_ENCODING             0x1D
#define MISCB5_I_PCG_FSB_O_ENCODING                 0x1D
#define FLAG12_I_PCG_FSB_O_ENCODING                 0x1D
#define DAI_INT_27_I_PCG_FSB_O_ENCODING             0x1D
#define SPIB_DS_I_PCG_FSB_O_ENCODING		    0x1D
#define SPIB_MOSI_I_PCG_FSB_O_ENCODING              0x1D
#define SPIB_MISO_I_PCG_FSB_O_ENCODING              0x1D

#define MISCA0_I_LOW_O_ENCODING                     0x1E
#define FLAG13_I_LOW_O_ENCODING                     0x1E
#define DAI_INT_28_I_LOW_O_ENCODING                 0x1E
#define MISCA1_I_LOW_O_ENCODING                     0x1E
#define FLAG14_I_LOW_O_ENCODING                     0x1E
#define DAI_INT_29_I_LOW_O_ENCODING                 0x1E
#define MISCA2_I_LOW_O_ENCODING                     0x1E
#define FLAG15_I_LOW_O_ENCODING                     0x1E
#define DAI_INT_30_I_LOW_O_ENCODING                 0x1E
#define MISCA3_I_LOW_O_ENCODING                     0x1E
#define DAI_INT_31_I_LOW_O_ENCODING                 0x1E
#define MISCA4_I_LOW_O_ENCODING                     0x1E
#define MISCA5_I_LOW_O_ENCODING                     0x1E
#define INV_MISCA4_I_LOW_O_ENCODING                 0x1E
#define INV_MISCA5_I_LOW_O_ENCODING                 0x1E
#define MISCB0_I_LOW_O_ENCODING                     0x1E
#define TIMER0_I_LOW_O_ENCODING                     0x1E
#define DAI_INT_22_I_LOW_O_ENCODING                 0x1E
#define MISCB1_I_LOW_O_ENCODING                     0x1E
#define TIMER1_I_LOW_O_ENCODING                     0x1E
#define DAI_INT_23_I_LOW_O_ENCODING                 0x1E
#define MISCB2_I_LOW_O_ENCODING                     0x1E
#define TIMER2_I_LOW_O_ENCODING                     0x1E
#define DAI_INT_24_I_LOW_O_ENCODING                 0x1E
#define MISCB3_I_LOW_O_ENCODING                     0x1E
#define FLAG10_I_LOW_O_ENCODING                     0x1E
#define DAI_INT_25_I_LOW_O_ENCODING                 0x1E
#define MISCB4_I_LOW_O_ENCODING                     0x1E
#define FLAG11_I_LOW_O_ENCODING                     0x1E
#define DAI_INT_26_I_LOW_O_ENCODING                 0x1E
#define MISCB5_I_LOW_O_ENCODING                     0x1E
#define FLAG12_I_LOW_O_ENCODING                     0x1E
#define DAI_INT_27_I_LOW_O_ENCODING                 0x1E
#define SPIB_DS_I_LOW_O_ENCODING                    0x1E
#define SPIB_MOSI_I_LOW_O_ENCODING                  0x1E
#define SPIB_MISO_I_LOW_O_ENCODING                  0x1E

#define MISCA0_I_HIGH_O_ENCODING                    0x1F
#define FLAG13_I_HIGH_O_ENCODING                    0x1F
#define DAI_INT_28_I_HIGH_O_ENCODING                0x1F
#define MISCA1_I_HIGH_O_ENCODING                    0x1F
#define FLAG14_I_HIGH_O_ENCODING                    0x1F
#define DAI_INT_29_I_HIGH_O_ENCODING                0x1F
#define MISCA2_I_HIGH_O_ENCODING                    0x1F
#define FLAG15_I_HIGH_O_ENCODING                    0x1F
#define DAI_INT_30_I_HIGH_O_ENCODING                0x1F
#define MISCA3_I_HIGH_O_ENCODING                    0x1F
#define DAI_INT_31_I_HIGH_O_ENCODING                0x1F
#define MISCA4_I_HIGH_O_ENCODING                    0x1F
#define MISCA5_I_HIGH_O_ENCODING                    0x1F
#define INV_MISCA4_I_HIGH_O_ENCODING                0x1F
#define INV_MISCA5_I_HIGH_O_ENCODING                0x1F
#define MISCB0_I_HIGH_O_ENCODING                    0x1F
#define TIMER0_I_HIGH_O_ENCODING                    0x1F
#define DAI_INT_22_I_HIGH_O_ENCODING                0x1F
#define MISCB1_I_HIGH_O_ENCODING                    0x1F
#define TIMER1_I_HIGH_O_ENCODING                    0x1F
#define DAI_INT_23_I_HIGH_O_ENCODING                0x1F
#define MISCB2_I_HIGH_O_ENCODING                    0x1F
#define TIMER2_I_HIGH_O_ENCODING                    0x1F
#define DAI_INT_24_I_HIGH_O_ENCODING                0x1F
#define MISCB3_I_HIGH_O_ENCODING                    0x1F
#define FLAG10_I_HIGH_O_ENCODING                    0x1F
#define DAI_INT_25_I_HIGH_O_ENCODING                0x1F
#define MISCB4_I_HIGH_O_ENCODING                    0x1F
#define FLAG11_I_HIGH_O_ENCODING                    0x1F
#define DAI_INT_26_I_HIGH_O_ENCODING                0x1F
#define MISCB5_I_HIGH_O_ENCODING                    0x1F
#define FLAG12_I_HIGH_O_ENCODING                    0x1F
#define DAI_INT_27_I_HIGH_O_ENCODING                0x1F
#define SPIB_DS_I_HIGH_O_ENCODING                   0x1F
#define SPIB_MOSI_I_HIGH_O_ENCODING                 0x1F
#define SPIB_MISO_I_HIGH_O_ENCODING                 0x1F

#define MISCA0_I_MISC_LOW_ENCODING                  0x1E
#define FLAG13_I_MISC_LOW_ENCODING                  0x1E
#define DAI_INT_28_I_MISC_LOW_ENCODING              0x1E
#define MISCA1_I_MISC_LOW_ENCODING                  0x1E
#define FLAG14_I_MISC_LOW_ENCODING                  0x1E
#define DAI_INT_29_I_MISC_LOW_ENCODING              0x1E
#define MISCA2_I_MISC_LOW_ENCODING                  0x1E
#define FLAG15_I_MISC_LOW_ENCODING                  0x1E
#define DAI_INT_30_I_MISC_LOW_ENCODING              0x1E
#define MISCA3_I_MISC_LOW_ENCODING                  0x1E
#define DAI_INT_31_I_MISC_LOW_ENCODING              0x1E
#define MISCA4_I_MISC_LOW_ENCODING                  0x1E
#define MISCA5_I_MISC_LOW_ENCODING                  0x1E
#define INV_MISCA4_I_MISC_LOW_ENCODING              0x1E
#define INV_MISCA5_I_MISC_LOW_ENCODING              0x1E
#define MISCB0_I_MISC_LOW_ENCODING                  0x1E
#define TIMER0_I_MISC_LOW_ENCODING                  0x1E
#define DAI_INT_22_I_MISC_LOW_ENCODING              0x1E
#define MISCB1_I_MISC_LOW_ENCODING                  0x1E
#define TIMER1_I_MISC_LOW_ENCODING                  0x1E
#define DAI_INT_23_I_MISC_LOW_ENCODING              0x1E
#define MISCB2_I_MISC_LOW_ENCODING                  0x1E
#define TIMER2_I_MISC_LOW_ENCODING                  0x1E
#define DAI_INT_24_I_MISC_LOW_ENCODING              0x1E
#define MISCB3_I_MISC_LOW_ENCODING                  0x1E
#define FLAG10_I_MISC_LOW_ENCODING                  0x1E
#define DAI_INT_25_I_MISC_LOW_ENCODING              0x1E
#define MISCB4_I_MISC_LOW_ENCODING                  0x1E
#define FLAG11_I_MISC_LOW_ENCODING                  0x1E
#define DAI_INT_26_I_MISC_LOW_ENCODING              0x1E
#define MISCB5_I_MISC_LOW_ENCODING                  0x1E
#define FLAG12_I_MISC_LOW_ENCODING                  0x1E
#define DAI_INT_27_I_MISC_LOW_ENCODING              0x1E
#define SPIB_DS_I_MISC_LOW_ENCODING                 0x1E
#define SPIB_MOSI_I_MISC_LOW_ENCODING               0x1E
#define SPIB_MISO_I_MISC_LOW_ENCODING               0x1E

#define MISCA0_I_MISC_HIGH_ENCODING                 0x1F
#define FLAG13_I_MISC_HIGH_ENCODING                 0x1F
#define DAI_INT_28_I_MISC_HIGH_ENCODING             0x1F
#define MISCA1_I_MISC_HIGH_ENCODING                 0x1F
#define FLAG14_I_MISC_HIGH_ENCODING                 0x1F
#define DAI_INT_29_I_MISC_HIGH_ENCODING             0x1F
#define MISCA2_I_MISC_HIGH_ENCODING                 0x1F
#define FLAG15_I_MISC_HIGH_ENCODING                 0x1F
#define DAI_INT_30_I_MISC_HIGH_ENCODING             0x1F
#define MISCA3_I_MISC_HIGH_ENCODING                 0x1F
#define DAI_INT_31_I_MISC_HIGH_ENCODING             0x1F
#define MISCA4_I_MISC_HIGH_ENCODING                 0x1F
#define MISCA5_I_MISC_HIGH_ENCODING                 0x1F
#define INV_MISCA4_I_MISC_HIGH_ENCODING             0x1F
#define INV_MISCA5_I_MISC_HIGH_ENCODING             0x1F
#define MISCB0_I_MISC_HIGH_ENCODING                 0x1F
#define TIMER0_I_MISC_HIGH_ENCODING                 0x1F
#define DAI_INT_22_I_MISC_HIGH_ENCODING             0x1F
#define MISCB1_I_MISC_HIGH_ENCODING                 0x1F
#define TIMER1_I_MISC_HIGH_ENCODING                 0x1F
#define DAI_INT_23_I_MISC_HIGH_ENCODING             0x1F
#define MISCB2_I_MISC_HIGH_ENCODING                 0x1F
#define TIMER2_I_MISC_HIGH_ENCODING                 0x1F
#define DAI_INT_24_I_MISC_HIGH_ENCODING             0x1F
#define MISCB3_I_MISC_HIGH_ENCODING                 0x1F
#define FLAG10_I_MISC_HIGH_ENCODING                 0x1F
#define DAI_INT_25_I_MISC_HIGH_ENCODING             0x1F
#define MISCB4_I_MISC_HIGH_ENCODING                 0x1F
#define FLAG11_I_MISC_HIGH_ENCODING                 0x1F
#define DAI_INT_26_I_MISC_HIGH_ENCODING             0x1F
#define MISCB5_I_MISC_HIGH_ENCODING                 0x1F
#define FLAG12_I_MISC_HIGH_ENCODING                 0x1F
#define DAI_INT_27_I_MISC_HIGH_ENCODING             0x1F
#define SPIB_DS_I_MISC_HIGH_ENCODING                0x1F
#define SPIB_MOSI_I_MISC_HIGH_ENCODING              0x1F
#define SPIB_MISO_I_MISC_HIGH_ENCODING              0x1F

#define MISCA4_INVERT_LOW_ENCODING                  0x0     
#define MISCA5_INVERT_LOW_ENCODING                  0x0     
                                                             
#define MISCA4_INVERT_ZERO_ENCODING                 0x0     
#define MISCA5_INVERT_ZERO_ENCODING                 0x0     
                                                            
#define INV_MISCA4_LOW_ENCODING                     0x0     
#define INV_MISCA5_LOW_ENCODING                     0x0     
                                                            
#define INV_MISCA4_ZERO_ENCODING                    0x0     
#define INV_MISCA5_ZERO_ENCODING                    0x0     
                                                             
#define MISCA4_INVERT_HIGH_ENCODING                 0x1     
#define MISCA5_INVERT_HIGH_ENCODING                 0x1     
                                                            
#define MISCA4_INVERT_ONE_ENCODING                  0x1     
#define MISCA5_INVERT_ONE_ENCODING                  0x1     
                                                            
#define INV_MISCA4_HIGH_ENCODING                    0x1     
#define INV_MISCA5_HIGH_ENCODING                    0x1     
                                                            
#define INV_MISCA4_ONE_ENCODING                     0x1     
#define INV_MISCA5_ONE_ENCODING                     0x1     



//--------------------------------------------------------
//  Group E: Register associations
//--------------------------------------------------------

#define MISCA0_I_DAI_PB01_O_REG             0x2470       /* SRU_EXT_MISCA */
#define FLAG13_I_DAI_PB01_O_REG             0x2470       /* SRU_EXT_MISCA */
#define DAI_INT_28_I_DAI_PB01_O_REG         0x2470       /* SRU_EXT_MISCA */
#define MISCA1_I_DAI_PB01_O_REG             0x2470       /* SRU_EXT_MISCA */
#define FLAG14_I_DAI_PB01_O_REG             0x2470       /* SRU_EXT_MISCA */
#define DAI_INT_29_I_DAI_PB01_O_REG         0x2470       /* SRU_EXT_MISCA */
#define MISCA2_I_DAI_PB01_O_REG             0x2470       /* SRU_EXT_MISCA */
#define FLAG15_I_DAI_PB01_O_REG             0x2470       /* SRU_EXT_MISCA */
#define DAI_INT_30_I_DAI_PB01_O_REG         0x2470       /* SRU_EXT_MISCA */
#define MISCA3_I_DAI_PB01_O_REG             0x2470       /* SRU_EXT_MISCA */
#define DAI_INT_31_I_DAI_PB01_O_REG         0x2470       /* SRU_EXT_MISCA */
#define MISCA4_I_DAI_PB01_O_REG             0x2470       /* SRU_EXT_MISCA */
#define MISCA5_I_DAI_PB01_O_REG             0x2470       /* SRU_EXT_MISCA */
#define INV_MISCA4_I_DAI_PB01_O_REG         0x2470       /* SRU_EXT_MISCA */
#define INV_MISCA5_I_DAI_PB01_O_REG         0x2470       /* SRU_EXT_MISCA */
#define SPIB_DS_I_DAI_PB01_O_REG            0x2470       /* SRU_EXT_MISCA */        
#define SPIB_MOSI_I_DAI_PB01_O_REG          0x2470       /* SRU_EXT_MISCA */        
#define SPIB_MISO_I_DAI_PB01_O_REG          0x2470       /* SRU_EXT_MISCA */        
#define MISCB0_I_DAI_PB01_O_REG             0x2471       /* SRU_EXT_MISCB */
#define TIMER0_I_DAI_PB01_O_REG             0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_22_I_DAI_PB01_O_REG         0x2471       /* SRU_EXT_MISCB */
#define MISCB1_I_DAI_PB01_O_REG             0x2471       /* SRU_EXT_MISCB */
#define TIMER1_I_DAI_PB01_O_REG             0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_23_I_DAI_PB01_O_REG         0x2471       /* SRU_EXT_MISCB */
#define MISCB2_I_DAI_PB01_O_REG             0x2471       /* SRU_EXT_MISCB */
#define TIMER2_I_DAI_PB01_O_REG             0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_24_I_DAI_PB01_O_REG         0x2471       /* SRU_EXT_MISCB */
#define MISCB3_I_DAI_PB01_O_REG             0x2471       /* SRU_EXT_MISCB */
#define FLAG10_I_DAI_PB01_O_REG             0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_25_I_DAI_PB01_O_REG         0x2471       /* SRU_EXT_MISCB */
#define MISCB4_I_DAI_PB01_O_REG             0x2471       /* SRU_EXT_MISCB */
#define FLAG11_I_DAI_PB01_O_REG             0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_26_I_DAI_PB01_O_REG         0x2471       /* SRU_EXT_MISCB */
#define MISCB5_I_DAI_PB01_O_REG             0x2471       /* SRU_EXT_MISCB */
#define FLAG12_I_DAI_PB01_O_REG             0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_27_I_DAI_PB01_O_REG         0x2471       /* SRU_EXT_MISCB */

#define MISCA0_I_DAI_PB02_O_REG             0x2470       /* SRU_EXT_MISCA */
#define FLAG13_I_DAI_PB02_O_REG             0x2470       /* SRU_EXT_MISCA */
#define DAI_INT_28_I_DAI_PB02_O_REG         0x2470       /* SRU_EXT_MISCA */
#define MISCA1_I_DAI_PB02_O_REG             0x2470       /* SRU_EXT_MISCA */
#define FLAG14_I_DAI_PB02_O_REG             0x2470       /* SRU_EXT_MISCA */
#define DAI_INT_29_I_DAI_PB02_O_REG         0x2470       /* SRU_EXT_MISCA */
#define MISCA2_I_DAI_PB02_O_REG             0x2470       /* SRU_EXT_MISCA */
#define FLAG15_I_DAI_PB02_O_REG             0x2470       /* SRU_EXT_MISCA */
#define DAI_INT_30_I_DAI_PB02_O_REG         0x2470       /* SRU_EXT_MISCA */
#define MISCA3_I_DAI_PB02_O_REG             0x2470       /* SRU_EXT_MISCA */
#define DAI_INT_31_I_DAI_PB02_O_REG         0x2470       /* SRU_EXT_MISCA */
#define MISCA4_I_DAI_PB02_O_REG             0x2470       /* SRU_EXT_MISCA */
#define MISCA5_I_DAI_PB02_O_REG             0x2470       /* SRU_EXT_MISCA */
#define INV_MISCA4_I_DAI_PB02_O_REG         0x2470       /* SRU_EXT_MISCA */
#define INV_MISCA5_I_DAI_PB02_O_REG         0x2470       /* SRU_EXT_MISCA */
#define SPIB_DS_I_DAI_PB02_O_REG            0x2470       /* SRU_EXT_MISCA */        
#define SPIB_MOSI_I_DAI_PB02_O_REG          0x2470       /* SRU_EXT_MISCA */        
#define SPIB_MISO_I_DAI_PB02_O_REG          0x2470       /* SRU_EXT_MISCA */        
#define MISCB0_I_DAI_PB02_O_REG             0x2471       /* SRU_EXT_MISCB */
#define TIMER0_I_DAI_PB02_O_REG             0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_22_I_DAI_PB02_O_REG         0x2471       /* SRU_EXT_MISCB */
#define MISCB1_I_DAI_PB02_O_REG             0x2471       /* SRU_EXT_MISCB */
#define TIMER1_I_DAI_PB02_O_REG             0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_23_I_DAI_PB02_O_REG         0x2471       /* SRU_EXT_MISCB */
#define MISCB2_I_DAI_PB02_O_REG             0x2471       /* SRU_EXT_MISCB */
#define TIMER2_I_DAI_PB02_O_REG             0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_24_I_DAI_PB02_O_REG         0x2471       /* SRU_EXT_MISCB */
#define MISCB3_I_DAI_PB02_O_REG             0x2471       /* SRU_EXT_MISCB */
#define FLAG10_I_DAI_PB02_O_REG             0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_25_I_DAI_PB02_O_REG         0x2471       /* SRU_EXT_MISCB */
#define MISCB4_I_DAI_PB02_O_REG             0x2471       /* SRU_EXT_MISCB */
#define FLAG11_I_DAI_PB02_O_REG             0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_26_I_DAI_PB02_O_REG         0x2471       /* SRU_EXT_MISCB */
#define MISCB5_I_DAI_PB02_O_REG             0x2471       /* SRU_EXT_MISCB */
#define FLAG12_I_DAI_PB02_O_REG             0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_27_I_DAI_PB02_O_REG         0x2471       /* SRU_EXT_MISCB */

#define MISCA0_I_DAI_PB03_O_REG             0x2470       /* SRU_EXT_MISCA */
#define FLAG13_I_DAI_PB03_O_REG             0x2470       /* SRU_EXT_MISCA */
#define DAI_INT_28_I_DAI_PB03_O_REG         0x2470       /* SRU_EXT_MISCA */
#define MISCA1_I_DAI_PB03_O_REG             0x2470       /* SRU_EXT_MISCA */
#define FLAG14_I_DAI_PB03_O_REG             0x2470       /* SRU_EXT_MISCA */
#define DAI_INT_29_I_DAI_PB03_O_REG         0x2470       /* SRU_EXT_MISCA */
#define MISCA2_I_DAI_PB03_O_REG             0x2470       /* SRU_EXT_MISCA */
#define FLAG15_I_DAI_PB03_O_REG             0x2470       /* SRU_EXT_MISCA */
#define DAI_INT_30_I_DAI_PB03_O_REG         0x2470       /* SRU_EXT_MISCA */
#define MISCA3_I_DAI_PB03_O_REG             0x2470       /* SRU_EXT_MISCA */
#define DAI_INT_31_I_DAI_PB03_O_REG         0x2470       /* SRU_EXT_MISCA */
#define MISCA4_I_DAI_PB03_O_REG             0x2470       /* SRU_EXT_MISCA */
#define MISCA5_I_DAI_PB03_O_REG             0x2470       /* SRU_EXT_MISCA */
#define INV_MISCA4_I_DAI_PB03_O_REG         0x2470       /* SRU_EXT_MISCA */
#define INV_MISCA5_I_DAI_PB03_O_REG         0x2470       /* SRU_EXT_MISCA */
#define SPIB_DS_I_DAI_PB03_O_REG            0x2470       /* SRU_EXT_MISCA */        
#define SPIB_MOSI_I_DAI_PB03_O_REG          0x2470       /* SRU_EXT_MISCA */        
#define SPIB_MISO_I_DAI_PB03_O_REG          0x2470       /* SRU_EXT_MISCA */        
#define MISCB0_I_DAI_PB03_O_REG             0x2471       /* SRU_EXT_MISCB */
#define TIMER0_I_DAI_PB03_O_REG             0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_22_I_DAI_PB03_O_REG         0x2471       /* SRU_EXT_MISCB */
#define MISCB1_I_DAI_PB03_O_REG             0x2471       /* SRU_EXT_MISCB */
#define TIMER1_I_DAI_PB03_O_REG             0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_23_I_DAI_PB03_O_REG         0x2471       /* SRU_EXT_MISCB */
#define MISCB2_I_DAI_PB03_O_REG             0x2471       /* SRU_EXT_MISCB */
#define TIMER2_I_DAI_PB03_O_REG             0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_24_I_DAI_PB03_O_REG         0x2471       /* SRU_EXT_MISCB */
#define MISCB3_I_DAI_PB03_O_REG             0x2471       /* SRU_EXT_MISCB */
#define FLAG10_I_DAI_PB03_O_REG             0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_25_I_DAI_PB03_O_REG         0x2471       /* SRU_EXT_MISCB */
#define MISCB4_I_DAI_PB03_O_REG             0x2471       /* SRU_EXT_MISCB */
#define FLAG11_I_DAI_PB03_O_REG             0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_26_I_DAI_PB03_O_REG         0x2471       /* SRU_EXT_MISCB */
#define MISCB5_I_DAI_PB03_O_REG             0x2471       /* SRU_EXT_MISCB */
#define FLAG12_I_DAI_PB03_O_REG             0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_27_I_DAI_PB03_O_REG         0x2471       /* SRU_EXT_MISCB */

#define MISCA0_I_DAI_PB04_O_REG             0x2470       /* SRU_EXT_MISCA */
#define FLAG13_I_DAI_PB04_O_REG             0x2470       /* SRU_EXT_MISCA */
#define DAI_INT_28_I_DAI_PB04_O_REG         0x2470       /* SRU_EXT_MISCA */
#define MISCA1_I_DAI_PB04_O_REG             0x2470       /* SRU_EXT_MISCA */
#define FLAG14_I_DAI_PB04_O_REG             0x2470       /* SRU_EXT_MISCA */
#define DAI_INT_29_I_DAI_PB04_O_REG         0x2470       /* SRU_EXT_MISCA */
#define MISCA2_I_DAI_PB04_O_REG             0x2470       /* SRU_EXT_MISCA */
#define FLAG15_I_DAI_PB04_O_REG             0x2470       /* SRU_EXT_MISCA */
#define DAI_INT_30_I_DAI_PB04_O_REG         0x2470       /* SRU_EXT_MISCA */
#define MISCA3_I_DAI_PB04_O_REG             0x2470       /* SRU_EXT_MISCA */
#define DAI_INT_31_I_DAI_PB04_O_REG         0x2470       /* SRU_EXT_MISCA */
#define MISCA4_I_DAI_PB04_O_REG             0x2470       /* SRU_EXT_MISCA */
#define MISCA5_I_DAI_PB04_O_REG             0x2470       /* SRU_EXT_MISCA */
#define INV_MISCA4_I_DAI_PB04_O_REG         0x2470       /* SRU_EXT_MISCA */
#define INV_MISCA5_I_DAI_PB04_O_REG         0x2470       /* SRU_EXT_MISCA */
#define SPIB_DS_I_DAI_PB04_O_REG            0x2470       /* SRU_EXT_MISCA */        
#define SPIB_MOSI_I_DAI_PB04_O_REG          0x2470       /* SRU_EXT_MISCA */        
#define SPIB_MISO_I_DAI_PB04_O_REG          0x2470       /* SRU_EXT_MISCA */        
#define MISCB0_I_DAI_PB04_O_REG             0x2471       /* SRU_EXT_MISCB */
#define TIMER0_I_DAI_PB04_O_REG             0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_22_I_DAI_PB04_O_REG         0x2471       /* SRU_EXT_MISCB */
#define MISCB1_I_DAI_PB04_O_REG             0x2471       /* SRU_EXT_MISCB */
#define TIMER1_I_DAI_PB04_O_REG             0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_23_I_DAI_PB04_O_REG         0x2471       /* SRU_EXT_MISCB */
#define MISCB2_I_DAI_PB04_O_REG             0x2471       /* SRU_EXT_MISCB */
#define TIMER2_I_DAI_PB04_O_REG             0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_24_I_DAI_PB04_O_REG         0x2471       /* SRU_EXT_MISCB */
#define MISCB3_I_DAI_PB04_O_REG             0x2471       /* SRU_EXT_MISCB */
#define FLAG10_I_DAI_PB04_O_REG             0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_25_I_DAI_PB04_O_REG         0x2471       /* SRU_EXT_MISCB */
#define MISCB4_I_DAI_PB04_O_REG             0x2471       /* SRU_EXT_MISCB */
#define FLAG11_I_DAI_PB04_O_REG             0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_26_I_DAI_PB04_O_REG         0x2471       /* SRU_EXT_MISCB */
#define MISCB5_I_DAI_PB04_O_REG             0x2471       /* SRU_EXT_MISCB */
#define FLAG12_I_DAI_PB04_O_REG             0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_27_I_DAI_PB04_O_REG         0x2471       /* SRU_EXT_MISCB */

#define MISCA0_I_DAI_PB05_O_REG             0x2470       /* SRU_EXT_MISCA */
#define FLAG13_I_DAI_PB05_O_REG             0x2470       /* SRU_EXT_MISCA */
#define DAI_INT_28_I_DAI_PB05_O_REG         0x2470       /* SRU_EXT_MISCA */
#define MISCA1_I_DAI_PB05_O_REG             0x2470       /* SRU_EXT_MISCA */
#define FLAG14_I_DAI_PB05_O_REG             0x2470       /* SRU_EXT_MISCA */
#define DAI_INT_29_I_DAI_PB05_O_REG         0x2470       /* SRU_EXT_MISCA */
#define MISCA2_I_DAI_PB05_O_REG             0x2470       /* SRU_EXT_MISCA */
#define FLAG15_I_DAI_PB05_O_REG             0x2470       /* SRU_EXT_MISCA */
#define DAI_INT_30_I_DAI_PB05_O_REG         0x2470       /* SRU_EXT_MISCA */
#define MISCA3_I_DAI_PB05_O_REG             0x2470       /* SRU_EXT_MISCA */
#define DAI_INT_31_I_DAI_PB05_O_REG         0x2470       /* SRU_EXT_MISCA */
#define MISCA4_I_DAI_PB05_O_REG             0x2470       /* SRU_EXT_MISCA */
#define MISCA5_I_DAI_PB05_O_REG             0x2470       /* SRU_EXT_MISCA */
#define INV_MISCA4_I_DAI_PB05_O_REG         0x2470       /* SRU_EXT_MISCA */
#define INV_MISCA5_I_DAI_PB05_O_REG         0x2470       /* SRU_EXT_MISCA */
#define SPIB_DS_I_DAI_PB05_O_REG            0x2470       /* SRU_EXT_MISCA */        
#define SPIB_MOSI_I_DAI_PB05_O_REG          0x2470       /* SRU_EXT_MISCA */        
#define SPIB_MISO_I_DAI_PB05_O_REG          0x2470       /* SRU_EXT_MISCA */        
#define MISCB0_I_DAI_PB05_O_REG             0x2471       /* SRU_EXT_MISCB */
#define TIMER0_I_DAI_PB05_O_REG             0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_22_I_DAI_PB05_O_REG         0x2471       /* SRU_EXT_MISCB */
#define MISCB1_I_DAI_PB05_O_REG             0x2471       /* SRU_EXT_MISCB */
#define TIMER1_I_DAI_PB05_O_REG             0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_23_I_DAI_PB05_O_REG         0x2471       /* SRU_EXT_MISCB */
#define MISCB2_I_DAI_PB05_O_REG             0x2471       /* SRU_EXT_MISCB */
#define TIMER2_I_DAI_PB05_O_REG             0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_24_I_DAI_PB05_O_REG         0x2471       /* SRU_EXT_MISCB */
#define MISCB3_I_DAI_PB05_O_REG             0x2471       /* SRU_EXT_MISCB */
#define FLAG10_I_DAI_PB05_O_REG             0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_25_I_DAI_PB05_O_REG         0x2471       /* SRU_EXT_MISCB */
#define MISCB4_I_DAI_PB05_O_REG             0x2471       /* SRU_EXT_MISCB */
#define FLAG11_I_DAI_PB05_O_REG             0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_26_I_DAI_PB05_O_REG         0x2471       /* SRU_EXT_MISCB */
#define MISCB5_I_DAI_PB05_O_REG             0x2471       /* SRU_EXT_MISCB */
#define FLAG12_I_DAI_PB05_O_REG             0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_27_I_DAI_PB05_O_REG         0x2471       /* SRU_EXT_MISCB */

#define MISCA0_I_DAI_PB06_O_REG             0x2470       /* SRU_EXT_MISCA */
#define FLAG13_I_DAI_PB06_O_REG             0x2470       /* SRU_EXT_MISCA */
#define DAI_INT_28_I_DAI_PB06_O_REG         0x2470       /* SRU_EXT_MISCA */
#define MISCA1_I_DAI_PB06_O_REG             0x2470       /* SRU_EXT_MISCA */
#define FLAG14_I_DAI_PB06_O_REG             0x2470       /* SRU_EXT_MISCA */
#define DAI_INT_29_I_DAI_PB06_O_REG         0x2470       /* SRU_EXT_MISCA */
#define MISCA2_I_DAI_PB06_O_REG             0x2470       /* SRU_EXT_MISCA */
#define FLAG15_I_DAI_PB06_O_REG             0x2470       /* SRU_EXT_MISCA */
#define DAI_INT_30_I_DAI_PB06_O_REG         0x2470       /* SRU_EXT_MISCA */
#define MISCA3_I_DAI_PB06_O_REG             0x2470       /* SRU_EXT_MISCA */
#define DAI_INT_31_I_DAI_PB06_O_REG         0x2470       /* SRU_EXT_MISCA */
#define MISCA4_I_DAI_PB06_O_REG             0x2470       /* SRU_EXT_MISCA */
#define MISCA5_I_DAI_PB06_O_REG             0x2470       /* SRU_EXT_MISCA */
#define INV_MISCA4_I_DAI_PB06_O_REG         0x2470       /* SRU_EXT_MISCA */
#define INV_MISCA5_I_DAI_PB06_O_REG         0x2470       /* SRU_EXT_MISCA */
#define SPIB_DS_I_DAI_PB06_O_REG            0x2470       /* SRU_EXT_MISCA */        
#define SPIB_MOSI_I_DAI_PB06_O_REG          0x2470       /* SRU_EXT_MISCA */        
#define SPIB_MISO_I_DAI_PB06_O_REG          0x2470       /* SRU_EXT_MISCA */        
#define MISCB0_I_DAI_PB06_O_REG             0x2471       /* SRU_EXT_MISCB */
#define TIMER0_I_DAI_PB06_O_REG             0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_22_I_DAI_PB06_O_REG         0x2471       /* SRU_EXT_MISCB */
#define MISCB1_I_DAI_PB06_O_REG             0x2471       /* SRU_EXT_MISCB */
#define TIMER1_I_DAI_PB06_O_REG             0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_23_I_DAI_PB06_O_REG         0x2471       /* SRU_EXT_MISCB */
#define MISCB2_I_DAI_PB06_O_REG             0x2471       /* SRU_EXT_MISCB */
#define TIMER2_I_DAI_PB06_O_REG             0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_24_I_DAI_PB06_O_REG         0x2471       /* SRU_EXT_MISCB */
#define MISCB3_I_DAI_PB06_O_REG             0x2471       /* SRU_EXT_MISCB */
#define FLAG10_I_DAI_PB06_O_REG             0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_25_I_DAI_PB06_O_REG         0x2471       /* SRU_EXT_MISCB */
#define MISCB4_I_DAI_PB06_O_REG             0x2471       /* SRU_EXT_MISCB */
#define FLAG11_I_DAI_PB06_O_REG             0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_26_I_DAI_PB06_O_REG         0x2471       /* SRU_EXT_MISCB */
#define MISCB5_I_DAI_PB06_O_REG             0x2471       /* SRU_EXT_MISCB */
#define FLAG12_I_DAI_PB06_O_REG             0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_27_I_DAI_PB06_O_REG         0x2471       /* SRU_EXT_MISCB */

#define MISCA0_I_DAI_PB07_O_REG             0x2470       /* SRU_EXT_MISCA */
#define FLAG13_I_DAI_PB07_O_REG             0x2470       /* SRU_EXT_MISCA */
#define DAI_INT_28_I_DAI_PB07_O_REG         0x2470       /* SRU_EXT_MISCA */
#define MISCA1_I_DAI_PB07_O_REG             0x2470       /* SRU_EXT_MISCA */
#define FLAG14_I_DAI_PB07_O_REG             0x2470       /* SRU_EXT_MISCA */
#define DAI_INT_29_I_DAI_PB07_O_REG         0x2470       /* SRU_EXT_MISCA */
#define MISCA2_I_DAI_PB07_O_REG             0x2470       /* SRU_EXT_MISCA */
#define FLAG15_I_DAI_PB07_O_REG             0x2470       /* SRU_EXT_MISCA */
#define DAI_INT_30_I_DAI_PB07_O_REG         0x2470       /* SRU_EXT_MISCA */
#define MISCA3_I_DAI_PB07_O_REG             0x2470       /* SRU_EXT_MISCA */
#define DAI_INT_31_I_DAI_PB07_O_REG         0x2470       /* SRU_EXT_MISCA */
#define MISCA4_I_DAI_PB07_O_REG             0x2470       /* SRU_EXT_MISCA */
#define MISCA5_I_DAI_PB07_O_REG             0x2470       /* SRU_EXT_MISCA */
#define INV_MISCA4_I_DAI_PB07_O_REG         0x2470       /* SRU_EXT_MISCA */
#define INV_MISCA5_I_DAI_PB07_O_REG         0x2470       /* SRU_EXT_MISCA */
#define SPIB_DS_I_DAI_PB07_O_REG            0x2470       /* SRU_EXT_MISCA */        
#define SPIB_MOSI_I_DAI_PB07_O_REG          0x2470       /* SRU_EXT_MISCA */        
#define SPIB_MISO_I_DAI_PB07_O_REG          0x2470       /* SRU_EXT_MISCA */        
#define MISCB0_I_DAI_PB07_O_REG             0x2471       /* SRU_EXT_MISCB */
#define TIMER0_I_DAI_PB07_O_REG             0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_22_I_DAI_PB07_O_REG         0x2471       /* SRU_EXT_MISCB */
#define MISCB1_I_DAI_PB07_O_REG             0x2471       /* SRU_EXT_MISCB */
#define TIMER1_I_DAI_PB07_O_REG             0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_23_I_DAI_PB07_O_REG         0x2471       /* SRU_EXT_MISCB */
#define MISCB2_I_DAI_PB07_O_REG             0x2471       /* SRU_EXT_MISCB */
#define TIMER2_I_DAI_PB07_O_REG             0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_24_I_DAI_PB07_O_REG         0x2471       /* SRU_EXT_MISCB */
#define MISCB3_I_DAI_PB07_O_REG             0x2471       /* SRU_EXT_MISCB */
#define FLAG10_I_DAI_PB07_O_REG             0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_25_I_DAI_PB07_O_REG         0x2471       /* SRU_EXT_MISCB */
#define MISCB4_I_DAI_PB07_O_REG             0x2471       /* SRU_EXT_MISCB */
#define FLAG11_I_DAI_PB07_O_REG             0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_26_I_DAI_PB07_O_REG         0x2471       /* SRU_EXT_MISCB */
#define MISCB5_I_DAI_PB07_O_REG             0x2471       /* SRU_EXT_MISCB */
#define FLAG12_I_DAI_PB07_O_REG             0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_27_I_DAI_PB07_O_REG         0x2471       /* SRU_EXT_MISCB */

#define MISCA0_I_DAI_PB08_O_REG             0x2470       /* SRU_EXT_MISCA */
#define FLAG13_I_DAI_PB08_O_REG             0x2470       /* SRU_EXT_MISCA */
#define DAI_INT_28_I_DAI_PB08_O_REG         0x2470       /* SRU_EXT_MISCA */
#define MISCA1_I_DAI_PB08_O_REG             0x2470       /* SRU_EXT_MISCA */
#define FLAG14_I_DAI_PB08_O_REG             0x2470       /* SRU_EXT_MISCA */
#define DAI_INT_29_I_DAI_PB08_O_REG         0x2470       /* SRU_EXT_MISCA */
#define MISCA2_I_DAI_PB08_O_REG             0x2470       /* SRU_EXT_MISCA */
#define FLAG15_I_DAI_PB08_O_REG             0x2470       /* SRU_EXT_MISCA */
#define DAI_INT_30_I_DAI_PB08_O_REG         0x2470       /* SRU_EXT_MISCA */
#define MISCA3_I_DAI_PB08_O_REG             0x2470       /* SRU_EXT_MISCA */
#define DAI_INT_31_I_DAI_PB08_O_REG         0x2470       /* SRU_EXT_MISCA */
#define MISCA4_I_DAI_PB08_O_REG             0x2470       /* SRU_EXT_MISCA */
#define MISCA5_I_DAI_PB08_O_REG             0x2470       /* SRU_EXT_MISCA */
#define INV_MISCA4_I_DAI_PB08_O_REG         0x2470       /* SRU_EXT_MISCA */
#define INV_MISCA5_I_DAI_PB08_O_REG         0x2470       /* SRU_EXT_MISCA */
#define SPIB_DS_I_DAI_PB08_O_REG            0x2470       /* SRU_EXT_MISCA */        
#define SPIB_MOSI_I_DAI_PB08_O_REG          0x2470       /* SRU_EXT_MISCA */        
#define SPIB_MISO_I_DAI_PB08_O_REG          0x2470       /* SRU_EXT_MISCA */        
#define MISCB0_I_DAI_PB08_O_REG             0x2471       /* SRU_EXT_MISCB */
#define TIMER0_I_DAI_PB08_O_REG             0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_22_I_DAI_PB08_O_REG         0x2471       /* SRU_EXT_MISCB */
#define MISCB1_I_DAI_PB08_O_REG             0x2471       /* SRU_EXT_MISCB */
#define TIMER1_I_DAI_PB08_O_REG             0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_23_I_DAI_PB08_O_REG         0x2471       /* SRU_EXT_MISCB */
#define MISCB2_I_DAI_PB08_O_REG             0x2471       /* SRU_EXT_MISCB */
#define TIMER2_I_DAI_PB08_O_REG             0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_24_I_DAI_PB08_O_REG         0x2471       /* SRU_EXT_MISCB */
#define MISCB3_I_DAI_PB08_O_REG             0x2471       /* SRU_EXT_MISCB */
#define FLAG10_I_DAI_PB08_O_REG             0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_25_I_DAI_PB08_O_REG         0x2471       /* SRU_EXT_MISCB */
#define MISCB4_I_DAI_PB08_O_REG             0x2471       /* SRU_EXT_MISCB */
#define FLAG11_I_DAI_PB08_O_REG             0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_26_I_DAI_PB08_O_REG         0x2471       /* SRU_EXT_MISCB */
#define MISCB5_I_DAI_PB08_O_REG             0x2471       /* SRU_EXT_MISCB */
#define FLAG12_I_DAI_PB08_O_REG             0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_27_I_DAI_PB08_O_REG         0x2471       /* SRU_EXT_MISCB */

#define MISCA0_I_DAI_PB09_O_REG             0x2470       /* SRU_EXT_MISCA */
#define FLAG13_I_DAI_PB09_O_REG             0x2470       /* SRU_EXT_MISCA */
#define DAI_INT_28_I_DAI_PB09_O_REG         0x2470       /* SRU_EXT_MISCA */
#define MISCA1_I_DAI_PB09_O_REG             0x2470       /* SRU_EXT_MISCA */
#define FLAG14_I_DAI_PB09_O_REG             0x2470       /* SRU_EXT_MISCA */
#define DAI_INT_29_I_DAI_PB09_O_REG         0x2470       /* SRU_EXT_MISCA */
#define MISCA2_I_DAI_PB09_O_REG             0x2470       /* SRU_EXT_MISCA */
#define FLAG15_I_DAI_PB09_O_REG             0x2470       /* SRU_EXT_MISCA */
#define DAI_INT_30_I_DAI_PB09_O_REG         0x2470       /* SRU_EXT_MISCA */
#define MISCA3_I_DAI_PB09_O_REG             0x2470       /* SRU_EXT_MISCA */
#define DAI_INT_31_I_DAI_PB09_O_REG         0x2470       /* SRU_EXT_MISCA */
#define MISCA4_I_DAI_PB09_O_REG             0x2470       /* SRU_EXT_MISCA */
#define MISCA5_I_DAI_PB09_O_REG             0x2470       /* SRU_EXT_MISCA */
#define INV_MISCA4_I_DAI_PB09_O_REG         0x2470       /* SRU_EXT_MISCA */
#define INV_MISCA5_I_DAI_PB09_O_REG         0x2470       /* SRU_EXT_MISCA */
#define SPIB_DS_I_DAI_PB09_O_REG            0x2470       /* SRU_EXT_MISCA */        
#define SPIB_MOSI_I_DAI_PB09_O_REG          0x2470       /* SRU_EXT_MISCA */        
#define SPIB_MISO_I_DAI_PB09_O_REG          0x2470       /* SRU_EXT_MISCA */        
#define MISCB0_I_DAI_PB09_O_REG             0x2471       /* SRU_EXT_MISCB */
#define TIMER0_I_DAI_PB09_O_REG             0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_22_I_DAI_PB09_O_REG         0x2471       /* SRU_EXT_MISCB */
#define MISCB1_I_DAI_PB09_O_REG             0x2471       /* SRU_EXT_MISCB */
#define TIMER1_I_DAI_PB09_O_REG             0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_23_I_DAI_PB09_O_REG         0x2471       /* SRU_EXT_MISCB */
#define MISCB2_I_DAI_PB09_O_REG             0x2471       /* SRU_EXT_MISCB */
#define TIMER2_I_DAI_PB09_O_REG             0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_24_I_DAI_PB09_O_REG         0x2471       /* SRU_EXT_MISCB */
#define MISCB3_I_DAI_PB09_O_REG             0x2471       /* SRU_EXT_MISCB */
#define FLAG10_I_DAI_PB09_O_REG             0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_25_I_DAI_PB09_O_REG         0x2471       /* SRU_EXT_MISCB */
#define MISCB4_I_DAI_PB09_O_REG             0x2471       /* SRU_EXT_MISCB */
#define FLAG11_I_DAI_PB09_O_REG             0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_26_I_DAI_PB09_O_REG         0x2471       /* SRU_EXT_MISCB */
#define MISCB5_I_DAI_PB09_O_REG             0x2471       /* SRU_EXT_MISCB */
#define FLAG12_I_DAI_PB09_O_REG             0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_27_I_DAI_PB09_O_REG         0x2471       /* SRU_EXT_MISCB */

#define MISCA0_I_DAI_PB10_O_REG             0x2470       /* SRU_EXT_MISCA */
#define FLAG13_I_DAI_PB10_O_REG             0x2470       /* SRU_EXT_MISCA */
#define DAI_INT_28_I_DAI_PB10_O_REG         0x2470       /* SRU_EXT_MISCA */
#define MISCA1_I_DAI_PB10_O_REG             0x2470       /* SRU_EXT_MISCA */
#define FLAG14_I_DAI_PB10_O_REG             0x2470       /* SRU_EXT_MISCA */
#define DAI_INT_29_I_DAI_PB10_O_REG         0x2470       /* SRU_EXT_MISCA */
#define MISCA2_I_DAI_PB10_O_REG             0x2470       /* SRU_EXT_MISCA */
#define FLAG15_I_DAI_PB10_O_REG             0x2470       /* SRU_EXT_MISCA */
#define DAI_INT_30_I_DAI_PB10_O_REG         0x2470       /* SRU_EXT_MISCA */
#define MISCA3_I_DAI_PB10_O_REG             0x2470       /* SRU_EXT_MISCA */
#define DAI_INT_31_I_DAI_PB10_O_REG         0x2470       /* SRU_EXT_MISCA */
#define MISCA4_I_DAI_PB10_O_REG             0x2470       /* SRU_EXT_MISCA */
#define MISCA5_I_DAI_PB10_O_REG             0x2470       /* SRU_EXT_MISCA */
#define INV_MISCA4_I_DAI_PB10_O_REG         0x2470       /* SRU_EXT_MISCA */
#define INV_MISCA5_I_DAI_PB10_O_REG         0x2470       /* SRU_EXT_MISCA */
#define SPIB_DS_I_DAI_PB10_O_REG            0x2470       /* SRU_EXT_MISCA */        
#define SPIB_MOSI_I_DAI_PB10_O_REG          0x2470       /* SRU_EXT_MISCA */        
#define SPIB_MISO_I_DAI_PB10_O_REG          0x2470       /* SRU_EXT_MISCA */        
#define MISCB0_I_DAI_PB10_O_REG             0x2471       /* SRU_EXT_MISCB */
#define TIMER0_I_DAI_PB10_O_REG             0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_22_I_DAI_PB10_O_REG         0x2471       /* SRU_EXT_MISCB */
#define MISCB1_I_DAI_PB10_O_REG             0x2471       /* SRU_EXT_MISCB */
#define TIMER1_I_DAI_PB10_O_REG             0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_23_I_DAI_PB10_O_REG         0x2471       /* SRU_EXT_MISCB */
#define MISCB2_I_DAI_PB10_O_REG             0x2471       /* SRU_EXT_MISCB */
#define TIMER2_I_DAI_PB10_O_REG             0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_24_I_DAI_PB10_O_REG         0x2471       /* SRU_EXT_MISCB */
#define MISCB3_I_DAI_PB10_O_REG             0x2471       /* SRU_EXT_MISCB */
#define FLAG10_I_DAI_PB10_O_REG             0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_25_I_DAI_PB10_O_REG         0x2471       /* SRU_EXT_MISCB */
#define MISCB4_I_DAI_PB10_O_REG             0x2471       /* SRU_EXT_MISCB */
#define FLAG11_I_DAI_PB10_O_REG             0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_26_I_DAI_PB10_O_REG         0x2471       /* SRU_EXT_MISCB */
#define MISCB5_I_DAI_PB10_O_REG             0x2471       /* SRU_EXT_MISCB */
#define FLAG12_I_DAI_PB10_O_REG             0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_27_I_DAI_PB10_O_REG         0x2471       /* SRU_EXT_MISCB */

#define MISCA0_I_DAI_PB11_O_REG             0x2470       /* SRU_EXT_MISCA */
#define FLAG13_I_DAI_PB11_O_REG             0x2470       /* SRU_EXT_MISCA */
#define DAI_INT_28_I_DAI_PB11_O_REG         0x2470       /* SRU_EXT_MISCA */
#define MISCA1_I_DAI_PB11_O_REG             0x2470       /* SRU_EXT_MISCA */
#define FLAG14_I_DAI_PB11_O_REG             0x2470       /* SRU_EXT_MISCA */
#define DAI_INT_29_I_DAI_PB11_O_REG         0x2470       /* SRU_EXT_MISCA */
#define MISCA2_I_DAI_PB11_O_REG             0x2470       /* SRU_EXT_MISCA */
#define FLAG15_I_DAI_PB11_O_REG             0x2470       /* SRU_EXT_MISCA */
#define DAI_INT_30_I_DAI_PB11_O_REG         0x2470       /* SRU_EXT_MISCA */
#define MISCA3_I_DAI_PB11_O_REG             0x2470       /* SRU_EXT_MISCA */
#define DAI_INT_31_I_DAI_PB11_O_REG         0x2470       /* SRU_EXT_MISCA */
#define MISCA4_I_DAI_PB11_O_REG             0x2470       /* SRU_EXT_MISCA */
#define MISCA5_I_DAI_PB11_O_REG             0x2470       /* SRU_EXT_MISCA */
#define INV_MISCA4_I_DAI_PB11_O_REG         0x2470       /* SRU_EXT_MISCA */
#define INV_MISCA5_I_DAI_PB11_O_REG         0x2470       /* SRU_EXT_MISCA */
#define SPIB_DS_I_DAI_PB11_O_REG            0x2470       /* SRU_EXT_MISCA */        
#define SPIB_MOSI_I_DAI_PB11_O_REG          0x2470       /* SRU_EXT_MISCA */        
#define SPIB_MISO_I_DAI_PB11_O_REG          0x2470       /* SRU_EXT_MISCA */        
#define MISCB0_I_DAI_PB11_O_REG             0x2471       /* SRU_EXT_MISCB */
#define TIMER0_I_DAI_PB11_O_REG             0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_22_I_DAI_PB11_O_REG         0x2471       /* SRU_EXT_MISCB */
#define MISCB1_I_DAI_PB11_O_REG             0x2471       /* SRU_EXT_MISCB */
#define TIMER1_I_DAI_PB11_O_REG             0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_23_I_DAI_PB11_O_REG         0x2471       /* SRU_EXT_MISCB */
#define MISCB2_I_DAI_PB11_O_REG             0x2471       /* SRU_EXT_MISCB */
#define TIMER2_I_DAI_PB11_O_REG             0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_24_I_DAI_PB11_O_REG         0x2471       /* SRU_EXT_MISCB */
#define MISCB3_I_DAI_PB11_O_REG             0x2471       /* SRU_EXT_MISCB */
#define FLAG10_I_DAI_PB11_O_REG             0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_25_I_DAI_PB11_O_REG         0x2471       /* SRU_EXT_MISCB */
#define MISCB4_I_DAI_PB11_O_REG             0x2471       /* SRU_EXT_MISCB */
#define FLAG11_I_DAI_PB11_O_REG             0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_26_I_DAI_PB11_O_REG         0x2471       /* SRU_EXT_MISCB */
#define MISCB5_I_DAI_PB11_O_REG             0x2471       /* SRU_EXT_MISCB */
#define FLAG12_I_DAI_PB11_O_REG             0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_27_I_DAI_PB11_O_REG         0x2471       /* SRU_EXT_MISCB */

#define MISCA0_I_DAI_PB12_O_REG             0x2470       /* SRU_EXT_MISCA */
#define FLAG13_I_DAI_PB12_O_REG             0x2470       /* SRU_EXT_MISCA */
#define DAI_INT_28_I_DAI_PB12_O_REG         0x2470       /* SRU_EXT_MISCA */
#define MISCA1_I_DAI_PB12_O_REG             0x2470       /* SRU_EXT_MISCA */
#define FLAG14_I_DAI_PB12_O_REG             0x2470       /* SRU_EXT_MISCA */
#define DAI_INT_29_I_DAI_PB12_O_REG         0x2470       /* SRU_EXT_MISCA */
#define MISCA2_I_DAI_PB12_O_REG             0x2470       /* SRU_EXT_MISCA */
#define FLAG15_I_DAI_PB12_O_REG             0x2470       /* SRU_EXT_MISCA */
#define DAI_INT_30_I_DAI_PB12_O_REG         0x2470       /* SRU_EXT_MISCA */
#define MISCA3_I_DAI_PB12_O_REG             0x2470       /* SRU_EXT_MISCA */
#define DAI_INT_31_I_DAI_PB12_O_REG         0x2470       /* SRU_EXT_MISCA */
#define MISCA4_I_DAI_PB12_O_REG             0x2470       /* SRU_EXT_MISCA */
#define MISCA5_I_DAI_PB12_O_REG             0x2470       /* SRU_EXT_MISCA */
#define INV_MISCA4_I_DAI_PB12_O_REG         0x2470       /* SRU_EXT_MISCA */
#define INV_MISCA5_I_DAI_PB12_O_REG         0x2470       /* SRU_EXT_MISCA */
#define SPIB_DS_I_DAI_PB12_O_REG            0x2470       /* SRU_EXT_MISCA */        
#define SPIB_MOSI_I_DAI_PB12_O_REG          0x2470       /* SRU_EXT_MISCA */        
#define SPIB_MISO_I_DAI_PB12_O_REG          0x2470       /* SRU_EXT_MISCA */        
#define MISCB0_I_DAI_PB12_O_REG             0x2471       /* SRU_EXT_MISCB */
#define TIMER0_I_DAI_PB12_O_REG             0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_22_I_DAI_PB12_O_REG         0x2471       /* SRU_EXT_MISCB */
#define MISCB1_I_DAI_PB12_O_REG             0x2471       /* SRU_EXT_MISCB */
#define TIMER1_I_DAI_PB12_O_REG             0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_23_I_DAI_PB12_O_REG         0x2471       /* SRU_EXT_MISCB */
#define MISCB2_I_DAI_PB12_O_REG             0x2471       /* SRU_EXT_MISCB */
#define TIMER2_I_DAI_PB12_O_REG             0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_24_I_DAI_PB12_O_REG         0x2471       /* SRU_EXT_MISCB */
#define MISCB3_I_DAI_PB12_O_REG             0x2471       /* SRU_EXT_MISCB */
#define FLAG10_I_DAI_PB12_O_REG             0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_25_I_DAI_PB12_O_REG         0x2471       /* SRU_EXT_MISCB */
#define MISCB4_I_DAI_PB12_O_REG             0x2471       /* SRU_EXT_MISCB */
#define FLAG11_I_DAI_PB12_O_REG             0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_26_I_DAI_PB12_O_REG         0x2471       /* SRU_EXT_MISCB */
#define MISCB5_I_DAI_PB12_O_REG             0x2471       /* SRU_EXT_MISCB */
#define FLAG12_I_DAI_PB12_O_REG             0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_27_I_DAI_PB12_O_REG         0x2471       /* SRU_EXT_MISCB */

#define MISCA0_I_DAI_PB13_O_REG             0x2470       /* SRU_EXT_MISCA */
#define FLAG13_I_DAI_PB13_O_REG             0x2470       /* SRU_EXT_MISCA */
#define DAI_INT_28_I_DAI_PB13_O_REG         0x2470       /* SRU_EXT_MISCA */
#define MISCA1_I_DAI_PB13_O_REG             0x2470       /* SRU_EXT_MISCA */
#define FLAG14_I_DAI_PB13_O_REG             0x2470       /* SRU_EXT_MISCA */
#define DAI_INT_29_I_DAI_PB13_O_REG         0x2470       /* SRU_EXT_MISCA */
#define MISCA2_I_DAI_PB13_O_REG             0x2470       /* SRU_EXT_MISCA */
#define FLAG15_I_DAI_PB13_O_REG             0x2470       /* SRU_EXT_MISCA */
#define DAI_INT_30_I_DAI_PB13_O_REG         0x2470       /* SRU_EXT_MISCA */
#define MISCA3_I_DAI_PB13_O_REG             0x2470       /* SRU_EXT_MISCA */
#define DAI_INT_31_I_DAI_PB13_O_REG         0x2470       /* SRU_EXT_MISCA */
#define MISCA4_I_DAI_PB13_O_REG             0x2470       /* SRU_EXT_MISCA */
#define MISCA5_I_DAI_PB13_O_REG             0x2470       /* SRU_EXT_MISCA */
#define INV_MISCA4_I_DAI_PB13_O_REG         0x2470       /* SRU_EXT_MISCA */
#define INV_MISCA5_I_DAI_PB13_O_REG         0x2470       /* SRU_EXT_MISCA */
#define SPIB_DS_I_DAI_PB13_O_REG            0x2470       /* SRU_EXT_MISCA */        
#define SPIB_MOSI_I_DAI_PB13_O_REG          0x2470       /* SRU_EXT_MISCA */        
#define SPIB_MISO_I_DAI_PB13_O_REG          0x2470       /* SRU_EXT_MISCA */        
#define MISCB0_I_DAI_PB13_O_REG             0x2471       /* SRU_EXT_MISCB */
#define TIMER0_I_DAI_PB13_O_REG             0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_22_I_DAI_PB13_O_REG         0x2471       /* SRU_EXT_MISCB */
#define MISCB1_I_DAI_PB13_O_REG             0x2471       /* SRU_EXT_MISCB */
#define TIMER1_I_DAI_PB13_O_REG             0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_23_I_DAI_PB13_O_REG         0x2471       /* SRU_EXT_MISCB */
#define MISCB2_I_DAI_PB13_O_REG             0x2471       /* SRU_EXT_MISCB */
#define TIMER2_I_DAI_PB13_O_REG             0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_24_I_DAI_PB13_O_REG         0x2471       /* SRU_EXT_MISCB */
#define MISCB3_I_DAI_PB13_O_REG             0x2471       /* SRU_EXT_MISCB */
#define FLAG10_I_DAI_PB13_O_REG             0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_25_I_DAI_PB13_O_REG         0x2471       /* SRU_EXT_MISCB */
#define MISCB4_I_DAI_PB13_O_REG             0x2471       /* SRU_EXT_MISCB */
#define FLAG11_I_DAI_PB13_O_REG             0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_26_I_DAI_PB13_O_REG         0x2471       /* SRU_EXT_MISCB */
#define MISCB5_I_DAI_PB13_O_REG             0x2471       /* SRU_EXT_MISCB */
#define FLAG12_I_DAI_PB13_O_REG             0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_27_I_DAI_PB13_O_REG         0x2471       /* SRU_EXT_MISCB */

#define MISCA0_I_DAI_PB14_O_REG             0x2470       /* SRU_EXT_MISCA */
#define FLAG13_I_DAI_PB14_O_REG             0x2470       /* SRU_EXT_MISCA */
#define DAI_INT_28_I_DAI_PB14_O_REG         0x2470       /* SRU_EXT_MISCA */
#define MISCA1_I_DAI_PB14_O_REG             0x2470       /* SRU_EXT_MISCA */
#define FLAG14_I_DAI_PB14_O_REG             0x2470       /* SRU_EXT_MISCA */
#define DAI_INT_29_I_DAI_PB14_O_REG         0x2470       /* SRU_EXT_MISCA */
#define MISCA2_I_DAI_PB14_O_REG             0x2470       /* SRU_EXT_MISCA */
#define FLAG15_I_DAI_PB14_O_REG             0x2470       /* SRU_EXT_MISCA */
#define DAI_INT_30_I_DAI_PB14_O_REG         0x2470       /* SRU_EXT_MISCA */
#define MISCA3_I_DAI_PB14_O_REG             0x2470       /* SRU_EXT_MISCA */
#define DAI_INT_31_I_DAI_PB14_O_REG         0x2470       /* SRU_EXT_MISCA */
#define MISCA4_I_DAI_PB14_O_REG             0x2470       /* SRU_EXT_MISCA */
#define MISCA5_I_DAI_PB14_O_REG             0x2470       /* SRU_EXT_MISCA */
#define INV_MISCA4_I_DAI_PB14_O_REG         0x2470       /* SRU_EXT_MISCA */
#define INV_MISCA5_I_DAI_PB14_O_REG         0x2470       /* SRU_EXT_MISCA */
#define SPIB_DS_I_DAI_PB14_O_REG            0x2470       /* SRU_EXT_MISCA */        
#define SPIB_MOSI_I_DAI_PB14_O_REG          0x2470       /* SRU_EXT_MISCA */        
#define SPIB_MISO_I_DAI_PB14_O_REG          0x2470       /* SRU_EXT_MISCA */        
#define MISCB0_I_DAI_PB14_O_REG             0x2471       /* SRU_EXT_MISCB */
#define TIMER0_I_DAI_PB14_O_REG             0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_22_I_DAI_PB14_O_REG         0x2471       /* SRU_EXT_MISCB */
#define MISCB1_I_DAI_PB14_O_REG             0x2471       /* SRU_EXT_MISCB */
#define TIMER1_I_DAI_PB14_O_REG             0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_23_I_DAI_PB14_O_REG         0x2471       /* SRU_EXT_MISCB */
#define MISCB2_I_DAI_PB14_O_REG             0x2471       /* SRU_EXT_MISCB */
#define TIMER2_I_DAI_PB14_O_REG             0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_24_I_DAI_PB14_O_REG         0x2471       /* SRU_EXT_MISCB */
#define MISCB3_I_DAI_PB14_O_REG             0x2471       /* SRU_EXT_MISCB */
#define FLAG10_I_DAI_PB14_O_REG             0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_25_I_DAI_PB14_O_REG         0x2471       /* SRU_EXT_MISCB */
#define MISCB4_I_DAI_PB14_O_REG             0x2471       /* SRU_EXT_MISCB */
#define FLAG11_I_DAI_PB14_O_REG             0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_26_I_DAI_PB14_O_REG         0x2471       /* SRU_EXT_MISCB */
#define MISCB5_I_DAI_PB14_O_REG             0x2471       /* SRU_EXT_MISCB */
#define FLAG12_I_DAI_PB14_O_REG             0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_27_I_DAI_PB14_O_REG         0x2471       /* SRU_EXT_MISCB */

#define MISCA0_I_DAI_PB15_O_REG             0x2470       /* SRU_EXT_MISCA */
#define FLAG13_I_DAI_PB15_O_REG             0x2470       /* SRU_EXT_MISCA */
#define DAI_INT_28_I_DAI_PB15_O_REG         0x2470       /* SRU_EXT_MISCA */
#define MISCA1_I_DAI_PB15_O_REG             0x2470       /* SRU_EXT_MISCA */
#define FLAG14_I_DAI_PB15_O_REG             0x2470       /* SRU_EXT_MISCA */
#define DAI_INT_29_I_DAI_PB15_O_REG         0x2470       /* SRU_EXT_MISCA */
#define MISCA2_I_DAI_PB15_O_REG             0x2470       /* SRU_EXT_MISCA */
#define FLAG15_I_DAI_PB15_O_REG             0x2470       /* SRU_EXT_MISCA */
#define DAI_INT_30_I_DAI_PB15_O_REG         0x2470       /* SRU_EXT_MISCA */
#define MISCA3_I_DAI_PB15_O_REG             0x2470       /* SRU_EXT_MISCA */
#define DAI_INT_31_I_DAI_PB15_O_REG         0x2470       /* SRU_EXT_MISCA */
#define MISCA4_I_DAI_PB15_O_REG             0x2470       /* SRU_EXT_MISCA */
#define MISCA5_I_DAI_PB15_O_REG             0x2470       /* SRU_EXT_MISCA */
#define INV_MISCA4_I_DAI_PB15_O_REG         0x2470       /* SRU_EXT_MISCA */
#define INV_MISCA5_I_DAI_PB15_O_REG         0x2470       /* SRU_EXT_MISCA */
#define SPIB_DS_I_DAI_PB15_O_REG            0x2470       /* SRU_EXT_MISCA */        
#define SPIB_MOSI_I_DAI_PB15_O_REG          0x2470       /* SRU_EXT_MISCA */        
#define SPIB_MISO_I_DAI_PB15_O_REG          0x2470       /* SRU_EXT_MISCA */        
#define MISCB0_I_DAI_PB15_O_REG             0x2471       /* SRU_EXT_MISCB */
#define TIMER0_I_DAI_PB15_O_REG             0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_22_I_DAI_PB15_O_REG         0x2471       /* SRU_EXT_MISCB */
#define MISCB1_I_DAI_PB15_O_REG             0x2471       /* SRU_EXT_MISCB */
#define TIMER1_I_DAI_PB15_O_REG             0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_23_I_DAI_PB15_O_REG         0x2471       /* SRU_EXT_MISCB */
#define MISCB2_I_DAI_PB15_O_REG             0x2471       /* SRU_EXT_MISCB */
#define TIMER2_I_DAI_PB15_O_REG             0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_24_I_DAI_PB15_O_REG         0x2471       /* SRU_EXT_MISCB */
#define MISCB3_I_DAI_PB15_O_REG             0x2471       /* SRU_EXT_MISCB */
#define FLAG10_I_DAI_PB15_O_REG             0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_25_I_DAI_PB15_O_REG         0x2471       /* SRU_EXT_MISCB */
#define MISCB4_I_DAI_PB15_O_REG             0x2471       /* SRU_EXT_MISCB */
#define FLAG11_I_DAI_PB15_O_REG             0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_26_I_DAI_PB15_O_REG         0x2471       /* SRU_EXT_MISCB */
#define MISCB5_I_DAI_PB15_O_REG             0x2471       /* SRU_EXT_MISCB */
#define FLAG12_I_DAI_PB15_O_REG             0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_27_I_DAI_PB15_O_REG         0x2471       /* SRU_EXT_MISCB */

#define MISCA0_I_DAI_PB16_O_REG             0x2470       /* SRU_EXT_MISCA */
#define FLAG13_I_DAI_PB16_O_REG             0x2470       /* SRU_EXT_MISCA */
#define DAI_INT_28_I_DAI_PB16_O_REG         0x2470       /* SRU_EXT_MISCA */
#define MISCA1_I_DAI_PB16_O_REG             0x2470       /* SRU_EXT_MISCA */
#define FLAG14_I_DAI_PB16_O_REG             0x2470       /* SRU_EXT_MISCA */
#define DAI_INT_29_I_DAI_PB16_O_REG         0x2470       /* SRU_EXT_MISCA */
#define MISCA2_I_DAI_PB16_O_REG             0x2470       /* SRU_EXT_MISCA */
#define FLAG15_I_DAI_PB16_O_REG             0x2470       /* SRU_EXT_MISCA */
#define DAI_INT_30_I_DAI_PB16_O_REG         0x2470       /* SRU_EXT_MISCA */
#define MISCA3_I_DAI_PB16_O_REG             0x2470       /* SRU_EXT_MISCA */
#define DAI_INT_31_I_DAI_PB16_O_REG         0x2470       /* SRU_EXT_MISCA */
#define MISCA4_I_DAI_PB16_O_REG             0x2470       /* SRU_EXT_MISCA */
#define MISCA5_I_DAI_PB16_O_REG             0x2470       /* SRU_EXT_MISCA */
#define INV_MISCA4_I_DAI_PB16_O_REG         0x2470       /* SRU_EXT_MISCA */
#define INV_MISCA5_I_DAI_PB16_O_REG         0x2470       /* SRU_EXT_MISCA */
#define SPIB_DS_I_DAI_PB16_O_REG            0x2470       /* SRU_EXT_MISCA */        
#define SPIB_MOSI_I_DAI_PB16_O_REG          0x2470       /* SRU_EXT_MISCA */        
#define SPIB_MISO_I_DAI_PB16_O_REG          0x2470       /* SRU_EXT_MISCA */        
#define MISCB0_I_DAI_PB16_O_REG             0x2471       /* SRU_EXT_MISCB */
#define TIMER0_I_DAI_PB16_O_REG             0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_22_I_DAI_PB16_O_REG         0x2471       /* SRU_EXT_MISCB */
#define MISCB1_I_DAI_PB16_O_REG             0x2471       /* SRU_EXT_MISCB */
#define TIMER1_I_DAI_PB16_O_REG             0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_23_I_DAI_PB16_O_REG         0x2471       /* SRU_EXT_MISCB */
#define MISCB2_I_DAI_PB16_O_REG             0x2471       /* SRU_EXT_MISCB */
#define TIMER2_I_DAI_PB16_O_REG             0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_24_I_DAI_PB16_O_REG         0x2471       /* SRU_EXT_MISCB */
#define MISCB3_I_DAI_PB16_O_REG             0x2471       /* SRU_EXT_MISCB */
#define FLAG10_I_DAI_PB16_O_REG             0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_25_I_DAI_PB16_O_REG         0x2471       /* SRU_EXT_MISCB */
#define MISCB4_I_DAI_PB16_O_REG             0x2471       /* SRU_EXT_MISCB */
#define FLAG11_I_DAI_PB16_O_REG             0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_26_I_DAI_PB16_O_REG         0x2471       /* SRU_EXT_MISCB */
#define MISCB5_I_DAI_PB16_O_REG             0x2471       /* SRU_EXT_MISCB */
#define FLAG12_I_DAI_PB16_O_REG             0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_27_I_DAI_PB16_O_REG         0x2471       /* SRU_EXT_MISCB */

#define MISCA0_I_DAI_PB17_O_REG             0x2470       /* SRU_EXT_MISCA */
#define FLAG13_I_DAI_PB17_O_REG             0x2470       /* SRU_EXT_MISCA */
#define DAI_INT_28_I_DAI_PB17_O_REG         0x2470       /* SRU_EXT_MISCA */
#define MISCA1_I_DAI_PB17_O_REG             0x2470       /* SRU_EXT_MISCA */
#define FLAG14_I_DAI_PB17_O_REG             0x2470       /* SRU_EXT_MISCA */
#define DAI_INT_29_I_DAI_PB17_O_REG         0x2470       /* SRU_EXT_MISCA */
#define MISCA2_I_DAI_PB17_O_REG             0x2470       /* SRU_EXT_MISCA */
#define FLAG15_I_DAI_PB17_O_REG             0x2470       /* SRU_EXT_MISCA */
#define DAI_INT_30_I_DAI_PB17_O_REG         0x2470       /* SRU_EXT_MISCA */
#define MISCA3_I_DAI_PB17_O_REG             0x2470       /* SRU_EXT_MISCA */
#define DAI_INT_31_I_DAI_PB17_O_REG         0x2470       /* SRU_EXT_MISCA */
#define MISCA4_I_DAI_PB17_O_REG             0x2470       /* SRU_EXT_MISCA */
#define MISCA5_I_DAI_PB17_O_REG             0x2470       /* SRU_EXT_MISCA */
#define INV_MISCA4_I_DAI_PB17_O_REG         0x2470       /* SRU_EXT_MISCA */
#define INV_MISCA5_I_DAI_PB17_O_REG         0x2470       /* SRU_EXT_MISCA */
#define SPIB_DS_I_DAI_PB17_O_REG            0x2470       /* SRU_EXT_MISCA */        
#define SPIB_MOSI_I_DAI_PB17_O_REG          0x2470       /* SRU_EXT_MISCA */        
#define SPIB_MISO_I_DAI_PB17_O_REG          0x2470       /* SRU_EXT_MISCA */        
#define MISCB0_I_DAI_PB17_O_REG             0x2471       /* SRU_EXT_MISCB */
#define TIMER0_I_DAI_PB17_O_REG             0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_22_I_DAI_PB17_O_REG         0x2471       /* SRU_EXT_MISCB */
#define MISCB1_I_DAI_PB17_O_REG             0x2471       /* SRU_EXT_MISCB */
#define TIMER1_I_DAI_PB17_O_REG             0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_23_I_DAI_PB17_O_REG         0x2471       /* SRU_EXT_MISCB */
#define MISCB2_I_DAI_PB17_O_REG             0x2471       /* SRU_EXT_MISCB */
#define TIMER2_I_DAI_PB17_O_REG             0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_24_I_DAI_PB17_O_REG         0x2471       /* SRU_EXT_MISCB */
#define MISCB3_I_DAI_PB17_O_REG             0x2471       /* SRU_EXT_MISCB */
#define FLAG10_I_DAI_PB17_O_REG             0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_25_I_DAI_PB17_O_REG         0x2471       /* SRU_EXT_MISCB */
#define MISCB4_I_DAI_PB17_O_REG             0x2471       /* SRU_EXT_MISCB */
#define FLAG11_I_DAI_PB17_O_REG             0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_26_I_DAI_PB17_O_REG         0x2471       /* SRU_EXT_MISCB */
#define MISCB5_I_DAI_PB17_O_REG             0x2471       /* SRU_EXT_MISCB */
#define FLAG12_I_DAI_PB17_O_REG             0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_27_I_DAI_PB17_O_REG         0x2471       /* SRU_EXT_MISCB */

#define MISCA0_I_DAI_PB18_O_REG             0x2470       /* SRU_EXT_MISCA */
#define FLAG13_I_DAI_PB18_O_REG             0x2470       /* SRU_EXT_MISCA */
#define DAI_INT_28_I_DAI_PB18_O_REG         0x2470       /* SRU_EXT_MISCA */
#define MISCA1_I_DAI_PB18_O_REG             0x2470       /* SRU_EXT_MISCA */
#define FLAG14_I_DAI_PB18_O_REG             0x2470       /* SRU_EXT_MISCA */
#define DAI_INT_29_I_DAI_PB18_O_REG         0x2470       /* SRU_EXT_MISCA */
#define MISCA2_I_DAI_PB18_O_REG             0x2470       /* SRU_EXT_MISCA */
#define FLAG15_I_DAI_PB18_O_REG             0x2470       /* SRU_EXT_MISCA */
#define DAI_INT_30_I_DAI_PB18_O_REG         0x2470       /* SRU_EXT_MISCA */
#define MISCA3_I_DAI_PB18_O_REG             0x2470       /* SRU_EXT_MISCA */
#define DAI_INT_31_I_DAI_PB18_O_REG         0x2470       /* SRU_EXT_MISCA */
#define MISCA4_I_DAI_PB18_O_REG             0x2470       /* SRU_EXT_MISCA */
#define MISCA5_I_DAI_PB18_O_REG             0x2470       /* SRU_EXT_MISCA */
#define INV_MISCA4_I_DAI_PB18_O_REG         0x2470       /* SRU_EXT_MISCA */
#define INV_MISCA5_I_DAI_PB18_O_REG         0x2470       /* SRU_EXT_MISCA */
#define SPIB_DS_I_DAI_PB18_O_REG            0x2470       /* SRU_EXT_MISCA */        
#define SPIB_MOSI_I_DAI_PB18_O_REG          0x2470       /* SRU_EXT_MISCA */        
#define SPIB_MISO_I_DAI_PB18_O_REG          0x2470       /* SRU_EXT_MISCA */        
#define MISCB0_I_DAI_PB18_O_REG             0x2471       /* SRU_EXT_MISCB */
#define TIMER0_I_DAI_PB18_O_REG             0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_22_I_DAI_PB18_O_REG         0x2471       /* SRU_EXT_MISCB */
#define MISCB1_I_DAI_PB18_O_REG             0x2471       /* SRU_EXT_MISCB */
#define TIMER1_I_DAI_PB18_O_REG             0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_23_I_DAI_PB18_O_REG         0x2471       /* SRU_EXT_MISCB */
#define MISCB2_I_DAI_PB18_O_REG             0x2471       /* SRU_EXT_MISCB */
#define TIMER2_I_DAI_PB18_O_REG             0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_24_I_DAI_PB18_O_REG         0x2471       /* SRU_EXT_MISCB */
#define MISCB3_I_DAI_PB18_O_REG             0x2471       /* SRU_EXT_MISCB */
#define FLAG10_I_DAI_PB18_O_REG             0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_25_I_DAI_PB18_O_REG         0x2471       /* SRU_EXT_MISCB */
#define MISCB4_I_DAI_PB18_O_REG             0x2471       /* SRU_EXT_MISCB */
#define FLAG11_I_DAI_PB18_O_REG             0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_26_I_DAI_PB18_O_REG         0x2471       /* SRU_EXT_MISCB */
#define MISCB5_I_DAI_PB18_O_REG             0x2471       /* SRU_EXT_MISCB */
#define FLAG12_I_DAI_PB18_O_REG             0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_27_I_DAI_PB18_O_REG         0x2471       /* SRU_EXT_MISCB */

#define MISCA0_I_DAI_PB19_O_REG             0x2470       /* SRU_EXT_MISCA */
#define FLAG13_I_DAI_PB19_O_REG             0x2470       /* SRU_EXT_MISCA */
#define DAI_INT_28_I_DAI_PB19_O_REG         0x2470       /* SRU_EXT_MISCA */
#define MISCA1_I_DAI_PB19_O_REG             0x2470       /* SRU_EXT_MISCA */
#define FLAG14_I_DAI_PB19_O_REG             0x2470       /* SRU_EXT_MISCA */
#define DAI_INT_29_I_DAI_PB19_O_REG         0x2470       /* SRU_EXT_MISCA */
#define MISCA2_I_DAI_PB19_O_REG             0x2470       /* SRU_EXT_MISCA */
#define FLAG15_I_DAI_PB19_O_REG             0x2470       /* SRU_EXT_MISCA */
#define DAI_INT_30_I_DAI_PB19_O_REG         0x2470       /* SRU_EXT_MISCA */
#define MISCA3_I_DAI_PB19_O_REG             0x2470       /* SRU_EXT_MISCA */
#define DAI_INT_31_I_DAI_PB19_O_REG         0x2470       /* SRU_EXT_MISCA */
#define MISCA4_I_DAI_PB19_O_REG             0x2470       /* SRU_EXT_MISCA */
#define MISCA5_I_DAI_PB19_O_REG             0x2470       /* SRU_EXT_MISCA */
#define INV_MISCA4_I_DAI_PB19_O_REG         0x2470       /* SRU_EXT_MISCA */
#define INV_MISCA5_I_DAI_PB19_O_REG         0x2470       /* SRU_EXT_MISCA */
#define SPIB_DS_I_DAI_PB19_O_REG            0x2470       /* SRU_EXT_MISCA */        
#define SPIB_MOSI_I_DAI_PB19_O_REG          0x2470       /* SRU_EXT_MISCA */        
#define SPIB_MISO_I_DAI_PB19_O_REG          0x2470       /* SRU_EXT_MISCA */        
#define MISCB0_I_DAI_PB19_O_REG             0x2471       /* SRU_EXT_MISCB */
#define TIMER0_I_DAI_PB19_O_REG             0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_22_I_DAI_PB19_O_REG         0x2471       /* SRU_EXT_MISCB */
#define MISCB1_I_DAI_PB19_O_REG             0x2471       /* SRU_EXT_MISCB */
#define TIMER1_I_DAI_PB19_O_REG             0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_23_I_DAI_PB19_O_REG         0x2471       /* SRU_EXT_MISCB */
#define MISCB2_I_DAI_PB19_O_REG             0x2471       /* SRU_EXT_MISCB */
#define TIMER2_I_DAI_PB19_O_REG             0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_24_I_DAI_PB19_O_REG         0x2471       /* SRU_EXT_MISCB */
#define MISCB3_I_DAI_PB19_O_REG             0x2471       /* SRU_EXT_MISCB */
#define FLAG10_I_DAI_PB19_O_REG             0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_25_I_DAI_PB19_O_REG         0x2471       /* SRU_EXT_MISCB */
#define MISCB4_I_DAI_PB19_O_REG             0x2471       /* SRU_EXT_MISCB */
#define FLAG11_I_DAI_PB19_O_REG             0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_26_I_DAI_PB19_O_REG         0x2471       /* SRU_EXT_MISCB */
#define MISCB5_I_DAI_PB19_O_REG             0x2471       /* SRU_EXT_MISCB */
#define FLAG12_I_DAI_PB19_O_REG             0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_27_I_DAI_PB19_O_REG         0x2471       /* SRU_EXT_MISCB */

#define MISCA0_I_DAI_PB20_O_REG             0x2470       /* SRU_EXT_MISCA */
#define FLAG13_I_DAI_PB20_O_REG             0x2470       /* SRU_EXT_MISCA */
#define DAI_INT_28_I_DAI_PB20_O_REG         0x2470       /* SRU_EXT_MISCA */
#define MISCA1_I_DAI_PB20_O_REG             0x2470       /* SRU_EXT_MISCA */
#define FLAG14_I_DAI_PB20_O_REG             0x2470       /* SRU_EXT_MISCA */
#define DAI_INT_29_I_DAI_PB20_O_REG         0x2470       /* SRU_EXT_MISCA */
#define MISCA2_I_DAI_PB20_O_REG             0x2470       /* SRU_EXT_MISCA */
#define FLAG15_I_DAI_PB20_O_REG             0x2470       /* SRU_EXT_MISCA */
#define DAI_INT_30_I_DAI_PB20_O_REG         0x2470       /* SRU_EXT_MISCA */
#define MISCA3_I_DAI_PB20_O_REG             0x2470       /* SRU_EXT_MISCA */
#define DAI_INT_31_I_DAI_PB20_O_REG         0x2470       /* SRU_EXT_MISCA */
#define MISCA4_I_DAI_PB20_O_REG             0x2470       /* SRU_EXT_MISCA */
#define MISCA5_I_DAI_PB20_O_REG             0x2470       /* SRU_EXT_MISCA */
#define INV_MISCA4_I_DAI_PB20_O_REG         0x2470       /* SRU_EXT_MISCA */
#define INV_MISCA5_I_DAI_PB20_O_REG         0x2470       /* SRU_EXT_MISCA */
#define SPIB_DS_I_DAI_PB20_O_REG            0x2470       /* SRU_EXT_MISCA */        
#define SPIB_MOSI_I_DAI_PB20_O_REG          0x2470       /* SRU_EXT_MISCA */        
#define SPIB_MISO_I_DAI_PB20_O_REG          0x2470       /* SRU_EXT_MISCA */        
#define MISCB0_I_DAI_PB20_O_REG             0x2471       /* SRU_EXT_MISCB */
#define TIMER0_I_DAI_PB20_O_REG             0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_22_I_DAI_PB20_O_REG         0x2471       /* SRU_EXT_MISCB */
#define MISCB1_I_DAI_PB20_O_REG             0x2471       /* SRU_EXT_MISCB */
#define TIMER1_I_DAI_PB20_O_REG             0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_23_I_DAI_PB20_O_REG         0x2471       /* SRU_EXT_MISCB */
#define MISCB2_I_DAI_PB20_O_REG             0x2471       /* SRU_EXT_MISCB */
#define TIMER2_I_DAI_PB20_O_REG             0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_24_I_DAI_PB20_O_REG         0x2471       /* SRU_EXT_MISCB */
#define MISCB3_I_DAI_PB20_O_REG             0x2471       /* SRU_EXT_MISCB */
#define FLAG10_I_DAI_PB20_O_REG             0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_25_I_DAI_PB20_O_REG         0x2471       /* SRU_EXT_MISCB */
#define MISCB4_I_DAI_PB20_O_REG             0x2471       /* SRU_EXT_MISCB */
#define FLAG11_I_DAI_PB20_O_REG             0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_26_I_DAI_PB20_O_REG         0x2471       /* SRU_EXT_MISCB */
#define MISCB5_I_DAI_PB20_O_REG             0x2471       /* SRU_EXT_MISCB */
#define FLAG12_I_DAI_PB20_O_REG             0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_27_I_DAI_PB20_O_REG         0x2471       /* SRU_EXT_MISCB */

#define MISCA0_I_TIMER0_O_REG               0x2470       /* SRU_EXT_MISCA */
#define FLAG13_I_TIMER0_O_REG               0x2470       /* SRU_EXT_MISCA */
#define DAI_INT_28_I_TIMER0_O_REG           0x2470       /* SRU_EXT_MISCA */
#define MISCA1_I_TIMER0_O_REG               0x2470       /* SRU_EXT_MISCA */
#define FLAG14_I_TIMER0_O_REG               0x2470       /* SRU_EXT_MISCA */
#define DAI_INT_29_I_TIMER0_O_REG           0x2470       /* SRU_EXT_MISCA */
#define MISCA2_I_TIMER0_O_REG               0x2470       /* SRU_EXT_MISCA */
#define FLAG15_I_TIMER0_O_REG               0x2470       /* SRU_EXT_MISCA */
#define DAI_INT_30_I_TIMER0_O_REG           0x2470       /* SRU_EXT_MISCA */
#define MISCA3_I_TIMER0_O_REG               0x2470       /* SRU_EXT_MISCA */
#define DAI_INT_31_I_TIMER0_O_REG           0x2470       /* SRU_EXT_MISCA */
#define MISCA4_I_TIMER0_O_REG               0x2470       /* SRU_EXT_MISCA */
#define MISCA5_I_TIMER0_O_REG               0x2470       /* SRU_EXT_MISCA */
#define INV_MISCA4_I_TIMER0_O_REG           0x2470       /* SRU_EXT_MISCA */
#define INV_MISCA5_I_TIMER0_O_REG           0x2470       /* SRU_EXT_MISCA */
#define SPIB_DS_I_TIMER0_O_REG              0x2470       /* SRU_EXT_MISCA */        
#define SPIB_MOSI_I_TIMER0_O_REG            0x2470       /* SRU_EXT_MISCA */        
#define SPIB_MISO_I_TIMER0_O_REG            0x2470       /* SRU_EXT_MISCA */
#define MISCB0_I_TIMER0_O_REG               0x2471       /* SRU_EXT_MISCB */
#define TIMER0_I_TIMER0_O_REG               0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_22_I_TIMER0_O_REG           0x2471       /* SRU_EXT_MISCB */
#define MISCB1_I_TIMER0_O_REG               0x2471       /* SRU_EXT_MISCB */
#define TIMER1_I_TIMER0_O_REG               0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_23_I_TIMER0_O_REG           0x2471       /* SRU_EXT_MISCB */
#define MISCB2_I_TIMER0_O_REG               0x2471       /* SRU_EXT_MISCB */
#define TIMER2_I_TIMER0_O_REG               0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_24_I_TIMER0_O_REG           0x2471       /* SRU_EXT_MISCB */
#define MISCB3_I_TIMER0_O_REG               0x2471       /* SRU_EXT_MISCB */
#define FLAG10_I_TIMER0_O_REG               0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_25_I_TIMER0_O_REG           0x2471       /* SRU_EXT_MISCB */
#define MISCB4_I_TIMER0_O_REG               0x2471       /* SRU_EXT_MISCB */
#define FLAG11_I_TIMER0_O_REG               0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_26_I_TIMER0_O_REG           0x2471       /* SRU_EXT_MISCB */
#define MISCB5_I_TIMER0_O_REG               0x2471       /* SRU_EXT_MISCB */
#define FLAG12_I_TIMER0_O_REG               0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_27_I_TIMER0_O_REG           0x2471       /* SRU_EXT_MISCB */

#define MISCA0_I_TIMER1_O_REG               0x2470       /* SRU_EXT_MISCA */
#define FLAG13_I_TIMER1_O_REG               0x2470       /* SRU_EXT_MISCA */
#define DAI_INT_28_I_TIMER1_O_REG           0x2470       /* SRU_EXT_MISCA */
#define MISCA1_I_TIMER1_O_REG               0x2470       /* SRU_EXT_MISCA */
#define FLAG14_I_TIMER1_O_REG               0x2470       /* SRU_EXT_MISCA */
#define DAI_INT_29_I_TIMER1_O_REG           0x2470       /* SRU_EXT_MISCA */
#define MISCA2_I_TIMER1_O_REG               0x2470       /* SRU_EXT_MISCA */
#define FLAG15_I_TIMER1_O_REG               0x2470       /* SRU_EXT_MISCA */
#define DAI_INT_30_I_TIMER1_O_REG           0x2470       /* SRU_EXT_MISCA */
#define MISCA3_I_TIMER1_O_REG               0x2470       /* SRU_EXT_MISCA */
#define DAI_INT_31_I_TIMER1_O_REG           0x2470       /* SRU_EXT_MISCA */
#define MISCA4_I_TIMER1_O_REG               0x2470       /* SRU_EXT_MISCA */
#define MISCA5_I_TIMER1_O_REG               0x2470       /* SRU_EXT_MISCA */
#define INV_MISCA4_I_TIMER1_O_REG           0x2470       /* SRU_EXT_MISCA */
#define INV_MISCA5_I_TIMER1_O_REG           0x2470       /* SRU_EXT_MISCA */
#define SPIB_DS_I_TIMER1_O_REG              0x2470       /* SRU_EXT_MISCA */        
#define SPIB_MOSI_I_TIMER1_O_REG            0x2470       /* SRU_EXT_MISCA */        
#define SPIB_MISO_I_TIMER1_O_REG            0x2470       /* SRU_EXT_MISCA */
#define MISCB0_I_TIMER1_O_REG               0x2471       /* SRU_EXT_MISCB */
#define TIMER0_I_TIMER1_O_REG               0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_22_I_TIMER1_O_REG           0x2471       /* SRU_EXT_MISCB */
#define MISCB1_I_TIMER1_O_REG               0x2471       /* SRU_EXT_MISCB */
#define TIMER1_I_TIMER1_O_REG               0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_23_I_TIMER1_O_REG           0x2471       /* SRU_EXT_MISCB */
#define MISCB2_I_TIMER1_O_REG               0x2471       /* SRU_EXT_MISCB */
#define TIMER2_I_TIMER1_O_REG               0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_24_I_TIMER1_O_REG           0x2471       /* SRU_EXT_MISCB */
#define MISCB3_I_TIMER1_O_REG               0x2471       /* SRU_EXT_MISCB */
#define FLAG10_I_TIMER1_O_REG               0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_25_I_TIMER1_O_REG           0x2471       /* SRU_EXT_MISCB */
#define MISCB4_I_TIMER1_O_REG               0x2471       /* SRU_EXT_MISCB */
#define FLAG11_I_TIMER1_O_REG               0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_26_I_TIMER1_O_REG           0x2471       /* SRU_EXT_MISCB */
#define MISCB5_I_TIMER1_O_REG               0x2471       /* SRU_EXT_MISCB */
#define FLAG12_I_TIMER1_O_REG               0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_27_I_TIMER1_O_REG           0x2471       /* SRU_EXT_MISCB */

#define MISCA0_I_TIMER2_O_REG               0x2470       /* SRU_EXT_MISCA */
#define FLAG13_I_TIMER2_O_REG               0x2470       /* SRU_EXT_MISCA */
#define DAI_INT_28_I_TIMER2_O_REG           0x2470       /* SRU_EXT_MISCA */
#define MISCA1_I_TIMER2_O_REG               0x2470       /* SRU_EXT_MISCA */
#define FLAG14_I_TIMER2_O_REG               0x2470       /* SRU_EXT_MISCA */
#define DAI_INT_29_I_TIMER2_O_REG           0x2470       /* SRU_EXT_MISCA */
#define MISCA2_I_TIMER2_O_REG               0x2470       /* SRU_EXT_MISCA */
#define FLAG15_I_TIMER2_O_REG               0x2470       /* SRU_EXT_MISCA */
#define DAI_INT_30_I_TIMER2_O_REG           0x2470       /* SRU_EXT_MISCA */
#define MISCA3_I_TIMER2_O_REG               0x2470       /* SRU_EXT_MISCA */
#define DAI_INT_31_I_TIMER2_O_REG           0x2470       /* SRU_EXT_MISCA */
#define MISCA4_I_TIMER2_O_REG               0x2470       /* SRU_EXT_MISCA */
#define MISCA5_I_TIMER2_O_REG               0x2470       /* SRU_EXT_MISCA */
#define INV_MISCA4_I_TIMER2_O_REG           0x2470       /* SRU_EXT_MISCA */
#define INV_MISCA5_I_TIMER2_O_REG           0x2470       /* SRU_EXT_MISCA */
#define SPIB_DS_I_TIMER2_O_REG              0x2470       /* SRU_EXT_MISCA */        
#define SPIB_MOSI_I_TIMER2_O_REG            0x2470       /* SRU_EXT_MISCA */        
#define SPIB_MISO_I_TIMER2_O_REG            0x2470       /* SRU_EXT_MISCA */
#define MISCB0_I_TIMER2_O_REG               0x2471       /* SRU_EXT_MISCB */
#define TIMER0_I_TIMER2_O_REG               0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_22_I_TIMER2_O_REG           0x2471       /* SRU_EXT_MISCB */
#define MISCB1_I_TIMER2_O_REG               0x2471       /* SRU_EXT_MISCB */
#define TIMER1_I_TIMER2_O_REG               0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_23_I_TIMER2_O_REG           0x2471       /* SRU_EXT_MISCB */
#define MISCB2_I_TIMER2_O_REG               0x2471       /* SRU_EXT_MISCB */
#define TIMER2_I_TIMER2_O_REG               0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_24_I_TIMER2_O_REG           0x2471       /* SRU_EXT_MISCB */
#define MISCB3_I_TIMER2_O_REG               0x2471       /* SRU_EXT_MISCB */
#define FLAG10_I_TIMER2_O_REG               0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_25_I_TIMER2_O_REG           0x2471       /* SRU_EXT_MISCB */
#define MISCB4_I_TIMER2_O_REG               0x2471       /* SRU_EXT_MISCB */
#define FLAG11_I_TIMER2_O_REG               0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_26_I_TIMER2_O_REG           0x2471       /* SRU_EXT_MISCB */
#define MISCB5_I_TIMER2_O_REG               0x2471       /* SRU_EXT_MISCB */
#define FLAG12_I_TIMER2_O_REG               0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_27_I_TIMER2_O_REG           0x2471       /* SRU_EXT_MISCB */

#define MISCA0_I_PDAP_STRB_O_REG            0x2470       /* SRU_EXT_MISCA */
#define FLAG13_I_PDAP_STRB_O_REG            0x2470       /* SRU_EXT_MISCA */
#define DAI_INT_28_I_PDAP_STRB_O_REG        0x2470       /* SRU_EXT_MISCA */
#define MISCA1_I_PDAP_STRB_O_REG            0x2470       /* SRU_EXT_MISCA */
#define FLAG14_I_PDAP_STRB_O_REG            0x2470       /* SRU_EXT_MISCA */
#define DAI_INT_29_I_PDAP_STRB_O_REG        0x2470       /* SRU_EXT_MISCA */
#define MISCA2_I_PDAP_STRB_O_REG            0x2470       /* SRU_EXT_MISCA */
#define FLAG15_I_PDAP_STRB_O_REG            0x2470       /* SRU_EXT_MISCA */
#define DAI_INT_30_I_PDAP_STRB_O_REG        0x2470       /* SRU_EXT_MISCA */
#define MISCA3_I_PDAP_STRB_O_REG            0x2470       /* SRU_EXT_MISCA */
#define DAI_INT_31_I_PDAP_STRB_O_REG        0x2470       /* SRU_EXT_MISCA */
#define MISCA4_I_PDAP_STRB_O_REG            0x2470       /* SRU_EXT_MISCA */
#define MISCA5_I_PDAP_STRB_O_REG            0x2470       /* SRU_EXT_MISCA */
#define INV_MISCA4_I_PDAP_STRB_O_REG        0x2470       /* SRU_EXT_MISCA */
#define INV_MISCA5_I_PDAP_STRB_O_REG        0x2470       /* SRU_EXT_MISCA */
#define SPIB_DS_I_PDAP_STRB_O_REG           0x2470       /* SRU_EXT_MISCA */        
#define SPIB_MOSI_I_PDAP_STRB_O_REG         0x2470       /* SRU_EXT_MISCA */        
#define SPIB_MISO_I_PDAP_STRB_O_REG         0x2470       /* SRU_EXT_MISCA */
#define MISCB0_I_PDAP_STRB_O_REG            0x2471       /* SRU_EXT_MISCB */
#define TIMER0_I_PDAP_STRB_O_REG            0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_22_I_PDAP_STRB_O_REG        0x2471       /* SRU_EXT_MISCB */
#define MISCB1_I_PDAP_STRB_O_REG            0x2471       /* SRU_EXT_MISCB */
#define TIMER1_I_PDAP_STRB_O_REG            0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_23_I_PDAP_STRB_O_REG        0x2471       /* SRU_EXT_MISCB */
#define MISCB2_I_PDAP_STRB_O_REG            0x2471       /* SRU_EXT_MISCB */
#define TIMER2_I_PDAP_STRB_O_REG            0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_24_I_PDAP_STRB_O_REG        0x2471       /* SRU_EXT_MISCB */
#define MISCB3_I_PDAP_STRB_O_REG            0x2471       /* SRU_EXT_MISCB */
#define FLAG10_I_PDAP_STRB_O_REG            0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_25_I_PDAP_STRB_O_REG        0x2471       /* SRU_EXT_MISCB */
#define MISCB4_I_PDAP_STRB_O_REG            0x2471       /* SRU_EXT_MISCB */
#define FLAG11_I_PDAP_STRB_O_REG            0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_26_I_PDAP_STRB_O_REG        0x2471       /* SRU_EXT_MISCB */
#define MISCB5_I_PDAP_STRB_O_REG            0x2471       /* SRU_EXT_MISCB */
#define FLAG12_I_PDAP_STRB_O_REG            0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_27_I_PDAP_STRB_O_REG        0x2471       /* SRU_EXT_MISCB */

#define MISCA0_I_PCG_CLKA_O_REG             0x2470       /* SRU_EXT_MISCA */
#define FLAG13_I_PCG_CLKA_O_REG             0x2470       /* SRU_EXT_MISCA */
#define DAI_INT_28_I_PCG_CLKA_O_REG         0x2470       /* SRU_EXT_MISCA */
#define MISCA1_I_PCG_CLKA_O_REG             0x2470       /* SRU_EXT_MISCA */
#define FLAG14_I_PCG_CLKA_O_REG             0x2470       /* SRU_EXT_MISCA */
#define DAI_INT_29_I_PCG_CLKA_O_REG         0x2470       /* SRU_EXT_MISCA */
#define MISCA2_I_PCG_CLKA_O_REG             0x2470       /* SRU_EXT_MISCA */
#define FLAG15_I_PCG_CLKA_O_REG             0x2470       /* SRU_EXT_MISCA */
#define DAI_INT_30_I_PCG_CLKA_O_REG         0x2470       /* SRU_EXT_MISCA */
#define MISCA3_I_PCG_CLKA_O_REG             0x2470       /* SRU_EXT_MISCA */
#define DAI_INT_31_I_PCG_CLKA_O_REG         0x2470       /* SRU_EXT_MISCA */
#define MISCA4_I_PCG_CLKA_O_REG             0x2470       /* SRU_EXT_MISCA */
#define MISCA5_I_PCG_CLKA_O_REG             0x2470       /* SRU_EXT_MISCA */
#define INV_MISCA4_I_PCG_CLKA_O_REG         0x2470       /* SRU_EXT_MISCA */
#define INV_MISCA5_I_PCG_CLKA_O_REG         0x2470       /* SRU_EXT_MISCA */
#define SPIB_DS_I_PCG_CLKA_O_REG            0x2470       /* SRU_EXT_MISCA */        
#define SPIB_MOSI_I_PCG_CLKA_O_REG          0x2470       /* SRU_EXT_MISCA */        
#define SPIB_MISO_I_PCG_CLKA_O_REG          0x2470       /* SRU_EXT_MISCA */
#define MISCB0_I_PCG_CLKA_O_REG             0x2471       /* SRU_EXT_MISCB */
#define TIMER0_I_PCG_CLKA_O_REG             0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_22_I_PCG_CLKA_O_REG         0x2471       /* SRU_EXT_MISCB */
#define MISCB1_I_PCG_CLKA_O_REG             0x2471       /* SRU_EXT_MISCB */
#define TIMER1_I_PCG_CLKA_O_REG             0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_23_I_PCG_CLKA_O_REG         0x2471       /* SRU_EXT_MISCB */
#define MISCB2_I_PCG_CLKA_O_REG             0x2471       /* SRU_EXT_MISCB */
#define TIMER2_I_PCG_CLKA_O_REG             0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_24_I_PCG_CLKA_O_REG         0x2471       /* SRU_EXT_MISCB */
#define MISCB3_I_PCG_CLKA_O_REG             0x2471       /* SRU_EXT_MISCB */
#define FLAG10_I_PCG_CLKA_O_REG             0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_25_I_PCG_CLKA_O_REG         0x2471       /* SRU_EXT_MISCB */
#define MISCB4_I_PCG_CLKA_O_REG             0x2471       /* SRU_EXT_MISCB */
#define FLAG11_I_PCG_CLKA_O_REG             0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_26_I_PCG_CLKA_O_REG         0x2471       /* SRU_EXT_MISCB */
#define MISCB5_I_PCG_CLKA_O_REG             0x2471       /* SRU_EXT_MISCB */
#define FLAG12_I_PCG_CLKA_O_REG             0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_27_I_PCG_CLKA_O_REG         0x2471       /* SRU_EXT_MISCB */

#define MISCA0_I_PCG_FSA_O_REG              0x2470       /* SRU_EXT_MISCA */
#define FLAG13_I_PCG_FSA_O_REG              0x2470       /* SRU_EXT_MISCA */
#define DAI_INT_28_I_PCG_FSA_O_REG          0x2470       /* SRU_EXT_MISCA */
#define MISCA1_I_PCG_FSA_O_REG              0x2470       /* SRU_EXT_MISCA */
#define FLAG14_I_PCG_FSA_O_REG              0x2470       /* SRU_EXT_MISCA */
#define DAI_INT_29_I_PCG_FSA_O_REG          0x2470       /* SRU_EXT_MISCA */
#define MISCA2_I_PCG_FSA_O_REG              0x2470       /* SRU_EXT_MISCA */
#define FLAG15_I_PCG_FSA_O_REG              0x2470       /* SRU_EXT_MISCA */
#define DAI_INT_30_I_PCG_FSA_O_REG          0x2470       /* SRU_EXT_MISCA */
#define MISCA3_I_PCG_FSA_O_REG              0x2470       /* SRU_EXT_MISCA */
#define DAI_INT_31_I_PCG_FSA_O_REG          0x2470       /* SRU_EXT_MISCA */
#define MISCA4_I_PCG_FSA_O_REG              0x2470       /* SRU_EXT_MISCA */
#define MISCA5_I_PCG_FSA_O_REG              0x2470       /* SRU_EXT_MISCA */
#define INV_MISCA4_I_PCG_FSA_O_REG          0x2470       /* SRU_EXT_MISCA */
#define INV_MISCA5_I_PCG_FSA_O_REG          0x2470       /* SRU_EXT_MISCA */
#define SPIB_DS_I_PCG_FSA_O_REG             0x2470       /* SRU_EXT_MISCA */        
#define SPIB_MOSI_I_PCG_FSA_O_REG           0x2470       /* SRU_EXT_MISCA */        
#define SPIB_MISO_I_PCG_FSA_O_REG           0x2470       /* SRU_EXT_MISCA */
#define MISCB0_I_PCG_FSA_O_REG              0x2471       /* SRU_EXT_MISCB */
#define TIMER0_I_PCG_FSA_O_REG              0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_22_I_PCG_FSA_O_REG          0x2471       /* SRU_EXT_MISCB */
#define MISCB1_I_PCG_FSA_O_REG              0x2471       /* SRU_EXT_MISCB */
#define TIMER1_I_PCG_FSA_O_REG              0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_23_I_PCG_FSA_O_REG          0x2471       /* SRU_EXT_MISCB */
#define MISCB2_I_PCG_FSA_O_REG              0x2471       /* SRU_EXT_MISCB */
#define TIMER2_I_PCG_FSA_O_REG              0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_24_I_PCG_FSA_O_REG          0x2471       /* SRU_EXT_MISCB */
#define MISCB3_I_PCG_FSA_O_REG              0x2471       /* SRU_EXT_MISCB */
#define FLAG10_I_PCG_FSA_O_REG              0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_25_I_PCG_FSA_O_REG          0x2471       /* SRU_EXT_MISCB */
#define MISCB4_I_PCG_FSA_O_REG              0x2471       /* SRU_EXT_MISCB */
#define FLAG11_I_PCG_FSA_O_REG              0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_26_I_PCG_FSA_O_REG          0x2471       /* SRU_EXT_MISCB */
#define MISCB5_I_PCG_FSA_O_REG              0x2471       /* SRU_EXT_MISCB */
#define FLAG12_I_PCG_FSA_O_REG              0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_27_I_PCG_FSA_O_REG          0x2471       /* SRU_EXT_MISCB */

#define MISCA0_I_PCG_CLKB_O_REG             0x2470       /* SRU_EXT_MISCA */
#define FLAG13_I_PCG_CLKB_O_REG             0x2470       /* SRU_EXT_MISCA */
#define DAI_INT_28_I_PCG_CLKB_O_REG         0x2470       /* SRU_EXT_MISCA */
#define MISCA1_I_PCG_CLKB_O_REG             0x2470       /* SRU_EXT_MISCA */
#define FLAG14_I_PCG_CLKB_O_REG             0x2470       /* SRU_EXT_MISCA */
#define DAI_INT_29_I_PCG_CLKB_O_REG         0x2470       /* SRU_EXT_MISCA */
#define MISCA2_I_PCG_CLKB_O_REG             0x2470       /* SRU_EXT_MISCA */
#define FLAG15_I_PCG_CLKB_O_REG             0x2470       /* SRU_EXT_MISCA */
#define DAI_INT_30_I_PCG_CLKB_O_REG         0x2470       /* SRU_EXT_MISCA */
#define MISCA3_I_PCG_CLKB_O_REG             0x2470       /* SRU_EXT_MISCA */
#define DAI_INT_31_I_PCG_CLKB_O_REG         0x2470       /* SRU_EXT_MISCA */
#define MISCA4_I_PCG_CLKB_O_REG             0x2470       /* SRU_EXT_MISCA */
#define MISCA5_I_PCG_CLKB_O_REG             0x2470       /* SRU_EXT_MISCA */
#define INV_MISCA4_I_PCG_CLKB_O_REG         0x2470       /* SRU_EXT_MISCA */
#define INV_MISCA5_I_PCG_CLKB_O_REG         0x2470       /* SRU_EXT_MISCA */
#define SPIB_DS_I_PCG_CLKB_O_REG            0x2470       /* SRU_EXT_MISCA */        
#define SPIB_MOSI_I_PCG_CLKB_O_REG          0x2470       /* SRU_EXT_MISCA */        
#define SPIB_MISO_I_PCG_CLKB_O_REG          0x2470       /* SRU_EXT_MISCA */
#define MISCB0_I_PCG_CLKB_O_REG             0x2471       /* SRU_EXT_MISCB */
#define TIMER0_I_PCG_CLKB_O_REG             0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_22_I_PCG_CLKB_O_REG         0x2471       /* SRU_EXT_MISCB */
#define MISCB1_I_PCG_CLKB_O_REG             0x2471       /* SRU_EXT_MISCB */
#define TIMER1_I_PCG_CLKB_O_REG             0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_23_I_PCG_CLKB_O_REG         0x2471       /* SRU_EXT_MISCB */
#define MISCB2_I_PCG_CLKB_O_REG             0x2471       /* SRU_EXT_MISCB */
#define TIMER2_I_PCG_CLKB_O_REG             0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_24_I_PCG_CLKB_O_REG         0x2471       /* SRU_EXT_MISCB */
#define MISCB3_I_PCG_CLKB_O_REG             0x2471       /* SRU_EXT_MISCB */
#define FLAG10_I_PCG_CLKB_O_REG             0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_25_I_PCG_CLKB_O_REG         0x2471       /* SRU_EXT_MISCB */
#define MISCB4_I_PCG_CLKB_O_REG             0x2471       /* SRU_EXT_MISCB */
#define FLAG11_I_PCG_CLKB_O_REG             0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_26_I_PCG_CLKB_O_REG         0x2471       /* SRU_EXT_MISCB */
#define MISCB5_I_PCG_CLKB_O_REG             0x2471       /* SRU_EXT_MISCB */
#define FLAG12_I_PCG_CLKB_O_REG             0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_27_I_PCG_CLKB_O_REG         0x2471       /* SRU_EXT_MISCB */

#define MISCA0_I_PCG_FSB_O_REG              0x2470       /* SRU_EXT_MISCA */
#define FLAG13_I_PCG_FSB_O_REG              0x2470       /* SRU_EXT_MISCA */
#define DAI_INT_28_I_PCG_FSB_O_REG          0x2470       /* SRU_EXT_MISCA */
#define MISCA1_I_PCG_FSB_O_REG              0x2470       /* SRU_EXT_MISCA */
#define FLAG14_I_PCG_FSB_O_REG              0x2470       /* SRU_EXT_MISCA */
#define DAI_INT_29_I_PCG_FSB_O_REG          0x2470       /* SRU_EXT_MISCA */
#define MISCA2_I_PCG_FSB_O_REG              0x2470       /* SRU_EXT_MISCA */
#define FLAG15_I_PCG_FSB_O_REG              0x2470       /* SRU_EXT_MISCA */
#define DAI_INT_30_I_PCG_FSB_O_REG          0x2470       /* SRU_EXT_MISCA */
#define MISCA3_I_PCG_FSB_O_REG              0x2470       /* SRU_EXT_MISCA */
#define DAI_INT_31_I_PCG_FSB_O_REG          0x2470       /* SRU_EXT_MISCA */
#define MISCA4_I_PCG_FSB_O_REG              0x2470       /* SRU_EXT_MISCA */
#define MISCA5_I_PCG_FSB_O_REG              0x2470       /* SRU_EXT_MISCA */
#define INV_MISCA4_I_PCG_FSB_O_REG          0x2470       /* SRU_EXT_MISCA */
#define INV_MISCA5_I_PCG_FSB_O_REG          0x2470       /* SRU_EXT_MISCA */
#define SPIB_DS_I_PCG_FSB_O_REG             0x2470       /* SRU_EXT_MISCA */        
#define SPIB_MOSI_I_PCG_FSB_O_REG           0x2470       /* SRU_EXT_MISCA */        
#define SPIB_MISO_I_PCG_FSB_O_REG           0x2470       /* SRU_EXT_MISCA */
#define MISCB0_I_PCG_FSB_O_REG              0x2471       /* SRU_EXT_MISCB */
#define TIMER0_I_PCG_FSB_O_REG              0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_22_I_PCG_FSB_O_REG          0x2471       /* SRU_EXT_MISCB */
#define MISCB1_I_PCG_FSB_O_REG              0x2471       /* SRU_EXT_MISCB */
#define TIMER1_I_PCG_FSB_O_REG              0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_23_I_PCG_FSB_O_REG          0x2471       /* SRU_EXT_MISCB */
#define MISCB2_I_PCG_FSB_O_REG              0x2471       /* SRU_EXT_MISCB */
#define TIMER2_I_PCG_FSB_O_REG              0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_24_I_PCG_FSB_O_REG          0x2471       /* SRU_EXT_MISCB */
#define MISCB3_I_PCG_FSB_O_REG              0x2471       /* SRU_EXT_MISCB */
#define FLAG10_I_PCG_FSB_O_REG              0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_25_I_PCG_FSB_O_REG          0x2471       /* SRU_EXT_MISCB */
#define MISCB4_I_PCG_FSB_O_REG              0x2471       /* SRU_EXT_MISCB */
#define FLAG11_I_PCG_FSB_O_REG              0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_26_I_PCG_FSB_O_REG          0x2471       /* SRU_EXT_MISCB */
#define MISCB5_I_PCG_FSB_O_REG              0x2471       /* SRU_EXT_MISCB */
#define FLAG12_I_PCG_FSB_O_REG              0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_27_I_PCG_FSB_O_REG          0x2471       /* SRU_EXT_MISCB */

#define MISCA0_I_MISC_LOW_O_REG             0x2470       /* SRU_EXT_MISCA */
#define FLAG13_I_MISC_LOW_O_REG             0x2470       /* SRU_EXT_MISCA */
#define DAI_INT_28_I_MISC_LOW_O_REG         0x2470       /* SRU_EXT_MISCA */
#define MISCA1_I_MISC_LOW_O_REG             0x2470       /* SRU_EXT_MISCA */
#define FLAG14_I_MISC_LOW_O_REG             0x2470       /* SRU_EXT_MISCA */
#define DAI_INT_29_I_MISC_LOW_O_REG         0x2470       /* SRU_EXT_MISCA */
#define MISCA2_I_MISC_LOW_O_REG             0x2470       /* SRU_EXT_MISCA */
#define FLAG15_I_MISC_LOW_O_REG             0x2470       /* SRU_EXT_MISCA */
#define DAI_INT_30_I_MISC_LOW_O_REG         0x2470       /* SRU_EXT_MISCA */
#define MISCA3_I_MISC_LOW_O_REG             0x2470       /* SRU_EXT_MISCA */
#define DAI_INT_31_I_MISC_LOW_O_REG         0x2470       /* SRU_EXT_MISCA */
#define MISCA4_I_MISC_LOW_O_REG             0x2470       /* SRU_EXT_MISCA */
#define MISCA5_I_MISC_LOW_O_REG             0x2470       /* SRU_EXT_MISCA */
#define INV_MISCA4_I_MISC_LOW_O_REG         0x2470       /* SRU_EXT_MISCA */
#define INV_MISCA5_I_MISC_LOW_O_REG         0x2470       /* SRU_EXT_MISCA */
#define SPIB_DS_I_MISC_LOW_O_REG            0x2470       /* SRU_EXT_MISCA */        
#define SPIB_MOSI_I_MISC_LOW_O_REG          0x2470       /* SRU_EXT_MISCA */        
#define SPIB_MISO_I_MISC_LOW_O_REG          0x2470       /* SRU_EXT_MISCA */
#define MISCB0_I_MISC_LOW_O_REG             0x2471       /* SRU_EXT_MISCB */
#define TIMER0_I_MISC_LOW_O_REG             0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_22_I_MISC_LOW_O_REG         0x2471       /* SRU_EXT_MISCB */
#define MISCB1_I_MISC_LOW_O_REG             0x2471       /* SRU_EXT_MISCB */
#define TIMER1_I_MISC_LOW_O_REG             0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_23_I_MISC_LOW_O_REG         0x2471       /* SRU_EXT_MISCB */
#define MISCB2_I_MISC_LOW_O_REG             0x2471       /* SRU_EXT_MISCB */
#define TIMER2_I_MISC_LOW_O_REG             0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_24_I_MISC_LOW_O_REG         0x2471       /* SRU_EXT_MISCB */
#define MISCB3_I_MISC_LOW_O_REG             0x2471       /* SRU_EXT_MISCB */
#define FLAG10_I_MISC_LOW_O_REG             0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_25_I_MISC_LOW_O_REG         0x2471       /* SRU_EXT_MISCB */
#define MISCB4_I_MISC_LOW_O_REG             0x2471       /* SRU_EXT_MISCB */
#define FLAG11_I_MISC_LOW_O_REG             0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_26_I_MISC_LOW_O_REG         0x2471       /* SRU_EXT_MISCB */
#define MISCB5_I_MISC_LOW_O_REG             0x2471       /* SRU_EXT_MISCB */
#define FLAG12_I_MISC_LOW_O_REG             0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_27_I_MISC_LOW_O_REG         0x2471       /* SRU_EXT_MISCB */

#define MISCA0_I_MISC_HIGH_O_REG            0x2470       /* SRU_EXT_MISCA */
#define FLAG13_I_MISC_HIGH_O_REG            0x2470       /* SRU_EXT_MISCA */
#define DAI_INT_28_I_MISC_HIGH_O_REG        0x2470       /* SRU_EXT_MISCA */
#define MISCA1_I_MISC_HIGH_O_REG            0x2470       /* SRU_EXT_MISCA */
#define FLAG14_I_MISC_HIGH_O_REG            0x2470       /* SRU_EXT_MISCA */
#define DAI_INT_29_I_MISC_HIGH_O_REG        0x2470       /* SRU_EXT_MISCA */
#define MISCA2_I_MISC_HIGH_O_REG            0x2470       /* SRU_EXT_MISCA */
#define FLAG15_I_MISC_HIGH_O_REG            0x2470       /* SRU_EXT_MISCA */
#define DAI_INT_30_I_MISC_HIGH_O_REG        0x2470       /* SRU_EXT_MISCA */
#define MISCA3_I_MISC_HIGH_O_REG            0x2470       /* SRU_EXT_MISCA */
#define DAI_INT_31_I_MISC_HIGH_O_REG        0x2470       /* SRU_EXT_MISCA */
#define MISCA4_I_MISC_HIGH_O_REG            0x2470       /* SRU_EXT_MISCA */
#define MISCA5_I_MISC_HIGH_O_REG            0x2470       /* SRU_EXT_MISCA */
#define INV_MISCA4_I_MISC_HIGH_O_REG        0x2470       /* SRU_EXT_MISCA */
#define INV_MISCA5_I_MISC_HIGH_O_REG        0x2470       /* SRU_EXT_MISCA */
#define SPIB_DS_I_MISC_HIGH_O_REG           0x2470       /* SRU_EXT_MISCA */        
#define SPIB_MOSI_I_MISC_HIGH_O_REG         0x2470       /* SRU_EXT_MISCA */        
#define SPIB_MISO_I_MISC_HIGH_O_REG         0x2470       /* SRU_EXT_MISCA */
#define MISCB0_I_MISC_HIGH_O_REG            0x2471       /* SRU_EXT_MISCB */
#define TIMER0_I_MISC_HIGH_O_REG            0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_22_I_MISC_HIGH_O_REG        0x2471       /* SRU_EXT_MISCB */
#define MISCB1_I_MISC_HIGH_O_REG            0x2471       /* SRU_EXT_MISCB */
#define TIMER1_I_MISC_HIGH_O_REG            0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_23_I_MISC_HIGH_O_REG        0x2471       /* SRU_EXT_MISCB */
#define MISCB2_I_MISC_HIGH_O_REG            0x2471       /* SRU_EXT_MISCB */
#define TIMER2_I_MISC_HIGH_O_REG            0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_24_I_MISC_HIGH_O_REG        0x2471       /* SRU_EXT_MISCB */
#define MISCB3_I_MISC_HIGH_O_REG            0x2471       /* SRU_EXT_MISCB */
#define FLAG10_I_MISC_HIGH_O_REG            0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_25_I_MISC_HIGH_O_REG        0x2471       /* SRU_EXT_MISCB */
#define MISCB4_I_MISC_HIGH_O_REG            0x2471       /* SRU_EXT_MISCB */
#define FLAG11_I_MISC_HIGH_O_REG            0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_26_I_MISC_HIGH_O_REG        0x2471       /* SRU_EXT_MISCB */
#define MISCB5_I_MISC_HIGH_O_REG            0x2471       /* SRU_EXT_MISCB */
#define FLAG12_I_MISC_HIGH_O_REG            0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_27_I_MISC_HIGH_O_REG        0x2471       /* SRU_EXT_MISCB */

#define MISCA0_I_LOW_REG                    0x2470       /* SRU_EXT_MISCA */
#define FLAG13_I_LOW_REG                    0x2470       /* SRU_EXT_MISCA */
#define DAI_INT_28_I_LOW_REG                0x2470       /* SRU_EXT_MISCA */
#define MISCA1_I_LOW_REG                    0x2470       /* SRU_EXT_MISCA */
#define FLAG14_I_LOW_REG                    0x2470       /* SRU_EXT_MISCA */
#define DAI_INT_29_I_LOW_REG                0x2470       /* SRU_EXT_MISCA */
#define MISCA2_I_LOW_REG                    0x2470       /* SRU_EXT_MISCA */
#define FLAG15_I_LOW_REG                    0x2470       /* SRU_EXT_MISCA */
#define DAI_INT_30_I_LOW_REG                0x2470       /* SRU_EXT_MISCA */
#define MISCA3_I_LOW_REG                    0x2470       /* SRU_EXT_MISCA */
#define DAI_INT_31_I_LOW_REG                0x2470       /* SRU_EXT_MISCA */
#define MISCA4_I_LOW_REG                    0x2470       /* SRU_EXT_MISCA */
#define MISCA5_I_LOW_REG                    0x2470       /* SRU_EXT_MISCA */
#define INV_MISCA4_I_LOW_REG                0x2470       /* SRU_EXT_MISCA */
#define INV_MISCA5_I_LOW_REG                0x2470       /* SRU_EXT_MISCA */
#define SPIB_DS_I_LOW_REG                   0x2470       /* SRU_EXT_MISCA */        
#define SPIB_MOSI_I_LOW_REG                 0x2470       /* SRU_EXT_MISCA */        
#define SPIB_MISO_I_LOW_REG                 0x2470       /* SRU_EXT_MISCA */
#define MISCB0_I_LOW_REG                    0x2471       /* SRU_EXT_MISCB */
#define TIMER0_I_LOW_REG                    0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_22_I_LOW_REG                0x2471       /* SRU_EXT_MISCB */
#define MISCB1_I_LOW_REG                    0x2471       /* SRU_EXT_MISCB */
#define TIMER1_I_LOW_REG                    0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_23_I_LOW_REG                0x2471       /* SRU_EXT_MISCB */
#define MISCB2_I_LOW_REG                    0x2471       /* SRU_EXT_MISCB */
#define TIMER2_I_LOW_REG                    0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_24_I_LOW_REG                0x2471       /* SRU_EXT_MISCB */
#define MISCB3_I_LOW_REG                    0x2471       /* SRU_EXT_MISCB */
#define FLAG10_I_LOW_REG                    0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_25_I_LOW_REG                0x2471       /* SRU_EXT_MISCB */
#define MISCB4_I_LOW_REG                    0x2471       /* SRU_EXT_MISCB */
#define FLAG11_I_LOW_REG                    0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_26_I_LOW_REG                0x2471       /* SRU_EXT_MISCB */
#define MISCB5_I_LOW_REG                    0x2471       /* SRU_EXT_MISCB */
#define FLAG12_I_LOW_REG                    0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_27_I_LOW_REG                0x2471       /* SRU_EXT_MISCB */

#define MISCA0_I_HIGH_REG                   0x2470       /* SRU_EXT_MISCA */
#define FLAG13_I_HIGH_REG                   0x2470       /* SRU_EXT_MISCA */
#define DAI_INT_28_I_HIGH_REG               0x2470       /* SRU_EXT_MISCA */
#define MISCA1_I_HIGH_REG                   0x2470       /* SRU_EXT_MISCA */
#define FLAG14_I_HIGH_REG                   0x2470       /* SRU_EXT_MISCA */
#define DAI_INT_29_I_HIGH_REG               0x2470       /* SRU_EXT_MISCA */
#define MISCA2_I_HIGH_REG                   0x2470       /* SRU_EXT_MISCA */
#define FLAG15_I_HIGH_REG                   0x2470       /* SRU_EXT_MISCA */
#define DAI_INT_30_I_HIGH_REG               0x2470       /* SRU_EXT_MISCA */
#define MISCA3_I_HIGH_REG                   0x2470       /* SRU_EXT_MISCA */
#define DAI_INT_31_I_HIGH_REG               0x2470       /* SRU_EXT_MISCA */
#define MISCA4_I_HIGH_REG                   0x2470       /* SRU_EXT_MISCA */
#define MISCA5_I_HIGH_REG                   0x2470       /* SRU_EXT_MISCA */
#define INV_MISCA4_I_HIGH_REG               0x2470       /* SRU_EXT_MISCA */
#define INV_MISCA5_I_HIGH_REG               0x2470       /* SRU_EXT_MISCA */
#define SPIB_DS_I_HIGH_REG                  0x2470       /* SRU_EXT_MISCA */        
#define SPIB_MOSI_I_HIGH_REG                0x2470       /* SRU_EXT_MISCA */        
#define SPIB_MISO_I_HIGH_REG                0x2470       /* SRU_EXT_MISCA */
#define MISCB0_I_HIGH_REG                   0x2471       /* SRU_EXT_MISCB */
#define TIMER0_I_HIGH_REG                   0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_22_I_HIGH_REG               0x2471       /* SRU_EXT_MISCB */
#define MISCB1_I_HIGH_REG                   0x2471       /* SRU_EXT_MISCB */
#define TIMER1_I_HIGH_REG                   0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_23_I_HIGH_REG               0x2471       /* SRU_EXT_MISCB */
#define MISCB2_I_HIGH_REG                   0x2471       /* SRU_EXT_MISCB */
#define TIMER2_I_HIGH_REG                   0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_24_I_HIGH_REG               0x2471       /* SRU_EXT_MISCB */
#define MISCB3_I_HIGH_REG                   0x2471       /* SRU_EXT_MISCB */
#define FLAG10_I_HIGH_REG                   0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_25_I_HIGH_REG               0x2471       /* SRU_EXT_MISCB */
#define MISCB4_I_HIGH_REG                   0x2471       /* SRU_EXT_MISCB */
#define FLAG11_I_HIGH_REG                   0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_26_I_HIGH_REG               0x2471       /* SRU_EXT_MISCB */
#define MISCB5_I_HIGH_REG                   0x2471       /* SRU_EXT_MISCB */
#define FLAG12_I_HIGH_REG                   0x2471       /* SRU_EXT_MISCB */
#define DAI_INT_27_I_HIGH_REG               0x2471       /* SRU_EXT_MISCB */

#define MISCA4_INVERT_LOW_REG               0x2470
#define MISCA5_INVERT_LOW_REG               0x2470       
                                                              
#define MISCA4_INVERT_ZERO_REG              0x2470       
#define MISCA5_INVERT_ZERO_REG              0x2470       
                                                              
#define INV_MISCA4_LOW_REG                  0x2470       
#define INV_MISCA5_LOW_REG                  0x2470       
                                                              
#define INV_MISCA4_ZERO_REG                 0x2470       
#define INV_MISCA5_ZERO_REG                 0x2470       
                                                              
#define MISCA4_INVERT_HIGH_REG              0x2470       
#define MISCA5_INVERT_HIGH_REG              0x2470       
                                                              
#define MISCA4_INVERT_ONE_REG               0x2470       
#define MISCA5_INVERT_ONE_REG               0x2470       
                                                              
#define INV_MISCA4_HIGH_REG                 0x2470       
#define INV_MISCA5_HIGH_REG                 0x2470       
                                                              
#define INV_MISCA4_ONE_REG                  0x2470       
#define INV_MISCA5_ONE_REG                  0x2470       
                                          

/*============================================================================*
 *                                                                            *
 *      GROUP F                                                               *
 *                                                                            *
 *============================================================================*/

//------------------------------------------------------------------------------
//  Group F: Input bitfield masks & shifts
//------------------------------------------------------------------------------

#define PBEN01_I_MASK                               0xFFFFFFC0
#define PBEN01_I_SHIFT                              0
#define PBEN02_I_MASK                               0xFFFFF03F
#define PBEN02_I_SHIFT                              6
#define PBEN03_I_MASK                               0xFFFC0FFF
#define PBEN03_I_SHIFT                              12
#define PBEN04_I_MASK                               0xFF03FFFF
#define PBEN04_I_SHIFT                              18
#define PBEN05_I_MASK                               0xC0FFFFFF
#define PBEN05_I_SHIFT                              24

#define PBEN06_I_MASK                               0xFFFFFFC0
#define PBEN06_I_SHIFT                              0
#define PBEN07_I_MASK                               0xFFFFF03F
#define PBEN07_I_SHIFT                              6
#define PBEN08_I_MASK                               0xFFFC0FFF
#define PBEN08_I_SHIFT                              12
#define PBEN09_I_MASK                               0xFF03FFFF
#define PBEN09_I_SHIFT                              18
#define PBEN10_I_MASK                               0xC0FFFFFF
#define PBEN10_I_SHIFT                              24

#define PBEN11_I_MASK                               0xFFFFFFC0
#define PBEN11_I_SHIFT                              0
#define PBEN12_I_MASK                               0xFFFFF03F
#define PBEN12_I_SHIFT                              6
#define PBEN13_I_MASK                               0xFFFC0FFF
#define PBEN13_I_SHIFT                              12
#define PBEN14_I_MASK                               0xFF03FFFF
#define PBEN14_I_SHIFT                              18
#define PBEN15_I_MASK                               0xC0FFFFFF
#define PBEN15_I_SHIFT                              24

#define PBEN16_I_MASK                               0xFFFFFFC0
#define PBEN16_I_SHIFT                              0
#define PBEN17_I_MASK                               0xFFFFF03F
#define PBEN17_I_SHIFT                              6
#define PBEN18_I_MASK                               0xFFFC0FFF
#define PBEN18_I_SHIFT                              12
#define PBEN19_I_MASK                               0xFF03FFFF
#define PBEN19_I_SHIFT                              18
#define PBEN20_I_MASK                               0xC0FFFFFF
#define PBEN20_I_SHIFT                              24



//------------------------------------------------------------------------------
//  Group F: Bitfield encodings
//------------------------------------------------------------------------------

#define PBEN01_I_LOW_ENCODING                       0x00
#define PBEN02_I_LOW_ENCODING                       0x00
#define PBEN03_I_LOW_ENCODING                       0x00
#define PBEN04_I_LOW_ENCODING                       0x00
#define PBEN05_I_LOW_ENCODING                       0x00
#define PBEN06_I_LOW_ENCODING                       0x00
#define PBEN07_I_LOW_ENCODING                       0x00
#define PBEN08_I_LOW_ENCODING                       0x00
#define PBEN09_I_LOW_ENCODING                       0x00
#define PBEN10_I_LOW_ENCODING                       0x00
#define PBEN11_I_LOW_ENCODING                       0x00
#define PBEN12_I_LOW_ENCODING                       0x00
#define PBEN13_I_LOW_ENCODING                       0x00
#define PBEN14_I_LOW_ENCODING                       0x00
#define PBEN15_I_LOW_ENCODING                       0x00
#define PBEN16_I_LOW_ENCODING                       0x00
#define PBEN17_I_LOW_ENCODING                       0x00
#define PBEN18_I_LOW_ENCODING                       0x00
#define PBEN19_I_LOW_ENCODING                       0x00
#define PBEN20_I_LOW_ENCODING                       0x00

#define PBEN01_I_HIGH_ENCODING                      0x01
#define PBEN02_I_HIGH_ENCODING                      0x01
#define PBEN03_I_HIGH_ENCODING                      0x01
#define PBEN04_I_HIGH_ENCODING                      0x01
#define PBEN05_I_HIGH_ENCODING                      0x01
#define PBEN06_I_HIGH_ENCODING                      0x01
#define PBEN07_I_HIGH_ENCODING                      0x01
#define PBEN08_I_HIGH_ENCODING                      0x01
#define PBEN09_I_HIGH_ENCODING                      0x01
#define PBEN10_I_HIGH_ENCODING                      0x01
#define PBEN11_I_HIGH_ENCODING                      0x01
#define PBEN12_I_HIGH_ENCODING                      0x01
#define PBEN13_I_HIGH_ENCODING                      0x01
#define PBEN14_I_HIGH_ENCODING                      0x01
#define PBEN15_I_HIGH_ENCODING                      0x01
#define PBEN16_I_HIGH_ENCODING                      0x01
#define PBEN17_I_HIGH_ENCODING                      0x01
#define PBEN18_I_HIGH_ENCODING                      0x01
#define PBEN19_I_HIGH_ENCODING                      0x01
#define PBEN20_I_HIGH_ENCODING                      0x01

#define PBEN01_I_PBEN_LOW_O_ENCODING                0x00
#define PBEN02_I_PBEN_LOW_O_ENCODING                0x00
#define PBEN03_I_PBEN_LOW_O_ENCODING                0x00
#define PBEN04_I_PBEN_LOW_O_ENCODING                0x00
#define PBEN05_I_PBEN_LOW_O_ENCODING                0x00
#define PBEN06_I_PBEN_LOW_O_ENCODING                0x00
#define PBEN07_I_PBEN_LOW_O_ENCODING                0x00
#define PBEN08_I_PBEN_LOW_O_ENCODING                0x00
#define PBEN09_I_PBEN_LOW_O_ENCODING                0x00
#define PBEN10_I_PBEN_LOW_O_ENCODING                0x00
#define PBEN11_I_PBEN_LOW_O_ENCODING                0x00
#define PBEN12_I_PBEN_LOW_O_ENCODING                0x00
#define PBEN13_I_PBEN_LOW_O_ENCODING                0x00
#define PBEN14_I_PBEN_LOW_O_ENCODING                0x00
#define PBEN15_I_PBEN_LOW_O_ENCODING                0x00
#define PBEN16_I_PBEN_LOW_O_ENCODING                0x00
#define PBEN17_I_PBEN_LOW_O_ENCODING                0x00
#define PBEN18_I_PBEN_LOW_O_ENCODING                0x00
#define PBEN19_I_PBEN_LOW_O_ENCODING                0x00
#define PBEN20_I_PBEN_LOW_O_ENCODING                0x00

#define PBEN01_I_PBEN_HIGH_O_ENCODING               0x01
#define PBEN02_I_PBEN_HIGH_O_ENCODING               0x01
#define PBEN03_I_PBEN_HIGH_O_ENCODING               0x01
#define PBEN04_I_PBEN_HIGH_O_ENCODING               0x01
#define PBEN05_I_PBEN_HIGH_O_ENCODING               0x01
#define PBEN06_I_PBEN_HIGH_O_ENCODING               0x01
#define PBEN07_I_PBEN_HIGH_O_ENCODING               0x01
#define PBEN08_I_PBEN_HIGH_O_ENCODING               0x01
#define PBEN09_I_PBEN_HIGH_O_ENCODING               0x01
#define PBEN10_I_PBEN_HIGH_O_ENCODING               0x01
#define PBEN11_I_PBEN_HIGH_O_ENCODING               0x01
#define PBEN12_I_PBEN_HIGH_O_ENCODING               0x01
#define PBEN13_I_PBEN_HIGH_O_ENCODING               0x01
#define PBEN14_I_PBEN_HIGH_O_ENCODING               0x01
#define PBEN15_I_PBEN_HIGH_O_ENCODING               0x01
#define PBEN16_I_PBEN_HIGH_O_ENCODING               0x01
#define PBEN17_I_PBEN_HIGH_O_ENCODING               0x01
#define PBEN18_I_PBEN_HIGH_O_ENCODING               0x01
#define PBEN19_I_PBEN_HIGH_O_ENCODING               0x01
#define PBEN20_I_PBEN_HIGH_O_ENCODING               0x01

#define PBEN01_I__MISCA_0_PBEN_O_ENCODING        0x02
#define PBEN02_I__MISCA_0_PBEN_O_ENCODING        0x02
#define PBEN03_I__MISCA_0_PBEN_O_ENCODING        0x02
#define PBEN04_I__MISCA_0_PBEN_O_ENCODING        0x02
#define PBEN05_I__MISCA_0_PBEN_O_ENCODING        0x02
#define PBEN06_I__MISCA_0_PBEN_O_ENCODING        0x02
#define PBEN07_I__MISCA_0_PBEN_O_ENCODING        0x02
#define PBEN08_I__MISCA_0_PBEN_O_ENCODING        0x02
#define PBEN09_I__MISCA_0_PBEN_O_ENCODING        0x02
#define PBEN10_I__MISCA_0_PBEN_O_ENCODING        0x02
#define PBEN11_I__MISCA_0_PBEN_O_ENCODING        0x02
#define PBEN12_I__MISCA_0_PBEN_O_ENCODING        0x02
#define PBEN13_I__MISCA_0_PBEN_O_ENCODING        0x02
#define PBEN14_I__MISCA_0_PBEN_O_ENCODING        0x02
#define PBEN15_I__MISCA_0_PBEN_O_ENCODING        0x02
#define PBEN16_I__MISCA_0_PBEN_O_ENCODING        0x02
#define PBEN17_I__MISCA_0_PBEN_O_ENCODING        0x02
#define PBEN18_I__MISCA_0_PBEN_O_ENCODING        0x02
#define PBEN19_I__MISCA_0_PBEN_O_ENCODING        0x02
#define PBEN20_I__MISCA_0_PBEN_O_ENCODING        0x02

#define PBEN01_I__MISCA_1_PBEN_O_ENCODING        0x03
#define PBEN02_I__MISCA_1_PBEN_O_ENCODING        0x03
#define PBEN03_I__MISCA_1_PBEN_O_ENCODING        0x03
#define PBEN04_I__MISCA_1_PBEN_O_ENCODING        0x03
#define PBEN05_I__MISCA_1_PBEN_O_ENCODING        0x03
#define PBEN06_I__MISCA_1_PBEN_O_ENCODING        0x03
#define PBEN07_I__MISCA_1_PBEN_O_ENCODING        0x03
#define PBEN08_I__MISCA_1_PBEN_O_ENCODING        0x03
#define PBEN09_I__MISCA_1_PBEN_O_ENCODING        0x03
#define PBEN10_I__MISCA_1_PBEN_O_ENCODING        0x03
#define PBEN11_I__MISCA_1_PBEN_O_ENCODING        0x03
#define PBEN12_I__MISCA_1_PBEN_O_ENCODING        0x03
#define PBEN13_I__MISCA_1_PBEN_O_ENCODING        0x03
#define PBEN14_I__MISCA_1_PBEN_O_ENCODING        0x03
#define PBEN15_I__MISCA_1_PBEN_O_ENCODING        0x03
#define PBEN16_I__MISCA_1_PBEN_O_ENCODING        0x03
#define PBEN17_I__MISCA_1_PBEN_O_ENCODING        0x03
#define PBEN18_I__MISCA_1_PBEN_O_ENCODING        0x03
#define PBEN19_I__MISCA_1_PBEN_O_ENCODING        0x03
#define PBEN20_I__MISCA_1_PBEN_O_ENCODING        0x03

#define PBEN01_I__MISCA_2_PBEN_O_ENCODING        0x04
#define PBEN02_I__MISCA_2_PBEN_O_ENCODING        0x04
#define PBEN03_I__MISCA_2_PBEN_O_ENCODING        0x04
#define PBEN04_I__MISCA_2_PBEN_O_ENCODING        0x04
#define PBEN05_I__MISCA_2_PBEN_O_ENCODING        0x04
#define PBEN06_I__MISCA_2_PBEN_O_ENCODING        0x04
#define PBEN07_I__MISCA_2_PBEN_O_ENCODING        0x04
#define PBEN08_I__MISCA_2_PBEN_O_ENCODING        0x04
#define PBEN09_I__MISCA_2_PBEN_O_ENCODING        0x04
#define PBEN10_I__MISCA_2_PBEN_O_ENCODING        0x04
#define PBEN11_I__MISCA_2_PBEN_O_ENCODING        0x04
#define PBEN12_I__MISCA_2_PBEN_O_ENCODING        0x04
#define PBEN13_I__MISCA_2_PBEN_O_ENCODING        0x04
#define PBEN14_I__MISCA_2_PBEN_O_ENCODING        0x04
#define PBEN15_I__MISCA_2_PBEN_O_ENCODING        0x04
#define PBEN16_I__MISCA_2_PBEN_O_ENCODING        0x04
#define PBEN17_I__MISCA_2_PBEN_O_ENCODING        0x04
#define PBEN18_I__MISCA_2_PBEN_O_ENCODING        0x04
#define PBEN19_I__MISCA_2_PBEN_O_ENCODING        0x04
#define PBEN20_I__MISCA_2_PBEN_O_ENCODING        0x04

#define PBEN01_I__MISCA_3_PBEN_O_ENCODING        0x05
#define PBEN02_I__MISCA_3_PBEN_O_ENCODING        0x05
#define PBEN03_I__MISCA_3_PBEN_O_ENCODING        0x05
#define PBEN04_I__MISCA_3_PBEN_O_ENCODING        0x05
#define PBEN05_I__MISCA_3_PBEN_O_ENCODING        0x05
#define PBEN06_I__MISCA_3_PBEN_O_ENCODING        0x05
#define PBEN07_I__MISCA_3_PBEN_O_ENCODING        0x05
#define PBEN08_I__MISCA_3_PBEN_O_ENCODING        0x05
#define PBEN09_I__MISCA_3_PBEN_O_ENCODING        0x05
#define PBEN10_I__MISCA_3_PBEN_O_ENCODING        0x05
#define PBEN11_I__MISCA_3_PBEN_O_ENCODING        0x05
#define PBEN12_I__MISCA_3_PBEN_O_ENCODING        0x05
#define PBEN13_I__MISCA_3_PBEN_O_ENCODING        0x05
#define PBEN14_I__MISCA_3_PBEN_O_ENCODING        0x05
#define PBEN15_I__MISCA_3_PBEN_O_ENCODING        0x05
#define PBEN16_I__MISCA_3_PBEN_O_ENCODING        0x05
#define PBEN17_I__MISCA_3_PBEN_O_ENCODING        0x05
#define PBEN18_I__MISCA_3_PBEN_O_ENCODING        0x05
#define PBEN19_I__MISCA_3_PBEN_O_ENCODING        0x05
#define PBEN20_I__MISCA_3_PBEN_O_ENCODING        0x05

#define PBEN01_I__MISCA_4_PBEN_O_ENCODING        0x06
#define PBEN02_I__MISCA_4_PBEN_O_ENCODING        0x06
#define PBEN03_I__MISCA_4_PBEN_O_ENCODING        0x06
#define PBEN04_I__MISCA_4_PBEN_O_ENCODING        0x06
#define PBEN05_I__MISCA_4_PBEN_O_ENCODING        0x06
#define PBEN06_I__MISCA_4_PBEN_O_ENCODING        0x06
#define PBEN07_I__MISCA_4_PBEN_O_ENCODING        0x06
#define PBEN08_I__MISCA_4_PBEN_O_ENCODING        0x06
#define PBEN09_I__MISCA_4_PBEN_O_ENCODING        0x06
#define PBEN10_I__MISCA_4_PBEN_O_ENCODING        0x06
#define PBEN11_I__MISCA_4_PBEN_O_ENCODING        0x06
#define PBEN12_I__MISCA_4_PBEN_O_ENCODING        0x06
#define PBEN13_I__MISCA_4_PBEN_O_ENCODING        0x06
#define PBEN14_I__MISCA_4_PBEN_O_ENCODING        0x06
#define PBEN15_I__MISCA_4_PBEN_O_ENCODING        0x06
#define PBEN16_I__MISCA_4_PBEN_O_ENCODING        0x06
#define PBEN17_I__MISCA_4_PBEN_O_ENCODING        0x06
#define PBEN18_I__MISCA_4_PBEN_O_ENCODING        0x06
#define PBEN19_I__MISCA_4_PBEN_O_ENCODING        0x06
#define PBEN20_I__MISCA_4_PBEN_O_ENCODING        0x06

#define PBEN01_I__MISCA_5_PBEN_O_ENCODING        0x07
#define PBEN02_I__MISCA_5_PBEN_O_ENCODING        0x07
#define PBEN03_I__MISCA_5_PBEN_O_ENCODING        0x07
#define PBEN04_I__MISCA_5_PBEN_O_ENCODING        0x07
#define PBEN05_I__MISCA_5_PBEN_O_ENCODING        0x07
#define PBEN06_I__MISCA_5_PBEN_O_ENCODING        0x07
#define PBEN07_I__MISCA_5_PBEN_O_ENCODING        0x07
#define PBEN08_I__MISCA_5_PBEN_O_ENCODING        0x07
#define PBEN09_I__MISCA_5_PBEN_O_ENCODING        0x07
#define PBEN10_I__MISCA_5_PBEN_O_ENCODING        0x07
#define PBEN11_I__MISCA_5_PBEN_O_ENCODING        0x07
#define PBEN12_I__MISCA_5_PBEN_O_ENCODING        0x07
#define PBEN13_I__MISCA_5_PBEN_O_ENCODING        0x07
#define PBEN14_I__MISCA_5_PBEN_O_ENCODING        0x07
#define PBEN15_I__MISCA_5_PBEN_O_ENCODING        0x07
#define PBEN16_I__MISCA_5_PBEN_O_ENCODING        0x07
#define PBEN17_I__MISCA_5_PBEN_O_ENCODING        0x07
#define PBEN18_I__MISCA_5_PBEN_O_ENCODING        0x07
#define PBEN19_I__MISCA_5_PBEN_O_ENCODING        0x07
#define PBEN20_I__MISCA_5_PBEN_O_ENCODING        0x07

#define PBEN01_I_SPORT0_CLK_PBEN_O_ENCODING         0x08
#define PBEN02_I_SPORT0_CLK_PBEN_O_ENCODING         0x08
#define PBEN03_I_SPORT0_CLK_PBEN_O_ENCODING         0x08
#define PBEN04_I_SPORT0_CLK_PBEN_O_ENCODING         0x08
#define PBEN05_I_SPORT0_CLK_PBEN_O_ENCODING         0x08
#define PBEN06_I_SPORT0_CLK_PBEN_O_ENCODING         0x08
#define PBEN07_I_SPORT0_CLK_PBEN_O_ENCODING         0x08
#define PBEN08_I_SPORT0_CLK_PBEN_O_ENCODING         0x08
#define PBEN09_I_SPORT0_CLK_PBEN_O_ENCODING         0x08
#define PBEN10_I_SPORT0_CLK_PBEN_O_ENCODING         0x08
#define PBEN11_I_SPORT0_CLK_PBEN_O_ENCODING         0x08
#define PBEN12_I_SPORT0_CLK_PBEN_O_ENCODING         0x08
#define PBEN13_I_SPORT0_CLK_PBEN_O_ENCODING         0x08
#define PBEN14_I_SPORT0_CLK_PBEN_O_ENCODING         0x08
#define PBEN15_I_SPORT0_CLK_PBEN_O_ENCODING         0x08
#define PBEN16_I_SPORT0_CLK_PBEN_O_ENCODING         0x08
#define PBEN17_I_SPORT0_CLK_PBEN_O_ENCODING         0x08
#define PBEN18_I_SPORT0_CLK_PBEN_O_ENCODING         0x08
#define PBEN19_I_SPORT0_CLK_PBEN_O_ENCODING         0x08
#define PBEN20_I_SPORT0_CLK_PBEN_O_ENCODING         0x08

#define PBEN01_I_SPORT0_FS_PBEN_O_ENCODING          0x09
#define PBEN02_I_SPORT0_FS_PBEN_O_ENCODING          0x09
#define PBEN03_I_SPORT0_FS_PBEN_O_ENCODING          0x09
#define PBEN04_I_SPORT0_FS_PBEN_O_ENCODING          0x09
#define PBEN05_I_SPORT0_FS_PBEN_O_ENCODING          0x09
#define PBEN06_I_SPORT0_FS_PBEN_O_ENCODING          0x09
#define PBEN07_I_SPORT0_FS_PBEN_O_ENCODING          0x09
#define PBEN08_I_SPORT0_FS_PBEN_O_ENCODING          0x09
#define PBEN09_I_SPORT0_FS_PBEN_O_ENCODING          0x09
#define PBEN10_I_SPORT0_FS_PBEN_O_ENCODING          0x09
#define PBEN11_I_SPORT0_FS_PBEN_O_ENCODING          0x09
#define PBEN12_I_SPORT0_FS_PBEN_O_ENCODING          0x09
#define PBEN13_I_SPORT0_FS_PBEN_O_ENCODING          0x09
#define PBEN14_I_SPORT0_FS_PBEN_O_ENCODING          0x09
#define PBEN15_I_SPORT0_FS_PBEN_O_ENCODING          0x09
#define PBEN16_I_SPORT0_FS_PBEN_O_ENCODING          0x09
#define PBEN17_I_SPORT0_FS_PBEN_O_ENCODING          0x09
#define PBEN18_I_SPORT0_FS_PBEN_O_ENCODING          0x09
#define PBEN19_I_SPORT0_FS_PBEN_O_ENCODING          0x09
#define PBEN20_I_SPORT0_FS_PBEN_O_ENCODING          0x09

#define PBEN01_I_SPORT0_DA_PBEN_O_ENCODING          0x0A
#define PBEN02_I_SPORT0_DA_PBEN_O_ENCODING          0x0A
#define PBEN03_I_SPORT0_DA_PBEN_O_ENCODING          0x0A
#define PBEN04_I_SPORT0_DA_PBEN_O_ENCODING          0x0A
#define PBEN05_I_SPORT0_DA_PBEN_O_ENCODING          0x0A
#define PBEN06_I_SPORT0_DA_PBEN_O_ENCODING          0x0A
#define PBEN07_I_SPORT0_DA_PBEN_O_ENCODING          0x0A
#define PBEN08_I_SPORT0_DA_PBEN_O_ENCODING          0x0A
#define PBEN09_I_SPORT0_DA_PBEN_O_ENCODING          0x0A
#define PBEN10_I_SPORT0_DA_PBEN_O_ENCODING          0x0A
#define PBEN11_I_SPORT0_DA_PBEN_O_ENCODING          0x0A
#define PBEN12_I_SPORT0_DA_PBEN_O_ENCODING          0x0A
#define PBEN13_I_SPORT0_DA_PBEN_O_ENCODING          0x0A
#define PBEN14_I_SPORT0_DA_PBEN_O_ENCODING          0x0A
#define PBEN15_I_SPORT0_DA_PBEN_O_ENCODING          0x0A
#define PBEN16_I_SPORT0_DA_PBEN_O_ENCODING          0x0A
#define PBEN17_I_SPORT0_DA_PBEN_O_ENCODING          0x0A
#define PBEN18_I_SPORT0_DA_PBEN_O_ENCODING          0x0A
#define PBEN19_I_SPORT0_DA_PBEN_O_ENCODING          0x0A
#define PBEN20_I_SPORT0_DA_PBEN_O_ENCODING          0x0A

#define PBEN01_I_SPORT0_Db_PBEN_O_ENCODING          0x0B
#define PBEN02_I_SPORT0_Db_PBEN_O_ENCODING          0x0B
#define PBEN03_I_SPORT0_Db_PBEN_O_ENCODING          0x0B
#define PBEN04_I_SPORT0_Db_PBEN_O_ENCODING          0x0B
#define PBEN05_I_SPORT0_Db_PBEN_O_ENCODING          0x0B
#define PBEN06_I_SPORT0_Db_PBEN_O_ENCODING          0x0B
#define PBEN07_I_SPORT0_Db_PBEN_O_ENCODING          0x0B
#define PBEN08_I_SPORT0_Db_PBEN_O_ENCODING          0x0B
#define PBEN09_I_SPORT0_Db_PBEN_O_ENCODING          0x0B
#define PBEN10_I_SPORT0_Db_PBEN_O_ENCODING          0x0B
#define PBEN11_I_SPORT0_Db_PBEN_O_ENCODING          0x0B
#define PBEN12_I_SPORT0_Db_PBEN_O_ENCODING          0x0B
#define PBEN13_I_SPORT0_Db_PBEN_O_ENCODING          0x0B
#define PBEN14_I_SPORT0_Db_PBEN_O_ENCODING          0x0B
#define PBEN15_I_SPORT0_Db_PBEN_O_ENCODING          0x0B
#define PBEN16_I_SPORT0_Db_PBEN_O_ENCODING          0x0B
#define PBEN17_I_SPORT0_Db_PBEN_O_ENCODING          0x0B
#define PBEN18_I_SPORT0_Db_PBEN_O_ENCODING          0x0B
#define PBEN19_I_SPORT0_Db_PBEN_O_ENCODING          0x0B
#define PBEN20_I_SPORT0_Db_PBEN_O_ENCODING          0x0B

#define PBEN01_I_SPORT1_CLK_PBEN_O_ENCODING         0x0C
#define PBEN02_I_SPORT1_CLK_PBEN_O_ENCODING         0x0C
#define PBEN03_I_SPORT1_CLK_PBEN_O_ENCODING         0x0C
#define PBEN04_I_SPORT1_CLK_PBEN_O_ENCODING         0x0C
#define PBEN05_I_SPORT1_CLK_PBEN_O_ENCODING         0x0C
#define PBEN06_I_SPORT1_CLK_PBEN_O_ENCODING         0x0C
#define PBEN07_I_SPORT1_CLK_PBEN_O_ENCODING         0x0C
#define PBEN08_I_SPORT1_CLK_PBEN_O_ENCODING         0x0C
#define PBEN09_I_SPORT1_CLK_PBEN_O_ENCODING         0x0C
#define PBEN10_I_SPORT1_CLK_PBEN_O_ENCODING         0x0C
#define PBEN11_I_SPORT1_CLK_PBEN_O_ENCODING         0x0C
#define PBEN12_I_SPORT1_CLK_PBEN_O_ENCODING         0x0C
#define PBEN13_I_SPORT1_CLK_PBEN_O_ENCODING         0x0C
#define PBEN14_I_SPORT1_CLK_PBEN_O_ENCODING         0x0C
#define PBEN15_I_SPORT1_CLK_PBEN_O_ENCODING         0x0C
#define PBEN16_I_SPORT1_CLK_PBEN_O_ENCODING         0x0C
#define PBEN17_I_SPORT1_CLK_PBEN_O_ENCODING         0x0C
#define PBEN18_I_SPORT1_CLK_PBEN_O_ENCODING         0x0C
#define PBEN19_I_SPORT1_CLK_PBEN_O_ENCODING         0x0C
#define PBEN20_I_SPORT1_CLK_PBEN_O_ENCODING         0x0C

#define PBEN01_I_SPORT1_FS_PBEN_O_ENCODING          0x0D
#define PBEN02_I_SPORT1_FS_PBEN_O_ENCODING          0x0D
#define PBEN03_I_SPORT1_FS_PBEN_O_ENCODING          0x0D
#define PBEN04_I_SPORT1_FS_PBEN_O_ENCODING          0x0D
#define PBEN05_I_SPORT1_FS_PBEN_O_ENCODING          0x0D
#define PBEN06_I_SPORT1_FS_PBEN_O_ENCODING          0x0D
#define PBEN07_I_SPORT1_FS_PBEN_O_ENCODING          0x0D
#define PBEN08_I_SPORT1_FS_PBEN_O_ENCODING          0x0D
#define PBEN09_I_SPORT1_FS_PBEN_O_ENCODING          0x0D
#define PBEN10_I_SPORT1_FS_PBEN_O_ENCODING          0x0D
#define PBEN11_I_SPORT1_FS_PBEN_O_ENCODING          0x0D
#define PBEN12_I_SPORT1_FS_PBEN_O_ENCODING          0x0D
#define PBEN13_I_SPORT1_FS_PBEN_O_ENCODING          0x0D
#define PBEN14_I_SPORT1_FS_PBEN_O_ENCODING          0x0D
#define PBEN15_I_SPORT1_FS_PBEN_O_ENCODING          0x0D
#define PBEN16_I_SPORT1_FS_PBEN_O_ENCODING          0x0D
#define PBEN17_I_SPORT1_FS_PBEN_O_ENCODING          0x0D
#define PBEN18_I_SPORT1_FS_PBEN_O_ENCODING          0x0D
#define PBEN19_I_SPORT1_FS_PBEN_O_ENCODING          0x0D
#define PBEN20_I_SPORT1_FS_PBEN_O_ENCODING          0x0D

#define PBEN01_I_SPORT1_DA_PBEN_O_ENCODING          0x0E
#define PBEN02_I_SPORT1_DA_PBEN_O_ENCODING          0x0E
#define PBEN03_I_SPORT1_DA_PBEN_O_ENCODING          0x0E
#define PBEN04_I_SPORT1_DA_PBEN_O_ENCODING          0x0E
#define PBEN05_I_SPORT1_DA_PBEN_O_ENCODING          0x0E
#define PBEN06_I_SPORT1_DA_PBEN_O_ENCODING          0x0E
#define PBEN07_I_SPORT1_DA_PBEN_O_ENCODING          0x0E
#define PBEN08_I_SPORT1_DA_PBEN_O_ENCODING          0x0E
#define PBEN09_I_SPORT1_DA_PBEN_O_ENCODING          0x0E
#define PBEN10_I_SPORT1_DA_PBEN_O_ENCODING          0x0E
#define PBEN11_I_SPORT1_DA_PBEN_O_ENCODING          0x0E
#define PBEN12_I_SPORT1_DA_PBEN_O_ENCODING          0x0E
#define PBEN13_I_SPORT1_DA_PBEN_O_ENCODING          0x0E
#define PBEN14_I_SPORT1_DA_PBEN_O_ENCODING          0x0E
#define PBEN15_I_SPORT1_DA_PBEN_O_ENCODING          0x0E
#define PBEN16_I_SPORT1_DA_PBEN_O_ENCODING          0x0E
#define PBEN17_I_SPORT1_DA_PBEN_O_ENCODING          0x0E
#define PBEN18_I_SPORT1_DA_PBEN_O_ENCODING          0x0E
#define PBEN19_I_SPORT1_DA_PBEN_O_ENCODING          0x0E
#define PBEN20_I_SPORT1_DA_PBEN_O_ENCODING          0x0E

#define PBEN01_I_SPORT1_Db_PBEN_O_ENCODING          0x0F
#define PBEN02_I_SPORT1_Db_PBEN_O_ENCODING          0x0F
#define PBEN03_I_SPORT1_Db_PBEN_O_ENCODING          0x0F
#define PBEN04_I_SPORT1_Db_PBEN_O_ENCODING          0x0F
#define PBEN05_I_SPORT1_Db_PBEN_O_ENCODING          0x0F
#define PBEN06_I_SPORT1_Db_PBEN_O_ENCODING          0x0F
#define PBEN07_I_SPORT1_Db_PBEN_O_ENCODING          0x0F
#define PBEN08_I_SPORT1_Db_PBEN_O_ENCODING          0x0F
#define PBEN09_I_SPORT1_Db_PBEN_O_ENCODING          0x0F
#define PBEN10_I_SPORT1_Db_PBEN_O_ENCODING          0x0F
#define PBEN11_I_SPORT1_Db_PBEN_O_ENCODING          0x0F
#define PBEN12_I_SPORT1_Db_PBEN_O_ENCODING          0x0F
#define PBEN13_I_SPORT1_Db_PBEN_O_ENCODING          0x0F
#define PBEN14_I_SPORT1_Db_PBEN_O_ENCODING          0x0F
#define PBEN15_I_SPORT1_Db_PBEN_O_ENCODING          0x0F
#define PBEN16_I_SPORT1_Db_PBEN_O_ENCODING          0x0F
#define PBEN17_I_SPORT1_Db_PBEN_O_ENCODING          0x0F
#define PBEN18_I_SPORT1_Db_PBEN_O_ENCODING          0x0F
#define PBEN19_I_SPORT1_Db_PBEN_O_ENCODING          0x0F
#define PBEN20_I_SPORT1_Db_PBEN_O_ENCODING          0x0F

#define PBEN01_I_SPORT2_CLK_PBEN_O_ENCODING         0x10
#define PBEN02_I_SPORT2_CLK_PBEN_O_ENCODING         0x10
#define PBEN03_I_SPORT2_CLK_PBEN_O_ENCODING         0x10
#define PBEN04_I_SPORT2_CLK_PBEN_O_ENCODING         0x10
#define PBEN05_I_SPORT2_CLK_PBEN_O_ENCODING         0x10
#define PBEN06_I_SPORT2_CLK_PBEN_O_ENCODING         0x10
#define PBEN07_I_SPORT2_CLK_PBEN_O_ENCODING         0x10
#define PBEN08_I_SPORT2_CLK_PBEN_O_ENCODING         0x10
#define PBEN09_I_SPORT2_CLK_PBEN_O_ENCODING         0x10
#define PBEN10_I_SPORT2_CLK_PBEN_O_ENCODING         0x10
#define PBEN11_I_SPORT2_CLK_PBEN_O_ENCODING         0x10
#define PBEN12_I_SPORT2_CLK_PBEN_O_ENCODING         0x10
#define PBEN13_I_SPORT2_CLK_PBEN_O_ENCODING         0x10
#define PBEN14_I_SPORT2_CLK_PBEN_O_ENCODING         0x10
#define PBEN15_I_SPORT2_CLK_PBEN_O_ENCODING         0x10
#define PBEN16_I_SPORT2_CLK_PBEN_O_ENCODING         0x10
#define PBEN17_I_SPORT2_CLK_PBEN_O_ENCODING         0x10
#define PBEN18_I_SPORT2_CLK_PBEN_O_ENCODING         0x10
#define PBEN19_I_SPORT2_CLK_PBEN_O_ENCODING         0x10
#define PBEN20_I_SPORT2_CLK_PBEN_O_ENCODING         0x10

#define PBEN01_I_SPORT2_FS_PBEN_O_ENCODING          0x11
#define PBEN02_I_SPORT2_FS_PBEN_O_ENCODING          0x11
#define PBEN03_I_SPORT2_FS_PBEN_O_ENCODING          0x11
#define PBEN04_I_SPORT2_FS_PBEN_O_ENCODING          0x11
#define PBEN05_I_SPORT2_FS_PBEN_O_ENCODING          0x11
#define PBEN06_I_SPORT2_FS_PBEN_O_ENCODING          0x11
#define PBEN07_I_SPORT2_FS_PBEN_O_ENCODING          0x11
#define PBEN08_I_SPORT2_FS_PBEN_O_ENCODING          0x11
#define PBEN09_I_SPORT2_FS_PBEN_O_ENCODING          0x11
#define PBEN10_I_SPORT2_FS_PBEN_O_ENCODING          0x11
#define PBEN11_I_SPORT2_FS_PBEN_O_ENCODING          0x11
#define PBEN12_I_SPORT2_FS_PBEN_O_ENCODING          0x11
#define PBEN13_I_SPORT2_FS_PBEN_O_ENCODING          0x11
#define PBEN14_I_SPORT2_FS_PBEN_O_ENCODING          0x11
#define PBEN15_I_SPORT2_FS_PBEN_O_ENCODING          0x11
#define PBEN16_I_SPORT2_FS_PBEN_O_ENCODING          0x11
#define PBEN17_I_SPORT2_FS_PBEN_O_ENCODING          0x11
#define PBEN18_I_SPORT2_FS_PBEN_O_ENCODING          0x11
#define PBEN19_I_SPORT2_FS_PBEN_O_ENCODING          0x11
#define PBEN20_I_SPORT2_FS_PBEN_O_ENCODING          0x11

#define PBEN01_I_SPORT2_DA_PBEN_O_ENCODING          0x12
#define PBEN02_I_SPORT2_DA_PBEN_O_ENCODING          0x12
#define PBEN03_I_SPORT2_DA_PBEN_O_ENCODING          0x12
#define PBEN04_I_SPORT2_DA_PBEN_O_ENCODING          0x12
#define PBEN05_I_SPORT2_DA_PBEN_O_ENCODING          0x12
#define PBEN06_I_SPORT2_DA_PBEN_O_ENCODING          0x12
#define PBEN07_I_SPORT2_DA_PBEN_O_ENCODING          0x12
#define PBEN08_I_SPORT2_DA_PBEN_O_ENCODING          0x12
#define PBEN09_I_SPORT2_DA_PBEN_O_ENCODING          0x12
#define PBEN10_I_SPORT2_DA_PBEN_O_ENCODING          0x12
#define PBEN11_I_SPORT2_DA_PBEN_O_ENCODING          0x12
#define PBEN12_I_SPORT2_DA_PBEN_O_ENCODING          0x12
#define PBEN13_I_SPORT2_DA_PBEN_O_ENCODING          0x12
#define PBEN14_I_SPORT2_DA_PBEN_O_ENCODING          0x12
#define PBEN15_I_SPORT2_DA_PBEN_O_ENCODING          0x12
#define PBEN16_I_SPORT2_DA_PBEN_O_ENCODING          0x12
#define PBEN17_I_SPORT2_DA_PBEN_O_ENCODING          0x12
#define PBEN18_I_SPORT2_DA_PBEN_O_ENCODING          0x12
#define PBEN19_I_SPORT2_DA_PBEN_O_ENCODING          0x12
#define PBEN20_I_SPORT2_DA_PBEN_O_ENCODING          0x12

#define PBEN01_I_SPORT2_Db_PBEN_O_ENCODING          0x13
#define PBEN02_I_SPORT2_Db_PBEN_O_ENCODING          0x13
#define PBEN03_I_SPORT2_Db_PBEN_O_ENCODING          0x13
#define PBEN04_I_SPORT2_Db_PBEN_O_ENCODING          0x13
#define PBEN05_I_SPORT2_Db_PBEN_O_ENCODING          0x13
#define PBEN06_I_SPORT2_Db_PBEN_O_ENCODING          0x13
#define PBEN07_I_SPORT2_Db_PBEN_O_ENCODING          0x13
#define PBEN08_I_SPORT2_Db_PBEN_O_ENCODING          0x13
#define PBEN09_I_SPORT2_Db_PBEN_O_ENCODING          0x13
#define PBEN10_I_SPORT2_Db_PBEN_O_ENCODING          0x13
#define PBEN11_I_SPORT2_Db_PBEN_O_ENCODING          0x13
#define PBEN12_I_SPORT2_Db_PBEN_O_ENCODING          0x13
#define PBEN13_I_SPORT2_Db_PBEN_O_ENCODING          0x13
#define PBEN14_I_SPORT2_Db_PBEN_O_ENCODING          0x13
#define PBEN15_I_SPORT2_Db_PBEN_O_ENCODING          0x13
#define PBEN16_I_SPORT2_Db_PBEN_O_ENCODING          0x13
#define PBEN17_I_SPORT2_Db_PBEN_O_ENCODING          0x13
#define PBEN18_I_SPORT2_Db_PBEN_O_ENCODING          0x13
#define PBEN19_I_SPORT2_Db_PBEN_O_ENCODING          0x13
#define PBEN20_I_SPORT2_Db_PBEN_O_ENCODING          0x13

#define PBEN01_I_SPORT3_CLK_PBEN_O_ENCODING         0x14
#define PBEN02_I_SPORT3_CLK_PBEN_O_ENCODING         0x14
#define PBEN03_I_SPORT3_CLK_PBEN_O_ENCODING         0x14
#define PBEN04_I_SPORT3_CLK_PBEN_O_ENCODING         0x14
#define PBEN05_I_SPORT3_CLK_PBEN_O_ENCODING         0x14
#define PBEN06_I_SPORT3_CLK_PBEN_O_ENCODING         0x14
#define PBEN07_I_SPORT3_CLK_PBEN_O_ENCODING         0x14
#define PBEN08_I_SPORT3_CLK_PBEN_O_ENCODING         0x14
#define PBEN09_I_SPORT3_CLK_PBEN_O_ENCODING         0x14
#define PBEN10_I_SPORT3_CLK_PBEN_O_ENCODING         0x14
#define PBEN11_I_SPORT3_CLK_PBEN_O_ENCODING         0x14
#define PBEN12_I_SPORT3_CLK_PBEN_O_ENCODING         0x14
#define PBEN13_I_SPORT3_CLK_PBEN_O_ENCODING         0x14
#define PBEN14_I_SPORT3_CLK_PBEN_O_ENCODING         0x14
#define PBEN15_I_SPORT3_CLK_PBEN_O_ENCODING         0x14
#define PBEN16_I_SPORT3_CLK_PBEN_O_ENCODING         0x14
#define PBEN17_I_SPORT3_CLK_PBEN_O_ENCODING         0x14
#define PBEN18_I_SPORT3_CLK_PBEN_O_ENCODING         0x14
#define PBEN19_I_SPORT3_CLK_PBEN_O_ENCODING         0x14
#define PBEN20_I_SPORT3_CLK_PBEN_O_ENCODING         0x14

#define PBEN01_I_SPORT3_FS_PBEN_O_ENCODING          0x15
#define PBEN02_I_SPORT3_FS_PBEN_O_ENCODING          0x15
#define PBEN03_I_SPORT3_FS_PBEN_O_ENCODING          0x15
#define PBEN04_I_SPORT3_FS_PBEN_O_ENCODING          0x15
#define PBEN05_I_SPORT3_FS_PBEN_O_ENCODING          0x15
#define PBEN06_I_SPORT3_FS_PBEN_O_ENCODING          0x15
#define PBEN07_I_SPORT3_FS_PBEN_O_ENCODING          0x15
#define PBEN08_I_SPORT3_FS_PBEN_O_ENCODING          0x15
#define PBEN09_I_SPORT3_FS_PBEN_O_ENCODING          0x15
#define PBEN10_I_SPORT3_FS_PBEN_O_ENCODING          0x15
#define PBEN11_I_SPORT3_FS_PBEN_O_ENCODING          0x15
#define PBEN12_I_SPORT3_FS_PBEN_O_ENCODING          0x15
#define PBEN13_I_SPORT3_FS_PBEN_O_ENCODING          0x15
#define PBEN14_I_SPORT3_FS_PBEN_O_ENCODING          0x15
#define PBEN15_I_SPORT3_FS_PBEN_O_ENCODING          0x15
#define PBEN16_I_SPORT3_FS_PBEN_O_ENCODING          0x15
#define PBEN17_I_SPORT3_FS_PBEN_O_ENCODING          0x15
#define PBEN18_I_SPORT3_FS_PBEN_O_ENCODING          0x15
#define PBEN19_I_SPORT3_FS_PBEN_O_ENCODING          0x15
#define PBEN20_I_SPORT3_FS_PBEN_O_ENCODING          0x15

#define PBEN01_I_SPORT3_DA_PBEN_O_ENCODING          0x16
#define PBEN02_I_SPORT3_DA_PBEN_O_ENCODING          0x16
#define PBEN03_I_SPORT3_DA_PBEN_O_ENCODING          0x16
#define PBEN04_I_SPORT3_DA_PBEN_O_ENCODING          0x16
#define PBEN05_I_SPORT3_DA_PBEN_O_ENCODING          0x16
#define PBEN06_I_SPORT3_DA_PBEN_O_ENCODING          0x16
#define PBEN07_I_SPORT3_DA_PBEN_O_ENCODING          0x16
#define PBEN08_I_SPORT3_DA_PBEN_O_ENCODING          0x16
#define PBEN09_I_SPORT3_DA_PBEN_O_ENCODING          0x16
#define PBEN10_I_SPORT3_DA_PBEN_O_ENCODING          0x16
#define PBEN11_I_SPORT3_DA_PBEN_O_ENCODING          0x16
#define PBEN12_I_SPORT3_DA_PBEN_O_ENCODING          0x16
#define PBEN13_I_SPORT3_DA_PBEN_O_ENCODING          0x16
#define PBEN14_I_SPORT3_DA_PBEN_O_ENCODING          0x16
#define PBEN15_I_SPORT3_DA_PBEN_O_ENCODING          0x16
#define PBEN16_I_SPORT3_DA_PBEN_O_ENCODING          0x16
#define PBEN17_I_SPORT3_DA_PBEN_O_ENCODING          0x16
#define PBEN18_I_SPORT3_DA_PBEN_O_ENCODING          0x16
#define PBEN19_I_SPORT3_DA_PBEN_O_ENCODING          0x16
#define PBEN20_I_SPORT3_DA_PBEN_O_ENCODING          0x16

#define PBEN01_I_SPORT3_Db_PBEN_O_ENCODING          0x17
#define PBEN02_I_SPORT3_Db_PBEN_O_ENCODING          0x17
#define PBEN03_I_SPORT3_Db_PBEN_O_ENCODING          0x17
#define PBEN04_I_SPORT3_Db_PBEN_O_ENCODING          0x17
#define PBEN05_I_SPORT3_Db_PBEN_O_ENCODING          0x17
#define PBEN06_I_SPORT3_Db_PBEN_O_ENCODING          0x17
#define PBEN07_I_SPORT3_Db_PBEN_O_ENCODING          0x17
#define PBEN08_I_SPORT3_Db_PBEN_O_ENCODING          0x17
#define PBEN09_I_SPORT3_Db_PBEN_O_ENCODING          0x17
#define PBEN10_I_SPORT3_Db_PBEN_O_ENCODING          0x17
#define PBEN11_I_SPORT3_Db_PBEN_O_ENCODING          0x17
#define PBEN12_I_SPORT3_Db_PBEN_O_ENCODING          0x17
#define PBEN13_I_SPORT3_Db_PBEN_O_ENCODING          0x17
#define PBEN14_I_SPORT3_Db_PBEN_O_ENCODING          0x17
#define PBEN15_I_SPORT3_Db_PBEN_O_ENCODING          0x17
#define PBEN16_I_SPORT3_Db_PBEN_O_ENCODING          0x17
#define PBEN17_I_SPORT3_Db_PBEN_O_ENCODING          0x17
#define PBEN18_I_SPORT3_Db_PBEN_O_ENCODING          0x17
#define PBEN19_I_SPORT3_Db_PBEN_O_ENCODING          0x17
#define PBEN20_I_SPORT3_Db_PBEN_O_ENCODING          0x17

#define PBEN01_I_SPORT4_CLK_PBEN_O_ENCODING         0x18
#define PBEN02_I_SPORT4_CLK_PBEN_O_ENCODING         0x18
#define PBEN03_I_SPORT4_CLK_PBEN_O_ENCODING         0x18
#define PBEN04_I_SPORT4_CLK_PBEN_O_ENCODING         0x18
#define PBEN05_I_SPORT4_CLK_PBEN_O_ENCODING         0x18
#define PBEN06_I_SPORT4_CLK_PBEN_O_ENCODING         0x18
#define PBEN07_I_SPORT4_CLK_PBEN_O_ENCODING         0x18
#define PBEN08_I_SPORT4_CLK_PBEN_O_ENCODING         0x18
#define PBEN09_I_SPORT4_CLK_PBEN_O_ENCODING         0x18
#define PBEN10_I_SPORT4_CLK_PBEN_O_ENCODING         0x18
#define PBEN11_I_SPORT4_CLK_PBEN_O_ENCODING         0x18
#define PBEN12_I_SPORT4_CLK_PBEN_O_ENCODING         0x18
#define PBEN13_I_SPORT4_CLK_PBEN_O_ENCODING         0x18
#define PBEN14_I_SPORT4_CLK_PBEN_O_ENCODING         0x18
#define PBEN15_I_SPORT4_CLK_PBEN_O_ENCODING         0x18
#define PBEN16_I_SPORT4_CLK_PBEN_O_ENCODING         0x18
#define PBEN17_I_SPORT4_CLK_PBEN_O_ENCODING         0x18
#define PBEN18_I_SPORT4_CLK_PBEN_O_ENCODING         0x18
#define PBEN19_I_SPORT4_CLK_PBEN_O_ENCODING         0x18
#define PBEN20_I_SPORT4_CLK_PBEN_O_ENCODING         0x18

#define PBEN01_I_SPORT4_FS_PBEN_O_ENCODING          0x19
#define PBEN02_I_SPORT4_FS_PBEN_O_ENCODING          0x19
#define PBEN03_I_SPORT4_FS_PBEN_O_ENCODING          0x19
#define PBEN04_I_SPORT4_FS_PBEN_O_ENCODING          0x19
#define PBEN05_I_SPORT4_FS_PBEN_O_ENCODING          0x19
#define PBEN06_I_SPORT4_FS_PBEN_O_ENCODING          0x19
#define PBEN07_I_SPORT4_FS_PBEN_O_ENCODING          0x19
#define PBEN08_I_SPORT4_FS_PBEN_O_ENCODING          0x19
#define PBEN09_I_SPORT4_FS_PBEN_O_ENCODING          0x19
#define PBEN10_I_SPORT4_FS_PBEN_O_ENCODING          0x19
#define PBEN11_I_SPORT4_FS_PBEN_O_ENCODING          0x19
#define PBEN12_I_SPORT4_FS_PBEN_O_ENCODING          0x19
#define PBEN13_I_SPORT4_FS_PBEN_O_ENCODING          0x19
#define PBEN14_I_SPORT4_FS_PBEN_O_ENCODING          0x19
#define PBEN15_I_SPORT4_FS_PBEN_O_ENCODING          0x19
#define PBEN16_I_SPORT4_FS_PBEN_O_ENCODING          0x19
#define PBEN17_I_SPORT4_FS_PBEN_O_ENCODING          0x19
#define PBEN18_I_SPORT4_FS_PBEN_O_ENCODING          0x19
#define PBEN19_I_SPORT4_FS_PBEN_O_ENCODING          0x19
#define PBEN20_I_SPORT4_FS_PBEN_O_ENCODING          0x19

#define PBEN01_I_SPORT4_DA_PBEN_O_ENCODING          0x1A
#define PBEN02_I_SPORT4_DA_PBEN_O_ENCODING          0x1A
#define PBEN03_I_SPORT4_DA_PBEN_O_ENCODING          0x1A
#define PBEN04_I_SPORT4_DA_PBEN_O_ENCODING          0x1A
#define PBEN05_I_SPORT4_DA_PBEN_O_ENCODING          0x1A
#define PBEN06_I_SPORT4_DA_PBEN_O_ENCODING          0x1A
#define PBEN07_I_SPORT4_DA_PBEN_O_ENCODING          0x1A
#define PBEN08_I_SPORT4_DA_PBEN_O_ENCODING          0x1A
#define PBEN09_I_SPORT4_DA_PBEN_O_ENCODING          0x1A
#define PBEN10_I_SPORT4_DA_PBEN_O_ENCODING          0x1A
#define PBEN11_I_SPORT4_DA_PBEN_O_ENCODING          0x1A
#define PBEN12_I_SPORT4_DA_PBEN_O_ENCODING          0x1A
#define PBEN13_I_SPORT4_DA_PBEN_O_ENCODING          0x1A
#define PBEN14_I_SPORT4_DA_PBEN_O_ENCODING          0x1A
#define PBEN15_I_SPORT4_DA_PBEN_O_ENCODING          0x1A
#define PBEN16_I_SPORT4_DA_PBEN_O_ENCODING          0x1A
#define PBEN17_I_SPORT4_DA_PBEN_O_ENCODING          0x1A
#define PBEN18_I_SPORT4_DA_PBEN_O_ENCODING          0x1A
#define PBEN19_I_SPORT4_DA_PBEN_O_ENCODING          0x1A
#define PBEN20_I_SPORT4_DA_PBEN_O_ENCODING          0x1A

#define PBEN01_I_SPORT4_Db_PBEN_O_ENCODING          0x1B
#define PBEN02_I_SPORT4_Db_PBEN_O_ENCODING          0x1B
#define PBEN03_I_SPORT4_Db_PBEN_O_ENCODING          0x1B
#define PBEN04_I_SPORT4_Db_PBEN_O_ENCODING          0x1B
#define PBEN05_I_SPORT4_Db_PBEN_O_ENCODING          0x1B
#define PBEN06_I_SPORT4_Db_PBEN_O_ENCODING          0x1B
#define PBEN07_I_SPORT4_Db_PBEN_O_ENCODING          0x1B
#define PBEN08_I_SPORT4_Db_PBEN_O_ENCODING          0x1B
#define PBEN09_I_SPORT4_Db_PBEN_O_ENCODING          0x1B
#define PBEN10_I_SPORT4_Db_PBEN_O_ENCODING          0x1B
#define PBEN11_I_SPORT4_Db_PBEN_O_ENCODING          0x1B
#define PBEN12_I_SPORT4_Db_PBEN_O_ENCODING          0x1B
#define PBEN13_I_SPORT4_Db_PBEN_O_ENCODING          0x1B
#define PBEN14_I_SPORT4_Db_PBEN_O_ENCODING          0x1B
#define PBEN15_I_SPORT4_Db_PBEN_O_ENCODING          0x1B
#define PBEN16_I_SPORT4_Db_PBEN_O_ENCODING          0x1B
#define PBEN17_I_SPORT4_Db_PBEN_O_ENCODING          0x1B
#define PBEN18_I_SPORT4_Db_PBEN_O_ENCODING          0x1B
#define PBEN19_I_SPORT4_Db_PBEN_O_ENCODING          0x1B
#define PBEN20_I_SPORT4_Db_PBEN_O_ENCODING          0x1B

#define PBEN01_I_SPORT5_CLK_PBEN_O_ENCODING         0x1C
#define PBEN02_I_SPORT5_CLK_PBEN_O_ENCODING         0x1C
#define PBEN03_I_SPORT5_CLK_PBEN_O_ENCODING         0x1C
#define PBEN04_I_SPORT5_CLK_PBEN_O_ENCODING         0x1C
#define PBEN05_I_SPORT5_CLK_PBEN_O_ENCODING         0x1C
#define PBEN06_I_SPORT5_CLK_PBEN_O_ENCODING         0x1C
#define PBEN07_I_SPORT5_CLK_PBEN_O_ENCODING         0x1C
#define PBEN08_I_SPORT5_CLK_PBEN_O_ENCODING         0x1C
#define PBEN09_I_SPORT5_CLK_PBEN_O_ENCODING         0x1C
#define PBEN10_I_SPORT5_CLK_PBEN_O_ENCODING         0x1C
#define PBEN11_I_SPORT5_CLK_PBEN_O_ENCODING         0x1C
#define PBEN12_I_SPORT5_CLK_PBEN_O_ENCODING         0x1C
#define PBEN13_I_SPORT5_CLK_PBEN_O_ENCODING         0x1C
#define PBEN14_I_SPORT5_CLK_PBEN_O_ENCODING         0x1C
#define PBEN15_I_SPORT5_CLK_PBEN_O_ENCODING         0x1C
#define PBEN16_I_SPORT5_CLK_PBEN_O_ENCODING         0x1C
#define PBEN17_I_SPORT5_CLK_PBEN_O_ENCODING         0x1C
#define PBEN18_I_SPORT5_CLK_PBEN_O_ENCODING         0x1C
#define PBEN19_I_SPORT5_CLK_PBEN_O_ENCODING         0x1C
#define PBEN20_I_SPORT5_CLK_PBEN_O_ENCODING         0x1C

#define PBEN01_I_SPORT5_FS_PBEN_O_ENCODING          0x1D
#define PBEN02_I_SPORT5_FS_PBEN_O_ENCODING          0x1D
#define PBEN03_I_SPORT5_FS_PBEN_O_ENCODING          0x1D
#define PBEN04_I_SPORT5_FS_PBEN_O_ENCODING          0x1D
#define PBEN05_I_SPORT5_FS_PBEN_O_ENCODING          0x1D
#define PBEN06_I_SPORT5_FS_PBEN_O_ENCODING          0x1D
#define PBEN07_I_SPORT5_FS_PBEN_O_ENCODING          0x1D
#define PBEN08_I_SPORT5_FS_PBEN_O_ENCODING          0x1D
#define PBEN09_I_SPORT5_FS_PBEN_O_ENCODING          0x1D
#define PBEN10_I_SPORT5_FS_PBEN_O_ENCODING          0x1D
#define PBEN11_I_SPORT5_FS_PBEN_O_ENCODING          0x1D
#define PBEN12_I_SPORT5_FS_PBEN_O_ENCODING          0x1D
#define PBEN13_I_SPORT5_FS_PBEN_O_ENCODING          0x1D
#define PBEN14_I_SPORT5_FS_PBEN_O_ENCODING          0x1D
#define PBEN15_I_SPORT5_FS_PBEN_O_ENCODING          0x1D
#define PBEN16_I_SPORT5_FS_PBEN_O_ENCODING          0x1D
#define PBEN17_I_SPORT5_FS_PBEN_O_ENCODING          0x1D
#define PBEN18_I_SPORT5_FS_PBEN_O_ENCODING          0x1D
#define PBEN19_I_SPORT5_FS_PBEN_O_ENCODING          0x1D
#define PBEN20_I_SPORT5_FS_PBEN_O_ENCODING          0x1D

#define PBEN01_I_SPORT5_DA_PBEN_O_ENCODING          0x1E
#define PBEN02_I_SPORT5_DA_PBEN_O_ENCODING          0x1E
#define PBEN03_I_SPORT5_DA_PBEN_O_ENCODING          0x1E
#define PBEN04_I_SPORT5_DA_PBEN_O_ENCODING          0x1E
#define PBEN05_I_SPORT5_DA_PBEN_O_ENCODING          0x1E
#define PBEN06_I_SPORT5_DA_PBEN_O_ENCODING          0x1E
#define PBEN07_I_SPORT5_DA_PBEN_O_ENCODING          0x1E
#define PBEN08_I_SPORT5_DA_PBEN_O_ENCODING          0x1E
#define PBEN09_I_SPORT5_DA_PBEN_O_ENCODING          0x1E
#define PBEN10_I_SPORT5_DA_PBEN_O_ENCODING          0x1E
#define PBEN11_I_SPORT5_DA_PBEN_O_ENCODING          0x1E
#define PBEN12_I_SPORT5_DA_PBEN_O_ENCODING          0x1E
#define PBEN13_I_SPORT5_DA_PBEN_O_ENCODING          0x1E
#define PBEN14_I_SPORT5_DA_PBEN_O_ENCODING          0x1E
#define PBEN15_I_SPORT5_DA_PBEN_O_ENCODING          0x1E
#define PBEN16_I_SPORT5_DA_PBEN_O_ENCODING          0x1E
#define PBEN17_I_SPORT5_DA_PBEN_O_ENCODING          0x1E
#define PBEN18_I_SPORT5_DA_PBEN_O_ENCODING          0x1E
#define PBEN19_I_SPORT5_DA_PBEN_O_ENCODING          0x1E
#define PBEN20_I_SPORT5_DA_PBEN_O_ENCODING          0x1E

#define PBEN01_I_SPORT5_Db_PBEN_O_ENCODING          0x1F
#define PBEN02_I_SPORT5_Db_PBEN_O_ENCODING          0x1F
#define PBEN03_I_SPORT5_Db_PBEN_O_ENCODING          0x1F
#define PBEN04_I_SPORT5_Db_PBEN_O_ENCODING          0x1F
#define PBEN05_I_SPORT5_Db_PBEN_O_ENCODING          0x1F
#define PBEN06_I_SPORT5_Db_PBEN_O_ENCODING          0x1F
#define PBEN07_I_SPORT5_Db_PBEN_O_ENCODING          0x1F
#define PBEN08_I_SPORT5_Db_PBEN_O_ENCODING          0x1F
#define PBEN09_I_SPORT5_Db_PBEN_O_ENCODING          0x1F
#define PBEN10_I_SPORT5_Db_PBEN_O_ENCODING          0x1F
#define PBEN11_I_SPORT5_Db_PBEN_O_ENCODING          0x1F
#define PBEN12_I_SPORT5_Db_PBEN_O_ENCODING          0x1F
#define PBEN13_I_SPORT5_Db_PBEN_O_ENCODING          0x1F
#define PBEN14_I_SPORT5_Db_PBEN_O_ENCODING          0x1F
#define PBEN15_I_SPORT5_Db_PBEN_O_ENCODING          0x1F
#define PBEN16_I_SPORT5_Db_PBEN_O_ENCODING          0x1F
#define PBEN17_I_SPORT5_Db_PBEN_O_ENCODING          0x1F
#define PBEN18_I_SPORT5_Db_PBEN_O_ENCODING          0x1F
#define PBEN19_I_SPORT5_Db_PBEN_O_ENCODING          0x1F
#define PBEN20_I_SPORT5_Db_PBEN_O_ENCODING          0x1F

#define PBEN01_I_TIMER0_PBEN_O_ENCODING             0x20
#define PBEN02_I_TIMER0_PBEN_O_ENCODING             0x20
#define PBEN03_I_TIMER0_PBEN_O_ENCODING             0x20
#define PBEN04_I_TIMER0_PBEN_O_ENCODING             0x20
#define PBEN05_I_TIMER0_PBEN_O_ENCODING             0x20
#define PBEN06_I_TIMER0_PBEN_O_ENCODING             0x20
#define PBEN07_I_TIMER0_PBEN_O_ENCODING             0x20
#define PBEN08_I_TIMER0_PBEN_O_ENCODING             0x20
#define PBEN09_I_TIMER0_PBEN_O_ENCODING             0x20
#define PBEN10_I_TIMER0_PBEN_O_ENCODING             0x20
#define PBEN11_I_TIMER0_PBEN_O_ENCODING             0x20
#define PBEN12_I_TIMER0_PBEN_O_ENCODING             0x20
#define PBEN13_I_TIMER0_PBEN_O_ENCODING             0x20
#define PBEN14_I_TIMER0_PBEN_O_ENCODING             0x20
#define PBEN15_I_TIMER0_PBEN_O_ENCODING             0x20
#define PBEN16_I_TIMER0_PBEN_O_ENCODING             0x20
#define PBEN17_I_TIMER0_PBEN_O_ENCODING             0x20
#define PBEN18_I_TIMER0_PBEN_O_ENCODING             0x20
#define PBEN19_I_TIMER0_PBEN_O_ENCODING             0x20
#define PBEN20_I_TIMER0_PBEN_O_ENCODING             0x20

#define PBEN01_I_TIMER1_PBEN_O_ENCODING             0x21
#define PBEN02_I_TIMER1_PBEN_O_ENCODING             0x21
#define PBEN03_I_TIMER1_PBEN_O_ENCODING             0x21
#define PBEN04_I_TIMER1_PBEN_O_ENCODING             0x21
#define PBEN05_I_TIMER1_PBEN_O_ENCODING             0x21
#define PBEN06_I_TIMER1_PBEN_O_ENCODING             0x21
#define PBEN07_I_TIMER1_PBEN_O_ENCODING             0x21
#define PBEN08_I_TIMER1_PBEN_O_ENCODING             0x21
#define PBEN09_I_TIMER1_PBEN_O_ENCODING             0x21
#define PBEN10_I_TIMER1_PBEN_O_ENCODING             0x21
#define PBEN11_I_TIMER1_PBEN_O_ENCODING             0x21
#define PBEN12_I_TIMER1_PBEN_O_ENCODING             0x21
#define PBEN13_I_TIMER1_PBEN_O_ENCODING             0x21
#define PBEN14_I_TIMER1_PBEN_O_ENCODING             0x21
#define PBEN15_I_TIMER1_PBEN_O_ENCODING             0x21
#define PBEN16_I_TIMER1_PBEN_O_ENCODING             0x21
#define PBEN17_I_TIMER1_PBEN_O_ENCODING             0x21
#define PBEN18_I_TIMER1_PBEN_O_ENCODING             0x21
#define PBEN19_I_TIMER1_PBEN_O_ENCODING             0x21
#define PBEN20_I_TIMER1_PBEN_O_ENCODING             0x21

#define PBEN01_I_TIMER2_PBEN_O_ENCODING             0x22
#define PBEN02_I_TIMER2_PBEN_O_ENCODING             0x22
#define PBEN03_I_TIMER2_PBEN_O_ENCODING             0x22
#define PBEN04_I_TIMER2_PBEN_O_ENCODING             0x22
#define PBEN05_I_TIMER2_PBEN_O_ENCODING             0x22
#define PBEN06_I_TIMER2_PBEN_O_ENCODING             0x22
#define PBEN07_I_TIMER2_PBEN_O_ENCODING             0x22
#define PBEN08_I_TIMER2_PBEN_O_ENCODING             0x22
#define PBEN09_I_TIMER2_PBEN_O_ENCODING             0x22
#define PBEN10_I_TIMER2_PBEN_O_ENCODING             0x22
#define PBEN11_I_TIMER2_PBEN_O_ENCODING             0x22
#define PBEN12_I_TIMER2_PBEN_O_ENCODING             0x22
#define PBEN13_I_TIMER2_PBEN_O_ENCODING             0x22
#define PBEN14_I_TIMER2_PBEN_O_ENCODING             0x22
#define PBEN15_I_TIMER2_PBEN_O_ENCODING             0x22
#define PBEN16_I_TIMER2_PBEN_O_ENCODING             0x22
#define PBEN17_I_TIMER2_PBEN_O_ENCODING             0x22
#define PBEN18_I_TIMER2_PBEN_O_ENCODING             0x22
#define PBEN19_I_TIMER2_PBEN_O_ENCODING             0x22
#define PBEN20_I_TIMER2_PBEN_O_ENCODING             0x22

#define PBEN01_I_FLAG10_PBEN_O_ENCODING             0x23
#define PBEN02_I_FLAG10_PBEN_O_ENCODING             0x23
#define PBEN03_I_FLAG10_PBEN_O_ENCODING             0x23
#define PBEN04_I_FLAG10_PBEN_O_ENCODING             0x23
#define PBEN05_I_FLAG10_PBEN_O_ENCODING             0x23
#define PBEN06_I_FLAG10_PBEN_O_ENCODING             0x23
#define PBEN07_I_FLAG10_PBEN_O_ENCODING             0x23
#define PBEN08_I_FLAG10_PBEN_O_ENCODING             0x23
#define PBEN09_I_FLAG10_PBEN_O_ENCODING             0x23
#define PBEN10_I_FLAG10_PBEN_O_ENCODING             0x23
#define PBEN11_I_FLAG10_PBEN_O_ENCODING             0x23
#define PBEN12_I_FLAG10_PBEN_O_ENCODING             0x23
#define PBEN13_I_FLAG10_PBEN_O_ENCODING             0x23
#define PBEN14_I_FLAG10_PBEN_O_ENCODING             0x23
#define PBEN15_I_FLAG10_PBEN_O_ENCODING             0x23
#define PBEN16_I_FLAG10_PBEN_O_ENCODING             0x23
#define PBEN17_I_FLAG10_PBEN_O_ENCODING             0x23
#define PBEN18_I_FLAG10_PBEN_O_ENCODING             0x23
#define PBEN19_I_FLAG10_PBEN_O_ENCODING             0x23
#define PBEN20_I_FLAG10_PBEN_O_ENCODING             0x23

#define PBEN01_I_FLAG11_PBEN_O_ENCODING             0x24
#define PBEN02_I_FLAG11_PBEN_O_ENCODING             0x24
#define PBEN03_I_FLAG11_PBEN_O_ENCODING             0x24
#define PBEN04_I_FLAG11_PBEN_O_ENCODING             0x24
#define PBEN05_I_FLAG11_PBEN_O_ENCODING             0x24
#define PBEN06_I_FLAG11_PBEN_O_ENCODING             0x24
#define PBEN07_I_FLAG11_PBEN_O_ENCODING             0x24
#define PBEN08_I_FLAG11_PBEN_O_ENCODING             0x24
#define PBEN09_I_FLAG11_PBEN_O_ENCODING             0x24
#define PBEN10_I_FLAG11_PBEN_O_ENCODING             0x24
#define PBEN11_I_FLAG11_PBEN_O_ENCODING             0x24
#define PBEN12_I_FLAG11_PBEN_O_ENCODING             0x24
#define PBEN13_I_FLAG11_PBEN_O_ENCODING             0x24
#define PBEN14_I_FLAG11_PBEN_O_ENCODING             0x24
#define PBEN15_I_FLAG11_PBEN_O_ENCODING             0x24
#define PBEN16_I_FLAG11_PBEN_O_ENCODING             0x24
#define PBEN17_I_FLAG11_PBEN_O_ENCODING             0x24
#define PBEN18_I_FLAG11_PBEN_O_ENCODING             0x24
#define PBEN19_I_FLAG11_PBEN_O_ENCODING             0x24
#define PBEN20_I_FLAG11_PBEN_O_ENCODING             0x24

#define PBEN01_I_FLAG12_PBEN_O_ENCODING             0x25
#define PBEN02_I_FLAG12_PBEN_O_ENCODING             0x25
#define PBEN03_I_FLAG12_PBEN_O_ENCODING             0x25
#define PBEN04_I_FLAG12_PBEN_O_ENCODING             0x25
#define PBEN05_I_FLAG12_PBEN_O_ENCODING             0x25
#define PBEN06_I_FLAG12_PBEN_O_ENCODING             0x25
#define PBEN07_I_FLAG12_PBEN_O_ENCODING             0x25
#define PBEN08_I_FLAG12_PBEN_O_ENCODING             0x25
#define PBEN09_I_FLAG12_PBEN_O_ENCODING             0x25
#define PBEN10_I_FLAG12_PBEN_O_ENCODING             0x25
#define PBEN11_I_FLAG12_PBEN_O_ENCODING             0x25
#define PBEN12_I_FLAG12_PBEN_O_ENCODING             0x25
#define PBEN13_I_FLAG12_PBEN_O_ENCODING             0x25
#define PBEN14_I_FLAG12_PBEN_O_ENCODING             0x25
#define PBEN15_I_FLAG12_PBEN_O_ENCODING             0x25
#define PBEN16_I_FLAG12_PBEN_O_ENCODING             0x25
#define PBEN17_I_FLAG12_PBEN_O_ENCODING             0x25
#define PBEN18_I_FLAG12_PBEN_O_ENCODING             0x25
#define PBEN19_I_FLAG12_PBEN_O_ENCODING             0x25
#define PBEN20_I_FLAG12_PBEN_O_ENCODING             0x25

#define PBEN01_I_FLAG13_PBEN_O_ENCODING             0x26
#define PBEN02_I_FLAG13_PBEN_O_ENCODING             0x26
#define PBEN03_I_FLAG13_PBEN_O_ENCODING             0x26
#define PBEN04_I_FLAG13_PBEN_O_ENCODING             0x26
#define PBEN05_I_FLAG13_PBEN_O_ENCODING             0x26
#define PBEN06_I_FLAG13_PBEN_O_ENCODING             0x26
#define PBEN07_I_FLAG13_PBEN_O_ENCODING             0x26
#define PBEN08_I_FLAG13_PBEN_O_ENCODING             0x26
#define PBEN09_I_FLAG13_PBEN_O_ENCODING             0x26
#define PBEN10_I_FLAG13_PBEN_O_ENCODING             0x26
#define PBEN11_I_FLAG13_PBEN_O_ENCODING             0x26
#define PBEN12_I_FLAG13_PBEN_O_ENCODING             0x26
#define PBEN13_I_FLAG13_PBEN_O_ENCODING             0x26
#define PBEN14_I_FLAG13_PBEN_O_ENCODING             0x26
#define PBEN15_I_FLAG13_PBEN_O_ENCODING             0x26
#define PBEN16_I_FLAG13_PBEN_O_ENCODING             0x26
#define PBEN17_I_FLAG13_PBEN_O_ENCODING             0x26
#define PBEN18_I_FLAG13_PBEN_O_ENCODING             0x26
#define PBEN19_I_FLAG13_PBEN_O_ENCODING             0x26
#define PBEN20_I_FLAG13_PBEN_O_ENCODING             0x26

#define PBEN01_I_FLAG14_PBEN_O_ENCODING             0x27
#define PBEN02_I_FLAG14_PBEN_O_ENCODING             0x27
#define PBEN03_I_FLAG14_PBEN_O_ENCODING             0x27
#define PBEN04_I_FLAG14_PBEN_O_ENCODING             0x27
#define PBEN05_I_FLAG14_PBEN_O_ENCODING             0x27
#define PBEN06_I_FLAG14_PBEN_O_ENCODING             0x27
#define PBEN07_I_FLAG14_PBEN_O_ENCODING             0x27
#define PBEN08_I_FLAG14_PBEN_O_ENCODING             0x27
#define PBEN09_I_FLAG14_PBEN_O_ENCODING             0x27
#define PBEN10_I_FLAG14_PBEN_O_ENCODING             0x27
#define PBEN11_I_FLAG14_PBEN_O_ENCODING             0x27
#define PBEN12_I_FLAG14_PBEN_O_ENCODING             0x27
#define PBEN13_I_FLAG14_PBEN_O_ENCODING             0x27
#define PBEN14_I_FLAG14_PBEN_O_ENCODING             0x27
#define PBEN15_I_FLAG14_PBEN_O_ENCODING             0x27
#define PBEN16_I_FLAG14_PBEN_O_ENCODING             0x27
#define PBEN17_I_FLAG14_PBEN_O_ENCODING             0x27
#define PBEN18_I_FLAG14_PBEN_O_ENCODING             0x27
#define PBEN19_I_FLAG14_PBEN_O_ENCODING             0x27
#define PBEN20_I_FLAG14_PBEN_O_ENCODING             0x27

#define PBEN01_I_FLAG15_PBEN_O_ENCODING             0x28
#define PBEN02_I_FLAG15_PBEN_O_ENCODING             0x28
#define PBEN03_I_FLAG15_PBEN_O_ENCODING             0x28
#define PBEN04_I_FLAG15_PBEN_O_ENCODING             0x28
#define PBEN05_I_FLAG15_PBEN_O_ENCODING             0x28
#define PBEN06_I_FLAG15_PBEN_O_ENCODING             0x28
#define PBEN07_I_FLAG15_PBEN_O_ENCODING             0x28
#define PBEN08_I_FLAG15_PBEN_O_ENCODING             0x28
#define PBEN09_I_FLAG15_PBEN_O_ENCODING             0x28
#define PBEN10_I_FLAG15_PBEN_O_ENCODING             0x28
#define PBEN11_I_FLAG15_PBEN_O_ENCODING             0x28
#define PBEN12_I_FLAG15_PBEN_O_ENCODING             0x28
#define PBEN13_I_FLAG15_PBEN_O_ENCODING             0x28
#define PBEN14_I_FLAG15_PBEN_O_ENCODING             0x28
#define PBEN15_I_FLAG15_PBEN_O_ENCODING             0x28
#define PBEN16_I_FLAG15_PBEN_O_ENCODING             0x28
#define PBEN17_I_FLAG15_PBEN_O_ENCODING             0x28
#define PBEN18_I_FLAG15_PBEN_O_ENCODING             0x28
#define PBEN19_I_FLAG15_PBEN_O_ENCODING             0x28
#define PBEN20_I_FLAG15_PBEN_O_ENCODING             0x28

#define PBEN01_I_SPIB_MISO_PBEN_O_ENCODING             0x29
#define PBEN02_I_SPIB_MISO_PBEN_O_ENCODING             0x29
#define PBEN03_I_SPIB_MISO_PBEN_O_ENCODING             0x29
#define PBEN04_I_SPIB_MISO_PBEN_O_ENCODING             0x29
#define PBEN05_I_SPIB_MISO_PBEN_O_ENCODING             0x29
#define PBEN06_I_SPIB_MISO_PBEN_O_ENCODING             0x29
#define PBEN07_I_SPIB_MISO_PBEN_O_ENCODING             0x29
#define PBEN08_I_SPIB_MISO_PBEN_O_ENCODING             0x29
#define PBEN09_I_SPIB_MISO_PBEN_O_ENCODING             0x29
#define PBEN10_I_SPIB_MISO_PBEN_O_ENCODING             0x29
#define PBEN11_I_SPIB_MISO_PBEN_O_ENCODING             0x29
#define PBEN12_I_SPIB_MISO_PBEN_O_ENCODING             0x29
#define PBEN13_I_SPIB_MISO_PBEN_O_ENCODING             0x29
#define PBEN14_I_SPIB_MISO_PBEN_O_ENCODING             0x29
#define PBEN15_I_SPIB_MISO_PBEN_O_ENCODING             0x29
#define PBEN16_I_SPIB_MISO_PBEN_O_ENCODING             0x29
#define PBEN17_I_SPIB_MISO_PBEN_O_ENCODING             0x29
#define PBEN18_I_SPIB_MISO_PBEN_O_ENCODING             0x29
#define PBEN19_I_SPIB_MISO_PBEN_O_ENCODING             0x29
#define PBEN20_I_SPIB_MISO_PBEN_O_ENCODING             0x29

#define PBEN01_I_SPIB_MOSI_PBEN_O_ENCODING             0x2A
#define PBEN02_I_SPIB_MOSI_PBEN_O_ENCODING             0x2A
#define PBEN03_I_SPIB_MOSI_PBEN_O_ENCODING             0x2A
#define PBEN04_I_SPIB_MOSI_PBEN_O_ENCODING             0x2A
#define PBEN05_I_SPIB_MOSI_PBEN_O_ENCODING             0x2A
#define PBEN06_I_SPIB_MOSI_PBEN_O_ENCODING             0x2A
#define PBEN07_I_SPIB_MOSI_PBEN_O_ENCODING             0x2A
#define PBEN08_I_SPIB_MOSI_PBEN_O_ENCODING             0x2A
#define PBEN09_I_SPIB_MOSI_PBEN_O_ENCODING             0x2A
#define PBEN10_I_SPIB_MOSI_PBEN_O_ENCODING             0x2A
#define PBEN11_I_SPIB_MOSI_PBEN_O_ENCODING             0x2A
#define PBEN12_I_SPIB_MOSI_PBEN_O_ENCODING             0x2A
#define PBEN13_I_SPIB_MOSI_PBEN_O_ENCODING             0x2A
#define PBEN14_I_SPIB_MOSI_PBEN_O_ENCODING             0x2A
#define PBEN15_I_SPIB_MOSI_PBEN_O_ENCODING             0x2A
#define PBEN16_I_SPIB_MOSI_PBEN_O_ENCODING             0x2A
#define PBEN17_I_SPIB_MOSI_PBEN_O_ENCODING             0x2A
#define PBEN18_I_SPIB_MOSI_PBEN_O_ENCODING             0x2A
#define PBEN19_I_SPIB_MOSI_PBEN_O_ENCODING             0x2A
#define PBEN20_I_SPIB_MOSI_PBEN_O_ENCODING             0x2A

#define PBEN01_I_SPIB_CLK_PBEN_O_ENCODING             0x2B
#define PBEN02_I_SPIB_CLK_PBEN_O_ENCODING             0x2B
#define PBEN03_I_SPIB_CLK_PBEN_O_ENCODING             0x2B
#define PBEN04_I_SPIB_CLK_PBEN_O_ENCODING             0x2B
#define PBEN05_I_SPIB_CLK_PBEN_O_ENCODING             0x2B
#define PBEN06_I_SPIB_CLK_PBEN_O_ENCODING             0x2B
#define PBEN07_I_SPIB_CLK_PBEN_O_ENCODING             0x2B
#define PBEN08_I_SPIB_CLK_PBEN_O_ENCODING             0x2B
#define PBEN09_I_SPIB_CLK_PBEN_O_ENCODING             0x2B
#define PBEN10_I_SPIB_CLK_PBEN_O_ENCODING             0x2B
#define PBEN11_I_SPIB_CLK_PBEN_O_ENCODING             0x2B
#define PBEN12_I_SPIB_CLK_PBEN_O_ENCODING             0x2B
#define PBEN13_I_SPIB_CLK_PBEN_O_ENCODING             0x2B
#define PBEN14_I_SPIB_CLK_PBEN_O_ENCODING             0x2B
#define PBEN15_I_SPIB_CLK_PBEN_O_ENCODING             0x2B
#define PBEN16_I_SPIB_CLK_PBEN_O_ENCODING             0x2B
#define PBEN17_I_SPIB_CLK_PBEN_O_ENCODING             0x2B
#define PBEN18_I_SPIB_CLK_PBEN_O_ENCODING             0x2B
#define PBEN19_I_SPIB_CLK_PBEN_O_ENCODING             0x2B
#define PBEN20_I_SPIB_CLK_PBEN_O_ENCODING             0x2B

#define PBEN01_I_SPIB_FLG0_PBEN_O_ENCODING             0x2C
#define PBEN02_I_SPIB_FLG0_PBEN_O_ENCODING             0x2C
#define PBEN03_I_SPIB_FLG0_PBEN_O_ENCODING             0x2C
#define PBEN04_I_SPIB_FLG0_PBEN_O_ENCODING             0x2C
#define PBEN05_I_SPIB_FLG0_PBEN_O_ENCODING             0x2C
#define PBEN06_I_SPIB_FLG0_PBEN_O_ENCODING             0x2C
#define PBEN07_I_SPIB_FLG0_PBEN_O_ENCODING             0x2C
#define PBEN08_I_SPIB_FLG0_PBEN_O_ENCODING             0x2C
#define PBEN09_I_SPIB_FLG0_PBEN_O_ENCODING             0x2C
#define PBEN10_I_SPIB_FLG0_PBEN_O_ENCODING             0x2C
#define PBEN11_I_SPIB_FLG0_PBEN_O_ENCODING             0x2C
#define PBEN12_I_SPIB_FLG0_PBEN_O_ENCODING             0x2C
#define PBEN13_I_SPIB_FLG0_PBEN_O_ENCODING             0x2C
#define PBEN14_I_SPIB_FLG0_PBEN_O_ENCODING             0x2C
#define PBEN15_I_SPIB_FLG0_PBEN_O_ENCODING             0x2C
#define PBEN16_I_SPIB_FLG0_PBEN_O_ENCODING             0x2C
#define PBEN17_I_SPIB_FLG0_PBEN_O_ENCODING             0x2C
#define PBEN18_I_SPIB_FLG0_PBEN_O_ENCODING             0x2C
#define PBEN19_I_SPIB_FLG0_PBEN_O_ENCODING             0x2C
#define PBEN20_I_SPIB_FLG0_PBEN_O_ENCODING             0x2C

#define PBEN01_I_SPIB_FLG1_PBEN_O_ENCODING             0x2D
#define PBEN02_I_SPIB_FLG1_PBEN_O_ENCODING             0x2D
#define PBEN03_I_SPIB_FLG1_PBEN_O_ENCODING             0x2D
#define PBEN04_I_SPIB_FLG1_PBEN_O_ENCODING             0x2D
#define PBEN05_I_SPIB_FLG1_PBEN_O_ENCODING             0x2D
#define PBEN06_I_SPIB_FLG1_PBEN_O_ENCODING             0x2D
#define PBEN07_I_SPIB_FLG1_PBEN_O_ENCODING             0x2D
#define PBEN08_I_SPIB_FLG1_PBEN_O_ENCODING             0x2D
#define PBEN09_I_SPIB_FLG1_PBEN_O_ENCODING             0x2D
#define PBEN10_I_SPIB_FLG1_PBEN_O_ENCODING             0x2D
#define PBEN11_I_SPIB_FLG1_PBEN_O_ENCODING             0x2D
#define PBEN12_I_SPIB_FLG1_PBEN_O_ENCODING             0x2D
#define PBEN13_I_SPIB_FLG1_PBEN_O_ENCODING             0x2D
#define PBEN14_I_SPIB_FLG1_PBEN_O_ENCODING             0x2D
#define PBEN15_I_SPIB_FLG1_PBEN_O_ENCODING             0x2D
#define PBEN16_I_SPIB_FLG1_PBEN_O_ENCODING             0x2D
#define PBEN17_I_SPIB_FLG1_PBEN_O_ENCODING             0x2D
#define PBEN18_I_SPIB_FLG1_PBEN_O_ENCODING             0x2D
#define PBEN19_I_SPIB_FLG1_PBEN_O_ENCODING             0x2D
#define PBEN20_I_SPIB_FLG1_PBEN_O_ENCODING             0x2D   

#define PBEN01_I_SPIB_FLG2_PBEN_O_ENCODING             0x2E
#define PBEN02_I_SPIB_FLG2_PBEN_O_ENCODING             0x2E
#define PBEN03_I_SPIB_FLG2_PBEN_O_ENCODING             0x2E
#define PBEN04_I_SPIB_FLG2_PBEN_O_ENCODING             0x2E
#define PBEN05_I_SPIB_FLG2_PBEN_O_ENCODING             0x2E
#define PBEN06_I_SPIB_FLG2_PBEN_O_ENCODING             0x2E
#define PBEN07_I_SPIB_FLG2_PBEN_O_ENCODING             0x2E
#define PBEN08_I_SPIB_FLG2_PBEN_O_ENCODING             0x2E
#define PBEN09_I_SPIB_FLG2_PBEN_O_ENCODING             0x2E
#define PBEN10_I_SPIB_FLG2_PBEN_O_ENCODING             0x2E
#define PBEN11_I_SPIB_FLG2_PBEN_O_ENCODING             0x2E
#define PBEN12_I_SPIB_FLG2_PBEN_O_ENCODING             0x2E
#define PBEN13_I_SPIB_FLG2_PBEN_O_ENCODING             0x2E
#define PBEN14_I_SPIB_FLG2_PBEN_O_ENCODING             0x2E
#define PBEN15_I_SPIB_FLG2_PBEN_O_ENCODING             0x2E
#define PBEN16_I_SPIB_FLG2_PBEN_O_ENCODING             0x2E
#define PBEN17_I_SPIB_FLG2_PBEN_O_ENCODING             0x2E
#define PBEN18_I_SPIB_FLG2_PBEN_O_ENCODING             0x2E
#define PBEN19_I_SPIB_FLG2_PBEN_O_ENCODING             0x2E
#define PBEN20_I_SPIB_FLG2_PBEN_O_ENCODING             0x2E

#define PBEN01_I_SPIB_FLG3_PBEN_O_ENCODING             0x2F
#define PBEN02_I_SPIB_FLG3_PBEN_O_ENCODING             0x2F
#define PBEN03_I_SPIB_FLG3_PBEN_O_ENCODING             0x2F
#define PBEN04_I_SPIB_FLG3_PBEN_O_ENCODING             0x2F
#define PBEN05_I_SPIB_FLG3_PBEN_O_ENCODING             0x2F
#define PBEN06_I_SPIB_FLG3_PBEN_O_ENCODING             0x2F
#define PBEN07_I_SPIB_FLG3_PBEN_O_ENCODING             0x2F
#define PBEN08_I_SPIB_FLG3_PBEN_O_ENCODING             0x2F
#define PBEN09_I_SPIB_FLG3_PBEN_O_ENCODING             0x2F
#define PBEN10_I_SPIB_FLG3_PBEN_O_ENCODING             0x2F
#define PBEN11_I_SPIB_FLG3_PBEN_O_ENCODING             0x2F
#define PBEN12_I_SPIB_FLG3_PBEN_O_ENCODING             0x2F
#define PBEN13_I_SPIB_FLG3_PBEN_O_ENCODING             0x2F
#define PBEN14_I_SPIB_FLG3_PBEN_O_ENCODING             0x2F
#define PBEN15_I_SPIB_FLG3_PBEN_O_ENCODING             0x2F
#define PBEN16_I_SPIB_FLG3_PBEN_O_ENCODING             0x2F
#define PBEN17_I_SPIB_FLG3_PBEN_O_ENCODING             0x2F
#define PBEN18_I_SPIB_FLG3_PBEN_O_ENCODING             0x2F
#define PBEN19_I_SPIB_FLG3_PBEN_O_ENCODING             0x2F
#define PBEN20_I_SPIB_FLG3_PBEN_O_ENCODING             0x2F



//------------------------------------------------------------------------------
//  Group F: Register associations
//------------------------------------------------------------------------------



#define PBEN01_I_LOW_REG                  0x2478   /* SRU_PINEN0 */
#define PBEN02_I_LOW_REG                  0x2478   /* SRU_PINEN0 */
#define PBEN03_I_LOW_REG                  0x2478   /* SRU_PINEN0 */
#define PBEN04_I_LOW_REG                  0x2478   /* SRU_PINEN0 */
#define PBEN05_I_LOW_REG                  0x2478   /* SRU_PINEN0 */
#define PBEN06_I_LOW_REG                  0x2479   /* SRU_PINEN1 */
#define PBEN07_I_LOW_REG                  0x2479   /* SRU_PINEN1 */
#define PBEN08_I_LOW_REG                  0x2479   /* SRU_PINEN1 */
#define PBEN09_I_LOW_REG                  0x2479   /* SRU_PINEN1 */
#define PBEN10_I_LOW_REG                  0x2479   /* SRU_PINEN1 */
#define PBEN11_I_LOW_REG                  0x247A   /* SRU_PINEN2 */
#define PBEN12_I_LOW_REG                  0x247A   /* SRU_PINEN2 */
#define PBEN13_I_LOW_REG                  0x247A   /* SRU_PINEN2 */
#define PBEN14_I_LOW_REG                  0x247A   /* SRU_PINEN2 */
#define PBEN15_I_LOW_REG                  0x247A   /* SRU_PINEN2 */
#define PBEN16_I_LOW_REG                  0x247B   /* SRU_PINEN3 */
#define PBEN17_I_LOW_REG                  0x247B   /* SRU_PINEN3 */
#define PBEN18_I_LOW_REG                  0x247B   /* SRU_PINEN3 */
#define PBEN19_I_LOW_REG                  0x247B   /* SRU_PINEN3 */
#define PBEN20_I_LOW_REG                  0x247B   /* SRU_PINEN3 */

#define PBEN01_I_HIGH_REG                 0x2478   /* SRU_PINEN0 */
#define PBEN02_I_HIGH_REG                 0x2478   /* SRU_PINEN0 */
#define PBEN03_I_HIGH_REG                 0x2478   /* SRU_PINEN0 */
#define PBEN04_I_HIGH_REG                 0x2478   /* SRU_PINEN0 */
#define PBEN05_I_HIGH_REG                 0x2478   /* SRU_PINEN0 */
#define PBEN06_I_HIGH_REG                 0x2479   /* SRU_PINEN1 */
#define PBEN07_I_HIGH_REG                 0x2479   /* SRU_PINEN1 */
#define PBEN08_I_HIGH_REG                 0x2479   /* SRU_PINEN1 */
#define PBEN09_I_HIGH_REG                 0x2479   /* SRU_PINEN1 */
#define PBEN10_I_HIGH_REG                 0x2479   /* SRU_PINEN1 */
#define PBEN11_I_HIGH_REG                 0x247A   /* SRU_PINEN2 */
#define PBEN12_I_HIGH_REG                 0x247A   /* SRU_PINEN2 */
#define PBEN13_I_HIGH_REG                 0x247A   /* SRU_PINEN2 */
#define PBEN14_I_HIGH_REG                 0x247A   /* SRU_PINEN2 */
#define PBEN15_I_HIGH_REG                 0x247A   /* SRU_PINEN2 */
#define PBEN16_I_HIGH_REG                 0x247B   /* SRU_PINEN3 */
#define PBEN17_I_HIGH_REG                 0x247B   /* SRU_PINEN3 */
#define PBEN18_I_HIGH_REG                 0x247B   /* SRU_PINEN3 */
#define PBEN19_I_HIGH_REG                 0x247B   /* SRU_PINEN3 */
#define PBEN20_I_HIGH_REG                 0x247B   /* SRU_PINEN3 */

#define PBEN01_I_PBEN_LOW_O_REG           0x2478   /* SRU_PINEN0 */
#define PBEN02_I_PBEN_LOW_O_REG           0x2478   /* SRU_PINEN0 */
#define PBEN03_I_PBEN_LOW_O_REG           0x2478   /* SRU_PINEN0 */
#define PBEN04_I_PBEN_LOW_O_REG           0x2478   /* SRU_PINEN0 */
#define PBEN05_I_PBEN_LOW_O_REG           0x2478   /* SRU_PINEN0 */
#define PBEN06_I_PBEN_LOW_O_REG           0x2479   /* SRU_PINEN1 */
#define PBEN07_I_PBEN_LOW_O_REG           0x2479   /* SRU_PINEN1 */
#define PBEN08_I_PBEN_LOW_O_REG           0x2479   /* SRU_PINEN1 */
#define PBEN09_I_PBEN_LOW_O_REG           0x2479   /* SRU_PINEN1 */
#define PBEN10_I_PBEN_LOW_O_REG           0x2479   /* SRU_PINEN1 */
#define PBEN11_I_PBEN_LOW_O_REG           0x247A   /* SRU_PINEN2 */
#define PBEN12_I_PBEN_LOW_O_REG           0x247A   /* SRU_PINEN2 */
#define PBEN13_I_PBEN_LOW_O_REG           0x247A   /* SRU_PINEN2 */
#define PBEN14_I_PBEN_LOW_O_REG           0x247A   /* SRU_PINEN2 */
#define PBEN15_I_PBEN_LOW_O_REG           0x247A   /* SRU_PINEN2 */
#define PBEN16_I_PBEN_LOW_O_REG           0x247B   /* SRU_PINEN3 */
#define PBEN17_I_PBEN_LOW_O_REG           0x247B   /* SRU_PINEN3 */
#define PBEN18_I_PBEN_LOW_O_REG           0x247B   /* SRU_PINEN3 */
#define PBEN19_I_PBEN_LOW_O_REG           0x247B   /* SRU_PINEN3 */
#define PBEN20_I_PBEN_LOW_O_REG           0x247B   /* SRU_PINEN3 */

#define PBEN01_I_PBEN_HIGH_O_REG          0x2478   /* SRU_PINEN0 */
#define PBEN02_I_PBEN_HIGH_O_REG          0x2478   /* SRU_PINEN0 */
#define PBEN03_I_PBEN_HIGH_O_REG          0x2478   /* SRU_PINEN0 */
#define PBEN04_I_PBEN_HIGH_O_REG          0x2478   /* SRU_PINEN0 */
#define PBEN05_I_PBEN_HIGH_O_REG          0x2478   /* SRU_PINEN0 */
#define PBEN06_I_PBEN_HIGH_O_REG          0x2479   /* SRU_PINEN1 */
#define PBEN07_I_PBEN_HIGH_O_REG          0x2479   /* SRU_PINEN1 */
#define PBEN08_I_PBEN_HIGH_O_REG          0x2479   /* SRU_PINEN1 */
#define PBEN09_I_PBEN_HIGH_O_REG          0x2479   /* SRU_PINEN1 */
#define PBEN10_I_PBEN_HIGH_O_REG          0x2479   /* SRU_PINEN1 */
#define PBEN11_I_PBEN_HIGH_O_REG          0x247A   /* SRU_PINEN2 */
#define PBEN12_I_PBEN_HIGH_O_REG          0x247A   /* SRU_PINEN2 */
#define PBEN13_I_PBEN_HIGH_O_REG          0x247A   /* SRU_PINEN2 */
#define PBEN14_I_PBEN_HIGH_O_REG          0x247A   /* SRU_PINEN2 */
#define PBEN15_I_PBEN_HIGH_O_REG          0x247A   /* SRU_PINEN2 */
#define PBEN16_I_PBEN_HIGH_O_REG          0x247B   /* SRU_PINEN3 */
#define PBEN17_I_PBEN_HIGH_O_REG          0x247B   /* SRU_PINEN3 */
#define PBEN18_I_PBEN_HIGH_O_REG          0x247B   /* SRU_PINEN3 */
#define PBEN19_I_PBEN_HIGH_O_REG          0x247B   /* SRU_PINEN3 */
#define PBEN20_I_PBEN_HIGH_O_REG          0x247B   /* SRU_PINEN3 */

#define PBEN01_I__MISCA_0_PBEN_O_REG   0x2478   /* SRU_PINEN0 */
#define PBEN02_I__MISCA_0_PBEN_O_REG   0x2478   /* SRU_PINEN0 */
#define PBEN03_I__MISCA_0_PBEN_O_REG   0x2478   /* SRU_PINEN0 */
#define PBEN04_I__MISCA_0_PBEN_O_REG   0x2478   /* SRU_PINEN0 */
#define PBEN05_I__MISCA_0_PBEN_O_REG   0x2478   /* SRU_PINEN0 */
#define PBEN06_I__MISCA_0_PBEN_O_REG   0x2479   /* SRU_PINEN1 */
#define PBEN07_I__MISCA_0_PBEN_O_REG   0x2479   /* SRU_PINEN1 */
#define PBEN08_I__MISCA_0_PBEN_O_REG   0x2479   /* SRU_PINEN1 */
#define PBEN09_I__MISCA_0_PBEN_O_REG   0x2479   /* SRU_PINEN1 */
#define PBEN10_I__MISCA_0_PBEN_O_REG   0x2479   /* SRU_PINEN1 */
#define PBEN11_I__MISCA_0_PBEN_O_REG   0x247A   /* SRU_PINEN2 */
#define PBEN12_I__MISCA_0_PBEN_O_REG   0x247A   /* SRU_PINEN2 */
#define PBEN13_I__MISCA_0_PBEN_O_REG   0x247A   /* SRU_PINEN2 */
#define PBEN14_I__MISCA_0_PBEN_O_REG   0x247A   /* SRU_PINEN2 */
#define PBEN15_I__MISCA_0_PBEN_O_REG   0x247A   /* SRU_PINEN2 */
#define PBEN16_I__MISCA_0_PBEN_O_REG   0x247B   /* SRU_PINEN3 */
#define PBEN17_I__MISCA_0_PBEN_O_REG   0x247B   /* SRU_PINEN3 */
#define PBEN18_I__MISCA_0_PBEN_O_REG   0x247B   /* SRU_PINEN3 */
#define PBEN19_I__MISCA_0_PBEN_O_REG   0x247B   /* SRU_PINEN3 */
#define PBEN20_I__MISCA_0_PBEN_O_REG   0x247B   /* SRU_PINEN3 */

#define PBEN01_I__MISCA_1_PBEN_O_REG   0x2478   /* SRU_PINEN0 */
#define PBEN02_I__MISCA_1_PBEN_O_REG   0x2478   /* SRU_PINEN0 */
#define PBEN03_I__MISCA_1_PBEN_O_REG   0x2478   /* SRU_PINEN0 */
#define PBEN04_I__MISCA_1_PBEN_O_REG   0x2478   /* SRU_PINEN0 */
#define PBEN05_I__MISCA_1_PBEN_O_REG   0x2478   /* SRU_PINEN0 */
#define PBEN06_I__MISCA_1_PBEN_O_REG   0x2479   /* SRU_PINEN1 */
#define PBEN07_I__MISCA_1_PBEN_O_REG   0x2479   /* SRU_PINEN1 */
#define PBEN08_I__MISCA_1_PBEN_O_REG   0x2479   /* SRU_PINEN1 */
#define PBEN09_I__MISCA_1_PBEN_O_REG   0x2479   /* SRU_PINEN1 */
#define PBEN10_I__MISCA_1_PBEN_O_REG   0x2479   /* SRU_PINEN1 */
#define PBEN11_I__MISCA_1_PBEN_O_REG   0x247A   /* SRU_PINEN2 */
#define PBEN12_I__MISCA_1_PBEN_O_REG   0x247A   /* SRU_PINEN2 */
#define PBEN13_I__MISCA_1_PBEN_O_REG   0x247A   /* SRU_PINEN2 */
#define PBEN14_I__MISCA_1_PBEN_O_REG   0x247A   /* SRU_PINEN2 */
#define PBEN15_I__MISCA_1_PBEN_O_REG   0x247A   /* SRU_PINEN2 */
#define PBEN16_I__MISCA_1_PBEN_O_REG   0x247B   /* SRU_PINEN3 */
#define PBEN17_I__MISCA_1_PBEN_O_REG   0x247B   /* SRU_PINEN3 */
#define PBEN18_I__MISCA_1_PBEN_O_REG   0x247B   /* SRU_PINEN3 */
#define PBEN19_I__MISCA_1_PBEN_O_REG   0x247B   /* SRU_PINEN3 */
#define PBEN20_I__MISCA_1_PBEN_O_REG   0x247B   /* SRU_PINEN3 */

#define PBEN01_I__MISCA_2_PBEN_O_REG   0x2478   /* SRU_PINEN0 */
#define PBEN02_I__MISCA_2_PBEN_O_REG   0x2478   /* SRU_PINEN0 */
#define PBEN03_I__MISCA_2_PBEN_O_REG   0x2478   /* SRU_PINEN0 */
#define PBEN04_I__MISCA_2_PBEN_O_REG   0x2478   /* SRU_PINEN0 */
#define PBEN05_I__MISCA_2_PBEN_O_REG   0x2478   /* SRU_PINEN0 */
#define PBEN06_I__MISCA_2_PBEN_O_REG   0x2479   /* SRU_PINEN1 */
#define PBEN07_I__MISCA_2_PBEN_O_REG   0x2479   /* SRU_PINEN1 */
#define PBEN08_I__MISCA_2_PBEN_O_REG   0x2479   /* SRU_PINEN1 */
#define PBEN09_I__MISCA_2_PBEN_O_REG   0x2479   /* SRU_PINEN1 */
#define PBEN10_I__MISCA_2_PBEN_O_REG   0x2479   /* SRU_PINEN1 */
#define PBEN11_I__MISCA_2_PBEN_O_REG   0x247A   /* SRU_PINEN2 */
#define PBEN12_I__MISCA_2_PBEN_O_REG   0x247A   /* SRU_PINEN2 */
#define PBEN13_I__MISCA_2_PBEN_O_REG   0x247A   /* SRU_PINEN2 */
#define PBEN14_I__MISCA_2_PBEN_O_REG   0x247A   /* SRU_PINEN2 */
#define PBEN15_I__MISCA_2_PBEN_O_REG   0x247A   /* SRU_PINEN2 */
#define PBEN16_I__MISCA_2_PBEN_O_REG   0x247B   /* SRU_PINEN3 */
#define PBEN17_I__MISCA_2_PBEN_O_REG   0x247B   /* SRU_PINEN3 */
#define PBEN18_I__MISCA_2_PBEN_O_REG   0x247B   /* SRU_PINEN3 */
#define PBEN19_I__MISCA_2_PBEN_O_REG   0x247B   /* SRU_PINEN3 */
#define PBEN20_I__MISCA_2_PBEN_O_REG   0x247B   /* SRU_PINEN3 */

#define PBEN01_I__MISCA_3_PBEN_O_REG   0x2478   /* SRU_PINEN0 */
#define PBEN02_I__MISCA_3_PBEN_O_REG   0x2478   /* SRU_PINEN0 */
#define PBEN03_I__MISCA_3_PBEN_O_REG   0x2478   /* SRU_PINEN0 */
#define PBEN04_I__MISCA_3_PBEN_O_REG   0x2478   /* SRU_PINEN0 */
#define PBEN05_I__MISCA_3_PBEN_O_REG   0x2478   /* SRU_PINEN0 */
#define PBEN06_I__MISCA_3_PBEN_O_REG   0x2479   /* SRU_PINEN1 */
#define PBEN07_I__MISCA_3_PBEN_O_REG   0x2479   /* SRU_PINEN1 */
#define PBEN08_I__MISCA_3_PBEN_O_REG   0x2479   /* SRU_PINEN1 */
#define PBEN09_I__MISCA_3_PBEN_O_REG   0x2479   /* SRU_PINEN1 */
#define PBEN10_I__MISCA_3_PBEN_O_REG   0x2479   /* SRU_PINEN1 */
#define PBEN11_I__MISCA_3_PBEN_O_REG   0x247A   /* SRU_PINEN2 */
#define PBEN12_I__MISCA_3_PBEN_O_REG   0x247A   /* SRU_PINEN2 */
#define PBEN13_I__MISCA_3_PBEN_O_REG   0x247A   /* SRU_PINEN2 */
#define PBEN14_I__MISCA_3_PBEN_O_REG   0x247A   /* SRU_PINEN2 */
#define PBEN15_I__MISCA_3_PBEN_O_REG   0x247A   /* SRU_PINEN2 */
#define PBEN16_I__MISCA_3_PBEN_O_REG   0x247B   /* SRU_PINEN3 */
#define PBEN17_I__MISCA_3_PBEN_O_REG   0x247B   /* SRU_PINEN3 */
#define PBEN18_I__MISCA_3_PBEN_O_REG   0x247B   /* SRU_PINEN3 */
#define PBEN19_I__MISCA_3_PBEN_O_REG   0x247B   /* SRU_PINEN3 */
#define PBEN20_I__MISCA_3_PBEN_O_REG   0x247B   /* SRU_PINEN3 */

#define PBEN01_I__MISCA_4_PBEN_O_REG   0x2478   /* SRU_PINEN0 */
#define PBEN02_I__MISCA_4_PBEN_O_REG   0x2478   /* SRU_PINEN0 */
#define PBEN03_I__MISCA_4_PBEN_O_REG   0x2478   /* SRU_PINEN0 */
#define PBEN04_I__MISCA_4_PBEN_O_REG   0x2478   /* SRU_PINEN0 */
#define PBEN05_I__MISCA_4_PBEN_O_REG   0x2478   /* SRU_PINEN0 */
#define PBEN06_I__MISCA_4_PBEN_O_REG   0x2479   /* SRU_PINEN1 */
#define PBEN07_I__MISCA_4_PBEN_O_REG   0x2479   /* SRU_PINEN1 */
#define PBEN08_I__MISCA_4_PBEN_O_REG   0x2479   /* SRU_PINEN1 */
#define PBEN09_I__MISCA_4_PBEN_O_REG   0x2479   /* SRU_PINEN1 */
#define PBEN10_I__MISCA_4_PBEN_O_REG   0x2479   /* SRU_PINEN1 */
#define PBEN11_I__MISCA_4_PBEN_O_REG   0x247A   /* SRU_PINEN2 */
#define PBEN12_I__MISCA_4_PBEN_O_REG   0x247A   /* SRU_PINEN2 */
#define PBEN13_I__MISCA_4_PBEN_O_REG   0x247A   /* SRU_PINEN2 */
#define PBEN14_I__MISCA_4_PBEN_O_REG   0x247A   /* SRU_PINEN2 */
#define PBEN15_I__MISCA_4_PBEN_O_REG   0x247A   /* SRU_PINEN2 */
#define PBEN16_I__MISCA_4_PBEN_O_REG   0x247B   /* SRU_PINEN3 */
#define PBEN17_I__MISCA_4_PBEN_O_REG   0x247B   /* SRU_PINEN3 */
#define PBEN18_I__MISCA_4_PBEN_O_REG   0x247B   /* SRU_PINEN3 */
#define PBEN19_I__MISCA_4_PBEN_O_REG   0x247B   /* SRU_PINEN3 */
#define PBEN20_I__MISCA_4_PBEN_O_REG   0x247B   /* SRU_PINEN3 */

#define PBEN01_I__MISCA_5_PBEN_O_REG   0x2478   /* SRU_PINEN0 */
#define PBEN02_I__MISCA_5_PBEN_O_REG   0x2478   /* SRU_PINEN0 */
#define PBEN03_I__MISCA_5_PBEN_O_REG   0x2478   /* SRU_PINEN0 */
#define PBEN04_I__MISCA_5_PBEN_O_REG   0x2478   /* SRU_PINEN0 */
#define PBEN05_I__MISCA_5_PBEN_O_REG   0x2478   /* SRU_PINEN0 */
#define PBEN06_I__MISCA_5_PBEN_O_REG   0x2479   /* SRU_PINEN1 */
#define PBEN07_I__MISCA_5_PBEN_O_REG   0x2479   /* SRU_PINEN1 */
#define PBEN08_I__MISCA_5_PBEN_O_REG   0x2479   /* SRU_PINEN1 */
#define PBEN09_I__MISCA_5_PBEN_O_REG   0x2479   /* SRU_PINEN1 */
#define PBEN10_I__MISCA_5_PBEN_O_REG   0x2479   /* SRU_PINEN1 */
#define PBEN11_I__MISCA_5_PBEN_O_REG   0x247A   /* SRU_PINEN2 */
#define PBEN12_I__MISCA_5_PBEN_O_REG   0x247A   /* SRU_PINEN2 */
#define PBEN13_I__MISCA_5_PBEN_O_REG   0x247A   /* SRU_PINEN2 */
#define PBEN14_I__MISCA_5_PBEN_O_REG   0x247A   /* SRU_PINEN2 */
#define PBEN15_I__MISCA_5_PBEN_O_REG   0x247A   /* SRU_PINEN2 */
#define PBEN16_I__MISCA_5_PBEN_O_REG   0x247B   /* SRU_PINEN3 */
#define PBEN17_I__MISCA_5_PBEN_O_REG   0x247B   /* SRU_PINEN3 */
#define PBEN18_I__MISCA_5_PBEN_O_REG   0x247B   /* SRU_PINEN3 */
#define PBEN19_I__MISCA_5_PBEN_O_REG   0x247B   /* SRU_PINEN3 */
#define PBEN20_I__MISCA_5_PBEN_O_REG   0x247B   /* SRU_PINEN3 */

#define PBEN01_I_SPORT0_CLK_PBEN_O_REG    0x2478   /* SRU_PINEN0 */
#define PBEN02_I_SPORT0_CLK_PBEN_O_REG    0x2478   /* SRU_PINEN0 */
#define PBEN03_I_SPORT0_CLK_PBEN_O_REG    0x2478   /* SRU_PINEN0 */
#define PBEN04_I_SPORT0_CLK_PBEN_O_REG    0x2478   /* SRU_PINEN0 */
#define PBEN05_I_SPORT0_CLK_PBEN_O_REG    0x2478   /* SRU_PINEN0 */
#define PBEN06_I_SPORT0_CLK_PBEN_O_REG    0x2479   /* SRU_PINEN1 */
#define PBEN07_I_SPORT0_CLK_PBEN_O_REG    0x2479   /* SRU_PINEN1 */
#define PBEN08_I_SPORT0_CLK_PBEN_O_REG    0x2479   /* SRU_PINEN1 */
#define PBEN09_I_SPORT0_CLK_PBEN_O_REG    0x2479   /* SRU_PINEN1 */
#define PBEN10_I_SPORT0_CLK_PBEN_O_REG    0x2479   /* SRU_PINEN1 */
#define PBEN11_I_SPORT0_CLK_PBEN_O_REG    0x247A   /* SRU_PINEN2 */
#define PBEN12_I_SPORT0_CLK_PBEN_O_REG    0x247A   /* SRU_PINEN2 */
#define PBEN13_I_SPORT0_CLK_PBEN_O_REG    0x247A   /* SRU_PINEN2 */
#define PBEN14_I_SPORT0_CLK_PBEN_O_REG    0x247A   /* SRU_PINEN2 */
#define PBEN15_I_SPORT0_CLK_PBEN_O_REG    0x247A   /* SRU_PINEN2 */
#define PBEN16_I_SPORT0_CLK_PBEN_O_REG    0x247B   /* SRU_PINEN3 */
#define PBEN17_I_SPORT0_CLK_PBEN_O_REG    0x247B   /* SRU_PINEN3 */
#define PBEN18_I_SPORT0_CLK_PBEN_O_REG    0x247B   /* SRU_PINEN3 */
#define PBEN19_I_SPORT0_CLK_PBEN_O_REG    0x247B   /* SRU_PINEN3 */
#define PBEN20_I_SPORT0_CLK_PBEN_O_REG    0x247B   /* SRU_PINEN3 */

#define PBEN01_I_SPORT0_FS_PBEN_O_REG     0x2478   /* SRU_PINEN0 */
#define PBEN02_I_SPORT0_FS_PBEN_O_REG     0x2478   /* SRU_PINEN0 */
#define PBEN03_I_SPORT0_FS_PBEN_O_REG     0x2478   /* SRU_PINEN0 */
#define PBEN04_I_SPORT0_FS_PBEN_O_REG     0x2478   /* SRU_PINEN0 */
#define PBEN05_I_SPORT0_FS_PBEN_O_REG     0x2478   /* SRU_PINEN0 */
#define PBEN06_I_SPORT0_FS_PBEN_O_REG     0x2479   /* SRU_PINEN1 */
#define PBEN07_I_SPORT0_FS_PBEN_O_REG     0x2479   /* SRU_PINEN1 */
#define PBEN08_I_SPORT0_FS_PBEN_O_REG     0x2479   /* SRU_PINEN1 */
#define PBEN09_I_SPORT0_FS_PBEN_O_REG     0x2479   /* SRU_PINEN1 */
#define PBEN10_I_SPORT0_FS_PBEN_O_REG     0x2479   /* SRU_PINEN1 */
#define PBEN11_I_SPORT0_FS_PBEN_O_REG     0x247A   /* SRU_PINEN2 */
#define PBEN12_I_SPORT0_FS_PBEN_O_REG     0x247A   /* SRU_PINEN2 */
#define PBEN13_I_SPORT0_FS_PBEN_O_REG     0x247A   /* SRU_PINEN2 */
#define PBEN14_I_SPORT0_FS_PBEN_O_REG     0x247A   /* SRU_PINEN2 */
#define PBEN15_I_SPORT0_FS_PBEN_O_REG     0x247A   /* SRU_PINEN2 */
#define PBEN16_I_SPORT0_FS_PBEN_O_REG     0x247B   /* SRU_PINEN3 */
#define PBEN17_I_SPORT0_FS_PBEN_O_REG     0x247B   /* SRU_PINEN3 */
#define PBEN18_I_SPORT0_FS_PBEN_O_REG     0x247B   /* SRU_PINEN3 */
#define PBEN19_I_SPORT0_FS_PBEN_O_REG     0x247B   /* SRU_PINEN3 */
#define PBEN20_I_SPORT0_FS_PBEN_O_REG     0x247B   /* SRU_PINEN3 */

#define PBEN01_I_SPORT0_DA_PBEN_O_REG     0x2478   /* SRU_PINEN0 */
#define PBEN02_I_SPORT0_DA_PBEN_O_REG     0x2478   /* SRU_PINEN0 */
#define PBEN03_I_SPORT0_DA_PBEN_O_REG     0x2478   /* SRU_PINEN0 */
#define PBEN04_I_SPORT0_DA_PBEN_O_REG     0x2478   /* SRU_PINEN0 */
#define PBEN05_I_SPORT0_DA_PBEN_O_REG     0x2478   /* SRU_PINEN0 */
#define PBEN06_I_SPORT0_DA_PBEN_O_REG     0x2479   /* SRU_PINEN1 */
#define PBEN07_I_SPORT0_DA_PBEN_O_REG     0x2479   /* SRU_PINEN1 */
#define PBEN08_I_SPORT0_DA_PBEN_O_REG     0x2479   /* SRU_PINEN1 */
#define PBEN09_I_SPORT0_DA_PBEN_O_REG     0x2479   /* SRU_PINEN1 */
#define PBEN10_I_SPORT0_DA_PBEN_O_REG     0x2479   /* SRU_PINEN1 */
#define PBEN11_I_SPORT0_DA_PBEN_O_REG     0x247A   /* SRU_PINEN2 */
#define PBEN12_I_SPORT0_DA_PBEN_O_REG     0x247A   /* SRU_PINEN2 */
#define PBEN13_I_SPORT0_DA_PBEN_O_REG     0x247A   /* SRU_PINEN2 */
#define PBEN14_I_SPORT0_DA_PBEN_O_REG     0x247A   /* SRU_PINEN2 */
#define PBEN15_I_SPORT0_DA_PBEN_O_REG     0x247A   /* SRU_PINEN2 */
#define PBEN16_I_SPORT0_DA_PBEN_O_REG     0x247B   /* SRU_PINEN3 */
#define PBEN17_I_SPORT0_DA_PBEN_O_REG     0x247B   /* SRU_PINEN3 */
#define PBEN18_I_SPORT0_DA_PBEN_O_REG     0x247B   /* SRU_PINEN3 */
#define PBEN19_I_SPORT0_DA_PBEN_O_REG     0x247B   /* SRU_PINEN3 */
#define PBEN20_I_SPORT0_DA_PBEN_O_REG     0x247B   /* SRU_PINEN3 */

#define PBEN01_I_SPORT0_Db_PBEN_O_REG     0x2478   /* SRU_PINEN0 */
#define PBEN02_I_SPORT0_Db_PBEN_O_REG     0x2478   /* SRU_PINEN0 */
#define PBEN03_I_SPORT0_Db_PBEN_O_REG     0x2478   /* SRU_PINEN0 */
#define PBEN04_I_SPORT0_Db_PBEN_O_REG     0x2478   /* SRU_PINEN0 */
#define PBEN05_I_SPORT0_Db_PBEN_O_REG     0x2478   /* SRU_PINEN0 */
#define PBEN06_I_SPORT0_Db_PBEN_O_REG     0x2479   /* SRU_PINEN1 */
#define PBEN07_I_SPORT0_Db_PBEN_O_REG     0x2479   /* SRU_PINEN1 */
#define PBEN08_I_SPORT0_Db_PBEN_O_REG     0x2479   /* SRU_PINEN1 */
#define PBEN09_I_SPORT0_Db_PBEN_O_REG     0x2479   /* SRU_PINEN1 */
#define PBEN10_I_SPORT0_Db_PBEN_O_REG     0x2479   /* SRU_PINEN1 */
#define PBEN11_I_SPORT0_Db_PBEN_O_REG     0x247A   /* SRU_PINEN2 */
#define PBEN12_I_SPORT0_Db_PBEN_O_REG     0x247A   /* SRU_PINEN2 */
#define PBEN13_I_SPORT0_Db_PBEN_O_REG     0x247A   /* SRU_PINEN2 */
#define PBEN14_I_SPORT0_Db_PBEN_O_REG     0x247A   /* SRU_PINEN2 */
#define PBEN15_I_SPORT0_Db_PBEN_O_REG     0x247A   /* SRU_PINEN2 */
#define PBEN16_I_SPORT0_Db_PBEN_O_REG     0x247B   /* SRU_PINEN3 */
#define PBEN17_I_SPORT0_Db_PBEN_O_REG     0x247B   /* SRU_PINEN3 */
#define PBEN18_I_SPORT0_Db_PBEN_O_REG     0x247B   /* SRU_PINEN3 */
#define PBEN19_I_SPORT0_Db_PBEN_O_REG     0x247B   /* SRU_PINEN3 */
#define PBEN20_I_SPORT0_Db_PBEN_O_REG     0x247B   /* SRU_PINEN3 */

#define PBEN01_I_SPORT1_CLK_PBEN_O_REG    0x2478   /* SRU_PINEN0 */
#define PBEN02_I_SPORT1_CLK_PBEN_O_REG    0x2478   /* SRU_PINEN0 */
#define PBEN03_I_SPORT1_CLK_PBEN_O_REG    0x2478   /* SRU_PINEN0 */
#define PBEN04_I_SPORT1_CLK_PBEN_O_REG    0x2478   /* SRU_PINEN0 */
#define PBEN05_I_SPORT1_CLK_PBEN_O_REG    0x2478   /* SRU_PINEN0 */
#define PBEN06_I_SPORT1_CLK_PBEN_O_REG    0x2479   /* SRU_PINEN1 */
#define PBEN07_I_SPORT1_CLK_PBEN_O_REG    0x2479   /* SRU_PINEN1 */
#define PBEN08_I_SPORT1_CLK_PBEN_O_REG    0x2479   /* SRU_PINEN1 */
#define PBEN09_I_SPORT1_CLK_PBEN_O_REG    0x2479   /* SRU_PINEN1 */
#define PBEN10_I_SPORT1_CLK_PBEN_O_REG    0x2479   /* SRU_PINEN1 */
#define PBEN11_I_SPORT1_CLK_PBEN_O_REG    0x247A   /* SRU_PINEN2 */
#define PBEN12_I_SPORT1_CLK_PBEN_O_REG    0x247A   /* SRU_PINEN2 */
#define PBEN13_I_SPORT1_CLK_PBEN_O_REG    0x247A   /* SRU_PINEN2 */
#define PBEN14_I_SPORT1_CLK_PBEN_O_REG    0x247A   /* SRU_PINEN2 */
#define PBEN15_I_SPORT1_CLK_PBEN_O_REG    0x247A   /* SRU_PINEN2 */
#define PBEN16_I_SPORT1_CLK_PBEN_O_REG    0x247B   /* SRU_PINEN3 */
#define PBEN17_I_SPORT1_CLK_PBEN_O_REG    0x247B   /* SRU_PINEN3 */
#define PBEN18_I_SPORT1_CLK_PBEN_O_REG    0x247B   /* SRU_PINEN3 */
#define PBEN19_I_SPORT1_CLK_PBEN_O_REG    0x247B   /* SRU_PINEN3 */
#define PBEN20_I_SPORT1_CLK_PBEN_O_REG    0x247B   /* SRU_PINEN3 */

#define PBEN01_I_SPORT1_FS_PBEN_O_REG     0x2478   /* SRU_PINEN0 */
#define PBEN02_I_SPORT1_FS_PBEN_O_REG     0x2478   /* SRU_PINEN0 */
#define PBEN03_I_SPORT1_FS_PBEN_O_REG     0x2478   /* SRU_PINEN0 */
#define PBEN04_I_SPORT1_FS_PBEN_O_REG     0x2478   /* SRU_PINEN0 */
#define PBEN05_I_SPORT1_FS_PBEN_O_REG     0x2478   /* SRU_PINEN0 */
#define PBEN06_I_SPORT1_FS_PBEN_O_REG     0x2479   /* SRU_PINEN1 */
#define PBEN07_I_SPORT1_FS_PBEN_O_REG     0x2479   /* SRU_PINEN1 */
#define PBEN08_I_SPORT1_FS_PBEN_O_REG     0x2479   /* SRU_PINEN1 */
#define PBEN09_I_SPORT1_FS_PBEN_O_REG     0x2479   /* SRU_PINEN1 */
#define PBEN10_I_SPORT1_FS_PBEN_O_REG     0x2479   /* SRU_PINEN1 */
#define PBEN11_I_SPORT1_FS_PBEN_O_REG     0x247A   /* SRU_PINEN2 */
#define PBEN12_I_SPORT1_FS_PBEN_O_REG     0x247A   /* SRU_PINEN2 */
#define PBEN13_I_SPORT1_FS_PBEN_O_REG     0x247A   /* SRU_PINEN2 */
#define PBEN14_I_SPORT1_FS_PBEN_O_REG     0x247A   /* SRU_PINEN2 */
#define PBEN15_I_SPORT1_FS_PBEN_O_REG     0x247A   /* SRU_PINEN2 */
#define PBEN16_I_SPORT1_FS_PBEN_O_REG     0x247B   /* SRU_PINEN3 */
#define PBEN17_I_SPORT1_FS_PBEN_O_REG     0x247B   /* SRU_PINEN3 */
#define PBEN18_I_SPORT1_FS_PBEN_O_REG     0x247B   /* SRU_PINEN3 */
#define PBEN19_I_SPORT1_FS_PBEN_O_REG     0x247B   /* SRU_PINEN3 */
#define PBEN20_I_SPORT1_FS_PBEN_O_REG     0x247B   /* SRU_PINEN3 */

#define PBEN01_I_SPORT1_DA_PBEN_O_REG     0x2478   /* SRU_PINEN0 */
#define PBEN02_I_SPORT1_DA_PBEN_O_REG     0x2478   /* SRU_PINEN0 */
#define PBEN03_I_SPORT1_DA_PBEN_O_REG     0x2478   /* SRU_PINEN0 */
#define PBEN04_I_SPORT1_DA_PBEN_O_REG     0x2478   /* SRU_PINEN0 */
#define PBEN05_I_SPORT1_DA_PBEN_O_REG     0x2478   /* SRU_PINEN0 */
#define PBEN06_I_SPORT1_DA_PBEN_O_REG     0x2479   /* SRU_PINEN1 */
#define PBEN07_I_SPORT1_DA_PBEN_O_REG     0x2479   /* SRU_PINEN1 */
#define PBEN08_I_SPORT1_DA_PBEN_O_REG     0x2479   /* SRU_PINEN1 */
#define PBEN09_I_SPORT1_DA_PBEN_O_REG     0x2479   /* SRU_PINEN1 */
#define PBEN10_I_SPORT1_DA_PBEN_O_REG     0x2479   /* SRU_PINEN1 */
#define PBEN11_I_SPORT1_DA_PBEN_O_REG     0x247A   /* SRU_PINEN2 */
#define PBEN12_I_SPORT1_DA_PBEN_O_REG     0x247A   /* SRU_PINEN2 */
#define PBEN13_I_SPORT1_DA_PBEN_O_REG     0x247A   /* SRU_PINEN2 */
#define PBEN14_I_SPORT1_DA_PBEN_O_REG     0x247A   /* SRU_PINEN2 */
#define PBEN15_I_SPORT1_DA_PBEN_O_REG     0x247A   /* SRU_PINEN2 */
#define PBEN16_I_SPORT1_DA_PBEN_O_REG     0x247B   /* SRU_PINEN3 */
#define PBEN17_I_SPORT1_DA_PBEN_O_REG     0x247B   /* SRU_PINEN3 */
#define PBEN18_I_SPORT1_DA_PBEN_O_REG     0x247B   /* SRU_PINEN3 */
#define PBEN19_I_SPORT1_DA_PBEN_O_REG     0x247B   /* SRU_PINEN3 */
#define PBEN20_I_SPORT1_DA_PBEN_O_REG     0x247B   /* SRU_PINEN3 */

#define PBEN01_I_SPORT1_Db_PBEN_O_REG     0x2478   /* SRU_PINEN0 */
#define PBEN02_I_SPORT1_Db_PBEN_O_REG     0x2478   /* SRU_PINEN0 */
#define PBEN03_I_SPORT1_Db_PBEN_O_REG     0x2478   /* SRU_PINEN0 */
#define PBEN04_I_SPORT1_Db_PBEN_O_REG     0x2478   /* SRU_PINEN0 */
#define PBEN05_I_SPORT1_Db_PBEN_O_REG     0x2478   /* SRU_PINEN0 */
#define PBEN06_I_SPORT1_Db_PBEN_O_REG     0x2479   /* SRU_PINEN1 */
#define PBEN07_I_SPORT1_Db_PBEN_O_REG     0x2479   /* SRU_PINEN1 */
#define PBEN08_I_SPORT1_Db_PBEN_O_REG     0x2479   /* SRU_PINEN1 */
#define PBEN09_I_SPORT1_Db_PBEN_O_REG     0x2479   /* SRU_PINEN1 */
#define PBEN10_I_SPORT1_Db_PBEN_O_REG     0x2479   /* SRU_PINEN1 */
#define PBEN11_I_SPORT1_Db_PBEN_O_REG     0x247A   /* SRU_PINEN2 */
#define PBEN12_I_SPORT1_Db_PBEN_O_REG     0x247A   /* SRU_PINEN2 */
#define PBEN13_I_SPORT1_Db_PBEN_O_REG     0x247A   /* SRU_PINEN2 */
#define PBEN14_I_SPORT1_Db_PBEN_O_REG     0x247A   /* SRU_PINEN2 */
#define PBEN15_I_SPORT1_Db_PBEN_O_REG     0x247A   /* SRU_PINEN2 */
#define PBEN16_I_SPORT1_Db_PBEN_O_REG     0x247B   /* SRU_PINEN3 */
#define PBEN17_I_SPORT1_Db_PBEN_O_REG     0x247B   /* SRU_PINEN3 */
#define PBEN18_I_SPORT1_Db_PBEN_O_REG     0x247B   /* SRU_PINEN3 */
#define PBEN19_I_SPORT1_Db_PBEN_O_REG     0x247B   /* SRU_PINEN3 */
#define PBEN20_I_SPORT1_Db_PBEN_O_REG     0x247B   /* SRU_PINEN3 */

#define PBEN01_I_SPORT2_CLK_PBEN_O_REG    0x2478   /* SRU_PINEN0 */
#define PBEN02_I_SPORT2_CLK_PBEN_O_REG    0x2478   /* SRU_PINEN0 */
#define PBEN03_I_SPORT2_CLK_PBEN_O_REG    0x2478   /* SRU_PINEN0 */
#define PBEN04_I_SPORT2_CLK_PBEN_O_REG    0x2478   /* SRU_PINEN0 */
#define PBEN05_I_SPORT2_CLK_PBEN_O_REG    0x2478   /* SRU_PINEN0 */
#define PBEN06_I_SPORT2_CLK_PBEN_O_REG    0x2479   /* SRU_PINEN1 */
#define PBEN07_I_SPORT2_CLK_PBEN_O_REG    0x2479   /* SRU_PINEN1 */
#define PBEN08_I_SPORT2_CLK_PBEN_O_REG    0x2479   /* SRU_PINEN1 */
#define PBEN09_I_SPORT2_CLK_PBEN_O_REG    0x2479   /* SRU_PINEN1 */
#define PBEN10_I_SPORT2_CLK_PBEN_O_REG    0x2479   /* SRU_PINEN1 */
#define PBEN11_I_SPORT2_CLK_PBEN_O_REG    0x247A   /* SRU_PINEN2 */
#define PBEN12_I_SPORT2_CLK_PBEN_O_REG    0x247A   /* SRU_PINEN2 */
#define PBEN13_I_SPORT2_CLK_PBEN_O_REG    0x247A   /* SRU_PINEN2 */
#define PBEN14_I_SPORT2_CLK_PBEN_O_REG    0x247A   /* SRU_PINEN2 */
#define PBEN15_I_SPORT2_CLK_PBEN_O_REG    0x247A   /* SRU_PINEN2 */
#define PBEN16_I_SPORT2_CLK_PBEN_O_REG    0x247B   /* SRU_PINEN3 */
#define PBEN17_I_SPORT2_CLK_PBEN_O_REG    0x247B   /* SRU_PINEN3 */
#define PBEN18_I_SPORT2_CLK_PBEN_O_REG    0x247B   /* SRU_PINEN3 */
#define PBEN19_I_SPORT2_CLK_PBEN_O_REG    0x247B   /* SRU_PINEN3 */
#define PBEN20_I_SPORT2_CLK_PBEN_O_REG    0x247B   /* SRU_PINEN3 */

#define PBEN01_I_SPORT2_FS_PBEN_O_REG     0x2478   /* SRU_PINEN0 */
#define PBEN02_I_SPORT2_FS_PBEN_O_REG     0x2478   /* SRU_PINEN0 */
#define PBEN03_I_SPORT2_FS_PBEN_O_REG     0x2478   /* SRU_PINEN0 */
#define PBEN04_I_SPORT2_FS_PBEN_O_REG     0x2478   /* SRU_PINEN0 */
#define PBEN05_I_SPORT2_FS_PBEN_O_REG     0x2478   /* SRU_PINEN0 */
#define PBEN06_I_SPORT2_FS_PBEN_O_REG     0x2479   /* SRU_PINEN1 */
#define PBEN07_I_SPORT2_FS_PBEN_O_REG     0x2479   /* SRU_PINEN1 */
#define PBEN08_I_SPORT2_FS_PBEN_O_REG     0x2479   /* SRU_PINEN1 */
#define PBEN09_I_SPORT2_FS_PBEN_O_REG     0x2479   /* SRU_PINEN1 */
#define PBEN10_I_SPORT2_FS_PBEN_O_REG     0x2479   /* SRU_PINEN1 */
#define PBEN11_I_SPORT2_FS_PBEN_O_REG     0x247A   /* SRU_PINEN2 */
#define PBEN12_I_SPORT2_FS_PBEN_O_REG     0x247A   /* SRU_PINEN2 */
#define PBEN13_I_SPORT2_FS_PBEN_O_REG     0x247A   /* SRU_PINEN2 */
#define PBEN14_I_SPORT2_FS_PBEN_O_REG     0x247A   /* SRU_PINEN2 */
#define PBEN15_I_SPORT2_FS_PBEN_O_REG     0x247A   /* SRU_PINEN2 */
#define PBEN16_I_SPORT2_FS_PBEN_O_REG     0x247B   /* SRU_PINEN3 */
#define PBEN17_I_SPORT2_FS_PBEN_O_REG     0x247B   /* SRU_PINEN3 */
#define PBEN18_I_SPORT2_FS_PBEN_O_REG     0x247B   /* SRU_PINEN3 */
#define PBEN19_I_SPORT2_FS_PBEN_O_REG     0x247B   /* SRU_PINEN3 */
#define PBEN20_I_SPORT2_FS_PBEN_O_REG     0x247B   /* SRU_PINEN3 */

#define PBEN01_I_SPORT2_DA_PBEN_O_REG     0x2478   /* SRU_PINEN0 */
#define PBEN02_I_SPORT2_DA_PBEN_O_REG     0x2478   /* SRU_PINEN0 */
#define PBEN03_I_SPORT2_DA_PBEN_O_REG     0x2478   /* SRU_PINEN0 */
#define PBEN04_I_SPORT2_DA_PBEN_O_REG     0x2478   /* SRU_PINEN0 */
#define PBEN05_I_SPORT2_DA_PBEN_O_REG     0x2478   /* SRU_PINEN0 */
#define PBEN06_I_SPORT2_DA_PBEN_O_REG     0x2479   /* SRU_PINEN1 */
#define PBEN07_I_SPORT2_DA_PBEN_O_REG     0x2479   /* SRU_PINEN1 */
#define PBEN08_I_SPORT2_DA_PBEN_O_REG     0x2479   /* SRU_PINEN1 */
#define PBEN09_I_SPORT2_DA_PBEN_O_REG     0x2479   /* SRU_PINEN1 */
#define PBEN10_I_SPORT2_DA_PBEN_O_REG     0x2479   /* SRU_PINEN1 */
#define PBEN11_I_SPORT2_DA_PBEN_O_REG     0x247A   /* SRU_PINEN2 */
#define PBEN12_I_SPORT2_DA_PBEN_O_REG     0x247A   /* SRU_PINEN2 */
#define PBEN13_I_SPORT2_DA_PBEN_O_REG     0x247A   /* SRU_PINEN2 */
#define PBEN14_I_SPORT2_DA_PBEN_O_REG     0x247A   /* SRU_PINEN2 */
#define PBEN15_I_SPORT2_DA_PBEN_O_REG     0x247A   /* SRU_PINEN2 */
#define PBEN16_I_SPORT2_DA_PBEN_O_REG     0x247B   /* SRU_PINEN3 */
#define PBEN17_I_SPORT2_DA_PBEN_O_REG     0x247B   /* SRU_PINEN3 */
#define PBEN18_I_SPORT2_DA_PBEN_O_REG     0x247B   /* SRU_PINEN3 */
#define PBEN19_I_SPORT2_DA_PBEN_O_REG     0x247B   /* SRU_PINEN3 */
#define PBEN20_I_SPORT2_DA_PBEN_O_REG     0x247B   /* SRU_PINEN3 */

#define PBEN01_I_SPORT2_Db_PBEN_O_REG     0x2478   /* SRU_PINEN0 */
#define PBEN02_I_SPORT2_Db_PBEN_O_REG     0x2478   /* SRU_PINEN0 */
#define PBEN03_I_SPORT2_Db_PBEN_O_REG     0x2478   /* SRU_PINEN0 */
#define PBEN04_I_SPORT2_Db_PBEN_O_REG     0x2478   /* SRU_PINEN0 */
#define PBEN05_I_SPORT2_Db_PBEN_O_REG     0x2478   /* SRU_PINEN0 */
#define PBEN06_I_SPORT2_Db_PBEN_O_REG     0x2479   /* SRU_PINEN1 */
#define PBEN07_I_SPORT2_Db_PBEN_O_REG     0x2479   /* SRU_PINEN1 */
#define PBEN08_I_SPORT2_Db_PBEN_O_REG     0x2479   /* SRU_PINEN1 */
#define PBEN09_I_SPORT2_Db_PBEN_O_REG     0x2479   /* SRU_PINEN1 */
#define PBEN10_I_SPORT2_Db_PBEN_O_REG     0x2479   /* SRU_PINEN1 */
#define PBEN11_I_SPORT2_Db_PBEN_O_REG     0x247A   /* SRU_PINEN2 */
#define PBEN12_I_SPORT2_Db_PBEN_O_REG     0x247A   /* SRU_PINEN2 */
#define PBEN13_I_SPORT2_Db_PBEN_O_REG     0x247A   /* SRU_PINEN2 */
#define PBEN14_I_SPORT2_Db_PBEN_O_REG     0x247A   /* SRU_PINEN2 */
#define PBEN15_I_SPORT2_Db_PBEN_O_REG     0x247A   /* SRU_PINEN2 */
#define PBEN16_I_SPORT2_Db_PBEN_O_REG     0x247B   /* SRU_PINEN3 */
#define PBEN17_I_SPORT2_Db_PBEN_O_REG     0x247B   /* SRU_PINEN3 */
#define PBEN18_I_SPORT2_Db_PBEN_O_REG     0x247B   /* SRU_PINEN3 */
#define PBEN19_I_SPORT2_Db_PBEN_O_REG     0x247B   /* SRU_PINEN3 */
#define PBEN20_I_SPORT2_Db_PBEN_O_REG     0x247B   /* SRU_PINEN3 */

#define PBEN01_I_SPORT3_CLK_PBEN_O_REG    0x2478   /* SRU_PINEN0 */
#define PBEN02_I_SPORT3_CLK_PBEN_O_REG    0x2478   /* SRU_PINEN0 */
#define PBEN03_I_SPORT3_CLK_PBEN_O_REG    0x2478   /* SRU_PINEN0 */
#define PBEN04_I_SPORT3_CLK_PBEN_O_REG    0x2478   /* SRU_PINEN0 */
#define PBEN05_I_SPORT3_CLK_PBEN_O_REG    0x2478   /* SRU_PINEN0 */
#define PBEN06_I_SPORT3_CLK_PBEN_O_REG    0x2479   /* SRU_PINEN1 */
#define PBEN07_I_SPORT3_CLK_PBEN_O_REG    0x2479   /* SRU_PINEN1 */
#define PBEN08_I_SPORT3_CLK_PBEN_O_REG    0x2479   /* SRU_PINEN1 */
#define PBEN09_I_SPORT3_CLK_PBEN_O_REG    0x2479   /* SRU_PINEN1 */
#define PBEN10_I_SPORT3_CLK_PBEN_O_REG    0x2479   /* SRU_PINEN1 */
#define PBEN11_I_SPORT3_CLK_PBEN_O_REG    0x247A   /* SRU_PINEN2 */
#define PBEN12_I_SPORT3_CLK_PBEN_O_REG    0x247A   /* SRU_PINEN2 */
#define PBEN13_I_SPORT3_CLK_PBEN_O_REG    0x247A   /* SRU_PINEN2 */
#define PBEN14_I_SPORT3_CLK_PBEN_O_REG    0x247A   /* SRU_PINEN2 */
#define PBEN15_I_SPORT3_CLK_PBEN_O_REG    0x247A   /* SRU_PINEN2 */
#define PBEN16_I_SPORT3_CLK_PBEN_O_REG    0x247B   /* SRU_PINEN3 */
#define PBEN17_I_SPORT3_CLK_PBEN_O_REG    0x247B   /* SRU_PINEN3 */
#define PBEN18_I_SPORT3_CLK_PBEN_O_REG    0x247B   /* SRU_PINEN3 */
#define PBEN19_I_SPORT3_CLK_PBEN_O_REG    0x247B   /* SRU_PINEN3 */
#define PBEN20_I_SPORT3_CLK_PBEN_O_REG    0x247B   /* SRU_PINEN3 */

#define PBEN01_I_SPORT3_FS_PBEN_O_REG     0x2478   /* SRU_PINEN0 */
#define PBEN02_I_SPORT3_FS_PBEN_O_REG     0x2478   /* SRU_PINEN0 */
#define PBEN03_I_SPORT3_FS_PBEN_O_REG     0x2478   /* SRU_PINEN0 */
#define PBEN04_I_SPORT3_FS_PBEN_O_REG     0x2478   /* SRU_PINEN0 */
#define PBEN05_I_SPORT3_FS_PBEN_O_REG     0x2478   /* SRU_PINEN0 */
#define PBEN06_I_SPORT3_FS_PBEN_O_REG     0x2479   /* SRU_PINEN1 */
#define PBEN07_I_SPORT3_FS_PBEN_O_REG     0x2479   /* SRU_PINEN1 */
#define PBEN08_I_SPORT3_FS_PBEN_O_REG     0x2479   /* SRU_PINEN1 */
#define PBEN09_I_SPORT3_FS_PBEN_O_REG     0x2479   /* SRU_PINEN1 */
#define PBEN10_I_SPORT3_FS_PBEN_O_REG     0x2479   /* SRU_PINEN1 */
#define PBEN11_I_SPORT3_FS_PBEN_O_REG     0x247A   /* SRU_PINEN2 */
#define PBEN12_I_SPORT3_FS_PBEN_O_REG     0x247A   /* SRU_PINEN2 */
#define PBEN13_I_SPORT3_FS_PBEN_O_REG     0x247A   /* SRU_PINEN2 */
#define PBEN14_I_SPORT3_FS_PBEN_O_REG     0x247A   /* SRU_PINEN2 */
#define PBEN15_I_SPORT3_FS_PBEN_O_REG     0x247A   /* SRU_PINEN2 */
#define PBEN16_I_SPORT3_FS_PBEN_O_REG     0x247B   /* SRU_PINEN3 */
#define PBEN17_I_SPORT3_FS_PBEN_O_REG     0x247B   /* SRU_PINEN3 */
#define PBEN18_I_SPORT3_FS_PBEN_O_REG     0x247B   /* SRU_PINEN3 */
#define PBEN19_I_SPORT3_FS_PBEN_O_REG     0x247B   /* SRU_PINEN3 */
#define PBEN20_I_SPORT3_FS_PBEN_O_REG     0x247B   /* SRU_PINEN3 */

#define PBEN01_I_SPORT3_DA_PBEN_O_REG     0x2478   /* SRU_PINEN0 */
#define PBEN02_I_SPORT3_DA_PBEN_O_REG     0x2478   /* SRU_PINEN0 */
#define PBEN03_I_SPORT3_DA_PBEN_O_REG     0x2478   /* SRU_PINEN0 */
#define PBEN04_I_SPORT3_DA_PBEN_O_REG     0x2478   /* SRU_PINEN0 */
#define PBEN05_I_SPORT3_DA_PBEN_O_REG     0x2478   /* SRU_PINEN0 */
#define PBEN06_I_SPORT3_DA_PBEN_O_REG     0x2479   /* SRU_PINEN1 */
#define PBEN07_I_SPORT3_DA_PBEN_O_REG     0x2479   /* SRU_PINEN1 */
#define PBEN08_I_SPORT3_DA_PBEN_O_REG     0x2479   /* SRU_PINEN1 */
#define PBEN09_I_SPORT3_DA_PBEN_O_REG     0x2479   /* SRU_PINEN1 */
#define PBEN10_I_SPORT3_DA_PBEN_O_REG     0x2479   /* SRU_PINEN1 */
#define PBEN11_I_SPORT3_DA_PBEN_O_REG     0x247A   /* SRU_PINEN2 */
#define PBEN12_I_SPORT3_DA_PBEN_O_REG     0x247A   /* SRU_PINEN2 */
#define PBEN13_I_SPORT3_DA_PBEN_O_REG     0x247A   /* SRU_PINEN2 */
#define PBEN14_I_SPORT3_DA_PBEN_O_REG     0x247A   /* SRU_PINEN2 */
#define PBEN15_I_SPORT3_DA_PBEN_O_REG     0x247A   /* SRU_PINEN2 */
#define PBEN16_I_SPORT3_DA_PBEN_O_REG     0x247B   /* SRU_PINEN3 */
#define PBEN17_I_SPORT3_DA_PBEN_O_REG     0x247B   /* SRU_PINEN3 */
#define PBEN18_I_SPORT3_DA_PBEN_O_REG     0x247B   /* SRU_PINEN3 */
#define PBEN19_I_SPORT3_DA_PBEN_O_REG     0x247B   /* SRU_PINEN3 */
#define PBEN20_I_SPORT3_DA_PBEN_O_REG     0x247B   /* SRU_PINEN3 */

#define PBEN01_I_SPORT3_Db_PBEN_O_REG     0x2478   /* SRU_PINEN0 */
#define PBEN02_I_SPORT3_Db_PBEN_O_REG     0x2478   /* SRU_PINEN0 */
#define PBEN03_I_SPORT3_Db_PBEN_O_REG     0x2478   /* SRU_PINEN0 */
#define PBEN04_I_SPORT3_Db_PBEN_O_REG     0x2478   /* SRU_PINEN0 */
#define PBEN05_I_SPORT3_Db_PBEN_O_REG     0x2478   /* SRU_PINEN0 */
#define PBEN06_I_SPORT3_Db_PBEN_O_REG     0x2479   /* SRU_PINEN1 */
#define PBEN07_I_SPORT3_Db_PBEN_O_REG     0x2479   /* SRU_PINEN1 */
#define PBEN08_I_SPORT3_Db_PBEN_O_REG     0x2479   /* SRU_PINEN1 */
#define PBEN09_I_SPORT3_Db_PBEN_O_REG     0x2479   /* SRU_PINEN1 */
#define PBEN10_I_SPORT3_Db_PBEN_O_REG     0x2479   /* SRU_PINEN1 */
#define PBEN11_I_SPORT3_Db_PBEN_O_REG     0x247A   /* SRU_PINEN2 */
#define PBEN12_I_SPORT3_Db_PBEN_O_REG     0x247A   /* SRU_PINEN2 */
#define PBEN13_I_SPORT3_Db_PBEN_O_REG     0x247A   /* SRU_PINEN2 */
#define PBEN14_I_SPORT3_Db_PBEN_O_REG     0x247A   /* SRU_PINEN2 */
#define PBEN15_I_SPORT3_Db_PBEN_O_REG     0x247A   /* SRU_PINEN2 */
#define PBEN16_I_SPORT3_Db_PBEN_O_REG     0x247B   /* SRU_PINEN3 */
#define PBEN17_I_SPORT3_Db_PBEN_O_REG     0x247B   /* SRU_PINEN3 */
#define PBEN18_I_SPORT3_Db_PBEN_O_REG     0x247B   /* SRU_PINEN3 */
#define PBEN19_I_SPORT3_Db_PBEN_O_REG     0x247B   /* SRU_PINEN3 */
#define PBEN20_I_SPORT3_Db_PBEN_O_REG     0x247B   /* SRU_PINEN3 */

#define PBEN01_I_SPORT4_CLK_PBEN_O_REG    0x2478   /* SRU_PINEN0 */
#define PBEN02_I_SPORT4_CLK_PBEN_O_REG    0x2478   /* SRU_PINEN0 */
#define PBEN03_I_SPORT4_CLK_PBEN_O_REG    0x2478   /* SRU_PINEN0 */
#define PBEN04_I_SPORT4_CLK_PBEN_O_REG    0x2478   /* SRU_PINEN0 */
#define PBEN05_I_SPORT4_CLK_PBEN_O_REG    0x2478   /* SRU_PINEN0 */
#define PBEN06_I_SPORT4_CLK_PBEN_O_REG    0x2479   /* SRU_PINEN1 */
#define PBEN07_I_SPORT4_CLK_PBEN_O_REG    0x2479   /* SRU_PINEN1 */
#define PBEN08_I_SPORT4_CLK_PBEN_O_REG    0x2479   /* SRU_PINEN1 */
#define PBEN09_I_SPORT4_CLK_PBEN_O_REG    0x2479   /* SRU_PINEN1 */
#define PBEN10_I_SPORT4_CLK_PBEN_O_REG    0x2479   /* SRU_PINEN1 */
#define PBEN11_I_SPORT4_CLK_PBEN_O_REG    0x247A   /* SRU_PINEN2 */
#define PBEN12_I_SPORT4_CLK_PBEN_O_REG    0x247A   /* SRU_PINEN2 */
#define PBEN13_I_SPORT4_CLK_PBEN_O_REG    0x247A   /* SRU_PINEN2 */
#define PBEN14_I_SPORT4_CLK_PBEN_O_REG    0x247A   /* SRU_PINEN2 */
#define PBEN15_I_SPORT4_CLK_PBEN_O_REG    0x247A   /* SRU_PINEN2 */
#define PBEN16_I_SPORT4_CLK_PBEN_O_REG    0x247B   /* SRU_PINEN3 */
#define PBEN17_I_SPORT4_CLK_PBEN_O_REG    0x247B   /* SRU_PINEN3 */
#define PBEN18_I_SPORT4_CLK_PBEN_O_REG    0x247B   /* SRU_PINEN3 */
#define PBEN19_I_SPORT4_CLK_PBEN_O_REG    0x247B   /* SRU_PINEN3 */
#define PBEN20_I_SPORT4_CLK_PBEN_O_REG    0x247B   /* SRU_PINEN3 */

#define PBEN01_I_SPORT4_FS_PBEN_O_REG     0x2478   /* SRU_PINEN0 */
#define PBEN02_I_SPORT4_FS_PBEN_O_REG     0x2478   /* SRU_PINEN0 */
#define PBEN03_I_SPORT4_FS_PBEN_O_REG     0x2478   /* SRU_PINEN0 */
#define PBEN04_I_SPORT4_FS_PBEN_O_REG     0x2478   /* SRU_PINEN0 */
#define PBEN05_I_SPORT4_FS_PBEN_O_REG     0x2478   /* SRU_PINEN0 */
#define PBEN06_I_SPORT4_FS_PBEN_O_REG     0x2479   /* SRU_PINEN1 */
#define PBEN07_I_SPORT4_FS_PBEN_O_REG     0x2479   /* SRU_PINEN1 */
#define PBEN08_I_SPORT4_FS_PBEN_O_REG     0x2479   /* SRU_PINEN1 */
#define PBEN09_I_SPORT4_FS_PBEN_O_REG     0x2479   /* SRU_PINEN1 */
#define PBEN10_I_SPORT4_FS_PBEN_O_REG     0x2479   /* SRU_PINEN1 */
#define PBEN11_I_SPORT4_FS_PBEN_O_REG     0x247A   /* SRU_PINEN2 */
#define PBEN12_I_SPORT4_FS_PBEN_O_REG     0x247A   /* SRU_PINEN2 */
#define PBEN13_I_SPORT4_FS_PBEN_O_REG     0x247A   /* SRU_PINEN2 */
#define PBEN14_I_SPORT4_FS_PBEN_O_REG     0x247A   /* SRU_PINEN2 */
#define PBEN15_I_SPORT4_FS_PBEN_O_REG     0x247A   /* SRU_PINEN2 */
#define PBEN16_I_SPORT4_FS_PBEN_O_REG     0x247B   /* SRU_PINEN3 */
#define PBEN17_I_SPORT4_FS_PBEN_O_REG     0x247B   /* SRU_PINEN3 */
#define PBEN18_I_SPORT4_FS_PBEN_O_REG     0x247B   /* SRU_PINEN3 */
#define PBEN19_I_SPORT4_FS_PBEN_O_REG     0x247B   /* SRU_PINEN3 */
#define PBEN20_I_SPORT4_FS_PBEN_O_REG     0x247B   /* SRU_PINEN3 */

#define PBEN01_I_SPORT4_DA_PBEN_O_REG     0x2478   /* SRU_PINEN0 */
#define PBEN02_I_SPORT4_DA_PBEN_O_REG     0x2478   /* SRU_PINEN0 */
#define PBEN03_I_SPORT4_DA_PBEN_O_REG     0x2478   /* SRU_PINEN0 */
#define PBEN04_I_SPORT4_DA_PBEN_O_REG     0x2478   /* SRU_PINEN0 */
#define PBEN05_I_SPORT4_DA_PBEN_O_REG     0x2478   /* SRU_PINEN0 */
#define PBEN06_I_SPORT4_DA_PBEN_O_REG     0x2479   /* SRU_PINEN1 */
#define PBEN07_I_SPORT4_DA_PBEN_O_REG     0x2479   /* SRU_PINEN1 */
#define PBEN08_I_SPORT4_DA_PBEN_O_REG     0x2479   /* SRU_PINEN1 */
#define PBEN09_I_SPORT4_DA_PBEN_O_REG     0x2479   /* SRU_PINEN1 */
#define PBEN10_I_SPORT4_DA_PBEN_O_REG     0x2479   /* SRU_PINEN1 */
#define PBEN11_I_SPORT4_DA_PBEN_O_REG     0x247A   /* SRU_PINEN2 */
#define PBEN12_I_SPORT4_DA_PBEN_O_REG     0x247A   /* SRU_PINEN2 */
#define PBEN13_I_SPORT4_DA_PBEN_O_REG     0x247A   /* SRU_PINEN2 */
#define PBEN14_I_SPORT4_DA_PBEN_O_REG     0x247A   /* SRU_PINEN2 */
#define PBEN15_I_SPORT4_DA_PBEN_O_REG     0x247A   /* SRU_PINEN2 */
#define PBEN16_I_SPORT4_DA_PBEN_O_REG     0x247B   /* SRU_PINEN3 */
#define PBEN17_I_SPORT4_DA_PBEN_O_REG     0x247B   /* SRU_PINEN3 */
#define PBEN18_I_SPORT4_DA_PBEN_O_REG     0x247B   /* SRU_PINEN3 */
#define PBEN19_I_SPORT4_DA_PBEN_O_REG     0x247B   /* SRU_PINEN3 */
#define PBEN20_I_SPORT4_DA_PBEN_O_REG     0x247B   /* SRU_PINEN3 */

#define PBEN01_I_SPORT4_Db_PBEN_O_REG     0x2478   /* SRU_PINEN0 */
#define PBEN02_I_SPORT4_Db_PBEN_O_REG     0x2478   /* SRU_PINEN0 */
#define PBEN03_I_SPORT4_Db_PBEN_O_REG     0x2478   /* SRU_PINEN0 */
#define PBEN04_I_SPORT4_Db_PBEN_O_REG     0x2478   /* SRU_PINEN0 */
#define PBEN05_I_SPORT4_Db_PBEN_O_REG     0x2478   /* SRU_PINEN0 */
#define PBEN06_I_SPORT4_Db_PBEN_O_REG     0x2479   /* SRU_PINEN1 */
#define PBEN07_I_SPORT4_Db_PBEN_O_REG     0x2479   /* SRU_PINEN1 */
#define PBEN08_I_SPORT4_Db_PBEN_O_REG     0x2479   /* SRU_PINEN1 */
#define PBEN09_I_SPORT4_Db_PBEN_O_REG     0x2479   /* SRU_PINEN1 */
#define PBEN10_I_SPORT4_Db_PBEN_O_REG     0x2479   /* SRU_PINEN1 */
#define PBEN11_I_SPORT4_Db_PBEN_O_REG     0x247A   /* SRU_PINEN2 */
#define PBEN12_I_SPORT4_Db_PBEN_O_REG     0x247A   /* SRU_PINEN2 */
#define PBEN13_I_SPORT4_Db_PBEN_O_REG     0x247A   /* SRU_PINEN2 */
#define PBEN14_I_SPORT4_Db_PBEN_O_REG     0x247A   /* SRU_PINEN2 */
#define PBEN15_I_SPORT4_Db_PBEN_O_REG     0x247A   /* SRU_PINEN2 */
#define PBEN16_I_SPORT4_Db_PBEN_O_REG     0x247B   /* SRU_PINEN3 */
#define PBEN17_I_SPORT4_Db_PBEN_O_REG     0x247B   /* SRU_PINEN3 */
#define PBEN18_I_SPORT4_Db_PBEN_O_REG     0x247B   /* SRU_PINEN3 */
#define PBEN19_I_SPORT4_Db_PBEN_O_REG     0x247B   /* SRU_PINEN3 */
#define PBEN20_I_SPORT4_Db_PBEN_O_REG     0x247B   /* SRU_PINEN3 */

#define PBEN01_I_SPORT5_CLK_PBEN_O_REG    0x2478   /* SRU_PINEN0 */
#define PBEN02_I_SPORT5_CLK_PBEN_O_REG    0x2478   /* SRU_PINEN0 */
#define PBEN03_I_SPORT5_CLK_PBEN_O_REG    0x2478   /* SRU_PINEN0 */
#define PBEN04_I_SPORT5_CLK_PBEN_O_REG    0x2478   /* SRU_PINEN0 */
#define PBEN05_I_SPORT5_CLK_PBEN_O_REG    0x2478   /* SRU_PINEN0 */
#define PBEN06_I_SPORT5_CLK_PBEN_O_REG    0x2479   /* SRU_PINEN1 */
#define PBEN07_I_SPORT5_CLK_PBEN_O_REG    0x2479   /* SRU_PINEN1 */
#define PBEN08_I_SPORT5_CLK_PBEN_O_REG    0x2479   /* SRU_PINEN1 */
#define PBEN09_I_SPORT5_CLK_PBEN_O_REG    0x2479   /* SRU_PINEN1 */
#define PBEN10_I_SPORT5_CLK_PBEN_O_REG    0x2479   /* SRU_PINEN1 */
#define PBEN11_I_SPORT5_CLK_PBEN_O_REG    0x247A   /* SRU_PINEN2 */
#define PBEN12_I_SPORT5_CLK_PBEN_O_REG    0x247A   /* SRU_PINEN2 */
#define PBEN13_I_SPORT5_CLK_PBEN_O_REG    0x247A   /* SRU_PINEN2 */
#define PBEN14_I_SPORT5_CLK_PBEN_O_REG    0x247A   /* SRU_PINEN2 */
#define PBEN15_I_SPORT5_CLK_PBEN_O_REG    0x247A   /* SRU_PINEN2 */
#define PBEN16_I_SPORT5_CLK_PBEN_O_REG    0x247B   /* SRU_PINEN3 */
#define PBEN17_I_SPORT5_CLK_PBEN_O_REG    0x247B   /* SRU_PINEN3 */
#define PBEN18_I_SPORT5_CLK_PBEN_O_REG    0x247B   /* SRU_PINEN3 */
#define PBEN19_I_SPORT5_CLK_PBEN_O_REG    0x247B   /* SRU_PINEN3 */
#define PBEN20_I_SPORT5_CLK_PBEN_O_REG    0x247B   /* SRU_PINEN3 */

#define PBEN01_I_SPORT5_FS_PBEN_O_REG     0x2478   /* SRU_PINEN0 */
#define PBEN02_I_SPORT5_FS_PBEN_O_REG     0x2478   /* SRU_PINEN0 */
#define PBEN03_I_SPORT5_FS_PBEN_O_REG     0x2478   /* SRU_PINEN0 */
#define PBEN04_I_SPORT5_FS_PBEN_O_REG     0x2478   /* SRU_PINEN0 */
#define PBEN05_I_SPORT5_FS_PBEN_O_REG     0x2478   /* SRU_PINEN0 */
#define PBEN06_I_SPORT5_FS_PBEN_O_REG     0x2479   /* SRU_PINEN1 */
#define PBEN07_I_SPORT5_FS_PBEN_O_REG     0x2479   /* SRU_PINEN1 */
#define PBEN08_I_SPORT5_FS_PBEN_O_REG     0x2479   /* SRU_PINEN1 */
#define PBEN09_I_SPORT5_FS_PBEN_O_REG     0x2479   /* SRU_PINEN1 */
#define PBEN10_I_SPORT5_FS_PBEN_O_REG     0x2479   /* SRU_PINEN1 */
#define PBEN11_I_SPORT5_FS_PBEN_O_REG     0x247A   /* SRU_PINEN2 */
#define PBEN12_I_SPORT5_FS_PBEN_O_REG     0x247A   /* SRU_PINEN2 */
#define PBEN13_I_SPORT5_FS_PBEN_O_REG     0x247A   /* SRU_PINEN2 */
#define PBEN14_I_SPORT5_FS_PBEN_O_REG     0x247A   /* SRU_PINEN2 */
#define PBEN15_I_SPORT5_FS_PBEN_O_REG     0x247A   /* SRU_PINEN2 */
#define PBEN16_I_SPORT5_FS_PBEN_O_REG     0x247B   /* SRU_PINEN3 */
#define PBEN17_I_SPORT5_FS_PBEN_O_REG     0x247B   /* SRU_PINEN3 */
#define PBEN18_I_SPORT5_FS_PBEN_O_REG     0x247B   /* SRU_PINEN3 */
#define PBEN19_I_SPORT5_FS_PBEN_O_REG     0x247B   /* SRU_PINEN3 */
#define PBEN20_I_SPORT5_FS_PBEN_O_REG     0x247B   /* SRU_PINEN3 */

#define PBEN01_I_SPORT5_DA_PBEN_O_REG     0x2478   /* SRU_PINEN0 */
#define PBEN02_I_SPORT5_DA_PBEN_O_REG     0x2478   /* SRU_PINEN0 */
#define PBEN03_I_SPORT5_DA_PBEN_O_REG     0x2478   /* SRU_PINEN0 */
#define PBEN04_I_SPORT5_DA_PBEN_O_REG     0x2478   /* SRU_PINEN0 */
#define PBEN05_I_SPORT5_DA_PBEN_O_REG     0x2478   /* SRU_PINEN0 */
#define PBEN06_I_SPORT5_DA_PBEN_O_REG     0x2479   /* SRU_PINEN1 */
#define PBEN07_I_SPORT5_DA_PBEN_O_REG     0x2479   /* SRU_PINEN1 */
#define PBEN08_I_SPORT5_DA_PBEN_O_REG     0x2479   /* SRU_PINEN1 */
#define PBEN09_I_SPORT5_DA_PBEN_O_REG     0x2479   /* SRU_PINEN1 */
#define PBEN10_I_SPORT5_DA_PBEN_O_REG     0x2479   /* SRU_PINEN1 */
#define PBEN11_I_SPORT5_DA_PBEN_O_REG     0x247A   /* SRU_PINEN2 */
#define PBEN12_I_SPORT5_DA_PBEN_O_REG     0x247A   /* SRU_PINEN2 */
#define PBEN13_I_SPORT5_DA_PBEN_O_REG     0x247A   /* SRU_PINEN2 */
#define PBEN14_I_SPORT5_DA_PBEN_O_REG     0x247A   /* SRU_PINEN2 */
#define PBEN15_I_SPORT5_DA_PBEN_O_REG     0x247A   /* SRU_PINEN2 */
#define PBEN16_I_SPORT5_DA_PBEN_O_REG     0x247B   /* SRU_PINEN3 */
#define PBEN17_I_SPORT5_DA_PBEN_O_REG     0x247B   /* SRU_PINEN3 */
#define PBEN18_I_SPORT5_DA_PBEN_O_REG     0x247B   /* SRU_PINEN3 */
#define PBEN19_I_SPORT5_DA_PBEN_O_REG     0x247B   /* SRU_PINEN3 */
#define PBEN20_I_SPORT5_DA_PBEN_O_REG     0x247B   /* SRU_PINEN3 */

#define PBEN01_I_SPORT5_Db_PBEN_O_REG     0x2478   /* SRU_PINEN0 */
#define PBEN02_I_SPORT5_Db_PBEN_O_REG     0x2478   /* SRU_PINEN0 */
#define PBEN03_I_SPORT5_Db_PBEN_O_REG     0x2478   /* SRU_PINEN0 */
#define PBEN04_I_SPORT5_Db_PBEN_O_REG     0x2478   /* SRU_PINEN0 */
#define PBEN05_I_SPORT5_Db_PBEN_O_REG     0x2478   /* SRU_PINEN0 */
#define PBEN06_I_SPORT5_Db_PBEN_O_REG     0x2479   /* SRU_PINEN1 */
#define PBEN07_I_SPORT5_Db_PBEN_O_REG     0x2479   /* SRU_PINEN1 */
#define PBEN08_I_SPORT5_Db_PBEN_O_REG     0x2479   /* SRU_PINEN1 */
#define PBEN09_I_SPORT5_Db_PBEN_O_REG     0x2479   /* SRU_PINEN1 */
#define PBEN10_I_SPORT5_Db_PBEN_O_REG     0x2479   /* SRU_PINEN1 */
#define PBEN11_I_SPORT5_Db_PBEN_O_REG     0x247A   /* SRU_PINEN2 */
#define PBEN12_I_SPORT5_Db_PBEN_O_REG     0x247A   /* SRU_PINEN2 */
#define PBEN13_I_SPORT5_Db_PBEN_O_REG     0x247A   /* SRU_PINEN2 */
#define PBEN14_I_SPORT5_Db_PBEN_O_REG     0x247A   /* SRU_PINEN2 */
#define PBEN15_I_SPORT5_Db_PBEN_O_REG     0x247A   /* SRU_PINEN2 */
#define PBEN16_I_SPORT5_Db_PBEN_O_REG     0x247B   /* SRU_PINEN3 */
#define PBEN17_I_SPORT5_Db_PBEN_O_REG     0x247B   /* SRU_PINEN3 */
#define PBEN18_I_SPORT5_Db_PBEN_O_REG     0x247B   /* SRU_PINEN3 */
#define PBEN19_I_SPORT5_Db_PBEN_O_REG     0x247B   /* SRU_PINEN3 */
#define PBEN20_I_SPORT5_Db_PBEN_O_REG     0x247B   /* SRU_PINEN3 */

#define PBEN01_I_TIMER0_PBEN_O_REG        0x2478   /* SRU_PINEN0 */
#define PBEN02_I_TIMER0_PBEN_O_REG        0x2478   /* SRU_PINEN0 */
#define PBEN03_I_TIMER0_PBEN_O_REG        0x2478   /* SRU_PINEN0 */
#define PBEN04_I_TIMER0_PBEN_O_REG        0x2478   /* SRU_PINEN0 */
#define PBEN05_I_TIMER0_PBEN_O_REG        0x2478   /* SRU_PINEN0 */
#define PBEN06_I_TIMER0_PBEN_O_REG        0x2479   /* SRU_PINEN1 */
#define PBEN07_I_TIMER0_PBEN_O_REG        0x2479   /* SRU_PINEN1 */
#define PBEN08_I_TIMER0_PBEN_O_REG        0x2479   /* SRU_PINEN1 */
#define PBEN09_I_TIMER0_PBEN_O_REG        0x2479   /* SRU_PINEN1 */
#define PBEN10_I_TIMER0_PBEN_O_REG        0x2479   /* SRU_PINEN1 */
#define PBEN11_I_TIMER0_PBEN_O_REG        0x247A   /* SRU_PINEN2 */
#define PBEN12_I_TIMER0_PBEN_O_REG        0x247A   /* SRU_PINEN2 */
#define PBEN13_I_TIMER0_PBEN_O_REG        0x247A   /* SRU_PINEN2 */
#define PBEN14_I_TIMER0_PBEN_O_REG        0x247A   /* SRU_PINEN2 */
#define PBEN15_I_TIMER0_PBEN_O_REG        0x247A   /* SRU_PINEN2 */
#define PBEN16_I_TIMER0_PBEN_O_REG        0x247B   /* SRU_PINEN3 */
#define PBEN17_I_TIMER0_PBEN_O_REG        0x247B   /* SRU_PINEN3 */
#define PBEN18_I_TIMER0_PBEN_O_REG        0x247B   /* SRU_PINEN3 */
#define PBEN19_I_TIMER0_PBEN_O_REG        0x247B   /* SRU_PINEN3 */
#define PBEN20_I_TIMER0_PBEN_O_REG        0x247B   /* SRU_PINEN3 */

#define PBEN01_I_TIMER1_PBEN_O_REG        0x2478   /* SRU_PINEN0 */
#define PBEN02_I_TIMER1_PBEN_O_REG        0x2478   /* SRU_PINEN0 */
#define PBEN03_I_TIMER1_PBEN_O_REG        0x2478   /* SRU_PINEN0 */
#define PBEN04_I_TIMER1_PBEN_O_REG        0x2478   /* SRU_PINEN0 */
#define PBEN05_I_TIMER1_PBEN_O_REG        0x2478   /* SRU_PINEN0 */
#define PBEN06_I_TIMER1_PBEN_O_REG        0x2479   /* SRU_PINEN1 */
#define PBEN07_I_TIMER1_PBEN_O_REG        0x2479   /* SRU_PINEN1 */
#define PBEN08_I_TIMER1_PBEN_O_REG        0x2479   /* SRU_PINEN1 */
#define PBEN09_I_TIMER1_PBEN_O_REG        0x2479   /* SRU_PINEN1 */
#define PBEN10_I_TIMER1_PBEN_O_REG        0x2479   /* SRU_PINEN1 */
#define PBEN11_I_TIMER1_PBEN_O_REG        0x247A   /* SRU_PINEN2 */
#define PBEN12_I_TIMER1_PBEN_O_REG        0x247A   /* SRU_PINEN2 */
#define PBEN13_I_TIMER1_PBEN_O_REG        0x247A   /* SRU_PINEN2 */
#define PBEN14_I_TIMER1_PBEN_O_REG        0x247A   /* SRU_PINEN2 */
#define PBEN15_I_TIMER1_PBEN_O_REG        0x247A   /* SRU_PINEN2 */
#define PBEN16_I_TIMER1_PBEN_O_REG        0x247B   /* SRU_PINEN3 */
#define PBEN17_I_TIMER1_PBEN_O_REG        0x247B   /* SRU_PINEN3 */
#define PBEN18_I_TIMER1_PBEN_O_REG        0x247B   /* SRU_PINEN3 */
#define PBEN19_I_TIMER1_PBEN_O_REG        0x247B   /* SRU_PINEN3 */
#define PBEN20_I_TIMER1_PBEN_O_REG        0x247B   /* SRU_PINEN3 */

#define PBEN01_I_TIMER2_PBEN_O_REG        0x2478   /* SRU_PINEN0 */
#define PBEN02_I_TIMER2_PBEN_O_REG        0x2478   /* SRU_PINEN0 */
#define PBEN03_I_TIMER2_PBEN_O_REG        0x2478   /* SRU_PINEN0 */
#define PBEN04_I_TIMER2_PBEN_O_REG        0x2478   /* SRU_PINEN0 */
#define PBEN05_I_TIMER2_PBEN_O_REG        0x2478   /* SRU_PINEN0 */
#define PBEN06_I_TIMER2_PBEN_O_REG        0x2479   /* SRU_PINEN1 */
#define PBEN07_I_TIMER2_PBEN_O_REG        0x2479   /* SRU_PINEN1 */
#define PBEN08_I_TIMER2_PBEN_O_REG        0x2479   /* SRU_PINEN1 */
#define PBEN09_I_TIMER2_PBEN_O_REG        0x2479   /* SRU_PINEN1 */
#define PBEN10_I_TIMER2_PBEN_O_REG        0x2479   /* SRU_PINEN1 */
#define PBEN11_I_TIMER2_PBEN_O_REG        0x247A   /* SRU_PINEN2 */
#define PBEN12_I_TIMER2_PBEN_O_REG        0x247A   /* SRU_PINEN2 */
#define PBEN13_I_TIMER2_PBEN_O_REG        0x247A   /* SRU_PINEN2 */
#define PBEN14_I_TIMER2_PBEN_O_REG        0x247A   /* SRU_PINEN2 */
#define PBEN15_I_TIMER2_PBEN_O_REG        0x247A   /* SRU_PINEN2 */
#define PBEN16_I_TIMER2_PBEN_O_REG        0x247B   /* SRU_PINEN3 */
#define PBEN17_I_TIMER2_PBEN_O_REG        0x247B   /* SRU_PINEN3 */
#define PBEN18_I_TIMER2_PBEN_O_REG        0x247B   /* SRU_PINEN3 */
#define PBEN19_I_TIMER2_PBEN_O_REG        0x247B   /* SRU_PINEN3 */
#define PBEN20_I_TIMER2_PBEN_O_REG        0x247B   /* SRU_PINEN3 */

#define PBEN01_I_FLAG10_PBEN_O_REG        0x2478   /* SRU_PINEN0 */
#define PBEN02_I_FLAG10_PBEN_O_REG        0x2478   /* SRU_PINEN0 */
#define PBEN03_I_FLAG10_PBEN_O_REG        0x2478   /* SRU_PINEN0 */
#define PBEN04_I_FLAG10_PBEN_O_REG        0x2478   /* SRU_PINEN0 */
#define PBEN05_I_FLAG10_PBEN_O_REG        0x2478   /* SRU_PINEN0 */
#define PBEN06_I_FLAG10_PBEN_O_REG        0x2479   /* SRU_PINEN1 */
#define PBEN07_I_FLAG10_PBEN_O_REG        0x2479   /* SRU_PINEN1 */
#define PBEN08_I_FLAG10_PBEN_O_REG        0x2479   /* SRU_PINEN1 */
#define PBEN09_I_FLAG10_PBEN_O_REG        0x2479   /* SRU_PINEN1 */
#define PBEN10_I_FLAG10_PBEN_O_REG        0x2479   /* SRU_PINEN1 */
#define PBEN11_I_FLAG10_PBEN_O_REG        0x247A   /* SRU_PINEN2 */
#define PBEN12_I_FLAG10_PBEN_O_REG        0x247A   /* SRU_PINEN2 */
#define PBEN13_I_FLAG10_PBEN_O_REG        0x247A   /* SRU_PINEN2 */
#define PBEN14_I_FLAG10_PBEN_O_REG        0x247A   /* SRU_PINEN2 */
#define PBEN15_I_FLAG10_PBEN_O_REG        0x247A   /* SRU_PINEN2 */
#define PBEN16_I_FLAG10_PBEN_O_REG        0x247B   /* SRU_PINEN3 */
#define PBEN17_I_FLAG10_PBEN_O_REG        0x247B   /* SRU_PINEN3 */
#define PBEN18_I_FLAG10_PBEN_O_REG        0x247B   /* SRU_PINEN3 */
#define PBEN19_I_FLAG10_PBEN_O_REG        0x247B   /* SRU_PINEN3 */
#define PBEN20_I_FLAG10_PBEN_O_REG        0x247B   /* SRU_PINEN3 */

#define PBEN01_I_FLAG11_PBEN_O_REG        0x2478   /* SRU_PINEN0 */
#define PBEN02_I_FLAG11_PBEN_O_REG        0x2478   /* SRU_PINEN0 */
#define PBEN03_I_FLAG11_PBEN_O_REG        0x2478   /* SRU_PINEN0 */
#define PBEN04_I_FLAG11_PBEN_O_REG        0x2478   /* SRU_PINEN0 */
#define PBEN05_I_FLAG11_PBEN_O_REG        0x2478   /* SRU_PINEN0 */
#define PBEN06_I_FLAG11_PBEN_O_REG        0x2479   /* SRU_PINEN1 */
#define PBEN07_I_FLAG11_PBEN_O_REG        0x2479   /* SRU_PINEN1 */
#define PBEN08_I_FLAG11_PBEN_O_REG        0x2479   /* SRU_PINEN1 */
#define PBEN09_I_FLAG11_PBEN_O_REG        0x2479   /* SRU_PINEN1 */
#define PBEN10_I_FLAG11_PBEN_O_REG        0x2479   /* SRU_PINEN1 */
#define PBEN11_I_FLAG11_PBEN_O_REG        0x247A   /* SRU_PINEN2 */
#define PBEN12_I_FLAG11_PBEN_O_REG        0x247A   /* SRU_PINEN2 */
#define PBEN13_I_FLAG11_PBEN_O_REG        0x247A   /* SRU_PINEN2 */
#define PBEN14_I_FLAG11_PBEN_O_REG        0x247A   /* SRU_PINEN2 */
#define PBEN15_I_FLAG11_PBEN_O_REG        0x247A   /* SRU_PINEN2 */
#define PBEN16_I_FLAG11_PBEN_O_REG        0x247B   /* SRU_PINEN3 */
#define PBEN17_I_FLAG11_PBEN_O_REG        0x247B   /* SRU_PINEN3 */
#define PBEN18_I_FLAG11_PBEN_O_REG        0x247B   /* SRU_PINEN3 */
#define PBEN19_I_FLAG11_PBEN_O_REG        0x247B   /* SRU_PINEN3 */
#define PBEN20_I_FLAG11_PBEN_O_REG        0x247B   /* SRU_PINEN3 */

#define PBEN01_I_FLAG12_PBEN_O_REG        0x2478   /* SRU_PINEN0 */
#define PBEN02_I_FLAG12_PBEN_O_REG        0x2478   /* SRU_PINEN0 */
#define PBEN03_I_FLAG12_PBEN_O_REG        0x2478   /* SRU_PINEN0 */
#define PBEN04_I_FLAG12_PBEN_O_REG        0x2478   /* SRU_PINEN0 */
#define PBEN05_I_FLAG12_PBEN_O_REG        0x2478   /* SRU_PINEN0 */
#define PBEN06_I_FLAG12_PBEN_O_REG        0x2479   /* SRU_PINEN1 */
#define PBEN07_I_FLAG12_PBEN_O_REG        0x2479   /* SRU_PINEN1 */
#define PBEN08_I_FLAG12_PBEN_O_REG        0x2479   /* SRU_PINEN1 */
#define PBEN09_I_FLAG12_PBEN_O_REG        0x2479   /* SRU_PINEN1 */
#define PBEN10_I_FLAG12_PBEN_O_REG        0x2479   /* SRU_PINEN1 */
#define PBEN11_I_FLAG12_PBEN_O_REG        0x247A   /* SRU_PINEN2 */
#define PBEN12_I_FLAG12_PBEN_O_REG        0x247A   /* SRU_PINEN2 */
#define PBEN13_I_FLAG12_PBEN_O_REG        0x247A   /* SRU_PINEN2 */
#define PBEN14_I_FLAG12_PBEN_O_REG        0x247A   /* SRU_PINEN2 */
#define PBEN15_I_FLAG12_PBEN_O_REG        0x247A   /* SRU_PINEN2 */
#define PBEN16_I_FLAG12_PBEN_O_REG        0x247B   /* SRU_PINEN3 */
#define PBEN17_I_FLAG12_PBEN_O_REG        0x247B   /* SRU_PINEN3 */
#define PBEN18_I_FLAG12_PBEN_O_REG        0x247B   /* SRU_PINEN3 */
#define PBEN19_I_FLAG12_PBEN_O_REG        0x247B   /* SRU_PINEN3 */
#define PBEN20_I_FLAG12_PBEN_O_REG        0x247B   /* SRU_PINEN3 */

#define PBEN01_I_FLAG13_PBEN_O_REG        0x2478   /* SRU_PINEN0 */
#define PBEN02_I_FLAG13_PBEN_O_REG        0x2478   /* SRU_PINEN0 */
#define PBEN03_I_FLAG13_PBEN_O_REG        0x2478   /* SRU_PINEN0 */
#define PBEN04_I_FLAG13_PBEN_O_REG        0x2478   /* SRU_PINEN0 */
#define PBEN05_I_FLAG13_PBEN_O_REG        0x2478   /* SRU_PINEN0 */
#define PBEN06_I_FLAG13_PBEN_O_REG        0x2479   /* SRU_PINEN1 */
#define PBEN07_I_FLAG13_PBEN_O_REG        0x2479   /* SRU_PINEN1 */
#define PBEN08_I_FLAG13_PBEN_O_REG        0x2479   /* SRU_PINEN1 */
#define PBEN09_I_FLAG13_PBEN_O_REG        0x2479   /* SRU_PINEN1 */
#define PBEN10_I_FLAG13_PBEN_O_REG        0x2479   /* SRU_PINEN1 */
#define PBEN11_I_FLAG13_PBEN_O_REG        0x247A   /* SRU_PINEN2 */
#define PBEN12_I_FLAG13_PBEN_O_REG        0x247A   /* SRU_PINEN2 */
#define PBEN13_I_FLAG13_PBEN_O_REG        0x247A   /* SRU_PINEN2 */
#define PBEN14_I_FLAG13_PBEN_O_REG        0x247A   /* SRU_PINEN2 */
#define PBEN15_I_FLAG13_PBEN_O_REG        0x247A   /* SRU_PINEN2 */
#define PBEN16_I_FLAG13_PBEN_O_REG        0x247B   /* SRU_PINEN3 */
#define PBEN17_I_FLAG13_PBEN_O_REG        0x247B   /* SRU_PINEN3 */
#define PBEN18_I_FLAG13_PBEN_O_REG        0x247B   /* SRU_PINEN3 */
#define PBEN19_I_FLAG13_PBEN_O_REG        0x247B   /* SRU_PINEN3 */
#define PBEN20_I_FLAG13_PBEN_O_REG        0x247B   /* SRU_PINEN3 */

#define PBEN01_I_FLAG14_PBEN_O_REG        0x2478   /* SRU_PINEN0 */
#define PBEN02_I_FLAG14_PBEN_O_REG        0x2478   /* SRU_PINEN0 */
#define PBEN03_I_FLAG14_PBEN_O_REG        0x2478   /* SRU_PINEN0 */
#define PBEN04_I_FLAG14_PBEN_O_REG        0x2478   /* SRU_PINEN0 */
#define PBEN05_I_FLAG14_PBEN_O_REG        0x2478   /* SRU_PINEN0 */
#define PBEN06_I_FLAG14_PBEN_O_REG        0x2479   /* SRU_PINEN1 */
#define PBEN07_I_FLAG14_PBEN_O_REG        0x2479   /* SRU_PINEN1 */
#define PBEN08_I_FLAG14_PBEN_O_REG        0x2479   /* SRU_PINEN1 */
#define PBEN09_I_FLAG14_PBEN_O_REG        0x2479   /* SRU_PINEN1 */
#define PBEN10_I_FLAG14_PBEN_O_REG        0x2479   /* SRU_PINEN1 */
#define PBEN11_I_FLAG14_PBEN_O_REG        0x247A   /* SRU_PINEN2 */
#define PBEN12_I_FLAG14_PBEN_O_REG        0x247A   /* SRU_PINEN2 */
#define PBEN13_I_FLAG14_PBEN_O_REG        0x247A   /* SRU_PINEN2 */
#define PBEN14_I_FLAG14_PBEN_O_REG        0x247A   /* SRU_PINEN2 */
#define PBEN15_I_FLAG14_PBEN_O_REG        0x247A   /* SRU_PINEN2 */
#define PBEN16_I_FLAG14_PBEN_O_REG        0x247B   /* SRU_PINEN3 */
#define PBEN17_I_FLAG14_PBEN_O_REG        0x247B   /* SRU_PINEN3 */
#define PBEN18_I_FLAG14_PBEN_O_REG        0x247B   /* SRU_PINEN3 */
#define PBEN19_I_FLAG14_PBEN_O_REG        0x247B   /* SRU_PINEN3 */
#define PBEN20_I_FLAG14_PBEN_O_REG        0x247B   /* SRU_PINEN3 */

#define PBEN01_I_FLAG15_PBEN_O_REG        0x2478   /* SRU_PINEN0 */
#define PBEN02_I_FLAG15_PBEN_O_REG        0x2478   /* SRU_PINEN0 */
#define PBEN03_I_FLAG15_PBEN_O_REG        0x2478   /* SRU_PINEN0 */
#define PBEN04_I_FLAG15_PBEN_O_REG        0x2478   /* SRU_PINEN0 */
#define PBEN05_I_FLAG15_PBEN_O_REG        0x2478   /* SRU_PINEN0 */
#define PBEN06_I_FLAG15_PBEN_O_REG        0x2479   /* SRU_PINEN1 */
#define PBEN07_I_FLAG15_PBEN_O_REG        0x2479   /* SRU_PINEN1 */
#define PBEN08_I_FLAG15_PBEN_O_REG        0x2479   /* SRU_PINEN1 */
#define PBEN09_I_FLAG15_PBEN_O_REG        0x2479   /* SRU_PINEN1 */
#define PBEN10_I_FLAG15_PBEN_O_REG        0x2479   /* SRU_PINEN1 */
#define PBEN11_I_FLAG15_PBEN_O_REG        0x247A   /* SRU_PINEN2 */
#define PBEN12_I_FLAG15_PBEN_O_REG        0x247A   /* SRU_PINEN2 */
#define PBEN13_I_FLAG15_PBEN_O_REG        0x247A   /* SRU_PINEN2 */
#define PBEN14_I_FLAG15_PBEN_O_REG        0x247A   /* SRU_PINEN2 */
#define PBEN15_I_FLAG15_PBEN_O_REG        0x247A   /* SRU_PINEN2 */
#define PBEN16_I_FLAG15_PBEN_O_REG        0x247B   /* SRU_PINEN3 */
#define PBEN17_I_FLAG15_PBEN_O_REG        0x247B   /* SRU_PINEN3 */
#define PBEN18_I_FLAG15_PBEN_O_REG        0x247B   /* SRU_PINEN3 */
#define PBEN19_I_FLAG15_PBEN_O_REG        0x247B   /* SRU_PINEN3 */
#define PBEN20_I_FLAG15_PBEN_O_REG        0x247B   /* SRU_PINEN3 */

#define PBEN01_I_SPIB_MISO_PBEN_O_REG        0x2478   /* SRU_PINEN0 */
#define PBEN02_I_SPIB_MISO_PBEN_O_REG        0x2478   /* SRU_PINEN0 */
#define PBEN03_I_SPIB_MISO_PBEN_O_REG        0x2478   /* SRU_PINEN0 */
#define PBEN04_I_SPIB_MISO_PBEN_O_REG        0x2478   /* SRU_PINEN0 */
#define PBEN05_I_SPIB_MISO_PBEN_O_REG        0x2478   /* SRU_PINEN0 */
#define PBEN06_I_SPIB_MISO_PBEN_O_REG        0x2479   /* SRU_PINEN1 */
#define PBEN07_I_SPIB_MISO_PBEN_O_REG        0x2479   /* SRU_PINEN1 */
#define PBEN08_I_SPIB_MISO_PBEN_O_REG        0x2479   /* SRU_PINEN1 */
#define PBEN09_I_SPIB_MISO_PBEN_O_REG        0x2479   /* SRU_PINEN1 */
#define PBEN10_I_SPIB_MISO_PBEN_O_REG        0x2479   /* SRU_PINEN1 */
#define PBEN11_I_SPIB_MISO_PBEN_O_REG        0x247A   /* SRU_PINEN2 */
#define PBEN12_I_SPIB_MISO_PBEN_O_REG        0x247A   /* SRU_PINEN2 */
#define PBEN13_I_SPIB_MISO_PBEN_O_REG        0x247A   /* SRU_PINEN2 */
#define PBEN14_I_SPIB_MISO_PBEN_O_REG        0x247A   /* SRU_PINEN2 */
#define PBEN15_I_SPIB_MISO_PBEN_O_REG        0x247A   /* SRU_PINEN2 */
#define PBEN16_I_SPIB_MISO_PBEN_O_REG        0x247B   /* SRU_PINEN3 */
#define PBEN17_I_SPIB_MISO_PBEN_O_REG        0x247B   /* SRU_PINEN3 */
#define PBEN18_I_SPIB_MISO_PBEN_O_REG        0x247B   /* SRU_PINEN3 */
#define PBEN19_I_SPIB_MISO_PBEN_O_REG        0x247B   /* SRU_PINEN3 */
#define PBEN20_I_SPIB_MISO_PBEN_O_REG        0x247B   /* SRU_PINEN3 */

#define PBEN01_I_SPIB_MOSI_PBEN_O_REG        0x2478   /* SRU_PINEN0 */
#define PBEN02_I_SPIB_MOSI_PBEN_O_REG        0x2478   /* SRU_PINEN0 */
#define PBEN03_I_SPIB_MOSI_PBEN_O_REG        0x2478   /* SRU_PINEN0 */
#define PBEN04_I_SPIB_MOSI_PBEN_O_REG        0x2478   /* SRU_PINEN0 */
#define PBEN05_I_SPIB_MOSI_PBEN_O_REG        0x2478   /* SRU_PINEN0 */
#define PBEN06_I_SPIB_MOSI_PBEN_O_REG        0x2479   /* SRU_PINEN1 */
#define PBEN07_I_SPIB_MOSI_PBEN_O_REG        0x2479   /* SRU_PINEN1 */
#define PBEN08_I_SPIB_MOSI_PBEN_O_REG        0x2479   /* SRU_PINEN1 */
#define PBEN09_I_SPIB_MOSI_PBEN_O_REG        0x2479   /* SRU_PINEN1 */
#define PBEN10_I_SPIB_MOSI_PBEN_O_REG        0x2479   /* SRU_PINEN1 */
#define PBEN11_I_SPIB_MOSI_PBEN_O_REG        0x247A   /* SRU_PINEN2 */
#define PBEN12_I_SPIB_MOSI_PBEN_O_REG        0x247A   /* SRU_PINEN2 */
#define PBEN13_I_SPIB_MOSI_PBEN_O_REG        0x247A   /* SRU_PINEN2 */
#define PBEN14_I_SPIB_MOSI_PBEN_O_REG        0x247A   /* SRU_PINEN2 */
#define PBEN15_I_SPIB_MOSI_PBEN_O_REG        0x247A   /* SRU_PINEN2 */
#define PBEN16_I_SPIB_MOSI_PBEN_O_REG        0x247B   /* SRU_PINEN3 */
#define PBEN17_I_SPIB_MOSI_PBEN_O_REG        0x247B   /* SRU_PINEN3 */
#define PBEN18_I_SPIB_MOSI_PBEN_O_REG        0x247B   /* SRU_PINEN3 */
#define PBEN19_I_SPIB_MOSI_PBEN_O_REG        0x247B   /* SRU_PINEN3 */
#define PBEN20_I_SPIB_MOSI_PBEN_O_REG        0x247B   /* SRU_PINEN3 */

#define PBEN01_I_SPIB_CLK_PBEN_O_REG        0x2478   /* SRU_PINEN0 */
#define PBEN02_I_SPIB_CLK_PBEN_O_REG        0x2478   /* SRU_PINEN0 */
#define PBEN03_I_SPIB_CLK_PBEN_O_REG        0x2478   /* SRU_PINEN0 */
#define PBEN04_I_SPIB_CLK_PBEN_O_REG        0x2478   /* SRU_PINEN0 */
#define PBEN05_I_SPIB_CLK_PBEN_O_REG        0x2478   /* SRU_PINEN0 */
#define PBEN06_I_SPIB_CLK_PBEN_O_REG        0x2479   /* SRU_PINEN1 */
#define PBEN07_I_SPIB_CLK_PBEN_O_REG        0x2479   /* SRU_PINEN1 */
#define PBEN08_I_SPIB_CLK_PBEN_O_REG        0x2479   /* SRU_PINEN1 */
#define PBEN09_I_SPIB_CLK_PBEN_O_REG        0x2479   /* SRU_PINEN1 */
#define PBEN10_I_SPIB_CLK_PBEN_O_REG        0x2479   /* SRU_PINEN1 */
#define PBEN11_I_SPIB_CLK_PBEN_O_REG        0x247A   /* SRU_PINEN2 */
#define PBEN12_I_SPIB_CLK_PBEN_O_REG        0x247A   /* SRU_PINEN2 */
#define PBEN13_I_SPIB_CLK_PBEN_O_REG        0x247A   /* SRU_PINEN2 */
#define PBEN14_I_SPIB_CLK_PBEN_O_REG        0x247A   /* SRU_PINEN2 */
#define PBEN15_I_SPIB_CLK_PBEN_O_REG        0x247A   /* SRU_PINEN2 */
#define PBEN16_I_SPIB_CLK_PBEN_O_REG        0x247B   /* SRU_PINEN3 */
#define PBEN17_I_SPIB_CLK_PBEN_O_REG        0x247B   /* SRU_PINEN3 */
#define PBEN18_I_SPIB_CLK_PBEN_O_REG        0x247B   /* SRU_PINEN3 */
#define PBEN19_I_SPIB_CLK_PBEN_O_REG        0x247B   /* SRU_PINEN3 */
#define PBEN20_I_SPIB_CLK_PBEN_O_REG        0x247B   /* SRU_PINEN3 */

#define PBEN01_I_SPIB_FLG0_PBEN_O_REG        0x2478   /* SRU_PINEN0 */
#define PBEN02_I_SPIB_FLG0_PBEN_O_REG        0x2478   /* SRU_PINEN0 */
#define PBEN03_I_SPIB_FLG0_PBEN_O_REG        0x2478   /* SRU_PINEN0 */
#define PBEN04_I_SPIB_FLG0_PBEN_O_REG        0x2478   /* SRU_PINEN0 */
#define PBEN05_I_SPIB_FLG0_PBEN_O_REG        0x2478   /* SRU_PINEN0 */
#define PBEN06_I_SPIB_FLG0_PBEN_O_REG        0x2479   /* SRU_PINEN1 */
#define PBEN07_I_SPIB_FLG0_PBEN_O_REG        0x2479   /* SRU_PINEN1 */
#define PBEN08_I_SPIB_FLG0_PBEN_O_REG        0x2479   /* SRU_PINEN1 */
#define PBEN09_I_SPIB_FLG0_PBEN_O_REG        0x2479   /* SRU_PINEN1 */
#define PBEN10_I_SPIB_FLG0_PBEN_O_REG        0x2479   /* SRU_PINEN1 */
#define PBEN11_I_SPIB_FLG0_PBEN_O_REG        0x247A   /* SRU_PINEN2 */
#define PBEN12_I_SPIB_FLG0_PBEN_O_REG        0x247A   /* SRU_PINEN2 */
#define PBEN13_I_SPIB_FLG0_PBEN_O_REG        0x247A   /* SRU_PINEN2 */
#define PBEN14_I_SPIB_FLG0_PBEN_O_REG        0x247A   /* SRU_PINEN2 */
#define PBEN15_I_SPIB_FLG0_PBEN_O_REG        0x247A   /* SRU_PINEN2 */
#define PBEN16_I_SPIB_FLG0_PBEN_O_REG        0x247B   /* SRU_PINEN3 */
#define PBEN17_I_SPIB_FLG0_PBEN_O_REG        0x247B   /* SRU_PINEN3 */
#define PBEN18_I_SPIB_FLG0_PBEN_O_REG        0x247B   /* SRU_PINEN3 */
#define PBEN19_I_SPIB_FLG0_PBEN_O_REG        0x247B   /* SRU_PINEN3 */
#define PBEN20_I_SPIB_FLG0_PBEN_O_REG        0x247B   /* SRU_PINEN3 */

#define PBEN01_I_SPIB_FLG1_PBEN_O_REG        0x2478   /* SRU_PINEN0 */
#define PBEN02_I_SPIB_FLG1_PBEN_O_REG        0x2478   /* SRU_PINEN0 */
#define PBEN03_I_SPIB_FLG1_PBEN_O_REG        0x2478   /* SRU_PINEN0 */
#define PBEN04_I_SPIB_FLG1_PBEN_O_REG        0x2478   /* SRU_PINEN0 */
#define PBEN05_I_SPIB_FLG1_PBEN_O_REG        0x2478   /* SRU_PINEN0 */
#define PBEN06_I_SPIB_FLG1_PBEN_O_REG        0x2479   /* SRU_PINEN1 */
#define PBEN07_I_SPIB_FLG1_PBEN_O_REG        0x2479   /* SRU_PINEN1 */
#define PBEN08_I_SPIB_FLG1_PBEN_O_REG        0x2479   /* SRU_PINEN1 */
#define PBEN09_I_SPIB_FLG1_PBEN_O_REG        0x2479   /* SRU_PINEN1 */
#define PBEN10_I_SPIB_FLG1_PBEN_O_REG        0x2479   /* SRU_PINEN1 */
#define PBEN11_I_SPIB_FLG1_PBEN_O_REG        0x247A   /* SRU_PINEN2 */
#define PBEN12_I_SPIB_FLG1_PBEN_O_REG        0x247A   /* SRU_PINEN2 */
#define PBEN13_I_SPIB_FLG1_PBEN_O_REG        0x247A   /* SRU_PINEN2 */
#define PBEN14_I_SPIB_FLG1_PBEN_O_REG        0x247A   /* SRU_PINEN2 */
#define PBEN15_I_SPIB_FLG1_PBEN_O_REG        0x247A   /* SRU_PINEN2 */
#define PBEN16_I_SPIB_FLG1_PBEN_O_REG        0x247B   /* SRU_PINEN3 */
#define PBEN17_I_SPIB_FLG1_PBEN_O_REG        0x247B   /* SRU_PINEN3 */
#define PBEN18_I_SPIB_FLG1_PBEN_O_REG        0x247B   /* SRU_PINEN3 */
#define PBEN19_I_SPIB_FLG1_PBEN_O_REG        0x247B   /* SRU_PINEN3 */
#define PBEN20_I_SPIB_FLG1_PBEN_O_REG        0x247B   /* SRU_PINEN3 */   

#define PBEN01_I_SPIB_FLG2_PBEN_O_REG        0x2478   /* SRU_PINEN0 */
#define PBEN02_I_SPIB_FLG2_PBEN_O_REG        0x2478   /* SRU_PINEN0 */
#define PBEN03_I_SPIB_FLG2_PBEN_O_REG        0x2478   /* SRU_PINEN0 */
#define PBEN04_I_SPIB_FLG2_PBEN_O_REG        0x2478   /* SRU_PINEN0 */
#define PBEN05_I_SPIB_FLG2_PBEN_O_REG        0x2478   /* SRU_PINEN0 */
#define PBEN06_I_SPIB_FLG2_PBEN_O_REG        0x2479   /* SRU_PINEN1 */
#define PBEN07_I_SPIB_FLG2_PBEN_O_REG        0x2479   /* SRU_PINEN1 */
#define PBEN08_I_SPIB_FLG2_PBEN_O_REG        0x2479   /* SRU_PINEN1 */
#define PBEN09_I_SPIB_FLG2_PBEN_O_REG        0x2479   /* SRU_PINEN1 */
#define PBEN10_I_SPIB_FLG2_PBEN_O_REG        0x2479   /* SRU_PINEN1 */
#define PBEN11_I_SPIB_FLG2_PBEN_O_REG        0x247A   /* SRU_PINEN2 */
#define PBEN12_I_SPIB_FLG2_PBEN_O_REG        0x247A   /* SRU_PINEN2 */
#define PBEN13_I_SPIB_FLG2_PBEN_O_REG        0x247A   /* SRU_PINEN2 */
#define PBEN14_I_SPIB_FLG2_PBEN_O_REG        0x247A   /* SRU_PINEN2 */
#define PBEN15_I_SPIB_FLG2_PBEN_O_REG        0x247A   /* SRU_PINEN2 */
#define PBEN16_I_SPIB_FLG2_PBEN_O_REG        0x247B   /* SRU_PINEN3 */
#define PBEN17_I_SPIB_FLG2_PBEN_O_REG        0x247B   /* SRU_PINEN3 */
#define PBEN18_I_SPIB_FLG2_PBEN_O_REG        0x247B   /* SRU_PINEN3 */
#define PBEN19_I_SPIB_FLG2_PBEN_O_REG        0x247B   /* SRU_PINEN3 */
#define PBEN20_I_SPIB_FLG2_PBEN_O_REG        0x247B   /* SRU_PINEN3 */

#define PBEN01_I_SPIB_FLG3_PBEN_O_REG        0x2478   /* SRU_PINEN0 */
#define PBEN02_I_SPIB_FLG3_PBEN_O_REG        0x2478   /* SRU_PINEN0 */
#define PBEN03_I_SPIB_FLG3_PBEN_O_REG        0x2478   /* SRU_PINEN0 */
#define PBEN04_I_SPIB_FLG3_PBEN_O_REG        0x2478   /* SRU_PINEN0 */
#define PBEN05_I_SPIB_FLG3_PBEN_O_REG        0x2478   /* SRU_PINEN0 */
#define PBEN06_I_SPIB_FLG3_PBEN_O_REG        0x2479   /* SRU_PINEN1 */
#define PBEN07_I_SPIB_FLG3_PBEN_O_REG        0x2479   /* SRU_PINEN1 */
#define PBEN08_I_SPIB_FLG3_PBEN_O_REG        0x2479   /* SRU_PINEN1 */
#define PBEN09_I_SPIB_FLG3_PBEN_O_REG        0x2479   /* SRU_PINEN1 */
#define PBEN10_I_SPIB_FLG3_PBEN_O_REG        0x2479   /* SRU_PINEN1 */
#define PBEN11_I_SPIB_FLG3_PBEN_O_REG        0x247A   /* SRU_PINEN2 */
#define PBEN12_I_SPIB_FLG3_PBEN_O_REG        0x247A   /* SRU_PINEN2 */
#define PBEN13_I_SPIB_FLG3_PBEN_O_REG        0x247A   /* SRU_PINEN2 */
#define PBEN14_I_SPIB_FLG3_PBEN_O_REG        0x247A   /* SRU_PINEN2 */
#define PBEN15_I_SPIB_FLG3_PBEN_O_REG        0x247A   /* SRU_PINEN2 */
#define PBEN16_I_SPIB_FLG3_PBEN_O_REG        0x247B   /* SRU_PINEN3 */
#define PBEN17_I_SPIB_FLG3_PBEN_O_REG        0x247B   /* SRU_PINEN3 */
#define PBEN18_I_SPIB_FLG3_PBEN_O_REG        0x247B   /* SRU_PINEN3 */
#define PBEN19_I_SPIB_FLG3_PBEN_O_REG        0x247B   /* SRU_PINEN3 */
#define PBEN20_I_SPIB_FLG3_PBEN_O_REG        0x247B   /* SRU_PINEN3 */

//==============================================================================
//                           End SRU MACRO definitions
//==============================================================================

#endif // __SRU_H___



